<?php
namespace application\personal_calendar;

use application\weblcms\ContentObjectPublication;
use application\weblcms\course\DataManager as CourseDataManager;
use application\weblcms\DataManager;
use application\weblcms\WeblcmsManager;
use application\weblcms\WeblcmsRights;
use common\libraries\AndCondition;
use common\libraries\EqualityCondition;
use common\libraries\InCondition;
use common\libraries\SubselectCondition;
use common\libraries\Translation;
use common\libraries\Utilities;
use common\libraries\WebApplication;
use repository\content_object\calendar_event\CalendarEvent;
use repository\ContentObject;
use repository\DataManager as DataManager2;

/**
 * $Id: personal_calendar_weblcms_connector.class.php 201 2009-11-13 12:34:51Z
 * chellee $
 *
 * @package application.personal_calendar.connector
 */

/**
 * This personal calendar connector allows the personal calendar to retrieve the
 * published calendar events in the WebLcms application.
 */
class PersonalCalendarWeblcmsConnector implements PersonalCalendarConnector
{
    const OCCURENCE_START = 'start';
    const OCCURENCE_END = 'end';
    
    /**
     *
     * @see PersonalCalendarConnector
     */
    public function get_events($user, $from_date, $to_date)
    {
        //$dm = \application\weblcms\DataManager :: get_instance();
        $condition = $this->get_conditions($user);
        
        $publications = DataManager :: retrieves
            (ContentObjectPublication :: class_name(), $condition);
        $result = array();

        while ($publication = $publications->next_result())
        {
            if (! WeblcmsRights :: get_instance()->is_allowed_in_courses_subtree(WeblcmsRights :: VIEW_RIGHT,
                $publication->get_id(), WeblcmsRights :: TYPE_PUBLICATION,
                $publication->get_course_id()))
            {
                continue;
            }
            $object = DataManager2 :: retrieve_content_object(
                $publication->get_content_object_id(),
                (CalendarEvent :: class_name())
            );
            
            if ($object->has_frequency())
            {
                $repeats = $object->get_repeats($from_date, $to_date);
                
                foreach ($repeats as $repeat)
                {
                    $event = new PersonalCalendarEvent();
                    $event->set_start_date($repeat[self :: OCCURENCE_START]);
                    $event->set_end_date($repeat[self :: OCCURENCE_END]);
                    $event->set_url('index.php?application=weblcms&amp;go=course_viewer&amp;tool_action=viewer&amp;browser=calendar&amp;course=' .
                            $publication->get_course_id() . '&amp;tool=' . $publication->get_tool() . '&amp;publication=' . $publication->get_id());
                    $event->set_title($object->get_title());
                    $event->set_content($object->get_description());
                    $event->set_source(Translation :: get('TypeName', null, WebApplication :: determine_namespace(WeblcmsManager :: APPLICATION_NAME)));
                    
                    $result[] = $event;
                }
            }
            elseif ($this->is_visible_event($object, $from_date, $to_date))
            {
                $event = new PersonalCalendarEvent();
                $event->set_start_date($object->get_start_date());
                $event->set_end_date($object->get_end_date());
                $event->set_url('index.php?application=weblcms&amp;go=course_viewer&amp;tool_action=viewer&amp;browser=calendar&amp;course=' . $publication->get_course_id() . '&amp;tool=' . $publication->get_tool() . '&amp;publication=' . $publication->get_id());
                $event->set_title($object->get_title());
                $event->set_content($object->get_description());
                $event->set_source(Translation :: get('TypeName', null, WeblcmsManager :: context()));
                $event->set_id($publication->get_id());
                $result[] = $event;
            }
        }
        return $result;
    }

    private function is_visible_event($event, $from_date, $end_date)
    {
        return ($event->get_start_date() >= $from_date && $event->get_start_date() <= $end_date) || ($event->get_end_date() >= $from_date && $event->get_end_date() <= $end_date) || ($event->get_start_date() < $from_date && $event->get_end_date() > $end_date);
    }

    public function get_conditions($user)
    {

        $condition = CourseDataManager :: get_user_courses_condition($user);
        $user_courses = CourseDataManager :: retrieve_user_courses($condition);

        $course_ids = array();

        while ($course = $user_courses->next_result())
        {
            $course_ids[] = $course->get_id();
        }
        
        $conditions = array();
        $conditions[] = new EqualityCondition('tool', 'calendar');
        $conditions[] = new EqualityCondition('hidden', 0);
        $conditions[] = new InCondition('course_id', $course_ids);
        
        $subselect_condition = new EqualityCondition(ContentObject :: PROPERTY_TYPE, CalendarEvent :: class_name());
        $conditions[] = new SubselectCondition(ContentObjectPublication :: PROPERTY_CONTENT_OBJECT_ID,
            ContentObject :: PROPERTY_ID, ContentObject :: get_table_name(), $subselect_condition, null,
            DataManager2 :: get_instance());
        
        return new AndCondition($conditions);
    }
    
    public function is_active()
    {
       
        $application = Utilities :: camelcase_to_underscores('application\weblcms');
        return WebApplication :: is_active($application);
    }
}
