<?php

namespace application\personal_messenger;

/** * ***************************************************************************
 * This class provides an interface for the message recipients. This class has
 * the construction to get the optional property 'recipient_name' that is filled
 * using a join statement joining the raw recipient data table with the table
 * holding the actual names to display.
 *
 * This class doesn't have an actual table in the database but "joins" the
 * tables 'message_user_recipient' and 'message_group_recipient'.
 *
 * The abstract class holds the default properties:
 * - message_id: the numeric ID of the PM;
 *
 * And the constants:
 * - recipient_name;
 *
 * @name Chamilo Personal Messenger > MessageRecipient
 * @version 1.0.0
 * @author Stijn Van Hoecke
 * **************************************************************************** */
abstract class MessageRecipient extends PersonalMessengerDataClass
{
    //**************************************************************************
    // GENERAL CONSTANTS
    //**************************************************************************
    const RECIPIENT_NAME = 'recipient_name';

    //**************************************************************************
    // PROPERTY CONSTANTS
    //**************************************************************************
    const PROPERTY_MESSAGE_RECIPIENT_MESSAGE_ID = 'message_id';

    //**************************************************************************
    // GENERAL GETTERS
    //**************************************************************************
    /**
     * Get the default properties of all recipient relations.
     * @return array The property names
     */
    public static function get_default_property_names($extended_property_names = array())
    {
        $extended_property_names[] = self :: PROPERTY_MESSAGE_RECIPIENT_MESSAGE_ID;
        return $extended_property_names;
    }

    //**************************************************************************
    // PROPERTY GETTERS
    //**************************************************************************
    /**
     * Returns the numeric ID of the message to which this recipient is linked.
     * @return int The message ID
     */
    public function get_message_id()
    {
        return $this->get_default_property(self :: PROPERTY_MESSAGE_RECIPIENT_MESSAGE_ID);
    }

    //**************************************************************************
    // PROPERTY SETTERS
    //**************************************************************************
    /**
     * Sets the numeric ID of the message to which this recipient is linked.
     * @param int $message_id The ID of the message
     */
    public function set_message_id($message_id)
    {
        $this->set_default_property(self :: PROPERTY_MESSAGE_RECIPIENT_MESSAGE_ID, $message_id);
    }

    //**************************************************************************
    // ABSTRACT FUNCTIONS
    //**************************************************************************
    abstract public function get_recipient_name();
}
