<?php

namespace application\personal_messenger;

use common\libraries\Theme;

/** * ***************************************************************************
 * This class is the global structure for all personal messenger folder actions.
 *
 * @name Chamilo Personal Messenger > PersonalMessengerFolder
 * @version 1.0.0
 * @author Hans De Bisschop
 * @author Dieter De Neef
 * @author Stijn Van Hoecke
 * **************************************************************************** */
class PersonalMessengerFolder
{
    //**************************************************************************
    // FOLDER CONSTANTS
    //**************************************************************************
    const INBOX = 1;
    const DISCUSSIONS = 2;
    const OUTBOX = 3;
    const IGNORED = 4;
    const TRASH = 5;

    //**************************************************************************
    // DEFAULTS
    //**************************************************************************
    const DEFAULT_FOLDER = self :: INBOX;
    const DEFAULT_SUBCATEGORIES_ALLOWED = false;

    //**************************************************************************
    // FOLDER FUNCTIONS
    //**************************************************************************
    /**
     * Returns all valid folders.
     * @return array The folders
     */
    public static function get_all_folders()
    {
        $folders = array();
        $folders[] = self :: INBOX;
        $folders[] = self :: DISCUSSIONS;
        $folders[] = self :: OUTBOX;
        $folders[] = self :: IGNORED;
        $folders[] = self :: TRASH;
        return $folders;
    }

    /**
     * Returns the name according to the specified folder.
     * @param int $folder The folder
     * @return String The name of the folder
     */
    public static function get_folder_name($folder)
    {
        switch ($folder)
        {
            case self :: INBOX :
                return 'inbox';
            case self :: DISCUSSIONS :
                return 'discussions';
            case self :: OUTBOX :
                return 'outbox';
            case self :: IGNORED :
                return 'ignored';
            case self :: TRASH :
                return 'trash';
            default :
                return 'unknown';
        }
    }

    /**
     * Returns if the user can create subcategories inside the folder.
     * @param int $folder The folder
     * @return boolean
     */
    public static function supports_subcategories($folder)
    {
        switch ($folder)
        {
            case self :: INBOX :
                return true;
            case self :: DISCUSSIONS :
                return true;
            case self :: OUTBOX :
                return false;
            case self :: IGNORED :
                return false;
            case self :: TRASH :
                return false;
            default :
                return self :: DEFAULT_SUBCATEGORIES_ALLOWED;
        }
    }

    /**
     * Checks if the constant for the given folder exists.
     * @param int $folder
     * @return boolean Indicating if the folder exists
     */
    public static function is_valid_folder($folder)
    {
        return in_array($folder, self :: get_all_folders());
    }

    /**
     * Returns the link to the icon related to a given folder.
     * @param int $folder The folder
     * @return String HTML representing the icon related to the given folder
     */
    public static function get_folder_icon_path($folder)
    {
        $icon_path = Theme :: get_image_path();
        $icon_path .= "folder_" . self :: get_folder_name($folder) . '.png';
        return $icon_path;
    }

    /**
     * Returns the link to the action icon related to a given folder.
     * @param int $folder The folder
     * @return String HTML representing the action icon related to the given folder
     */
    public static function get_folder_action_icon_path($folder)
    {
        $icon_path = Theme :: get_image_path();
        $icon_path .= "action_folder_" . self :: get_folder_name($folder) . '.png';
        return $icon_path;
    }

    /**
     * Creates the conditions with which messages must match for the given folder.
     * @param int $folder
     * @param int $category (optional)
     * @return Condition
     */
    public static function get_conditions($folder, $category_id = 0)
    {
        return PersonalMessengerDataManager :: get_folder_conditions($folder, $category_id);
    }

}
