<?php

namespace application\personal_messenger;

use common\libraries\ActionBarRenderer;
use common\libraries\Application;
use common\libraries\BreadcrumbTrail;
use common\libraries\Breadcrumb;
use common\libraries\Request;
use common\libraries\Theme;
use common\libraries\ToolbarItem;
use common\libraries\Translation;
use common\libraries\Utilities;

/** * ***************************************************************************
 * This offers the functionality to move a message between categories.
 *
 * @name Chamilo Personal Messenger > PersonalMessengerManagerCategoryMoverComponent
 * @version 1.0.0
 * @author Stijn Van Hoecke
 * **************************************************************************** */
class PersonalMessengerManagerCategoryMoverComponent extends PersonalMessengerManager
{

    //**************************************************************************
    // RUN
    //**************************************************************************
    /**
     * Runs this component.
     */
    public function run()
    {
        $folder = Request :: get(self :: PARAM_FOLDER);

        if (PersonalMessengerFolder :: is_valid_folder($folder))
        {
            // construct the form
            $form = new MoveForm($this, $folder, $this->get_url());

            if ($form->validate())
            {
                $failures = 0;

                // get the IDs
                $message_ids = Request :: get(self :: PARAM_MESSAGE_ID);

                // checks
                if (!empty($message_ids))
                {
                    // construct array if id is single element
                    if (!is_array($message_ids))
                    {
                        $message_ids = array($message_ids);
                    }

                    // for each message, move
                    foreach ($message_ids as $message_id)
                    {
                        if (!$form->move_message($message_id))
                        {
                            $failures++;
                        }
                    }
                }

                // feedback
                if ($failures)
                {
                    $feedback_message = Translation :: get(count($message_ids) == 1 ? 'MessageNotMoved' : 'MessagesNotMoved');
                }
                else
                {
                    $feedback_message = Translation :: get(count($message_ids) == 1 ? 'MessageMoved' : 'MessagesMoved');
                }

                // go back
                $go_back_parameters = array();
                if (Request :: get(PersonalMessengerManager :: PARAM_REFERER))
                {
                    $go_back_parameters[Application :: PARAM_ACTION] = Request :: get(self :: PARAM_REFERER);
                    $go_back_parameters[self :: PARAM_MESSAGE_ID] = Request :: get(self :: PARAM_MESSAGE_ID);

                    if (Request :: get(self :: PARAM_SWITCH_CURRENT_CATEGORY))
                    {
                        $go_back_parameters[self :: PARAM_CATEGORY_ID] = $form->get_destination_category_id();
                    }
                }
                else
                {
                    $go_back_parameters[Application :: PARAM_ACTION] = self :: DEFAULT_ACTION;
                }

                $this->redirect($feedback_message, ($failures ? true : false), $go_back_parameters);
            }
            else
            {
                $this->display_header();
                echo $this->get_action_bar()->as_html();
                $form->display();
                $this->display_footer();
            }
        }
    }

    //**************************************************************************
    // ADDITIONAL FUNCTIONS
    //**************************************************************************
    /**
     * Returns the action bar.
     * @return ActionBar
     */
    public function get_action_bar()
    {
        // default bar
        $action_bar = new ActionBarRenderer(ActionBarRenderer :: TYPE_HORIZONTAL);

        // additional functions
        // MANAGE CATEGORIES
        $manage_cat_url = $this->get_url(array(Application :: PARAM_ACTION => self :: ACTION_MANAGE_CATEGORIES));
        $manage_cat_item = new ToolbarItem(
                        Translation :: get('ManageCategories', null, Utilities :: COMMON_LIBRARIES),
                        Theme :: get_common_image_path() . 'action_category.png',
                        $manage_cat_url,
                        ToolbarItem :: DISPLAY_ICON_AND_LABEL,
                        false
        );

        // CONSTRUCT TOOLBAR
        if (PersonalMessengerFolder :: supports_subcategories($this->get_folder()))
        {
            $action_bar->add_common_action($manage_cat_item);
        }

        // return
        return $action_bar;
    }

    /**
     * Adds the additional parameters to the URL.
     * @return array
     */
    public function get_additional_parameters()
    {
        return array(self :: PARAM_REFERER, self :: PARAM_MESSAGE_ID, self :: PARAM_FOLDER);
    }

    /**
     * Adds the additional breadcrumbs for navigation.
     * @param BreadcrumbTrail $breadcrumbtrail
     */
    public function add_additional_breadcrumbs(BreadcrumbTrail $breadcrumbtrail)
    {
        $browser_folder_url = $this->get_browser_url($this->get_folder(), $this->get_category_id());

        // browser
        $breadcrumbtrail->add(new Breadcrumb(
                        $browser_folder_url,
                        Translation :: get('PersonalMessengerManagerBrowserComponent')
        ));

        // folder
        $breadcrumbtrail->add(new Breadcrumb(
                        $browser_folder_url,
                        Translation :: get(Utilities:: underscores_to_camelcase(
                                        PersonalMessengerFolder :: get_folder_name($this->get_folder())
                        ))));
    }

}
