<?php

namespace application\personal_messenger;

use common\libraries\Translation;
use common\libraries\Theme;
use common\libraries\Toolbar;
use common\libraries\ToolbarItem;

/** * ***************************************************************************
 * Cell renderer for the discussion user table.
 *
 * @name Chamilo Personal Messenger > DiscussionUserTableCellRenderer
 * @version 1.0.0
 * @author Stijn Van Hoecke
 * **************************************************************************** */
class DiscussionUserTableCellRenderer extends DefaultUserTableCellRenderer
{

    //**************************************************************************
    // GENERAL VARIABLES
    //**************************************************************************
    /**
     * The user manager component
     */
    private $user_manager;
    /**
     * The message.
     */
    private $message;

    //**************************************************************************
    // CONSTRUCTOR
    //**************************************************************************
    /**
     * Constructor
     * @param PersonalMessengerManagerComponent $user_manager The user manager component
     * @param Message $message The message
     */
    public function __construct($user_manager, $message)
    {
        parent :: __construct();
        $this->user_manager = $user_manager;
        $this->message = $message;
    }

    //**************************************************************************
    // GENERAL FUNCTIONS
    //**************************************************************************
    /**
     * Renders a table cell based on the given parameters column and message.
     * The message contains additional parameters obtained by a join between the
     * user message and actual message in the database data manager.
     * @param MessageTableColumn $column The column which should be rendered
     * @param Message $message_with_optional_properties The message to render
     * @return String A HTML representation of the rendered table cell
     */
    public function render_cell($column, $user)
    {
        // get action links
        if ($column === DiscussionUserTableColumnModel :: get_modification_column())
        {
            return $this->get_modification_links($user);
        }

        return parent :: render_cell($column, $user);
    }

    /**
     * Gets the action links to display.
     * @param Message $message_with_optional_parameters The message for which the action links should be returned
     * @return String A HTML representation of the action links
     */
    private function get_modification_links($user)
    {
        // CONSTRUCT TOOLBAR DEPENDING ON THE TYPE OF MESSAGE AND THE FOLDER
        $toolbar = new Toolbar(Toolbar :: TYPE_HORIZONTAL);

        // general available actions
        if ($this->message->get_author_id() == $this->user_manager->get_user_id())
        {
            // DELETE USER
            $delete_url = $this->user_manager->get_message_delete_user_url(
                            $this->message,
                            $user->get_id(),
                            PersonalMessengerManager :: ACTION_MANAGE_USERS_DISCUSSION
            );
            $delete_item = new ToolbarItem(
                            Translation :: get('RemoveParticipant'),
                            Theme :: get_image_path() . 'action_remove_participant.png',
                            $delete_url,
                            ToolbarItem :: DISPLAY_ICON_AND_LABEL,
                            true
            );

            // DELETE USER NA
            $delete_na_url = null;
            $delete_na_item = new ToolbarItem(
                            Translation :: get('RemoveParticipantNA'),
                            Theme :: get_image_path() . 'action_remove_participant_na.png',
                            $delete_na_url,
                            ToolbarItem :: DISPLAY_ICON_AND_LABEL,
                            true
            );

            $toolbar->add_item($this->user_manager->get_user_id() != $user->get_id() ? $delete_item : $delete_na_item);
        }

        return $toolbar->as_html();
    }

}
