<?php

namespace application\personal_messenger;

use common\libraries\Application;
use common\libraries\Breadcrumb;
use common\libraries\BreadcrumbTrail;
use common\libraries\Display;
use common\libraries\Request;
use common\libraries\Translation;
use common\libraries\Utilities;

/** * ***************************************************************************
 * This component offers the functionality to update a message.
 *
 * @name Chamilo Personal Messenger > PersonalMessengerManagerCreatorComponent
 * @version 1.0.0
 * @author Stijn Van Hoecke
 * @uses MessageForm
 * **************************************************************************** */
class PersonalMessengerManagerUpdaterComponent extends PersonalMessengerManager
{

    //**************************************************************************
    // VARIABLES
    //**************************************************************************
    /**
     * The message we updat.
     * @var Message
     */
    private $message;

    //**************************************************************************
    // RUN
    //**************************************************************************
    /**
     * Runs this component and displays its output.
     */
    public function run()
    {
        $this->message = $this->get_message();
        $form = new MessageForm(
                        MessageForm :: TYPE_EDIT,
                        $this->message,
                        $this->get_url(),
                        $this->get_user(),
                        $this,
                        !$this->message->is_discussion(),
                        false
        );

        if ($form->validate())
        {
            $success = $form->update_message();

            // go back
            $go_back_parameters = array();
            if (Request :: get(self :: PARAM_REFERER))
            {
                $go_back_parameters[Application :: PARAM_ACTION] = Request :: get(self :: PARAM_REFERER);
                $go_back_parameters[self :: PARAM_MESSAGE_ID] = Request :: get(self :: PARAM_MESSAGE_ID);
            }
            else
            {
                $go_back_parameters[Application :: PARAM_ACTION] = self :: DEFAULT_ACTION;
            }

            $this->redirect(
                    Translation :: get(
                            $success ? 'ObjectUpdated' : 'ObjectNotUpdated',
                            array('OBJECT' => Translation :: get($this->message->is_discussion() ? 'Discussion' : 'Message')),
                            Utilities :: COMMON_LIBRARIES
                    ),
                    ($success ? false : true),
                    $go_back_parameters
            );
        }
        else
        {
            $this->display_header();
            $form->display();
            $this->display_footer();
        }
    }

    //**************************************************************************
    // GENERAL FUNCTIONS
    //**************************************************************************
    /**
     * Returns the message after performing some checks.
     * @return Message
     */
    public function get_message()
    {
        // get message id
        $message_id = Request :: get(self :: PARAM_MESSAGE_ID);
        if (!$message_id)
        {
            $this->display_header();
            Display :: error_message(Translation :: get('NoMessageSelected'));
            $this->display_footer();
            exit();
        }
        else
        {
            // retrieve the message
            $message = $this->retrieve_message_by_id($message_id);

            // if there is no message
            if (!$message)
            {
                $this->display_header();
                Display :: error_message(Translation :: get('NoMessageSelected'));
                $this->display_footer();
                exit();
            }
            else
            {
                // check if the user is allowed to edit the message
                if ($message->get_author_id() != $this->get_user_id())
                {
                    $this->display_header();
                    Display :: error_message(Translation :: get('NotAllowed', null, Utilities :: COMMON_LIBRARIES));
                    $this->display_footer();
                    exit();
                }
                else
                {
                    return $message;
                }
            }
        }
    }

    //**************************************************************************
    // ADDITIONAL FUNCTIONS
    //**************************************************************************
    /**
     * Adds the additional parameters to the URL.
     * @return array
     */
    public function get_additional_parameters()
    {
        return array(self :: PARAM_REFERER, self :: PARAM_MESSAGE_ID);
    }

    /**
     * Adds the additional breadcrumbs for navigation.
     * @param BreadcrumbTrail $breadcrumbtrail
     */
    public function add_additional_breadcrumbs(BreadcrumbTrail $breadcrumbtrail)
    {
        $browser_folder_url = $this->get_browser_url($this->get_folder(), $this->get_category_id());

        // browser
        $breadcrumbtrail->add(new Breadcrumb(
                        $browser_folder_url,
                        Translation :: get('PersonalMessengerManagerBrowserComponent')
        ));

        // folder
        $breadcrumbtrail->add(new Breadcrumb(
                        $browser_folder_url,
                        Translation :: get(Utilities:: underscores_to_camelcase(
                                        PersonalMessengerFolder :: get_folder_name($this->get_folder()))
                        )
        ));

        // message
        $message = $this->retrieve_message_by_id(Request :: get(self :: PARAM_MESSAGE_ID));
        if ($message)
        {
            $message_url = $this->get_message_view_url($message);
            $breadcrumbtrail->add(new Breadcrumb($message_url,
                            Translation :: get(
                                    $message->is_discussion() ? 'PersonalMessengerManagerViewerDiscussionComponent' : 'PersonalMessengerManagerViewerMessageComponent'
                            )
            ));
        }
    }

}
