<?php

namespace application\personal_messenger;

use common\libraries\Application;
use common\libraries\Request;
use common\libraries\Translation;
use common\libraries\Utilities;

/** * ***************************************************************************
 * This component offers the functionality to remove a user from a message.
 *
 * @name Chamilo Personal Messenger > PersonalMessengerManagerUserDeleterComponent
 * @version 1.0.0
 * @author Stijn Van Hoecke
 * **************************************************************************** */
class PersonalMessengerManagerUserDeleterComponent extends PersonalMessengerManager
{

    //**************************************************************************
    // RUN
    //**************************************************************************
    /**
     * Runs this component and displays its output.
     */
    public function run()
    {
        // get the message
        $message = $this->get_message();

        // get the IDs
        $user_ids = Request :: get(self :: PARAM_USER_ID);

        $failures = $this->delete_users($message, $user_ids);

        // feedback
        if ($failures)
        {
            $feedback_message = Translation :: get(
                            count($user_ids) == 1 ? 'ObjectNotDeleted' : 'ObjectsNotDeleted',
                            array('OBJECT' => Translation :: get(count($user_ids) == 1 ? 'Participant' : 'Participants')),
                            Utilities :: COMMON_LIBRARIES
            );
        }
        else
        {
            $feedback_message = Translation :: get(
                            count($user_ids) == 1 ? 'ObjectDeleted' : 'ObjectsDeleted',
                            array('OBJECT' => Translation :: get(count($user_ids) == 1 ? 'Participant' : 'Participants')),
                            Utilities :: COMMON_LIBRARIES
            );
        }

        // go back
        $go_back_parameters = array();
        if (Request :: get(self :: PARAM_REFERER))
        {
            $go_back_parameters[Application :: PARAM_ACTION] = Request :: get(self :: PARAM_REFERER);
            $go_back_parameters[self :: PARAM_MESSAGE_ID] = Request :: get(self :: PARAM_MESSAGE_ID);
        }
        else
        {
            $go_back_parameters[Application :: PARAM_ACTION] = self :: DEFAULT_ACTION;
        }

        $this->redirect($feedback_message, ($failures ? true : false), $go_back_parameters);
    }

    //**************************************************************************
    // GENERAL FUNCTIONS
    //**************************************************************************
    /**
     * Returns the message after performing some checks.
     * @return Message
     */
    public function get_message()
    {
        // get message id
        $message_id = Request :: get(self :: PARAM_MESSAGE_ID);
        if (!$message_id)
        {
            $this->display_header();
            Display :: error_message(Translation :: get('NoMessageSelected'));
            $this->display_footer();
            exit();
        }
        else
        {
            // retrieve the message
            $message = $this->retrieve_message_by_id($message_id);

            // if there is no message
            if (!$message)
            {
                $this->display_header();
                Display :: error_message(Translation :: get('NoMessageSelected'));
                $this->display_footer();
                exit();
            }
            else
            {
                // check if the user is allowed to delete a user from this message
                if ($message->get_author_id() != $this->get_user_id())
                {
                    $this->display_header();
                    Display :: error_message(Translation :: get('NotAllowed', null, Utilities :: COMMON_LIBRARIES));
                    $this->display_footer();
                    exit();
                }
                else
                {
                    return $message;
                }
            }
        }
    }

}
