<?php

namespace application\personal_messenger;

use common\libraries\ObjectTableColumn;
use common\libraries\ObjectTableColumnModel;
use user\User;

/** * ***************************************************************************
 * This class is the default table column model for ignored users.
 *
 * @name Chamilo Personal Messenger > DefaultUserTableColumnModel
 * @version 1.0.0
 * @author Stijn Van Hoecke
 * **************************************************************************** */
class DefaultUserTableColumnModel extends ObjectTableColumnModel
{
    //**************************************************************************
    // GENERAL CONSTANTS
    //**************************************************************************
    const DEFAULT_SORT_COLUMN = 0; // last name column
    const DEFAULT_SORT_ORDER = SORT_ASC; // ascending order

    
    //**************************************************************************
    // CONSTRUCTOR
    //**************************************************************************
    /**
     * Constructor
     * Creates a model based on the given folder
     */
    public function __construct()
    {
        parent :: __construct(self :: get_default_columns(), self :: DEFAULT_SORT_COLUMN, self :: DEFAULT_SORT_ORDER);
    }
    
    //**************************************************************************
    // STATIC FUNCTIONS
    //**************************************************************************
    /**
     * Gets the default columns for this model
     * @return ContentObjectTableColumn[]
     */
    private static function get_default_columns()
    {
        $columns = array();
        $user_alias = \user\DataManager :: get_alias(User :: get_table_name());
        
        //fixed first column LAST NAME
        $columns[] = new ObjectTableColumn(User :: PROPERTY_LASTNAME, true, $user_alias);
        
        // fixed second column FIRST NAME
        $columns[] = new ObjectTableColumn(User :: PROPERTY_FIRSTNAME, true, $user_alias);
        
        return $columns;
    }

}
