<?php

namespace application\portfolio\integration\core\home;

use application\handbook\DataManager;
use application\handbook\integration\core\home\Display;
use common\libraries\AndCondition;
use common\libraries\EqualityCondition;
use common\libraries\OrCondition;
use common\libraries\PropertyConditionVariable;
use common\libraries\Session;
use common\libraries\StaticConditionVariable;
use common\libraries\Translation;
use repository\ContentObject;


class Connector
{

   /**
     * Returns a list of objects for the specified types.
     *
     * @param array $types
     * @return array
     */
    static function get_objects($types) {
        $result = array();

        $conditions = array();
        $conditions[] = new EqualityCondition(ContentObject :: PROPERTY_OWNER_ID, Session :: get_user_id());

        $types_condition = array();
        foreach ($types as $type) {
            $types_condition[] = new EqualityCondition(ContentObject :: PROPERTY_TYPE, $type);
        }
        $conditions[] = new OrCondition($types_condition);
        $condition = new AndCondition($conditions);

        $rdm = RepositoryDataManager :: get_instance();
        $objects = $rdm->retrieve_content_objects($condition);

        if ($objects->size() == 0) {
            $result[0] = Translation :: get('CreateObjectFirst');
        } else {
            while ($object = $objects->next_result()) {
                $result[$object->get_id()] = $object->get_title();
            }
        }

        return $result;
    }

    /**
     * Returns a list of objects that can be linked to a portfolio block.
     *
     * @return array
     */
    function get_portfolio_objects() {
        return self::get_objects(PortfolioDisplay::get_supported_types());
    }


    function get_portfolio_publications()
    {
        $publications = PortfolioDataManager::get_instance()->retrieve_portfolio_publications();
        $result = array();
        while($publications && $pub=$publications->next_result())
        {
            $co = RepositoryDataManager::get_instance()->retrieve_content_object($pub->get_content_object_id());
            $result[$co->get_id()] = $co->get_title();
        }
        return $result;
    }
}
