<?php
namespace application\portfolio\integration\core\home;

use application\portfolio\PortfolioManager;
use common\libraries\AndCondition;
use common\libraries\DataClassRetrievesParameters;
use common\libraries\EqualityCondition;
use common\libraries\PropertyConditionVariable;
use common\libraries\Session;
use common\libraries\StaticConditionVariable;
use common\libraries\StringUtilities;
use common\libraries\Translation;
use repository\content_object\bookmark\Bookmark;
use repository\ContentObject;
use repository\ContentObjectDisplay;
use repository\DataManager;
use repository\integration\core\home\Block;

//require_once CoreApplication :: get_application_class_path('repository') . 'blocks/content_object_block.class.php';
//require_once CoreApplication :: get_application_class_path('portfolio') . 'lib/portfolio_menu.class.php';

/**
 * Block to display all bookmarks for the portfolio application
 */
class PortfolioBookmarkDisplay extends Block
{

//    function __construct($parent, $block_info, $configuration)
//    {
//        parent::__construct($parent, $block_info, $configuration);
//        $this->default_title = Translation::get('PortfolioBookmarks');
//    }
    
     public function __construct($parent, $block_info, $configuration)
    {
        parent :: __construct($parent, $block_info, $configuration, Translation :: get('PortfolioBookmarks'));
    }

    function is_configured()
    {
        //no configuration needed for now
        return true;
    }

    function is_visible()
    {
        if ($this->is_empty() && !$this->show_when_empty())
        {
            return false;
        }

        return true; //i.e.display on homepage when anonymous
    }

    function is_hidable()
    {
        return true;
    }

    function is_deletable()
    {
        return true;
    }

    /**
     * Returns the html to display when the block is configured.
     *
     * @return string
     */
    function display_content()
    {

        $bookmarks_resultset = $this->get_bookmarks();
        while ($bookmarks_resultset && $bm = $bookmarks_resultset->next_result())
        {
            $display = ContentObjectDisplay :: factory($bm);
            $html[] = $display->get_short_html();
            $html[] = '</br>';
        }
        return implode(StringUtilities::NEW_LINE, $html);
    }
    
    function show_when_empty()
    {
        $configuration = $this->get_configuration();
        $result = isset($configuration['show_when_empty']) ? $configuration['show_when_empty'] : true;
        $result = (bool) $result;
        return $result;
    }

    function is_empty()
    {
        $bookmarks = $this->get_bookmarks();
        return $bookmarks->size() == 0;
    }

    function get_bookmarks()
    {       
        $conditions = array();
        $conditions[] = new EqualityCondition(
            new PropertyConditionVariable(Bookmark :: class_name(), Bookmark :: PROPERTY_APPLICATION),
            new StaticConditionVariable(PortfolioManager::APPLICATION_NAME)
        );
        $conditions[] = new EqualityCondition(
            new PropertyConditionVariable(ContentObject :: class_name(), ContentObject :: PROPERTY_OWNER_ID),
            new StaticConditionVariable(Session :: get_user_id())
        );

        $condition = new AndCondition($conditions);
        $parameters = new DataClassRetrievesParameters($condition);

        $bookmarks_resultset = DataManager :: retrieve_active_content_objects(Bookmark :: class_name(), $parameters);

        return $bookmarks_resultset;
    }
    

}


?>
