<?php
namespace application\portfolio;

use common\libraries\FormValidator;
use common\libraries\Translation;
use common\libraries\Utilities;

/**
 * $Id: portfolio_publication_form.class.php 206 2009-11-13 13:08:01Z chellee $
 *
 * @package application.portfolio.forms
 */

/**
 * This class describes the form for a PortfolioPublication object.
 *
 * @author Sven Vanpoucke
 */
class PortfolioIntroductionForm extends FormValidator
{
    const TYPE_CREATE = 1;
    const TYPE_EDIT = 2;
    const TYPE_CREATE_DEFAULTS = 3;
    const RIGHT_VIEW = 'view';
    const RIGHT_EDIT = 'edit';
    const RIGHT_VIEW_FEEDBACK = 'viewFeedback';
    const RIGHT_GIVE_FEEDBACK = 'giveFeedback';
    const INHERIT_OR_SET = 'inherit_set';

    private $user;

    public function __construct($form_type, $action, $user)
    {
        parent :: __construct('portfolio_publication_settings', 'post', $action);

        $this->user = $user;
        $this->form_type = $form_type;

        if ($this->form_type == self :: TYPE_EDIT)
        {
            $this->build_editing_form($type);
        }
        elseif ($this->form_type == self :: TYPE_CREATE)
        {
            $this->build_creation_form($type);
        }
    }

    public function build_basic_form()
    {
        $label = Translation :: get('IntroductionTextLabel');
        $textfield = $this->add_html_editor('introduction', $label, true);
        $message = Translation :: get('IntroductionMaximumLengthMessage');
        $this->addRule('introduction', $message, 'maxlength', '21844');
    }

    public function build_editing_form($type)
    {
        $this->build_basic_form($type);

        $buttons[] = $this->createElement(
            'style_submit_button',
            'submit',
            Translation :: get('Update', null, Utilities :: COMMON_LIBRARIES),
            array('class' => 'positive update'));
        $buttons[] = $this->createElement(
            'style_reset_button',
            'reset',
            Translation :: get('Reset', null, Utilities :: COMMON_LIBRARIES),
            array('class' => 'normal empty'));

        $this->addGroup($buttons, 'buttons', null, '&nbsp;', false);

        $defaults = array();
        $pdm = PortfolioDataManager :: get_instance();
        $info = $pdm->retrieve_portfolio_information_by_user($this->user->get_id());

        if (isset($info) && $info != false)
        {
            $defaults['introduction'] = $info->get_introduction();
        }

        parent :: setDefaults($defaults);
    }

    public function build_creation_form($type)
    {
        $this->build_basic_form($type);

        $buttons[] = $this->createElement(
            'style_submit_button',
            'submit',
            Translation :: get('Create', null, Utilities :: COMMON_LIBRARIES),
            array('class' => 'positive'));
        $buttons[] = $this->createElement(
            'style_reset_button',
            'reset',
            Translation :: get('Reset', null, Utilities :: COMMON_LIBRARIES),
            array('class' => 'normal empty'));

        $this->addGroup($buttons, 'buttons', null, '&nbsp;', false);
    }

    public function create_portfolio_introduction()
    {
        $values = $this->exportValues();

        $pdm = PortfolioDataManager :: get_instance();
        $info = $pdm->retrieve_portfolio_information_by_user($this->user->get_id());

        if (! isset($info) || $info == false)
        {
            $info = new PortfolioInformation();
            $info->set_user_id($this->user->get_id());
            $info->set_last_updated_date(time());

            $info->set_last_updated_item_type(PortfolioRights :: TYPE_PORTFOLIO_FOLDER);
            $info->set_last_action(PortfolioInformation :: ACTION_PORTFOLIO_INTRODUCTION_SET);
            $success &= $info->create();
        }

        $info->set_introduction($values[introduction]);

        return $info->update();
    }
}
