<?php
namespace application\portfolio;
use common\libraries\DataClass;
use common\libraries\Translation;
use common\libraries\Utilities;

/**
 * This class describes information on the status of a user's portfolio publications
 *
 * @author Nathalie Blocry
 */
class PortfolioInformation extends DataClass
{
    const CLASS_NAME = __CLASS__;
    
    const PROPERTY_USER_ID = "user_id";
    const PROPERTY_LAST_UPDATED_DATE = "last_updated_date";
    const PROPERTY_LAST_UPDATED_ITEM = "last_updated_item_id";
    const PROPERTY_LAS_UPDATED_ITEM_TYPE = "last_updated_item_type";
    const PROPERTY_LAST_ACTION = "last_action";
    const PROPERTY_INTRODUCTION_TEXT = "introduction";
    
    //possible actions to be logged on portfolio
    const ACTION_PORTFOLIO_ADDED = 1;
    const ACTION_ITEM_ADDED = 2;
    const ACTION_EDITED = 4;
    const ACTION_FIRST_PORTFOLIO_CREATED = 5;
    const ACTION_DELETED = 6;
    const ACTION_PORTFOLIO_INTRODUCTION_SET = 7;

    /**
     * Get the default properties
     * @return array The property names.
     */
    public static function get_default_property_names()
    {
        return parent :: get_default_property_names(array(self :: PROPERTY_USER_ID, self :: PROPERTY_LAST_UPDATED_DATE, 
                self :: PROPERTY_LAST_UPDATED_ITEM, self :: PROPERTY_LAS_UPDATED_ITEM_TYPE, self :: PROPERTY_LAST_ACTION, 
                self :: PROPERTY_INTRODUCTION_TEXT));
    }

    /**
     * inherited
     */
    public function get_data_manager()
    {
        return PortfolioDataManager :: get_instance();
    }

    public function create()
    {
        $dm = PortfolioDataManager :: get_instance();
        $info = $dm->create_portfolio_information($this);
        return $info;
    }

    public function update()
    {
        $dm = PortfolioDataManager :: get_instance();
        $success = $dm->update_portfolio_information($this);
        return $success;
    }

    public static function get_table_name()
    {
        return Utilities :: get_classname_from_namespace(self :: CLASS_NAME, true);
    }

    public function get_introduction()
    {
        return $this->get_default_property(self :: PROPERTY_INTRODUCTION_TEXT);
    }

    public function set_introduction($intro_text)
    {
        $this->set_default_property(self :: PROPERTY_INTRODUCTION_TEXT, $intro_text);
    }

    public function get_user_id()
    {
        return $this->get_default_property(self :: PROPERTY_USER_ID);
    }

    public function set_user_id($user_id)
    {
        $this->set_default_property(self :: PROPERTY_USER_ID, $user_id);
    }

    public function get_last_updated_date()
    {
        return $this->get_default_property(self :: PROPERTY_LAST_UPDATED_DATE);
    }

    public function set_last_updated_date($last_updated_date)
    {
        $this->set_default_property(self :: PROPERTY_LAST_UPDATED_DATE, $last_updated_date);
    }

    public function get_last_updated_item_id()
    {
        return $this->get_default_property(self :: PROPERTY_LAST_UPDATED_ITEM);
    }

    public function set_last_updated_item_id($last_updated_item_id)
    {
        $this->set_default_property(self :: PROPERTY_LAST_UPDATED_ITEM, $last_updated_item_id);
    }

    public function get_last_updated_item_type()
    {
        return $this->get_default_property(self :: PROPERTY_LAS_UPDATED_ITEM_TYPE);
    }

    public function set_last_updated_item_type($last_updated_item_type)
    {
        $this->set_default_property(self :: PROPERTY_LAS_UPDATED_ITEM_TYPE, $last_updated_item_type);
    }

    public function get_last_action()
    {
        return $this->get_default_property(self :: PROPERTY_LAST_ACTION);
    }

    public function set_last_action($last_action)
    {
        $this->set_default_property(self :: PROPERTY_LAST_ACTION, $last_action);
    }

    public function get_portfolio_info_text()
    {
        //TODO implement text
        $text = '<div class="portfolio_introduction">';
        
        if ($this->get_introduction() == 'intro')
        {
            $text .= Translation :: get('PortfolioIntroductionStandardText');
            $text .= '<br />';
            $text .= '<br />';
            $text .= '<br />';
        }
        else
        {
            $text .= $this->get_introduction();
            $text .= '<br />';
            $text .= '<br />';
            $text .= '<br />';
        }
        
        $text .= '</div><div>';
        $text .= Translation :: get('LastChangedDate');
        $text .= ' : ';
        //TODO: formating according to locale & timezone!
        $text .= date('j/m/Y (H:i)', $this->get_last_updated_date());
        $text .= '</div><div>';
        $text .= Translation :: get('LastChangedAction');
        $text .= ' : ';
        $action = $this->get_last_action();
        if ($action == self :: ACTION_EDITED)
        {
            $text .= Translation :: get('PortfolioEdited');
        }
        else 
            if ($action == self :: ACTION_FIRST_PORTFOLIO_CREATED)
            {
                $text .= Translation :: get('FirstPortfolioCreated');
            }
            else 
                if ($action == self :: ACTION_ITEM_ADDED)
                {
                    $text .= Translation :: get('ObjectAdded', array('OBJECT' => Translation :: get('Portfolio')));
                }
                else 
                    if ($action == self :: ACTION_PORTFOLIO_ADDED)
                    {
                        $text .= Translation :: get('ObjectAdded', array(
                                'OBJECT' => Translation :: get('PortfolioItem')));
                    }
                    else 
                        if ($action == self :: ACTION_DELETED)
                        {
                            $text .= Translation :: get('ObjectDeleted', array(
                                    'OBJECT' => Translation :: get('Portfolio')));
                        }
                        else 
                            if ($action == self :: ACTION_PORTFOLIO_INTRODUCTION_SET)
                            {
                                $text .= Translation :: get('ObjectChanged', array(
                                        'OBJECT' => Translation :: get('PortfolioIntro')));
                            }
                            else 
                                if ($action == self :: ACTION_PORTFOLIO_INTRODUCTION_SET)
                                {
                                    $text .= Translation :: get('ObjectChanged', array(
                                            'OBJECT' => Translation :: get('PortfolioIntro')));
                                }
                                else
                                {
                                    $text .= Translation :: get('ObjectChanged', array(
                                            'OBJECT' => Translation :: get('Portfolio')));
                                }
        //        $text .= '</div><div>';
        //        $text .= Translation :: get('LastChangedItem');
        //        $text .= ' : ';
        //        $type = $this->get_last_updated_item_type();
        //        $id = $this->get_last_updated_item_id();
        //        if($type == PortfolioRights::TYPE_PORTFOLIO_FOLDER)
        //        {
        //
        //            if($id && $id != 0)
        //            {
        //                $pub = ContentObject::get_by_id($id);
        //            }
        //            if($pub)
        //            {
        //                $text .= $pub->get_title();
        //            }
        //        }
        //        else if($type == PortfolioRights::TYPE_PORTFOLIO_ITEM || $type == PortfolioRights::TYPE_PORTFOLIO_SUB_FOLDER )
        //        {
        //            $rdm = RepositoryDataManager :: get_instance();
        //
        //            $item = \repository\DataManager :: retrieve_content_object($id);
        //            if($item)
        //            {
        //                $text .= $item->get_title();
        //            }
        //        }
        //        else
        //        {
        //            $text.=$type;
        //            $text.='  /  ' ;
        //            $text.=$id;
        //
        //        }
        

        $text .= '</div>';
        
        return $text;
    }

}
