<?php
namespace application\portfolio;

use common\libraries\DataClassRetrievesParameters;

use common\libraries\DelegateComponent;

use common\libraries\Breadcrumb;
use common\libraries\BreadcrumbTrail;
use common\libraries\Translation;
use common\libraries\Utilities;

use common\libraries\InCondition;
use common\libraries\Theme;
use repository\ContentObject;
use repository\content_object\portfolio\Portfolio;


/**
 * Component to create a new portfolio_publication object
 *
 * @author Sven Vanpoucke
 */
class PortfolioManagerPortfolioPublicationCreatorComponent extends PortfolioManager implements
        \repository\viewer\ViewerInterface,
        DelegateComponent
{

    /**
     * Runs this component and displays its output.
     */
    public function run()
    {
        $trail = BreadcrumbTrail :: get_instance();
       
        $user = \user\DataManager :: retrieve(\user\User :: class_name(), (int) $this->get_user_id());
        $trail->add(
                new Breadcrumb(
                        $this->get_url(
                                array(PortfolioManager :: PARAM_ACTION => PortfolioManager :: ACTION_VIEW_PORTFOLIO,
                                        PortfolioManager :: PARAM_PORTFOLIO_OWNER_ID => $this->get_user_id())),
                        Translation :: get('ViewObject', array('OBJECT' => Translation :: get('Portfolio')),
                                Utilities :: COMMON_LIBRARIES) . ' ' . $user->get_fullname()));

      
        $html = array();
        if (! \repository\viewer\Manager :: is_ready_to_be_published())
        {
            $repository_viewer = \repository\viewer\Manager :: construct($this);
            $repository_viewer->run();
        }
        else
        {
            $object = \repository\viewer\Manager :: get_selected_objects();

            if (! is_array($object))
            {
                $object = array($object);
            }

            $portfolio_publication = new PortfolioPublication();

            $form = new PortfolioPublicationForm(PortfolioPublicationForm :: TYPE_CREATE, $portfolio_publication,
                    $this->get_url(
                            array(\repository\viewer\Manager :: PARAM_ACTION => \repository\viewer\Manager :: ACTION_PUBLISHER,
                                    \repository\viewer\Manager :: PARAM_ID => $object)), $this->get_user(),
                    PortfolioRights :: TYPE_PORTFOLIO_FOLDER);

            if ($form->validate())
            {
                $success = $form->create_portfolio_publications($object);
                $this->redirect(
                        $success ? Translation :: get('ObjectCreated',
                                array('OBJECT' => Translation :: get('Portfolio')), Utilities :: COMMON_LIBRARIES) : Translation :: get(
                                'ObjectNotCreated', array('OBJECT' => Translation :: get('Portfolio')),
                                Utilities :: COMMON_LIBRARIES), ! $success,
                        array(PortfolioManager :: PARAM_ACTION => PortfolioManager :: ACTION_VIEW_PORTFOLIO,
                                PortfolioManager :: PARAM_PORTFOLIO_OWNER_ID => $this->get_user_id()));
            }
            else
            {
                $condition = new InCondition(ContentObject :: PROPERTY_ID, $object, ContentObject :: get_table_name());
                $parameters = new DataClassRetrievesParameters($condition);
                $content_objects = \repository\DataManager :: retrieve_active_content_objects(
                        ContentObject :: class_name(), $parameters);

                $html[] = '<div class="content_object padding_10">';
                $html[] = '<div class="title">' . Translation :: get('SelectedContentObjects', null,
                        Utilities :: COMMON_LIBRARIES) . '</div>';
                $html[] = '<div class="description">';
                $html[] = '<ul class="attachments_list">';

                while ($content_object = $content_objects->next_result())
                {
                    $html[] = '<li><img src="' . $content_object->get_icon_path(Theme :: ICON_MINI) . '" alt="' . htmlentities(
                            Translation :: get(ContentObject :: type_to_class($content_object->get_type()), null,
                                    Utilities :: COMMON_LIBRARIES, 'repository')) . '"/> ' . $content_object->get_title() . '</li>';
                }

                $html[] = '</ul>';
                $html[] = '</div>';
                $html[] = '</div>';
                $html[] = $form->toHtml();
                $html[] = '<div style="clear: both;"></div>';

                $this->display_header();
                echo implode("\n", $html);
                $this->display_footer();
            }
        }
    }

    public function get_allowed_content_object_types()
    {
        return array(Portfolio :: class_name());
    }
}
