<?php

namespace application\portfolio;

use common\libraries\DelegateComponent;
use repository\ContentObjectRendition;
use repository\ContentObjectRenditionImplementation;
use common\libraries\Request;
use user\User;
use common\libraries\BreadcrumbTrail;
use common\libraries\Breadcrumb;
use common\libraries\Translation;
use common\libraries\Utilities;
use common\libraries\PlatformSetting;
use common\libraries\ActionBarRenderer;
use common\libraries\ToolbarItem;
use common\libraries\Theme;
use common\extensions\feedback_manager\FeedbackManager;
use repository\ContentObjectForm;
use repository\content_object\portfolio\Portfolio;
use core\admin\AdminDataManager;
use repository\content_object\portfolio_item\PortfolioItem;
use repository\ContentObject;
use common\libraries\Application;
use repository\content_object\bookmark\Bookmark;

/**
 * $Id: viewer.class.php 206 2009-11-13 13:08:01Z chellee $
 *
 * @package application.portfolio.portfolio_manager.component
 */

/**
 * portfolio component which allows the user to browse his portfolio_publications
 *
 * @author Sven Vanpoucke
 */
class PortfolioManagerViewerComponent extends PortfolioManager implements DelegateComponent
{

    private $action_bar;
    private $selected_object;
    private $publication;
    private $portfolio_item;
    private $cid;
    private $pid;
    private $owner_user_id;
    private $viewing_right = true;
    private $editing_right = false;
    private $feedback_giving_right = true;
    private $feedback_viewing_right = false;
    private $permission_setting_right;
    private $specific_html;
    private $portfolio_identifier;
    private $current_user_id;
    private $actions = array();
    private $possible_types;

    const PROPERTY_PID = 'pid';
    const PROPERTY_CID = 'cid';
    const PROPERTY_ROOT = 'root';
    const ACTION_VIEW = 'view';
    const ACTION_FEEDBACK = 'feedback';
    const ACTION_EDIT = 'edit';
    const ACTION_PERMISSIONS = 'properties';

    public function run()
    {

        $this->get_owner();
        $this->get_identifiers();
        $this->get_rights();
        if ($this->viewing_right)
        {
            $this->set_actions();
            $this->get_current_object();
        }
        $current_action = Request :: get('action') ? Request :: get('action') : 'view';
        call_user_func(array($this, 'display_' . $current_action . '_page'));
    }

    public function get_owner()
    {
        $this->owner_user_id = Request :: get(PortfolioManager :: PARAM_PORTFOLIO_OWNER_ID);

        // if anonymous user's portfolio --> redirect to browser
        $owner = \user\DataManager :: retrieve(\user\User :: class_name(), (int) $this->owner_user_id);
        if (!$owner)
        {
            $this->redirect(
                    Translation :: get('PortfolioDoesNotExist'), false, array(PortfolioManager :: PARAM_ACTION => PortfolioManager :: ACTION_BROWSE));
        }

        if (!$owner || $owner->is_anonymous_user())
        {
            $this->redirect(
                    Translation :: get('BrowsePortfoliosAsAnonymousUser'), false, array(PortfolioManager :: PARAM_ACTION => PortfolioManager :: ACTION_BROWSE));
        }
    }

    public function get_identifiers()
    {
        $this->pid = Request :: get(self :: PROPERTY_PID);
        $this->cid = Request :: get(self :: PROPERTY_CID);

        $this->possible_types = array();
        if ($this->cid)
        {
            $this->portfolio_identifier = $this->cid;
            $this->possible_types[] = PortfolioRights :: TYPE_PORTFOLIO_ITEM;
            $this->possible_types[] = PortfolioRights :: TYPE_PORTFOLIO_SUB_FOLDER;
        }
        else
        if ($this->pid)
        {
            $this->portfolio_identifier = $this->pid;
            $this->possible_types[] = PortfolioRights :: TYPE_PORTFOLIO_FOLDER;
        }
        else
        {
            $this->portfolio_identifier = self :: PROPERTY_ROOT;
        }

        if ($this->get_user() instanceof User)
        {
            $this->current_user_id = $this->get_user_id();
        }
        else
        {
            // anonymous user
            $this->current_user_id = 1;
        }
    }

    public function get_rights()
    {
        if ($this->portfolio_identifier != self :: PROPERTY_ROOT)
        {
            $rights = PortfolioRights :: get_rights(
                            $this->current_user_id, $this->portfolio_identifier, $this->possible_types, $this->owner_user_id);
        }
        $this->viewing_right = $rights[PortfolioRights :: VIEW_RIGHT];
        $this->editing_right = $rights[PortfolioRights :: EDIT_RIGHT];
        $this->feedback_viewing_right = $rights[PortfolioRights :: VIEW_FEEDBACK_RIGHT];
        $this->feedback_giving_right = $rights[PortfolioRights :: GIVE_FEEDBACK_RIGHT];
        $this->permission_setting_right = $rights[PortfolioRights :: SET_PERMISSIONS_RIGHT];

        if ($this->portfolio_identifier == self :: PROPERTY_ROOT && $this->current_user_id != 1)
        {
            // root can be seen by every user
            $this->viewing_right = true;
        }
    }

    public function set_actions()
    {
        $this->actions[] = self :: ACTION_VIEW;
        if ($this->editing_right)
        {
            $this->actions[] = self :: ACTION_EDIT;
        }
        if ($this->feedback_viewing_right || $this->feedback_giving_right)
        {
            $this->actions[] = self :: ACTION_FEEDBACK;
        }
        if ($this->permission_setting_right)
        {
            $this->actions[] = self :: ACTION_PERMISSIONS;
        }
    }

    public function get_current_object()
    {
        // get the object
        if ($this->pid && $this->cid)
        {
            // get complex_content_object
            $wrapper = \repository\DataManager :: retrieve_complex_content_object_item($this->cid);
            // get portfolio_item
            $this->selected_object = \repository\DataManager :: retrieve_content_object($wrapper->get_ref());
            if ($this->selected_object instanceof PortfolioItem)
            {
                // get content object
                $this->portfolio_item = $this->selected_object;
                $this->selected_object = \repository\DataManager :: retrieve_content_object(
                                $this->selected_object->get_reference());
            }
        }
        elseif ($this->pid && !$this->cid)
        {
            $publication = PortfolioDataManager :: get_instance()->retrieve_portfolio_publication($this->pid);
            $this->publication = $publication;
            $this->selected_object = \repository\DataManager :: retrieve_content_object(
                            $publication->get_content_object());
            BreadcrumbTrail :: get_instance()->add(
                    new Breadcrumb(
                            null,
                            Translation :: get('View', array('NAME' => $this->selected_object->get_title()))));
        }
    }

    public function display_header()
    {
        $trail = new BreadcrumbTrail();
        $trail->add(
                new Breadcrumb(
                        $this->get_url(array(PortfolioManager :: PARAM_ACTION => PortfolioManager :: ACTION_BROWSE)),
                        Translation :: get(
                                'BrowseObjects', array('OBJECT' => Translation :: get('Portfolios')), Utilities :: COMMON_LIBRARIES)));
        if (!isset($this->owner_user_id))
        {
            $this->owner_user_id = $_REQUEST[PortfolioManager :: PARAM_PORTFOLIO_OWNER_ID];
        }
        $user = \user\DataManager :: retrieve(\user\User :: class_name(), (int) $this->owner_user_id);
        $trail->add(
                new Breadcrumb(
                        $this->get_url(
                                array(
                                    PortfolioManager :: PARAM_ACTION => PortfolioManager :: ACTION_VIEW_PORTFOLIO,
                                    PortfolioManager :: PARAM_PORTFOLIO_OWNER_ID => $this->current_user_id)),
                        Translation :: get(
                                'ViewObject', array('OBJECT' => Translation :: get('Portfolio')), Utilities :: COMMON_LIBRARIES) . ' ' . $user->get_fullname()));

        if ($current_action == self :: ACTION_VIEW)
        {
            $trail->add_help('portfolio viewer');
        }
        else
        if ($current_action == self :: ACTION_FEEDBACK)
        {
            $trail->add_help('portfolio feedback');
        }
        if ($current_action == self :: ACTION_PERMISSIONS)
        {
            $trail->add_help('portfolio permissions');
        }
        if ($current_action == self :: ACTION_EDIT)
        {
            $trail->add_help('portfolio edit');
        }
        else
        {
            $trail->add_help('portfolio general');
        }

        if ($this->get_user() instanceof User)
        {
            $this->current_user_id = $this->get_user_id();
        }
        else
        {
            // anonymous user
            $this->current_user_id = 1;
        }

        $this->action_bar = $this->get_action_bar();
        $html[] = $this->action_bar->as_html();

        $html[] = '<div id="action_bar_browser">';
        $html[] = '<div style="width: 18%; float: left; overflow: auto;">';

        if (PlatformSetting :: get('display_user_picture', __NAMESPACE__))
        {
            $user = \user\DataManager :: retrieve(\user\User :: class_name(), (int) $this->owner_user_id);

            $html[] = '<div style="text-align: center;">';
            $html[] = '<img src="' . $user->get_full_picture_url() . '" WIDTH=80% />';
            $html[] = '</div><br />';
        }
        $menu = new PortfolioMenu(
                        $this->get_user(),
                        'index.php?go=' . self :: ACTION_VIEW_PORTFOLIO . '&application=portfolio&' .
                        PortfolioManager :: PARAM_PORTFOLIO_OWNER_ID . '=' . $this->owner_user_id . '&pid=%s&cid=%s',
                        $this->pid,
                        $this->cid,
                        $this->owner_user_id);
        $html[] = $menu->render_as_tree();
        $html[] = '</div>';

        $html[] = '<div style="width: 80%; overflow: auto;">';
        $html[] = '<div class="tabbed-pane"><ul class="tabbed-pane-tabs">';
        $current_action = Request :: get('action') ? Request :: get('action') : 'view';
        foreach ($this->actions as $action)
        {
            $html[] = '<li><a';
            if ($action == $current_action)
            {
                $html[] = ' class="current"';
            }

            $html[] = ' href="' .
                    $this->get_url(
                            array(
                                'pid' => $this->pid,
                                'cid' => $this->cid,
                                PortfolioManager :: PARAM_PORTFOLIO_OWNER_ID => $this->owner_user_id,
                                'action' => $action)) . '">' . htmlentities(Translation :: get(ucfirst($action) . 'Title'));
            if ($action == 'feedback')
            {
                $html[] = '[' . AdminDataManager :: get_instance()->count_feedback_publications(
                                $this->pid, $this->cid, PortfolioManager :: APPLICATION_NAME) . ']';
            }
            $html[] = '</a></li>';
        }

        $html[] = '</ul><div class="tabbed-pane-content">';

        parent :: display_header();
        echo implode("\n", $html);
        if ($this->specific_html)
        {
            echo $this->specific_html;
        }
    }

    public function display_footer()
    {
        $current_action = Request :: get('action') ? Request :: get('action') : 'view';

        $html = array();

        $html[] = '<div></div>';
        $html[] = '</div>';
        $html[] = '</div>';
        $html[] = '</div>';
        $html[] = '</div>';

        $trail = new BreadcrumbTrail();

        $user = \user\DataManager :: retrieve(\user\User :: class_name(), (int) $this->owner_user_id);
        if ($current_user_id != 1)
        {
            $trail->add(
                    new Breadcrumb(
                            $this->get_url(
                                    array(
                                        PortfolioManager :: PARAM_ACTION => PortfolioManager :: ACTION_VIEW_PORTFOLIO,
                                        PortfolioManager :: PARAM_PORTFOLIO_OWNER_ID => $current_user_id)),
                            Translation :: get(
                                    'ViewObject', array('OBJECT' => Translation :: get('Portfolio')), Utilities :: COMMON_LIBRARIES) . ' ' . $user->get_fullname()));
        }

        echo implode("\n", $html);
        parent :: display_footer();
    }

    public function add_actionbar_item($item)
    {
        if ($this->action_bar != null)
        {
            $this->action_bar->add_tool_action($item);
        }
    }

    public function get_action_bar()
    {
        $action_bar = new ActionBarRenderer(ActionBarRenderer :: TYPE_HORIZONTAL);

        if ($this->owner_user_id == $this->get_user_id())
        {
            $action_bar->add_common_action(
                    new ToolbarItem(
                            Translation :: get('PublishNewPortfolio'),
                            Theme :: get_image_path(ContentObject :: get_content_object_type_namespace('portfolio')) .
                            'logo/22.png',
                            $this->get_create_portfolio_publication_url(),
                            ToolbarItem :: DISPLAY_ICON_AND_LABEL));

            if ($this->portfolio_identifier == self :: PROPERTY_ROOT)
            {
                $action_bar->add_common_action(
                        new ToolbarItem(
                                Translation :: get('ChangeIntroductionText'),
                                Theme :: get_image_path(ContentObject :: get_content_object_type_namespace('portfolio')) .
                                'logo/22.png',
                                $this->get_create_portfolio_introduction_url(),
                                ToolbarItem :: DISPLAY_ICON_AND_LABEL));
            }
        }

        if ($this->selected_object && $this->editing_right)
        {


            if ($this->selected_object instanceof Portfolio)
            {
                if ($this->cid)
                {
                    $portfolio = $this->cid;
                }
                else
                {
                    $portfolio = $this->pid;
                }
                $parent = $this->selected_object->get_id();
               
                $action_bar->add_common_action(
                        new ToolbarItem(
                                Translation :: get('AddNewItemToPortfolio'),
                                Theme :: get_common_image_path() . 'action_create.png',
                                $this->get_create_portfolio_item_url($parent, $portfolio),
                                ToolbarItem :: DISPLAY_ICON_AND_LABEL));

                // BUILDER POPUP

                $builder_params = array();
                $builder_params[Application :: PARAM_APPLICATION] = PortfolioManager :: APPLICATION_NAME;
                $builder_params[PortfolioManager :: PARAM_ACTION] = PortfolioManager :: ACTION_BUILD;
                $builder_params[PortfolioManager :: PARAM_PARENT] = $this->selected_object->get_id();

                $builder_preview_url = $this->get_url($builder_params);
                $builder_onclick = '" onclick="javascript:openPopup(\'' . $builder_preview_url . '\'); return false;';
                $action_bar->add_common_action(
                        new ToolbarItem(
                                Translation :: get('BuildPortfolio'),
                                Theme :: get_common_image_path() . 'action_build.png',
                                $builder_preview_url,
                                ToolbarItem :: DISPLAY_ICON_AND_LABEL,
                                false,
                                $builder_onclick,
                                '_blank'));
            }

            if ($this->pid)
            {
                
            }
        }

        if ($this->selected_object)
        {
            if (!$this->cid)
            {
                $url = $this->get_delete_portfolio_publication_url($this->pid);
                $action_bar->add_common_action(
                        new ToolbarItem(
                                Translation :: get('DeletePortfolioPublication'),
                                Theme :: get_common_image_path() . 'action_delete.png',
                                $url,
                                ToolbarItem :: DISPLAY_ICON_AND_LABEL));
            }
            else
            if ($this->editing_right)
            {
                $url = $this->get_delete_portfolio_item_url($this->cid);
                $action_bar->add_common_action(
                        new ToolbarItem(
                                Translation :: get('Delete_portfolio_item'),
                                Theme :: get_common_image_path() . 'action_delete.png',
                                $url,
                                ToolbarItem :: DISPLAY_ICON_AND_LABEL));
            }
        }

        // BOOKMARK
        $params_bookmark = array();
        $params_bookmark[Application :: PARAM_APPLICATION] = PortfolioManager :: APPLICATION_NAME;
        $params_bookmark[PortfolioManager :: PARAM_ACTION] = PortfolioManager :: ACTION_CREATE_BOOKMARK;
        $params_bookmark[PortfolioManager :: PARAM_PORTFOLIO_OWNER_ID] = Request :: get(
                        PortfolioManager :: PARAM_PORTFOLIO_OWNER_ID);
        $preview_url_bookmark = $this->get_url($params_bookmark);
        $onclick = '" onclick="javascript:openPopup(\'' . $preview_url_bookmark . '\'); return false;';
        $action_bar->add_common_action(
                new ToolbarItem(
                        Translation :: get('MakeBookmark'),
                        ContentObject :: icon_path(Bookmark :: context(), Theme :: ICON_MINI),
                        $preview_url_bookmark,
                        ToolbarItem :: DISPLAY_ICON_AND_LABEL,
                        false,
                        $onclick,
                        '_blank'));

        $params = array();
        $params[PortfolioManager :: PARAM_ACTION] = PortfolioManager :: ACTION_BROWSE;
        $params[PortfolioManager :: PARAM_PORTFOLIO_OWNER_ID] = null;
        $url = $this->get_url($params);

        $action_bar->add_tool_action(
                new ToolbarItem(
                        Translation :: get('Browse'),
                        Theme :: get_common_image_path() . 'action_browser.png',
                        $url,
                        ToolbarItem :: DISPLAY_ICON_AND_LABEL));

        return $action_bar;
    }

    public function display_view_page()
    {
        $html = array();

        if ($this->selected_object)
        {
            // display information on the portfolio publication
            $html[] = ContentObjectRenditionImplementation :: launch(
                            $this->selected_object, ContentObjectRendition :: FORMAT_HTML, ContentObjectRendition :: VIEW_FULL, $this);
        }
        else
        if ($this->viewing_right == false)
        {
            // display a warning that the user does not have viewing rights
            // on the item
            $html[] = Translation :: get('NoPermissionToViewItem');
        }
        else
        {
            $dm = PortfolioDataManager :: get_instance();
            $info = $dm->retrieve_portfolio_information_by_user($this->owner_user_id);
            if ($info)
            {
                $html[] = $info->get_portfolio_info_text();
            }
            else
            {
                $html[] = '<div>';
                $html[] = Translation :: get('PortfolioNotUpdatedYet');
                $html[] = '<br />';
                $html[] = '<br />';
                $html[] = '</div>';
                $html[] = '<div class="portfolio_introduction">';
                $html[] = Translation :: get('PortfolioIntroductionStandardText');
                $html[] = '</div>';
            }
        }

        $this->display_header();
        echo implode("\n", $html);
        $this->display_footer();
    }

    // TODO refactor this code to work with the new submanager structure of the
    // feedback manager????
    public function display_feedback_page()
    {
        $this->set_parameter('action', Request :: get('action'));
        $this->set_parameter(self :: PROPERTY_CID, $this->cid);
        $this->set_parameter(self :: PROPERTY_PID, $this->pid);
        $this->set_parameter(
                PortfolioManager :: PARAM_PORTFOLIO_OWNER_ID, Request :: get(PortfolioManager :: PARAM_PORTFOLIO_OWNER_ID));

        $my_right = "nothing";

        if ($this->feedback_viewing_right)
        {
            $my_right = "view";
        }
        else
        {
            $html[] = '<br /><div id="no_rights">';
            $html[] = Translation :: get('NoPermissionToViewFeedback');
            $html[] = '</div><br />';
        }
        if (!isset($this->feedback_giving_right) || $this->feedback_giving_right)
        {
            if ($my_right == "view")
            {
                $my_right = "view_give";
            }
            else
            {
                $my_right = "give";
            }
        }
        else
        {
            $html[] = '<div id="no_rights">';
            $html[] = Translation :: get('NoPermissionToGiveFeedback');
            $html[] = '</div><br />';
        }

        if ($html)
        {
            $this->specific_html = implode("\n", $html);
        }

        switch ($my_right)
        {
            case 'nothing' :
                $feedback_manager = null;
                break;
            case 'view' :
                if (($this->get_parameter(FeedbackManager :: PARAM_ACTION) != FeedbackManager :: ACTION_UPDATE_FEEDBACK) || ($this->get_parameter(
                                FeedbackManager :: PARAM_ACTION) != FeedbackManager :: ACTION_DELETE_FEEDBACK))
                {
                    $this->set_parameter(
                            FeedbackManager :: PARAM_ACTION, FeedbackManager :: ACTION_BROWSE_ONLY_FEEDBACK);
                }
                $feedback_manager = new FeedbackManager(
                                $this,
                                PortfolioManager :: APPLICATION_NAME,
                                $this->pid,
                                $this->cid);
                break;
            case 'give' :
                if (($this->get_parameter(FeedbackManager :: PARAM_ACTION) != FeedbackManager :: ACTION_UPDATE_FEEDBACK) || ($this->get_parameter(
                                FeedbackManager :: PARAM_ACTION) != FeedbackManager :: ACTION_DELETE_FEEDBACK))
                {
                    $this->set_parameter(
                            FeedbackManager :: PARAM_ACTION, FeedbackManager :: ACTION_CREATE_ONLY_FEEDBACK);
                }
                $feedback_manager = new FeedbackManager(
                                $this,
                                PortfolioManager :: APPLICATION_NAME,
                                $this->pid,
                                $this->cid);
                break;
            case 'view_give' :
                if (($this->get_parameter(FeedbackManager :: PARAM_ACTION) != FeedbackManager :: ACTION_UPDATE_FEEDBACK) || ($this->get_parameter(
                                FeedbackManager :: PARAM_ACTION) != FeedbackManager :: ACTION_DELETE_FEEDBACK))
                {
                    $this->set_parameter(FeedbackManager :: PARAM_ACTION, FeedbackManager :: ACTION_BROWSE_FEEDBACK);
                }
                $feedback_manager = new FeedbackManager(
                                $this,
                                PortfolioManager :: APPLICATION_NAME,
                                $this->pid,
                                $this->cid);
                break;
            default :
                if (($this->get_parameter(FeedbackManager :: PARAM_ACTION) != FeedbackManager :: ACTION_UPDATE_FEEDBACK) || ($this->get_parameter(
                                FeedbackManager :: PARAM_ACTION) != FeedbackManager :: ACTION_DELETE_FEEDBACK))
                {
                    $this->set_parameter(FeedbackManager :: PARAM_ACTION, FeedbackManager :: ACTION_BROWSE_FEEDBACK);
                }
                $feedback_manager = new FeedbackManager(
                                $this,
                                PortfolioManager :: APPLICATION_NAME,
                                $this->pid,
                                $this->cid);
                break;
        }
        if ($feedback_manager != null)
        {
            $feedback_manager->run();
        }
    }

    public function display_edit_page()
    {
        $html = array();
        $success = true;
        $allow_new_version = (!$this->selected_object instanceof Portfolio);

        $form = ContentObjectForm :: factory(
                        ContentObjectForm :: TYPE_EDIT, $this->selected_object, 'content_object_form', 'post', $this->get_url(
                                array(
                                    PortfolioManager :: PARAM_PORTFOLIO_OWNER_ID => $this->owner_user_id,
                                    'pid' => $this->pid,
                                    'cid' => $this->cid,
                                    'action' => 'edit')), null, null, $allow_new_version);

        if ($form->validate())
        {
            if ($this->cid)
            {
                if (!$this->selected_object instanceof Portfolio)
                {
                    $type = PortfolioRights :: TYPE_PORTFOLIO_ITEM;
                }
                else
                {
                    $type = PortfolioRights :: TYPE_PORTFOLIO_SUB_FOLDER;
                }
            }
            else
            {
                $type = PortfolioRights :: TYPE_PORTFOLIO_FOLDER;
            }
            $success &= $form->update_content_object();
            $success &= PortfolioManager :: update_portfolio_info(
                            $this->selected_object->get_id(), $type, PortfolioInformation :: ACTION_EDITED, $this->owner_user_id);

            if ($form->is_version())
            {
                $object = $form->get_content_object();
                if ($this->publication)
                {
                    $this->publication->set_content_object($object->get_latest_version()->get_id());
                    $success &= $this->publication->update(false);
                }
                else
                {
                    $this->portfolio_item->set_reference($object->get_latest_version()->get_id());
                    $success &= $this->portfolio_item->update();
                }
            }

            $this->redirect(
                    $success ? Translation :: get('PortfolioUpdated') : Translation :: get('PortfolioNotUpdated'), !$success, array(
                PortfolioManager :: PARAM_ACTION => PortfolioManager :: ACTION_VIEW_PORTFOLIO,
                PortfolioManager :: PARAM_PORTFOLIO_OWNER_ID => $this->owner_user_id,
                'pid' => $this->pid,
                'cid' => $this->cid));
        }
        else
        {
            $html[] = $form->toHtml();
        }

        $this->display_header();
        echo implode("\n", $html);
        $this->display_footer();
    }

    public function display_properties_page()
    {
        $html = array();
        if ($this->cid)
        {
            if (!$this->selected_object instanceof Portfolio)
            {
                $type = PortfolioRights :: TYPE_PORTFOLIO_ITEM;
            }
            else
            {
                $type = PortfolioRights :: TYPE_PORTFOLIO_SUB_FOLDER;
            }
        }
        else
        {
            $type = PortfolioRights :: TYPE_PORTFOLIO_FOLDER;
        }

        $form = new PortfolioPublicationForm(
                        PortfolioPublicationForm :: TYPE_EDIT,
                        $this->publication,
                        $this->get_url(
                                array(
                                    PortfolioManager :: PARAM_PORTFOLIO_OWNER_ID => $this->get_user_id(),
                                    'pid' => $this->pid,
                                    'cid' => $this->cid,
                                    'action' => 'properties')),
                        $this->get_user(),
                        $type);

        if ($form->validate())
        {
            $success = $form->update_portfolio_publication($type);
            $this->redirect(
                    $success ? Translation :: get('PortfolioPropertiesUpdated') :
                            Translation :: get('PortfolioPropertiesNotUpdated'), !$success, array(
                PortfolioManager :: PARAM_ACTION => PortfolioManager :: ACTION_VIEW_PORTFOLIO,
                PortfolioManager :: PARAM_PORTFOLIO_OWNER_ID => $this->get_user_id(),
                'pid' => $this->pid,
                'cid' => $this->cid));
        }
        else
        {
            $html[] = $form->toHtml();
        }

        $this->display_header();
        echo implode("\n", $html);
        $this->display_footer();
    }

}
