<?php
namespace application\weblcms\course;

use application\weblcms\CourseManagementRights;
use common\libraries\Translation;
use common\libraries\BreadcrumbTrail;
use common\libraries\Breadcrumb;

/**
 * This class describes an action to subscribe to a course
 *
 * @package \application\weblcms\course
 *
 * @author Yannick & Tristan
 * @author Sven Vanpoucke - Hogeschool Gent - Refactoring
 * @author Anthony Hurst (Hogeschool Gent)
 */
class CourseManagerSubscribeComponent extends CourseManager
{

    /**
     * **************************************************************************************************************
     * Inherited Functionality *
     * **************************************************************************************************************
     */

    /**
     * Runs this component and displays its output.
     */
    public function run()
    {
        $failures = 0;

        $course_management_rights = CourseManagementRights :: get_instance();
        $course_ids = $this->get_selected_course_ids();

        foreach ($course_ids as $course_id)
        {
            if ($course_management_rights->is_allowed(CourseManagementRights :: DIRECT_SUBSCRIBE_RIGHT, $course_id))
            {
                $course_user_relation = new CourseUserRelation();
                $course_user_relation->set_user_id($this->get_user_id());
                $course_user_relation->set_course_id($course_id);
                $course_user_relation->set_status(CourseUserRelation :: STATUS_STUDENT);

                if (!$course_user_relation->create())
                {
                    $failures++;
                }
            }
            else
            {
                $failures++;
            }
        }

        $message = $this->get_result(
            $failures,
            count($course_ids),
            'UserNotSubscribedToSelectedCourses',
            'UserNotSubscribedToSelectedCourse',
            'UserSubscribedToSelectedCourses',
            'UserSubscribedToSelectedCourse'
        );

        $this->redirect(
            $message, ($failures > 0), array(
                self :: PARAM_ACTION => self :: ACTION_BROWSE_UNSUBSCRIBED_COURSES
            ), array(self :: PARAM_COURSE_ID)
        );
    }

    /**
     * Breadcrumbs are built semi automatically with the given application,
     * subapplication, component...
     * Use this function to add other breadcrumbs between the application /
     * subapplication and the current component
     *
     * @param $breadcrumbtrail \common\libraries\BreadcrumbTrail
     */
    public function add_additional_breadcrumbs(BreadcrumbTrail $breadcrumbtrail)
    {
        $breadcrumbtrail->add_help('weblcms_course_subscriber');
        $breadcrumbtrail->add(
            new Breadcrumb(
                $this->get_browse_course_url(),
                Translation :: get('CourseManagerBrowseUnsubscribedCoursesComponent')
            )
        );
    }

    /**
     * Returns additional parameters that need to be registered and are used in
     * every url generated by this component
     *
     * @return String[]
     */
    public function get_additional_parameters()
    {
        return array(self :: PARAM_COURSE_ID);
    }
}
