<?php
namespace application\weblcms\integration\core\home;

use application\weblcms\ContentObjectPublication;
use application\weblcms\tool\assignment\AssignmentTool;
use application\weblcms\integration\tracking\CourseVisit;
use common\libraries\AndCondition;
use common\libraries\Application;
use common\libraries\EqualityCondition;
use common\libraries\InequalityCondition;
use common\libraries\Redirect;
use common\libraries\Translation;
use repository\ContentObjectRenderer;
use repository\content_object\assignment\Assignment;
use tracking\TrackingDataManager;
use application\weblcms\DataManager as WeblcmsDataManager;

/**
 * A notificationblock for new assignment submissions (assignmenttool)
 */
class AssignmentSubmissions extends Block
{

    public function display_content()
    {
        $user_id = $this->get_user_id();

        // Retrieve the assignments of the user
        $conditions = array();
        $conditions[] = new EqualityCondition(
            \application\weblcms\ContentObjectPublication :: PROPERTY_PUBLISHER_ID,
            $user_id);
        $conditions[] = new EqualityCondition(
            \application\weblcms\ContentObjectPublication :: PROPERTY_TOOL,
            Assignment :: get_table_name());

        $condition = new AndCondition($conditions);

        $assignment_publications_resultset = WeblcmsDataManager :: retrieves(
            ContentObjectPublication :: class_name(),
            $condition);

        if ($assignment_publications_resultset->size() == 0)
        {
            return Translation :: get('YouDoNotOwnAnyAssignments');
        }

        $tracking_data_manager = TrackingDataManager :: get_instance();
        $items = array();
        while ($publication = $assignment_publications_resultset->next_result())
        {
            //Retrieve last time the publication was accessed
            $course_tool = WeblcmsDataManager::retrieve_course_tool_by_name($publication->get_tool());
            $last_access_time = $this->get_last_visit($user_id, $publication->get_course_id(), $course_tool->get_id(),
            $publication->get_category_id(), $publication->get_id());


            $item = array();
            $conditions = array();
            $conditions[] = new EqualityCondition(
                \application\weblcms\integration\tracking\AssignmentSubmission :: PROPERTY_PUBLICATION_ID,
                $publication->get_id());
            $conditions[] = new InequalityCondition(
                \application\weblcms\integration\tracking\AssignmentSubmission :: PROPERTY_DATE_SUBMITTED,
                InequalityCondition :: GREATER_THAN,
                $last_access_time);
            $condition = new AndCondition($conditions);
            $submissions_resultset = $tracking_data_manager->retrieve_tracker_items(
                \application\weblcms\integration\tracking\AssignmentSubmission :: get_table_name(),
                $condition,
                null,
                null,
                null,
                \application\weblcms\integration\tracking\AssignmentSubmission :: class_name());
            $object = $publication->get_content_object();
            $item[title] = $object->get_title();
            $parameters = array(
                \application\weblcms\WeblcmsManager :: PARAM_COURSE => $publication->get_course_id(),
                Application :: PARAM_ACTION => \application\weblcms\WeblcmsManager :: ACTION_VIEW_COURSE,
                \application\weblcms\WeblcmsManager :: PARAM_TOOL => Assignment :: get_table_name(),
                \application\weblcms\WeblcmsManager :: PARAM_TOOL_ACTION => AssignmentTool :: ACTION_BROWSE_SUBMITTERS,
                \application\weblcms\Tool :: PARAM_BROWSER_TYPE => ContentObjectRenderer :: TYPE_TABLE,
                \application\weblcms\Tool :: PARAM_PUBLICATION_ID => $publication->get_id());
            $item[link] = Redirect :: get_link(\application\weblcms\WeblcmsManager :: APPLICATION_NAME, $parameters);

            $counter = $submissions_resultset->size();

            if ($counter > 0)
            {
                $item[count] = $counter;
                $items[] = $item;
            }
        }

        $html = $this->display_new_items($items);
        if (count($html) == 0)
        {
            return Translation :: get('NoNewSubmissionsSinceLastVisit');
        }
        return implode('', $html);
    }

    public function display_new_items($items)
    {
        $html = array();
        foreach ($items as $item)
        {
            $html[] = '<a href="' . $item[link] . '">' . $item[title] . '</a>: ';
            $html[] = $item[count] . ' ' . Translation :: get('New') . '<br />';
        }
        return $html;
    }

    private function get_last_visit($user_id, $course_id, $tool_id, $category_id, $publication_id)
    {
        $course_visit = new CourseVisit();
        $course_visit->set_user_id($user_id);
        $course_visit->set_course_id($course_id);
        $course_visit->set_tool_id($tool_id);
        if($category_id == 0)
            $category_id = null;
        $course_visit->set_category_id($category_id);
        $course_visit->set_publication_id($publication_id);
        $course_visit = $course_visit->retrieve_course_visit_with_current_data();

        if(!$course_visit)
        {
            return 0;
        }

        return $course_visit->get_last_access_date();
    }
}
