<?php
namespace application\weblcms\integration\core\home;

use application\weblcms\FilteredCourseListRenderer;
use common\libraries\Redirect;
use application\weblcms\SelectedCourseTypeCourseListRenderer;
use application\weblcms\course_type\CourseType;

/**
 * This class represents a block that shows courses of a specific type
 */
class CourseTypeCourseList extends Block
{

    private $renderer;

    private $course_type;

    function display_content()
    {
        $html = array();
        $renderer = $this->get_renderer();
        $renderer->show_new_publication_icons();
        $html[] = $renderer->as_html(false);

        return implode("\n", $html);
    }

    function get_title()
    {
        $course_type = $this->course_type;
        if ($course_type && ! is_null($course_type))
        {
            return $course_type->get_title();
        }
        else
        {
            return parent :: get_title();
        }
    }

    function get_renderer()
    {
        if ($this->renderer == null)
        {
            //$this->renderer = new SelectedCourseTypeCourseListRenderer($this, $this->get_link_target(), $course_type);
            $this->renderer = new FilteredCourseListRenderer(
                $this, $this->get_link_target(), $this->get_course_type_id()
            );
        }
        return $this->renderer;
    }

    function get_course_type()
    {
        $configuration = $this->get_configuration();
        if (isset($configuration['course_type']))
        {
            $condition = new \common\libraries\EqualityCondition(
                CourseType :: PROPERTY_ID,
                $configuration['course_type']);
            $this->course_type = \application\weblcms\course_type\DataManager :: retrieve(
                CourseType :: class_name(),
                $condition);
        }
        else
        {
            $this->course_type = null;
        }

        return $this->course_type;
    }

    /**
     * Returns the course type id of the selected course type or 0 if no course type is not selected
     *
     * @return int
     */
    protected function get_course_type_id()
    {
        $course_type = $this->get_course_type();
        return $course_type ? $course_type->get_id() : 0;
    }

    /**
     * We need to override this because else we would redirect to the home page
     *
     * @param $parameters
     */
    function get_link($parameters)
    {
        return $this->get_parent()->get_link($parameters, null, null, Redirect :: TYPE_APPLICATION);
    }

    function is_visible()
    {
        if ($this->get_user() instanceof \user\User)
        {
            if ($this->is_empty() && ! $this->show_when_empty())
            {
                return false;
            }

            return true; // i.e.display on homepage when anonymous
        }
        else
        {
            return false;
        }
    }

    function show_when_empty()
    {
        $configuration = $this->get_configuration();

        $result = isset($configuration['show_when_empty']) ? $configuration['show_when_empty'] : true;
        $result = (bool) $result;
        return $result;
    }

    function is_empty()
    {
        return $this->get_renderer()->get_courses()->size() <= 0;
    }

    function show_empty_courses()
    {
        return true;
    }
}

?>