<?php
namespace application\weblcms\integration\core\home;

use application\weblcms\ContentObjectPublication;
use repository\ContentObject;
use repository\ContentObjectRenderer;
use application\weblcms\tool\document\DocumentTool;
use common\libraries\Application;
use common\libraries\Redirect;
use common\libraries\Theme;
use common\libraries\Translation;

/**
 * A notification block for new document submissions (document tool) based on new_assignments.class.php
 *
 * @author Parcifal Aertssen (Howest)
 */
class NewDocuments extends NewBlock
{

    public function display_content()
    {
        $publications = $this->get_content(self :: TOOL_DOCUMENT);
        $html = $this->display_new_items($publications);

        if (count($html) < 3)
        {
            return Translation :: get('NoNewDocumentsSinceLastVisit');
        }
        return implode("\n", $html);
    }

    public function display_new_items($publications)
    {
        ksort($publications);
        $icon = '<img src="' . $this->get_new_documents_icon() . '"/>';

        $html = array();
        $html[] = '<ul style="padding: 0px; margin: 0px 0px 0px 15px;">';
        $current_course_id = - 1;
        foreach ($publications as $publication)
        {
            $course_id = $publication[ContentObjectPublication :: PROPERTY_COURSE_ID];
            $id = $publication[ContentObjectPublication :: PROPERTY_ID];
            $title = $publication[ContentObject :: PROPERTY_TITLE];

            if ($course_id != $current_course_id)
            {
                $current_course_id = $course_id;
                $html[] = '<li>' . $this->get_course_by_id($current_course_id)->get_title() . '</li>';
            }

            $parameters = array(\application\weblcms\WeblcmsManager :: PARAM_COURSE => $course_id,
                    Application :: PARAM_ACTION => \application\weblcms\WeblcmsManager :: ACTION_VIEW_COURSE,
                    \application\weblcms\WeblcmsManager :: PARAM_TOOL => 'document',
                    \application\weblcms\WeblcmsManager :: PARAM_TOOL_ACTION => DocumentTool :: ACTION_VIEW_DOCUMENTS,
                    \application\weblcms\Tool :: PARAM_BROWSER_TYPE => ContentObjectRenderer :: TYPE_TABLE,
                    \application\weblcms\Tool :: PARAM_PUBLICATION_ID => $id);
            $link = Redirect :: get_link(\application\weblcms\WeblcmsManager :: APPLICATION_NAME, $parameters);

            $html[] = '<a href="' . $link . '">' . $icon . ' ' . $title . '</a><br />';
        }
        $html[] = '</ul>';
        return $html;
    }

    private function get_new_documents_icon()
    {
        return Theme :: get_image_path(\application\weblcms\Tool :: get_tool_type_namespace(self :: TOOL_DOCUMENT)) . 'logo/' . Theme :: ICON_MINI . '_new.png';
    }
}
