<?php
namespace application\weblcms\integration\reporting;

use application\weblcms\ContentObjectPublication;
use application\weblcms\course\Course;
use application\weblcms\course\DataManager as CourseDataManager;
use reporting\ReportingData;
use reporting\ReportingChartFormatter;
use reporting\ReportingFormatter;
use common\libraries\DatetimeUtilities;
use common\libraries\EqualityCondition;
use common\libraries\Translation;
use application\weblcms\integration\tracking\DataManager as WeblcmsTrackingDataManager;

class MostActiveInactiveLastDetailBlock extends CourseBlock
{

    public function count_data()
    {
        $reporting_data = new ReportingData();

        $this->add_reporting_data_rows_for_course_visit_data($reporting_data);
        $reporting_data->add_row(Translation :: get('LastPublication'));

        $courses = CourseDataManager :: retrieves(Course :: class_name());
        while ($course = $courses->next_result())
        {
            $condition = new EqualityCondition(ContentObjectPublication :: PROPERTY_COURSE_ID, $course->get_id());
            $publications = \application\weblcms\DataManager :: retrieve_content_object_publications($condition);

            while ($publication = $publications->next_result())
            {
                $last_publication = DatetimeUtilities :: format_locale_date(null, $publication->get_modified_date());
            }

            $reporting_data->add_category($course->get_title());

            $course_visit = WeblcmsTrackingDataManager :: retrieve_course_access_summary_data($course->get_id());
            $this->add_reporting_data_from_course_visit_as_row($course->get_title(), $reporting_data, $course_visit);

            $reporting_data->add_data_category_row(
                $course->get_title(),
                Translation :: get('LastPublication'),
                $last_publication
            );
        }
        return $reporting_data;
    }

    public function retrieve_data()
    {
        return $this->count_data();
    }

    public function get_application()
    {
        return \application\weblcms\WeblcmsManager :: APPLICATION_NAME;
    }

    public function get_views()
    {
        return array(\reporting\viewer\HtmlBlockRendition :: VIEW_TABLE);
    }
}
