<?php
namespace application\weblcms\integration\reporting;

use application\weblcms\course\Course;
use application\weblcms\course\DataManager as CourseDataManager;
use application\weblcms\integration\tracking\CourseVisit;
use reporting\ReportingData;
use reporting\ReportingFormatter;
use common\libraries\Translation;
use application\weblcms\integration\tracking\DataManager as WeblcmsTrackingDataManager;

class MostActiveInactiveLastVisitBlock extends CourseBlock
{

    public function count_data()
    {
        $reporting_data = new ReportingData();

        $course_count = $courses = CourseDataManager :: count(Course :: class_name());

        $reporting_data->set_categories(
            array(
                Translation :: get('Past24hr'),
                Translation :: get('PastWeek'),
                Translation :: get('PastMonth'),
                Translation :: get('PastYear'),
                Translation :: get('MoreThanOneYear'),
                Translation :: get('NeverAccessed')
            )
        );

        $reporting_data->set_rows(array(Translation :: get('TimesAccessed')));

        $past_24_hours = WeblcmsTrackingDataManager :: count_courses_with_last_access_after_time(time() - 86400);
        $past_week = WeblcmsTrackingDataManager :: count_courses_with_last_access_after_time(time() - 604800);
        $past_month = WeblcmsTrackingDataManager :: count_courses_with_last_access_after_time(time() - 18144000);
        $past_year = WeblcmsTrackingDataManager :: count_courses_with_last_access_after_time(time() - 31536000);
        $more_than_one_year = WeblcmsTrackingDataManager :: count_courses_with_last_access_before_time(
            time() - 31536000
        );

        $never = $course_count - ($past_year + $more_than_one_year);

        $reporting_data->add_data_category_row(
            Translation :: get('Past24hr'),
            Translation :: get('TimesAccessed'),
            $past_24_hours
        );

        $reporting_data->add_data_category_row(
            Translation :: get('PastWeek'),
            Translation :: get('TimesAccessed'),
            $past_week
        );

        $reporting_data->add_data_category_row(
            Translation :: get('PastMonth'),
            Translation :: get('TimesAccessed'),
            $past_month
        );

        $reporting_data->add_data_category_row(
            Translation :: get('PastYear'),
            Translation :: get('TimesAccessed'),
            $past_year
        );

        $reporting_data->add_data_category_row(
            Translation :: get('MoreThanOneYear'),
            Translation :: get('TimesAccessed'),
            $more_than_one_year
        );

        $reporting_data->add_data_category_row(
            Translation :: get('NeverAccessed'),
            Translation :: get('TimesAccessed'),
            $never
        );

        return $reporting_data;
    }

    public function retrieve_data()
    {
        return $this->count_data();
    }

    public function get_application()
    {
        return \application\weblcms\WeblcmsManager :: APPLICATION_NAME;
    }

    public function get_views()
    {
        return array(\reporting\viewer\HtmlBlockRendition :: VIEW_TABLE);
    }
}
