<?php
namespace application\weblcms\integration\reporting;

use application\weblcms\ContentObjectPublication;
use common\libraries\EqualityCondition;
use reporting\ReportingData;
use reporting\ReportingFormatter;
use common\libraries\Translation;
use user\UserDataManager;
use common\libraries\Request;
use common\libraries\DatetimeUtilities;
use common\libraries\Utilities;
use common\libraries\PlatformSetting;

/**
 *
 * @package application.weblcms.php.reporting.blocks Reporting block with an overview of all assessment attempts in a
 *          course
 * @author Alexander Van Paemel
 */
class AssessmentOverviewBlock extends ToolBlock
{

    public function count_data()
    {
        $reporting_data = new ReportingData();
        $reporting_data->set_rows(
            array(
                Translation :: get('Name'),
                Translation :: get('OfficialCode', null, 'user'),
                Translation :: get('Title'),
                Translation :: get('Date'),
                Translation :: get('Score')
            )
        );

        $course_id = $this->get_course_id();
        $attempts_tracker = new \application\weblcms\integration\tracking\AssessmentAttempt();
        $condition = new EqualityCondition(
            \application\weblcms\integration\tracking\AssessmentAttempt :: PROPERTY_COURSE_ID,
            $course_id);
        $trackers = $attempts_tracker->retrieve_tracker_items($condition);

        $attempts = array();
        foreach ($trackers as $key => $tracker)
        {
            $attempts[$tracker->get_user_id()][$tracker->get_assessment_id()][] = $tracker;
        }

        $count = 1;
        foreach ($attempts as $key => $user_attempts)
        {
            $user = UserDataManager :: get_instance()->retrieve_user($key);
            foreach ($user_attempts as $key => $pub_attempts)
            {
                $pub = \application\weblcms\DataManager :: retrieve(ContentObjectPublication :: class_name(), $key);

                if(!$pub instanceof ContentObjectPublication)
                {
                    continue;
                }

                $score = $this->get_score($pub_attempts);
                $date = DatetimeUtilities :: format_locale_date(
                    Translation :: get('DateFormatShort', null, Utilities :: COMMON_LIBRARIES) . ', ' .
                         Translation :: get('TimeNoSecFormat', null, Utilities :: COMMON_LIBRARIES),
                        $score['date']);
                $score = $score['score'];
                if ($score < PlatformSetting :: get('passing_percentage'))
                {
                    $score = '<span style="color:red">' . $score . '</span>';
                }
                else
                {
                    $score = '<span style="color:green">' . $score . '</span>';
                }

                $reporting_data->add_category($count);

                $reporting_data->add_data_category_row($count, Translation :: get('Name'), $user->get_fullname());

                $reporting_data->add_data_category_row(
                    $count, Translation :: get('OfficialCode', null, 'user'), $user->get_official_code()
                );

                $reporting_data->add_data_category_row(
                    $count,
                    Translation :: get('Title', null, Utilities :: COMMON_LIBRARIES),
                    $pub->get_content_object()->get_title()
                );

                $reporting_data->add_data_category_row($count, Translation :: get('Date'), $date);
                $reporting_data->add_data_category_row($count, Translation :: get('Score'), $score);
                $count ++;
            }
        }

        $reporting_data->hide_categories();
        return $reporting_data;
    }

    private function get_score($attempts)
    {
        $score_type = (Request :: post('sel')) ? Request :: post('sel') : Request :: get('sel');
        $score = array();

        switch ($score_type)
        {
            case self :: SCORE_TYPE_AVG :
                foreach ($attempts as $attempt)
                {
                    $score['score'] += $attempt->get_total_score();
                }
                $score['score'] = number_format($score['score'] / count($attempts), 1);
                $score['date'] = $attempts[0]->get_start_time();
                return $score;
            case self :: SCORE_TYPE_MIN :
                foreach ($attempts as $attempt)
                {
                    if (count($score) == 0 || $attempt->get_total_score() < $score['score'])
                    {
                        $score['score'] = $attempt->get_total_score();
                        $score['date'] = $attempt->get_start_time();
                    }
                }
                return $score;
            case self :: SCORE_TYPE_MAX :
                foreach ($attempts as $attempt)
                {
                    if (count($score) == 0 || $attempt->get_total_score() > $score['score'])
                    {
                        $score['score'] = $attempt->get_total_score();
                        $score['date'] = $attempt->get_start_time();
                    }
                }
                return $score;
            case self :: SCORE_TYPE_FIRST :
                foreach ($attempts as $attempt)
                {
                    if (count($score) == 0 || $attempt->get_start_time() < $score['date'])
                    {
                        $score['score'] = $attempt->get_total_score();
                        $score['date'] = $attempt->get_start_time();
                    }
                }
                return $score;
            case self :: SCORE_TYPE_LAST :
                foreach ($attempts as $attempt)
                {
                    if (count($score) == 0 || $attempt->get_start_time() > $score['date'])
                    {
                        $score['score'] = $attempt->get_total_score();
                        $score['date'] = $attempt->get_start_time();
                    }
                }
                return $score;
            default :
                foreach ($attempts as $attempt)
                {
                    $score['score'] += $attempt->get_total_score();
                }
                $score['score'] = number_format($score['score'] / count($attempts), 1);
                $score['date'] = $attempts[0]->get_start_time();
                return $score;
        }
    }

    public function retrieve_data()
    {
        return $this->count_data();
    }

    public function get_views()
    {
        return array(\reporting\viewer\HtmlBlockRendition :: VIEW_TABLE);
    }
}
