<?php
namespace application\weblcms\integration\reporting;

use application\weblcms\ContentObjectPublication;
use application\weblcms\Tool;
use application\weblcms\WeblcmsManager;
use application\weblcms\course\Course;
use application\weblcms\course\DataManager as CourseDataManager;
use application\weblcms\integration\tracking\AssessmentAttempt;
use common\libraries\Application;
use common\libraries\DatetimeUtilities;
use common\libraries\PropertyConditionVariable;
use common\libraries\Request;
use common\libraries\Redirect;
use common\libraries\StaticConditionVariable;
use common\libraries\Translation;
use reporting\ReportingFormatter;
use reporting\ReportingData;
use repository\content_object\assessment\Assessment;
use common\libraries\EqualityCondition;
use user\User;
use user\UserDataManager;
use common\libraries\AndCondition;
use common\libraries\PlatformSetting;

/**
 *
 * @package application.weblcms.php.reporting.blocks Reporting block displaying information about the assessment and the
 *          user
 * @author Joris Willems <joris.willems@gmail.com>
 * @author Alexander Van Paemel
 */
class AssessmentUserInformationBlock extends AssessmentUsersBlock
{

    public function count_data()
    {
        $reporting_data = new ReportingData();

        $categories = $this->get_assessment_information_headers();
        $categories = array_merge($categories, $this->get_user_reporting_info_headers());

        $reporting_data->set_categories($categories);

        $user_id = Request :: get('users');
        $user = \user\DataManager :: retrieve(User :: class_name(), $user_id);

        $publication = \application\weblcms\DataManager :: retrieve_by_id(
            ContentObjectPublication :: class_name(),
            $this->get_publication_id()
        );

        $this->add_category_from_array(
            Translation :: get('Details'), $this->get_assessment_information($publication), $reporting_data
        );

        $user_attempts = $this->calculate_user_attempt_summary_data();

        $reporting_info = $this->get_user_reporting_info($user, $user_attempts[$user->get_id()]);
        $this->add_category_from_array(Translation :: get('Details'), $reporting_info, $reporting_data);

        $reporting_data->set_rows(array(Translation :: get('Details')));

        return $reporting_data;
    }

    public function retrieve_data()
    {
        return $this->count_data();
    }

    public function get_views()
    {
        return array(\reporting\viewer\HtmlBlockRendition :: VIEW_TABLE);
    }

    protected function get_assessment_attempts_condition()
    {
        $conditions = array();

        $conditions[] = parent :: get_assessment_attempts_condition();

        $conditions[] = new EqualityCondition(
            new PropertyConditionVariable(
                AssessmentAttempt :: class_name(), AssessmentAttempt :: PROPERTY_USER_ID
            ),
            new StaticConditionVariable($this->get_user_id())
        );

        return new AndCondition($conditions);
    }
}
