<?php
namespace application\weblcms\integration\reporting;

use common\libraries\ObjectTableOrder;
use application\weblcms\integration\tracking\AssignmentSubmission;
use application\weblcms\integration\core\home\AssignmentSubmissions;
use application\weblcms\tool\course_group\CourseGroup;
use \application\weblcms\tool\assignment\DataManager as AssignmentDataManager;

/**
 *
 * @package application.weblcms.php.reporting.blocks Reporting block with an overview of all course groups the
 *          assignment is published for and their submission stats
 * @author Joris Willems <joris.willems@gmail.com>
 * @author Alexander Van Paemel
 * @author Anthony Hurst (Hogeschool Gent)
 */
class AssignmentCourseGroupsBlock extends AssignmentSubmittersBlock
{

    /**
     * #Override (Implementation) Obtains the course groups registered to see the assignment, sorted alphabetically by
     * name.
     *
     * @return array() The course groups registered to see the assignment, sorted alphabetically by name.
     */
    public function get_submitters()
    {
        $order_by = array();
        $order_by[] = new ObjectTableOrder(CourseGroup :: PROPERTY_NAME);
        return \application\weblcms\DataManager :: retrieve_publication_target_course_groups(
            $this->get_publication_id(),
            $this->get_course_id(),
            null,
            null,
            $order_by)->as_array();
    }

    /**
     * #Override (Implementation) Gets the users, course groups and platform groups the assignment was published for
     * plus the number of their submissions and feedbacks
     */
    public function get_submitters_data()
    {
        // Course groups
        $submissions_resultset = AssignmentDataManager :: retrieve_submissions_by_submitter_type(
            $this->get_publication_id(),
            AssignmentSubmission :: SUBMITTER_TYPE_COURSE_GROUP
        );

        $this->get_array_from_resultset($submissions_resultset, $this->submissions);

        $feedbacks_resultset = AssignmentDataManager :: retrieve_submitter_feedbacks(
            $this->get_publication_id(),
            AssignmentSubmission :: SUBMITTER_TYPE_COURSE_GROUP
        );
        $this->get_array_from_resultset($feedbacks_resultset, $this->feedbacks);
    }

    /**
     * Obtains trhe name of the submitter.
     *
     * @param $submitter type The submitter whose name is to be obtained.
     * @return string The name of the submitter.
     */
    public function get_submitter_name($submitter)
    {
        return $submitter->get_name();
    }
}
