<?php
namespace application\weblcms\integration\reporting;

use application\weblcms\ContentObjectPublication;
use application\weblcms\WeblcmsManager;
use reporting\Reporting;
use reporting\ReportingData;
use reporting\ReportingFormatter;
use common\libraries\Application;
use common\libraries\AndCondition;
use common\libraries\EqualityCondition;
use common\libraries\Translation;
use repository\content_object\assignment\Assignment;

/**
 *
 * @package application.weblcms.php.reporting.blocks Reporting block with an overview of all submissions of an
 *          assignment from a user/group
 * @author Joris Willems <joris.willems@gmail.com>
 * @author Alexander Van Paemel
 * @author Anthony Hurst (Hogeschool Gent)
 */
class AssignmentSubmitterSubmissionsBlock extends AssignmentReportingManager
{

    private static $COLUMN_DATE_SUBMITTED;

    private static $COLUMN_INDIVIDUAL_SUBMITTER;

    private static $COLUMN_IP_ADDRESS;

    private static $COLUMN_NUMBER_OF_FEEDBACKS;

    private static $COLUMN_SCORE;

    private static $COLUMN_TITLE;

    /**
     * Instatiates the column headers.
     *
     * @param $parent type Pass-through variable. Please refer to parent class(es) for more details. &param type
     *            $vertical Pass-through variable. Please refer to parent class(es) for more details.
     */
    public function __construct($parent, $vertical)
    {
        self :: $COLUMN_TITLE = Translation :: get('Title');
        self :: $COLUMN_INDIVIDUAL_SUBMITTER = Translation :: get('SubmittedBy');
        self :: $COLUMN_DATE_SUBMITTED = Translation :: get('DateSubmitted');
        self :: $COLUMN_SCORE = Translation :: get('Score');
        self :: $COLUMN_NUMBER_OF_FEEDBACKS = Translation :: get('NumberOfFeedbacks');
        self :: $COLUMN_IP_ADDRESS = Translation :: get('IpAddress');
        parent :: __construct($parent, $vertical);
    }

    public function count_data()
    {
        $reporting_data = new ReportingData();
        $reporting_data_headers = array(
            self :: $COLUMN_TITLE,
            self :: $COLUMN_DATE_SUBMITTED,
            self :: $COLUMN_SCORE,
            self :: $COLUMN_NUMBER_OF_FEEDBACKS,
            self :: $COLUMN_IP_ADDRESS);
        if ($this->get_submitter_type() !=
             \application\weblcms\integration\tracking\AssignmentSubmission :: SUBMITTER_TYPE_USER)
        {
            array_splice($reporting_data_headers, 1, 0, array(self :: $COLUMN_INDIVIDUAL_SUBMITTER));
        }
        $reporting_data->set_rows($reporting_data_headers);

        $assignment = \application\weblcms\DataManager :: retrieve(
            ContentObjectPublication :: class_name(),
            $this->get_publication_id())->get_content_object();

        $conditions = array();

        $conditions[] = new EqualityCondition(
            \application\weblcms\integration\tracking\AssignmentSubmission :: PROPERTY_SUBMITTER_ID,
            $this->get_target_id());

        $conditions[] = new EqualityCondition(
            \application\weblcms\integration\tracking\AssignmentSubmission :: PROPERTY_SUBMITTER_TYPE,
            $this->get_submitter_type());

        $conditions[] = new EqualityCondition(
            \application\weblcms\integration\tracking\AssignmentSubmission :: PROPERTY_PUBLICATION_ID,
            $this->get_publication_id());

        $condition = new AndCondition($conditions);

        $submission_trackers = \application\weblcms\integration\tracking\AssignmentSubmission :: get_data(
            \application\weblcms\integration\tracking\AssignmentSubmission :: CLASS_NAME,
            WeblcmsManager :: APPLICATION_NAME,
            $condition)->as_array();

        foreach ($submission_trackers as $key => $submission_tracker)
        {
            $score = null;

            $date_submitted = $this->format_date_html(
                $submission_tracker->get_date_submitted(),
                $assignment->get_end_time());

            $condition = new EqualityCondition(
                \application\weblcms\integration\tracking\SubmissionScore :: PROPERTY_SUBMISSION_ID,
                $submission_tracker->get_id());

            $score_trackers = \application\weblcms\integration\tracking\SubmissionScore :: get_data(
                \application\weblcms\integration\tracking\SubmissionScore :: CLASS_NAME,
                WeblcmsManager :: APPLICATION_NAME,
                $condition)->as_array();

            if ($score_trackers[0])
            {
                $score = $this->format_score_html($score_trackers[0]->get_score());
            }

            $condition = new EqualityCondition(
                \application\weblcms\integration\tracking\SubmissionFeedback :: PROPERTY_SUBMISSION_ID,
                $submission_tracker->get_id());

            $number_feedbacks = \application\weblcms\integration\tracking\SubmissionFeedback :: count_data(
                \application\weblcms\integration\tracking\SubmissionFeedback :: CLASS_NAME,
                WeblcmsManager :: APPLICATION_NAME,
                $condition);

            if ($this->get_submitter_type() !=
                 \application\weblcms\integration\tracking\AssignmentSubmission :: SUBMITTER_TYPE_USER)
            {
                // Link to the User
                $individual_submitter = $this->generate_user_name_link($submission_tracker->get_user_id());
            }
            // Link to submission details.
            $title = $this->generate_submission_title_link($submission_tracker);

            $reporting_data->add_category($key);
            $reporting_data->add_data_category_row($key, self :: $COLUMN_TITLE, $title);
            $reporting_data->add_data_category_row($key, self :: $COLUMN_DATE_SUBMITTED, $date_submitted);
            $reporting_data->add_data_category_row($key, self :: $COLUMN_SCORE, $score);
            $reporting_data->add_data_category_row($key, self :: $COLUMN_NUMBER_OF_FEEDBACKS, $number_feedbacks);
            $reporting_data->add_data_category_row(
                $key,
                self :: $COLUMN_IP_ADDRESS,
                $submission_tracker->get_ip_address());

            if ($this->get_submitter_type() !=
                 \application\weblcms\integration\tracking\AssignmentSubmission :: SUBMITTER_TYPE_USER)
            {
                $reporting_data->add_data_category_row(
                    $key,
                    self :: $COLUMN_INDIVIDUAL_SUBMITTER,
                    $individual_submitter);
            }
        }
        $reporting_data->hide_categories();
        return $reporting_data;
    }

    public function retrieve_data()
    {
        return $this->count_data();
    }

    public function get_views()
    {
        return array(\reporting\viewer\HtmlBlockRendition :: VIEW_TABLE);
    }
}
