<?php
namespace application\weblcms\integration\reporting;

use application\weblcms\WeblcmsManager;
use application\weblcms\ContentObjectPublication;
use application\weblcms\tool\assignment\AssignmentTool;
use common\libraries\Translation;
use common\libraries\EqualityCondition;
use common\libraries\Application;
use common\libraries\Redirect;
use common\libraries\Utilities;
use common\libraries\DatetimeUtilities;
use common\libraries\InCondition;
use common\libraries\AndCondition;
use common\libraries\Theme;
use reporting\ReportingData;
use reporting\ReportingFormatter;
use reporting\ReportingManager;
use reporting\Reporting;
use repository\content_object\assignment\Assignment;

/**
 *
 * @package application.weblcms.php.reporting.blocks Reporting block with an overiew of the assignments the user has
 *          sent a submission for
 * @author Joris Willems <joris.willems@gmail.com>
 * @author Alexander Van Paemel
 */
class CourseUserAssignmentInformationBlock extends ToolBlock
{

    public function count_data()
    {
        $reporting_data = new ReportingData();
        $reporting_data->set_rows(
            array(
                Translation :: get('Title'),
                Translation :: get('NumberOfSubmissions'),
                Translation :: get('LastSubmission'),
                Translation :: get('NumberOfFeedbacks'),
                Translation :: get('AverageScore'),
                Translation :: get('Submissions')));

        $user_id = $this->get_user_id();
        $subm_tracker = new \application\weblcms\integration\tracking\AssignmentSubmission();
        $conditions = array();

        $conditions[] = new EqualityCondition(
            \application\weblcms\integration\tracking\AssignmentSubmission :: PROPERTY_SUBMITTER_ID,
            $user_id);

        $conditions[] = new EqualityCondition(
            \application\weblcms\integration\tracking\AssignmentSubmission :: PROPERTY_SUBMITTER_TYPE,
            \application\weblcms\integration\tracking\AssignmentSubmission :: SUBMITTER_TYPE_USER);

        $condition = new AndCondition($conditions);
        $submissions = $subm_tracker->retrieve_tracker_items($condition);

        $feedback_tracker = new \application\weblcms\integration\tracking\SubmissionFeedback();
        $score_tracker = new \application\weblcms\integration\tracking\SubmissionScore();
        $img = '<img src="' . Theme :: get_common_image_path() . 'action_reporting.png" title="' .
             Translation :: get('Details') . '" />';

        $pub_submissions = array();
        foreach ($submissions as $key => $submission)
        {
            $pub_submissions[$submission->get_publication_id()]['count'] ++;
            $pub_submissions[$submission->get_publication_id()]['subm_ids'][] = $submission->get_id();

            if ($pub_submissions[$submission->get_publication_id()]['last'] == null ||
                 $submission->get_date_submitted() > $pub_submissions[$submission->get_publication_id()]['last'])
            {
                $pub_submissions[$submission->get_publication_id()]['last'] = $submission->get_date_submitted();
            }
        }

        $conditions = array();

        $conditions[] = new EqualityCondition(
            ContentObjectPublication :: PROPERTY_TOOL, Assignment :: get_type_name()
        );

        $conditions[] = new EqualityCondition(ContentObjectPublication :: PROPERTY_COURSE_ID, $this->get_course_id());
        $condition = new AndCondition($conditions);
        $publications_resultset = \application\weblcms\DataManager :: retrieve_content_object_publications($condition);

        $params = array();
        $params[Application :: PARAM_ACTION] = WeblcmsManager :: ACTION_VIEW_COURSE;
        $params[Application :: PARAM_APPLICATION] = WeblcmsManager :: APPLICATION_NAME;
        $params[WeblcmsManager :: PARAM_COURSE] = $this->get_course_id();
        $params[WeblcmsManager :: PARAM_TOOL] = Assignment :: get_type_name();
        $params[WeblcmsManager :: PARAM_TOOL_ACTION] = AssignmentTool :: ACTION_BROWSE_SUBMITTERS;

        $params_detail = $this->get_parent()->get_parameters();
        $params_detail[ReportingManager :: PARAM_TEMPLATE_ID] = CourseSubmitterSubmissionsTemplate :: class_name();
        $params_detail[AssignmentTool :: PARAM_TARGET_ID] = $user_id;
        $params_detail[AssignmentTool :: PARAM_SUBMITTER_TYPE] = \application\weblcms\integration\tracking\AssignmentSubmission :: SUBMITTER_TYPE_USER;

        $key = 0;
        while ($publication = $publications_resultset->next_result())
        {
            if (! \application\weblcms\DataManager :: is_publication_target_user(
                $user_id,
                $publication[ContentObjectPublication :: PROPERTY_ID]))
            {
                continue;
            }
            ++ $key;
            $feedback_count = $score_display = $last = $link = null;

            $assignment = \repository\DataManager :: retrieve_content_object(
                $publication[ContentObjectPublication :: PROPERTY_CONTENT_OBJECT_ID]);

            $params[WeblcmsManager :: PARAM_PUBLICATION] = $publication[ContentObjectPublication :: PROPERTY_ID];
            $url_title = Redirect :: get_url($params);

            if ($pub_submissions[$publication[ContentObjectPublication :: PROPERTY_ID]])
            {

                $last = DatetimeUtilities :: format_locale_date(
                    Translation :: get('DateFormatShort', null, Utilities :: COMMON_LIBRARIES) . ', ' .
                         Translation :: get('TimeNoSecFormat', null, Utilities :: COMMON_LIBRARIES),
                        $pub_submissions[$publication[ContentObjectPublication :: PROPERTY_ID]]['last']);

                $params_detail[WeblcmsManager :: PARAM_PUBLICATION] = $publication[ContentObjectPublication :: PROPERTY_ID];
                $link = '<a href="' . $this->get_parent()->get_url($params_detail) . '">' . $img . '</a>';

                $condition = new InCondition(
                    \application\weblcms\integration\tracking\SubmissionFeedback :: PROPERTY_SUBMISSION_ID,
                    $pub_submissions[$publication[ContentObjectPublication :: PROPERTY_ID]]['subm_ids']);

                $feedback_count = $feedback_tracker->count_tracker_items($condition);

                $score_display = null;

                $condition = new InCondition(
                    \application\weblcms\integration\tracking\SubmissionScore :: PROPERTY_SUBMISSION_ID,
                    $pub_submissions[$publication[ContentObjectPublication :: PROPERTY_ID]]['subm_ids']);

                $scores = $score_tracker->retrieve_tracker_items($condition);
                foreach ($scores as $score)
                {
                    $score_display += $score->get_score();
                }
                if ($scores != null)
                {
                    $score_display = $this->get_score_bar($score_display / count($scores));
                }
            }
            else
            {
                $pub_submissions[$publication[ContentObjectPublication :: PROPERTY_ID]]['count'] = 0;
            }
            $reporting_data->add_category($key);
            $reporting_data->add_data_category_row(
                $key,
                Translation :: get('Title'),
                '<a href="' . $url_title . '">' . $assignment->get_title() . '</a>');
            $reporting_data->add_data_category_row(
                $key,
                Translation :: get('NumberOfSubmissions'),
                $pub_submissions[$publication[ContentObjectPublication :: PROPERTY_ID]]['count']);
            $reporting_data->add_data_category_row($key, Translation :: get('LastSubmission'), $last);
            $reporting_data->add_data_category_row($key, Translation :: get('NumberOfFeedbacks'), $feedback_count);
            $reporting_data->add_data_category_row($key, Translation :: get('AverageScore'), $score_display);
            $reporting_data->add_data_category_row($key, Translation :: get('Submissions'), $link);
        }
        $reporting_data->hide_categories();
        return $reporting_data;
    }

    public function retrieve_data()
    {
        return $this->count_data();
    }

    public function get_views()
    {
        return array(\reporting\viewer\HtmlBlockRendition :: VIEW_TABLE);
    }
}
