<?php
namespace application\weblcms\integration\reporting;

use application\weblcms\Tool;
use application\weblcms\WeblcmsManager;
use application\weblcms\ContentObjectPublication;
use common\libraries\EqualityCondition;
use common\libraries\AndCondition;
use common\libraries\Application;
use common\libraries\Redirect;
use common\libraries\Translation;
use common\libraries\Theme;
use common\libraries\Utilities;
use reporting\ReportingManager;
use reporting\Reporting;
use reporting\ReportingData;
use reporting\ReportingFormatter;
use repository\content_object\learning_path\LearningPath;

/**
 *
 * @package application.weblcms.php.reporting.blocks Reporting block displaying all learning paths within a course and
 *          their attempt stats
 * @author Joris Willems <joris.willems@gmail.com>
 * @author Alexander Van Paemel
 */
class LearningPathBlock extends CourseBlock
{

    public function count_data()
    {
        $reporting_data = new ReportingData();
        $reporting_data->set_rows(
            array(
                Translation :: get('Title'),
                Translation :: get('NumberOfUsersAttempted'),
                Translation :: get('AverageProgress'),
                Translation :: get('LearningPathDetails')));

        $course_id = $this->get_course_id();
        $tool = Utilities :: get_classname_from_namespace(LearningPath :: class_name(), true);
        $tracker = new \application\weblcms\integration\tracking\LpAttempt();
        $img = '<img src="' . Theme :: get_common_image_path() . 'action_reporting.png" title="' .
             Translation :: get('Details') . '" />';
        $count = 1;

        $conditions = array();
        $conditions[] = new EqualityCondition(ContentObjectPublication :: PROPERTY_COURSE_ID, $course_id);
        $conditions[] = new EqualityCondition(ContentObjectPublication :: PROPERTY_TOOL, $tool);
        $condition = new AndCondition($conditions);
        $pub_resultset = \application\weblcms\DataManager :: retrieve_content_object_publications($condition);

        while ($pub = $pub_resultset->next_result())
        {
            $condition = new EqualityCondition(
                \application\weblcms\integration\tracking\LpAttempt :: PROPERTY_LP_ID,
                $pub[ContentObjectPublication :: PROPERTY_ID]);
            $attempts = $tracker->retrieve_tracker_items($condition);

            $progress = null;
            foreach ($attempts as $attempt)
            {
                $progress += $attempt->get_progress();
            }
            if (count($attempts) != 0)
            {
                $progress = $this->get_progress_bar($progress / count($attempts));
            }

            $params = $this->get_parent()->get_parameters();
            $params[ReportingManager :: PARAM_TEMPLATE_ID] = LearningPathAttemptsTemplate :: class_name();
            $params[WeblcmsManager :: PARAM_PUBLICATION] = $pub[ContentObjectPublication :: PROPERTY_ID];
            $link = '<a href="' . $this->get_parent()->get_url($params) . '">' . $img . '</a>';

            $params = array();
            $params[Application :: PARAM_ACTION] = WeblcmsManager :: ACTION_VIEW_COURSE;
            $params[Application :: PARAM_APPLICATION] = WeblcmsManager :: APPLICATION_NAME;
            $params[WeblcmsManager :: PARAM_COURSE] = $course_id;
            $params[WeblcmsManager :: PARAM_TOOL] = $tool;
            $params[WeblcmsManager :: PARAM_PUBLICATION] = $pub[ContentObjectPublication :: PROPERTY_ID];
            $params[WeblcmsManager :: PARAM_TOOL_ACTION] = Tool :: ACTION_VIEW;
            $url_title = Redirect :: get_url($params);

            $content_object = \repository\DataManager :: retrieve_content_object(
                $pub[ContentObjectPublication :: PROPERTY_CONTENT_OBJECT_ID]);

            $reporting_data->add_category($count);
            $reporting_data->add_data_category_row(
                $count,
                Translation :: get('Title'),
                '<a href="' . $url_title . '">' . $content_object->get_title() . '</a>');
            $reporting_data->add_data_category_row(
                $count,
                Translation :: get('NumberOfUsersAttempted'),
                count($attempts));
            $reporting_data->add_data_category_row($count, Translation :: get('AverageProgress'), $progress);
            $reporting_data->add_data_category_row($count, Translation :: get('LearningPathDetails'), $link);

            $count ++;
        }
        $reporting_data->hide_categories();

        return $reporting_data;
    }

    public function retrieve_data()
    {
        return $this->count_data();
    }

    public function get_views()
    {
        return array(\reporting\viewer\HtmlBlockRendition :: VIEW_TABLE);
    }
}
