<?php
namespace application\weblcms\integration\reporting;

use common\libraries\Utilities;
use application\weblcms\ContentObjectPublication;
use application\weblcms\Tool;
use application\weblcms\WeblcmsManager;
use application\weblcms\course\Course;
use application\weblcms\course\DataManager as CourseDataManager;
use common\libraries\Application;
use common\libraries\Redirect;
use common\libraries\Translation;
use reporting\ReportingFormatter;
use reporting\ReportingData;
use common\libraries\EqualityCondition;
use user\UserDataManager;
use common\libraries\AndCondition;
use repository\content_object\learning_path\LearningPath;

/**
 *
 * @package application.weblcms.php.reporting.blocks Reporting block displaying information about the assessment and
 *          access details
 * @author Joris Willems <joris.willems@gmail.com>
 * @author Alexander Van Paemel
 */
class LearningPathAttemptProgressInformationBlock extends ToolBlock
{

    public function count_data()
    {
        $reporting_data = new ReportingData();
        $reporting_data->set_categories(
            array(
                Translation :: get('Title'),
                Translation :: get('Course'),
                Translation :: get('User'),
                Translation :: get('Progress')));

        $publication_id = $this->get_publication_id();
        $course_id = $this->get_course_id();
        $user_id = $this->get_user_id();

        $user_data_manager = UserDataManager :: get_instance();
        $user = $user_data_manager->retrieve_user($user_id);

        $publication = \application\weblcms\DataManager :: retrieve(
            ContentObjectPublication :: class_name(),
            $publication_id);

        $course = CourseDataManager :: retrieve_by_id(Course :: class_name(), $course_id);

        $assessment = $publication->get_content_object();
        $conditions = array();
        $conditions[] = new EqualityCondition(
            \application\weblcms\integration\tracking\LpAttempt :: PROPERTY_LP_ID,
            $publication_id);
        $conditions[] = new EqualityCondition(
            \application\weblcms\integration\tracking\LpAttempt :: PROPERTY_USER_ID,
            $user_id);
        $condition = new AndCondition($conditions);
        $attempts_tracker = new \application\weblcms\integration\tracking\LpAttempt();
        $lp_attempts = $attempts_tracker->retrieve_tracker_items($condition);

        $progress = $this->get_progress_bar($lp_attempts[0]->get_progress());

        $params = array();
        $params[Application :: PARAM_ACTION] = WeblcmsManager :: ACTION_VIEW_COURSE;
        $params[Application :: PARAM_APPLICATION] = WeblcmsManager :: APPLICATION_NAME;
        $params[WeblcmsManager :: PARAM_COURSE] = $course_id;
        $params[WeblcmsManager :: PARAM_TOOL] = Utilities :: get_classname_from_namespace(LearningPath :: class_name(), true);
        $params[WeblcmsManager :: PARAM_PUBLICATION] = $publication_id;
        $params[WeblcmsManager :: PARAM_TOOL_ACTION] = Tool :: ACTION_VIEW;
        $url_title = Redirect :: get_url($params);

        $reporting_data->set_rows(array(Translation :: get('Details')));
        $reporting_data->add_data_category_row(
            Translation :: get('Title'),
            Translation :: get('Details'),
            '<a href="' . $url_title . '">' . $assessment->get_title() . '</a>');
        $reporting_data->add_data_category_row(
            Translation :: get('Course'),
            Translation :: get('Details'),
            $course->get_title());
        $reporting_data->add_data_category_row(
            Translation :: get('User'),
            Translation :: get('Details'),
            $user->get_fullname());
        $reporting_data->add_data_category_row(Translation :: get('Progress'), Translation :: get('Details'), $progress);
        return $reporting_data;
    }

    public function retrieve_data()
    {
        return $this->count_data();
    }

    public function get_views()
    {
        return array(\reporting\viewer\HtmlBlockRendition :: VIEW_TABLE);
    }
}
