<?php
namespace application\weblcms\integration\reporting;

use common\libraries\Utilities;
use application\weblcms\ContentObjectPublication;
use reporting\Reporting;
use reporting\ReportingData;
use reporting\ReportingFormatter;
use common\libraries\Application;
use common\libraries\AndCondition;
use common\libraries\EqualityCondition;
use common\libraries\Path;
use common\libraries\Translation;
use common\libraries\ObjectTableOrder;
use repository\content_object\learning_path\LearningPath;
use application\weblcms\course\DataManager as CourseDataManager;

/**
 *
 * @package application.weblcms.php.reporting.blocks Reporting block with an overview of progress of each learning path
 *          per user
 * @author Joris Willems <joris.willems@gmail.com>
 * @author Alexander Van Paemel
 */
class LearningPathProgressUsersBlock extends ToolBlock
{

    public function count_data()
    {
        $reporting_data = new ReportingData();
        $course_id = $this->get_course_id();
        $users = CourseDataManager :: get_instance()->retrieve_all_course_users($course_id)->as_array();

        $conditions = array();
        $conditions[] = new EqualityCondition(ContentObjectPublication :: PROPERTY_COURSE_ID, $course_id);
        $conditions[] = new EqualityCondition(
            ContentObjectPublication :: PROPERTY_TOOL,
            Utilities :: get_classname_from_namespace(LearningPath :: class_name(), true));
        $condition = new AndCondition($conditions);

        $order_by = new ObjectTableOrder(ContentObjectPublication :: PROPERTY_MODIFIED_DATE, SORT_DESC);

        $publication_resultset = \application\weblcms\DataManager :: retrieve_content_object_publications(
            $condition,
            $order_by);

        $publications = array();
        $headings = array();
        $headings[] = Translation :: get('Name');
        while ($publication = $publication_resultset->next_result())
        {
            $publications[] = $publication;
            $content_object = \repository\DataManager :: retrieve_content_object(
                $publication[ContentObjectPublication :: PROPERTY_CONTENT_OBJECT_ID]);

            if ($publication_resultset->size() > 5)
            {
                $headings[] = substr($content_object->get_title(), 0, 14);
            }
            else
            {
                $headings[] = $content_object->get_title();
            }
        }

        $reporting_data->set_rows($headings);

        foreach ($users as $key => $user)
        {
            $reporting_data->add_category($key);
            $reporting_data->add_data_category_row(
                $key,
                Translation :: get('Name'),
                \user\User :: fullname($user[\user\User :: PROPERTY_FIRSTNAME], $user[\user\User :: PROPERTY_LASTNAME]));

            foreach ($publications as $publication)
            {
                $content_object = \repository\DataManager :: retrieve_content_object(
                    $publication[ContentObjectPublication :: PROPERTY_CONTENT_OBJECT_ID]);

                if ($publication_resultset->size() > 5)
                {
                    $title = substr($content_object->get_title(), 0, 14);
                }
                else
                {
                    $title = $content_object->get_title();
                }

                $tracker = new \application\weblcms\integration\tracking\LpAttempt();
                $conditions = array();
                $conditions[] = new EqualityCondition(
                    \application\weblcms\integration\tracking\LpAttempt :: PROPERTY_LP_ID,
                    $publication[ContentObjectPublication :: PROPERTY_ID]);
                $conditions[] = new EqualityCondition(
                    \application\weblcms\integration\tracking\LpAttempt :: PROPERTY_USER_ID,
                    $user[\user\User :: PROPERTY_ID]);
                $condition = new AndCondition($conditions);
                $attempt_trackers = $tracker->retrieve_tracker_items($condition);

                if (count($attempt_trackers) < 1)
                {
                    if (\application\weblcms\DataManager :: is_publication_target_user(
                        $user->get_id(),
                        $publication[ContentObjectPublication :: PROPERTY_ID],
                        $course_id))
                    {
                        $reporting_data->add_data_category_row($key, $title, null);
                        continue;
                    }

                    $reporting_data->add_data_category_row($key, $title, 'X');
                    continue;
                }

                $progress = $attempt_trackers[0]->get_progress();

                switch ($progress)
                {
                    case 0 :
                        $progress = '<span style="color:red">' . $progress . '%</span>';
                        break;
                    case 100 :
                        $progress = '<span style="color:green">' . $progress . '%</span>';
                        break;
                    default :
                        $progress = '<span style="color:orange">' . $progress . '%</span>';
                        break;
                }

                $reporting_data->add_data_category_row($key, $title, $progress);
            }
        }
        $reporting_data->hide_categories();
        return $reporting_data;
    }

    public function retrieve_data()
    {
        return $this->count_data();
    }

    public function get_views()
    {
        return array(\reporting\viewer\HtmlBlockRendition :: VIEW_TABLE);
    }
}
