<?php
namespace application\weblcms\integration\reporting;

use application\weblcms\ContentObjectPublication;
use application\weblcms\Tool;
use application\weblcms\WeblcmsManager;
use application\weblcms\integration\tracking\CourseVisit;
use application\weblcms\tool\reporting\ReportingTool;
use common\libraries\AndCondition;
use common\libraries\DatetimeUtilities;
use common\libraries\EqualityCondition;
use common\libraries\Path;
use common\libraries\Translation;
use common\libraries\Request;
use common\libraries\Redirect;
use common\libraries\Session;
use reporting\ReportingData;
use reporting\ReportingFormatter;
use application\weblcms\integration\tracking\DataManager as WeblcmsTrackingDataManager;
use user\User;

class PublicationAccessBlock extends ToolBlock
{

    public function count_data()
    {
        $reporting_data = new ReportingData();

        $reporting_data->set_rows(
            array(
                Translation :: get('User', null, 'user'),
                Translation :: get('OfficialCode', null, 'user')
            )
        );

        $this->add_reporting_data_rows_for_course_visit_data($reporting_data);

        $course_visits = $this->retrieve_course_visits();

        for ($counter = 0; $counter < $course_visits->size(); $counter ++)
        {
            $reporting_data->add_category($counter);
        }

        $counter = 0;

        while($course_visit = $course_visits->next_result())
        {
            $user = \user\DataManager :: retrieve_by_id(User :: class_name(), $course_visit->get_user_id());

            $reporting_data->add_data_category_row(
                $counter, Translation :: get('User', null, 'user'), $user->get_fullname()
            );

            $reporting_data->add_data_category_row(
                $counter, Translation :: get('OfficialCode', null, 'user'), $user->get_official_code()
            );

            $this->add_reporting_data_from_course_visit_as_row($counter, $reporting_data, $course_visit);

            $counter++;
        }

        $reporting_data->hide_categories();
        return $reporting_data;
    }

    public function retrieve_data()
    {
        return $this->count_data();
    }

    /**
     * Retrieves the course visit records for the given publication
     *
     * @return \common\libraries\ResultSet
     */
    public function retrieve_course_visits()
    {
        $content_object_publication_id = $this->get_publication_id();
        $content_object_publication = \application\weblcms\DataManager :: retrieve(
            ContentObjectPublication :: class_name(), $content_object_publication_id
        );

        $category_id = $content_object_publication->get_category_id();
        $category_id = $category_id ? $category_id : null;

        $tool_id = $this->get_tool_registration($content_object_publication->get_tool())->get_id();

        $condition = WeblcmsTrackingDataManager :: get_course_visit_conditions_by_course_data(
            $content_object_publication->get_course_id(), $tool_id, $category_id, $content_object_publication_id,
            false
        );

        return WeblcmsTrackingDataManager :: retrieves(CourseVisit :: class_name(), $condition);
    }

    public function get_application()
    {
        return \application\weblcms\WeblcmsManager :: APPLICATION_NAME;
    }

    public function get_views()
    {
        return array(\reporting\viewer\HtmlBlockRendition :: VIEW_TABLE);
    }
}
