<?php
namespace application\weblcms\integration\reporting;

use application\weblcms\WeblcmsManager;
use application\weblcms\ContentObjectPublication;
use application\weblcms\Tool;
use reporting\ReportingData;
use reporting\ReportingFormatter;
use common\libraries\PatternMatchCondition;
use common\libraries\DatetimeUtilities;
use common\libraries\Utilities;
use common\libraries\Request;
use common\libraries\Translation;
use common\libraries\Application;
use common\libraries\Redirect;
use application\weblcms\tool\reporting\ReportingTool;

class PublicationDetailBlock extends ToolBlock
{

    public function count_data()
    {
        $reporting_data = new ReportingData();

        $course_id = $this->get_course_id();
        $tool = Request :: get(ReportingTool :: PARAM_REPORTING_TOOL);
        $pid = Request :: get(Tool :: PARAM_PUBLICATION_ID);

        $content_object_publication = \application\weblcms\DataManager :: retrieve(ContentObjectPublication :: class_name(), $pid);

        if (empty($content_object_publication))
        {
            $content_object_publication = \repository\DataManager :: retrieve_content_object($pid);
            $title = $content_object_publication->get_title();
            $id = $content_object_publication->get_id();
            $descr = $content_object_publication->get_description();
        }
        else
        {
            $title = $content_object_publication->get_content_object()->get_title();
            $id = $pid;
            $descr = $content_object_publication->get_content_object()->get_description();
        }

        $params = array();
        $params[Application :: PARAM_ACTION] = WeblcmsManager :: ACTION_VIEW_COURSE;
        $params[Application :: PARAM_APPLICATION] = WeblcmsManager :: APPLICATION_NAME;
        $params[WeblcmsManager :: PARAM_COURSE] = $course_id;
        $params[WeblcmsManager :: PARAM_TOOL] = $tool;
        $params[WeblcmsManager :: PARAM_PUBLICATION] = $id;
        $params[WeblcmsManager :: PARAM_TOOL_ACTION] = Tool :: ACTION_VIEW;
        $url = Redirect :: get_url($params);

        $reporting_data->set_categories(
            array(
                Translation :: get('Title'),
                Translation :: get('Description'),
            )
        );

        $this->add_reporting_data_categories_for_course_visit_data($reporting_data);

        $reporting_data->set_rows(array(Translation :: get('count')));

        $reporting_data->add_data_category_row(
            Translation :: get('Title'),
            Translation :: get('count'),
            '<a href="' . $url . '">' . $title . '</a>'
        );

        $reporting_data->add_data_category_row(
            Translation :: get('Description'),
            Translation :: get('count'),
            Utilities :: truncate_string($descr, 50)
        );

        $course_visit = $this->get_course_visit_summary_from_publication($content_object_publication);

        $this->add_reporting_data_from_course_visit_as_category(
            Translation :: get('count'), $reporting_data, $course_visit
        );

        return $reporting_data;
    }

    public function retrieve_data()
    {
        return $this->count_data();
    }

    public function get_application()
    {
        return \application\weblcms\WeblcmsManager :: APPLICATION_NAME;
    }

    public function get_views()
    {
        return array(\reporting\viewer\HtmlBlockRendition :: VIEW_TABLE);
    }
}
