<?php
namespace application\weblcms\integration\reporting;

use application\weblcms\ContentObjectPublication;
use application\weblcms\WeblcmsManager;
use application\weblcms\Tool;
use application\weblcms\CourseSetting;
use application\weblcms\CourseSettingsController;
use application\weblcms\CourseTool;
use application\weblcms\integration\tracking\CourseVisit;
use application\weblcms\tool\reporting\ReportingTool;
use common\libraries\PropertyConditionVariable;
use common\libraries\Redirect;
use common\libraries\Application;
use common\libraries\StaticConditionVariable;
use common\libraries\Theme;
use common\libraries\Utilities;
use common\libraries\AndCondition;
use common\libraries\EqualityCondition;
use common\libraries\Toolbar;
use common\libraries\ToolbarItem;
use common\libraries\Translation;
use common\libraries\DatetimeUtilities;
use reporting\Reporting;
use reporting\ReportingFormatter;
use reporting\ReportingManager;
use repository\content_object\assignment\Assignment;
use repository\content_object\assessment\Assessment;
use repository\content_object\learning_path\LearningPath;
use application\weblcms\integration\tracking\DataManager as WeblcmsTrackingDataManager;

/**
 *
 * @package application.weblcms.php.reporting.blocks Reporting block with an overiew of the tools and their access
 *          details
 * @author Joris Willems <joris.willems@gmail.com>
 * @author Alexander Van Paemel
 */
class LastAccessToToolsBlock extends ToolAccessBlock
{

    public function count_data()
    {
        $reporting_data = parent :: count_data();

        $reporting_data->add_row(Translation :: get('Actions'));

        $course_id = $this->get_course_id();
        $user_id = $this->get_user_id();

        $tool_names = $reporting_data->get_categories();
        foreach($tool_names as $tool_name)
        {
            $publications = $this->count_tool_publications($tool_name);
            if ($publications > 0)
            {
                $params = $this->get_parent()->get_parameters();
                $params[ReportingManager :: PARAM_TEMPLATE_ID] = ToolPublicationsDetailTemplate :: class_name();
                $params[WeblcmsManager :: PARAM_USERS] = $user_id;
                $params[ReportingTool :: PARAM_REPORTING_TOOL] = $tool_name;
                $url = $this->get_parent()->get_url($params);

                $toolbar = new Toolbar();
                $toolbar->add_item(
                    new ToolbarItem(
                        Translation :: get('ViewPublications'),
                        Theme :: get_common_image_path() . 'action_reporting.png',
                        $url,
                        ToolbarItem :: DISPLAY_ICON));

                switch ($tool_name)
                {
                    case Utilities :: get_classname_from_namespace(Assignment :: class_name(), true) :
                        $params = $this->get_parent()->get_parameters();
                        $params[ReportingManager :: PARAM_TEMPLATE_ID] = AssignmentScoresTemplate :: class_name();
                        $params[WeblcmsManager :: PARAM_COURSE] = $course_id;

                        $url_detail = $this->get_parent()->get_url($params);

                        $toolbar->add_item(
                            new ToolbarItem(
                                Translation :: get('ViewScores'),
                                Theme :: get_common_image_path() . 'action_view_results.png',
                                $url_detail,
                                ToolbarItem :: DISPLAY_ICON));

                        break;
                    case Utilities :: get_classname_from_namespace(Assessment :: class_name(), true) :
                        $params = $this->get_parent()->get_parameters();
                        $params[ReportingManager :: PARAM_TEMPLATE_ID] = AssessmentScoresTemplate :: class_name();
                        $params[WeblcmsManager :: PARAM_COURSE] = $course_id;

                        $url_detail = $this->get_parent()->get_url($params);

                        $toolbar->add_item(
                            new ToolbarItem(
                                Translation :: get('ViewScores'),
                                Theme :: get_common_image_path() . 'action_view_results.png',
                                $url_detail,
                                ToolbarItem :: DISPLAY_ICON));

                        break;
                    case Utilities :: get_classname_from_namespace(LearningPath :: class_name(), true) :
                        $params = $this->get_parent()->get_parameters();
                        $params[ReportingManager :: PARAM_TEMPLATE_ID] = LearningPathProgressTemplate :: class_name();
                        $params[WeblcmsManager :: PARAM_COURSE] = $course_id;

                        $url_detail = $this->get_parent()->get_url($params);

                        $toolbar->add_item(
                            new ToolbarItem(
                                Translation :: get('ViewProgressUsers'),
                                Theme :: get_common_image_path() . 'action_view_results.png',
                                $url_detail,
                                ToolbarItem :: DISPLAY_ICON));
                        break;
                }

                $reporting_data->add_data_category_row($tool_name, Translation :: get('Actions'), $toolbar->as_html());
            }
       }

        return $reporting_data;
    }

    /**
     * Returns the summary data for this course
     *
     * @return RecordResultSet
     */
    public function retrieve_course_summary_data()
    {
        return WeblcmsTrackingDataManager :: retrieve_tools_access_summary_data($this->get_course_id());
    }

    /**
     * Returns the condition for the tools publication count
     *
     * @param string $tool_name
     *
     * @return AndCondition
     */
    public function get_tool_publications_condition($tool_name)
    {
        $conditions = array();

        $conditions[] = parent :: get_tool_publications_condition($tool_name);

        $conditions[] = new EqualityCondition(
            new PropertyConditionVariable(
                ContentObjectPublication :: class_name(), ContentObjectPublication :: PROPERTY_COURSE_ID
            ),
            new StaticConditionVariable($this->get_course_id())
        );


        return new AndCondition($conditions);
    }
}
