<?php
namespace application\weblcms\integration\reporting;

use common\libraries\DataClassRetrievesParameters;
use application\weblcms\ContentObjectPublication;
use application\weblcms\Tool;
use application\weblcms\WeblcmsManager;
use reporting\ReportingData;
use reporting\ReportingFormatter;
use repository\ComplexContentObjectItem;
use common\libraries\EqualityCondition;
use common\libraries\Translation;

class WikiMostEditedPageBlock extends ToolBlock
{

    public function count_data()
    {
        $reporting_data = new ReportingData();
        $reporting_data->set_rows(array(Translation :: get('MostEditedPage'), Translation :: get('NumberOfEdits')));

        $publication = \application\weblcms\DataManager :: retrieve(
            ContentObjectPublication :: class_name(),
            $this->get_publication_id());

        $wiki = $publication->get_content_object();
        $complex_content_object_items = \repository\DataManager :: retrieve_complex_content_object_items(
            ComplexContentObjectItem :: class_name(),
            new DataClassRetrievesParameters(
                new EqualityCondition(
                    ComplexContentObjectItem :: PROPERTY_PARENT,
                    $wiki->get_id(),
                    ComplexContentObjectItem :: get_table_name())))->as_array();

        if (empty($complex_content_object_items))
        {
            return $reporting_data;
        }
        else
        {
            $most_edits = 0;
            $most_edited_page = null;

            foreach ($complex_content_object_items as $complex_content_object_item)
            {
                $page_edits = \repository\DataManager :: retrieve_content_object(
                    $complex_content_object_item->get_ref())->get_version_count();

                if ($page_edits >= $most_edits)
                {
                    $most_edits = $page_edits;
                    $most_edited_page = $complex_content_object_item;
                }
            }

            $url = 'index.php?go=' . WeblcmsManager :: ACTION_VIEW_COURSE . '&course=' . $this->get_course_id() .
                 '&tool=' . $this->get_tool() . '&application=weblcms&' . Tool :: PARAM_PUBLICATION_ID . '=' .
                 $this->get_publication_id() . '&tool_action=' . Tool :: ACTION_VIEW . '&display_action=view_item&selected_cloi=' .
                 $most_edited_page->get_id();

            $reporting_data->add_category(0);
            $reporting_data->add_data_category_row(
                0,
                Translation :: get('MostEditedPage'),
                '<a href="' . $url . '">' . $most_edited_page->get_ref_object()->get_title() . '</a>');
            $reporting_data->add_data_category_row(0, Translation :: get('NumberOfEdits'), $most_edits);
            $reporting_data->hide_categories();
        }

        return $reporting_data;
    }

    public function retrieve_data()
    {
        return $this->count_data();
    }

    public function get_views()
    {
        return array(\reporting\viewer\HtmlBlockRendition :: VIEW_TABLE);
    }
}
