<?php
namespace application\weblcms\integration\reporting;

use common\libraries\BreadcrumbTrail;
use common\libraries\DataClassRetrievesParameters;
use application\weblcms\ContentObjectPublication;
use application\weblcms\WeblcmsManager;
use common\libraries\Request;
use common\libraries\Translation;
use reporting\ReportingManager;
use reporting\ReportingTemplate;
use common\libraries\Breadcrumb;
use common\libraries\EqualityCondition;
use repository\ComplexContentObjectItem;

/**
 *
 * @package application.weblcms.php.reporting.templates Reporting template with information about the assessment, the
 *          attempts per user and questions score stats
 * @author Joris Willems <joris.willems@gmail.com>
 * @author Alexander Van Paemel
 */
class AssessmentAttemptsTemplate extends ReportingTemplate
{

    public function __construct($parent)
    {
        parent :: __construct($parent);

        $this->publication_id = Request :: get(WeblcmsManager :: PARAM_PUBLICATION);
        if ($this->publication_id)
        {
            $this->set_parameter(WeblcmsManager :: PARAM_PUBLICATION, $this->publication_id);
        }

        $sel = (Request :: post('sel')) ? Request :: post('sel') : Request :: get('sel');
        if ($sel)
        {
            $this->set_parameter('sel', $sel);
        }

        // Retrieve the questions of the assessment
        $publication = \application\weblcms\DataManager :: retrieve(
            ContentObjectPublication :: class_name(),
            $this->publication_id);
        $condition = new EqualityCondition(
            ComplexContentObjectItem :: PROPERTY_PARENT,
            $publication->get_content_object_id());
        $questions_resultset = \repository\DataManager :: retrieve_complex_content_object_items(
            ComplexContentObjectItem :: class_name(),
            new DataClassRetrievesParameters($condition));

        while ($question = $questions_resultset->next_result())
        {
            $this->th_titles[] = $question->get_ref_object()->get_title();
        }

        $this->add_reporting_block($this->get_assessment_information());
        $this->add_reporting_block($this->get_assessment_questions());
        $this->add_reporting_block($this->get_assessment_users());
        $this->add_reporting_block($this->get_assessment_attempts());
        $this->add_reporting_block($this->get_assessment_questions_users());

        $this->add_breadcrumbs();
    }

    public function display_context()
    {
    }

    public function get_application()
    {
        return WeblcmsManager :: APPLICATION_NAME;
    }

    public function get_assessment_information()
    {
        $assessment_block = new AssessmentInformationBlock($this);
        return $assessment_block;
    }

    public function get_assessment_users()
    {
        $assessment_block = new AssessmentUsersBlock($this);
        $assessment_block->set_sorting(3, SORT_DESC);
        return $assessment_block;
    }

    /**
     * Returns the assessment attempts block
     *
     * @return AssessmentAttemptsBlock
     */
    public function get_assessment_attempts()
    {
        $assessment_block = new AssessmentAttemptsBlock($this);
        $assessment_block->set_sorting(3, SORT_DESC);
        return $assessment_block;
    }

    public function get_assessment_questions()
    {
        $assessment_block = new AssessmentQuestionsBlock($this);
        $assessment_block->set_sorting(3, SORT_DESC);
        return $assessment_block;
    }

    public function get_assessment_questions_users()
    {
        $assessment_block = new AssessmentQuestionsUsersBlock($this, true);
        $assessment_block->set_sorting(0);
        return $assessment_block;
    }

    /**
     * Adds the breadcrumbs to the breadcrumbtrail
     */
    protected function add_breadcrumbs()
    {
        $assessment = \application\weblcms\DataManager :: retrieve(
            ContentObjectPublication :: class_name(),
            $this->publication_id)->get_content_object();

        $trail = BreadcrumbTrail :: get_instance();

        $trail->add(
            new Breadcrumb(
                $this->get_url(
                    array(\reporting\Viewer\Manager :: PARAM_BLOCK_ID => 2),
                    array(ReportingManager :: PARAM_TEMPLATE_ID)
                ),
                Translation :: get('Assessments')
            )
        );

        $trail->add(new Breadcrumb($this->get_url(), $assessment->get_title()));
    }
}
