<?php
namespace application\weblcms\integration\reporting;

use application\weblcms\ContentObjectPublication;
use application\weblcms\WeblcmsManager;
use application\weblcms\tool\reporting\ReportingTool;
use common\libraries\Breadcrumb;
use common\libraries\BreadcrumbTrail;
use common\libraries\Request;
use common\libraries\Translation;
use reporting\ReportingTemplate;
use user\UserDataManager;
use reporting\ReportingManager;
use reporting\Reporting;
use common\libraries\Utilities;

/**
 *
 * @package application.weblcms.php.reporting.templates Reporting template with the assessment question attempts of one
 *          user
 * @author Joris Willems <joris.willems@gmail.com>
 * @author Alexander Van Paemel
 */
class AssessmentQuestionAttemptsUserTemplate extends ReportingTemplate
{

    public function __construct($parent)
    {
        parent :: __construct($parent);

        $this->question_id = Request :: get(ReportingTool :: PARAM_QUESTION);
        $this->publication_id = Request :: get(WeblcmsManager :: PARAM_PUBLICATION);
        $this->user_id = Request :: get(WeblcmsManager :: PARAM_USERS);

        $this->add_reporting_block($this->get_question_user_information());
        $this->add_reporting_block($this->get_question_attempts_user());

        $this->add_breadcrumbs();
    }

    public function display_context()
    {
    }

    public function get_application()
    {
        return WeblcmsManager :: APPLICATION_NAME;
    }

    /**
     * Returns the block for the question user information
     */
    public function get_question_user_information()
    {
        $question_attempts_user_block = new AssessmentQuestionUserInformationBlock($this);
        $question_attempts_user_block->set_sorting(0, SORT_DESC);
        return $question_attempts_user_block;
    }

    public function get_question_attempts_user()
    {
        if ($this->question_id)
        {
            $this->set_parameter(ReportingTool :: PARAM_QUESTION, $this->question_id);
        }

        if ($this->publication_id)
        {
            $this->set_parameter(WeblcmsManager :: PARAM_PUBLICATION, $this->publication_id);
        }

        if ($this->user_id)
        {
            $this->set_parameter(WeblcmsManager :: PARAM_USERS, $this->user_id);
        }

        $question_attempts_user_block = new AssessmentQuestionAttemptsUserBlock($this);
        $question_attempts_user_block->set_sorting(0, SORT_DESC);
        return $question_attempts_user_block;
    }

    /**
     * Adds the breadcrumbs to the breadcrumbtrail
     */
    protected function add_breadcrumbs()
    {
        $assessment = \application\weblcms\DataManager :: retrieve(
            ContentObjectPublication :: class_name(),
            $this->publication_id)->get_content_object();

        $question = \repository\DataManager :: retrieve_complex_content_object_item($this->question_id);

        $trail = BreadcrumbTrail :: get_instance();

        $trail->add(
            new Breadcrumb(
                $this->get_url(
                    array(\reporting\Viewer\Manager :: PARAM_BLOCK_ID => 2),
                    array(
                        ReportingManager :: PARAM_TEMPLATE_ID
                    )
                ),
                Translation :: get('Assessments')
            )
        );

        $filters = array(WeblcmsManager :: PARAM_USERS, ReportingTool :: PARAM_QUESTION);

        $params = array();
        $params[ReportingManager :: PARAM_TEMPLATE_ID] = AssessmentAttemptsTemplate :: class_name();
        $params[WeblcmsManager :: PARAM_PUBLICATION] = $this->publication_id;

        $trail->add(
            new Breadcrumb($this->get_url($params, $filters), $assessment->get_title())
        );

        $params[\reporting\Viewer\Manager :: PARAM_BLOCK_ID] = 2;

        $trail->add(
            new Breadcrumb($this->get_url($params, $filters), Translation :: get('Questions'))
        );

        $filters = array(WeblcmsManager :: PARAM_USERS);

        $params = array();
        $params[ReportingManager :: PARAM_TEMPLATE_ID] = AssessmentQuestionUsersTemplate :: class_name();
        $params[WeblcmsManager :: PARAM_PUBLICATION] = $this->publication_id;

        $trail->add(
            new Breadcrumb($this->get_url($params, $filters), $question->get_ref_object()->get_title())
        );

        $params[\reporting\Viewer\Manager :: PARAM_BLOCK_ID] = 0;

        $trail->add(
            new Breadcrumb($this->get_url($params, $filters), Translation :: get('Users'))
        );

        $trail->add(
            new Breadcrumb($this->get_url(), \user\DataManager :: get_fullname_from_user($this->user_id))
        );
    }
}
