<?php
namespace application\weblcms\integration\reporting;

use application\weblcms\WeblcmsManager;
use application\weblcms\ContentObjectPublication;
use application\weblcms\tool\reporting\ReportingTool;
use common\libraries\Breadcrumb;
use common\libraries\BreadcrumbTrail;
use common\libraries\Request;
use common\libraries\Translation;
use reporting\ReportingTemplate;
use reporting\ReportingManager;
use reporting\Reporting;
use common\libraries\Utilities;

/**
 *
 * @package application.weblcms.php.reporting.templates Reporting template with an overview of the scores of an
 *          assessment question per user
 * @author Joris Willems <joris.willems@gmail.com>
 * @author Alexander Van Paemel
 */
class AssessmentQuestionUsersTemplate extends ReportingTemplate
{

    public function __construct($parent)
    {
        parent :: __construct($parent);

        $this->initialize_parameters();

        $this->add_reporting_block($this->get_question_information_block());
        $this->add_reporting_block($this->get_question_options_block());
        $this->add_reporting_block($this->get_question_users());
        $this->add_reporting_block($this->get_question_attempts());

        $this->add_breadcrumbs();
    }

    private function initialize_parameters()
    {
        $this->question_id = Request :: get(ReportingTool :: PARAM_QUESTION);
        if ($this->question_id)
        {
            $this->set_parameter(ReportingTool :: PARAM_QUESTION, $this->question_id);
        }

        $this->publication_id = Request :: get(WeblcmsManager :: PARAM_PUBLICATION);
        if ($this->publication_id)
        {
            $this->set_parameter(WeblcmsManager :: PARAM_PUBLICATION, $this->publication_id);
        }

        $this->user_id = Request :: get(WeblcmsManager :: PARAM_USERS);
        if ($this->user_id)
        {
            $this->set_parameter(WeblcmsManager :: PARAM_USERS, $this->user_id);
        }
    }

    public function display_context()
    {
    }

    public function get_application()
    {
        return WeblcmsManager :: APPLICATION_NAME;
    }

    /**
     * Returns the question information block
     */
    public function get_question_information_block()
    {
        $question_block = new AssessmentQuestionInformationBlock($this);
        $question_block->set_sorting(0);
        return $question_block;
    }

    public function get_question_users()
    {
        $question_block = new AssessmentQuestionUsersBlock($this);
        $question_block->set_sorting(0);
        return $question_block;
    }

    public function get_question_attempts()
    {
        $question_block = new AssessmentQuestionAttemptsBlock($this);
        $question_block->set_sorting(0, SORT_DESC);
        return $question_block;
    }

    /**
     * Returns the block of the question options
     *
     * @return AssessmentQuestionOptionsBlock
     */
    public function get_question_options_block()
    {
        $question_options_block = AssessmentQuestionOptionsBlock :: factory($this);
        $question_options_block->set_sorting(0);
        return $question_options_block;
    }

    /**
     * Adds the breadcrumbs to the breadcrumbtrail
     */
    protected function add_breadcrumbs()
    {
        $assessment = \application\weblcms\DataManager :: retrieve(
            ContentObjectPublication :: class_name(),
            $this->publication_id)->get_content_object();

        $question = \repository\DataManager :: retrieve_complex_content_object_item($this->question_id);

        $trail = BreadcrumbTrail :: get_instance();

        $trail->add(
            new Breadcrumb(
                $this->get_url(
                    array(\reporting\Viewer\Manager :: PARAM_BLOCK_ID => 2),
                    array(ReportingManager :: PARAM_TEMPLATE_ID)
                ),
                Translation :: get('Assessments')
            )
        );

        $params = array();
        $params[ReportingManager :: PARAM_TEMPLATE_ID] = AssessmentAttemptsTemplate :: class_name();
        $params[WeblcmsManager :: PARAM_PUBLICATION] = $this->publication_id;

        $trail->add(
            new Breadcrumb($this->get_url($params), $assessment->get_title())
        );

        $params[\reporting\Viewer\Manager :: PARAM_BLOCK_ID] = 2;

        $trail->add(
            new Breadcrumb($this->get_url($params), Translation :: get('Questions'))
        );

        $trail->add(
            new Breadcrumb($this->get_url(), $question->get_ref_object()->get_title())
        );
    }
}
