<?php
namespace application\weblcms\integration\reporting;

use application\weblcms\Tool;
use application\weblcms\ContentObjectPublication;
use application\weblcms\WeblcmsManager;
use application\weblcms\tool\assignment\AssignmentTool;
use application\weblcms\tool\reporting\ReportingTool;
use common\libraries\Breadcrumb;
use common\libraries\Request;
use common\libraries\Translation;
use common\libraries\Utilities;
use reporting\Reporting;
use reporting\ReportingManager;
use reporting\ReportingTemplate;

/**
 * Description of assignment_submissions_reporting_template
 *
 * @author Anthony Hurst (Hogeschool Gent)
 */
class AssignmentSubmissionsTemplate extends ReportingTemplate
{

    public function __construct($parent)
    {
        parent :: __construct($parent);

        $this->publication_id = Request :: get(WeblcmsManager :: PARAM_PUBLICATION);

        $assignment = \application\weblcms\DataManager :: retrieve(
            ContentObjectPublication :: class_name(),
            $this->publication_id)->get_content_object();

        $this->init_parameters();
        $this->add_reporting_block($this->get_assignment_information());
        $this->add_reporting_block($this->get_assignment_submissions());

        $custom_breadcrumbs = array();

        $parameters = array();
        $parameters[WeblcmsManager :: PARAM_TOOL] = WeblcmsManager :: ACTION_REPORTING;
        // $parameters[ReportingManager :: PARAM_TEMPLATE_ID] = Reporting :: get_name_registration(Utilities ::
        // camelcase_to_underscores('CourseAssignmentSubmittersReportingTemplate'), WeblcmsManager ::
        // APPLICATION_NAME)->get_id();
        $parameters[Tool :: PARAM_PUBLICATION_ID] = $this->publication_id;
        $parameters[WeblcmsManager :: PARAM_TOOL_ACTION] = ReportingTool :: ACTION_VIEW;
        $custom_breadcrumbs[] = new Breadcrumb($this->get_url($parameters), $assignment->get_title());
        $custom_breadcrumbs[] = new Breadcrumb($this->get_url(), Translation :: get('SubmissionsOverview'));
        $this->set_custom_breadcrumb_trail($custom_breadcrumbs);
    }

    public function get_additional_parameters()
    {
        return array(
            Tool :: PARAM_PUBLICATION_ID,
            AssignmentTool :: PARAM_TARGET_ID,
            AssignmentTool :: PARAM_SUBMITTER_TYPE);
    }

    protected function get_assignment_information()
    {
        return new AssignmentInformationBlock($this);
    }

    protected function get_assignment_submissions()
    {
        return AssignmentSubmissionsBlock :: get_instance($this);
    }

    public function display_context()
    {
    }

    public function get_application()
    {
        return WeblcmsManager :: APPLICATION_NAME;
    }

    private function init_parameters()
    {
        if ($this->publication_id)
        {
            $this->set_parameter(WeblcmsManager :: PARAM_PUBLICATION, $this->publication_id);
        }
    }
}
