<?php
namespace application\weblcms\integration\reporting;

use application\weblcms\ContentObjectPublication;
use application\weblcms\WeblcmsManager;
use common\libraries\Request;
use reporting\ReportingTemplate;
use common\libraries\Breadcrumb;

/**
 *
 * @package application.weblcms.php.reporting.templates Reporting template with information about the assignment and the
 *          users, course groups and platform groups the assignment is published for
 * @author Joris Willems <joris.willems@gmail.com>
 * @author Alexander Van Paemel
 * @author Anthony Hurst (Hogeschool Gent)
 */
class CourseAssignmentSubmittersTemplate extends ReportingTemplate
{

    public function __construct($parent)
    {
        parent :: __construct($parent);

        $this->publication_id = Request :: get(WeblcmsManager :: PARAM_PUBLICATION);

        $assignment = \application\weblcms\DataManager :: retrieve(
            ContentObjectPublication :: class_name(),
            $this->publication_id)->get_content_object();

        $this->init_parameters();
        $this->add_reporting_block($this->get_assignment_information());

        if (! $assignment->get_allow_group_submissions())
        {
            $this->add_reporting_block($this->get_assignment_users());
        }
        else
        {
            $this->add_reporting_block($this->get_assignment_course_groups());
            $this->add_reporting_block($this->get_assignment_groups());
        }

        $custom_breadcrumbs = array();
        $custom_breadcrumbs[] = new Breadcrumb($this->get_url(), $assignment->get_title());
        $this->set_custom_breadcrumb_trail($custom_breadcrumbs);
    }

    public function display_context()
    {
    }

    private function init_parameters()
    {
        if ($this->publication_id)
        {
            $this->set_parameter(WeblcmsManager :: PARAM_PUBLICATION, $this->publication_id);
        }
    }

    public function get_application()
    {
        return WeblcmsManager :: APPLICATION_NAME;
    }

    public function get_assignment_information()
    {
        $assignment_block = new AssignmentInformationBlock($this);
        return $assignment_block;
    }

    public function get_assignment_users()
    {
        $assignment_submissions_block = new AssignmentUsersBlock($this);
        $assignment_submissions_block->set_sorting(2, SORT_DESC);
        return $assignment_submissions_block;
    }

    public function get_assignment_course_groups()
    {
        $assignment_submissions_block = new AssignmentCourseGroupsBlock($this);
        $assignment_submissions_block->set_sorting(2, SORT_DESC);
        return $assignment_submissions_block;
    }

    public function get_assignment_groups()
    {
        $assignment_submissions_block = new AssignmentPlatformGroupsBlock($this);
        $assignment_submissions_block->set_sorting(2, SORT_DESC);
        return $assignment_submissions_block;
    }
}
