<?php
namespace application\weblcms\integration\tracking;

use common\libraries\AndCondition;
use common\libraries\EqualityCondition;
use common\libraries\Translation;

class AssessmentAttempt extends \tracking\SimpleTracker
{
    const CLASS_NAME = __CLASS__;

    const PROPERTY_USER_ID = 'user_id';
    const PROPERTY_COURSE_ID = 'course_id';
    const PROPERTY_ASSESSMENT_ID = 'assessment_id';
    const PROPERTY_TOTAL_SCORE = 'total_score';
    const PROPERTY_STATUS = 'status';
    const PROPERTY_START_TIME = 'start_time';
    const PROPERTY_END_TIME = 'end_time';
    const PROPERTY_TOTAL_TIME = 'total_time';

    const STATUS_NOT_COMPLETED = 1;
    const STATUS_COMPLETED = 2;

    public function validate_parameters(array $parameters = array())
    {
        $status = $parameters[self :: PROPERTY_STATUS];

        $this->set_user_id($parameters[self :: PROPERTY_USER_ID]);
        $this->set_course_id($parameters[self :: PROPERTY_COURSE_ID]);
        $this->set_assessment_id($parameters[self :: PROPERTY_ASSESSMENT_ID]);
        $this->set_start_time(time());

        if ($status)
        {
            $this->set_status($status);
        }

        $this->set_total_score($parameters[self :: PROPERTY_TOTAL_SCORE]);
    }

    public static function get_default_property_names()
    {
        return parent :: get_default_property_names(
            array(
                self :: PROPERTY_USER_ID, self :: PROPERTY_COURSE_ID, self :: PROPERTY_ASSESSMENT_ID,
                self :: PROPERTY_TOTAL_SCORE, self :: PROPERTY_STATUS,
                self :: PROPERTY_START_TIME, self :: PROPERTY_END_TIME, self :: PROPERTY_TOTAL_TIME
            )
        );
    }

    public function get_user_id()
    {
        return $this->get_default_property(self :: PROPERTY_USER_ID);
    }

    public function set_user_id($user_id)
    {
        $this->set_default_property(self :: PROPERTY_USER_ID, $user_id);
    }

    public function get_course_id()
    {
        return $this->get_default_property(self :: PROPERTY_COURSE_ID);
    }

    public function set_course_id($course_id)
    {
        $this->set_default_property(self :: PROPERTY_COURSE_ID, $course_id);
    }

    public function get_assessment_id()
    {
        return $this->get_default_property(self :: PROPERTY_ASSESSMENT_ID);
    }

    public function set_assessment_id($assessment_id)
    {
        $this->set_default_property(self :: PROPERTY_ASSESSMENT_ID, $assessment_id);
    }

    public function get_total_score()
    {
        return $this->get_default_property(self :: PROPERTY_TOTAL_SCORE);
    }

    public function set_total_score($total_score)
    {
        $this->set_default_property(self :: PROPERTY_TOTAL_SCORE, $total_score);
    }

    public function get_status()
    {
        return $this->get_default_property(self :: PROPERTY_STATUS);
    }

    public function set_status($status)
    {
        $this->set_default_property(self :: PROPERTY_STATUS, $status);
    }

    public function get_start_time()
    {
        return $this->get_default_property(self :: PROPERTY_START_TIME);
    }

    public function set_start_time($start_time)
    {
        $this->set_default_property(self :: PROPERTY_START_TIME, $start_time);
    }

    public function get_end_time()
    {
        return $this->get_default_property(self :: PROPERTY_END_TIME);
    }

    public function set_end_time($end_time)
    {
        $this->set_default_property(self :: PROPERTY_END_TIME, $end_time);
    }

    public function get_total_time()
    {
        return $this->get_default_property(self :: PROPERTY_TOTAL_TIME);
    }

    public function set_total_time($total_time)
    {
        $this->set_default_property(self :: PROPERTY_TOTAL_TIME, $total_time);
    }

    public function get_times_taken($publication, $user_id = null)
    {
        $condition = new EqualityCondition(self :: PROPERTY_ASSESSMENT_ID, $publication->get_id());

        if ($user_id)
        {
            $conditions = array();
            $conditions[] = $condition;
            $conditions[] = new EqualityCondition(self :: PROPERTY_USER_ID, $user_id);
            $condition = new AndCondition($conditions);
        }

        $trackers = $this->retrieve_tracker_items($condition);
        return count($trackers);
    }

    public function get_average_score($publication, $user_id = null)
    {
        $condition = new EqualityCondition(self :: PROPERTY_ASSESSMENT_ID, $publication->get_id());

        if ($user_id)
        {
            $conditions = array();
            $conditions[] = $condition;
            $conditions[] = new EqualityCondition(self :: PROPERTY_USER_ID, $user_id);
            $condition = new AndCondition($conditions);
        }

        $trackers = $this->retrieve_tracker_items($condition);
        $num = count($trackers);

        $total_score = 0;

        foreach ($trackers as $tracker)
        {
            $total_score += $tracker->get_total_score();
        }

        $total_score = round($total_score / $num, 2);
        return $total_score;
    }

    public static function get_table_name()
    {
        return 'weblcms_' . parent :: get_table_name();
    }

    /**
     * Returns the status as a string
     *
     * @return string
     */
    public function get_status_as_string()
    {
        return $this->get_status() == self :: STATUS_COMPLETED ? Translation :: get('Completed') :
            Translation :: get('NotCompleted');
    }
}
