<?php
namespace application\weblcms;

use repository\content_object\document\Document;
use repository\content_object\webpage\Webpage;
use repository\content_object\file\File;
use application\weblcms\course\CourseGroupRelation;
use application\weblcms\course\DataManager as CourseDataManager;
use common\libraries\AndCondition;
use common\libraries\InCondition;
use common\libraries\InequalityCondition;
use common\libraries\ObjectTableOrder;
use group\DataManager as GroupDataManager;
use repository\content_object\announcement\Announcement;
use repository\content_object\assignment\Assignment;
use rights\TemplateEntity;

/**
 * Description of weblcms_new_block
 *
 * @author Anthony Hurst (Hogeschool Gent)
 */
class WeblcmsNewBlock extends WeblcmsBlock
{
    const TOOL_ANNOUNCEMENT = 'announcement';
    const TOOL_ASSIGNMENT = 'assignment';
    const TOOL_DOCUMENT = 'document';
    
    private $courses;

    public function get_content($tool)
    {
        // Direct user courses.
        $condition = CourseDataManager :: get_user_courses_condition($this->get_user());
        $user_courses = CourseDataManager :: retrieve_user_courses($condition)->as_array();
        
        // Courses active through group membership.
        $groups = GroupDataManager :: retrieve_all_subscribed_groups_array($this->get_user_id());
        
        $group_ids = array();
        foreach ($groups as $group)
        {
            $group_ids[$group->get_id()] = $group->get_id();
        }
        
        $condition = new InCondition(CourseGroupRelation :: PROPERTY_GROUP_ID, $group_ids, CourseGroupRelation :: get_table_name());
        
        array_splice($user_courses, 0, 0, CourseDataManager :: retrieve_user_courses($condition)->as_array());
        
        $this->courses = array();
        foreach ($user_courses as $user_course)
        {
            $this->courses[$user_course->get_id()] = $user_course;
        }
        
        $course_settings_controller = CourseSettingsController :: get_instance();
        $unique_publications = array();
        foreach ($this->courses as $course)
        {
            if ($course_settings_controller->get_course_setting($course->get_id(), CourseSettingsConnector :: VISIBILITY) == 1)
            {
                $condition = $this->get_publication_conditions($course, $tool);
                $course_module_id = DataManager :: retrieve_course_tool_by_name($tool)->get_id();
                $location = WeblcmsRights :: get_instance()->get_weblcms_location_by_identifier_from_courses_subtree(WeblcmsRights :: TYPE_COURSE_MODULE, $course_module_id, $course->get_id());
                
                $entities = array();
                $entities[CourseGroupEntity :: ENTITY_TYPE] = CourseGroupEntity :: get_instance($course->get_id());
                $entities[CourseUserEntity :: ENTITY_TYPE] = CourseUserEntity :: get_instance();
                $entities[CoursePlatformGroupEntity :: ENTITY_TYPE] = CoursePlatformGroupEntity :: get_instance();
                $entities[TemplateEntity :: ENTITY_TYPE] = TemplateEntity :: get_instance();
                
                $publications = DataManager :: retrieve_content_object_publications_with_view_right_granted_in_category_location(
                        $location, $entities, $condition,
                        new ObjectTableOrder(ContentObjectPublication :: PROPERTY_DISPLAY_ORDER_INDEX, SORT_DESC));
                
                if ($publications == 0)
                {
                    continue;
                }
                foreach ($publications->as_array() as $publication)
                {
                    $unique_publications[$course->get_id() . '.' . $publication->get_id()] = $publication;
                }
            }
        }
        return $unique_publications;
    }

    private function get_publication_conditions($course, $tool)
    {
        $type = null;
        switch ($tool)
        {
            case self :: TOOL_ANNOUNCEMENT :
                $type = Announcement :: class_name();
                break;
            case self :: TOOL_ASSIGNMENT :
                $type = Assignment :: class_name();
                break;
            case self :: TOOL_DOCUMENT :
                $type = array(Document :: class_name(), File :: class_name(), Webpage :: class_name());
                break;
        }
        $last_visit_date = DataManager :: get_last_visit_date($course->get_id(), $this->get_user_id(), $tool, 0);
        
        $conditions = array();
        $conditions[] = DataManager :: get_publications_condition($course, $this->get_user(), $tool, $type);
        $conditions[] = new InequalityCondition(ContentObjectPublication :: PROPERTY_PUBLICATION_DATE, InequalityCondition :: GREATER_THAN_OR_EQUAL, $last_visit_date);
        return new AndCondition($conditions);
    }

    protected function get_course_by_id($course_id)
    {
        return $this->courses[$course_id];
    }
}
