<?php

namespace application\weblcms;

use common\libraries\EqualityCondition;
use common\extensions\rights_editor_manager\RightsEditorManager;
use common\libraries\AndCondition;
use group\DataManager as GroupDataManager;
use common\libraries\InequalityCondition;
use group\Group;
use group\GroupRelUser;
use application\weblcms\course\CourseUserRelation;

class CoursesRightsEditorManager extends RightsEditorManager
{
    const PARAM_COURSE_GROUP = 'course_group';
    const ACTION_SET_COURSE_GROUP_RIGHTS = 'set_course_group_rights';

    const TYPE_COURSE_GROUP = 'course_group';

    public function __construct($parent, $locations)
    {
        parent :: __construct($parent, $locations);

        $this->set_types(
            array(self :: TYPE_USER, self :: TYPE_GROUP, self :: TYPE_COURSE_GROUP, self :: TYPE_TEMPLATE)
        );

        $relation_condition =
            new EqualityCondition(CourseGroupRelation :: PROPERTY_COURSE_ID, $parent->get_course_id());

        $platform_group_relations = $course_group_relations = \application\weblcms\course\DataManager :: retrieves(
            CourseGroupRelation :: class_name(),
            $relation_condition
        );

        $users = array();

        while ($platform_group_relation = $platform_group_relations->next_result())
        {
            $platform_group_id = $platform_group_relation->get_group_object()->get_id();
            $platform_groups[] = $platform_group_id;

            // get the users in that platform group
            $eq_condition = new EqualityCondition(GroupRelUser :: PROPERTY_GROUP_ID, $platform_group_id);
            $group_rel_users_dataset = GroupDataManager :: retrieves(GroupRelUser :: class_name(), $eq_condition);
            while ($group_rel_user = $group_rel_users_dataset->next_result())
            {
                $users[] = $group_rel_user->get_user_id();
            }

            $group = GroupDataManager :: retrieve_by_id(Group :: class_name, $platform_group_id);

            $children_conditions = array();

            $children_conditions[] = new InequalityCondition(
                Group :: PROPERTY_LEFT_VALUE,
                InequalityCondition :: GREATER_THAN, $group->get_left_value()
            );

            $children_conditions[] = new InequalityCondition(
                Group :: PROPERTY_RIGHT_VALUE,
                InequalityCondition :: LESS_THAN, $group->get_right_value()
            );

            $children_condition = new AndCondition($children_conditions);

            // retrieve the subgroups subscribed implicitly
            $child_groups = GroupDataManager :: retrieves(Group :: class_name(), $children_condition);
            while ($cgroup = $child_groups->next_result())
            {
                $platform_groups[] = $cgroup->get_id();
                $eq_condition = new EqualityCondition(GroupRelUser :: PROPERTY_GROUP_ID, $platform_group_id);
                $group_rel_users_dataset = GroupDataManager :: retrieves(GroupRelUser :: class_name(), $eq_condition);

                while ($group_rel_user = $group_rel_users_dataset->next_result())
                {
                    $users[] = $group_rel_user->get_user_id();
                }
            }
        }

        $users[] = $parent->get_user();

        $conditions = array();
        $conditions[] =
            new EqualityCondition(CourseUserRelation :: PROPERTY_COURSE_ID, $this->get_parent()->get_course_id());
        $conditions[] =
            new EqualityCondition(CourseUserRelation :: PROPERTY_STATUS, CourseUserRelation :: STATUS_STUDENT);

        $relations = \application\weblcms\course\DataManager :: retrieves(
            CourseUserRelation :: class_name(),
            new AndCondition($conditions)
        );

        while ($relation = $relations->next_result())
        {
            $users[] = $relation->get_user();
        }

        $this->limit_users($users);

        $platform_groups[] = 1;
        $excluded_groups[] = 1; // exclude chamilo parent group
        $this->limit_groups($platform_groups);
        $this->exclude_groups($excluded_groups);
    }
}
