<?php
namespace application\weblcms;

use application\weblcms\course\DataManager as CourseDataManager;
use application\weblcms\course\Course;
use common\libraries\DatetimeUtilities;
use user\UserDataManager;
use common\libraries\Redirect;
use common\libraries\Application;
use common\libraries\InCondition;
use common\libraries\Utilities;
use common\libraries\ObjectTableOrder;
use common\libraries\Path;
use common\libraries\Translation;
use common\libraries\PublicationRSS;
use group\DataManager as GroupDataManager;
use application\weblcms\CourseSettingsController;
use application\weblcms\course\CourseGroupRelation;
use rights\TemplateEntity;

class WeblcmsPublicationRSS extends PublicationRSS
{

    public function __construct()
    {
        Utilities :: set_application(WeblcmsManager :: APPLICATION_NAME);
        parent :: __construct(
            Translation :: get('WeblcmsPublicationRSSTitle'),
            htmlspecialchars(Path :: get(WEB_PATH)),
            Translation :: get('WeblcmsPublicationRSSDescription'),
            htmlspecialchars(Path :: get(WEB_PATH)));
    }

    public function retrieve_items($user)
    {
        return $this->get_publications($user);

        /*
         * $pubs = DataManager :: retrieve_content_object_publications( $this->get_access_condition($user), new
         * ObjectTableOrder(ContentObjectPublication :: PROPERTY_PUBLICATION_DATE, SORT_DESC), 0, 20 ); // ,
         * $publications = array(); while ($pub = $pubs->next_result()) { if ($this->is_visible_for_user($user, $pub)) {
         * $publications[] = $pub; } } return $publications;
         */
    }

    public function add_item($publication, $channel)
    {
        $course = CourseDataManager :: retrieve(
            Course :: class_name(),
            $publication[ContentObjectPublication :: PROPERTY_COURSE_ID]);

        $co = $publication[ContentObjectPublication :: PROPERTY_CONTENT_OBJECT_ID];
        if (! is_object($co))
        {
            $co = \repository\DataManager :: retrieve_content_object($co);
        }

        $title = $co->get_title();
        $description = '<b>' . Translation :: get('Course') . ': </b>' . $course->get_title() . '<br />';
        $description .= '<b>' . Translation :: get('Tool') . ': </b>' . Translation :: get(
            Utilities :: underscores_to_camelcase($publication[ContentObjectPublication :: PROPERTY_TOOL])) . '<br />';

        $format = Translation :: get('DateFormatShort', null, Utilities :: COMMON_LIBRARIES) . ', ' .
             Translation :: get('TimeNoSecFormat', null, Utilities :: COMMON_LIBRARIES);

        $description .= '<b>' . Translation :: get('Published') . ': </b>' . DatetimeUtilities :: format_locale_date(
            $format,
            $publication[ContentObjectPublication :: PROPERTY_PUBLICATION_DATE]) . '<br />';

        $description .= '<b>' . Translation :: get('Publisher') . ': </b>' . UserDataManager :: get_instance()->retrieve_user(
            $publication[ContentObjectPublication :: PROPERTY_PUBLISHER_ID])->get_fullname() . '<br />';

        $description .= '<br />' . $co->get_description();

        $channel->add_item(
            htmlspecialchars($title),
            htmlspecialchars($this->get_url($publication)),
            htmlspecialchars($description));
    }

    public function get_url($pub)
    {
        $params = array();
        $params[Application :: PARAM_ACTION] = WeblcmsManager :: ACTION_VIEW_COURSE;
        $params[WeblcmsManager :: PARAM_COURSE_USER] = $pub[ContentObjectPublication :: PROPERTY_COURSE_ID];
        $params[ContentObjectPublication :: PROPERTY_TOOL] = $pub[ContentObjectPublication :: PROPERTY_TOOL];
        return str_replace('/application/weblcms/php/lib/rss/', '/', Path :: get(WEB_PATH)) .
             Redirect :: get_link(WeblcmsManager :: APPLICATION_NAME, $params);
    }

    public function is_visible_for_user($user, $pub)
    {
        return $user->is_platform_admin() || ($user->get_id() == $pub->get_publisher_id()) ||
             $pub->is_visible_for_target_users();
    }

    private function get_publications($user)
    {
        // Direct user courses.
        $condition = CourseDataManager :: get_user_courses_condition($user);
        $courses = CourseDataManager :: retrieve_user_courses($condition)->as_array();

        // Courses active through group membership.
        $groups = GroupDataManager :: retrieve_all_subscribed_groups_array($user->get_id());

        $group_ids = array();
        foreach ($groups as $group)
        {
            $group_ids[$group->get_id()] = $group->get_id();
        }

        $condition = new InCondition(
            CourseGroupRelation :: PROPERTY_GROUP_ID,
            $group_ids,
            CourseGroupRelation :: get_table_name());

        array_splice($courses, 0, 0, CourseDataManager :: retrieve_user_courses($condition)->as_array());

        $course_settings_controller = CourseSettingsController :: get_instance();
        $unique_publications = array();
        foreach ($courses as $course)
        {
            if ($course_settings_controller->get_course_setting($course->get_id(), CourseSettingsConnector :: VISIBILITY) ==
                 1)
            {
                $condition = DataManager :: get_publications_condition($course, $user);
                /*$condition = DataManager :: get_publications_condition(
                    $course, $user, 'announcement',
                    Announcement::class_name()
                );
                $course_module_id = DataManager :: retrieve_course_tool_by_name('announcement')->get_id();
                $location = WeblcmsRights :: get_instance()->get_weblcms_location_by_identifier_from_courses_subtree(
                    WeblcmsRights :: TYPE_COURSE_MODULE, $course_module_id, $course->get_id());
                 */


                $location = WeblcmsRights :: get_instance()->get_weblcms_location_by_identifier_from_courses_subtree(
                    WeblcmsRights :: TYPE_ROOT,
                    $course->get_id(),
                    $course->get_id());

                $entities = array();
                $entities[CourseGroupEntity :: ENTITY_TYPE] = CourseGroupEntity :: get_instance($course->get_id());
                $entities[CourseUserEntity :: ENTITY_TYPE] = CourseUserEntity :: get_instance();
                $entities[CoursePlatformGroupEntity :: ENTITY_TYPE] = CoursePlatformGroupEntity :: get_instance(
                    $course->get_id());
                $entities[TemplateEntity :: ENTITY_TYPE] = TemplateEntity :: get_instance();

                $publications = DataManager :: retrieve_content_object_publications_with_view_right_granted_in_category_location(
                    $location,
                    $entities,
                    $condition,
                    new ObjectTableOrder(ContentObjectPublication :: PROPERTY_DISPLAY_ORDER_INDEX, SORT_DESC));

                if ($publications == 0)
                {
                    continue;
                }
                foreach ($publications->as_array() as $publication)
                {
                    $unique_publications[$course->get_id() . '.' . $publication[ContentObjectPublication :: PROPERTY_ID]] = $publication;
                }
            }
        }
        return $unique_publications;
    }
}
