<?php

namespace application\weblcms;

use common\libraries\NotificationMessage;
use common\libraries\ObjectTableFormActions;
use common\libraries\PropertyConditionVariable;
use common\libraries\StaticConditionVariable;

use common\libraries\Application;

use common\libraries\Display;

use common\libraries\Categorizable;
use common\libraries\PatternMatchCondition;
use common\libraries\TableFormActions;
use repository\ContentObject;
use common\libraries\ObjectTableFormAction;
use common\libraries\ActionBarRenderer;
use common\libraries\ToolbarItem;
use common\libraries\Theme;
use common\libraries\OrCondition;
use common\libraries\InCondition;
use common\libraries\Utilities;
use common\libraries\ObjectTableOrder;
use common\libraries\AndCondition;
use common\libraries\InequalityCondition;
use common\libraries\EqualityCondition;
use common\libraries\Request;
use common\libraries\Translation;
use common\libraries\ResourceManager;
use common\libraries\Path;
use common\libraries\DynamicVisualTabsRenderer;
use common\libraries\DynamicVisualTab;
use common\libraries\Session;
use common\libraries\PlatformSetting;

/**
 * $Id: viewer.class.php 200 2009-11-13 12:30:04Z kariboe $
 *
 * @package repository.lib.complex_display.assessment.component
 */

class ToolComponentBrowserComponent extends ToolComponent
{

    private $action_bar;
    private $introduction_text;
    private $publication_category_tree;
    private $publications;
    private $is_course_admin;

    public function run()
    {
        // set if we are browsing as course admin, used for displaying the
        // additional tabs and actions
        $this->is_course_admin = $this->get_course()->is_course_admin($this->get_user());

        if (!$this->is_allowed(WeblcmsRights :: VIEW_RIGHT))
        {
            $this->redirect(
                Translation :: get("NotAllowed"), true, array(), array(
                    WeblcmsManager :: PARAM_TOOL,
                    Tool :: PARAM_ACTION,
                    WeblcmsManager :: PARAM_CATEGORY
                )
            );
        }

        $this->introduction_text = $this->get_parent()->get_introduction_text();
        $this->action_bar = $this->get_action_bar();

        $this->publication_category_tree = new PublicationCategoriesTree($this);

        $publication_renderer =
            ContentObjectPublicationListRenderer :: factory($this->get_parent()->get_browser_type(), $this);

        $actions = new TableFormActions(__NAMESPACE__, Tool :: PARAM_ACTION);

        $actions->add_form_action(
            new ObjectTableFormAction(Tool :: ACTION_DELETE, Translation :: get(
                'RemoveSelected', null, Utilities :: COMMON_LIBRARIES
            ))
        );

        $actions->add_form_action(
            new ObjectTableFormAction(Tool :: ACTION_TOGGLE_VISIBILITY, Translation :: get('ToggleVisibility'), false)
        );

        if ($this->is_allowed(WeblcmsRights :: EDIT_RIGHT) && $this->get_parent() instanceof Categorizable)
        {
            $actions->add_form_action(
                new ObjectTableFormAction(Tool :: ACTION_MOVE_TO_CATEGORY, Translation :: get(
                    'MoveSelected', null, Utilities :: COMMON_LIBRARIES
                ), false)
            );
        }

        $publication_renderer->set_actions($actions);
        if ($this->get_parent()->get_browser_type() == ContentObjectPublicationListRenderer :: TYPE_GALLERY ||
            $this->get_parent()->get_browser_type() == ContentObjectPublicationListRenderer :: TYPE_SLIDESHOW
        )
        {
            $messages = Session :: retrieve(Application :: PARAM_MESSAGES);
            $messages[Application :: PARAM_MESSAGE_TYPE][] = NotificationMessage :: TYPE_WARNING;
            $messages[Application :: PARAM_MESSAGE][] = Translation :: get('BrowserWarningPreview');

            Session :: register(Application :: PARAM_MESSAGES, $messages);
        }

        $this->display_header();

        $html = array();

        $course_settings_controller = CourseSettingsController :: get_instance();

        if ($course_settings_controller->get_course_setting(
            $this->get_course_id(), CourseSettingsConnector :: ALLOW_INTRODUCTION_TEXT
        )
        )
        {
            $html[] = $this->get_parent()->display_introduction_text($this->introduction_text);
        }

        $html[] = $this->action_bar->as_html();
        $html[] = '<div id="action_bar_browser" style="width:100%;">';

        if ($this->get_parent() instanceof Categorizable)
        {
            $html[] = '<div class="tree_menu_on_top" style="max-height:150px; overflow: auto;">';
            $html[] = '<div id="tree_menu_hide_container" class="tree_menu_hide_container" style="float: right;' .
                'overflow: auto; ">';
            $html[] = '<a id="tree_menu_action_hide" class="tree_menu_hide" href="#">' . Translation :: get('ShowAll') .
                '</a>';
            $html[] = '</div>';
            $html[] = '<div id=tree style="width:90%;overflow: auto;">';
            $html[] = $this->publication_category_tree->render_as_tree();
            $html[] = '</div>';
            $html[] = ResourceManager :: get_instance()->get_resource_html(
                Path :: get(WEB_APP_PATH) . 'weblcms/resources/javascript/tree_menu.js'
            );
            $html[] = '</div>';

            $cat_id = intval(Request :: get(WeblcmsManager :: PARAM_CATEGORY));

            if (!$cat_id || $cat_id == 0)
            {
                $cat_name = Translation :: get('Root');
            }
            else
            {
                $category = $this->retrieve_category($cat_id);
                if ($category)
                {
                    $cat_name = $category->get_name();
                }
                else
                {
                    $cat_name = Translation :: get('Root');
                }
            }
            $html[] = '<div style="color: #797268; font-weight: bold;">'.
                    Translation :: get('CurrentCategory').': ' . $cat_name . '</div><br />';
        }

        echo implode("\n", $html);

        $html = array();

        $type = $this->get_publication_type();

        $tabs = new DynamicVisualTabsRenderer('tool_browser');

        if ($this->is_course_admin)
        {
            $tabs->add_tab(
                new DynamicVisualTab(Tool :: PUBLICATION_TYPE_ALL, Translation :: get('AllPublications'),
                    Theme :: get_common_image_path() . 'treemenu/shared_objects.png', $this->get_url(
                        array(
                            Tool :: PARAM_BROWSE_PUBLICATION_TYPE => Tool :: PUBLICATION_TYPE_ALL
                        )
                    ), $type == Tool :: PUBLICATION_TYPE_ALL)
            );
        }

        $tabs->add_tab(
            new DynamicVisualTab(Tool :: PUBLICATION_TYPE_FROM_ME, Translation :: get('PublishedForMe'),
                Theme :: get_common_image_path() . 'treemenu/shared_objects.png', $this->get_url(
                    array(
                        Tool :: PARAM_BROWSE_PUBLICATION_TYPE => Tool :: PUBLICATION_TYPE_FOR_ME
                    )
                ), $type == Tool :: PUBLICATION_TYPE_FOR_ME)
        );

        $tabs->add_tab(
            new DynamicVisualTab(Tool :: PUBLICATION_TYPE_FROM_ME, Translation :: get('MyPublications'),
                Theme :: get_common_image_path() . 'treemenu/publication.png', $this->get_url(
                    array(
                        Tool :: PARAM_BROWSE_PUBLICATION_TYPE => Tool :: PUBLICATION_TYPE_FROM_ME
                    )
                ), $type == Tool :: PUBLICATION_TYPE_FROM_ME)
        );
        $html[] = $publication_renderer->as_html();
        $html[] = '<div class="clear"></div>';

        if (method_exists($this->get_parent(), 'show_additional_information'))
        {
            $this->get_parent()->show_additional_information($this);
        }
        $html[] = '</div>';

        $tabs->set_content(implode("\n", $html));

        if ($this->get_publication_count() > 0 &&
            $this->get_parent()->get_tool_registration()->get_section_type() == CourseSection :: TYPE_DISABLED
        )
        {
            Display :: warning_message(Translation :: get('ToolInvisible'));
        }

        echo $tabs->render();

        $this->display_footer();
    }

    /**
     * Retrieves the publications
     *
     * @return array An array of ContentObjectPublication objects
     */
    public function get_publications($offset, $max_objects, ObjectTableOrder $object_table_order)
    {
        if (empty($this->publications))
        {

            if ($this->get_publication_type() == Tool :: PUBLICATION_TYPE_FROM_ME)
            {

                $publications_resultset = DataManager :: retrieve_my_publications(
                    $this->get_location(), $this->get_entities(), $this->get_publication_conditions(),
                    $object_table_order, $offset, $max_objects, $this->get_user_id()
                );
            }
            elseif ($this->get_publication_type() == Tool :: PUBLICATION_TYPE_ALL)
            {
                $publications_resultset = DataManager :: retrieve_content_object_publications(
                    $this->get_publication_conditions(), $object_table_order, $offset, $max_objects
                );
            }
            else
            {
                $publications_resultset =
                    DataManager :: retrieve_content_object_publications_with_view_right_granted_in_category_location(
                        $this->get_location(), $this->get_entities(), $this->get_publication_conditions(),
                        $object_table_order, $offset, $max_objects, $this->get_user_id()
                    );
            }
            if ($publications_resultset)
            {
                $this->publications = $publications_resultset->as_array();
            }
        }

        if ($this->publications)
        {
            return $this->publications;
        }
    }

    /**
     * Retrieves the number of published content objects
     *
     * @return int
     */
    public function get_publication_count()
    {
        if ($this->get_publication_type() == Tool :: PUBLICATION_TYPE_FROM_ME)
        {
            $count = DataManager :: count_my_publications(
                $this->get_location(), $this->get_entities(), $this->get_publication_conditions(), $this->get_user_id()
            );
        }
        elseif ($this->get_publication_type() == Tool :: PUBLICATION_TYPE_ALL)
        {
            $count = DataManager :: count_content_object_publications($this->get_publication_conditions());
        }
        else
        {
            $count = DataManager :: count_content_object_publications_with_view_right_granted_in_category_location(
                $this->get_location(), $this->get_entities(), $this->get_publication_conditions(), $this->get_user_id()
            );
        }
        return $count;
    }

    public function get_action_bar()
    {
        $action_bar = new ActionBarRenderer(ActionBarRenderer :: TYPE_HORIZONTAL);

        $action_bar->set_search_url($this->get_url());
        if ($this->is_allowed(WeblcmsRights :: ADD_RIGHT))
        {
            $publish_type = PlatformSetting :: get('display_publication_screen', __NAMESPACE__);
            if ($publish_type == Tool :: PUBLISH_TYPE_BOTH)
            {
                $action_bar->add_common_action(
                    new ToolbarItem(Translation :: get('QuickPublish', null, Utilities :: COMMON_LIBRARIES),
                        Theme :: get_common_image_path() . 'action_publish.png', $this->get_url(
                            array(
                                Tool :: PARAM_ACTION       => Tool :: ACTION_PUBLISH,
                                Tool :: PARAM_PUBLISH_MODE => Tool :: PUBLISH_MODE_QUICK
                            )
                        ), ToolbarItem :: DISPLAY_ICON_AND_LABEL)
                );
            }
            
            //added tool dependent publish button
            $tool_dependent_publish = PlatformSetting :: get('tool_dependent_publish_button', __NAMESPACE__);
            
            if ($tool_dependent_publish == Tool :: PUBLISH_INDEPENDENT)
            {
                $action_bar->add_common_action(
                    new ToolbarItem(Translation :: get('Publish', null, Utilities :: COMMON_LIBRARIES),
                        Theme :: get_common_image_path() . 'action_publish.png', $this->get_url(
                            array(
                                Tool :: PARAM_ACTION => Tool :: ACTION_PUBLISH
                            )
                        ), ToolbarItem :: DISPLAY_ICON_AND_LABEL)
                );
            }
            else {
                $tool = Request :: get('tool');
                $action_bar->add_common_action(
                    new ToolbarItem(Translation :: get('PublishToolDependent', array (
                        'TYPE' => Translation :: get ('TypeNameSingle', null, 'application\weblcms\tool\\' . $tool)
                    ), Utilities :: COMMON_LIBRARIES),
                        Theme :: get_common_image_path() . 'action_publish.png', $this->get_url(
                            array(
                                Tool :: PARAM_ACTION => Tool :: ACTION_PUBLISH
                            )
                        ), ToolbarItem :: DISPLAY_ICON_AND_LABEL)
                );
            }
 

        }

        if ($this->is_course_admin)
        {
            $action_bar->add_common_action(
                new ToolbarItem(Translation :: get('ManageRights', null, Utilities :: COMMON_LIBRARIES),
                    Theme :: get_common_image_path() . 'action_rights.png', $this->get_url(
                        array(
                            Tool :: PARAM_ACTION             => Tool :: ACTION_EDIT_RIGHTS,
                            WeblcmsManager :: PARAM_CATEGORY => Request :: get(WeblcmsManager :: PARAM_CATEGORY)
                        )
                    ), ToolbarItem :: DISPLAY_ICON_AND_LABEL)
            );
        }

        $action_bar->add_common_action(
            new ToolbarItem(Translation :: get('ShowAll', null, Utilities :: COMMON_LIBRARIES),
                Theme :: get_common_image_path() . 'action_browser.png', $this->get_url(
                    array(
                        Tool :: PARAM_ACTION => null
                    )
                ), ToolbarItem :: DISPLAY_ICON_AND_LABEL)
        );

        if ($this->is_course_admin && $this->get_parent() instanceof Categorizable)
        {
            $action_bar->add_common_action(
                new ToolbarItem(Translation :: get('ManageCategories', null, Utilities :: COMMON_LIBRARIES),
                    Theme :: get_common_image_path() . 'action_category.png', $this->get_url(
                        array(
                            Tool :: PARAM_ACTION => Tool :: ACTION_MANAGE_CATEGORIES
                        )
                    ), ToolbarItem :: DISPLAY_ICON_AND_LABEL)
            );
        }

        $course_settings_controller = CourseSettingsController :: get_instance();

        if (!$this->introduction_text && $this->is_course_admin && $course_settings_controller->get_course_setting(
            $this->get_course_id(), CourseSettingsConnector :: ALLOW_INTRODUCTION_TEXT
        )
        )
        {
            $action_bar->add_common_action(
                new ToolbarItem(Translation :: get('PublishIntroductionText', null, Utilities :: COMMON_LIBRARIES),
                    Theme :: get_common_image_path() . 'action_introduce.png', $this->get_url(
                        array(
                            Tool :: PARAM_ACTION => Tool :: ACTION_PUBLISH_INTRODUCTION
                        )
                    ), ToolbarItem :: DISPLAY_ICON_AND_LABEL)
            );
        }

        if (method_exists($this->get_parent(), 'get_tool_actions'))
        {
            $action_bar->set_tool_actions($this->get_parent()->get_tool_actions());
        }

        $browser_types = $this->get_parent()->get_available_browser_types();

        if (count($browser_types) > 1)
        {
            foreach ($browser_types as $browser_type)
            {
                $action_bar->add_tool_action(
                    new ToolbarItem(Translation :: get(
                        Utilities :: underscores_to_camelcase($browser_type) . 'View', null,
                        Utilities :: COMMON_LIBRARIES
                    ), Theme :: get_common_image_path() . 'view_' . $browser_type . '.png', $this->get_url(
                        array(
                            Tool :: PARAM_BROWSER_TYPE => $browser_type
                        )
                    ), ToolbarItem :: DISPLAY_ICON_AND_LABEL)
                );
            }
        }

        return $action_bar;
    }

    public function get_publication_conditions()
    {
        $conditions = array();

        $type = $this->get_publication_type();
        switch ($type)
        {
            // Begin with the publisher condition when FROM_ME and add the
            // remaining conditions. Skip the publisher
            // condition when ALL.
            case Tool :: PUBLICATION_TYPE_FROM_ME :
                $va_id = Session :: get(Tool :: PARAM_VIEW_AS_ID);
                $course_id = Session :: get(Tool :: PARAM_VIEW_AS_COURSE_ID);
                $user_id = Session :: get_user_id();

                $publisher_id = (isset($va_id) && isset($course_id) && $course_id == $this->get_course_id()) ?
                    $va_id : $user_id;

                $conditions[] = new EqualityCondition(
                    new PropertyConditionVariable(
                        ContentObjectPublication :: class_name(),
                        ContentObjectPublication :: PROPERTY_PUBLISHER_ID
                    ),
                    new StaticConditionVariable($publisher_id)
                );

            case Tool :: PUBLICATION_TYPE_ALL :
                $conditions[] = new EqualityCondition(
                    new PropertyConditionVariable(
                        ContentObjectPublication :: class_name(),
                        ContentObjectPublication :: PROPERTY_COURSE_ID
                    ),
                    new StaticConditionVariable($this->get_course_id())
                );

                if ($this->get_tool_id())
                {
                    $conditions[] = new EqualityCondition(
                        new PropertyConditionVariable(
                            ContentObjectPublication :: class_name(),
                            ContentObjectPublication :: PROPERTY_TOOL
                        ),
                        new StaticConditionVariable($this->get_tool_id())
                    );
                }

                $category_id = Request :: get(WeblcmsManager :: PARAM_CATEGORY);
                if (!$category_id)
                {
                    $category_id = 0;
                }

                $conditions[] = new EqualityCondition(
                    new PropertyConditionVariable(
                        ContentObjectPublication :: class_name(),
                        ContentObjectPublication :: PROPERTY_CATEGORY_ID
                    ),
                    new StaticConditionVariable($category_id)
                );

                break;
            default :

                $from_date_variables = new PropertyConditionVariable(
                    ContentObjectPublication :: class_name(),
                    ContentObjectPublication :: PROPERTY_FROM_DATE
                );

                $to_date_variable = new PropertyConditionVariable(
                    ContentObjectPublication :: class_name(),
                    ContentObjectPublication :: PROPERTY_TO_DATE
                );

                $time_conditions = array();

                $time_conditions[] = new EqualityCondition(
                    new PropertyConditionVariable(
                        ContentObjectPublication :: class_name(),
                        ContentObjectPublication :: PROPERTY_HIDDEN
                    ),
                    new StaticConditionVariable(0)
                );

                $forever_conditions = array();

                $forever_conditions[] = new EqualityCondition(
                    $from_date_variables,
                    new StaticConditionVariable(0)
                );

                $forever_conditions[] = new EqualityCondition(
                    $to_date_variable,
                    new StaticConditionVariable(0)
                );

                $forever_condition = new AndCondition($forever_conditions);

                $between_conditions = array();

                $between_conditions[] = new InequalityCondition(
                    $from_date_variables,
                    InequalityCondition :: LESS_THAN_OR_EQUAL,
                    new StaticConditionVariable(time())
                );

                $between_conditions[] = new InequalityCondition(
                    $to_date_variable,
                    InequalityCondition :: GREATER_THAN_OR_EQUAL,
                    new StaticConditionVariable(time())
                );

                $between_condition = new AndCondition($between_conditions);

                $time_conditions[] = new OrCondition(array($forever_condition, $between_condition));

                $conditions[] = new AndCondition($time_conditions);
                break;
        }

        if ($this->get_search_condition())
        {
            $conditions[] = $this->get_search_condition();
        }

        $conditions[] = new InCondition(
            new PropertyConditionVariable(ContentObject :: class_name(), ContentObject :: PROPERTY_TYPE),
            $this->get_allowed_types()
        );

        if (method_exists($this->get_parent(), 'get_tool_conditions'))
        {
            foreach ($this->get_parent()->get_tool_conditions() as $tool_condition)
            {
                $conditions[] = $tool_condition;
            }
        }

        if ($conditions)
        {
            return new AndCondition($conditions);
        }
        else
        {
            return null;
        }
    }

    public function get_search_condition()
    {
        $query = $this->action_bar->get_query();
        if (isset($query) && $query != '')
        {
            $conditions[] = new PatternMatchCondition(
                new PropertyConditionVariable(
                    ContentObject :: class_name(), ContentObject :: PROPERTY_TITLE
                ),
                '*' . $query . '*'
            );

            $conditions[] = new PatternMatchCondition(
                new PropertyConditionVariable(
                    ContentObject :: class_name(), ContentObject :: PROPERTY_DESCRIPTION
                ),
                '*' . $query . '*'
            );

            return new OrCondition($conditions);
        }

        return null;
    }

    public function count_tool_categories()
    {
        $conditions = array();

        $conditions[] = new EqualityCondition(ContentObjectPublicationCategory :: PROPERTY_TOOL, $this->get_tool_id());

        $conditions[] = new EqualityCondition(
            ContentObjectPublicationCategory :: PROPERTY_COURSE, $this->get_course_id()
        );

        $condition = new AndCondition($conditions);

        return DataManager :: count(ContentObjectPublicationCategory :: class_name(), $condition);
    }

    private function retrieve_category($category_id)
    {
        $conditions[] = new EqualityCondition(ContentObjectPublicationCategory :: PROPERTY_ID, $category_id);

        $conditions[] = new EqualityCondition(
            ContentObjectPublicationCategory :: PROPERTY_COURSE, $this->get_parent()->get_course_id()
        );

        $conditions[] = new EqualityCondition(
            ContentObjectPublicationCategory :: PROPERTY_TOOL, $this->get_parent()->get_tool_id()
        );

        $condition = new AndCondition($conditions);

        $objects = DataManager :: retrieves(ContentObjectPublicationCategory :: class_name(), $condition);

        return $objects->next_result();
    }

    public function get_publication_type()
    {
        $type = Request :: get(Tool :: PARAM_BROWSE_PUBLICATION_TYPE);
        if (!$type)
        {
            if ($this->is_course_admin)
            {
                $type = Tool :: PUBLICATION_TYPE_ALL;
            }
            else
            {
                $type = Tool :: PUBLICATION_TYPE_FOR_ME;
            }
        }

        return $type;
    }

    /**
     * Returns the default object table order for the browser.
     * Can be "overridden" by the individual component to
     * force a different order if needed. Because the individual component is
     * not an actual implementation but merely
     * this parent, there is a check if the method exists.
     *
     * @return ObjectTableOrder
     */
    public function get_default_order_property()
    {
        if (method_exists($this->get_parent(), 'get_default_order_property'))
        {
            return $this->get_parent()->get_default_order_property();
        }
        return new ObjectTableOrder(ContentObjectPublication :: PROPERTY_DISPLAY_ORDER_INDEX);
    }

    public function tool_category_has_new_publications($category_id)
    {
        return DataManager :: tool_category_has_new_publications(
            $this->get_tool_id(), $this->get_user(), $this->get_course(), $category_id
        );
    }

}
