<?php
namespace application\weblcms;

use common\libraries\DelegateComponent;
use repository\content_object\introduction\Introduction;

use common\libraries\Breadcrumb;
use common\libraries\BreadcrumbTrail;
use common\libraries\Request;
use common\libraries\Translation;
use repository\ContentObjectForm;
use repository\RepositoryRights;

/**
 * $Id: edit.class.php 216 2009-11-13 14:08:06Z kariboe $
 *
 * @package application.lib.weblcms.tool.component
 */
class ToolComponentContentObjectUpdaterComponent extends ToolComponent implements DelegateComponent
{

    public function run()
    {

        $pid = Request :: get(Tool :: PARAM_PUBLICATION_ID) ? Request :: get(Tool :: PARAM_PUBLICATION_ID) :
            $_POST[Tool :: PARAM_PUBLICATION_ID];

        $publication = DataManager :: retrieve(ContentObjectPublication :: class_name(), $pid);

        $has_edit_right = (
            $publication->get_content_object()->get_owner_id() == $this->get_user_id() ||
            RepositoryRights :: get_instance()->is_allowed_in_user_subtree(
                RepositoryRights :: COLLABORATE_RIGHT, $publication->get_content_object()->get_id(),
                RepositoryRights :: TYPE_USER_CONTENT_OBJECT, $publication->get_content_object()->get_owner_id(),
                $this->get_user_id()
            )
        );

        if ($has_edit_right)
        {
            $content_object = $publication->get_content_object();

            if($content_object->get_type() == Introduction :: class_name())
            {
                $publication->ignore_display_order();
            }

            BreadcrumbTrail :: get_instance()->add(
                new Breadcrumb(
                    $this->get_url(), Translation :: get(
                        'ToolContentObjectUpdateComponent', array(
                            'TITLE' => $content_object->get_title()
                        )
                    )
                )
            );

            $form = ContentObjectForm :: factory(
                ContentObjectForm :: TYPE_EDIT, $content_object, 'edit', 'post', $this->get_url(
                    array(
                        Tool :: PARAM_PUBLICATION_ID => $pid
                    )
                )
            );

            if ($form->validate())
            {
                $form->update_content_object();
                if ($form->is_version())
                {
                    $publication->set_content_object_id($content_object->get_latest_version_id());
                    $publication->update();
                }

                $tool = $this->get_tool_id();

                if ($tool == 'learning_path')
                {
                    $params[Tool :: PARAM_ACTION] = null;
                    $params['display_action'] = 'view';
                    $params[Tool :: PARAM_PUBLICATION_ID] = Request :: get(Tool :: PARAM_PUBLICATION_ID);
                }

                if ($tool != 'learning_path')
                {
                    $filter = array(Tool :: PARAM_ACTION, Tool :: PARAM_PUBLICATION_ID);
                }
                else
                {
                    $filter = array();
                }

                $this->redirect(Translation :: get('ContentObjectUpdated'), false, $params, $filter);
            }
            else
            {
                $this->display_header();
                $form->display();
                $this->display_footer();
            }
        }
        else
        {
            $this->redirect(
                Translation :: get("NotAllowed"), true, array(
                    Tool :: PARAM_PUBLICATION_ID => null,
                    'tool_action'                => null
                )
            );
        }
    }
}
