<?php

namespace application\weblcms;

use common\libraries\Breadcrumb;

use common\libraries\BreadcrumbTrail;

use common\libraries\Request;
use common\libraries\InCondition;
use common\libraries\Translation;
use common\libraries\Utilities;

use repository\ShareManager;

/**
 * Tool component which can be used to share content objects
 */
class ToolComponentShareContentObjectsComponent extends ToolComponent
{

    /**
     * Runs this component and displays its output.
     */
    public function run()
    {
        $publication_ids = Request :: get(Tool :: PARAM_PUBLICATION_ID);

        if (!empty($publication_ids))
        {
            if (!is_array($publication_ids))
            {
                $publication_ids = array($publication_ids);
            }

            $condition = new InCondition(ContentObjectPublication :: PROPERTY_ID, $publication_ids);
            $publications = DataManager :: retrieves(ContentObjectPublication :: class_name(), $condition);

            $content_objects = array();

            while ($publication = $publications->next_result())
            {
                $content_objects[] = $publication->get_content_object();
            }

            if (count($content_objects) == 1)
            {
                BreadcrumbTrail :: get_instance()->add(
                    new Breadcrumb(
                        $this->get_url(), Translation :: get(
                            'ShareContentObject', array(
                                'TITLE' => $content_objects[0]->get_title()
                            )
                        )
                    )
                );
            }
            else
            {
                BreadcrumbTrail :: get_instance()->add(
                    new Breadcrumb($this->get_url(), Translation :: get('ShareContentObjects'))
                );
            }

            ShareManager :: launch($this, $content_objects);
        }
        else
        {
            $this->display_warning_page(
                htmlentities(
                    Translation :: get(
                        'NoObjectSelected', array(
                            'OBJECT' => Translation :: get('ContentObject')
                        ), Utilities :: COMMON_LIBRARIES
                    )
                )
            );
        }
    }

    public function add_additional_breadcrumbs(BreadcrumbTrail $breadcrumbtrail)
    {
        $breadcrumbtrail->add_help('tool_share_content_objects');
    }
}
