<?php
namespace application\weblcms;

use common\libraries\Breadcrumb;

use common\libraries\DelegateComponent;

use common\extensions\feedback_manager\FeedbackManager;
use common\libraries\ActionBarRenderer;
use common\libraries\BreadcrumbTrail;
use common\libraries\ObjectNotExistException;
use common\libraries\Request;
use common\libraries\Translation;
use common\libraries\Utilities;

/**
 * $Id: viewer.class.php 200 2009-11-13 12:30:04Z kariboe $
 *
 * @package repository.lib.complex_display.assessment.component
 */
class ToolComponentViewerComponent extends ToolComponent implements DelegateComponent
{

    private $action_bar;
    private $html;
    private $feedback_allowed;

    public function run()
    {
        // check if the content object has indeed been published for the user
        $publication = DataManager :: retrieve(ContentObjectPublication :: class_name(), $this->get_publication_id());

        if(!$publication)
        {
            throw new ObjectNotExistException(Translation :: get('Publication'), $this->get_publication_id());
        }

        if (!$this->is_allowed(WeblcmsRights :: VIEW_RIGHT, $publication))
        {
            $this->redirect(
                Translation :: get("NotAllowed", null, Utilities :: COMMON_LIBRARIES), true, array(), array(
                    Tool :: PARAM_ACTION, Tool :: PARAM_PUBLICATION_ID
                )
            );
        }

        $object = $publication->get_content_object();

        BreadcrumbTrail :: get_instance()->add(
            new Breadcrumb(
                $this->get_url(),
                Translation :: get('ToolViewerComponent', array('TITLE' => $object->get_title()))
            )
        );

        $this->action_bar = $this->get_action_bar();
        $renderer = new ContentObjectPublicationDetailsRenderer($this);
        $this->html = $renderer->as_html();

        $course_settings_controller = CourseSettingsController :: get_instance();
        $this->feedback_allowed = $course_settings_controller->get_course_setting(
            $this->get_course_id(), CourseSettingsConnector :: ALLOW_FEEDBACK
        );

        $feedback_manager = new FeedbackManager(
            $this,
            WeblcmsManager :: APPLICATION_NAME,
            $publication->get_id(),
            null,
            null,
            $this->is_allowed(WeblcmsRights :: DELETE_RIGHT, $publication)
        );

        return $feedback_manager->run();
    }

    public function display_header()
    {
        parent :: display_header();
        echo $this->action_bar->as_html();
        echo '<div id="action_bar_browser">';
        echo $this->html;
    }

    public function display_footer()
    {
        echo '</div>';
        parent :: display_footer();
    }

    public function get_publication_id()
    {
        return Request :: get(Tool :: PARAM_PUBLICATION_ID);
    }

    public function get_publication_count()
    {
        return 1;
    }

    public function get_action_bar()
    {
        $action_bar = new ActionBarRenderer(ActionBarRenderer :: TYPE_HORIZONTAL);

        if ($this->is_allowed(WeblcmsRights :: EDIT_RIGHT))
        {
            $action_bar->add_common_action($this->get_access_details_toolbar_item($this));
        }

        if (method_exists($this->get_parent(), 'get_tool_actions'))
        {
            $action_bar->set_tool_actions($this->get_parent()->get_tool_actions());
        }

        return $action_bar;
    }

    public function add_actionbar_item($item)
    {
        $this->action_bar->add_common_action($item);
    }

    public function can_create_feedback()
    {
        return $this->feedback_allowed;
    }

    public function can_update_feedback($feedback)
    {
        return $this->feedback_allowed;
    }

    public function can_delete_feedback($feedback)
    {
        return $this->feedback_allowed;
    }

    public function set_action_bar($action_bar)
    {
        $this->action_bar = $action_bar;
    }

}
