<?php

namespace application\weblcms;

use common\libraries\Request;
use common\extensions\repo_viewer\RepoViewer;
use common\libraries\BreadcrumbTrail;
use common\libraries\Breadcrumb;
use user\UserDataManager;
use repository\ContentObject;
use repository\RepositoryDataManager;
use repository\content_object\handbook_item\HandbookItem;
use repository\ComplexContentObjectItem;
use common\libraries\Translation;
use common\libraries\Utilities;
use repository\content_object\link\Link;
use common\libraries\EqualityCondition;
use repository\content_object\youtube\Youtube;
use repository\content_object\rss_feed\RssFeed;
use repository\content_object\handbook\Handbook;
use common\extensions\repo_viewer\RepoViewerInterface;
use repository\content_object\document\Document;
use common\extensions\repo_viewer\RepoViewerBrowserComponent;
use repository\content_object\bookmark\BookmarkForm;
use repository\content_object\bookmark\Bookmark;
use common\libraries\Display;
use common\libraries\Application;
use application\weblcms\WeblcmsManager;
use application\weblcms\DataManager as WeblcmsDataManager;
use application\weblcms\course\DataManager as CourseDataManager;

/**
 * Component to create a bookmark for a course
 */
class WeblcmsManagerCourseBookmarkCreatorComponent extends WeblcmsManager implements \repository\viewer\ViewerInterface
{

    private $parents;
    private $items = array();

    /**
     * Runs this component and displays its output.
     */
    function run()
    {
        $course_id = Request :: get(WeblcmsManager::PARAM_COURSE);


        $content_object = new Bookmark();

        //set title
        $wdm = WeblcmsDataManager::get_instance();
        $course = CourseDataManager::retrieve_course($course_id);
        $title = $course->get_title();
        $content_object->set_title($title);

        //set application
        $content_object->set_application(self::APPLICATION_NAME);

        //set url
        $params_bookmark = array();
        $params_bookmark[Application::PARAM_APPLICATION] = WeblcmsManager::APPLICATION_NAME;
        $params_bookmark[Application::PARAM_ACTION] = WeblcmsManager::ACTION_VIEW_COURSE;
        $params_bookmark[WeblcmsManager::PARAM_COURSE] = $course_id;


        $url_bookmark = $this->get_url($params_bookmark);
        $content_object->set_url($url_bookmark);

        $params_form = array();
        $params_form[Application::PARAM_APPLICATION] = WeblcmsManager::APPLICATION_NAME;
        $params_form[WeblcmsManager :: PARAM_ACTION] = WeblcmsManager :: ACTION_CREATE_BOOKMARK;
        $params_form[WeblcmsManager :: PARAM_COURSE] = $course_id;

        $method = 'post';
        $action = $this->get_url($params_form);
        $extra = null;
        $additional_elements = null;
        $allow_new_version = false;
        $form = new BookmarkForm(BookmarkForm::TYPE_CREATE, $content_object, $form_name, $method, $action, $extra, $additional_elements, $allow_new_version);

        if ($form->validate())
        {
            $values = $form->exportValues();
            //create bookmark
            $bookmark = new Bookmark();
            $bookmark->set_title($values[ContentObject :: PROPERTY_TITLE]);
            $bookmark->set_description($values[ContentObject :: PROPERTY_DESCRIPTION]);
            $bookmark->set_application($values[Bookmark::PROPERTY_APPLICATION]);
            $bookmark->set_url($values[Bookmark::PROPERTY_URL]);
            $bookmark->set_owner_id($this->get_user_id());

            $success = $bookmark->create();



            $this->display_header();

            if ($success)
            {
                echo Translation::get('CreatingBookmarkSuccess');
                echo '<script type="text/javascript">window.close()</script>';
            }
            else
            {
                echo Translation::get('CreatingBookmarkFailure');
            }


            $this->display_footer();
        }
        else
        {

            //set title
            $content_object->set_title($title);

            //set application
            $content_object->set_application(self::APPLICATION_NAME);

            //set url
            $params_bookmark = array();
            $params_bookmark[Application::PARAM_APPLICATION] = WeblcmsManager::APPLICATION_NAME;
            $params_bookmark[Application::PARAM_ACTION] = WeblcmsManager::ACTION_VIEW_COURSE;
            $params_bookmark[WeblcmsManager::PARAM_COURSE] = $course_id;

            $url_bookmark = $this->get_url($params_bookmark);
            $content_object->set_url($url_bookmark);

            $params_form = array();
            $params_form[Application::PARAM_APPLICATION] = WeblcmsManager::APPLICATION_NAME;
            $params_form[WeblcmsManager :: PARAM_ACTION] = WeblcmsManager :: ACTION_CREATE_BOOKMARK;
            $params_form[WeblcmsManager :: PARAM_COURSE] = $course_id;

            $html[] = $form->toHtml();


            $this->display_header();
            echo implode("\n", $html);
            $this->display_footer();
        }
    }

    function get_allowed_content_object_types()
    {
        return Handbook::get_allowed_content();
    }

    function display_header()
    {
        Display :: small_header();
    }

    function display_footer()
    {
        Display::small_footer();
    }

}

?>