<?php
namespace application\weblcms;

use common\libraries\Header;
use core\admin\AdminManager;
use common\libraries\Redirect;
use common\libraries\Display;
use common\libraries\DynamicTabsRenderer;
use common\libraries\Breadcrumb;
use common\libraries\BreadcrumbTrail;
use common\libraries\Translation;
use common\libraries\Utilities;

/**
 * $Id: course_importer.class.php 218 2009-11-13 14:21:26Z kariboe $
 *
 * @package application.lib.weblcms.weblcms_manager.component
 */
/**
 * Weblcms component allows the use to import a course
 */
class WeblcmsManagerCourseImporterComponent extends WeblcmsManager
{

    /**
     * Runs this component and displays its output.
     */
    public function run()
    {
        Header :: set_section('admin');

        if (!$this->get_user()->is_platform_admin())
        {
            throw new \common\libraries\NotAllowedException();
        }

        $form = new CourseImportForm(CourseImportForm :: TYPE_IMPORT, $this->get_url());

        if ($form->validate())
        {
            $success = $form->import_courses();
            $this->redirect(
                Translation :: get(
                    $success ? 'CsvCoursesProcessed' : 'CsvCoursesNotProcessed'
                ) . '<br />' . $form->get_failed_csv(), ($success ? false : true)
            );
        }
        else
        {
            $this->display_header();
            echo '<div class="clear"></div><br />';
            $form->display();
            $this->display_extra_information();
            $this->display_footer();
        }
    }

    public function display_extra_information()
    {
        $html = array();
        $html[] = '<p>' . Translation :: get('CSVMustLookLike') . ' (' . Translation :: get(
            'MandatoryFields'
        ) . ')</p>';
        $html[] = '<blockquote>';
        $html[] = '<pre>';
        $html[] = '<b>action</b>;<b>code</b>;<b>title</b>;<b>category</b>;<b>teacher</b>;<b>course_type</b>;language';
        $html[] = 'A;BIO0015;Biology;BIO;username;Curricula;en';
        $html[] = '</pre>';
        $html[] = '</blockquote>';
        $html[] = '<p>' . Translation :: get('Details') . '</p>';
        $html[] = '<blockquote>';
        $html[] = '<u><b>Action</u></b>'; // NO translation! This field is
        // always plain 'action' in each CSV,
        // regardless the language!
        $html[] = '<br />A: ' . Translation :: get('Add', null, Utilities :: COMMON_LIBRARIES);
        $html[] = '<br />U: ' . Translation :: get('Update', null, Utilities :: COMMON_LIBRARIES);
        $html[] = '<br />D: ' . Translation :: get('Delete', null, Utilities :: COMMON_LIBRARIES);
        $html[] = '</blockquote>';

        echo implode($html, "\n");
    }

    public function add_additional_breadcrumbs(BreadcrumbTrail $breadcrumbtrail)
    {

        if ($this->get_user()->is_platform_admin())
        {
            $breadcrumbtrail->add(
                new Breadcrumb(
                    Redirect :: get_link(
                        AdminManager :: APPLICATION_NAME, array(
                            AdminManager :: PARAM_ACTION => AdminManager :: ACTION_ADMIN_BROWSER
                        ), array(), false, Redirect :: TYPE_CORE
                    ), Translation :: get('TypeName', null, 'core\admin')
                )
            );
            $breadcrumbtrail->add(
                new Breadcrumb(
                    Redirect :: get_link(
                        AdminManager :: APPLICATION_NAME, array(
                            AdminManager :: PARAM_ACTION              => AdminManager :: ACTION_ADMIN_BROWSER,
                            DynamicTabsRenderer :: PARAM_SELECTED_TAB => WeblcmsManager :: APPLICATION_NAME
                        ), array(), false, Redirect :: TYPE_CORE
                    ), Translation :: get('Courses')
                )
            );
        }
    }

    public function get_additional_parameters()
    {
        return array();
    }
}
