<?php
namespace application\weblcms;

use common\libraries\Header;
use core\admin\AdminManager;
use common\libraries\Redirect;
use common\libraries\Application;
use common\libraries\DynamicTabsRenderer;
use common\libraries\Breadcrumb;
use common\libraries\BreadcrumbTrail;
use common\libraries\Request;
use common\libraries\Translation;
use application\weblcms\course\Course;
use application\weblcms\course\DataManager as CourseDataManager;



/**
 * Component to edit an existing request object
 *
 * @author Yannick Meert
 */
class WeblcmsManagerCourseRequestAllowComponent extends WeblcmsManager
{

    private $request_type;

    /**
     * Runs this component and displays its output.
     */
    public function run()
    {
        $request_ids = Request :: get(WeblcmsManager :: PARAM_REQUEST);
        $this->request_type = Request :: get(WeblcmsManager :: PARAM_REQUEST_TYPE);
        $failures = 0;

        if ($this->get_user()->is_platform_admin())
        {
            Header :: set_section('admin');
        }

        if (!$this->get_user()->is_platform_admin())
        {
            throw new \common\libraries\NotAllowedException();
        }

        $request_method = null;

        switch ($this->request_type)
        {
            case CommonRequest :: SUBSCRIPTION_REQUEST :
                $request_method = 'retrieve_request';
                break;
            case CommonRequest :: CREATION_REQUEST :
                $request_method = 'retrieve_course_create_request';
                break;
        }

        if (!is_null($request_ids) && $this->get_user()->is_platform_admin())
        {
            if (!is_array($request_ids))
            {
                $request_ids = array($request_ids);
            }

            foreach ($request_ids as $request_id)
            {
                if (!$this->update_request($request_id))
                {
                    $failures++;
                }
            }
            if ($failures)
            {
                if (count($request_ids) == 1)
                {
                    $message = 'SelectedRequestNotAllowed';
                }
                else
                {
                    $message = 'SelectedRequestsNotAllowed';
                }
            }
            else
            {
                if (count($request_ids) == 1)
                {
                    $message = 'SelectedRequestAllowed';
                }
                else
                {
                    $message = 'SelectedRequestsAllowed';
                }
            }
            $this->redirect(
                Translation :: get($message), ($failures ? true : false), array(
                    Application :: PARAM_ACTION          => WeblcmsManager :: ACTION_ADMIN_REQUEST_BROWSER,
                    WeblcmsManager :: PARAM_REQUEST      => null,
                    WeblcmsManager :: PARAM_REQUEST_TYPE => $this->request_type,
                    WeblcmsManager :: PARAM_REQUEST_VIEW =>
                    WeblcmsManagerAdminRequestBrowserComponent :: ALLOWED_REQUEST_VIEW
                )
            );
        }
        else
        {
            $this->display_error_page(htmlentities(Translation :: get('NoRequestsSelected')));
        }
    }

    public function update_request($request_id)
    {

        $request_method = null;

        switch ($this->request_type)
        {
            case CommonRequest :: SUBSCRIPTION_REQUEST :
                $class_name = CourseRequest :: class_name();
                break;
            case CommonRequest :: CREATION_REQUEST :
                $class_name = CourseCreateRequest :: class_name();
                break;
        }

        $request = CourseDataManager :: retrieve_by_id($class_name, $request_id);

        if ($this->request_type == CommonRequest :: CREATION_REQUEST)
        {
            $course = new Course();

            $course->set_title($request->get_course_name());
            $course->set_course_type_id($request->get_course_type_id());
            $course->set_visual_code(strtoupper(uniqid()));
            $course->set_titular_id($request->get_user_id());

            if (!$course->create())
            {
                return false;
            }

            $course_settings = array();
            $course_settings[CourseSettingsController :: SETTING_PARAM_COURSE_SETTINGS]
            [CourseSettingsConnector :: TITULAR] = $course->get_titular_id();

            if (!$course->create_course_settings_from_values($course_settings, true))
            {
                return false;
            }

            if (!CourseManagementRights :: get_instance()->create_rights_from_values($course, array()))
            {
                return false;
            }

            if (!CourseDataManager :: subscribe_user_to_course($course->get_id(), '1', $request->get_user_id()))
            {
                return false;
            }
        }
        else
        {
            if (!CourseDataManager :: subscribe_user_to_course($request->get_course_id(), '5', $request->get_user_id()))
            {
                return false;
            }
        }

        $request->set_decision_date(time());
        $request->set_decision(CommonRequest :: ALLOWED_DECISION);
        return $request->update($request);
    }

    public function add_additional_breadcrumbs(BreadcrumbTrail $breadcrumbtrail)
    {

        if ($this->get_user()->is_platform_admin())
        {
            $breadcrumbtrail->add(
                new Breadcrumb(
                    Redirect :: get_link(
                        AdminManager :: APPLICATION_NAME, array(
                            AdminManager :: PARAM_ACTION => AdminManager :: ACTION_ADMIN_BROWSER
                        ), array(), false, Redirect :: TYPE_CORE
                    ), Translation :: get('TypeName', null, 'core\admin')
                )
            );
            $breadcrumbtrail->add(
                new Breadcrumb(
                    Redirect :: get_link(
                        AdminManager :: APPLICATION_NAME, array(
                            AdminManager :: PARAM_ACTION              => AdminManager :: ACTION_ADMIN_BROWSER,
                            DynamicTabsRenderer :: PARAM_SELECTED_TAB => WeblcmsManager :: APPLICATION_NAME
                        ), array(), false, Redirect :: TYPE_CORE
                    ), Translation :: get('Courses')
                )
            );
            $breadcrumbtrail->add(
                new Breadcrumb(
                    $this->get_url(
                        array(
                            WeblcmsManager :: PARAM_ACTION => WeblcmsManager :: ACTION_ADMIN_REQUEST_BROWSER
                        )
                    ), Translation :: get('Requests')
                )
            );
        }
        else
        {
            $breadcrumbtrail->add(
                new Breadcrumb(
                    $this->get_url(array(WeblcmsManager :: PARAM_ACTION => null)), Translation :: get('Coursetypes')
                )
            );
        }
    }

    public function get_additional_parameters()
    {
        return array();
    }

}
