<?php
namespace application\weblcms;

use common\libraries\Authentication;
use common\libraries\NotCondition;
use common\libraries\AndCondition;
use common\libraries\EqualityCondition;
use common\libraries\Request;

use user\User;
use user\UserDataManager;

/**
 * $Id: xml_course_user_group_feed.php 218 2009-11-13 14:21:26Z kariboe $
 *
 * @package application.lib.weblcms.xml_feeds
 */
require_once dirname(__FILE__) . '/../../../../common/global.inc.php';

if (Authentication :: is_valid())
{
    $user_result_set = retrieve_users();
    $users = array();

    if ($user_result_set)
    {
        while ($user = $user_result_set->next_result())
        {
            $users[] = $user;
        }
    }
}

header('Content-Type: text/xml');
echo '<?xml version="1.0" encoding="iso-8859-1"?>', "\n", '<tree>', "\n";

dump_tree($users);

echo '</tree>';

function dump_tree($users)
{
    if (contains_results($users))
    {
        echo '<node id="user" classes="category unlinked" title="Users">', "\n";
        foreach ($users as $user)
        {
            echo '<leaf id="user_' . $user->get_id() . '" classes="' . 'type type_user' . '" title="';
            echo htmlentities($user->get_username()) . '" description="';
            echo htmlentities($user->get_fullname()) . '"/>' . "\n";
        }
        echo '</node>', "\n";
    }
}

function contains_results($objects)
{
    if (count($objects))
    {
        return true;
    }
    return false;
}

function retrieve_users()
{
    $course_id = Request :: get('course');

    if (! $course_id)
    {
        return null;
    }

    $udm = UserDataManager :: get_instance();

    // Retrieve the users directly subscribed to the course
    $relation_condition = new EqualityCondition(CourseUserRelation :: PROPERTY_COURSE_ID, $course_id);

    $course_user_relation_result_set = \application\weblcms\course\DataManager :: retrieves(
        CourseUserRelation :: class_name(),
        $relation_condition
    );

    $user_ids = array();
    while ($course_user = $course_user_relation_result_set->next_result())
    {
        $user_ids[] = $course_user->get_user();
    }

    // Retrieve the users subscribed through platform groups
    $relation_condition = new EqualityCondition(CourseGroupRelation :: PROPERTY_COURSE_ID, $course_id);

    $course_group_relations = $course_group_relations = \application\weblcms\course\DataManager :: retrieves(
        CourseGroupRelation :: class_name(),
        $relation_condition
    );

    $group_users = array();

    while ($group_relation = $course_group_relations->next_result())
    {
        $group = $group_relation->get_group_object();
        $group_user_ids = $group->get_users(true, true);

        $group_users = array_merge($group_users, $group_user_ids);
    }

    $user_ids = array_merge($user_ids, $group_users);

    if (count($user_ids) == 0)
    {
        return;
    }

    $search_query = Request :: post('query');

    // Set the conditions for the search query
    if ($search_query && $search_query != '')
    {
        $conditions[] = Utilities :: query_to_condition(
            $search_query, array(
                User :: PROPERTY_USERNAME, User :: PROPERTY_FIRSTNAME, User :: PROPERTY_LASTNAME
            )
        );
    }

    $conditions[] = new InCondition(User :: PROPERTY_ID, $user_ids);

    $exclude = Request :: post('exclude');

    if ($exclude)
    {
        if (! is_array($exclude))
        {
            $exclude = array($exclude);
        }

        $exclude_conditions = array();

        foreach ($exclude as $id)
        {
            $id = explode('_', $id);
            $exclude_condition = new NotCondition(new EqualityCondition(User :: PROPERTY_ID, $id[1]));

            $exclude_conditions[] = $exclude_condition;
        }

        if (count($exclude_conditions) > 0)
        {
            $conditions[] = new AndCondition($exclude_conditions);
        }
    }

    // Combine the conditions
    $count = count($conditions);
    if ($count > 1)
    {
        $condition = new AndCondition($conditions);
    }

    if ($count == 1)
    {
        $condition = $conditions[0];
    }

    return $udm->retrieve_users(
        $condition, null, null, array(
            new ObjectTableOrder(User :: PROPERTY_LASTNAME),
            new ObjectTableOrder(User :: PROPERTY_FIRSTNAME)
        )
    );
}
