<?php

namespace application\weblcms;

use application\weblcms\course\Course;
use application\weblcms\course\DataManager as CourseDataManager;
use common\libraries\Request;
use common\libraries\DataClassRetrievesParameters;
use common\libraries\EqualityCondition;
use common\libraries\Authentication;
use common\libraries\ObjectTableOrder;
use user\UserDataManager;

/**
 * $Id: xml_group_feed.php
 *
 * @author Pieterjan Broekaert
 */
require_once dirname(__FILE__) . '/../../../../common/global.inc.php';

$publications_tree = array();
$course = null;
$user = null;

if (Authentication :: is_valid())
{
    $category_id = Request :: get('parent_id');

    $condition = new EqualityCondition(ContentObjectPublicationCategory :: PROPERTY_PARENT, $category_id);

    $categories_tree = DataManager :: retrieves(
        ContentObjectPublicationCategory :: class_name(),
        new DataClassRetrievesParameters(
            $condition, null, null, new ObjectTableOrder(ContentObjectPublicationCategory :: PROPERTY_DISPLAY_ORDER)
        )
    )->as_array();
}

header('Content-Type: text/xml');
echo '<?xml version="1.0" encoding="UTF-8"?>' . "\n", '<tree>' . "\n";
echo dump_tree($categories_tree);
echo '</tree>';

function dump_tree($categories)
{
    if (contains_results($categories))
    {
        dump_categories_tree($categories);
    }
}

function dump_categories_tree($categories)
{
    foreach ($categories as $category)
    {
        $has_children = has_sub_categories($category->get_id()) ? 1 : 0;
        $class = get_category_class($category);

        echo '<leaf id="' . $category->get_id() . '" classes="' . $class . '" has_children="';
        echo $has_children . '" title="' . htmlspecialchars($category->get_name()) . '" description="';
        echo htmlspecialchars($category->get_name()) . '"/>' . "\n";
    }
}

function contains_results($objects)
{
    if (count($objects))
    {
        return true;
    }
    return false;
}

function has_sub_categories($category_id)
{

    $condition = new EqualityCondition(ContentObjectPublicationCategory :: PROPERTY_PARENT, $category_id);
    return DataManager :: count(ContentObjectPublicationCategory :: class_name(), $condition) > 0;
}

function get_category_class(ContentObjectPublicationCategory $category)
{
    global $course, $user;

    if (!$course)
    {
        $course = CourseDataManager :: retrieve(Course :: class_name(), $category->get_course());
    }

    if (!$user)
    {
        $user = UserDataManager :: get_current_user();
    }

    if (DataManager :: tool_category_has_new_publications(
        $category->get_tool(),
        $user,
        $course,
        $category->get_id()
    )
    )
    {
        return 'new_category';
    }
    else
    {
        return 'category';
    }
}
