<?php
namespace application\weblcms\request;

use repository\ContentObject;
use common\libraries\DelegateComponent;
use common\libraries\Display;
use common\libraries\DatetimeUtilities;
use common\libraries\ObjectTableOrder;
use common\libraries\PropertiesTable;
use common\libraries\SortableTableFromArray;
use common\libraries\InCondition;
use common\libraries\AndCondition;
use common\libraries\EqualityCondition;
use common\libraries\NewObjectTableSupport;
use common\libraries\Theme;
use common\libraries\DynamicContentTab;
use common\libraries\DynamicTabsRenderer;
use common\libraries\Path;
use common\libraries\PlatformSetting;
use common\libraries\ToolbarItem;
use common\libraries\ActionBarRenderer;
use common\libraries\Filesystem;
use common\libraries\Translation;

class BrowserComponent extends Manager implements NewObjectTableSupport, DelegateComponent
{

    private $table_type;

    function run()
    {
        $condition = new EqualityCondition(Request :: PROPERTY_USER_ID, $this->get_user_id());
        $user_requests = DataManager :: count(Request :: class_name(), $condition);

        if ($user_requests > 0 || \application\weblcms\request\rights\Rights :: get_instance()->request_is_allowed())
        {
            $tabs = new DynamicTabsRenderer('request');

            if ($user_requests > 0)
            {
                $this->table_type = RequestTable :: TYPE_PERSONAL;
                $table = new RequestTable($this);
                $tabs->add_tab(
                    new DynamicContentTab(
                        'personal_request',
                        Translation :: get('YourRequests'),
                        Theme :: get_image_path() . 'tab/personal_request.png',
                        $table->as_html()));
            }

            if (\application\weblcms\request\rights\Rights :: get_instance()->request_is_allowed())
            {
                $target_users = \application\weblcms\request\rights\Rights :: get_instance()->get_target_users(
                    $this->get_user());

                if (count($target_users) > 0)
                {
                    $target_condition = new InCondition(Request :: PROPERTY_USER_ID, $target_users);
                }
                else
                {
                    $target_condition = new EqualityCondition(Request :: PROPERTY_USER_ID, - 1);
                }

                $conditions = array();
                $conditions[] = new EqualityCondition(Request :: PROPERTY_DECISION, Request :: DECISION_PENDING);
                if (! $this->get_user()->is_platform_admin())
                {
                    $conditions[] = $target_condition;
                }
                $condition = new AndCondition($conditions);

                if (DataManager :: count(Request :: class_name(), $condition) > 0)
                {
                    $this->table_type = RequestTable :: TYPE_PENDING;
                    $table = new RequestTable($this);
                    $tabs->add_tab(
                        new DynamicContentTab(
                            RequestTable :: TYPE_PENDING,
                            Translation :: get('PendingRequests'),
                            Theme :: get_image_path() . 'decision/22/' . Request :: DECISION_PENDING . '.png',
                            $table->as_html()));
                }

                $conditions = array();
                $conditions[] = new EqualityCondition(Request :: PROPERTY_DECISION, Request :: DECISION_GRANTED);
                if (! $this->get_user()->is_platform_admin())
                {
                    $conditions[] = $target_condition;
                }
                $condition = new AndCondition($conditions);

                if (DataManager :: count(Request :: class_name(), $condition) > 0)
                {
                    $this->table_type = RequestTable :: TYPE_GRANTED;
                    $table = new RequestTable($this);
                    $tabs->add_tab(
                        new DynamicContentTab(
                            RequestTable :: TYPE_GRANTED,
                            Translation :: get('GrantedRequests'),
                            Theme :: get_image_path() . 'decision/22/' . Request :: DECISION_GRANTED . '.png',
                            $table->as_html()));
                }

                $conditions = array();
                $conditions[] = new EqualityCondition(Request :: PROPERTY_DECISION, Request :: DECISION_DENIED);
                if (! $this->get_user()->is_platform_admin())
                {
                    $conditions[] = $target_condition;
                }
                $condition = new AndCondition($conditions);

                if (DataManager :: count(Request :: class_name(), $condition) > 0)
                {
                    $this->table_type = RequestTable :: TYPE_DENIED;
                    $table = new RequestTable($this);
                    $tabs->add_tab(
                        new DynamicContentTab(
                            RequestTable :: TYPE_DENIED,
                            Translation :: get('DeniedRequests'),
                            Theme :: get_image_path() . 'decision/22/' . Request :: DECISION_DENIED . '.png',
                            $table->as_html()));
                }
            }
        }

        if ($user_requests > 0 ||
             (\application\weblcms\request\rights\Rights :: get_instance()->request_is_allowed() && $tabs->size() > 0) ||
             $this->get_user()->is_platform_admin())
        {
            $this->display_header();
            echo $this->get_action_bar()->as_html();
            echo $tabs->render();
            $this->display_footer();
        }
        else
        {
            $this->redirect(
                Translation :: get('NoRequestsFormDirectly'),
                null,
                array(self :: PARAM_ACTION => self :: ACTION_CREATE));
            // echo Display :: normal_message(Translation :: get('NoRequestsPending'), true);
        }
    }

    /**
     *
     * @see @see common\libraries.NewObjectTableSupport::get_object_table_condition()
     */
    public function get_object_table_condition($object_table_class_name)
    {
        $conditions = array();

        switch ($this->table_type)
        {
            case RequestTable :: TYPE_PENDING :
                $conditions[] = new EqualityCondition(Request :: PROPERTY_DECISION, Request :: DECISION_PENDING);
                break;
            case RequestTable :: TYPE_PERSONAL :
                $conditions[] = new EqualityCondition(Request :: PROPERTY_USER_ID, $this->get_user_id());
                break;
            case RequestTable :: TYPE_GRANTED :
                $conditions[] = new EqualityCondition(Request :: PROPERTY_DECISION, Request :: DECISION_GRANTED);
                break;
            case RequestTable :: TYPE_DENIED :
                $conditions[] = new EqualityCondition(Request :: PROPERTY_DECISION, Request :: DECISION_DENIED);
                break;
        }

        if (! $this->get_user()->is_platform_admin() &&
             \application\weblcms\request\rights\Rights :: get_instance()->request_is_allowed() &&
             $this->table_type != RequestTable :: TYPE_PERSONAL)
        {
            $target_users = \application\weblcms\request\rights\Rights :: get_instance()->get_target_users(
                $this->get_user());

            if (count($target_users) > 0)
            {
                $conditions[] = new InCondition(Request :: PROPERTY_USER_ID, $target_users);
            }
            else
            {
                $conditions[] = new EqualityCondition(Request :: PROPERTY_USER_ID, - 1);
            }
        }

        return new AndCondition($conditions);
    }

    function get_action_bar()
    {
        $action_bar = new ActionBarRenderer(ActionBarRenderer :: TYPE_HORIZONTAL);

        if ($this->request_allowed())
        {
            $action_bar->add_common_action(
                new ToolbarItem(
                    Translation :: get('RequestCourse'),
                    Theme :: get_image_path() . 'action/request.png',
                    $this->get_url(array(self :: PARAM_ACTION => self :: ACTION_CREATE))));
        }

        if ($this->get_user()->is_platform_admin())
        {
            $action_bar->add_tool_action(
                new ToolbarItem(
                    Translation :: get('ConfigureManagementRights'),
                    Theme :: get_image_path() . 'action/rights.png',
                    $this->get_url(array(self :: PARAM_ACTION => self :: ACTION_RIGHTS))));
        }

        return $action_bar;
    }

    function get_table_type()
    {
        return $this->table_type;
    }
}
?>