<?php
namespace application\weblcms\request;

use application\weblcms\CourseManagementRights;
use application\weblcms\CourseSettingsConnector;
use application\weblcms\CourseSettingsController;
use application\weblcms\course\Course;
use common\libraries\Mail;
use common\libraries\PlatformSetting;
use common\libraries\Filesystem;
use common\libraries\Utilities;
use common\libraries\NotAllowedException;
use common\libraries\Translation;
use common\libraries\NoObjectSelectedException;

class GranterComponent extends Manager
{

    function run()
    {
        if (! \application\weblcms\request\rights\Rights :: get_instance()->request_is_allowed())
        {
            throw new NotAllowedException();
        }
        
        $ids = \common\libraries\Request :: get(self :: PARAM_REQUEST_ID);
        $failures = 0;
        
        if (! empty($ids))
        {
            if (! is_array($ids))
            {
                $ids = array($ids);
            }
            
            foreach ($ids as $id)
            {
                $request = DataManager :: retrieve(Request :: class_name(), (int) $id);
                
                if (! \application\weblcms\request\rights\Rights :: get_instance()->is_target_user(
                    $this->get_user(), 
                    $request->get_user_id()) && ! $this->get_user()->is_platform_admin())
                {
                    $failures ++;
                }
                else
                {
                    
                    if ($request->was_granted())
                    {
                        $failures ++;
                    }
                    else
                    {
                        $course = new Course();
                        
                        $course->set_title($request->get_name());
                        $course->set_course_type_id($request->get_course_type_id());
                        $course->set_visual_code(strtoupper(uniqid()));
                        $course->set_titular_id($request->get_user_id());
                        
                        if ($course->create())
                        {
                            $course_settings = array();
                            $course_settings[CourseSettingsController :: SETTING_PARAM_COURSE_SETTINGS][CourseSettingsConnector :: TITULAR] = $course->get_titular_id();
                            
                            if ($course->create_course_settings_from_values($course_settings, true))
                            {
                                if (CourseManagementRights :: get_instance()->create_rights_from_values(
                                    $course, 
                                    array()))
                                {
                                    if (\application\weblcms\course\DataManager :: subscribe_user_to_course(
                                        $course->get_id(), 
                                        '1',
                                        $request->get_user_id()))
                                    {
                                        $request->set_decision(Request :: DECISION_GRANTED);
                                        $request->set_decision_date(time());
                                        
                                        if (! $request->update())
                                        {
                                            $failures ++;
                                        }
                                        else
                                        {
                                            $this->send_mail($request);
                                        }
                                    }
                                    else
                                    {
                                        $failures ++;
                                    }
                                }
                                else
                                {
                                    $failures ++;
                                }
                            }
                            else
                            {
                                $failures ++;
                            }
                        }
                        else
                        {
                            $failures ++;
                        }
                    }
                }
            }
            
            if ($failures)
            {
                if (count($ids) == 1)
                {
                    $message = 'ObjectNotGranted';
                    $parameter = array('OBJECT' => Translation :: get('Request'));
                }
                elseif (count($ids) > $failures)
                {
                    $message = 'SomeObjectsNotGranted';
                    $parameter = array('OBJECTS' => Translation :: get('Requests'));
                }
                else
                {
                    $message = 'ObjectsNotGranted';
                    $parameter = array('OBJECTS' => Translation :: get('Requests'));
                }
            }
            else
            {
                if (count($ids) == 1)
                {
                    $message = 'ObjectGranted';
                    $parameter = array('OBJECT' => Translation :: get('Request'));
                }
                else
                {
                    $message = 'ObjectsGranted';
                    $parameter = array('OBJECTS' => Translation :: get('Requests'));
                }
            }
            
            $this->redirect(
                Translation :: get($message, $parameter, Utilities :: COMMON_LIBRARIES), 
                ($failures ? true : false), 
                array(self :: PARAM_ACTION => self :: ACTION_BROWSE));
        }
        else
        {
            $this->display_error_page(
                htmlentities(
                    Translation :: get(
                        'NoObjectSelected', 
                        array('OBJECT' => Translation :: get('Request')), 
                        Utilities :: COMMON_LIBRARIES)));
        }
    }

    function send_mail(Request $request)
    {
        set_time_limit(3600);
        
        $recipient = $request->get_user();
        
        $title = Translation :: get(
            'RequestGrantedMailTitle', 
            array('PLATFORM' => PlatformSetting :: get('site_name'), 'NAME' => $request->get_name()));
        
        $body = Translation :: get(
            'RequestGrantedMailBody', 
            array(
                'USER' => $recipient->get_fullname(), 
                'PLATFORM' => PlatformSetting :: get('site_name'), 
                'NAME' => $request->get_name()));
        
        $mail = Mail :: factory(
            $title, 
            $body, 
            array($recipient->get_email()), 
            array(
                Mail :: NAME => PlatformSetting :: get('administrator_firstname') . '_' .
                     PlatformSetting :: get('administrator_surname'), 
                    Mail :: EMAIL => PlatformSetting :: get('administrator_email')));
        
        $mail->send();
    }
}
?>