<?php
namespace application\weblcms\request;

use common\libraries\Theme;
use common\libraries\Utilities;
use common\libraries\ToolbarItem;
use common\libraries\Translation;
use common\libraries\Filesystem;
use common\libraries\Toolbar;
use common\libraries\DatetimeUtilities;
use common\libraries\NewObjectTableCellRendererActionsColumnSupport;
use common\libraries\NewObjectTableCellRenderer;

class RequestTableCellRenderer extends NewObjectTableCellRenderer implements 
    NewObjectTableCellRendererActionsColumnSupport
{

    public function render_cell($column, $object)
    {
        switch ($column->get_name())
        {
            case Translation :: get('User') :
                return $object->get_user()->get_fullname();
            case Request :: PROPERTY_CREATION_DATE :
                return DatetimeUtilities :: format_locale_date(null, $object->get_creation_date());
            case Request :: PROPERTY_DECISION_DATE :
                return DatetimeUtilities :: format_locale_date(null, $object->get_decision_date());
            case Request :: PROPERTY_DECISION :
                return $object->get_decision_icon();
        }
        return parent :: render_cell($column, $object);
    }

    function get_object_actions($object)
    {
        $toolbar = new Toolbar();
        
        if (\application\weblcms\request\rights\Rights :: get_instance()->request_is_allowed())
        {
            if (! $object->was_granted())
            {
                $toolbar->add_item(
                    new ToolbarItem(
                        Translation :: get('Grant'), 
                        Theme :: get_image_path() . 'action/grant.png', 
                        $this->get_component()->get_url(
                            array(
                                Manager :: PARAM_ACTION => Manager :: ACTION_GRANT, 
                                Manager :: PARAM_REQUEST_ID => $object->get_id())), 
                        ToolbarItem :: DISPLAY_ICON));
            }
            
            if ($object->is_pending())
            {
                $toolbar->add_item(
                    new ToolbarItem(
                        Translation :: get('Deny'), 
                        Theme :: get_image_path() . 'action/deny.png', 
                        $this->get_component()->get_url(
                            array(
                                Manager :: PARAM_ACTION => Manager :: ACTION_DENY, 
                                Manager :: PARAM_REQUEST_ID => $object->get_id())), 
                        ToolbarItem :: DISPLAY_ICON));
            }
        }
        
        if ($this->get_component()->get_user()->is_platform_admin() ||
             ($this->get_component()->get_user_id() == $object->get_user_id() && $object->is_pending()))
        {
            $toolbar->add_item(
                new ToolbarItem(
                    Translation :: get('Delete', null, Utilities :: COMMON_LIBRARIES), 
                    Theme :: get_common_image_path() . 'action_delete.png', 
                    $this->get_component()->get_url(
                        array(
                            Manager :: PARAM_ACTION => Manager :: ACTION_DELETE, 
                            Manager :: PARAM_REQUEST_ID => $object->get_id())), 
                    ToolbarItem :: DISPLAY_ICON));
        }
        
        return $toolbar->as_html();
    }
}
?>