<?php
namespace application\weblcms\request\rights;

use rights\DataManager as RightsDataManager;
use group\DataManager as GroupDataManager;
use common\libraries\ToolbarItem;
use common\libraries\Translation;
use common\libraries\Theme;
use common\libraries\Utilities;
use common\libraries\DataClass;

/**
 *
 * @author Hans De Bisschop
 */
class RightsLocationEntityRightGroup extends DataClass
{
    const CLASS_NAME = __CLASS__;

    /**
     * Request properties
     */
    const PROPERTY_LOCATION_ENTITY_RIGHT_ID = 'location_entity_right_id';
    const PROPERTY_GROUP_ID = 'group_id';

    /**
     * The group of the RightsLocationEntityRightGroup
     *
     * @var \group\Group
     */
    private $group;

    /**
     *
     * @var RightsLocationEntityRightGroup
     */
    private $location_entity_right;

    /**
     * Get the default properties
     *
     * @param $extended_property_names multitype:string
     * @return multitype:string The property names.
     */
    static function get_default_property_names($extended_property_names = array())
    {
        $extended_property_names[] = self :: PROPERTY_LOCATION_ENTITY_RIGHT_ID;
        $extended_property_names[] = self :: PROPERTY_GROUP_ID;

        return parent :: get_default_property_names($extended_property_names);
    }

    /**
     * Get the data class data manager
     *
     * @return DataManagerInterface
     */
    function get_data_manager()
    {
        return DataManager :: get_instance();
    }

    function get_location_entity_right_id()
    {
        return $this->get_default_property(self :: PROPERTY_LOCATION_ENTITY_RIGHT_ID);
    }

    function get_location_entity_right()
    {
        if (! isset($this->location_entity_right))
        {
            $this->location_entity_right = RightsDataManager :: retrieve_rights_location_entity_right_by_id(
                __NAMESPACE__,
                $this->get_location_entity_right_id());
        }
        return $this->location_entity_right;
    }

    function set_location_entity_right_id($location_entity_right_id)
    {
        $this->set_default_property(self :: PROPERTY_LOCATION_ENTITY_RIGHT_ID, $location_entity_right_id);
    }

    function get_group_id()
    {
        return $this->get_default_property(self :: PROPERTY_GROUP_ID);
    }

    function get_group()
    {
        if (! isset($this->group))
        {
            $this->group = \group\DataManager :: retrieve_by_id(\group\Group :: class_name(), $this->get_group_id());
        }
        return $this->group;
    }

    function set_group_id($group_id)
    {
        $this->set_default_property(self :: PROPERTY_GROUP_ID, $group_id);
    }

    /**
     *
     * @return string The table name of the data class
     */
    static function get_table_name()
    {
        return Utilities :: get_classname_from_namespace(self :: CLASS_NAME, true);
    }
}
