<?php
namespace application\weblcms\tool\announcement;

use common\libraries\DelegateComponent;

use application\weblcms\ContentObjectPublication;
use application\weblcms\Tool;
use repository\content_object\calendar_event\CalendarEvent;
use repository\ContentObject;
use application\weblcms\ContentObjectPublicationListRenderer;
use common\libraries\ToolbarItem;
use common\libraries\Theme;
use common\libraries\InequalityCondition;
use common\libraries\Request;
use application\weblcms\ToolComponent;
use common\libraries\Translation;
use common\libraries\Utilities;
use common\libraries\ObjectTableOrder;

class AnnouncementToolBrowserComponent extends AnnouncementTool implements DelegateComponent
{
    const PARAM_FILTER = 'filter';
    const FILTER_TODAY = 'today';
    const FILTER_THIS_WEEK = 'week';
    const FILTER_THIS_MONTH = 'month';

    public function run()
    {
        ToolComponent :: launch($this);
    }

    public function get_tool_actions()
    {
        if ($this->get_browser_type() != ContentObjectPublicationListRenderer :: TYPE_CALENDAR)
        {
            $tool_actions = array();
            $tool_actions[] = new ToolbarItem(
                Translation :: get('ShowToday', null, Utilities :: COMMON_LIBRARIES),
                Theme :: get_common_image_path() . 'action_browser.png', $this->get_url(
                    array(
                        Tool :: PARAM_ACTION => null, self :: PARAM_FILTER => self :: FILTER_TODAY
                    )
                ), ToolbarItem :: DISPLAY_ICON_AND_LABEL
            );
            $tool_actions[] = new ToolbarItem(
                Translation :: get('ShowThisWeek', null, Utilities :: COMMON_LIBRARIES),
                Theme :: get_common_image_path() . 'action_browser.png', $this->get_url(
                    array(
                        Tool :: PARAM_ACTION => null, self :: PARAM_FILTER => self :: FILTER_THIS_WEEK
                    )
                ), ToolbarItem :: DISPLAY_ICON_AND_LABEL
            );
            $tool_actions[] = new ToolbarItem(
                Translation :: get('ShowThisMonth', null, Utilities :: COMMON_LIBRARIES),
                Theme :: get_common_image_path() . 'action_browser.png', $this->get_url(
                    array(
                        Tool :: PARAM_ACTION => null, self :: PARAM_FILTER => self :: FILTER_THIS_MONTH
                    )
                ), ToolbarItem :: DISPLAY_ICON_AND_LABEL
            );
            return $tool_actions;
        }
    }

    public function get_tool_conditions()
    {
        $conditions = array();
        $filter = Request :: get(self :: PARAM_FILTER);

        switch ($filter)
        {
            case self :: FILTER_TODAY :
                $time = mktime(0, 0, 0, date('m', time()), date('d', time()), date('Y', time()));
                $conditions[] = new InequalityCondition(
                    ContentObjectPublication :: PROPERTY_MODIFIED_DATE,
                    InequalityCondition :: GREATER_THAN_OR_EQUAL, $time
                );
                break;
            case self :: FILTER_THIS_WEEK :
                $time = strtotime('Next Monday', strtotime('-1 Week', time()));
                $conditions[] = new InequalityCondition(
                    ContentObjectPublication :: PROPERTY_MODIFIED_DATE,
                    InequalityCondition :: GREATER_THAN_OR_EQUAL, $time
                );
                break;
            case self :: FILTER_THIS_MONTH :
                $time = mktime(0, 0, 0, date('m', time()), 1, date('Y', time()));
                $conditions[] = new InequalityCondition(
                    ContentObjectPublication :: PROPERTY_MODIFIED_DATE,
                    InequalityCondition :: GREATER_THAN_OR_EQUAL, $time
                );
                break;
        }

        return $conditions;
    }

    public function convert_content_object_publication_to_calendar_event($publication, $from_time, $to_time)
    {
        $calendar_event = ContentObject :: factory(CalendarEvent :: class_name());

        $calendar_event->set_title($publication[ContentObject :: PROPERTY_TITLE]);
        $calendar_event->set_description($publication[ContentObject :: PROPERTY_DESCRIPTION]);
        $calendar_event->set_start_date($publication[ContentObjectPublication :: PROPERTY_MODIFIED_DATE]);
        $calendar_event->set_end_date($publication[ContentObjectPublication :: PROPERTY_MODIFIED_DATE]);
        $calendar_event->set_frequency(CalendarEvent :: FREQ_NONE);

        return $calendar_event;
    }

    /**
     * "Overrides" the default value of the generic browser component.
     *
     * @return ObjectTableOrder
     */
    public function get_default_order_property()
    {
        return new ObjectTableOrder(ContentObjectPublication :: PROPERTY_MODIFIED_DATE, SORT_DESC);
    }

    public function get_additional_parameters()
    {
        return array(self :: PARAM_BROWSE_PUBLICATION_TYPE);
    }

}
