<?php
namespace application\weblcms\tool\assessment;


use application\weblcms\Tool;
use common\libraries\Utilities;

use common\libraries\Toolbar;
use common\libraries\ToolbarItem;
use common\libraries\Theme;
use common\libraries\Translation;

use repository\DefaultContentObjectTableCellRenderer;
use repository\content_object\assessment\Assessment;
use repository\ContentObject;

/**
 * $Id: assessment_results_table_overview_cell_renderer.class.php 216 2009-11-13
 * 14:08:06Z kariboe $
 *
 * @package
 *          application.lib.weblcms.tool.assessment.component.assessment_results_table_student
 */

/**
 * This class is a cell renderer for a publication candidate table
 */
class AssessmentResultsTableOverviewStudentCellRenderer extends DefaultContentObjectTableCellRenderer
{
    private $table_actions;
    private $browser;

    /**
     * Constructor.
     *
     * @param $publish_url_format string
     *            URL for publishing the selected
     *            learning object.
     * @param $edit_and_publish_url_format string
     *            URL for editing and publishing
     *            the selected learning object.
     */
    public function __construct($browser)
    {
        $this->table_actions = array();
        $this->browser = $browser;
    }

    /*
    * Inherited
    */
    public function render_cell($column, $publication)
    {
        $assessment = $publication->get_content_object();
        if ($column === AssessmentResultsTableOverviewStudentColumnModel :: get_action_column())
        {
            return $this->get_actions($publication);
        }
        else
        {
            switch ($column->get_name())
            {
                case Translation :: get(Assessment :: PROPERTY_TITLE) :
                    return $assessment->get_title();
                case Translation :: get(Assessment :: PROPERTY_TYPE) :
                    return Translation :: get(
                        'TypeName', array(), Utilities :: get_namespace_from_classname($assessment->get_type())
                    );
                case Translation :: get(Assessment :: PROPERTY_AVERAGE_SCORE) :
                    $track = new \application\weblcms\integration\tracking\AssessmentAttempt();
                    $avg = $track->get_average_score($publication, $this->browser->get_user_id());
                    if (!isset($avg))
                    {
                        return 'No results';
                    }
                    else
                    {
                        return $avg . '%';
                    }
                case Translation :: get(Assessment :: PROPERTY_TIMES_TAKEN) :
                    $track = new \application\weblcms\integration\tracking\AssessmentAttempt();
                    return $track->get_times_taken($publication, $this->browser->get_user_id());
                default :
                    return '';
            }
        }
    }

    public function get_actions($publication)
    {
        $toolbar = new Toolbar();

        $toolbar->add_item(
            new ToolbarItem(
                Translation :: get('ViewResults'),
                Theme :: get_common_image_path() . 'action_view_results.png', $this->browser->get_url(
                    array(
                        Tool :: PARAM_ACTION => AssessmentTool :: ACTION_VIEW_RESULTS,
                        AssessmentTool :: PARAM_ASSESSMENT => $publication->get_id()
                    )
                ), ToolbarItem :: DISPLAY_ICON
            )
        );

        return $toolbar->as_html();
    }

}
