<?php
namespace application\weblcms\tool\assessment;

use repository\content_object\assessment\Assessment;
use common\libraries\DataClass;
use common\libraries\StaticConditionVariable;
use common\libraries\PropertyConditionVariable;
use repository\ContentObject;


use application\weblcms\ContentObjectPublication;

use application\weblcms\WeblcmsRights;
use application\weblcms\WeblcmsManager;
use application\weblcms\DataManager as WeblcmsDataManager;

use common\libraries\SubselectCondition;
use common\libraries\OrCondition;
use common\libraries\Utilities;
use common\libraries\AndCondition;
use common\libraries\EqualityCondition;
use common\libraries\ObjectTableDataProvider;

/**
 * $Id: assessment_results_table_overview_data_provider.class.php 216 2009-11-13
 * 14:08:06Z kariboe $
 *
 * @package
 *          application.lib.weblcms.tool.assessment.component.assessment_results_table_student
 */
/**
 * This class represents a data provider for a results candidate table
 */
class AssessmentResultsTableOverviewStudentDataProvider extends ObjectTableDataProvider
{
    /**
     * The user id of the current active user.
     */
    private $owner;
    /**
     * The possible types of learning objects which can be selected.
     */
    private $types;
    /**
     * The search query, or null if none.
     */
    private $query;

    private $parent;

    private $pid;

    /**
     * Constructor.
     *
     * @param $owner int
     *            The user id of the current active user.
     * @param $types array
     *            The possible types of learning objects which can be
     *            selected.
     * @param $query string
     *            The search query.
     */
    public function __construct($parent, $owner, $pid = null, $types = array(), $query = null)
    {
        $this->types = $types;
        $this->owner = $owner;
        $this->query = $query;
        $this->parent = $parent;
        $this->pid = $pid;
    }

    /*
    * Inherited
    */
    public function get_objects($offset, $count, $order_property = null)
    {
        $order_property = $this->get_order_property($order_property);

        if ($this->pid == null)
        {
            $publications = $this->get_publications();
        }
        else
        {
            $publications = $this->get_publication($this->pid);
        }
        // return $this->get_user_assessments($publications);

        return $publications;
    }

    public function get_publication($pid)
    {
        $publication = \application\weblcms\DataManager :: retrieve(
            ContentObjectPublication :: class_name(),
            $pid
        );

        if (
            !$publication->is_visible_for_target_users() && !(
                $this->parent->is_allowed(WeblcmsRights :: DELETE_RIGHT) ||
                $this->parent->is_allowed(WeblcmsRights :: EDIT_RIGHT)
            )
        )
        {
            return array();
        }
        return array($publication);
    }

    public function get_publications()
    {
        $course = $this->parent->get_course_id();

        $conditions = array();
        $conditions[] = new EqualityCondition(ContentObjectPublication :: PROPERTY_COURSE_ID, $course);
        $conditions[] = new EqualityCondition(ContentObjectPublication :: PROPERTY_TOOL, 'assessment');

        $subselect_condition = $this->get_condition();

        $conditions[] = new SubselectCondition(
            new PropertyConditionVariable(
                ContentObjectPublication :: class_name(), ContentObjectPublication :: PROPERTY_CONTENT_OBJECT_ID
            ),
            new PropertyConditionVariable(ContentObject :: class_name(), ContentObject :: PROPERTY_ID),
            ContentObject :: get_table_name(), $subselect_condition
        );

        $parent = $this->parent;
        $category = $parent->get_parameter(WeblcmsManager :: PARAM_CATEGORY);
        $category = $category ? $category : 0;
        $conditions[] = new EqualityCondition(
            ContentObjectPublication :: PROPERTY_CATEGORY_ID, $category, ContentObjectPublication :: get_table_name()
        );

        $condition = new AndCondition($conditions);

        $publications = WeblcmsDataManager ::
            retrieve_content_object_publications_with_view_right_granted_in_category_location(
                $this->parent->get_location(), $this->parent->get_entities(), $condition, array(), 0 , -1,
                $this->parent->get_user_id()
            );

        $visible_publications = array();

        if ($publications)
        {
            while ($publication = $publications->next_result())
            {
                // If the results is hidden and the user is not allowed to DELETE or
                // EDIT, don't show this results
                if (!$this->is_visible_for_target_users($publication))
                {
                    continue;
                }
                $visible_publications[] = DataClass :: factory(ContentObjectPublication :: class_name(), $publication);
            }
        }
        return $visible_publications;
    }

    /**
     * Checks if the publication is visible for target users
     *
     * @param mixed $publication
     *
     * @return bool
     */
    protected function is_visible_for_target_users($publication)
    {
        return (
            !$publication[ContentObjectPublication :: PROPERTY_HIDDEN] &&
            (
                $publication[ContentObjectPublication :: PROPERTY_FROM_DATE] == 0 &&
                $publication[ContentObjectPublication :: PROPERTY_TO_DATE] == 0
            ) ||
            (
                $publication[ContentObjectPublication :: PROPERTY_FROM_DATE] <= time() &&
                time() <= $publication[ContentObjectPublication :: PROPERTY_TO_DATE]
            )
        );
    }

    /*
    * Inherited
    */
    public function get_object_count()
    {
        return count($this->get_publications());
    }

    /**
     * Gets the condition by which the learning objects should be selected.
     *
     * @return Condition The condition.
     */
    public function get_condition()
    {
        $conds = array();

        $type_cond = array();
        $types = array(Assessment :: class_name());
        foreach ($types as $type)
        {
            $type_cond[] = new EqualityCondition(ContentObject :: PROPERTY_TYPE, $type);
        }
        $conds[] = new OrCondition($type_cond);
        $qcond = Utilities :: query_to_condition($this->query);
        if (!is_null($qcond))
        {
            $conds[] = $qcond;
        }
        return new AndCondition($conds);
    }
}
