<?php
namespace application\weblcms\tool\assessment;

use application\weblcms\integration\tracking\AssessmentAttempt;
use application\weblcms\integration\tracking\QuestionAttempt;
use application\weblcms\integration\tracking\DataManager as WeblcmsTrackingDataManager;
use common\libraries\StaticConditionVariable;
use common\libraries\PropertyConditionVariable;
use common\libraries\DataClassRetrieveParameters;
use application\weblcms\WeblcmsManager;
use application\weblcms\ContentObjectPublication;
use common\libraries\AndCondition;
use common\libraries\EqualityCondition;
use common\libraries\Request;
use common\libraries\Path;
use common\libraries\Translation;
use common\libraries\DelegateComponent;
use tracking\Event;
use repository\ComplexDisplay;
use repository\content_object\hotpotatoes\Hotpotatoes;
use repository\content_object\assessment\AssessmentComplexDisplaySupport;
use application\weblcms\Tool;
use application\weblcms\WeblcmsRights;
use common\libraries\Utilities;

/**
 * $Id: assessment_tester.class.php 216 2009-11-13 14:08:06Z kariboe $
 *
 * @package application.lib.weblcms.tool.assessment.component
 *
 * @author Previous Author Unknown
 * @author Sven Vanpoucke - Hogeschool Gent - Cleanup
 */
class AssessmentToolComplexDisplayComponent extends AssessmentTool implements AssessmentComplexDisplaySupport,
    DelegateComponent
{
    private $pub;
    private $assessment;
    private $publication_id;

    /**
     * The assessmnet attempt
     *
     * @var AssessmentAttempt
     */
    private $assessment_attempt;

    /**
     * The question attempt
     *
     * @var QuestionAttempt[]
     */
    private $question_attempts;

    public function run()
    {
        // Retrieving assessment
        if (Request :: get(Tool :: PARAM_PUBLICATION_ID))
        {
            $this->publication_id = Request :: get(Tool :: PARAM_PUBLICATION_ID);

            $this->pub = \application\weblcms\DataManager :: retrieve(
                ContentObjectPublication :: class_name(),
                $this->publication_id);

            if (!$this->pub || ! $this->is_allowed(WeblcmsRights :: VIEW_RIGHT, $this->pub))
            {
                $this->redirect(
                    Translation :: get("NotAllowed", null, Utilities :: COMMON_LIBRARIES),
                    true,
                    array(),
                    array(Tool :: PARAM_ACTION, Tool :: PARAM_PUBLICATION_ID));
            }

            $this->assessment = $this->pub->get_content_object();
            $this->set_parameter(Tool :: PARAM_PUBLICATION_ID, $this->publication_id);
        }

        // Checking statistics

        $track = new AssessmentAttempt();
        $conditions[] = new EqualityCondition(
            AssessmentAttempt :: PROPERTY_ASSESSMENT_ID,
            $this->publication_id);
        $conditions[] = new EqualityCondition(
            AssessmentAttempt :: PROPERTY_USER_ID,
            $this->get_user_id());
        $condition = new AndCondition($conditions);
        $trackers = $track->retrieve_tracker_items($condition);

        $count = count($trackers);

        foreach ($trackers as $tracker)
        {
            if ($tracker->get_status() == AssessmentAttempt :: STATUS_NOT_COMPLETED)
            {
                $this->assessment_attempt = $tracker;
                $count --;
                break;
            }
        }

        if ($this->assessment->get_maximum_attempts() != 0 && $count >= $this->assessment->get_maximum_attempts())
        {
            $this->display_header();
            $this->display_error_message(Translation :: get('YouHaveReachedYourMaximumAttempts'));
            $this->display_footer();
            return;
        }

        if (! $this->assessment_attempt)
        {
            $this->assessment_attempt = $this->create_assessment_attempt();
        }

        // Executing assessment

        if ($this->assessment->get_type() == Hotpotatoes :: class_name())
        {
            $this->display_header(null);

            $path = $this->assessment->add_javascript(
                Path :: get(WEB_PATH) . 'application/weblcms/php/ajax/hotpotatoes_save_score.php',
                $this->get_assessment_back_url(),
                $this->assessment_attempt->get_id());
            // $path = $this->assessment->get_test_path();
            echo '<iframe src="' . $path . '" width="100%" height="600">
  				 <p>Your browser does not support iframes.</p>
				 </iframe>';

            $this->display_footer();
        }
        else
        {
            if ($this->assessment->count_questions() == 0)
            {
                $this->redirect(
                    Translation :: get("EmptyAssessment"),
                    true,
                    array(),
                    array(Tool :: PARAM_ACTION, Tool :: PARAM_PUBLICATION_ID));
            }

            ComplexDisplay :: launch($this->assessment->get_type(), $this);
        }
    }

    /**
     * Returns the root content object for the complex display
     *
     * @return Assessment
     */
    public function get_root_content_object()
    {
        return $this->assessment;
    }

    /**
     * Creates a new assessment attempt
     *
     * @return AssessmentAttempt
     */
    public function create_assessment_attempt()
    {
        $arguments = array(
            AssessmentAttempt :: PROPERTY_ASSESSMENT_ID => $this->publication_id,
            AssessmentAttempt :: PROPERTY_USER_ID => $this->get_user_id(),
            AssessmentAttempt :: PROPERTY_COURSE_ID => $this->get_course_id(),
            AssessmentAttempt :: PROPERTY_TOTAL_SCORE => 0
        );

        $tracker = Event :: trigger('attempt_assessment', WeblcmsManager :: context(), $arguments);
        return $tracker[0];
    }

    /**
     * Returns the id of the current assessment attempt
     *
     * @return int
     */
    public function get_assessment_current_attempt_id()
    {
        return $this->assessment_attempt->get_id();
    }

    /**
     * Saves the assessment answer of a question to the database
     *
     * @param int $complex_question_id
     * @param string $answer
     * @param int $score
     */
    public function save_assessment_answer($complex_question_id, $answer, $score, $hint)
    {
        $question_attempt = $this->get_assessment_question_attempt($complex_question_id);

        $question_attempt->set_answer($answer);
        $question_attempt->set_score($score);
        $question_attempt->set_hint($hint);

        $question_attempt->update();
    }

    /**
     * Saves the result of the assessment to the database
     *
     * @param int $total_score
     */
    public function save_assessment_result($total_score)
    {
        $assessment_attempt = $this->assessment_attempt;

        $assessment_attempt->set_total_score($total_score);
        $assessment_attempt->set_end_time(time());
        $assessment_attempt->set_status(AssessmentAttempt :: STATUS_COMPLETED);

        $assessment_attempt->set_total_time(
            $assessment_attempt->get_total_time() +
            ($assessment_attempt->get_end_time() - $assessment_attempt->get_start_time())
        );

        $assessment_attempt->update();
    }

    /**
     * Returns the assessment question attempts
     *
     * @return QuestionAttempt[]
     */
    public function get_assessment_question_attempts()
    {
        if(is_null($this->question_attempts))
        {
            $this->question_attempts = $this->retrieve_question_attempts();
        }

        return $this->question_attempts;
    }

    /**
     * Retrieves the question attempts for the selected assessment attempt
     *
     * @return QuestionAttempt[]
     */
    protected function retrieve_question_attempts()
    {
        $question_attempts = array();

        $condition = new EqualityCondition(
            new PropertyConditionVariable(
                QuestionAttempt :: class_name(), QuestionAttempt :: PROPERTY_ASSESSMENT_ATTEMPT_ID
            ),
            new StaticConditionVariable($this->assessment_attempt->get_id())
        );

        $question_attempts_result_set = WeblcmsTrackingDataManager :: retrieves(
            QuestionAttempt :: class_name(), $condition
        );

        while($question_attempt = $question_attempts_result_set->next_result())
        {
            $question_attempts[$question_attempt->get_question_cid()] = $question_attempt;
        }

        return $question_attempts;
    }

    /**
     * Registers the question ids
     *
     * @param int[] $question_ids
     */
    public function register_question_ids($question_ids)
    {
        foreach($question_ids as $complex_question_id)
        {
            $parameters = array();

            $parameters[QuestionAttempt :: PROPERTY_ASSESSMENT_ATTEMPT_ID] = $this->assessment_attempt->get_id();
            $parameters[QuestionAttempt :: PROPERTY_QUESTION_CID] = $complex_question_id;
            $parameters[QuestionAttempt :: PROPERTY_ANSWER] = '';
            $parameters[QuestionAttempt :: PROPERTY_SCORE] = 0;
            $parameters[QuestionAttempt :: PROPERTY_FEEDBACK] = '';
            $parameters[QuestionAttempt :: PROPERTY_HINT] = 0;

            $trackers = Event :: trigger('attempt_question', WeblcmsManager :: context(), $parameters);

            $this->question_attempts[$complex_question_id] = $trackers[0];
        }
    }

    /**
     * Returns the registered question ids
     *
     * @return int[] $question_ids
     */
    public function get_registered_question_ids()
    {
        $question_ids = array();

        $question_attempts = $this->get_assessment_question_attempts();
        foreach($question_attempts as $question_attempt)
        {
            $question_ids[] = $question_attempt->get_question_cid();
        }

        return $question_ids;
    }

    /**
     * Gets a single question attempt by a given question id
     *
     * @param int $complex_question_id
     *
     * @return QuestionAttempt
     */
    public function get_assessment_question_attempt($complex_question_id)
    {
        return $this->question_attempts[$complex_question_id];
    }

    public function get_assessment_continue_url()
    {
        return null;
    }

    public function get_assessment_back_url()
    {
        return $this->get_url(
            array(Tool :: PARAM_ACTION => AssessmentTool :: ACTION_BROWSE),
            array(Tool :: PARAM_PUBLICATION_ID));
    }

    public function get_assessment_feedback_configuration($complex_content_object_item)
    {
        $parameters = new DataClassRetrieveParameters(
            new EqualityCondition(
                new PropertyConditionVariable(Publication :: class_name(), Publication :: PROPERTY_PUBLICATION_ID),
                new StaticConditionVariable($this->pub->get_id())));
        $assessment_publication = DataManager :: retrieve(Publication :: class_name(), $parameters);

        return $assessment_publication->get_feedback_configuration();
    }

    public function get_assessment_parameters()
    {
        return array();
    }

    public function get_additional_parameters()
    {
        return array(Tool :: PARAM_PUBLICATION_ID);
    }

    public function get_assessment_current_url()
    {
        return $this->get_url();
    }

    // METHODS FOR COMPLEX DISPLAY RIGHTS
    public function is_allowed_to_edit_content_object()
    {
        return $this->is_allowed(WeblcmsRights :: EDIT_RIGHT, $this->publication);
    }

    public function is_allowed_to_view_content_object()
    {
        return $this->is_allowed(WeblcmsRights :: VIEW_RIGHT, $this->publication);
    }

    public function is_allowed_to_add_child()
    {
        return $this->is_allowed(WeblcmsRights :: EDIT_RIGHT, $this->publication);
    }

    public function is_allowed_to_delete_child()
    {
        return $this->is_allowed(WeblcmsRights :: EDIT_RIGHT, $this->publication);
    }

    public function is_allowed_to_delete_feedback()
    {
        return $this->is_allowed(WeblcmsRights :: EDIT_RIGHT, $this->publication);
    }

    public function is_allowed_to_edit_feedback()
    {
        return $this->is_allowed(WeblcmsRights :: EDIT_RIGHT, $this->publication);
    }

}
