<?php
namespace application\weblcms\tool\assessment;

use repository\ExportParameters;
use application\weblcms\ContentObjectPublication;

use application\weblcms\Tool;


use application\weblcms\WeblcmsManager;

use application\weblcms\WeblcmsRights;
use common\libraries\AndCondition;
use common\libraries\EqualityCondition;
use common\libraries\InCondition;
use common\libraries\Request;
use common\libraries\Translation;
use common\libraries\Utilities;
use repository\content_object\assessment_open_question\AssessmentOpenQuestion;
use repository\ComplexContentObjectItem;
use repository\ContentObjectExport;
use repository\ContentObjectExportController;


/**
 * $Id: assessment_document_saver.class.php 216 2009-11-13 14:08:06Z kariboe $
 *
 * @package application.lib.weblcms.tool.assessment.component
 */

class AssessmentToolDocumentSaverComponent extends AssessmentTool
{

    public function run()
    {
        if (!$this->is_allowed(WeblcmsRights :: EDIT_RIGHT))
        {
            $this->redirect(
                Translation :: get('NotAllowed', null, Utilities :: COMMON_LIBRARIES), true, array(),
                array(Tool :: PARAM_ACTION, Tool :: PARAM_PUBLICATION_ID)
            );
        }

        if (Request :: get(AssessmentTool :: PARAM_USER_ASSESSMENT))
        {
            $this->retrieve_assessment_attempt_documents(Request :: get(AssessmentTool :: PARAM_USER_ASSESSMENT));
        }
        elseif (Request :: get(AssessmentTool :: PARAM_ASSESSMENT))
        {
            $this->retrieve_assessment_documents(Request :: get(AssessmentTool :: PARAM_ASSESSMENT));
        }
    }

    /**
     * Retrieves all the documents submitted in all the attempts of the
     * assessment.
     *
     * @param $publication_id int
     *            The id of the assessment publication.
     */
    protected function retrieve_assessment_documents($publication_id)
    {
        $open_document_question_ids = $this->retrieve_open_document_question_ids($publication_id);

        $condition = new EqualityCondition(\application\weblcms\integration\tracking\AssessmentAttempt :: PROPERTY_ASSESSMENT_ID, $publication_id);
        $assessment_attempt_trackers = \application\weblcms\integration\tracking\AssessmentAttempt :: get_data(
            \application\weblcms\integration\tracking\AssessmentAttempt :: CLASS_NAME, WeblcmsManager :: APPLICATION_NAME, $condition
        )->as_array();
        $assessment_attempt_tracker_ids = array();
        foreach ($assessment_attempt_trackers as $assessment_attempt_tracker)
        {
            $assessment_attempt_tracker_ids[] = $assessment_attempt_tracker->get_id();
        }

        $this->retrieve_assessment_attempts_documents($open_document_question_ids, $assessment_attempt_tracker_ids);
    }

    /**
     * Retrieves the documents submitted in a set of assessment attempts.
     *
     * @param $open_document_question_ids array
     *            The ids of the open questions where documents may be added.
     * @param $assessment_attempt_tracker_ids array
     *            The ids of the assessment attempts for which the added
     *            documents are to be downloaded.
     */
    protected function retrieve_assessment_attempts_documents(
        $open_document_question_ids, $assessment_attempt_tracker_ids
    )
    {
        if (!is_array($assessment_attempt_tracker_ids))
        {
            $assessment_attempt_tracker_ids = array($assessment_attempt_tracker_ids);
        }

        if (count($open_document_question_ids) < 1)
        {
            $this->redirect_to_previous('NoOpenDocumentQuestions');
        }

        $conditions = array();
        $conditions[] = new InCondition(
            \application\weblcms\integration\tracking\QuestionAttempt :: PROPERTY_QUESTION_CID, $open_document_question_ids
        );
        $conditions[] = new InCondition(
            \application\weblcms\integration\tracking\QuestionAttempt :: PROPERTY_ASSESSMENT_ATTEMPT_ID, $assessment_attempt_tracker_ids
        );
        $condition = new AndCondition($conditions);
        $question_attempt_trackers = \application\weblcms\integration\tracking\QuestionAttempt :: get_data(
            \application\weblcms\integration\tracking\QuestionAttempt :: CLASS_NAME, WeblcmsManager :: APPLICATION_NAME, $condition
        )->as_array();

        $document_ids = array();
        foreach ($question_attempt_trackers as $question_attempt_tracker)
        {
            $answer = unserialize($question_attempt_tracker->get_answer());
            if (!is_null($answer[2]) && strlen($answer[2]) > 0)
            {
                // Assign key to get ids without duplicates.
                $document_ids[$answer[2]] = $answer[2];
            }
        }

        if (count($document_ids) < 1)
        {
            $this->redirect_to_previous('NoDocumentsForAssessment');
        }
        $parameters = new ExportParameters($this->get_user_id(), ContentObjectExport :: FORMAT_ZIP, $document_ids);
        $exporter = ContentObjectExportController :: factory($parameters);
        $exporter->download();
    }

    /**
     * Downloads the documents for a single assessment attempt.
     *
     * @param $assessment_attempt_tracker_id int
     *            The id of the assessment attempt.
     */
    protected function retrieve_assessment_attempt_documents($assessment_attempt_tracker_id)
    {
        $condition = new EqualityCondition(
            \application\weblcms\integration\tracking\AssessmentAttempt :: PROPERTY_ID, $assessment_attempt_tracker_id
        );
        $assessment_attempts = \application\weblcms\integration\tracking\AssessmentAttempt :: get_data(
            \application\weblcms\integration\tracking\AssessmentAttempt :: CLASS_NAME, WeblcmsManager :: APPLICATION_NAME, $condition
        )->as_array();

        $open_document_question_ids = $this->retrieve_open_document_question_ids(
            $assessment_attempts[0]->get_assessment_id()
        );

        $this->retrieve_assessment_attempts_documents($open_document_question_ids, $assessment_attempt_tracker_id);
    }

    /**
     * Retrieves the ids of the open questions where documents may be added.
     *
     * @param $publication_id int
     *            The id of the assessment publication to be searched.
     * @return array The ids of the open questions where documents may be added.
     */
    protected function retrieve_open_document_question_ids($publication_id)
    {
        $publication = \application\weblcms\DataManager :: retrieve(
            ContentObjectPublication :: class_name(),
            $publication_id
        );

        $condition = new EqualityCondition(
            ComplexContentObjectItem :: PROPERTY_PARENT, $publication->get_content_object_id()
        );
        $complex_questions = \repository\DataManager :: retrieve_complex_content_object_items
        (ComplexContentObjectItem :: class_name(), $condition)
            ->as_array();

        // Array of open question ids in the publication that permit documents
        // to be submitted.
        $open_document_question_ids = array();
        foreach ($complex_questions as $complex_question)
        {
            if (
                $complex_question->get_ref_object()->get_type() == AssessmentOpenQuestion :: class_name() &&
                $this->is_open_question_document_allowed($complex_question->get_ref_object())
            )
            {
                $open_document_question_ids[] = $complex_question->get_id();
            }
        }
        return $open_document_question_ids;
    }

    public function is_open_question_document_allowed($open_question)
    {
        switch ($open_question->get_question_type())
        {
            case AssessmentOpenQuestion :: TYPE_OPEN_WITH_DOCUMENT :
            case AssessmentOpenQuestion :: TYPE_DOCUMENT :
                return true;
        }
        return false;
    }

    private function redirect_to_previous($message)
    {
        $params = array();
        $params[Tool :: PARAM_ACTION] = AssessmentTool :: ACTION_VIEW_RESULTS;
        $params[AssessmentTool :: PARAM_ASSESSMENT] = Request :: get(AssessmentTool :: PARAM_ASSESSMENT);
        $params[AssessmentTool :: PARAM_USER_ASSESSMENT] = Request :: get(AssessmentTool :: PARAM_USER_ASSESSMENT);

        $this->redirect(Translation :: get($message), false, $params);
    }
}
