<?php
namespace application\weblcms\tool\assessment;

use repository\content_object\assessment\FeedbackDisplayConfiguration;
use common\libraries\DataClass;

/**
 *
 * @author Hans De Bisschop
 * @package application.weblcms.tool.assessment
 */
class Publication extends DataClass
{
    const CLASS_NAME = __CLASS__;

    /**
     * Assessment publication properties
     */
    const PROPERTY_PUBLICATION_ID = 'publication_id';
    const PROPERTY_FEEDBACK_SUMMARY = 'feedback_summary';
    const PROPERTY_FEEDBACK_PER_PAGE = 'feedback_per_page';
    const PROPERTY_FEEDBACK_TYPE = 'feedback_type';
    const PROPERTY_HINTS_ALLOWED = 'hints_allowed';

    /**
     * Get the default properties
     *
     * @return array The property names.
     */
    public static function get_default_property_names()
    {
        return parent :: get_default_property_names(
            array(
                self :: PROPERTY_PUBLICATION_ID,
                self :: PROPERTY_FEEDBACK_SUMMARY,
                self :: PROPERTY_FEEDBACK_PER_PAGE,
                self :: PROPERTY_FEEDBACK_TYPE,
                self :: PROPERTY_HINTS_ALLOWED));
    }

    public function get_data_manager()
    {
        return DataManager :: get_instance();
    }

    public function get_publication_id()
    {
        return $this->get_default_property(self :: PROPERTY_PUBLICATION_ID);
    }

    public function set_publication_id($publication_id)
    {
        $this->set_default_property(self :: PROPERTY_PUBLICATION_ID, $publication_id);
    }

    /**
     *
     * @return boolean
     */
    public function get_feedback_summary()
    {
        return $this->get_default_property(self :: PROPERTY_FEEDBACK_SUMMARY);
    }

    public function set_feedback_summary($feedback_summary)
    {
        $this->set_default_property(self :: PROPERTY_FEEDBACK_SUMMARY, $feedback_summary);
    }

    /**
     *
     * @return boolean
     */
    public function get_hints_allowed()
    {
        return $this->get_default_property(self :: PROPERTY_HINTS_ALLOWED);
    }

    public function set_hints_allowed($hints_allowed)
    {
        $this->set_default_property(self :: PROPERTY_HINTS_ALLOWED, $hints_allowed);
    }

    /**
     *
     * @return boolean
     */
    public function get_feedback_per_page()
    {
        return $this->get_default_property(self :: PROPERTY_FEEDBACK_PER_PAGE);
    }

    public function set_feedback_per_page($feedback_per_page)
    {
        $this->set_default_property(self :: PROPERTY_FEEDBACK_PER_PAGE, $feedback_per_page);
    }

    /**
     *
     * @return int
     */
    public function get_feedback_type()
    {
        return $this->get_default_property(self :: PROPERTY_FEEDBACK_TYPE);
    }

    public function set_feedback_type($feedback_type)
    {
        $this->set_default_property(self :: PROPERTY_FEEDBACK_TYPE, $feedback_type);
    }

    /**
     *
     * @return \repository\content_object\assessment\FeedbackDisplayConfiguration
     */
    public function get_feedback_configuration()
    {
        return new FeedbackDisplayConfiguration(
            $this->get_hints_allowed(),
            $this->get_feedback_summary(),
            $this->get_feedback_per_page(),
            $this->get_feedback_type());
    }
}
