<?php
namespace application\weblcms\tool\assessment;

use application\weblcms\integration\tracking\AssessmentAttempt;
use common\libraries\DataClassPropertyTableColumn;
use common\libraries\RecordTableColumnModel;
use common\libraries\TableColumnModelActionsColumnSupport;
use user\User;
use common\libraries\DataClassRetrieveParameters;
use common\libraries\EqualityCondition;
use common\libraries\PropertyConditionVariable;
use common\libraries\StaticConditionVariable;
use application\weblcms\WeblcmsRights;

/**
 * This class represents a column model for the attempts of an assessment
 *
 * @author Sven Vanpoucke - Hogeschool Gent
 */
class AssessmentAttemptTableColumnModel extends RecordTableColumnModel implements TableColumnModelActionsColumnSupport
{
    const DEFAULT_ORDER_COLUMN_INDEX = 1;

    /**
     * **************************************************************************************************************
     * Inherited Functionality *
     * **************************************************************************************************************
     */

    /**
     * Initializes the columns for the table
     */
    public function initialize_columns()
    {
        $this->add_column(new DataClassPropertyTableColumn(User :: class_name(), User :: PROPERTY_FIRSTNAME));
        $this->add_column(new DataClassPropertyTableColumn(User :: class_name(), User :: PROPERTY_LASTNAME));
        $this->add_column(new DataClassPropertyTableColumn(User :: class_name(), User :: PROPERTY_OFFICIAL_CODE));

        $this->add_column(
            new DataClassPropertyTableColumn(AssessmentAttempt :: class_name(), AssessmentAttempt :: PROPERTY_START_TIME));

        $this->add_column(
            new DataClassPropertyTableColumn(AssessmentAttempt :: class_name(), AssessmentAttempt :: PROPERTY_END_TIME));

        $this->add_column(
            new DataClassPropertyTableColumn(AssessmentAttempt :: class_name(), AssessmentAttempt :: PROPERTY_TOTAL_TIME));

        $publication = $this->get_component()->get_publication();
        $parameters = new DataClassRetrieveParameters(
            new EqualityCondition(
                new PropertyConditionVariable(Publication :: class_name(), Publication :: PROPERTY_PUBLICATION_ID),
                new StaticConditionVariable($publication->get_id())));
        $assessment_publication = DataManager :: retrieve(Publication :: class_name(), $parameters);

        if ($this->get_component()->is_allowed(WeblcmsRights :: EDIT_RIGHT, $publication) || (($assessment_publication->get_feedback_configuration()->get_feedback_summary() ||
             $assessment_publication->get_feedback_configuration()->get_feedback_per_page()) &&
             $assessment_publication->get_feedback_configuration()->display_numeric_feedback()))
        {
            $this->add_column(
                new DataClassPropertyTableColumn(
                    AssessmentAttempt :: class_name(),
                    AssessmentAttempt :: PROPERTY_TOTAL_SCORE));
        }

        $this->add_column(
            new DataClassPropertyTableColumn(AssessmentAttempt :: class_name(), AssessmentAttempt :: PROPERTY_STATUS));
    }
}