<?php

namespace application\weblcms\tool\assessment;

use application\weblcms\ObjectPublicationTableCellRenderer;
use application\weblcms\Tool;
use repository\content_object\hotpotatoes\Hotpotatoes;
use repository\ContentObject;
use application\weblcms\ContentObjectPublication;
use common\libraries\Translation;
use repository\content_object\assessment\Assessment;

/**
 * Extension on the content object publication table cell renderer for this tool
 *
 * @author Sven Vanpoucke - Hogeschool Gent
 */
class PublicationTableCellRenderer extends ObjectPublicationTableCellRenderer
{
    /****************************************************************************************************************
     * Inherited Functionality                                                                                      *
     ****************************************************************************************************************/

    /**
     * Renders a cell for a given object
     *
     * @param $column \common\libraries\ObjectTableColumn
     *
     * @param mixed $publication
     *
     * @return String
     */
    public function render_cell($column, $publication)
    {
        switch($column->get_name())
        {
            case ContentObject :: PROPERTY_TYPE:
            {
                $type = Translation :: get(
                    'TypeName', $publication[ContentObject :: PROPERTY_TYPE]
                );

                if ($publication[ContentObjectPublication :: PROPERTY_HIDDEN])
                {
                    return '<span style="color: gray">' . $type . '</span>';
                }
                else
                {
                    return $type;
                }

                break;
            }
            case ContentObject :: PROPERTY_TITLE:
            {
                if ($publication[ContentObject :: PROPERTY_TYPE] == Hotpotatoes :: class_name())
                {
                    $details_url = $this->get_component()->get_url(
                        array(
                            Tool :: PARAM_PUBLICATION_ID => $publication[ContentObjectPublication :: PROPERTY_ID],
                            Tool :: PARAM_ACTION => Tool :: ACTION_DISPLAY_COMPLEX_CONTENT_OBJECT
                        )
                    );

                    return '<a href="' . $details_url . '">' . parent :: render_cell(
                        $column, $publication
                    ) . '</a>';
                }
            }
        }

        return parent :: render_cell($column, $publication);
    }

}