<?php
namespace application\weblcms\tool\assignment;

use common\libraries\DelegateComponent;
use application\weblcms\ContentObjectPublication;
use repository\content_object\calendar_event\CalendarEvent;
use repository\ContentObject;
use application\weblcms\ToolComponent;
use repository\content_object\assignment\Assignment;
use application\weblcms\WeblcmsRights;
use common\libraries\ToolbarItem;
use common\libraries\Translation;
use common\libraries\Theme;
use application\weblcms\Tool;
use reporting\ReportingManager;
use reporting\Reporting;
use common\libraries\Utilities;
use application\weblcms\WeblcmsManager;
use application\weblcms\tool\reporting\ReportingTool;

/**
 *
 * @package application.weblcms.tool.assignment.php.component Browser for assignments with calendar functionality.
 * @author Joris Willems <joris.willems@gmail.com>
 * @author Alexander Van Paemel
 */
class AssignmentToolBrowserComponent extends AssignmentTool implements DelegateComponent
{

    public function run()
    {
        ToolComponent :: launch($this);
    }

    public function get_tool_actions()
    {
        $tool_actions = array();

        if ($this->is_allowed(WeblcmsRights :: EDIT_RIGHT))
        {
            $tool_actions[] = new ToolbarItem(
                Translation :: get('ScoresOverview'),
                Theme :: get_common_image_path() . 'action_statistics.png',
                $this->get_url(
                    array(
                        WeblcmsManager :: PARAM_TOOL => WeblcmsManager :: ACTION_REPORTING,
                        ReportingManager :: PARAM_TEMPLATE_ID => \application\weblcms\integration\reporting\AssignmentScoresTemplate :: class_name(),
                        WeblcmsManager :: PARAM_TOOL_ACTION => ReportingTool :: ACTION_VIEW)),
                ToolbarItem :: DISPLAY_ICON_AND_LABEL);
        }

        return $tool_actions;
    }

    public function convert_content_object_publication_to_calendar_event($publication, $from_time, $to_time)
    {
        $object = \repository\DataManager :: retrieve_content_object(
            $publication[ContentObjectPublication :: PROPERTY_CONTENT_OBJECT_ID],
            Assignment :: class_name());

        $calendar_event = ContentObject :: factory(CalendarEvent :: class_name());

        $calendar_event->set_title($object->get_title());
        $calendar_event->set_description($object->get_description());
        if ($object instanceof Assignment)
        {
            $calendar_event->set_start_date($object->get_start_time());
            $calendar_event->set_end_date($object->get_end_time());
        }
        else
        {
            $calendar_event->set_start_date($object->get_start_date());
            $calendar_event->set_end_date($object->get_end_date());
        }

        $calendar_event->set_frequency(CalendarEvent :: FREQ_NONE);

        return $calendar_event;
    }
}
