<?php
namespace application\weblcms\tool\assignment;

use application\weblcms\course\CourseGroupRelation;
use application\weblcms\tool\course_group\CourseGroup;
use application\weblcms\ContentObjectPublication;
use application\weblcms\CourseGroupEntity;
use application\weblcms\CoursePlatformGroupEntity;
use application\weblcms\Tool;
use application\weblcms\WeblcmsManager;
use application\weblcms\WeblcmsRights;
use common\libraries\Request;
use common\libraries\Breadcrumb;
use common\libraries\DatetimeUtilities;
use common\libraries\EqualityCondition;
use common\libraries\Translation;
use common\libraries\Utilities;
use group\DataManager as GroupDataManager;
use group\Group;
use repository\content_object\assignment\Assignment;
use repository\ContentObjectCopier;
use tracking\Event;
use user\UserDataManager;
use application\weblcms\tool\course_group\DataManager as CourseGroupDataManager;

/**
 *
 * @package application.weblcms.tool.assignment.php.component Component for the submission feedback form for assignment
 * @author Joris Willems <joris.willems@gmail.com>
 * @author Alexander Van Paemel
 * @author Bert De Clercq (Hogeschool Gent)
 * @author Anthony Hurst (Hogeschool Gent)
 */
class AssignmentToolSubmissionSubmitComponent extends SubmissionsManager implements \repository\viewer\ViewerInterface
{

    /**
     * The id of the assignment publication
     *
     * @var int
     */
    private $publication_id;

    /**
     * The assignment content object
     *
     * @var Assignment
     */
    private $assignment;

    public function run()
    {
        $choices = $this->compile_choices();
        // display submission form when content object created in RepoViewer
        if ($this->assignment->get_allow_group_submissions() && count($choices) == 0 ||
             ! $this->is_allowed(WeblcmsRights :: VIEW_RIGHT))
        {
            if ($this->is_allowed(WeblcmsRights :: EDIT_RIGHT))
            {
                $this->redirect(
                    Translation :: get('NoOwnGroups'),
                    true,
                    array(
                        Tool :: PARAM_ACTION => AssignmentTool :: ACTION_BROWSE_SUBMITTERS,
                        Tool :: PARAM_PUBLICATION_ID => $this->publication_id));
            }
            $this->redirect(
                Translation :: get("NoOwnGroups", null, Utilities :: COMMON_LIBRARIES),
                true,
                array(
                    Tool :: PARAM_ACTION => AssignmentTool :: ACTION_STUDENT_BROWSE_SUBMISSIONS,
                    Tool :: PARAM_PUBLICATION_ID => $this->publication_id));
        }
        if (\repository\viewer\Manager :: is_ready_to_be_published())
        {
            if ($this->assignment->get_allow_group_submissions() && (! $this->get_target_id() || $this->get_submitter_type() ==
                 \application\weblcms\integration\tracking\AssignmentSubmission :: SUBMITTER_TYPE_USER))
            {
                $form = new SubmissionSubmitForm(
                    $choices,
                    $this->get_url(
                        array(
                            \repository\viewer\Manager :: PARAM_ACTION => \repository\viewer\Manager :: ACTION_PUBLISHER,
                            \repository\viewer\Manager :: PARAM_CONTENT_OBJECT_TYPE => Request :: get(
                                \repository\viewer\Manager :: PARAM_CONTENT_OBJECT_TYPE),
                            \repository\viewer\Manager :: PARAM_ID => $this->get_repo_objects())));

                // create submission feedback tracker when form is valid
                if ($form->validate())
                {
                    $values = $form->exportValues();
                    $submitter_type = substr(
                        $values[\application\weblcms\integration\tracking\AssignmentSubmission :: PROPERTY_SUBMITTER_ID],
                        0,
                        1);
                    $submitter_id = substr(
                        $values[\application\weblcms\integration\tracking\AssignmentSubmission :: PROPERTY_SUBMITTER_ID],
                        1);
                    $submitter = $this->create_tracker($submitter_id, $this->get_repo_objects(), $submitter_type);
                    $this->redirect(
                        Translation :: get('SubmissionCreated'),
                        false,
                        array(
                            Tool :: PARAM_ACTION => AssignmentTool :: ACTION_BROWSE_SUBMISSIONS,
                            Tool :: PARAM_PUBLICATION_ID => $this->publication_id,
                            AssignmentTool :: PARAM_SUBMITTER_TYPE => substr($submitter, 0, 1),
                            AssignmentTool :: PARAM_TARGET_ID => substr($submitter, 1)));
                }
                else // display submission form
                {
                    $this->display_header();
                    echo $form->toHtml();
                    $this->display_footer();
                }
            }
            else
            {
                $submitter = $this->create_tracker($this->get_target_id(), $this->get_repo_objects());
                $this->redirect(
                    Translation :: get('SubmissionCreated'),
                    false,
                    array(
                        Tool :: PARAM_ACTION => AssignmentTool :: ACTION_BROWSE_SUBMISSIONS,
                        Tool :: PARAM_PUBLICATION_ID => $this->publication_id,
                        AssignmentTool :: PARAM_SUBMITTER_TYPE => substr($submitter, 0, 1),
                        AssignmentTool :: PARAM_TARGET_ID => substr($submitter, 1)));
            }
        }
        else // construct RepoViewer when no content object created yet
        {
            if ($this->check_start_end_time())
            {
                $repository_viewer = \repository\viewer\Manager :: construct($this);
                $repository_viewer->set_maximum_select(\repository\viewer\Manager :: SELECT_SINGLE);
                $repository_viewer->run();
            }
        }
    }

    /**
     * Returns an array of the course and platform groups the currently logged in user is a member of.
     *
     * @return array The course groups and platform groups
     */
    private function compile_choices()
    {
        $choices = array();

        $target_entities = WeblcmsRights :: get_instance()->get_target_entities(
            WeblcmsRights :: VIEW_RIGHT,
            WeblcmsManager :: context(),
            $this->publication_id,
            WeblcmsRights :: TYPE_PUBLICATION,
            $this->get_course()->get_id(),
            WeblcmsRights :: TREE_TYPE_COURSE);

        if ($this->get_target_id())
        {
            switch ($this->get_submitter_type())
            {
                case \application\weblcms\integration\tracking\AssignmentSubmission :: SUBMITTER_TYPE_COURSE_GROUP :
                    return array(
                        \application\weblcms\integration\tracking\AssignmentSubmission :: SUBMITTER_TYPE_COURSE_GROUP .
                             $this->get_target_id() => CourseGroupDataManager :: retrieve_by_id(
                                CourseGroup :: class_name(),
                                $this->get_target_id())->get_name());
                case \application\weblcms\integration\tracking\AssignmentSubmission :: SUBMITTER_TYPE_PLATFORM_GROUP :
                    return array(
                        \application\weblcms\integration\tracking\AssignmentSubmission :: SUBMITTER_TYPE_PLATFORM_GROUP .
                             $this->get_target_id() => GroupDataManager :: retrieve_by_id(
                                Group :: class_name(),
                                $this->get_target_id())->get_name());
            }
        }

        $groups_resultset = CourseGroupDataManager :: retrieve_course_groups_from_user(
            $this->get_user()->get_id(),
            $this->get_course()->get_id());
        if ($target_entities[0])
        {
            // add all course groups the user is member of
            while ($course_group = $groups_resultset->next_result())
            {
                $choices[\application\weblcms\integration\tracking\AssignmentSubmission :: SUBMITTER_TYPE_COURSE_GROUP .
                     $course_group->get_id()] = $course_group->get_name();
            }

            // retrieve platform groups subscribed to course
            $cgr_condition = new EqualityCondition(
                CourseGroupRelation :: PROPERTY_COURSE_ID,
                $this->get_course()->get_id());

            $cgr_resultset = $course_group_relations = \application\weblcms\course\DataManager :: retrieves(
                CourseGroupRelation :: class_name(),
                $cgr_condition);

            $group_ids = array();
            while ($course_group_rel = $cgr_resultset->next_result())
            {
                $group_ids[] = $course_group_rel->get_group_id();
            }
            // ***********************************************************************************************//
            // DMTODO Problem with caching. Once caching problems fixed, remove the following line of code. //
            // ***********************************************************************************************//
            \common\libraries\DataClassCache :: truncate(\group\Group :: CLASS_NAME);
            // ***********************************************************************************************//
            // DMTODO End remove code. //
            // ***********************************************************************************************//
            $groups_resultset = GroupDataManager :: retrieve_groups_and_subgroups($group_ids);
        }
        else
        {
            // add the target course groups the user is member of
            $targets = CourseGroupDataManager :: retrieve_course_groups_and_subgroups(
                $target_entities[CourseGroupEntity :: ENTITY_TYPE]);

            $target_course_groups = array();
            while ($target = $targets->next_result())
            {
                $target_course_groups[$target->get_id()] = $target->get_id();
            }
            while ($course_group = $groups_resultset->next_result())
            {
                if ($target_course_groups[$course_group->get_id()])
                {
                    $choices[\application\weblcms\integration\tracking\AssignmentSubmission :: SUBMITTER_TYPE_COURSE_GROUP .
                         $course_group->get_id()] = $course_group->get_name();
                }
            }

            // retrieve target platform groups
            $groups_resultset = GroupDataManager :: retrieve_groups_and_subgroups(
                $target_entities[CoursePlatformGroupEntity :: ENTITY_TYPE]);
        }

        // add platform groups the user is member of
        while ($group = $groups_resultset->next_result())
        {
            if (GroupDataManager :: is_group_member($group->get_id(), $this->get_user()->get_id()))
            {
                $choices[\application\weblcms\integration\tracking\AssignmentSubmission :: SUBMITTER_TYPE_PLATFORM_GROUP .
                     $group->get_id()] = $group->get_name();
            }
        }

        return $choices;
    }

    /**
     * Checks if the Assignment has started or stopped yet
     */
    private function check_start_end_time()
    {
        if ($this->assignment->get_start_time() > time())
        {
            $this->display_header();
            $date = DatetimeUtilities :: format_locale_date(
                Translation :: get('DateFormatShort', null, Utilities :: COMMON_LIBRARIES) . ', ' .
                     Translation :: get('TimeNoSecFormat', null, Utilities :: COMMON_LIBRARIES),
                    $this->assignment->get_start_time());
            echo Translation :: get('AssignmentNotStarted') . Translation :: get('StartTime') . ': ' . $date;
            $this->display_footer();

            return false;
        }

        if ($this->assignment->get_end_time() < time() && $this->assignment->get_allow_late_submissions() == 0)
        {
            $this->display_header();
            $date = DatetimeUtilities :: format_locale_date(
                Translation :: get('DateFormatShort', null, Utilities :: COMMON_LIBRARIES) . ', ' .
                     Translation :: get('TimeNoSecFormat', null, Utilities :: COMMON_LIBRARIES),
                    $this->assignment->get_end_time());
            echo Translation :: get('AssignmentEnded') . Translation :: get('EndTime') . ': ' . $date;
            $this->display_footer();

            return false;
        }

        return true;
    }

    public function get_allowed_content_object_types()
    {
        return explode(',', $this->assignment->get_allowed_types());
    }

    /**
     * Gets the selected object id of the RepoViewer
     *
     * @return int
     */
    public function get_repo_objects()
    {
        return \repository\viewer\Manager :: get_selected_objects();
    }

    /**
     * Creates an assignment submission tracker
     *
     * @param $authors string
     * @param $submitter_type int
     * @param int $submitter id
     */
    public function create_tracker($submitter_id, $repo_object, $submitter_type = null)
    {
        $object = \repository\DataManager :: retrieve_content_object($repo_object);

        // Create a folder assignment in the root folder
        $assignement_category_id = \repository\DataManager :: get_repository_category_by_name_or_create_new(
            $this->assignment->get_owner_id(),
            Translation :: get("Assignments"));

        // Create a folder course in the assignment folder
        $course_category_id = \repository\DataManager :: get_repository_category_by_name_or_create_new(
            $this->assignment->get_owner_id(),
            $this->get_course()->get_visual_code() . ' - ' . $this->get_course()->get_title(),
            $assignement_category_id);

        // Create a folder with the name of the assignment in the course folder
        $category_id = \repository\DataManager :: get_repository_category_by_name_or_create_new(
            $this->assignment->get_owner_id(),
            $this->assignment->get_title(),
            $course_category_id);

        if (is_null($submitter_type))
        {
            $submitter_type = $this->get_submitter_type();
        }

        $copier = new ContentObjectCopier(
            array($object->get_id()),
            $this->get_user_id(),
            $this->assignment->get_owner_id(),
            $category_id);
        $content_object_ids = $copier->run();
        
        if(count($content_object_ids) > 0)
        {
            foreach ($content_object_ids as $content_object_id)
            {
                $submitter_name = $this->get_submitter_name($submitter_type, $submitter_id);
                $new_object = \repository\DataManager :: retrieve_content_object($content_object_id);
                $new_object->set_title($submitter_name . ' - ' . $new_object->get_title());
                if (self :: is_downloadable($new_object))
                {
                    $new_object->set_filename($submitter_name . ' - ' . $new_object->get_filename());
                }
                $new_object->update();
            }
        }
        else
        {
            foreach($copier->get_messages() as $type)
            {
                $messages .= implode("\n", $type);
            }
            
            $this->redirect($messages, true,
                    array(
                        Tool :: PARAM_ACTION => AssignmentTool :: ACTION_BROWSE_SUBMISSIONS,
                        Tool :: PARAM_PUBLICATION_ID => $this->publication_id,
                        AssignmentTool :: PARAM_SUBMITTER_TYPE => substr($submitter, 0, 1),
                        AssignmentTool :: PARAM_TARGET_ID => substr($submitter, 1)));
        }
        $arguments = array(
            \application\weblcms\integration\tracking\AssignmentSubmission :: PROPERTY_PUBLICATION_ID => $this->publication_id,
            \application\weblcms\integration\tracking\AssignmentSubmission :: PROPERTY_CONTENT_OBJECT_ID => $content_object_id,
            \application\weblcms\integration\tracking\AssignmentSubmission :: PROPERTY_SUBMITTER_ID => $submitter_id,
            \application\weblcms\integration\tracking\AssignmentSubmission :: PROPERTY_DATE_SUBMITTED => time(),
            \application\weblcms\integration\tracking\AssignmentSubmission :: PROPERTY_SUBMITTER_TYPE => $submitter_type,
            \application\weblcms\integration\tracking\AssignmentSubmission :: PROPERTY_USER_ID => $this->get_user_id(),
            \application\weblcms\integration\tracking\AssignmentSubmission :: PROPERTY_IP_ADDRESS => $_SERVER['REMOTE_ADDR']);
        Event :: trigger('submission_assignment', WeblcmsManager :: context(), $arguments);
        return $submitter_type . $submitter_id;
    }

    /**
     * Returns the name of the submitter as a string.
     *
     * @param $submitter_type int
     * @param $submitter_id int
     * @return string The name of the submitter
     */
    private function get_submitter_name($submitter_type, $submitter_id)
    {
        switch ($submitter_type)
        {
            case \application\weblcms\integration\tracking\AssignmentSubmission :: SUBMITTER_TYPE_USER :
                return UserDataManager :: get_instance()->retrieve_user($submitter_id)->get_fullname();
            case \application\weblcms\integration\tracking\AssignmentSubmission :: SUBMITTER_TYPE_COURSE_GROUP :
                return \application\weblcms\DataManager :: retrieve(CourseGroup :: class_name(), $submitter_id)->get_name();
            case \application\weblcms\integration\tracking\AssignmentSubmission :: SUBMITTER_TYPE_PLATFORM_GROUP :
                return GroupDataManager :: retrieve_by_id(Group :: class_name(), $submitter_id)->get_name();
        }
    }

    public function add_additional_breadcrumbs(BreadcrumbTrail $breadcrumbtrail)
    {
        $this->publication_id = Request :: get(Tool :: PARAM_PUBLICATION_ID);

        $publication = \application\weblcms\DataManager :: retrieve(
            ContentObjectPublication :: class_name(),
            $this->get_publication_id());

        if (! $this->is_allowed(WeblcmsRights :: VIEW_RIGHT, $publication))
        {
            $this->redirect(
                Translation :: get("NotAllowed", null, Utilities :: COMMON_LIBRARIES),
                true,
                array(),
                array(
                    Tool :: PARAM_ACTION,
                    Tool :: PARAM_PUBLICATION_ID,
                    AssignmentTool :: PARAM_TARGET_ID,
                    AssignmentTool :: PARAM_SUBMITTER_TYPE));
        }

        $this->assignment = $publication->get_content_object();

        $breadcrumbtrail->add(
            new Breadcrumb(
                $this->get_url(array(Tool :: PARAM_ACTION => Tool :: ACTION_BROWSE)),
                Translation :: get('AssignmentToolBrowserComponent')));
        if ($this->is_allowed(WeblcmsRights :: EDIT_RIGHT))
        {
            $breadcrumbtrail->add(
                new Breadcrumb(
                    $this->get_url(
                        array(
                            Tool :: PARAM_ACTION => AssignmentTool :: ACTION_BROWSE_SUBMITTERS,
                            Tool :: PARAM_PUBLICATION_ID => $this->publication_id)),
                    $this->assignment->get_title()));
        }
        else
        {
            $breadcrumbtrail->add(
                new Breadcrumb(
                    $this->get_url(
                        array(
                            Tool :: PARAM_ACTION => AssignmentTool :: ACTION_STUDENT_BROWSE_SUBMISSIONS,
                            Tool :: PARAM_PUBLICATION_ID => $this->publication_id)),
                    $this->assignment->get_title()));
        }
    }

    public function get_additional_parameters()
    {
        return array(
            Tool :: PARAM_PUBLICATION_ID,
            AssignmentTool :: PARAM_TARGET_ID,
            AssignmentTool :: PARAM_SUBMITTER_TYPE);
    }
}
