<?php

namespace application\weblcms\tool\assignment;

use application\weblcms\WeblcmsRights;
use common\libraries\ObjectTable;
use common\libraries\ObjectTableFormAction;
use common\libraries\ObjectTableFormActions;
use common\libraries\Translation;

/**
 * Table to browse a list of submissions for a certain user.
 *
 * @author Anthony Hurst (Hogeschool Gent)
 * @author Bert De Clercq (Hogeschool Gent)
 */
class SubmitterUserSubmissionsBrowserTable extends ObjectTable
{

    const DEFAULT_NAME = 'submitter_user_submissions_browser_table';

    /**
     * Constructs an instance of SubmitterUserSubmissionsBrowserTable.
     *
     * @param $browser WeblcmsManagerComponent
     * @param $search_condition type
     *            the search condition entered.
     */
    public function __construct($browser, $search_condition, $table_conditions)
    {
        $model = new SubmitterSubmissionsTableColumnModel();
        $data_provider = new SubmitterUserSubmissionsBrowserTableDataProvider(
            $browser, $search_condition, $table_conditions
        );
        $cell_renderer = new SubmitterUserSubmissionsBrowserTableCellRenderer($browser);
        parent :: __construct($data_provider, self :: DEFAULT_NAME, $model, $cell_renderer);

        $actions = new ObjectTableFormActions(__NAMESPACE__, AssignmentTool :: PARAM_ACTION);
        if ($browser->is_allowed(WeblcmsRights :: EDIT_RIGHT))
        {
            $actions->add_form_action(
                new ObjectTableFormAction(
                    AssignmentTool :: ACTION_DOWNLOAD_SUBMISSIONS, Translation :: get('DownloadSelected'), false
                )
            );
            $actions->add_form_action(
                new ObjectTableFormAction(
                    AssignmentTool :: ACTION_DELETE_SUBMISSION, Translation :: get('DeleteSelected')
                )
            );
        }
        $this->set_form_actions($actions);
    }

}
