<?php

namespace application\weblcms\tool\assignment;

use application\weblcms\ContentObjectPublication;
use application\weblcms\Tool;


use common\libraries\FormValidator;
use common\libraries\Request;
use common\libraries\Translation;
use common\libraries\Utilities;
use repository\content_object\assignment\Assignment;

/**
 *
 * @package application.weblcms.tool.assignment.php.component
 *
 *          Custom form for the submissions feedback
 *
 * @author Joris Willems <joris.willems@gmail.com>
 * @author Alexander Van Paemel
 * @author Bert De Clercq (Hogeschool Gent)
 */
class SubmissionSubmitForm extends FormValidator
{

    public function __construct($choices, $url = '')
    {
        parent :: __construct('assignment', 'post', $url);

        $publication_id = Request :: get(Tool :: PARAM_PUBLICATION_ID);

        // Retrieving assignment
        $publication = \application\weblcms\DataManager :: retrieve(
            ContentObjectPublication :: class_name(),
            $publication_id
        );

        $assignment = $publication->get_content_object();

        $this->addElement('category', Translation :: get('Properties', null, Utilities :: COMMON_LIBRARIES));
        // Submit as
        if ($assignment->get_allow_group_submissions())
        {
            $this->addElement(
                'select', \application\weblcms\integration\tracking\AssignmentSubmission :: PROPERTY_SUBMITTER_ID,
                Translation :: get('SubmitAs'), $choices
            );
        }
        $this->addElement('category');

        $buttons = array();
        $buttons[] = $this->createElement(
            'style_submit_button', 'submit', Translation :: get('Submit', null, Utilities :: COMMON_LIBRARIES),
            array('class' => 'positive publish')
        );
        $buttons[] = $this->createElement(
            'style_reset_button', 'reset', Translation :: get('Reset', null, Utilities :: COMMON_LIBRARIES),
            array('class' => 'normal empty')
        );
        $this->addGroup($buttons, 'buttons', null, '&nbsp;', false);

        $this->setDefaults();
    }

    public function setDefaults($defaults = array())
    {
        $submitter_type = Request :: get(\application\weblcms\integration\tracking\AssignmentSubmission :: PROPERTY_SUBMITTER_TYPE);

        if (
            $submitter_type == \application\weblcms\integration\tracking\AssignmentSubmission :: SUBMITTER_TYPE_COURSE_GROUP ||
            $submitter_type == \application\weblcms\integration\tracking\AssignmentSubmission :: SUBMITTER_TYPE_PLATFORM_GROUP
        )
        {
            $defaults[\application\weblcms\integration\tracking\AssignmentSubmission :: PROPERTY_SUBMITTER_ID] =
                $submitter_type . Request :: get(AssignmentTool :: PARAM_TARGET_ID);
        }

        parent :: setDefaults($defaults);
    }

}
