<?php
namespace application\weblcms\tool\course_deleter;

use common\libraries\DelegateComponent;
use application\weblcms\course\Course;

use common\libraries\Display;
use common\libraries\Translation;

use application\weblcms\WeblcmsManager;

/**
 * this component is used to delete a course
 *
 * @author Mattias De Pauw - Hogeschool Gent
 */

class CourseDeleterToolCourseDeleterComponent extends CourseDeleterTool implements DelegateComponent
{
    public $delete_page;

    /**
     * this function checks whether the user have the rights to delete the form,
     * build the delete form and checks if the form is valid
     *
     */
    public function run()
    {
        if (!$this->get_course()->is_course_admin($this->get_parent()->get_user()))
        {
            throw new \common\libraries\NotAllowedException();
        }

        $this->delete_page = new CourseDeleterForm($this);
        $this->delete_page->buildForm();

        if ($this->delete_page->validate())
        {
            $this->delete_course();

        }
        else
        {
            $this->display_header();
            Display :: error_message(Translation :: get('DeleteReDoMessage'));
            $this->delete_page->display();
            $this->display_footer();
        }
    }

    /**
     * Deletes the course and redirect
     */
    public function delete_course()
    {
        $course = \application\weblcms\course\DataManager :: retrieve(Course :: class_name(), $this->get_course_id());

        if (!$course->delete())
        {
            throw new \Exception(Translation :: get('CourseDeleteFailed'));

        }
        else
        {
            $this->redirect(
                Translation :: get('CourseDeleted'), false, array(
                    WeblcmsManager :: PARAM_ACTION => WeblcmsManager :: ACTION_VIEW_WEBLCMS_HOME
                )
            );
        }
    }
}
