<?php
namespace application\weblcms\tool\course_group;

use common\libraries\DelegateComponent;
use application\weblcms\tool\course_group\CourseGroup;

use application\weblcms\WeblcmsRights;
use application\weblcms\WeblcmsManager;
use application\weblcms\Tool;
use common\libraries\PatternMatchCondition;
use common\libraries\ActionBarRenderer;
use common\libraries\ToolbarItem;
use user\User;
use user\UserDataManager;
use common\libraries\Application;
use common\libraries\Theme;
use common\libraries\OrCondition;
use common\libraries\Breadcrumb;
use common\libraries\BreadcrumbTrail;
use common\libraries\Request;
use common\libraries\Translation;
use common\libraries\Utilities;
use common\libraries\Redirect;
use application\weblcms\tool\document\DocumentTool;
use repository\content_object\forum\Forum;
use application\weblcms\tool\forum\ForumTool;
use common\libraries\NewObjectTableSupport;
use common\libraries\NotAllowedException;
use common\libraries\ObjectNotExistException;

/**
 * $Id: course_group_unsubscribe_browser.class.php 216 2009-11-13 14:08:06Z kariboe $
 *
 * @package application.lib.weblcms.tool.course_group.component
 */

class CourseGroupToolUnsubscribeBrowserComponent extends CourseGroupTool implements 
        NewObjectTableSupport, 
        DelegateComponent
{
    
    private $action_bar;
    
    private $course_group;

    public function run()
    {
        $course_group_id = Request :: get(CourseGroupTool :: PARAM_COURSE_GROUP);

        $course_group = DataManager :: retrieve_by_id(
            CourseGroup :: class_name(),
            $course_group_id
        );
        if(! $course_group)
            throw new ObjectNotExistException(Translation :: get('CourseGroup'), $course_group_id);

        $this->course_group = $course_group;
        BreadcrumbTrail :: get_instance()->add(new Breadcrumb($this->get_url(), Translation :: get('UnsubscribeBrowserComponent', array(
                'GROUPNAME' => $course_group->get_name()))));
        
        $this->action_bar = $this->get_action_bar();
        $html[] = '<div style="clear: both;">&nbsp;</div>';
        
        if (Request :: get(WeblcmsManager :: PARAM_USERS))
        {
            //$udm = UserDataManager :: get_instance();
            

            $users = Request :: get(WeblcmsManager :: PARAM_USERS);
            if (! is_array($users))
            {
                $users = array($users);
            }
            
            foreach ($users as $user)
            {
                // $user = $udm->retrieve_user();
                $course_group->unsubscribe_users($user);
            }
            
            $message = Translation :: get( count($users) > 1 ? 'UsersUnsubscribed' : 'UserUnsubscribed');
            $this->redirect($message, false, array(
                    Tool :: PARAM_ACTION => CourseGroupTool :: ACTION_UNSUBSCRIBE, 
                    CourseGroupTool :: PARAM_COURSE_GROUP => $course_group_id));
        }
        
        $table = new CourseGroupSubscribedUserBrowserTable($this);
        $html[] = $this->action_bar->as_html();
        
        $html[] = '<div class="clear"></div><div class="content_object" style="background-image: url(' . Theme :: get_common_image_path() . 'place_group.png);">';
        $html[] = '<div class="title">' . $course_group->get_name() . '</div>';
        $html[] = '<div class="description">' . $course_group->get_description() . '</div>';
        $html[] = '<div class="clear"></div>';
        $html[] = '<b>' . Translation :: get('NumberOfMembers') . ':</b> ' . $course_group->count_members();
        $html[] = '<br /><b>' . Translation :: get('MaximumMembers') . ':</b> ' . $course_group->get_max_number_of_members();
        $html[] = '<br /><b>' . Translation :: get('SelfRegistrationAllowed') . ':</b> ' . ($course_group->is_self_registration_allowed() ? Translation :: get('ConfirmYes', null, Utilities :: COMMON_LIBRARIES) : Translation :: get('ConfirmNo', null, Utilities :: COMMON_LIBRARIES));
        $html[] = '<br /><b>' . Translation :: get('SelfUnRegistrationAllowed') . ':</b> ' . ($course_group->is_self_unregistration_allowed() ? Translation :: get('ConfirmYes', null, Utilities :: COMMON_LIBRARIES) : Translation :: get('ConfirmNo', null, Utilities :: COMMON_LIBRARIES));
        $html[] = '<br /><b>' . Translation :: get('RandomlySubscribed') . ':</b> ' . ($course_group->is_random_registration_done() ? Translation :: get('ConfirmYes', null, Utilities :: COMMON_LIBRARIES) : Translation :: get('ConfirmNo', null, Utilities :: COMMON_LIBRARIES));
        $html[] = '</div>';
        
        $html[] = '<div class="content_object" style="background-image: url(' . Theme :: get_common_image_path() . 'place_users.png);">';
        $html[] = '<div class="title">' . Translation :: get('Users', null, 'user') . '</div>';
        $html[] = $table->as_html();
        $html[] = '</div>';
        //$trail = BreadcrumbTrail :: get_instance();
        

        $this->display_header();
        echo implode($html, "\n");
        $this->display_footer();
    }

    public function get_action_bar()
    {
        $course_group = $this->course_group;
        $action_bar = new ActionBarRenderer(ActionBarRenderer :: TYPE_HORIZONTAL);
        
        $action_bar->set_search_url($this->get_url());
        $parameters[WeblcmsManager :: PARAM_COURSE_GROUP] = $course_group->get_id();
        
        $action_bar->add_common_action(new ToolbarItem(Translation :: get('ShowAll', null, Utilities :: COMMON_LIBRARIES), Theme :: get_common_image_path() . 'action_browser.png', $this->get_url($parameters), ToolbarItem :: DISPLAY_ICON_AND_LABEL));
        
        $user = $this->get_parent()->get_user();
        
        $parameters = array();
        $parameters[WeblcmsManager :: PARAM_COURSE_GROUP] = $course_group->get_id();
        
        if (! $this->get_parent()->is_teacher())
        {
            if ($course_group->is_self_registration_allowed() && ! $course_group->is_member($user))
            {
                $parameters = array();
                $parameters[WeblcmsManager :: PARAM_COURSE_GROUP] = $course_group->get_id();
                $parameters[CourseGroupTool :: PARAM_COURSE_GROUP_ACTION] = CourseGroupTool :: ACTION_USER_SELF_SUBSCRIBE;
                $subscribe_url = $this->get_url($parameters);
                
                $action_bar->add_common_action(new ToolbarItem(Translation :: get('SubscribeToGroup'), Theme :: get_common_image_path() . 'action_subscribe.png', $subscribe_url, ToolbarItem :: DISPLAY_ICON_AND_LABEL));
            }
            
            if ($course_group->is_self_unregistration_allowed() && $course_group->is_member($user))
            {
                $parameters = array();
                $parameters[WeblcmsManager :: PARAM_COURSE_GROUP] = $course_group->get_id();
                $parameters[CourseGroupTool :: PARAM_COURSE_GROUP_ACTION] = CourseGroupTool :: ACTION_USER_SELF_UNSUBSCRIBE;
                $unsubscribe_url = $this->get_url($parameters);
                
                $action_bar->add_common_action(new ToolbarItem(Translation :: get('UnSubscribeFromGroup'), Theme :: get_common_image_path() . 'action_unsubscribe.png', $unsubscribe_url, ToolbarItem :: DISPLAY_ICON_AND_LABEL));
            }
        }
        else
        {
            $parameters = array();
            $parameters[WeblcmsManager :: PARAM_COURSE_GROUP] = $course_group->get_id();
            $parameters[CourseGroupTool :: PARAM_COURSE_GROUP_ACTION] = CourseGroupTool :: ACTION_MANAGE_SUBSCRIPTIONS;
            $subscribe_url = $this->get_url($parameters);
            
            $action_bar->add_common_action(new ToolbarItem(Translation :: get('SubscribeUsers'), Theme :: get_common_image_path() . 'action_subscribe.png', $subscribe_url, ToolbarItem :: DISPLAY_ICON_AND_LABEL));
            
            // export the list to a spreadsheet
            $parameters_export_subscriptions_overview = array();
            $parameters_export_subscriptions_overview[Tool :: PARAM_ACTION] = CourseGroupTool :: ACTION_EXPORT_SUBSCRIPTIONS_OVERVIEW;
            $parameters_export_subscriptions_overview[CourseGroupTool :: PARAM_COURSE_GROUP] = $course_group->get_id();
            $action_bar->add_common_action(new ToolbarItem(Translation :: get('Export', null, Utilities :: COMMON_LIBRARIES), Theme :: get_common_image_path() . 'action_backup.png', $this->get_url($parameters_export_subscriptions_overview), ToolbarItem :: DISPLAY_ICON_AND_LABEL));
        }
        
        // $action_bar->add_tool_action(new ToolbarItem(Translation ::
        // get('Edit'), Theme :: get_common_image_path().'action_edit.png',
        // $this->get_url(array(CourseGroupTool :: PARAM_ACTION =>
        // CourseGroupTool :: ACTION_PUBLISH)), ToolbarItem ::
        // DISPLAY_ICON_AND_LABEL));
        // $action_bar->add_tool_action(new ToolbarItem(Translation ::
        // get('Delete'), Theme :: get_common_image_path().'action_delete.png',
        // $this->get_url(), ToolbarItem :: DISPLAY_ICON_AND_LABEL));
        

        if ($course_group->get_document_category_id())
        {
            $type_name = 'document';
            $params = array();
            $params[Application :: PARAM_APPLICATION] = WeblcmsManager :: APPLICATION_NAME;
            $params[Application :: PARAM_ACTION] = WeblcmsManager :: ACTION_VIEW_COURSE;
            $params[WeblcmsManager :: PARAM_COURSE] = $course_group->get_course_code();
            $params[WeblcmsManager :: PARAM_TOOL] = $type_name;
            $params[WeblcmsManager :: PARAM_TOOL_ACTION] = DocumentTool :: ACTION_BROWSE;
            $params[WeblcmsManager :: PARAM_CATEGORY] = $course_group->get_document_category_id();
            $url = $this->get_url($params);
            
            $namespace = Tool :: get_tool_type_namespace($type_name);
            $action_bar->add_tool_action(new ToolbarItem(Translation :: get('TypeName', null, $namespace), Theme :: get_image_path($namespace) . 'logo/16.png', $url, ToolbarItem :: DISPLAY_ICON_AND_LABEL));
        }
        
        if ($course_group->get_forum_category_id())
        {
            $type_name = 'forum';
            $params = array();
            $params[Application :: PARAM_APPLICATION] = WeblcmsManager :: APPLICATION_NAME;
            $params[Application :: PARAM_ACTION] = WeblcmsManager :: ACTION_VIEW_COURSE;
            $params[WeblcmsManager :: PARAM_COURSE] = $course_group->get_course_code();
            $params[WeblcmsManager :: PARAM_TOOL] = $type_name;
            $params[WeblcmsManager :: PARAM_TOOL_ACTION] = ForumTool :: ACTION_BROWSE;
            $params[WeblcmsManager :: PARAM_CATEGORY] = $course_group->get_forum_category_id();
            $url = $this->get_url($params);
            
            $namespace = Tool :: get_tool_type_namespace($type_name);
            $action_bar->add_tool_action(new ToolbarItem(Translation :: get('TypeName', null, $namespace), Theme :: get_image_path($namespace) . 'logo/16.png', $url, ToolbarItem :: DISPLAY_ICON_AND_LABEL));
        }
        
        return $action_bar;
    }

    public function get_condition()
    {
        $query = $this->action_bar->get_query();
        if (isset($query) && $query != '')
        {
            $conditions[] = new PatternMatchCondition(User :: PROPERTY_USERNAME, '*' . $query . '*');
            $conditions[] = new PatternMatchCondition(User :: PROPERTY_FIRSTNAME, '*' . $query . '*');
            $conditions[] = new PatternMatchCondition(User :: PROPERTY_LASTNAME, '*' . $query . '*');
            return new OrCondition($conditions);
        }
    }

    public function get_course_group()
    {
        return $this->course_group;
    }

    public function add_additional_breadcrumbs(BreadcrumbTrail $breadcrumbtrail)
    {
        $breadcrumbtrail->add_help('weblcms_course_group_unsubscribe_browser');
    }

    public function get_additional_parameters()
    {
        return array(CourseGroupTool :: PARAM_COURSE_GROUP);
    }

    public function get_object_table_condition($object_table_class_name)
    {
        return $this->get_condition();
    }
}
