<?php
namespace application\weblcms\tool\course_group\upgrade\v4_0_0;

use application\weblcms\tool\course_group\CourseGroup;
use common\libraries\CommonDataManager;
use common\libraries\package\Package;
use common\libraries\Translation;
use common\libraries\Utilities;

/**
 * Database upgrader for Chamilo v4.0.0.
 * 
 */
class Upgrader extends \common\libraries\package\Upgrader
{

    /**
     * Constructor
     */
    public function __construct()
    {
        parent :: __construct(\application\weblcms\tool\course_group\DataManager :: get_instance());
    }

    /*
     * (non-PHPdoc) @see \common\libraries\package\Upgrader::run()
     */
    public function run()
    {
        $data_manager = $this->get_data_manager();
        $upgrader = $this;

        $success = $data_manager->transactional(
            function ($c) use($data_manager, $upgrader)
            {
                $success = $upgrader->upgrade_db($data_manager);

                if($success)
                {
                    $upgrader->add_message(\common\libraries\package\Upgrader :: TYPE_NORMAL, Translation :: get('UpgradeDatabaseSuccessful', null, Utilities :: COMMON_LIBRARIES));
                }
                else
                {
                    $upgrader->add_message(\common\libraries\package\Upgrader :: TYPE_ERROR, Translation :: get('UpgradeDatabaseFailed', null, Utilities :: COMMON_LIBRARIES));
                }

                if(!$success)
                {
                    return false;
                }

                $success = $upgrader->update_db_content($data_manager);

                if($success)
                {
                    $upgrader->add_message(\common\libraries\package\Upgrader :: TYPE_NORMAL, Translation :: get('UpdateDatabaseContentSuccessful', null, Utilities :: COMMON_LIBRARIES));
                }
                else
                {
                    $upgrader->add_message(\common\libraries\package\Upgrader :: TYPE_ERROR, Translation :: get('UpdateDatabaseContentFailed', null, Utilities :: COMMON_LIBRARIES));
                }

                if(!$success)
                {
                    return false;
                }

                $success = $upgrader->upgrade_registration($data_manager);

                if($success)
                {
                    $upgrader->add_message(\common\libraries\package\Upgrader :: TYPE_NORMAL, Translation :: get('UpgradeRegistrationSuccessful', null, Utilities :: COMMON_LIBRARIES));
                }
                else
                {
                    $upgrader->add_message(\common\libraries\package\Upgrader :: TYPE_ERROR, Translation :: get('UpgradeRegistrationFailed', null, Utilities :: COMMON_LIBRARIES));
                }

                return $success;

            });

        if ($success)
        {
            $this->add_message(self :: TYPE_NORMAL, Translation :: get('UpgradeDatabaseSuccessful', null, Utilities :: COMMON_LIBRARIES));
        }
        else
        {
            $this->add_message(self :: TYPE_ERROR, Translation :: get('UpgradeDatabaseFailed', null, Utilities :: COMMON_LIBRARIES));
        }

        return $success;
    }

    /**
     * Upgrade the table structures
     * @param $data_manager
     * @return bool
     */
    public function upgrade_db($data_manager)
    {
        // Modify existing columns
        $course_group_column_attributes = array(
            CourseGroup :: PROPERTY_SELF_REG => array(
                'type' => 'integer',
                'length' => '5',
                'unsigned' => true,
                'notnull' => true,
                'default' => '0'),
            CourseGroup :: PROPERTY_SELF_UNREG => array(
                'type' => 'integer',
                'length' => '5',
                'unsigned' => true,
                'notnull' => true,
                'default' => '0'),
            CourseGroup :: PROPERTY_NAME => array(
                'type' => 'text',
                'length' => '100',
                'notnull' => true,
                'default' => ''),
            CourseGroup :: PROPERTY_DESCRIPTION => array(
                'type' => 'text',
                'notnull' => false,
                'default' => null)
        );

        foreach ($course_group_column_attributes as $column => $attributes)
        {
            $success = $data_manager->alter_storage_unit(
                \common\libraries\DataManager :: ALTER_STORAGE_UNIT_CHANGE,
                CourseGroup :: get_table_name(),
                $column,
                $attributes
            );

            if(! $success)
            {
                return false;
            }
        }

        // Add new columns
        $course_group_column_attributes = array(
            CourseGroup :: PROPERTY_MAX_NUMBER_OF_COURSE_GROUP_PER_MEMBER => array(
                'type' => 'integer',
                'length' => '5',
                'unsigned' => true,
                'default' => '0'),
            CourseGroup :: PROPERTY_RANDOM_REG => array(
                'type' => 'integer',
                'length' => '5',
                'unsigned' => true,
                'notnull' => true,
                'default' => '0'),
        );

        foreach ($course_group_column_attributes as $column => $attributes)
        {
            $success = $data_manager->alter_storage_unit(
                \common\libraries\DataManager :: ALTER_STORAGE_UNIT_ADD,
                CourseGroup :: get_table_name(),
                $column,
                $attributes
            );

            if(! $success)
            {
                return false;
            }
        }

        // Remove old tables
        if($data_manager->storage_unit_exist('course_group_group_relation'))
        {
            if (! $data_manager->drop_storage_unit('course_group_group_relation'))
            {
                return false;
            }
        }

        if (! $success)
        {
            return false;
        }       


        // TODO :: Migrate data from old tables?


        return true;
    }

    /**
     * Performs the database content upgrade.
     *
     * @param type $data_manager
     * @return boolean
     */
    public function update_db_content($data_manager)
    {
        return true;
    }

    /**
     * Upgrades the package registration.
     *
     * @return boolean
     */
    public function upgrade_registration($data_manager)
    {
        $package = Package :: get($data_manager :: context());
        $registration = CommonDataManager :: get_registration($data_manager :: context());
        $registration->set_version($package->get_version());
        if (! $registration->update())
        {
            return false;
        }
        return true;
    }
}
