<?php
namespace application\weblcms\tool\document;

use repository\content_object\file\File;
use common\libraries\StaticConditionVariable;
use common\libraries\PropertyConditionVariable;

use common\libraries\DelegateComponent;
use repository\content_object\calendar_event\CalendarEvent;
use repository\content_object\document\Document;
use application\weblcms\ContentObjectPublication;
use application\weblcms\Tool;
use common\libraries\PatternMatchCondition;
use repository\ContentObject;
use application\weblcms\ContentObjectPublicationListRenderer;
use common\libraries\SubselectCondition;
use common\libraries\ToolbarItem;
use common\libraries\Theme;
use common\libraries\OrCondition;
use common\libraries\InequalityCondition;
use common\libraries\Request;
use application\weblcms\ToolComponent;
use common\libraries\Translation;
use common\libraries\Utilities;

class DocumentToolBrowserComponent extends DocumentTool implements DelegateComponent
{
    const PARAM_FILTER = 'filter';
    const FILTER_TODAY = 'today';
    const FILTER_THIS_WEEK = 'week';
    const FILTER_THIS_MONTH = 'month';

    public function run()
    {
        ToolComponent :: launch($this);
    }

    public function get_tool_actions()
    {
        $tool_actions = array();
        $tool_actions[] = new ToolbarItem(Translation :: get('ShowToday', null, Utilities :: COMMON_LIBRARIES),
            Theme :: get_common_image_path() . 'action_browser.png', $this->get_url(
                array(
                    Tool :: PARAM_ACTION => null, self :: PARAM_FILTER => self :: FILTER_TODAY
                )
            ), ToolbarItem :: DISPLAY_ICON_AND_LABEL);

        $tool_actions[] = new ToolbarItem(Translation :: get('ShowThisWeek', null, Utilities :: COMMON_LIBRARIES),
            Theme :: get_common_image_path() . 'action_browser.png', $this->get_url(
                array(
                    Tool :: PARAM_ACTION => null, self :: PARAM_FILTER => self :: FILTER_THIS_WEEK
                )
            ), ToolbarItem :: DISPLAY_ICON_AND_LABEL);

        $tool_actions[] = new ToolbarItem(Translation :: get('ShowThisMonth', null, Utilities :: COMMON_LIBRARIES),
            Theme :: get_common_image_path() . 'action_browser.png', $this->get_url(
                array(
                    Tool :: PARAM_ACTION => null, self :: PARAM_FILTER => self :: FILTER_THIS_MONTH
                )
            ), ToolbarItem :: DISPLAY_ICON_AND_LABEL);

        $tool_actions[] = new ToolbarItem(Translation :: get('Download'),
            Theme :: get_common_image_path() . 'action_save.png', $this->get_url(
                array(
                    DocumentTool :: PARAM_ACTION => DocumentTool :: ACTION_ZIP_AND_DOWNLOAD
                )
            ), ToolbarItem :: DISPLAY_ICON_AND_LABEL);

        return $tool_actions;
    }

    public function get_tool_conditions()
    {
        $conditions = array();
        $filter = Request :: get(self :: PARAM_FILTER);

        switch ($filter)
        {
            case self :: FILTER_TODAY :
                $time = mktime(0, 0, 0, date('m', time()), date('d', time()), date('Y', time()));
                break;
            case self :: FILTER_THIS_WEEK :
                $time = strtotime('Next Monday', strtotime('-1 Week', time()));
                break;
            case self :: FILTER_THIS_MONTH :
                $time = mktime(0, 0, 0, date('m', time()), 1, date('Y', time()));
                break;
        }

        if($filter)
        {
            $conditions[] = new InequalityCondition(
                new PropertyConditionVariable(
                    ContentObjectPublication :: class_name(), ContentObjectPublication :: PROPERTY_MODIFIED_DATE
                ),
                InequalityCondition :: GREATER_THAN_OR_EQUAL,
                new StaticConditionVariable($time)
            );
        }

        $browser_type = $this->get_browser_type();
        if ($browser_type == ContentObjectPublicationListRenderer :: TYPE_GALLERY ||
            $browser_type == ContentObjectPublicationListRenderer :: TYPE_SLIDESHOW
        )
        {
            $classes = array();

            if(ContentObject :: is_available('repository\content_object\document\Document'))
            {
                $classes[] = Document :: class_name();
            }

            if(ContentObject :: is_available('repository\content_object\file\File'))
            {
                $classes[] = File :: class_name();
            }

            $image_subselect_conditions = array();

            foreach($classes as $class)
            {
                $image_types = $class :: get_image_types();
                $image_conditions = array();
                foreach ($image_types as $image_type)
                {
                    $image_conditions[] = new PatternMatchCondition(
                        new PropertyConditionVariable($class, $class :: PROPERTY_FILENAME),
                        '*.' . $image_type
                    );
                }

                $image_condition = new OrCondition($image_conditions);

                $image_subselect_conditions[] = new SubselectCondition(
                    new PropertyConditionVariable(
                        ContentObjectPublication :: class_name(),
                        ContentObjectPublication :: PROPERTY_CONTENT_OBJECT_ID
                    ),
                    new PropertyConditionVariable(
                        $class,
                        $class :: PROPERTY_ID
                    ),
                    $class :: get_table_name(), $image_condition
                );
            }

            $conditions[] = new OrCondition($image_subselect_conditions);
        }

        return $conditions;
    }

    public function convert_content_object_publication_to_calendar_event($publication, $from_time, $to_time)
    {
        $calendar_event = ContentObject :: factory(CalendarEvent :: class_name());

        $calendar_event->set_title($publication[ContentObject :: PROPERTY_TITLE]);
        $calendar_event->set_description($publication[ContentObject :: PROPERTY_DESCRIPTION]);
        $calendar_event->set_start_date($publication[ContentObjectPublication :: PROPERTY_MODIFIED_DATE]);
        $calendar_event->set_end_date($publication[ContentObjectPublication :: PROPERTY_MODIFIED_DATE]);
        $calendar_event->set_frequency(CalendarEvent :: FREQ_NONE);

        return $calendar_event;
    }

    public function get_additional_parameters()
    {
        return array(self :: PARAM_BROWSE_PUBLICATION_TYPE);
    }
}
