<?php

namespace application\weblcms\tool\forum;

use application\weblcms\ContentObjectPublication;
use repository\content_object\forum\DataManager as ForumDataManager;
use repository\ContentObject;

use application\weblcms\Tool;
use application\weblcms\WeblcmsRights;


use common\libraries\Request;
use common\libraries\Translation;
use common\libraries\NotAllowedException;

/**
 * Class responsible for the creation of forum subscribes at application level.
 *
 * @author Mattias De Pauw - Hogeschool Gent
 * @author Maarten Volckaert - Hogeschool Gent
 */
class ForumToolForumSubscribeComponent extends ForumTool
{
    public function run()
    {
        $this->publication_id = Request :: get(Tool :: PARAM_PUBLICATION_ID);

        $publication = \application\weblcms\DataManager :: retrieve(
            ContentObjectPublication :: class_name(),
            $this->publication_id
        );

        if (!$this->is_allowed(WeblcmsRights :: VIEW_RIGHT, $publication))
        {
            throw new NotAllowedException();
        }

        $object = \repository\DataManager :: retrieve_content_object(
            Request :: get(ForumTool :: PARAM_FORUM_ID)
        );

        $succes = ForumDataManager :: create_subscribe($this->get_user_id(), $object->get_id());
        if ($succes)
        {
            $message = Translation :: get(
                "SuccesSubscribe", null, ContentObject :: get_content_object_type_namespace('forum')
            );
        }
        else
        {
            $message = Translation :: get(
                "UnSuccesSubscribe", null, ContentObject :: get_content_object_type_namespace('forum')
            );
        }
        $this->redirect(
            $message, ($succes ? false : true),
            array(ForumTool :: PARAM_ACTION => ForumTool :: ACTION_BROWSE),
            array(ForumTool::PARAM_PUBLICATION_ID)
        );
    }

    public function get_additional_parameters()
    {
        return array(self :: PARAM_PUBLICATION_ID);
    }
}
