<?php

namespace application\weblcms\tool\forum;

use application\weblcms\ContentObjectPublication;
use repository\content_object\forum\DataManager as ForumDataManager;
use repository\ContentObject;

use application\weblcms\Tool;
use application\weblcms\WeblcmsRights;


use common\libraries\Request;
use common\libraries\Translation;
use common\libraries\NotAllowedException;


/**
 * This represents the component for unsubscribing a forum at forum application level.
 *
 * @author Mattias De Pauw - Hogeschool Gent
 * @author Maarten Volckaert - Hogeschool Gent
 * ForumManagerForumUnsubscribeComponent
 */
class ForumToolForumUnsubscribeComponent extends ForumTool
{
    public function run()
    {
        $this->publication_id = Request :: get(Tool :: PARAM_PUBLICATION_ID);

        $publication = \application\weblcms\DataManager :: retrieve(
            ContentObjectPublication :: class_name(),
            $this->publication_id
        );

        if (!$this->is_allowed(WeblcmsRights :: VIEW_RIGHT, $publication))
        {
            throw new NotAllowedException();
        }

        $success = false;
        $subscribe_id = Request :: get(ForumTool :: PARAM_SUBSCRIBE_ID);

        if ($subscribe_id)
        {
            $subscribe = ForumDataManager :: retrieve_subscribe($subscribe_id, $this->get_user_id());
            $success = $subscribe && $subscribe->delete();
        }
        
        $message = Translation :: get($success ? "SuccesUnSubscribe" : "UnSuccesUnSubscribe",
                null, ContentObject :: get_content_object_type_namespace('forum'));

        $this->redirect(
            $message, !$success,
            array(ForumTool :: PARAM_ACTION => ForumTool :: ACTION_BROWSE), array(ForumTool::PARAM_PUBLICATION_ID)
        );
    }

    public function get_additional_parameters()
    {
        return array(self :: PARAM_PUBLICATION_ID);
    }
}
