<?php
namespace application\weblcms\tool\user;

use application\weblcms\WeblcmsRights;
use application\weblcms\WeblcmsManager;
use application\weblcms\Tool;
use common\libraries\PatternMatchCondition;
use common\libraries\ActionBarRenderer;
use common\libraries\ToolbarItem;
use common\libraries\Display;
use common\libraries\Theme;
use common\libraries\OrCondition;
use common\libraries\InCondition;
use common\libraries\NotCondition;
use common\libraries\AndCondition;
use common\libraries\EqualityCondition;
use common\libraries\Request;
use common\libraries\Translation;
use group\Group;
use group\DataManager as GroupDataManager;

/**
 * $Id: user_group_subscribe_browser.class.php 216 2009-11-13 14:08:06Z kariboe
 * $
 *
 * @package application.lib.weblcms.tool.user.component
 */
class UserToolGroupSubscribeBrowserComponent extends UserTool
{
    private $action_bar;
    private $group;
    private $root_group;
    private $subscribed_groups;

    public function run()
    {
        if (!$this->is_allowed(WeblcmsRights :: EDIT_RIGHT))
        {
            Display :: not_allowed();
        }

        $this->subscribed_groups = $this->get_subscribed_platformgroup_ids($this->get_course_id());
        $this->action_bar = $this->get_action_bar();

        $this->display_header();

        echo $this->action_bar->as_html();
        echo $this->get_group_menu();
        echo $this->get_group_subscribe_html();

        $this->display_footer();
    }

    public function get_group_subscribe_html()
    {
        $table = new UnsubscribedGroupBrowserTable($this, $this->get_parameters(), $this->get_condition());

        $html = array();
        $html[] = '<div style="width: 70%; float: right;">';
        $html[] = $table->as_html();
        $html[] = '</div>';

        return implode($html, "\n");
    }

    public function get_group_menu()
    {
        $tree = new PlatformgroupMenuRenderer($this, array($this->get_root_group()->get_id()));

        $html = array();
        $html[] = '<div style="overflow: auto; width: 29%; float: left;">';
        $html[] = $tree->render_as_tree();
        $html[] = '</div>';

        return implode($html, "\n");
    }

    public function get_action_bar()
    {
        $action_bar = new ActionBarRenderer(ActionBarRenderer :: TYPE_HORIZONTAL);

        $action_bar->set_search_url($this->get_url());

        $action_bar->add_common_action(
            new ToolbarItem(
                Translation :: get('ViewUsers'),
                Theme :: get_common_image_path() . 'action_browser.png',
                $this->get_url(array(UserTool :: PARAM_ACTION => UserTool :: ACTION_UNSUBSCRIBE_BROWSER)),
                ToolbarItem :: DISPLAY_ICON_AND_LABEL
            )
        );

        return $action_bar;
    }

    public function get_group()
    {
        if (!$this->group)
        {
            $this->group = Request :: get(WeblcmsManager :: PARAM_GROUP);

            if (!$this->group)
            {
                $this->group = $this->get_root_group()->get_id();
            }

        }

        return $this->group;
    }

    public function get_root_group()
    {
        if (!$this->root_group)
        {
            $group = GroupDataManager :: retrieve(
                Group :: class_name(), new EqualityCondition(Group :: PROPERTY_PARENT_ID, 0)
            );
            $this->root_group = $group;
        }

        return $this->root_group;
    }

    public function get_condition()
    {
        $conditions[] = new EqualityCondition(Group :: PROPERTY_PARENT_ID, $this->get_group());

        // filter already subscribed groups
        if ($this->subscribed_groups)
        {
            $conditions[] = new NotCondition(new InCondition(Group :: PROPERTY_ID, $this->subscribed_groups));
        }

        $query = $this->action_bar->get_query();
        if (isset($query) && $query != '')
        {
            $conditions2[] = new PatternMatchCondition(Group :: PROPERTY_NAME, '*' . $query . '*');
            $conditions2[] = new PatternMatchCondition(Group :: PROPERTY_DESCRIPTION, '*' . $query . '*');
            $conditions[] = new OrCondition($conditions2);
        }

        return new AndCondition($conditions);
    }

    public function get_additional_parameters()
    {
        return array(self :: PARAM_TAB, WeblcmsManager :: PARAM_GROUP);
    }
}
