<?php
namespace application\weblcms\tool\user;
use application\weblcms\WeblcmsRights;
use application\weblcms\Tool;
use common\libraries\Translation;
use common\libraries\ToolbarItem;
use common\libraries\Toolbar;
use common\libraries\Theme;
use user\UserManager;
use group\GroupRelUser;
use group\DefaultGroupRelUserTableCellRenderer;
use common\libraries\Request;

/**
 * * ***************************************************************************
 * Cell renderer for a platform group rel user browser table.
 *
 * @author Stijn Van Hoecke
 *
 *         ****************************************************************************
 */
class PlatformGroupRelUserBrowserTableCellRenderer extends DefaultGroupRelUserTableCellRenderer
{
    /**
     * The repository browser component
     */
    private $browser;

    /**
     * Constructor
     *
     * @param $browser RepositoryManagerBrowserComponent
     */
    public function __construct($browser)
    {
        parent :: __construct();
        $this->browser = $browser;
    }

    // Inherited
    public function render_cell($column, $groupreluser)
    {
        if ($column === PlatformGroupRelUserBrowserTableColumnModel :: get_modification_column())
        {
            return $this->get_modification_links($groupreluser);
        }

        // Add special features here
        switch ($column->get_name())
        {
            // Exceptions that need post-processing go here ...
            case GroupRelUser :: PROPERTY_USER_ID :
                $user_id = parent :: render_cell($column, $groupreluser);
                $user = UserManager :: retrieve_user($user_id);
                return $user->get_fullname();
        }
        return parent :: render_cell($column, $groupreluser);
    }

    /**
     * Gets the action links to display
     *
     * @param $content_object ContentObject
     *            The learning object for which the
     *            action links should be returned
     * @return string A HTML representation of the action links
     */
    private function get_modification_links($groupreluser)
    {
        // construct the toolbar
        $toolbar = new Toolbar(Toolbar :: TYPE_HORIZONTAL);

        // always show details
        $parameters = array();
        $parameters[Tool :: PARAM_ACTION] = UserTool :: ACTION_USER_DETAILS;
        $parameters[UserTool :: PARAM_TAB] = Request :: get(UserTool :: PARAM_TAB);
        $parameters[UserTool :: PARAM_OBJECTS] = $groupreluser->get_user_id();
        $details_url = $this->browser->get_url($parameters);

        $toolbar->add_item(
            new ToolbarItem(
                Translation :: get('Details'),
                Theme :: get_common_image_path() . 'action_details.png',
                $details_url,
                ToolbarItem :: DISPLAY_ICON
            )
        );

        // if we have editing rights, display the reporting action but never
        // allow unsubscribe
        if ($this->browser->is_allowed(WeblcmsRights :: EDIT_RIGHT))
        {
            $toolbar->add_item(
                new ToolbarItem(
                    Translation :: get('UnsubscribeNotAvailableForGroups'),
                    Theme :: get_common_image_path() . 'action_unsubscribe_na.png',
                    null,
                    ToolbarItem :: DISPLAY_ICON
                )
            );

            $params = array();
            $params[UserTool :: PARAM_OBJECTS] = $groupreluser->get_user_id();
            $params[Tool :: PARAM_ACTION] = UserTool :: ACTION_REPORTING;
            $parameters[UserTool :: PARAM_TAB] = Request :: get(UserTool :: PARAM_TAB);
            $reporting_url = $this->browser->get_url($params);

            $toolbar->add_item(
                new ToolbarItem(
                    Translation :: get('Report'),
                    Theme :: get_common_image_path() . 'action_reporting.png',
                    $reporting_url, ToolbarItem :: DISPLAY_ICON
                )
            );
        }

        // return
        return $toolbar->as_html();
    }
}
