<?php
namespace application\weblcms\tool\user;

use application\weblcms\Tool;
use common\libraries\Breadcrumb;
use common\libraries\BreadcrumbTrail;
use common\libraries\Translation;
use application\weblcms\WeblcmsManager;

use application\weblcms\integration\tracking\UserStatusChange;

use common\libraries\Application;
use common\libraries\Request;
use common\libraries\Display;
use application\weblcms\WeblcmsRights;

use tracking\Event;

abstract class UserToolStatusChangerComponent extends UserTool
{

    protected $object;

    public function run()
    {
        if (!$this->is_allowed(WeblcmsRights :: EDIT_RIGHT))
        {
            Display :: not_allowed();
        }

        $objects = Request :: get(self :: PARAM_OBJECTS);
        // $status = Request :: get(self :: PARAM_STATUS);

        if (!$objects /* || !$status*/)
        {
            throw new \common\libraries\NoObjectSelectedException(Translation :: get('UserRelation'));
        }

        if (!is_array($objects))
        {
            $objects = array($objects);
        }

        $failed = 0;

        foreach ($objects as $this->object)
        {
            $relation = $this->get_relation(); // abstract -> implementation
            // gets user or group relation
            if (!$relation)
            {
                $failed++;
            }

            // $relation->set_status($status);
            $relation->set_status($this->get_status());
            if (!$relation->update())
            {
                $failed++;
            }
            
            $parameters[UserStatusChange :: PROPERTY_USER_ID] = $this->get_user_id();
            $parameters[UserStatusChange :: PROPERTY_SUBJECT_ID] = $this->object;
            $parameters[UserStatusChange :: PROPERTY_NEW_STATUS] = $this->get_status();
            $parameters[UserStatusChange :: PROPERTY_COURSE_ID] = Request :: get(WeblcmsManager :: PARAM_COURSE);
            $parameters[UserStatusChange :: PROPERTY_DATE] = time();
            Event :: trigger('user_status_change',WeblcmsManager :: context(), $parameters);
            
        }

        $message = $this->get_general_result(
            $failed, count($objects), Translation :: get('UserStatus'), Translation :: get('UserStatusses'),
            Application :: RESULT_TYPE_UPDATED
        );

        $this->redirect(
            $message, $failed > 0, array(
                Tool :: PARAM_ACTION  => UserTool :: ACTION_UNSUBSCRIBE_BROWSER,
                UserTool :: PARAM_TAB => Request :: get(UserTool :: PARAM_TAB)
            )
        );

    }

    public function add_additional_breadcrumbs(BreadcrumbTrail $breadcrumbtrail)
    {
        $breadcrumbtrail->add(
            new Breadcrumb(
                $this->get_url(
                    array(
                        Tool :: PARAM_ACTION  => UserTool :: ACTION_UNSUBSCRIBE_BROWSER,
                        UserTool :: PARAM_TAB => Request :: get(UserTool :: PARAM_TAB)
                    )
                ),
                Translation :: get('UserToolUnsubscribeUserBrowserComponent')
            )
        );
    }

    public function get_additional_parameters()
    {
        $params = array();
        $params[] = self :: PARAM_TAB;
        if (Request :: get(WeblcmsManager :: PARAM_GROUP))
        {
            $params[] = WeblcmsManager :: PARAM_GROUP;
        }
        return $params;
    }

    abstract public function get_relation();

    abstract public function get_status();
}
