<?php
namespace application\weblcms\tool\user;

use application\weblcms\WeblcmsManager;
use common\libraries\TableFormAction;
use common\libraries\TableFormActions;
use common\libraries\TableFormActionsSupport;
use common\libraries\RecordTable;
use application\weblcms\Tool;
use common\libraries\Request;
use common\libraries\ObjectTable;
use common\libraries\Translation;

/**
 * Table to display a list of users in a course (direct subscribed or platform group).
 *
 * @author Stijn Van Hoecke
 * @author Sven Vanpoucke - Hogeschool Gent - Refactoring from ObjectTable to RecordTable
 */
class SubscribedUserBrowserTable extends RecordTable implements TableFormActionsSupport
{
    const TABLE_IDENTIFIER = UserTool :: PARAM_OBJECTS;

    /****************************************************************************************************************
     * Inherited Functionality                                                                                      *
     ****************************************************************************************************************/

    public function get_implemented_form_actions()
    {
        $actions = new TableFormActions(__NAMESPACE__, Tool :: PARAM_ACTION);

        if ($this->get_component()->is_course_admin($this->get_component()->get_user()))
        {
            // if we are not editing groups
            if (!Request :: get(WeblcmsManager :: PARAM_GROUP))
            {
                $actions->add_form_action(
                    new TableFormAction(
                        UserTool :: ACTION_UNSUBSCRIBE,
                        Translation :: get('UnsubscribeSelected'),
                        false
                    )
                );

                // make teacher
                $actions->add_form_action(
                    new TableFormAction(
                        UserTool :: ACTION_CHANGE_USER_STATUS_TEACHER,
                        Translation :: get('MakeTeacher'),
                        false
                    )
                );

                // make student
                $actions->add_form_action(
                    new TableFormAction(
                        UserTool :: ACTION_CHANGE_USER_STATUS_STUDENT,
                        Translation :: get('MakeStudent'),
                        false
                    )
                );
            }
        }

        return $actions;
    }
}
