<?php

namespace application\weblcms\tool\user;

use common\libraries\DelegateComponent;
use common\libraries\StaticConditionVariable;
use common\libraries\PropertyConditionVariable;


use application\weblcms\WeblcmsRights;
use application\weblcms\WeblcmsManager;
use application\weblcms\Tool;
use common\libraries\PatternMatchCondition;
use group\DataManager as GroupDataManager;
use group\Group;
use group\GroupRelUser;
use common\libraries\ActionBarRenderer;
use common\libraries\ToolbarItem;
use user\User;
use common\libraries\Display;
use common\libraries\Theme;
use common\libraries\OrCondition;
use common\libraries\BreadcrumbTrail;
use common\libraries\AndCondition;
use common\libraries\EqualityCondition;
use common\libraries\Request;
use common\libraries\Translation;
use common\libraries\DynamicVisualTab;
use common\libraries\DynamicVisualTabsRenderer;
use common\libraries\Utilities;
use common\libraries\ActionBarSearchForm;
use common\libraries\ConditionProperty;
use application\weblcms\tool\course_group\CourseGroupTool;
use common\libraries\TableSupport;
use application\weblcms\course\CourseUserRelation;
use application\weblcms\course\CourseGroupRelation;

/**
 *
 * @author Stijn Van Hoecke
 * @package application.lib.weblcms.tool.user.component
 */
class UserToolUnsubscribeBrowserComponent extends UserTool implements TableSupport, DelegateComponent
{
    const TAB_ALL = 1;
    const TAB_USERS = 2;
    const TAB_PLATFORM_GROUPS_SUBGROUPS = 3;
    const TAB_PLATFORM_GROUPS_USERS = 4;

    const PLATFORM_GROUP_ROOT_ID = 0;

    private $action_bar;
    private $current_tab;
    private $tabs;

    public function run()
    {
        // default all tab, unless specified
        $this->current_tab = self :: TAB_ALL;
        if (Request :: get(self :: PARAM_TAB))
        {
            $this->current_tab = Request :: get(self :: PARAM_TAB);
        }

        // display header
        $this->display_header();
        // display actionbar
        $this->action_bar = $this->get_action_bar();
        echo $this->action_bar->as_html();

        // display tabs
        echo $this->get_tabs();

        // echo $this->display_users();
        // display footer
        $this->display_footer();
    }

    // **************************************************************************
    // TABS FUNCTIONS
    // **************************************************************************
    /**
     * Creates the tab structure.
     *
     * @return String HTML of the tab(s)
     */
    private function get_tabs()
    {
        $html = $this->get_tabs_header();
        $html .= $this->get_tabs_content();
        $html .= $this->get_tabs_footer();
        return $html;
    }

    /**
     * Creates the header for the tabs.
     *
     * @return String HTML of the header
     */
    private function get_tabs_header()
    {
        $html = array();

        $this->tabs = new DynamicVisualTabsRenderer('weblcms_course_user_browser');

        // all tab
        $link = $this->get_url(array(self :: PARAM_TAB => self :: TAB_ALL), array(WeblcmsManager :: PARAM_GROUP));
        $tab_name = Translation :: get('AllSubscriptions');

        $this->tabs->add_tab(
            new DynamicVisualTab(
                self :: TAB_ALL,
                $tab_name,
                Theme :: get_common_image_path() . 'place_users.png',
                $link,
                $this->current_tab == self :: TAB_ALL
            )
        );

        // users tab
        $link = $this->get_url(array(self :: PARAM_TAB => self :: TAB_USERS), array(WeblcmsManager :: PARAM_GROUP));
        $tab_name = Translation :: get('DirectSubscriptions');

        $this->tabs->add_tab(
            new DynamicVisualTab(
                self :: TAB_USERS,
                $tab_name,
                Theme :: get_common_image_path() . 'place_user.png',
                $link,
                $this->current_tab == self :: TAB_USERS
            )
        );

        // groups tab
        $link = $this->get_url(array(self :: PARAM_TAB => self :: TAB_PLATFORM_GROUPS_SUBGROUPS));
        $tab_name = Translation :: get('GroupSubscriptions');
        $selected = $this->current_tab == self :: TAB_PLATFORM_GROUPS_SUBGROUPS ||
            $this->current_tab == self :: TAB_PLATFORM_GROUPS_USERS;

        $this->tabs->add_tab(
            new DynamicVisualTab(
                self :: TAB_PLATFORM_GROUPS_SUBGROUPS,
                $tab_name,
                Theme :: get_common_image_path() . 'place_group.png',
                $link,
                $selected
            )
        );

        // render
        $html[] = $this->tabs->header();
        $html[] = $this->tabs->body_header();

        return implode("\n", $html);
    }

    /**
     * Creates the content of the selected tab.
     *
     * @return String HTML of the content
     */
    private function get_tabs_content()
    {
        switch ($this->current_tab)
        {
            case self :: TAB_ALL :
                return $this->get_all_users_tab();
            case self :: TAB_USERS :
                return $this->get_direct_users_tab();
            case self :: TAB_PLATFORM_GROUPS_SUBGROUPS :
                return $this->get_platformgroups_tab();
            case self :: TAB_PLATFORM_GROUPS_USERS :
                return $this->get_platformgroups_tab();
        }
    }

    /**
     * Creates the footer for the tabs.
     *
     * @return String HTML of the footer
     */
    private function get_tabs_footer()
    {
        $html = array();
        $html[] = $this->tabs->body_footer();
        $html[] = $this->tabs->footer();
        return implode("\n", $html);
    }

    private function get_all_users_tab()
    {
        // $parameters = $this->get_parameters();
        // $parameters[ActionBarSearchForm :: PARAM_SIMPLE_SEARCH_QUERY] =
        // $this->action_bar->get_query();
        $table = new AllSubscribedUserBrowserTable($this);
        return $table->as_html();
    }

    private function get_direct_users_tab()
    {
        // $parameters = $this->get_parameters();
        // $parameters[ActionBarSearchForm :: PARAM_SIMPLE_SEARCH_QUERY] =
        // $this->action_bar->get_query();
        $table = new SubscribedUserBrowserTable($this);
        return $table->as_html();
    }

    private function get_platformgroups_tab()
    {
        $menu_tree = $this->get_menu_tree();
        $html = array();
        if ($menu_tree)
        {
            $html[] = '<div style="float: left; width: 18%; overflow: auto; height: 500px;">';
            $html[] = $menu_tree->render_as_tree();
            $html[] = '</div>';
            $html[] = '<div style="float: right; width: 80%;">';
            $html[] = $this->get_platformgroup_tabs();
            $html[] = '</div>';
        }
        else
        {
            $html[] = Translation :: get('NoGroupsSubscribed');
        }

        return implode("\n", $html);
    }

    // **************************************************************************
    // PLATFORMGROUP TABS FUNCTIONS
    // **************************************************************************
    /**
     * Creates the tab structure.
     *
     * @return String HTML of the tab(s)
     */
    private function get_platformgroup_tabs()
    {
        $html = $this->get_platformgroup_tabs_header();
        $html .= $this->get_platformgroup_tabs_content();
        $html .= $this->get_platformgroup_tabs_footer();
        return $html;
    }

    /**
     * Creates the header for the tabs.
     *
     * @return String HTML of the header
     */
    private function get_platformgroup_tabs_header()
    {
        $html = array();

        $tabs = new DynamicVisualTabsRenderer('weblcms_course_user_platformgroups_browser');

        // subgroups tab
        $link = $this->get_url(array(self :: PARAM_TAB => self :: TAB_PLATFORM_GROUPS_SUBGROUPS));
        if ($this->get_group() != self :: PLATFORM_GROUP_ROOT_ID)
        {
            $tab_name = Translation :: get('Subgroups', null, Utilities :: COMMON_LIBRARIES);
            $tab_selected = $this->current_tab == self :: TAB_PLATFORM_GROUPS_SUBGROUPS;
        }
        else
        {
            $tab_name = Translation :: get('Subscribed');
            $tab_selected = true;
        }
        $tabs->add_tab(
            new DynamicVisualTab(
                self :: TAB_PLATFORM_GROUPS_SUBGROUPS,
                $tab_name,
                Theme :: get_image_path('group') . 'logo/' . Theme :: ICON_MINI . '.png',
                $link,
                $tab_selected
            )
        );

        // no users tab if the root is selected
        if ($this->get_group() != self :: PLATFORM_GROUP_ROOT_ID)
        {
            // users tab
            $link = $this->get_url(array(self :: PARAM_TAB => self :: TAB_PLATFORM_GROUPS_USERS));
            $tab_name = Translation :: get('Users', null, Utilities :: COMMON_LIBRARIES);

            $tabs->add_tab(
                new DynamicVisualTab(
                    self :: TAB_PLATFORM_GROUPS_USERS,
                    $tab_name,
                    Theme :: get_image_path('user') . 'logo/' . Theme :: ICON_MINI . '.png',
                    $link,
                    $this->current_tab == self :: TAB_PLATFORM_GROUPS_USERS
                )
            );
        }
        else
        {
            // reset tab (users tab doesn't exists)
            if ($this->current_tab == self :: TAB_PLATFORM_GROUPS_USERS)
            {
                $this->current_tab = self :: TAB_PLATFORM_GROUPS_SUBGROUPS;
            }
        }

        // render
        $html[] = $tabs->header();
        $html[] = DynamicVisualTabsRenderer :: body_header();

        return implode("\n", $html);
    }

    /**
     * Creates the content of the selected tab.
     *
     * @return String HTML of the content
     */
    private function get_platformgroup_tabs_content()
    {
        switch ($this->current_tab)
        {
            case self :: TAB_PLATFORM_GROUPS_SUBGROUPS :
                return $this->get_platformgroups_subgroups_tab();
            case self :: TAB_PLATFORM_GROUPS_USERS :
                return $this->get_platformgroups_users_tab();
        }
    }

    /**
     * Creates the footer for the tabs.
     *
     * @return String HTML of the footer
     */
    private function get_platformgroup_tabs_footer()
    {
        $html = array();
        $html[] = $this->tabs->body_footer();
        $html[] = $this->tabs->footer();
        return implode("\n", $html);
    }

    private function get_platformgroups_subgroups_tab()
    {
        // build table
        $parameters = $this->get_parameters();
        $parameters[ActionBarSearchForm :: PARAM_SIMPLE_SEARCH_QUERY] = $this->action_bar->get_query();

        if ($this->get_group() != self :: PLATFORM_GROUP_ROOT_ID)
        {
            $table = new SubSubscribedPlatformGroupBrowserTable($this, $this->get_parameters(), $this->get_condition());
        }
        else
        {
            $table =
                new DirectSubscribedPlatformGroupBrowserTable($this, $this->get_parameters(), $this->get_condition());
        }

        return $table->as_html();
    }

    private function get_platformgroups_users_tab()
    {
        $parameters = $this->get_parameters();
        $table = new PlatformGroupRelUserBrowserTable($this, $parameters, $this->get_condition());
        return $table->as_html();
    }

    public function get_menu_tree()
    {
        $root_ids = $this->get_subscribed_platformgroup_ids($this->get_course_id());
        if (count($root_ids) > 0)
        {
            return new PlatformgroupMenuRenderer($this, $root_ids, true);
        }
        return null;
    }

    public function display_users()
    {
        $has_subscribed_groups = $this->get_course()->has_subscribed_groups();
        $html = array();

        if ($has_subscribed_groups)
        {
            $html[] = '<div style="float: left; width: 14%; overflow: auto;">';
            $html[] = $this->get_course_group_menu();
            $html[] = '</div>';
            $html[] = '<div style="float: right; width: 84%;">';
        }

        $html[] = $this->get_user_unsubscribe_html();

        if ($has_subscribed_groups)
        {
            $html[] = '</div>';
            $html[] = '<div class="clear"></div>';
        }

        return implode("\n", $html);
    }

    public function get_user_unsubscribe_html()
    {
        $parameters = $this->get_parameters();
        $table = new SubscribedUserBrowserTable($this, $parameters, $this->get_condition());
        return $table->as_html();
    }

    public function get_course_group_menu()
    {
        $group = Request :: get(WeblcmsManager :: PARAM_GROUP);
        $course_group_user_menu = new CourseGroupUserMenu($this->get_course(), $group);
        return $course_group_user_menu->render_as_tree();
    }

    public function get_group()
    {
        $group = Request :: get(WeblcmsManager :: PARAM_GROUP);
        if (!$group)
        {
            return self :: PLATFORM_GROUP_ROOT_ID;
        }
        return $group;
    }

    public function get_action_bar()
    {
        $action_bar = new ActionBarRenderer(ActionBarRenderer :: TYPE_HORIZONTAL);

        $parameters = array();

        $group_id = Request :: get(WeblcmsManager :: PARAM_GROUP);
        if (isset($group_id))
        {
            $parameters[WeblcmsManager :: PARAM_GROUP] = $group_id;
        }

        if ($this->is_allowed(WeblcmsRights :: EDIT_RIGHT))
        {
            $action_bar->add_common_action(
                new ToolbarItem(
                    Translation :: get('SubscribeUsers'),
                    Theme :: get_image_path() . 'action_subscribe_user.png',
                    $this->get_url(array(UserTool :: PARAM_ACTION => UserTool :: ACTION_SUBSCRIBE_USER_BROWSER)),
                    ToolbarItem :: DISPLAY_ICON_AND_LABEL
                )
            );

            $action_bar->add_common_action(
                new ToolbarItem(
                    Translation :: get('SubscribeGroups'),
                    Theme :: get_image_path() . 'action_subscribe_group.png',
                    $this->get_url(array(UserTool :: PARAM_ACTION => UserTool :: ACTION_SUBSCRIBE_GROUP_BROWSER)),
                    ToolbarItem :: DISPLAY_ICON_AND_LABEL
                )
            );

            $param_export_subscriptions_overview = array();
            $param_export_subscriptions_overview[WeblcmsManager :: PARAM_TOOL] = CourseGroupTool :: TOOL_NAME;
            $param_export_subscriptions_overview[Tool :: PARAM_ACTION] =
                CourseGroupTool :: ACTION_EXPORT_SUBSCRIPTIONS_OVERVIEW;
            $param_export_subscriptions_overview[CourseGroupTool :: PARAM_TAB] = 1;

            $action_bar->add_tool_action(
                new ToolbarItem(
                    Translation :: get('ExportUserList'),
                    Theme :: get_common_image_path() . 'action_backup.png',
                    $this->get_url($param_export_subscriptions_overview),
                    ToolbarItem :: DISPLAY_ICON_AND_LABEL
                )
            );
        }

        $action_bar->set_search_url($this->get_url($parameters));
        $show_all_url = $this->get_url();

        $show_all_item = new ToolbarItem(
            Translation :: get('ShowAll', null, Utilities :: COMMON_LIBRARIES),
            Theme :: get_common_image_path() . 'action_browser.png',
            $show_all_url,
            ToolbarItem :: DISPLAY_ICON_AND_LABEL,
            false
        );

        $action_bar->add_common_action($show_all_item);

        return $action_bar;
    }

    public function get_condition()
    {
        $conditions = array();
        $search_condition = $this->get_search_condition();
        if ($search_condition)
        {
            $conditions[] = $search_condition;
        }

        if ($this->current_tab == self :: TAB_USERS)
        {
            $conditions[] = new EqualityCondition(
                new PropertyConditionVariable(
                    CourseUserRelation :: class_name(), CourseUserRelation :: PROPERTY_COURSE_ID
                ),
                new StaticConditionVariable($this->get_course_id())
            );
        }
        elseif ($this->current_tab == self :: TAB_PLATFORM_GROUPS_SUBGROUPS)
        {
            if ($this->get_group() == self :: PLATFORM_GROUP_ROOT_ID)
            {
                $conditions[] = new EqualityCondition(
                    new PropertyConditionVariable(
                        CourseGroupRelation :: class_name(), CourseGroupRelation :: PROPERTY_COURSE_ID
                    ),
                    new StaticConditionVariable($this->get_course_id())
                );
            }
            else
            {
                $conditions[] = new EqualityCondition(
                    new PropertyConditionVariable(Group :: class_name(), Group :: PROPERTY_PARENT_ID),
                    new StaticConditionVariable(Request :: get(WeblcmsManager :: PARAM_GROUP))
                );
            }
        }
        elseif ($this->current_tab == self :: TAB_PLATFORM_GROUPS_USERS)
        {
            $conditions[] = new EqualityCondition(
                new PropertyConditionVariable(GroupRelUser :: class_name(), GroupRelUser :: PROPERTY_GROUP_ID),
                new StaticConditionVariable(Request :: get(WeblcmsManager :: PARAM_GROUP))
            );
        }
        if ($conditions)
        {
            return new AndCondition($conditions);
        }
        return null;
    }

    public function get_search_condition()
    {
        $query = $this->action_bar->get_query();
        if (isset($query) && $query != '')
        {
            $conditions = array();
            switch ($this->current_tab)
            {
                case self :: TAB_ALL :
                case self :: TAB_USERS :
                    $storage_unit = \user\DataManager :: get_instance()->get_alias(User :: get_table_name());
                    $conditions[] = $this->action_bar->get_conditions(
                        array(
                            new ConditionProperty(User :: PROPERTY_OFFICIAL_CODE, $storage_unit, true),
                            new ConditionProperty(User :: PROPERTY_LASTNAME, $storage_unit, true),
                            new ConditionProperty(User :: PROPERTY_FIRSTNAME, $storage_unit, true),
                            new ConditionProperty(User :: PROPERTY_USERNAME, $storage_unit, true),
                            new ConditionProperty(User :: PROPERTY_EMAIL, $storage_unit, true)
                        )
                    );
                    break;
                case self :: TAB_PLATFORM_GROUPS_SUBGROUPS :
                    $conditions[] = new PatternMatchCondition(
                        Group :: PROPERTY_NAME,
                        '*' . $query . '*',
                        GroupDataManager :: get_instance()->get_alias(Group :: get_table_name()),
                        true
                    );
                    $conditions[] = new PatternMatchCondition(
                        Group :: PROPERTY_CODE,
                        '*' . $query . '*',
                        GroupDataManager :: get_instance()->get_alias(Group :: get_table_name()),
                        true
                    );
                    $conditions[] = new PatternMatchCondition(
                        Group :: PROPERTY_DESCRIPTION,
                        '*' . $query . '*',
                        GroupDataManager :: get_instance()->get_alias(Group :: get_table_name()),
                        true
                    );
                    break;
                case self :: TAB_PLATFORM_GROUPS_USERS :
                    return null;
            }
            return new OrCondition($conditions);
        }
        return null;
    }

    public function get_additional_parameters()
    {
        $parameters = array();
        $parameters[] = self :: PARAM_TAB;

        $current_tab = self :: TAB_ALL;
        if (Request :: get(self :: PARAM_TAB))
        {
            $current_tab = Request :: get(self :: PARAM_TAB);
        }

        if ($current_tab != self :: TAB_ALL && $current_tab != self :: TAB_USERS)
        {
            $parameters[] = WeblcmsManager :: PARAM_GROUP;
        }

        return $parameters;
    }

    public function add_additional_breadcrumbs(BreadcrumbTrail $breadcrumbtrail)
    {
        $breadcrumbtrail->add_help('weblcms_user_unsubscribe_browser');
    }

    public function get_table_condition($object_table_class_name)
    {
        return $this->get_condition();
    }

    public function get_parameters()
    {
        $parameters = parent :: get_parameters();
        $parameters[ActionBarSearchForm :: PARAM_SIMPLE_SEARCH_QUERY] = $this->action_bar->get_query();
        return $parameters;
    }

    public function is_course_admin($user)
    {
        return $this->get_course()->is_course_admin($user);
    }

}
