<?php
namespace application\weblcms\tool\wiki;

use application\weblcms\ContentObjectPublication;
use application\weblcms\Tool;
use application\weblcms\WeblcmsRights;
use common\libraries\DelegateComponent;
use common\libraries\Translation;
use common\libraries\Request;
use common\libraries\Utilities;
use repository\content_object\wiki\WikiComplexDisplaySupport;
use repository\ComplexDisplay;

/**
 * $Id: wiki_viewer.class.php 216 2009-11-13 14:08:06Z kariboe $
 *
 * @package application.lib.weblcms.tool.wiki.component
 */
class WikiToolComplexDisplayComponent extends WikiTool implements DelegateComponent, WikiComplexDisplaySupport
{

    private $publication;

    public function run()
    {
        $publication_id = Request :: get(Tool :: PARAM_PUBLICATION_ID);
        $this->set_parameter(Tool :: PARAM_PUBLICATION_ID, $publication_id);

        $this->publication = \application\weblcms\DataManager :: retrieve(
            ContentObjectPublication :: class_name(),
            $publication_id);

        if (! $this->is_allowed(WeblcmsRights :: VIEW_RIGHT, $this->publication))
        {
            $this->redirect(
                Translation :: get("NotAllowed", null, Utilities :: COMMON_LIBRARIES),
                true,
                array(),
                array(Tool :: PARAM_ACTION, Tool :: PARAM_PUBLICATION_ID));
        }

        ComplexDisplay :: launch($this->publication->get_content_object()->get_type(), $this);
    }

    public function get_root_content_object()
    {
        return $this->publication->get_content_object();
    }

    public function get_publication()
    {
        return $this->publication;
    }

    public function get_additional_parameters()
    {
        return array(Tool :: PARAM_PUBLICATION_ID);
    }

    public function get_wiki_page_statistics_reporting_template_name()
    {
        return \application\weblcms\integration\reporting\WikiPageTemplate :: class_name();
    }

    public function get_wiki_statistics_reporting_template_name()
    {
        return \application\weblcms\integration\reporting\WikiTemplate :: class_name();
    }

    // METHODS FOR COMPLEX DISPLAY RIGHTS
    public function is_allowed_to_edit_content_object()
    {
        return $this->is_allowed(WeblcmsRights :: VIEW_RIGHT, $this->publication);
    }

    public function is_allowed_to_view_content_object()
    {
        return $this->is_allowed(WeblcmsRights :: VIEW_RIGHT, $this->publication);
    }

    public function is_allowed_to_add_child()
    {
        return $this->is_allowed(WeblcmsRights :: VIEW_RIGHT, $this->publication);
    }

    public function is_allowed_to_delete_child()
    {
        return $this->is_allowed(WeblcmsRights :: EDIT_RIGHT, $this->publication);
    }

    public function is_allowed_to_delete_feedback()
    {
        return $this->is_allowed(WeblcmsRights :: EDIT_RIGHT, $this->publication);
    }

    public function is_allowed_to_edit_feedback()
    {
        return $this->is_allowed(WeblcmsRights :: EDIT_RIGHT, $this->publication);
    }
}
