<?php

namespace common\extensions\category_manager;

use common\libraries\Translation;
use common\libraries\Request;
use common\libraries\BreadcrumbTrail;
use common\libraries\Breadcrumb;
use common\libraries\EqualityCondition;

/**
 * Component that toggles the visibility of categories.
 * @author Tom Goethals
 */
class CategoryManagerVisibilityTogglerComponent extends CategoryManagerComponent
{
    /**
     * Toggles the visibility of the category passed by id, if possible.
     * If no category is found or the user has insufficient rights, an error is shown.
     * Otherwise, the change is made and the user is redirected to the previous page.
     */
    public function run()
    {
        $ids = Request :: get(CategoryManager :: PARAM_CATEGORY_ID);

        if (! $this->get_user())
        {
            $this->display_header();
            Display :: error_message(Translation :: get("NotAllowed"));
            $this->display_footer();
            exit();
        }
        
        /*foreach ($ids as $id)
        {
            if (! $this->browser->allowed_to_change_category_visibility($id))
            {
                $this->display_header();
                Display :: error_message(Translation :: get("NotAllowed"));
                $this->display_footer();
                exit();
            }
        }*/

        if ($ids)
        {
            if (! is_array($ids))
            {
                $ids = array($ids);
            }

            $bool = true;

            foreach ($ids as $id)
            {
                $bool = $this->toggle_category_visibility($id);
            }

            if (count($ids) == 1)
            {
                $message = $bool ? 'CategoryVisibilityToggled' : 'CategoryVisibilityNotToggled';
            }
            else
            {
                $message = $bool ? 'CategoriesVisibilityToggled' : 'CategoriesVisibilityNotToggled';
            }

            $this->redirect(Translation :: get($message), ($bool ? false : true), array(
                    CategoryManager :: PARAM_ACTION => CategoryManager :: ACTION_BROWSE_CATEGORIES,
                    CategoryManager :: PARAM_CATEGORY_ID => $this->redirect_to_parent));
        }
        else
        {
            $trail = BreadcrumbTrail :: get_instance();
            $trail->add_help('category_manager_visibility_toggler');
            $trail->add(new Breadcrumb($this->get_url(array(
                    CategoryManager :: PARAM_ACTION => CategoryManager :: ACTION_BROWSE_CATEGORIES)), Translation :: get('CategoryManagerBrowserComponent')));
            $this->set_parameter(CategoryManager :: PARAM_CATEGORY_ID, Request :: get(CategoryManager :: PARAM_CATEGORY_ID));
            $trail->add(new Breadcrumb($this->get_url(), Translation :: get('CategoryManagerVisibilityTogglerComponent')));

            $this->display_header();
            echo Translation :: get("NoObjectSelected");
            $this->display_footer();
        }
    }
    
    /**
     * Toggles the visibility on a category, if the category supports visibility toggling.
     * @param id The id of the category
     * @return True if the visibility has been succesfully changed, false otherwise.
     */
    public function toggle_category_visibility($id)
    {
        /*$continue = true;
        $children = $this->retrieve_categories(new EqualityCondition(PlatformCategory :: PROPERTY_PARENT, $id));
        foreach ($children as $child)
        {
            $continue = $this->toggle_categories_visibility_recursive($child);
        }*/

        //if ($continue)
        //{
            $categories = $this->retrieve_categories(new EqualityCondition(PlatformCategory :: PROPERTY_ID, $id));
            $category = $categories->next_result();
            if ($category instanceof CategoryVisibilitySupported) 
            {
                if ($this->allowed_to_change_category_visibility($id))
                {
                    $this->redirect_to_parent = $category->get_parent();
                    $category->toggle_visibility();
                    return $category->update();
                }
            }
        //}
        return false;
    }
}
