<?php
namespace common\extensions\category_manager;
use common\libraries\ObjectTable;
use common\libraries\Utilities;
use common\libraries\ObjectTableFormActions;
use common\libraries\ObjectTableFormAction;
use common\libraries\Translation;
use common\libraries\Request;
/**
 * $Id: category_browser_table.class.php 205 2009-11-13 12:57:33Z vanpouckesven $
 * @package application.common.category_manager.component.category_browser
 */
/**
 * Table to display a set of learning objects.
 */
class CategoryBrowserTable extends ObjectTable
{
    const DEFAULT_NAME = 'category_table';

    /**
     * Constructor
     * @see ContentObjectTable::ContentObjectTable()
     */
    public function __construct($browser, $parameters, $condition)
    {
        $model = new CategoryBrowserTableColumnModel($browser);
        $renderer = new CategoryBrowserTableCellRenderer($browser);
        $data_provider = new CategoryBrowserTableDataProvider($browser, $condition);
        parent :: __construct($data_provider, Utilities :: get_classname_from_namespace(__CLASS__, true), $model, $renderer);
        $this->set_additional_parameters($parameters);
        
        $actions = new ObjectTableFormActions(__NAMESPACE__, CategoryManager :: PARAM_ACTION);

        if($browser->supports_impact_view())
        {
            $actions->add_form_action(new ObjectTableFormAction(CategoryManager :: ACTION_IMPACT_VIEW,
                    Translation :: get('RemoveSelected', null, Utilities :: COMMON_LIBRARIES), false));
        }
        else
        {
            $actions->add_form_action(new ObjectTableFormAction(CategoryManager :: ACTION_DELETE_CATEGORY,
                    Translation :: get('RemoveSelected', null, Utilities :: COMMON_LIBRARIES)));
        }
        
        if ($browser->get_subcategories_allowed())
        {
            $actions->add_form_action(new ObjectTableFormAction(CategoryManager :: ACTION_CHANGE_CATEGORY_PARENT, Translation :: get('MoveSelected', null, Utilities :: COMMON_LIBRARIES), false));
        }
        
        //add check for VisibilitSupported marker interface
        if ($browser->category_visibility_supported()) {
            $actions->add_form_action(new ObjectTableFormAction(CategoryManager :: ACTION_TOGGLE_CATEGORY_VISIBILITY, Translation :: get('ToggleVisibility', null, Utilities :: COMMON_LIBRARIES)));
        }
        
        $this->set_form_actions($actions);
        
        $this->set_default_row_count(20);
    }

    public function handle_table_action()
    {
        $class = Utilities :: get_classname_from_namespace(__CLASS__, true);
        $ids = self :: get_selected_ids($class);
        Request :: set_get(CategoryManager :: PARAM_CATEGORY_ID, $ids);
    }
}
