<?php
namespace common\extensions\external_repository_manager\implementation\google_docs;

use repository\content_object\document\Document;
use common\extensions\external_repository_manager\Component;
use common\libraries\Request;
use common\libraries\Translation;
use common\libraries\StringUtilities;
use common\libraries\PlatformSetting;
use common\libraries\Filesystem;
use common\libraries\Application;
use common\libraries\Utilities;
use repository\ContentObject;
use repository\ExternalSync;
use repository\RepositoryManager;
use repository\content_object\file\File;

class ImporterComponent extends Manager
{

    public function run()
    {
        Component :: launch($this);
    }

    public function import_external_repository_object($external_object)
    {
        if ($external_object->is_importable())
        {
            $export_format = Request :: get(Manager :: PARAM_EXPORT_FORMAT);

            if (! in_array($export_format, $external_object->get_export_types()))
            {
                $export_format = 'pdf';
            }

            $document = ContentObject :: factory(
                    (ContentObject :: is_available('repository\content_object\document\Document') ? Document :: class_name() : File :: class_name()));
            $document->set_title($external_object->get_title());

            if (PlatformSetting :: get('description_required', 'repository') && StringUtilities :: is_null_or_empty(
                    $external_object->get_description()))
            {
                $document->set_description('-');
            }
            else
            {
                $document->set_description($external_object->get_description());
            }

            $document->set_owner_id($this->get_user_id());
            $document->set_filename(
                    Filesystem :: create_safe_name($external_object->get_title()) . '.' . $export_format);

            $document->set_in_memory_file($external_object->get_content_data($export_format));
            // $document->set_in_memory_file($this->get_external_repository_manager_connector()->download_external_repository_object($external_object,
            // $export_format));

            if ($document->create())
            {
                ExternalSync :: quicksave($document, $external_object, $this->get_external_repository()->get_id());

                $parameters = $this->get_parameters();
                $parameters[Application :: PARAM_ACTION] = RepositoryManager :: ACTION_BROWSE_CONTENT_OBJECTS;
                $this->redirect(Translation :: get('ObjectImported', null, Utilities :: COMMON_LIBRARIES), false,
                        $parameters,
                        array(Manager :: PARAM_EXTERNAL_REPOSITORY, Manager :: PARAM_EXTERNAL_REPOSITORY_MANAGER_ACTION));
            }
            else
            {
                $parameters = $this->get_parameters();
                $parameters[Manager :: PARAM_EXTERNAL_REPOSITORY_MANAGER_ACTION] = Manager :: ACTION_VIEW_EXTERNAL_REPOSITORY;
                $parameters[Manager :: PARAM_EXTERNAL_REPOSITORY_ID] = $external_object->get_id();
                $this->redirect(Translation :: get('ObjectFailedImported', null, Utilities :: COMMON_LIBRARIES), true,
                        $parameters);
            }
        }
        else
        {
            $parameters = $this->get_parameters();
            $parameters[Manager :: PARAM_EXTERNAL_REPOSITORY_MANAGER_ACTION] = Manager :: ACTION_VIEW_EXTERNAL_REPOSITORY;
            $parameters[Manager :: PARAM_EXTERNAL_REPOSITORY_ID] = $external_object->get_id();
            $this->redirect(null, false, $parameters);
        }
    }
}
