<?php
namespace common\extensions\external_repository_manager\implementation\vimeo;

use repository\content_object\document\Document;
use repository\ContentObject;
use common\extensions\external_repository_manager\ExternalRepositoryRenderer;
use common\libraries\Request;
use common\libraries\Translation;
use common\libraries\ActionBarSearchForm;
use common\libraries\PatternMatchCondition;
use common\libraries\OrCondition;
use repository\ExternalSetting;
use repository\content_object\file\File;

/**
 *
 * @author Hans De Bisschop
 */
class Manager extends \common\extensions\external_repository_manager\Manager
{
    const REPOSITORY_TYPE = 'vimeo';
    const PARAM_FEED_TYPE = 'feed';
    const FEED_TYPE_GENERAL = 1;
    const FEED_TYPE_MY_PHOTOS = 2;

    /**
     *
     * @param $application Application
     */
    public function __construct($external_repository, $application)
    {
        parent :: __construct($external_repository, $application);
        $this->set_parameter(self :: PARAM_FEED_TYPE, Request :: get(self :: PARAM_FEED_TYPE));
    }

    /*
     * (non-PHPdoc) @see common/extensions/external_repository_manager/Manager#validate_settings()
     */
    public function validate_settings($external_repository)
    {
        $key = ExternalSetting :: get('consumer_key', $external_repository->get_id());
        $secret = ExternalSetting :: get('consumer_secret', $external_repository->get_id());

        if (! $key || ! $secret)
        {
            return false;
        }
        return true;
    }

    /*
     * (non-PHPdoc) @see common/extensions/external_repository_manager/Manager#support_sorting_direction()
     */
    public function support_sorting_direction()
    {
        return true;
    }

    /**
     *
     * @param \common\extensions\external_repository_manager\ExternalObject $object
     * @return string
     */
    public function get_external_repository_object_viewing_url($object)
    {
        $parameters = array();
        $parameters[self :: PARAM_EXTERNAL_REPOSITORY_MANAGER_ACTION] = self :: ACTION_VIEW_EXTERNAL_REPOSITORY;
        $parameters[self :: PARAM_EXTERNAL_REPOSITORY_ID] = $object->get_id();

        return $this->get_url($parameters);
    }

    /*
     * (non-PHPdoc) @see common/extensions/external_repository_manager/Manager#get_menu_items()
     */
    public function get_menu_items()
    {
        $menu_items = array();

        $my_photos = array();
        $my_photos['title'] = Translation :: get('MyVideos');
        $my_photos['url'] = $this->get_url(
            array(self :: PARAM_FEED_TYPE => self :: FEED_TYPE_MY_PHOTOS),
            array(ActionBarSearchForm :: PARAM_SIMPLE_SEARCH_QUERY));
        $my_photos['class'] = 'user';
        $menu_items[] = $my_photos;

        $general = array();
        $general['title'] = Translation :: get('Public');
        $general['url'] = $this->get_url(
            array(self :: PARAM_FEED_TYPE => self :: FEED_TYPE_GENERAL),
            array(ActionBarSearchForm :: PARAM_SIMPLE_SEARCH_QUERY));
        $general['class'] = 'home';
        $menu_items[] = $general;

        // $most_recent = array();
        // $most_recent['title'] = Translation :: get('MostRecent');
        // $most_recent['url'] = $this->get_url(array(self :: PARAM_FEED_TYPE =>
        // self :: FEED_TYPE_MOST_RECENT),
        // array(ActionBarSearchForm :: PARAM_SIMPLE_SEARCH_QUERY));
        // $most_recent['class'] = 'recent';
        // $menu_items[] = $most_recent;
        //
        // $most_interesting = array();
        // $most_interesting['title'] = Translation :: get('MostInteresting');
        // $most_interesting['url'] = $this->get_url(array(self ::
        // PARAM_FEED_TYPE => self ::
        // FEED_TYPE_MOST_INTERESTING), array(ActionBarSearchForm ::
        // PARAM_SIMPLE_SEARCH_QUERY));
        // $most_interesting['class'] = 'interesting';
        // $menu_items[] = $most_interesting;

        return $menu_items;
    }

    /*
     * (non-PHPdoc) @see common/extensions/external_repository_manager/Manager#is_ready_to_be_used()
     */
    public function is_ready_to_be_used()
    {
        return false;
    }

    /*
     * (non-PHPdoc) @see common/extensions/external_repository_manager/Manager#get_external_repository_actions()
     */
    public function get_external_repository_actions()
    {
        $actions = array(
            self :: ACTION_BROWSE_EXTERNAL_REPOSITORY,
            self :: ACTION_UPLOAD_EXTERNAL_REPOSITORY,
            self :: ACTION_EXPORT_EXTERNAL_REPOSITORY);

        $is_platform = $this->get_user()->is_platform_admin() && (count(
            ExternalSetting :: get_all($this->get_external_repository()->get_id())) > 0);

        if ($is_platform)
        {
            $actions[] = self :: ACTION_CONFIGURE_EXTERNAL_REPOSITORY;
        }

        return $actions;
    }

    /*
     * (non-PHPdoc) @see common/extensions/external_repository_manager/Manager#get_available_renderers()
     */
    public function get_available_renderers()
    {
        return array(
            ExternalRepositoryRenderer :: TYPE_GALLERY,
            ExternalRepositoryRenderer :: TYPE_SLIDESHOW,
            ExternalRepositoryRenderer :: TYPE_TABLE);
    }

    /*
     * (non-PHPdoc) @see common/extensions/external_repository_manager/Manager#get_content_object_type_conditions()
     */
    public function get_content_object_type_conditions()
    {
        if (ContentObject :: is_available('repository\content_object\document\Document'))
        {
            $video_types = Document :: get_video_types();
            $video_conditions = array();
            foreach ($video_types as $video_type)
            {
                $video_conditions[] = new PatternMatchCondition(
                    Document :: PROPERTY_FILENAME,
                    '*.' . $video_type,
                    Document :: get_type_name());
            }
        }
        else
        {
            $video_types = File :: get_video_types();
            $video_conditions = array();
            foreach ($video_types as $video_type)
            {
                $video_conditions[] = new PatternMatchCondition(
                    File :: PROPERTY_FILENAME,
                    '*.' . $video_type,
                    File :: get_type_name());
            }
        }

        return new OrCondition($video_conditions);
    }

    /**
     *
     * @return string
     */
    public function get_repository_type()
    {
        return self :: REPOSITORY_TYPE;
    }

    /**
     * Helper function for the SubManager class, pending access to class constants via variables in PHP 5.3 e.g. $name =
     * $class :: DEFAULT_ACTION DO NOT USE IN THIS SUBMANAGER'S CONTEXT Instead use: - self :: DEFAULT_ACTION in the
     * context of this class - YourSubManager :: DEFAULT_ACTION in all other application classes
     */
    public static function get_default_action()
    {
        return self :: DEFAULT_ACTION;
    }

    /**
     * Helper function for the SubManager class, pending access to class constants via variables in PHP 5.3 e.g. $name =
     * $class :: PARAM_ACTION DO NOT USE IN THIS SUBMANAGER'S CONTEXT Instead use: - self :: PARAM_ACTION in the context
     * of this class - YourSubManager :: PARAM_ACTION in all other application classes
     */
    public static function get_action_parameter()
    {
        return self :: PARAM_EXTERNAL_REPOSITORY_MANAGER_ACTION;
    }
}
