<?php
namespace common\extensions\feedback_manager;

use common\libraries\DatetimeUtilities;
use common\libraries\Path;
use common\libraries\Theme;
use common\libraries\ToolbarItem;
use core\admin\AdminDataManager;
use common\libraries\Translation;
use common\libraries\Utilities;

/**
 * $Id: browser.class.php 191 2009-11-13 11:50:28Z chellee $
 *
 * @package application.common.feedback_manager.component
 */

/**
 * Description of browserclass
 *
 * @author Pieter Hens
 */

class FeedbackManagerBrowserComponent extends FeedbackManager
{

    const TITLE_MARKER = '<!-- /title -->';
    const DESCRIPTION_MARKER = '<!-- /description -->';

    private $html;

    public function run()
    {
        $html = $this->as_html();

        $this->display_header();
        echo $html;
        $this->display_footer();
    }

    public function as_html()
    {
        $application = $this->get_application();
        $publication_id = $this->get_publication_id();
        $complex_wrapper_id = $this->get_complex_wrapper_id();
        $action = $this->get_action();
        $html = array();

        $form = new FeedbackManagerForm($this->get_url());

        if ($form->validate())
        {
            $success =
                $form->create_feedback($this->get_user()->get_id(), $publication_id, $complex_wrapper_id, $application);
            $this->redirect(
                Translation :: get(
                    $success ? 'ObjectCreated' : 'ObjectNotCreated', array(
                        'OBJECT' => Translation :: get('Feedback')
                    ), Utilities :: COMMON_LIBRARIES
                ), !$success );
        }
        else
        {
            $can_create_feedback = $this->can_create_feedback();

            if ($can_create_feedback)
            {
                $html[] = '<h3>' . Translation :: get('Feedback') . '</h3>';
                $this->render_create_action();
            }

            $feedbackpublications =
                $this->retrieve_feedback_publications($publication_id, $complex_wrapper_id, $application);
            $feedback_count = AdminDataManager :: get_instance()->count_feedback_publications(
                $publication_id, $complex_wrapper_id, $application
            );

            $counter = 0;
            while ($feedback = $feedbackpublications->next_result())
            {
                $counter ++;

                if ($counter == 4)
                {
                    $html[] = '<br /><a href="#" id="showfeedback" style="display:none; float:left;">' .
                        Translation :: get('ShowAllFeedback') . '[' . ($feedback_count - 3) . ']</a><br><br>';
                    $html[] =
                        '<a href="#" id="hidefeedback" style="display:none; font-size: 80%; font-weight: normal;">(' .
                            Translation :: get('HideAllFeedback') . ')</a>';
                    $html[] = '<div id="feedbacklist">';
                }
                $html[] = $this->render_feedback($feedback);
            }

            if ($counter > 3)
            {
                $html[] = '</div>';
            }

            if ($can_create_feedback)
            {
                $html[] = $form->toHtml();
            }

            $html[] = '<script type="text/javascript" src="' . Path :: namespace_to_full_path(
                Utilities::COMMON_LIBRARIES, true
            ) . '/resources/javascript/feedback_list.js' . '"></script>';
        }

        $this->html = $html;

        return implode("\n", $this->html);
    }

    public function render_feedback($feedback)
    {

        $id = $feedback->get_fid();
        $feedback_object = \repository\DataManager :: retrieve_content_object($id);
        $html = array();
        $html[] =
            '<div class="content_object" style="background-image: url(' . $feedback_object->get_icon_path() . ');">';
        $html[] = '<div class="title">' . Utilities :: htmlentities($feedback_object->get_title());
        $html[] = '<span class="publication_info">';
        $html[] = $this->render_publication_information($feedback_object);
        $html[] = '</span>';
        $html[] = '</div>';
        $html[] = self :: TITLE_MARKER;
        $html[] = $this->get_description($feedback_object);

        if ($this->get_user()->get_id() == $feedback_object->get_owner_id())
        {
            $html[] = '<div class="publication_actions">';

            if ($this->can_delete_feedback($feedback))
            {
                $html[] = $this->render_delete_action($feedback);
            }

            if ($this->can_update_feedback($feedback))
            {
                $html[] = $this->render_update_action($feedback);
            }

            $html[] = '</div>';
        }
        else
        {
            if ($this->has_delete_right())
            {
                $html[] = '<div class="publication_actions">';
                $html[] = $this->render_delete_action($feedback);
                $html[] = '</div>';
            }
        }

        $html[] = '</div>';

        return implode("\n", $html);
    }

    public function get_description($feedback)
    {
        $feedback_description_renderer = new FeedbackDescriptionRenderer($this, $feedback);

        return $feedback_description_renderer->render();
    }

    public function render_delete_action($feedback)
    {
        $delete_url = $this->get_url(
            array(
                FeedbackManager :: PARAM_ACTION => FeedbackManager :: ACTION_DELETE_FEEDBACK,
                FeedbackManager :: PARAM_FEEDBACK_ID => $feedback->get_id()
            )
        );

        $title = Translation :: get('Delete', null, Utilities::COMMON_LIBRARIES);

        $delete_link = '<a href="' . $delete_url . '" onclick="return confirm(\'' .
            addslashes(Translation :: get('Confirm', null, Utilities :: COMMON_LIBRARIES)) . '\');"><img src="' .
            Theme :: get_common_image_path() . 'action_delete.png"  alt="' .
            $title . '" title="' . $title . '"/></a>';

        return $delete_link;
    }

    public function render_update_action($feedback)
    {
        $update_url = $this->get_url(
            array(
                FeedbackManager :: PARAM_ACTION => FeedbackManager :: ACTION_UPDATE_FEEDBACK,
                FeedbackManager :: PARAM_FEEDBACK_ID => $feedback->get_id()
            )
        );

        $title = Translation :: get('Edit', null, Utilities::COMMON_LIBRARIES);
        $update_link =
            '<a href="' . $update_url . '"><img src="' . Theme :: get_common_image_path() . 'action_edit.png"  alt="' .
                $title . '" title="' . $title . '"/></a>';

        return $update_link;
    }

    public function render_create_action()
    {
        $create_url = $this->get_url(
            array(
                FeedbackManager :: PARAM_ACTION => FeedbackManager :: ACTION_CREATE_FEEDBACK
            )
        );
        $item = new ToolbarItem(Translation :: get('AddFeedback', null, Utilities :: COMMON_LIBRARIES),
            Theme :: get_common_image_path() . 'action_create.png', $create_url, ToolbarItem :: DISPLAY_ICON_AND_LABEL);
        $this->get_parent()->add_actionbar_item($item);
    }

    public function render_publication_information($feedback)
    {
        $html = array();
        $html[] = '(';
        $html[] = $feedback->get_owner_fullname();
        $html[] = ' - ';
        $html[] = $this->format_date($feedback->get_creation_date());
        $html[] = ')';

        return implode("\n", $html);
    }

    public function format_date($date)
    {
        $date_format = Translation :: get('DateTimeFormatLong', null, Utilities :: COMMON_LIBRARIES);

        return DatetimeUtilities :: format_locale_date($date_format, $date);
    }
}
