<?php
namespace common\extensions\invitation_manager;

use common\libraries\DataClass;
use common\libraries\Utilities;
use core\admin\AdminDataManager;

/**
 * @package admin.lib
 * @author Hans De Bisschop
 */

class Invitation extends DataClass
{
    const CLASS_NAME = __CLASS__;
    
    const PROPERTY_APPLICATION = 'application';
    const PROPERTY_DATE = 'date';
    const PROPERTY_EXPIRATION_DATE = 'expiration_date';
    const PROPERTY_CODE = 'code';
    const PROPERTY_PARAMETERS = 'parameters';
    const PROPERTY_ANONYMOUS = 'anonymous';
    const PROPERTY_TITLE = 'title';
    const PROPERTY_MESSAGE = 'message';
    const PROPERTY_EMAIL = 'email';
    const PROPERTY_RIGHTS_TEMPLATES = 'rights_templates';
    const PROPERTY_USER_CREATED = 'user_created';
    const PROPERTY_RIGHTS_CLASS_METHOD = 'rights_class_method';

    public function get_data_manager()
    {
        return AdminDataManager :: get_instance();
    }

    /**
     * Get the default properties of all users.
     * @return array The property names.
     */
    public static function get_default_property_names()
    {
        return parent :: get_default_property_names(array(self :: PROPERTY_APPLICATION, self :: PROPERTY_DATE, 
                self :: PROPERTY_EXPIRATION_DATE, self :: PROPERTY_CODE, self :: PROPERTY_PARAMETERS, 
                self :: PROPERTY_ANONYMOUS, self :: PROPERTY_TITLE, self :: PROPERTY_MESSAGE, self :: PROPERTY_EMAIL, 
                self :: PROPERTY_RIGHTS_TEMPLATES, self :: PROPERTY_USER_CREATED, self :: PROPERTY_RIGHTS_CLASS_METHOD));
    }

    public function get_application()
    {
        return $this->get_default_property(self :: PROPERTY_APPLICATION);
    }

    public function set_application($application)
    {
        $this->set_default_property(self :: PROPERTY_APPLICATION, $application);
    }

    public function get_date()
    {
        return $this->get_default_property(self :: PROPERTY_DATE);
    }

    public function set_date($date)
    {
        $this->set_default_property(self :: PROPERTY_DATE, $date);
    }

    public function get_expiration_date()
    {
        return $this->get_default_property(self :: PROPERTY_EXPIRATION_DATE);
    }

    public function set_expiration_date($expiration_date)
    {
        $this->set_default_property(self :: PROPERTY_EXPIRATION_DATE, $expiration_date);
    }

    public function get_code()
    {
        return $this->get_default_property(self :: PROPERTY_CODE);
    }

    public function set_code($code)
    {
        $this->set_default_property(self :: PROPERTY_CODE, $code);
    }

    public function get_parameters()
    {
        return $this->get_default_property(self :: PROPERTY_PARAMETERS);
    }

    public function set_parameters($parameters)
    {
        $this->set_default_property(self :: PROPERTY_PARAMETERS, $parameters);
    }

    public function get_anonymous()
    {
        return $this->get_default_property(self :: PROPERTY_ANONYMOUS);
    }

    public function set_anonymous($anonymous)
    {
        $this->set_default_property(self :: PROPERTY_ANONYMOUS, $anonymous);
    }

    public function get_title()
    {
        return $this->get_default_property(self :: PROPERTY_TITLE);
    }

    public function set_title($title)
    {
        $this->set_default_property(self :: PROPERTY_TITLE, $title);
    }

    public function get_message()
    {
        return $this->get_default_property(self :: PROPERTY_MESSAGE);
    }

    public function set_message($message)
    {
        $this->set_default_property(self :: PROPERTY_MESSAGE, $message);
    }

    public function get_email()
    {
        return $this->get_default_property(self :: PROPERTY_EMAIL);
    }

    public function set_email($email)
    {
        $this->set_default_property(self :: PROPERTY_EMAIL, $email);
    }

    public function get_rights_templates()
    {
        return unserialize($this->get_default_property(self :: PROPERTY_RIGHTS_TEMPLATES));
    }

    public function set_rights_templates($rights_templates)
    {
        $this->set_default_property(self :: PROPERTY_RIGHTS_TEMPLATES, serialize($rights_templates));
    }

    public function get_user_created()
    {
        return $this->get_default_property(self :: PROPERTY_USER_CREATED);
    }

    public function set_user_created($user_created)
    {
        $this->set_default_property(self :: PROPERTY_USER_CREATED, serialize($user_created));
    }

    public function get_rights_class_method()
    {
        return unserialize($this->get_default_property(self :: PROPERTY_RIGHTS_CLASS_METHOD));
    }

    public function set_rights_class_method($rights_class_method)
    {
        $this->set_default_property(self :: PROPERTY_RIGHTS_CLASS_METHOD, $rights_class_method);
    }

    public function is_valid()
    {
        return (time() >= $this->get_expiration_date() || $this->get_expiration_date() == 0) && ! $this->get_user_created();
    }

    public function is_anonymous()
    {
        return $this->get_anonymous();
    }

    public static function get_table_name()
    {
        return Utilities :: get_classname_from_namespace(self :: CLASS_NAME, true);
    }

    public function create()
    {
        $this->set_code(md5(uniqid()));
        $this->set_date(time());
        $this->set_user_created(0);
        parent :: create();
    }
}
