<?php
use common\libraries\Path;
use common\libraries\PlatformSetting;
use common\libraries\Utilities;
use common\libraries\Session;
use common\libraries\package\Package;

/**
 * Set up the environment. Do not make any call to the DB - which happen to be somwhat slow. Those are done in
 * conf.inc.php. Recommendation: - standard pages should include global.inc.php - pages requiring a lighter/faster entry
 * point call include this file and make another include to conf later on when this becomes required.
 *
 * @package common
 */

// This path will be useful to include the other intialisation files
require_once __DIR__ . '/libraries/php/shared/utilities.class.php';

// include the main Chamilo platform configuration file
$main_configuration_file_path = __DIR__ . '/configuration/configuration.php';
if (file_exists($main_configuration_file_path))
{
    require_once ($main_configuration_file_path);
}
else
{
    die(Utilities :: get_global_error_message());
}

// Add the path to the pear packages to the include path
require_once __DIR__ . '/libraries/php/shared/filesystem/path.class.php';

ini_set(
    'include_path',
    realpath(Path :: get_plugin_path() . 'pear') . PATH_SEPARATOR . realpath(
        Path :: get_plugin_path() . 'google/library'));

set_exception_handler('\common\libraries\Utilities::handle_exception');
set_error_handler('\common\libraries\Utilities::handle_error');

Utilities :: register_autoload();

// Verify common\libraries versions
$package_info = Package :: get('common\libraries');
$registration = \common\libraries\CommonDataManager :: get_registration('common\libraries');

if ($package_info->get_version() != $registration->get_version())
{
    $theme = \common\libraries\Request :: get('theme');
    $server_type = \common\libraries\Request :: get('server_type');
    $time = \common\libraries\Request :: get('time');

    if (! $theme && ! $server_type && ! $time)
    {
        \common\libraries\Request :: set_get(
            \common\libraries\Application :: PARAM_APPLICATION,
            \core\lynx\Manager :: APPLICATION_NAME);
        \common\libraries\Request :: set_get(
            \common\libraries\Application :: PARAM_ACTION,
            \core\lynx\Manager :: ACTION_UPGRADE);
    }

    $settings = array(
        'platform_language',
        'platform_timezone',
        'institution',
        'site_name',
        'server_type',
        'theme',
        'hide_dcda_markup',
        'session_timeout');

    foreach ($settings as $setting)
    {
        $old_setting = \common\libraries\PlatformSetting :: get($setting, 'admin');
        \common\libraries\PlatformSetting :: get_instance()->set_value($setting, $old_setting);
    }

    $language_interface = $platform_language = \common\libraries\PlatformSetting :: get('platform_language');

    $conditions = array();
    $conditions[] = new \common\libraries\EqualityCondition(
        new \common\libraries\PropertyConditionVariable(
            \common\libraries\Language :: class_name(),
            \common\libraries\Language :: PROPERTY_ISOCODE),
        new \common\libraries\StaticConditionVariable($platform_language));
    $conditions[] = new \common\libraries\EqualityCondition(
        new \common\libraries\PropertyConditionVariable(
            \common\libraries\Language :: class_name(),
            \common\libraries\Language :: PROPERTY_AVAILABLE),
        new \common\libraries\StaticConditionVariable(1));

    $parameters = new \common\libraries\DataClassCountParameters(new \common\libraries\AndCondition($conditions));
    \common\libraries\DataClassCountCache :: set_cache(
        \common\libraries\Language :: class_name(),
        $parameters->hash(),
        1);
}

// Set the default language interface before starting the session because starting the session can trigger a new
// translate call
$language_interface = PlatformSetting :: get('platform_language');

// Start session
Session :: start(true);