<?php
namespace common\libraries\form;

/*
 * Abstract class for an element Each element inherits from this class
 */
abstract class Element
{
    protected $name;
    protected $label;
    public $value;
    protected $elementerror;

    protected $attribute_storage;
    protected $rule_storage;

    private $renderer;
    private $parent;

    private $fully_qualified_name;
    private $fully_qualified_id;

    /*
     * Constructor for an element
     */
    public function __construct($name, $value = null, $label = null)
    {
        $this->name = $name;
        $this->value = $value;
        $this->label = $label;

        $this->attribute_storage = new AttributeStorage();
        $this->rule_storage = new RuleStorage();

        $this->renderer = null;
        $this->parent = null;
    }

    /*
     * Getter for the name of the element
     */
    public function get_name()
    {
        return $this->name;
    }

    public function set_name($name)
    {
        $this->name = $name;
    }

    /*
     * Getter for the label of the element
     */
    public function get_label()
    {
        return $this->label;
    }

    /*
     * Setter for the label of the element
     */
    public function set_label($label)
    {
        $this->label = $label;
    }

    /*
     * Get the attribute storage
     */
    public function get_attribute_storage()
    {
        return $this->attribute_storage;
    }

    /*
     * Function to get the value of an element
     */
    public function get_value()
    {
        if (isset($_POST[$this->name]))
        {
            return $_POST[$this->name];
        }
        else
            return $this->value;
    }

    /*
     * function to set the value of an element
     */
    public function set_value($value)
    {
        $this->value = $value;
    }

    /*
     * Getter for the rule_storage of the element
     */
    public function get_rule_storage()
    {
        return $this->rule_storage;
    }

    /*
     * This function returns the errors that occur when a rule isn't respected
     */
    public function get_errors()
    {
        return $this->elementerror;
    }

    /*
     * Check if value of the element is valid
     */
    public function is_valid()
    {
        $valid = true;

        foreach ($this->get_rule_storage()->get_rules() as $rule)
        {
            $valid = $rule->control($this->get_value());
            if (! $valid)
            {
                $this->elementerror .= $rule->get_message() . '<br/>';
                $valid = false;
                break;
            }
        }

        return $valid;
    }

    public function render()
    {
        $renderer = $this->get_renderer() ? $this->get_renderer() : new DefaultElementRenderer($this);
        return $renderer->render();
    }

    public function is_required()
    {
        foreach ($this->get_rule_storage()->get_rules() as $rule)
        {
            if ($rule instanceof Required)
            {
                return true;
            }
        }

        return false;
    }

    public function set_renderer($render)
    {
        $this->renderer = $render;
    }

    public function get_renderer()
    {
        return $this->renderer;
    }

    public function set_parent($parent)
    {
        $this->parent = $parent;
    }

    public function get_parent()
    {
        return $this->parent;
    }

    public function get_fully_qualified_name()
    {
        if (! isset($this->fully_qualified_name))
        {
            if (! is_null($this->get_parent()))
            {
                $parts = array();
                $parts[] = $this->get_name();

                $parent = $this->get_parent();

                while ($parent)
                {
                    if ($parent instanceof Container && $parent->get_append())
                    {
                        $parts[] = $parent->get_name();
                    }
                    $parent = $parent->get_parent();
                }

                if (count($parts) > 1)
                {
                    $base = array_pop($parts);
                    $this->fully_qualified_name = $base . '[' . implode('][', array_reverse($parts)) . ']';
                }
                else
                {
                    $this->fully_qualified_name = $this->get_name();
                }
            }
            else
            {
                $this->fully_qualified_name = $this->get_name();
            }
        }

        return $this->fully_qualified_name;
    }

    public function get_fully_qualified_id()
    {
        if (! isset($this->fully_qualified_id))
        {
            $this->fully_qualified_id = str_replace(array('[', ']'), array('_', ''), $this->get_fully_qualified_name());
        }

        return $this->fully_qualified_id;
    }
}
