<?php
namespace common\libraries\form;
/*
 * Class to render the elements that were added to the form
 */

abstract class Renderer
{
    const PARAMETER_PATTERN = '#\{([A-Z0-9\-_]+)\}#e';
    const PARAMETER_REPLACEMENT = 'isset($parameters[\'\\1\']) ? $parameters[\'\\1\'] : \'\'';

    /**
     *
     * @var \common\libraries\Element
     */
    protected $element;

    /**
     *
     * @var string
     */
    protected $template;

    /**
     * Constructor
     *
     * @param $element \common\libraries\Element
     * @param $template string
     */
    public function __construct($element, $template)
    {
        $this->element = $element;
        $this->template = $template;
    }

    /**
     *
     * @param $element \common\libraries\Element
     */
    public function set_element($element)
    {
        $this->element = $element;
    }

    /**
     *
     * @return \common\libraries\Element
     */
    public function get_element()
    {
        return $this->element;
    }

    /**
     *
     * @param $template string
     */
    public function set_template($template)
    {
        $this->template = $template;
    }

    /**
     *
     * @return string
     */
    public function get_template()
    {
        return $this->template;
    }

    abstract public function get_default_template();

    abstract public function get_parameters();

    public function render()
    {
        $template = $this->get_template() ? $this->get_template() : $this->get_default_template();
        return self :: process($template, $this->get_parameters());
    }

    public static function process($template, $parameters)
    {
        return preg_replace(self :: PARAMETER_PATTERN, self :: PARAMETER_REPLACEMENT, $template);
    }
}
