<?php
namespace common\libraries;

use user\UserDataManager;
use user\UserSetting;

use common\libraries\Utilities;
use common\libraries\DataClass;
use common\libraries\EqualityCondition;
use common\libraries\SystemCache;
use common\libraries\CacheSynch;
/**
 *
 * @package admin.lib $Id: setting.class.php 168 2009-11-12 11:53:23Z vanpouckesven $
 * @author Hans De Bisschop
 */

class Setting extends DataClass
{
    const CLASS_NAME = __CLASS__;

    /**
     *
     * @deprecated Use PROPERTY_CONTEXT instead
     */
    const PROPERTY_APPLICATION = 'context';

    const PROPERTY_CONTEXT = 'context';
    const PROPERTY_VARIABLE = 'variable';
    const PROPERTY_VALUE = 'value';
    const PROPERTY_USER_SETTING = 'user_setting';

    /**
     * Get the default properties of all settings.
     *
     * @return array The property names.
     */
    public static function get_default_property_names()
    {
        return parent :: get_default_property_names(
                array(self :: PROPERTY_CONTEXT, self :: PROPERTY_VARIABLE, self :: PROPERTY_VALUE,
                        self :: PROPERTY_USER_SETTING));
    }

    /**
     * inherited
     */
    public function get_data_manager()
    {
        return CommonDataManager :: get_instance();
    }

    /**
     * Returns the application of this setting object
     *
     * @return string The setting application
     * @deprecated Use get_context instead
     */
    public function get_application()
    {
        return $this->get_context();
    }

    public function get_context()
    {
        return $this->get_default_property(self :: PROPERTY_CONTEXT);
    }

    /**
     * Returns the variable of this setting object
     *
     * @return string the variable
     */
    public function get_variable()
    {
        return $this->get_default_property(self :: PROPERTY_VARIABLE);
    }

    /**
     * Returns the value of this setting object
     *
     * @return string the value
     */
    public function get_value()
    {
        return $this->get_default_property(self :: PROPERTY_VALUE);
    }

    /**
     * Sets the application of this setting.
     *
     * @param $application string the setting application.
     * @deprecated Use set_context instead
     */
    public function set_application($application)
    {
        $this->set_context($application);
    }

    public function set_context($context)
    {
        $this->set_default_property(self :: PROPERTY_CONTEXT, $context);
    }

    /**
     * Sets the variable of this setting.
     *
     * @param $variable string the variable.
     */
    public function set_variable($variable)
    {
        $this->set_default_property(self :: PROPERTY_VARIABLE, $variable);
    }

    /**
     * Sets the value of this setting.
     *
     * @param $value string the value.
     */
    public function set_value($value)
    {
        $this->set_default_property(self :: PROPERTY_VALUE, $value);
    }

    /**
     * Returns the user_setting of this setting object
     *
     * @return string the user_setting
     */
    public function get_user_setting()
    {
        return $this->get_default_property(self :: PROPERTY_USER_SETTING);
    }

    /**
     * Sets the user_setting of this setting.
     *
     * @param $user_setting string the user_setting.
     */
    public function set_user_setting($user_setting)
    {
        $this->set_default_property(self :: PROPERTY_USER_SETTING, $user_setting);
    }

    public static function get_table_name()
    {
        return Utilities :: get_classname_from_namespace(self :: CLASS_NAME, true);
    }

    public function delete()
    {
        if (! parent :: delete())
        {
            return false;
        }
        else
        {
            if ($this->get_user_setting())
            {
                $condition = new EqualityCondition(UserSetting :: PROPERTY_SETTING_ID, $this->get_id());
                if (! UserDataManager :: get_instance()->delete_user_settings($condition))
                {
                    return false;
                }
                else
                {
                    $this->on_change();
                    return true;
                }
            }
            else
            {
                $this->on_change();
                return true;
            }
        }
    }

    public function create()
    {
        return $this->on_change(parent :: create());
    }

    public function update()
    {
        return $this->on_change(parent :: update());
    }

    protected function on_change($success = true)
    {
        if (! $success)
        {
            return $success;
        }

        if (CacheSynch :: VARIABLE_NAME == $this->get_variable())
        {
            return $success;
        }
        SystemCache :: reset_system(true);

        return $success;

    }
}
