<?php
namespace common\libraries;

class CommonDataManager extends DataManager
{

    public static $registrations;
    const REGISTRATION_CONTEXT = 1;
    const REGISTRATION_TYPE = 2;

    /**
     * Gets the type of DataManager to be instantiated
     *
     * @return string
     */
    public static function get_type()
    {
        return 'doctrine';
    }

    public static function is_registered($context)
    {
        $registration = self :: get_registration($context);
        return ($registration instanceof Registration);
    }

    /**
     *
     * @param $registration Registration
     */
    public static function update_registration_cache($registration)
    {
        self :: $registrations[self :: REGISTRATION_TYPE][$registration->get_type()][$registration->get_context()] = $registration;
        self :: $registrations[self :: REGISTRATION_CONTEXT][$registration->get_context()] = $registration;
    }

    /**
     *
     * @param \common\libraries\Registration $context
     */
    public static function get_registration($context)
    {
        if (! self :: $registrations)
        {
            self :: init_registrations();
        }

        return self :: $registrations[self :: REGISTRATION_CONTEXT][$context];
    }

    public static function get_registrations()
    {
        return self :: $registrations;
    }

    /**
     *
     * @param string $type
     * @return multitype:\common\libraries\Registration
     */
    public static function get_registrations_by_type($type)
    {
        if (! self :: $registrations)
        {
            self :: init_registrations();
        }

        return self :: $registrations[self :: REGISTRATION_TYPE][$type];
    }

    public static function reset_registrations()
    {
        SystemCache :: system()->admin_data_manager_registrations = null;
        self :: $registrations = array();
        SystemCache :: reset_system();
        DataClassResultSetCache :: truncate(Registration :: class_name());
    }

    public static function init_registrations()
    {
        if ($registrations = SystemCache :: system()->admin_data_manager_registrations)
        {
            self :: $registrations = $registrations;
            return;
        }

        self :: $registrations = array();

        $registrations = self :: retrieves(Registration :: class_name());
        while ($registration = $registrations->next_result())
        {
            self :: $registrations[self :: REGISTRATION_TYPE][$registration->get_type()][$registration->get_context()] = $registration;
            self :: $registrations[self :: REGISTRATION_CONTEXT][$registration->get_context()] = $registration;
        }

        SystemCache :: system()->admin_data_manager_registrations = self :: $registrations;
    }

    public static function retrieve_setting_contexts(Condition $condition = null)
    {
        $parameters = new DataClassDistinctParameters($condition, Setting :: PROPERTY_CONTEXT);
        return self :: distinct(Setting :: class_name(), $parameters);
    }

    /**
     *
     * @param string $variable
     * @param string $context
     * @return \common\libraries\Setting
     */
    public static function retrieve_setting_from_variable_name($variable, $context = 'core\admin')
    {
        $conditions = array();
        $conditions[] = new EqualityCondition(Setting :: PROPERTY_CONTEXT, $context);
        $conditions[] = new EqualityCondition(Setting :: PROPERTY_VARIABLE, $variable);
        $condition = new AndCondition($conditions);

        return self :: retrieve(Setting :: class_name(), $condition);
    }

    public static function is_language_active($isocode)
    {
        $conditions = array();
        $conditions[] = new EqualityCondition(
            new PropertyConditionVariable(Language :: class_name(), Language :: PROPERTY_ISOCODE),
            new StaticConditionVariable($isocode));
        $conditions[] = new EqualityCondition(
            new PropertyConditionVariable(Language :: class_name(), Language :: PROPERTY_AVAILABLE),
            new StaticConditionVariable(1));

        $parameters = new DataClassCountParameters(new AndCondition($conditions));

        return self :: count(Language :: class_name(), $parameters) == 1;
    }

    public static function retrieve_language_from_english_name($english_name)
    {
        $condition = new EqualityCondition(
            new PropertyConditionVariable(Language :: class_name(), Language :: PROPERTY_ENGLISH_NAME),
            new StaticConditionVariable($english_name));
        $parameters = new DataClassRetrieveParameters($condition);
        return self :: retrieve(Language :: class_name(), $parameters);
    }

    public static function retrieve_language_from_isocode($isocode)
    {
        $condition = new EqualityCondition(
            new PropertyConditionVariable(Language :: class_name(), Language :: PROPERTY_ISOCODE),
            new StaticConditionVariable($isocode)
        );

        $parameters = new DataClassRetrieveParameters($condition);

        return self :: retrieve(Language :: class_name(), $parameters);
    }

    public static function get_languages($use_folder_name_as_key = true)
    {
        $options = array();

        $languages = self :: retrieves(Language :: class_name());
        while ($language = $languages->next_result())
        {
            if ($use_folder_name_as_key)
            {
                $key = $language->get_isocode();
            }
            else
            {
                $key = $language->get_id();
            }
            $options[$key] = $language->get_original_name();
        }

        return $options;
    }
}
