<?php
namespace common\libraries;

/**
 * A case condition variable that describes a case in a select query
 *
 * @author Sven Vanpoucke - Hogeschool Gent
 *
 * @package common.libraries
 */
class CaseConditionVariable extends ConditionVariable
{
    /**
     * The case_elements name of the DataClass object
     *
     * @var string
     */
    private $case_elements;

    /**
     * The alias of the case
     *
     * @var string
     */
    private $alias;

    /**
     * Constructor
     *
     * @param CaseElementConditionVariable[] $case_elements
     * @param string $alias
     */
    public function __construct($case_elements = array(), $alias = null)
    {
        $this->case_elements = $case_elements;
        $this->alias = $alias;
    }

    /**
     * Get the case_elements
     *
     * @return string
     */
    public function get_case_elements()
    {
        return $this->case_elements;
    }

    /**
     * Set the case_elements
     *
     * @param string $case_elements
     */
    public function set_case_elements($case_elements)
    {
        $this->case_elements = $case_elements;
    }

    /**
     * Adds a case element to the case elements
     *
     * @param CaseElementConditionVariable $case_element
     */
    public function add(CaseElementConditionVariable $case_element)
    {
        $this->case_elements[] = $case_element;
    }

    /**
     * Get the alias
     *
     * @return string
     */
    public function get_alias()
    {
        return $this->alias;
    }

    /**
     * Set the alias
     *
     * @param string $alias
     */
    public function set_alias($alias)
    {
        $this->alias = $alias;
    }

    /**
     * Get an md5 representation of this object for identification purposes
     *
     * @param string[] $hash_parts
     *
     * @return string
     */
    public function hash($hash_parts = array())
    {
        foreach($this->case_elements as $case_element)
        {
            $hash_parts[] = $case_element->hash();
        }

        sort($hash_parts);

        $hash_parts[] = $this->alias;

        return parent :: hash($hash_parts);
    }
}
