<?php
namespace common\libraries;

/**
 * A ConditionVariable that describes a function on another ConditionVariable
 * 
 * @author Hans De Bisschop
 * @package common.libraries
 */
class FunctionConditionVariable extends ConditionVariable
{
    /**
     * A constant defining a sum
     * 
     * @var int
     */
    CONST SUM = 1;
    
    /**
     * A constant defining a count
     * 
     * @var int
     */
    CONST COUNT = 2;
    
    /**
     * A constant defining a minimum
     * 
     * @var int
     */
    CONST MIN = 3;
    
    /**
     * A constant defining a maximum
     * 
     * @var int
     */
    CONST MAX = 4;

    /**
     * A constant defining a distinct
     *
     * @var int
     */
    CONST DISTINCT = 5;

    /**
     * The ConditionVariable
     * 
     * @var \common\libraries\ConditionVariable
     */
    private $condition_variable;

    /**
     * The function to apply to the ConditionVariable
     * 
     * @var int
     */
    private $function;

    private $alias;

    /**
     * Constructor
     * 
     * @param \common\libraries\ConditionVariable $condition_variable
     * @param int $function
     */
    public function __construct($function, $condition_variable, $alias = null)
    {
        $this->condition_variable = $condition_variable;
        $this->function = $function;
        $this->alias = $alias;
    }

    /**
     * Get the ConditionVariable on the condition_variable side of the operation
     * 
     * @return \common\libraries\ConditionVariable
     */
    public function get_condition_variable()
    {
        return $this->condition_variable;
    }

    /**
     * Set the ConditionVariable on the condition_variable side of the operation
     * 
     * @param $condition_variable \common\libraries\ConditionVariable
     */
    public function set_condition_variable($condition_variable)
    {
        $this->condition_variable = $condition_variable;
    }

    /**
     * Get the ConditionVariable on the function side of the operation
     * 
     * @return \common\libraries\ConditionVariable
     */
    public function get_function()
    {
        return $this->function;
    }

    /**
     * Set the ConditionVariable on the function side of the operation
     * 
     * @param $function \common\libraries\ConditionVariable
     */
    public function set_function($function)
    {
        $this->function = $function;
    }

    public function get_alias()
    {
        return $this->alias;
    }

    public function set_alias($alias)
    {
        $this->alias = $alias;
    }

    /**
     * Get an md5 representation of this object for identification purposes
     * 
     * @param $hash_parts multitype:string
     * @return string
     */
    public function hash($hash_parts = array())
    {
        $hash_parts[] = $this->condition_variable->hash();
        $hash_parts[] = $this->function;
        $hash_parts[] = $this->alias;
        
        return parent :: hash($hash_parts);
    }
}
