<?php
namespace common\libraries;

/**
 * $Id: platform_setting.class.php 168 2009-11-12 11:53:23Z vanpouckesven $
 *
 * @package common.configuration
 */

/**
 * This class represents the current configurable settings. They are retrieved from the DB via the AdminDataManager
 *
 * @author Hans De Bisschop
 */
class PlatformSetting
{

    /**
     * Instance of this class for the singleton pattern.
     */
    private static $instance;
    /**
     * Parameters defined in the configuration. Stored as an associative array.
     */
    private $params;

    /**
     * Constructor.
     */
    public function __construct()
    {
        $this->params = array();
        $this->load_platform_settings();
    }

    /**
     * Returns the instance of this class.
     *
     * @return PlatformSetting The instance.
     */
    public static function get_instance()
    {
        if (! isset(self :: $instance))
        {
            self :: set_instance(new self());
        }
        return self :: $instance;
    }

    public static function set_instance($instance)
    {
        self :: $instance = $instance;
    }

    /**
     * Gets a parameter from the configuration.
     *
     * @param $section string The name of the section in which the parameter is located.
     * @param $name string The parameter name.
     * @return mixed The parameter value.
     * @deprecated use instance method get_value instead
     */
    public static function get($variable, $application = 'core\admin')
    {
        return self :: get_instance()->get_value($variable, $application);
    }

    public function get_value($variable, $application = 'core\admin')
    {
        if (isset($this->params[$application]) && isset($this->params[$application][$variable]))
        {
            return $this->params[$application][$variable];
        }
        return null;
    }

    public function set_value($variable, $value, $application = 'core\admin')
    {
        $this->params[$application][$variable] = $value;
    }

    public function load_platform_settings()
    {
        // installer broken!!
        if ($params = SystemCache :: system()->platform_settings_params)
        {
            $this->params = $params;
            return;
        }
        $settings = CommonDataManager :: retrieves(Setting :: class_name());
        while ($setting = $settings->next_result())
        {
            $this->params[$setting->get_application()][$setting->get_variable()] = $setting->get_value();
        }

        SystemCache :: system()->platform_settings_params = $this->params;
    }

    public static function application_has_settings($application)
    {
        $instance = self :: get_instance();
        $params = & $instance->params;
        return isset($params[$application]);
    }

}
