<?php
namespace common\libraries;

/**
 *
 * @package common.libraries
 * @author Hans De Bisschop
 * @author Magali Gillard
 */
abstract class CoreApplication extends Application
{
    const CLASS_NAME = __CLASS__;

    public static $application_path_cache = array();

    /**
     * Get a list of available core packages
     *
     * @deprecated Use get_packages_from_filesystem() now
     * @return multitype:string
     */
    public static function get_list()
    {
        $applications = array();
        $applications[] = 'core\admin';
        $applications[] = 'tracking';
        $applications[] = 'repository';
        $applications[] = 'core\migration';
        $applications[] = 'user';
        $applications[] = 'group';
        $applications[] = 'core\help';
        $applications[] = 'rights';
        $applications[] = 'core\home';
        $applications[] = 'core\menu';
        $applications[] = 'webservice';
        $applications[] = 'reporting';
        $applications[] = 'core\metadata';
        $applications[] = 'core\context_linker';
        $applications[] = 'core\lynx';

        return $applications;
    }

    /**
     *
     * @param string $application_name
     * @return string
     */
    public static function get_application_namespace($application_name)
    {
        $path = Path :: get(SYS_PATH) . 'core/' . $application_name . '/';
        if (is_dir($path))
        {
            return 'core\\' . $application_name;
        }
        else
        {
            return $application_name;
        }
    }

    /**
     *
     * @param string $application
     * @return boolean
     */
    public static function exists($application)
    {
        if (! isset(self :: $application_path_cache[$application]))
        {
            $application_path = Path :: namespace_to_full_path(self :: get_application_namespace($application));
            self :: $application_path_cache[$application] = is_dir($application_path);
        }
        return self :: $application_path_cache[$application];
    }

    /**
     *
     * @return multitype:string
     */
    public static function get_packages_from_filesystem()
    {
        $applications = array();

        $applications[] = 'tracking';
        $applications[] = 'repository';
        $applications[] = 'user';
        $applications[] = 'group';
        $applications[] = 'rights';
        $applications[] = 'webservice';
        $applications[] = 'reporting';

        $directories = Filesystem :: get_directory_content(
            Path :: get_core_path(),
            Filesystem :: LIST_DIRECTORIES,
            false);

        foreach ($directories as $directory)
        {
            $application_name = basename($directory);

            if (self :: is_application_name($application_name))
            {
                $applications[] = self :: get_application_namespace($application_name);
            }
        }

        return $applications;
    }

    /**
     *
     * @param boolean $include_installed
     * @return \common\libraries\package\PackageList
     */
    public static function get_installable_packages($include_installed = false)
    {
        $package_list = new \common\libraries\package\PackageList(
            'core',
            Translation :: get('Core', null, Utilities :: COMMON_LIBRARIES));

        foreach (self :: get_packages_from_filesystem() as $application)
        {
            $class = $application . '\Manager';

            if (! CommonDataManager :: get_registration($application) || $include_installed)
            {
                $package_list->add_package($application);
            }

            if (! class_exists($class))
            {
                $class = $application . '\\' . Utilities :: underscores_to_camelcase(
                    Utilities :: get_package_name_from_namespace($application)) . 'Manager';
            }

            if (class_exists($class))
            {
                $application_package_list = $class :: get_installable_application_packages($include_installed);
                if ($application_package_list instanceof \common\libraries\package\PackageList)
                {
                    $package_list->add_child($application_package_list);
                }
            }
        }

        return $package_list;
    }

    /**
     *
     * @param boolean $include_installed
     * @return boolean
     */
    public static function get_installable_application_packages($include_installed = false)
    {
        return false;
    }
}
