<?php
namespace common\libraries;

/**
 * Class to display the footer of a HTML-page
 */
class Footer
{

    /**
     * Create a new Footer
     */
    public function __construct()
    {
    }

    public function get_setting($variable, $application)
    {
        return PlatformSetting :: get($variable, $application);
    }

    /**
     * Display the footer
     */
    public function display()
    {
        echo $this->toHtml();
    }

    /**
     * Returns the HTML code for the footer
     */
    public function toHtml()
    {
        $translator = Translation :: get_instance();

        $output[] = '<div class="clear">&nbsp;</div> <!-- "clearing" div to make sure that footer stays below the main and right column sections -->';
        $output[] = '</div> <!-- end of #main" started at the end of banner.inc.php -->';

        $registration = CommonDataManager :: get_registration('core\menu');
        if ($registration instanceof Registration && $registration->is_active())
        {
            $show_sitemap = $this->get_setting('show_sitemap', 'core\menu');

            if (Authentication :: is_valid() && $show_sitemap == '1')
            {
                $output[] = '<div id="sitemap">';
                $output[] = '<div class="categories">';
                $user = \user\DataManager :: retrieve(\user\User :: class_name(), (int) Session :: get_user_id());

                $output[] = \core\menu\MenuRenderer :: as_html(\core\menu\MenuRenderer :: TYPE_SITE_MAP, $user);
                $output[] = '<div class="clear"></div>';
                $output[] = '</div>';
                $output[] = '<div class="clear"></div>';
                $output[] = '</div>';
            }
        }

        $output[] = '<div id="footer"> <!-- start of #footer section -->';
        $output[] = '<div id="copyright">';
        $output[] = '<div class="logo">';
        $output[] = '<a href="http://www.chamilo.org"><img src="' . Theme :: get_common_image_path() .
             'logo_footer.png" alt="footer"/></a>';
        $output[] = '</div>';
        $output[] = '<div class="links">';

        $links = array();
        $links[] = DatetimeUtilities :: format_locale_date(
            $translator->get('DateFormatShort', null, Utilities :: COMMON_LIBRARIES) . ', ' .
                 $translator->get('TimeNoSecFormat', null, Utilities :: COMMON_LIBRARIES),
                time());
        $links[] = '<a href="' . $this->get_setting('institution_url', 'core\admin') . '" target="about:blank">' .
             $this->get_setting('institution', 'core\admin') . '</a>';

        if ($this->get_setting('show_administrator_data', 'core\admin') == '1')
        {
            $admin_data = Translation :: get('Manager');
            $admin_data .= ':&nbsp;';

            $administrator_email = $this->get_setting('administrator_email', 'core\admin');
            $administrator_website = $this->get_setting('administrator_website', 'core\admin');

            if (! empty($administrator_email) && ! empty($administrator_website))
            {
                $email = Display :: encrypted_mailto_link(
                    $this->get_setting('administrator_email', 'core\admin'),
                    $this->get_setting('administrator_surname', 'core\admin') . ' ' .
                         $this->get_setting('administrator_firstname', 'core\admin'));

                $admin_data = Translation :: get(
                    'ManagerContactWebsite',
                    array('EMAIL' => $email, 'WEBSITE' => $administrator_website));
            }
            else
            {
                if (! empty($administrator_email))
                {
                    $admin_data = Translation :: get('Manager');
                    $admin_data .= ':&nbsp;';

                    $admin_data .= Display :: encrypted_mailto_link(
                        $this->get_setting('administrator_email', 'core\admin'),
                        $this->get_setting('administrator_surname', 'core\admin') . ' ' .
                             $this->get_setting('administrator_firstname', 'core\admin'));
                }

                if (! empty($administrator_website))
                {
                    $admin_data = Translation :: get('Support');
                    $admin_data .= ':&nbsp;';

                    $admin_data .= '<a href="' . $administrator_website . '">' .
                         $this->get_setting('administrator_surname', 'core\admin') . ' ' .
                         $this->get_setting('administrator_firstname', 'core\admin') . '</a>';
                }
            }

            $links[] = $admin_data;
        }

        if ($this->get_setting('show_version_data', 'core\admin') == '1')
        {
            $links[] = htmlspecialchars(Translation :: get('Version')) . ' ' .
                 $this->get_setting('version', 'core\admin');
        }

        // $links[] = Translation :: get('License');
        // $links[] = Translation :: get('PrivacyPolicy');
        // $links[] = '<a href="http://www.chamilo.org">http://www.chamilo.org</a>';

        $world = PlatformSetting :: get('whoisonlineaccess');

        if ($world == "1" || (key_exists('_uid', $_SESSION) && $world == "2"))
        {
            $links[] = '<a href="' .
                 htmlspecialchars(Path :: get(WEB_PATH) . 'index.php?go=whois_online&application=admin') . '">' .
                 Translation :: get('WhoisOnline') . '</a>';
        }

        $links[] = '&copy;&nbsp;' . date('Y');

        $output[] = implode('&nbsp;|&nbsp;', $links);

        $output[] = '</div>';
        $output[] = '<div class="clear"></div>';
        $output[] = '</div>';

        $output[] = '   </div> <!-- end of #footer -->';
        $output[] = '  </div> <!-- end of #outerframe opened in header -->';
        $output[] = ' </body>';
        $output[] = '</html>';
        // $output[] = '<script type="text/javascript">( function($) { $(window).unload(function() { alert("ByeNow!");
        // }); })(jQuery);</script>';

        // hidden memory usage in source
        $output[] = '<!-- Memory Usage: ' . memory_get_peak_usage(1) . ' bytes -->';

        return implode("\n", $output);

        // $template = Theme :: get_template();
        // //$template->add_filename('footer.tpl');
        // $template->set_filenames(array('footer' => 'common/footer.tpl'));
        //
        // $show_sitemap = $this->get_setting('show_sitemap', 'menu');
        //
        // if (Authentication :: is_valid() && $show_sitemap == '1')
        // {
        // $udm = UserDataManager :: get_instance();
        // $user = $udm->retrieve_user(Session :: get_user_id());
        //
        // $menu = MenuRenderer :: as_html(MenuRenderer :: TYPE_SITE_MAP, $user);
        //
        // $template->assign_block_vars('SITEMAP', array('MENU' => $menu));
        // }
        //
        // $template->assign_var('WEB_PATH', Path :: get(WEB_PATH));
        //
        // $links = array();
        // $links[] = DatetimeUtilities :: format_locale_date(Translation :: get('DateFormatShort', null, Utilities ::
        // COMMON_LIBRARIES) . ', ' . Translation :: get('TimeNoSecFormat', null, Utilities :: COMMON_LIBRARIES),
        // time());
        // $links[] = '<a href="' . $this->get_setting('institution_url', 'core\admin') . '" target="about:blank">' .
        // $this->get_setting('institution', 'core\admin') . '</a>';
        //
        // if ($this->get_setting('show_administrator_data', 'core\admin') == 'true')
        // {
        // $admin_data .= Translation :: get('Manager');
        // $admin_data .= ':&nbsp;';
        // $admin_data .= Display :: encrypted_mailto_link($this->get_setting('administrator_email', 'core\admin'),
        // $this->get_setting('administrator_surname', 'core\admin') . ' ' .
        // $this->get_setting('administrator_firstname',
        // 'core\admin'));
        //
        // $links[] = $admin_data;
        // }
        //
        // if ($this->get_setting('show_version_data', 'core\admin') == '1')
        // {
        // $links[] = Translation :: get('Version') . ' ' . $this->get_setting('version', 'core\admin');
        // }
        //
        // $world = PlatformSetting :: get('whoisonlineaccess');
        //
        // if ($world == "1" || $_SESSION['_uid'] && $world == "2")
        // {
        // $links[] = '<a href="' . Path :: get(WEB_PATH) . 'core.php?go=whois_online&amp;application=admin">' .
        // Translation :: get('WhoisOnline') . '</a>';
        // }
        //
        // $links[] = '&copy;&nbsp;' . date('Y');
        //
        // $links = implode('&nbsp;|&nbsp;', $links);
        //
        // $template->assign_var('LINKS', $links);
        //
        // return $template->render('footer');
    }
}
