<?php
namespace common\libraries;

/**
 * $Id: theme.class.php 128 2009-11-09 13:13:20Z vanpouckesven $
 *
 * @package common.html
 */
// Theme-paths
define('WEB_THEME_PATH', 'WEB_THEME_PATH');
define('SYS_THEME_PATH', 'SYS_THEME_PATH');
define('WEB_IMG_PATH', 'WEB_IMG_PATH');
define('SYS_IMG_PATH', 'SYS_IMG_PATH');
define('WEB_CSS_PATH', 'WEB_CSS_PATH');
define('SYS_CSS_PATH', 'SYS_CSS_PATH');
class Theme
{
    const ICON_MINI = 16;
    const ICON_SMALL = 22;
    const ICON_MEDIUM = 32;
    const ICON_BIG = 48;
    const ICON_HUGE = 64;

    /**
     * Instance of this class for the singleton pattern.
     */
    private static $instance;

    /**
     * The theme we're currently using
     */
    private $theme;

    /**
     * The application we're currently rendering
     */
    private $application;

    /**
     * The template engine
     *
     * @var ChamiloTemplate The chamilo templating object
     */
    private $template;

    /**
     * Cache for results of get_path() method.
     */
    private $path;

    public function __construct()
    {
        $this->theme = PlatformSetting :: get('theme');
        $this->template = new Phpbb2TemplateWrapper($this->theme);
        // $this->template = ChamiloTemplate :: get_instance($this->theme);

        $this->path = array();
    }

    /**
     * Returns the template engine
     *
     * @return Phpbb2TemplateWrapper
     */
    public static function get_template()
    {
        return self :: get_instance()->template;
    }

    public static function get_theme()
    {
        return self :: get_instance()->theme;
    }

    public static function set_theme($theme)
    {
        $instance = self :: get_instance();
        $instance->theme = $theme;

        $template = $instance->get_template();
        $template->set_theme($theme);
        $template->reset();

        $instance->path = array();
    }

    public static function get_application()
    {
        return self :: get_instance()->application;
    }

    public static function set_application($application)
    {
        $instance = self :: get_instance();
        $instance->application = $application;
    }

    public function get_path($path_type)
    {
        if (isset($this->path[$path_type]))
        {
            return $this->path[$path_type];
        }

        switch ($path_type)
        {
            case WEB_IMG_PATH :
                return $this->path[$path_type] = Path :: get(WEB_LAYOUT_PATH) . $this->get_theme() . '/images/';
            case SYS_IMG_PATH :
                return $this->path[$path_type] = Path :: get(SYS_LAYOUT_PATH) . $this->get_theme() . '/images/';
            case WEB_CSS_PATH :
                return $this->path[$path_type] = Path :: get(WEB_LAYOUT_PATH) . $this->get_theme() . '/css/';
            case SYS_CSS_PATH :
                return $this->path[$path_type] = Path :: get(SYS_LAYOUT_PATH) . $this->get_theme() . '/css/';
            case WEB_THEME_PATH :
                return $this->path[$path_type] = Path :: get(WEB_LAYOUT_PATH) . $this->get_theme() . '/';
            case SYS_THEME_PATH :
                return $this->path[$path_type] = Path :: get(SYS_LAYOUT_PATH) . $this->get_theme() . '/';
        }
    }

    /**
     * Get the web path to the theme's folder
     */
    public static function get_theme_path()
    {
        return self :: get_instance()->get_path(WEB_THEME_PATH);
    }

    /**
     * Get the web path to the application's css file
     */
    public static function get_css_path($application = null, $web = true)
    {
        $instance = self :: get_instance();
        $application = (is_null($application) ? $instance->get_application() : $application);
        if ($web)
        {
            return Path :: namespace_to_full_path(Application :: determine_namespace($application), true) .
                 'resources/css/' . $instance->get_theme() . '/' . $instance->get_theme() . '.css';
        }
        else
        {
            return Path :: namespace_to_full_path(Application :: determine_namespace($application)) . 'resources/css/' .
                 $instance->get_theme() . '/' . $instance->get_theme() . '.css';
        }
    }

    /**
     * Get the web path to the general css file
     */
    public static function get_common_css_path($web = true)
    {
        $instance = self :: get_instance();
        return ($web ? Path :: get_web_common_libraries_path() : Path :: get_common_libraries_path()) . 'resources/css/' .
             $instance->get_theme() . '/' . $instance->get_theme() . '.css';

        // return Path :: get_web_common_libraries_path() . 'resources/css/' . $instance->get_theme() . '/' .
    // $instance->get_theme() . '.css';
    }

    /**
     * Get the path to the application's image folder
     */
    public static function get_image_path($context = null)
    {
        $instance = self :: get_instance();

        if (! $context)
        {
            $backtrace = debug_backtrace();
            $called_class = $backtrace[1]['class'];
            // $called_class = get_called_class();
            $context = Utilities :: get_namespace_from_classname($called_class);
        }

        $called_class = explode('\\', $context);
        return Path :: get(WEB_PATH) . implode('/', $called_class) . '/resources/images/' . $instance->get_theme() . '/';
    }

    /**
     * Get the system path to the application's image folder
     */
    public static function get_image_system_path($context = null)
    {
        $instance = self :: get_instance();

        if (! $context)
        {
            $backtrace = debug_backtrace();
            // $called_class = $backtrace[1]['class'];
            $called_class = get_called_class();
            $context = Utilities :: get_namespace_from_classname($called_class);
        }

        $called_class = explode('\\', $context);
        return Path :: get(SYS_PATH) . implode('/', $called_class) . '/resources/images/' . $instance->get_theme() . '/';
    }

    /**
     * Get the path to the general image folder
     */
    public static function get_common_image_path()
    {
        $instance = self :: get_instance();
        return Path :: get_web_common_libraries_path() . 'resources/images/' . $instance->get_theme() . '/';
    }

    /**
     * Get the system path to the general image folder
     */
    public static function get_common_image_system_path()
    {
        $instance = self :: get_instance();
        return Path :: get_common_libraries_path() . 'resources/images/' . $instance->get_theme() . '/';
    }

    public static function get_instance()
    {
        if (! isset(self :: $instance))
        {
            self :: $instance = new self();
        }
        return self :: $instance;
    }

    public static function get_themes()
    {
        $options = array();

        $path = Path :: get(SYS_LAYOUT_PATH);
        $directories = Filesystem :: get_directory_content($path, Filesystem :: LIST_DIRECTORIES, false);

        foreach ($directories as $index => & $directory)
        {
            if (substr($directory, 0, 1) != '.')
            {
                $options[$directory] = Utilities :: underscores_to_camelcase($directory);
            }
        }

        return $options;
    }

    public static function get_common_image($image, $extension = 'png', $label = null, $href = null,
        $display = ToolbarItem :: DISPLAY_ICON_AND_LABEL, $confirmation = false)
    {
        $icon = new ToolbarItem(
            $label,
            self :: get_common_image_path() . $image . '.' . $extension,
            $href,
            $display,
            $confirmation);
        return $icon->as_html();
    }

    public static function get_image($image, $extension = 'png', $label = null, $href = null,
        $display = ToolbarItem :: DISPLAY_ICON_AND_LABEL, $confirmation = false, $context = null)
    {
        if (! $context)
        {
            $backtrace = debug_backtrace();
            $called_class = $backtrace[1]['class'];
            $context = Utilities :: get_namespace_from_classname($called_class);
        }

        $icon = new ToolbarItem(
            $label,
            self :: get_image_path($context) . $image . '.' . $extension,
            $href,
            $display,
            $confirmation);
        return $icon->as_html();
    }

    public static function get_treemenu_image($image, $extension = 'png', $label = null, $href = null,
        $display = ToolbarItem :: DISPLAY_ICON_AND_LABEL, $confirmation = false)
    {
        return self :: get_common_image('treemenu/' . $image, $extension, $label, $href, $display, $confirmation);
    }

    public static function get_treemenu_type_image($image, $extension = 'png', $label = null, $href = null,
        $display = ToolbarItem :: DISPLAY_ICON_AND_LABEL, $confirmation = false)
    {
        return self :: get_common_image('treemenu_types/' . $image, $extension, $label, $href, $display, $confirmation);
    }
}
