<?php
namespace common\libraries;

/**
 * This class represents a column for a table that is based on a property from the data class
 *
 * @author Sven Vanpoucke - Hogeschool Gent
 */
class DataClassPropertyTableColumn extends NewTableColumn
{
    /**
     * The class name of the dataclass
     *
     * @var string
     */
    private $class_name;

    /****************************************************************************************************************
     * Constructor                                                                                                  *
     ****************************************************************************************************************/

    /**
     * Constructor
     *
     * @param string $class_name - The class name
     * @param string $property - The property or the property alias
     * @param string $title - [OPTIONAL] default translated title from property
     * @param bool $sortable - Whether or not the column is sortable
     */
    public function __construct($class_name, $property, $title = null, $sortable = true)
    {
        $this->class_name = $class_name;

        $context = $class_name :: context();

        if(!$title)
        {
            $title = Translation :: get(Utilities :: underscores_to_camelcase($property), null, $context);
        }

        $data_manager = $this->determine_data_manager($class_name);
        $storage_unit_alias = $data_manager->get_alias($class_name :: get_table_name());

        parent :: __construct($property, $title, $sortable, $storage_unit_alias);
    }

    /**
     * Returns the data manager
     */
    protected function determine_data_manager($class_name)
    {
        $context = $class_name :: context();
        $data_manager_class = $context . '\\DataManager';

        if(class_exists($data_manager_class))
        {
            return $data_manager_class :: get_instance();
        }
        else
        {
            return $class_name :: get_data_manager();
        }
    }

    /****************************************************************************************************************
     * Getters & Setters                                                                                            *
     ****************************************************************************************************************/

    /**
     * Returns the class name
     *
     * @return string
     */
    public function get_class_name()
    {
        return $this->class_name;
    }

    /**
     * Sets the class name
     *
     * @param string $class_name
     */
    public function set_class_name($class_name)
    {
        $this->class_name = $class_name;
    }
}
