<?php
namespace common\libraries;

/**
 * Describes a property for the gallery table
 *
 * @author Sven Vanpoucke - Hogeschool Gent - Refactoring from old gallery table property
 */
class GalleryTableProperty
{
    /**
     * The property of the object which will be displayed in this
     * column.
     *
     * @var string
     */
    private $property;

    /**
     * The title of the column.
     *
     * @var string
     */
    private $title;

    /**
     * The alias of the storage unit
     *
     * @var string
     */
    private $storage_unit_alias;

    /**
     * Constructor
     *
     * @param string $property
     * @param string $title
     * @param string $storage_unit_alias
     */
    public function __construct($property, $title = null, $storage_unit_alias = null)
    {
        $this->property = $property;

        if(is_null($title))
        {
            $backtrace = debug_backtrace();
            $called_class = $backtrace[1]['class'];
            $context = Utilities :: get_namespace_from_classname($called_class);

            $title = Translation :: get(Utilities :: underscores_to_camelcase($property), array(), $context);
        }

        $this->set_title($title);
        $this->set_storage_u = $storage_unit_alias;
    }

    /**
     * Returns the property
     *
     * @return string
     */
    public function get_property()
    {
        return $this->property;
    }

    /**
     * Sets the property
     *
     * @param string $property
     */
    public function set_property($property)
    {
        $this->property = $property;
    }

    /**
     * Gets the title of this column.
     * @return string
     */
    public function get_title()
    {
        return $this->title;
    }

    /**
     * Sets the title of this column.
     *
     * @param string $title
     */
    public function set_title($title)
    {
        $this->title = $title;
    }

    /**
     * Returns the storage unit alias
     *
     * @return string
     */
    public function get_storage_unit_alias()
    {
        return $this->storage_unit_alias;
    }

    /**
     * Returns the storage unit alias
     *
     * @param string $storage_unit_alias
     */
    public function set_storage_unit_alias($storage_unit_alias)
    {
        $this->storage_unit_alias = $storage_unit_alias;
    }

    /**
     * Returns the name of this property
     *
     * @return string
     */
    public function get_name()
    {
        return $this->property;
    }
}
