<?php
namespace common\libraries;

class SessionHandler
{

    private $save_path;

    private $name;

    private $lifetime = 9999999999999;

    public function __construct()
    {
        // see warning for php < 5.4: http://www.php.net/manual/en/function.session-set-save-handler.php
        register_shutdown_function('session_write_close');
    }

    public function open($save_path, $name)
    {
        $this->save_path = $save_path;
        $this->name = $name;
        $this->lifetime = PlatformSetting :: get('session_timeout') * 60;

        return true;
    }

    public function close()
    {
        return true;
    }

    public function read($session_id)
    {
        $conditions = array();
        $conditions[] = new EqualityCondition(\user\Session :: PROPERTY_SESSION_ID, $session_id);
        $conditions[] = new EqualityCondition(\user\Session :: PROPERTY_NAME, $this->name);
        $conditions[] = new EqualityCondition(\user\Session :: PROPERTY_SAVE_PATH, $this->save_path);

        $condition = new AndCondition($conditions);
        $session = \user\DataManager :: retrieve(\user\Session :: class_name(), $condition);

        if ($session instanceof \user\Session)
        {
            if ($session->is_valid())
            {
                return base64_decode($session->get_data());
            }
            else
            {
                $this->destroy($session_id);
            }
        }

        return '';
    }

    public function write($session_id, $data)
    {
        $data = base64_encode($data);

        $conditions = array();
        $conditions[] = new EqualityCondition(\user\Session :: PROPERTY_SESSION_ID, $session_id);
        $conditions[] = new EqualityCondition(\user\Session :: PROPERTY_NAME, $this->name);
        $conditions[] = new EqualityCondition(\user\Session :: PROPERTY_SAVE_PATH, $this->save_path);

        $condition = new AndCondition($conditions);

        $session = \user\DataManager :: retrieve(
            \user\Session :: class_name(),
            new DataClassRetrieveParameters($condition));

        if ($session instanceof \user\Session)
        {
            $session->set_data($data);
            $session->set_modified(time());
            return $session->update();
        }
        else
        {
            $session = new \user\Session();
            $session->set_modified(time());
            $session->set_lifetime($this->lifetime);
            $session->set_data($data);
            $session->set_name($this->name);
            $session->set_save_path($this->save_path);
            $session->set_session_id($session_id);
            return $session->create();
        }
    }

    public function destroy($session_id)
    {
        $conditions = array();
        $conditions[] = new EqualityCondition(\user\Session :: PROPERTY_SESSION_ID, $session_id);
        $conditions[] = new EqualityCondition(\user\Session :: PROPERTY_NAME, $this->name);
        $conditions[] = new EqualityCondition(\user\Session :: PROPERTY_SAVE_PATH, $this->save_path);

        $condition = new AndCondition($conditions);

        return \user\DataManager :: deletes(\user\Session :: class_name(), $condition);
    }

    public function garbage($max_lifetime)
    {
        $border = time() - $this->lifetime;
        $condition = new InequalityCondition(
            \user\Session :: PROPERTY_MODIFIED,
            InequalityCondition :: LESS_THAN,
            $border);
        return \user\DataManager :: deletes(\user\Session :: class_name(), $condition);
    }
}
