<?php
namespace common\libraries;

/**
 *
 * @package common.libraries.storage
 * @author Hans De Bisschop
 */
abstract class ConditionTranslator
{
    private $condition;
    private $deprecated_storage_unit;
    private $connection;
    public static $source;

    public function __construct(Condition $condition, $deprecated_storage_unit = null, $connection = null)
    {
        $this->condition = $condition;
        $this->deprecated_storage_unit = $deprecated_storage_unit;
        $this->connection = $connection;
    }

    public function get_condition()
    {
        return $this->condition;
    }

    public function get_deprecated_storage_unit()
    {
        return $this->deprecated_storage_unit;
    }

    public function get_connection()
    {
        return $this->connection;
    }

    abstract public function translate();

    public static function factory($type, Condition $condition, $deprecated_storage_unit = null, $connection = null)
    {
        $class = __NAMESPACE__ . '\\' . Utilities :: underscores_to_camelcase($type) . Utilities :: get_classname_from_object($condition) . 'Translator';

        return new $class($condition, $deprecated_storage_unit, $connection);
    }

    public static function render($type, Condition $condition, $deprecated_storage_unit = null, $connection = null)
    {
        $backtrace = debug_backtrace();
        if (! is_subclass_of($backtrace[2]['class'], 'common\libraries\ConditionTranslator'))
        {
            $i = 1;
            $back = Utilities :: get_namespace_from_classname($backtrace[$i]['class']);
            while ($back == 'common\libraries' && $i <= count($backtrace))
            {
                $i ++;
                $back = Utilities :: get_namespace_from_classname($backtrace[$i]['class']);
            }

            self :: $source = $back;
        }
        return self :: factory($type, $condition, $deprecated_storage_unit, $connection)->translate();
    }
}
