<?php
namespace common\libraries;

/**
 * A base class to allow DataClass caching for singular retrieves, result sets, counts, distinct sets, . ..
 *
 * @author Hans De Bisschop
 * @package common.libraries
 */
class DataClassCache
{

    /**
     * The instance of the DataClassCache
     *
     * @var \common\libraries\DataClassCache
     */
    private static $instance;

    /**
     * The cache
     *
     * @var multitype:multitype:mixed
     */
    private $cache;

    /**
     * Get an instance of the DataClassCache
     *
     * @return \common\libraries\DataClassCache
     */
    public static function get_instance()
    {
        if (! isset(self :: $instance))
        {
            self :: $instance = new self();
        }
        return self :: $instance;
    }

    /**
     * Get a DataClass object from the cache
     *
     * @param $class string
     * @param $parameters \common\libraries\DataClassResultParameters
     * @return boolean
     */
    public static function get($class, DataClassParameters $parameters)
    {
        $instance = self :: get_instance();

        if (self :: exists($class, $parameters))
        {
            return $instance->cache[$class][$parameters->hash()];
        }
        else

        {
            return false;
        }
    }

    /**
     * Returns whether a DataClass object exists in the cache
     *
     * @param $class string
     * @param $parameters \common\libraries\DataClassParameters
     * @return boolean
     */
    public static function exists($class, DataClassParameters $parameters)
    {
        $instance = self :: get_instance();

        if (isset($instance->cache[$class][$parameters->hash()]))
        {
            return true;
        }
        else
        {
            return false;
        }
    }

    /**
     * Clear the cache for a specific DataClass type
     *
     * @param $class string
     * @return boolean
     */
    public static function truncate($class)
    {
        $instance = self :: get_instance();

        if (isset($instance->cache[$class]))
        {
            unset($instance->cache[$class]);
        }

        return true;
    }

    /**
     * Clear the cache for a set of specific DataClass types
     *
     * @param $classes multitype:string
     * @return boolean
     */
    public static function truncates($classes = array())
    {
        foreach ($classes as $class)
        {
            if (! self :: truncate($class))
            {
                return false;
            }
        }

        return true;
    }

    /**
     * Set the cache value for a specific DataClass object type, hash
     *
     * @param $class string
     * @param $hash string
     * @param $value mixed
     */
    public static function set_cache($class, $hash, $value)
    {
        $instance = self :: get_instance();
        $instance->cache[$class][$hash] = $value;
    }

    public static function reset()
    {
        $instance = self :: get_instance();
        $instance->cache = array();
    }
}
