<?php
namespace common\libraries;

use Exception;

/**
 * A class to allow caching of DataClassResultSet objects
 *
 * @author Hans De Bisschop
 * @package common.libraries
 */
class DataClassResultSetCache extends DataClassCache
{

    /**
     * Add a DataClassResultSet object to the cache
     *
     * @param $result_set \common\libraries\DataClassResultSet
     * @param $parameters \common\libraries\DataClassRetrievesParameters
     * @throws Exception
     * @return boolean
     */
    public static function add(DataClassResultSet $result_set, DataClassRetrievesParameters $parameters)
    {
        if (! $parameters instanceof DataClassRetrievesParameters)
        {
            throw new Exception('Illegal parameters passed to the DataClassResultSetCache');
        }

        if (! $result_set instanceof DataClassResultSet)
        {
            $type = is_object($result_set) ? get_class($result_set) : gettype($result_set);
            throw new Exception(
                'The DataClassResultSetCache cache only allows for caching of ResultSet objects. Currently trying to add: ' .
                     $type . '.');
        }

        if (! DataClassCache :: get($result_set->get_class_name(), $parameters))
        {
            DataClassCache :: set_cache($result_set->get_class_name(), $parameters->hash(), $result_set);
        }

        return true;
    }
}
