<?php
namespace common\libraries;

class DataClassNoResultException extends \Exception
{
    /**
     * The fully qualified class name of the DataClass object retrieval for which no result was found
     *
     * @var string
     */
    private $class_name;

    /**
     * The parameters of the DataClass object retrieval for which no result was found
     *
     * @var \common\libraries\DataClassResultParameters
     */
    private $parameters;

    /**
     * Constructor
     *
     * @param $class_name string
     * @param $parameters \common\libraries\DataClassRetrieveParameters
     * @param $message string
     * @param $code int
     * @param $previous \Exception
     */
    public function __construct($class_name, $parameters, $message = null, $code = null, $previous = null)
    {
        $this->class_name = $class_name;
        $this->parameters = $parameters;

        if (is_null($message))
        {
            $message = Translation :: get('DataClassNoResultException', array('CLASS_NAME' => $class_name));
        }

        parent :: __construct($message, $code, $previous);
    }

    /**
     * Get the fully qualified class name of the DataClass object retrieval for which no result was found
     *
     * @return string
     */
    public function get_class_name()
    {
        return $this->class_name;
    }

    /**
     * Set the fully qualified class name of the DataClass object retrieval for which no result was found
     *
     * @param $class_name string
     */
    public function set_class_name($class_name)
    {
        $this->class_name = $class_name;
    }

    /**
     * Get the parameters of the DataClass object retrieval for which no result was found
     *
     * @return \common\libraries\DataClassRetrieveParameters
     */
    public function get_parameters()
    {
        return $this->parameters;
    }

    /**
     * Set the parameters of the DataClass object retrieval for which no result was found
     *
     * @param $parameters \common\libraries\DataClassRetrieveParameters
     */
    public function set_parameters($parameters)
    {
        $this->parameters = $parameters;
    }

}
