<?php
namespace common\libraries;

/**
 * This class describes a table you want to join with
 * 
 * @author Sven Vanpoucke - Hogeschool Gent
 * @author Gillard Magali and Hans De Bisschop - Erasmushogeschool Brussel
 */
class DataClassJoin extends Join
{

    /**
     * **************************************************************************************************************
     * Properties *
     * **************************************************************************************************************
     */
    
    /**
     * The classname of the data_class you want to join with
     * 
     * @var string
     */
    private $data_class;

    /**
     * **************************************************************************************************************
     * Constructor *
     * **************************************************************************************************************
     */
    
    /**
     * Constructs this data class join
     * 
     * @param string $data_class
     * @param \common\libraries\Condition $condition
     * @param int $type
     */
    public function __construct($data_class,\common\libraries\Condition $condition = null, $type = self :: TYPE_NORMAL)
    {
        $this->set_data_class($data_class);
        $data_manager = $data_class :: context() . '\DataManager';
        parent :: __construct($data_class :: get_table_name(), $data_manager :: get_instance(), $condition, $type);
    }

    /**
     * **************************************************************************************************************
     * Getters & Setters *
     * **************************************************************************************************************
     */
    
    /**
     * Returns the data class name
     * 
     * @return string
     */
    public function get_data_class()
    {
        return $this->data_class;
    }

    /**
     * Sets the data class name
     * 
     * @param string $data_class
     */
    public function set_data_class($data_class)
    {
        $this->data_class = $data_class;
    }
}
