<?php
namespace common\libraries;

interface DataManagerInterface
{

    /**
     * Initializes the connection with the storage layer
     */
    public function initialize();

    /**
     * Creates a storage unit
     *
     * @param $name string
     * @param $properties multitype
     * @param $indexes multitype
     */
    public function create_storage_unit($name, $properties, $indexes);

    /**
     * Counts the objects in a storage unit with a given condition
     *
     * @param $class string
     * @param $condition \common\libraries\Condition
     * @return int
     */
    public function count($class, $condition);

    /**
     * Retrieve an object from a storage unit with a given condition and order
     *
     * @param $class string
     * @param $paremeters \common\libraries\DataClassResultParameters
     * @return Ambigous <\common\libraries\DataClass|boolean>
     */
    public function retrieve($class, $paremeters = null);

    /**
     * Retrieves multiple object with a given set of DataClassResultSet parameters
     *
     * @param $class string
     * @param $parameters \common\libraries\DataClassRetrievesParameters
     * @return \common\libraries\DataClassResultSet
     */
    public function retrieves($class, DataClassRetrievesParameters $parameters);

    /**
     * Update functionality (can only be used when the storage unit has an ID
     * property)
     *
     * @param $object \common\libraries\DataClass
     * @param $condition Condition
     * @return boolean
     */
    public function update($object, $condition = false);

    /**
     * Deletes an object from a storage unit with a given condition
     *
     * @param $class string
     * @param $condition \common\libraries\Condition
     * @return boolean
     */
    public function new_delete($class, $condition);

    /**
     * Creates an object in a storage unit
     *
     * @param $object \common\libraries\DataClass
     * @return boolean
     */
    public function create($object);
}
