<?php
namespace common\libraries;

/**
 * Describes the group by functionality of a query. Uses ConditionVariable to define the group_by's
 *
 * @author Sven Vanpoucke - Hogeschool Gent
 */
class GroupBy implements Hashable
{
    /**
     * List of ConditionVariables to group by
     *
     * @var ConditionVariable[]
     */
    private $group_by;

    /**
     * Constructor
     *
     * @param $group_by ConditionVariable[]
     */
    public function __construct($group_by = array())
    {
        $this->group_by = (is_array($group_by) ? $group_by : func_get_args());
    }

    /**
     * Gets the joins
     *
     * @return ConditionVariable[]
     */
    public function get()
    {
        return $this->group_by;
    }

    /**
     * Adds a group by
     *
     * @param ConditionVariable $group_by
     */
    public function add($group_by)
    {
        $this->group_by[] = $group_by;
    }

    /**
     * Hashes this
     * @return string
     */
    public function hash()
    {
        $hashes = array();

        foreach ($this->group_by as $group_by)
        {
            $hashes[] = $group_by->hash();
        }

        sort($hashes);

        return md5(serialize($hashes));
    }
}
