<?php
namespace common\libraries;

/**
 *
 * @package common.libraries.storage.doctrine
 * @author Hans De Bisschop
 */
class DoctrinePatternMatchConditionTranslator extends DoctrineConditionTranslator
{

    public function translate()
    {
        if ($this->get_condition()->get_name() instanceof ConditionVariable)
        {
            return DoctrineConditionVariableTranslator :: render(
                $this->get_condition()->get_name(),
                $this->get_connection()) . ' LIKE ' . DoctrineDatabase :: quote(
                $this->search_string($this->get_condition()->get_pattern()),
                null,
                true,
                false,
                $this->get_connection());
        }
        else
        {
            $storage_unit = $this->get_deprecated_storage_unit();

            $name = $this->get_condition()->get_name();
            $condition_storage_unit = $this->get_condition()->get_storage_unit();

            if (! is_null($condition_storage_unit))
            {
                if ($this->get_condition()->is_alias())
                {
                    $storage_unit = $condition_storage_unit;
                }
                else
                {
                    $namespace = self :: $source;
                    $data_manager = $namespace . '\DataManager';
                    if (! class_exists($data_manager))
                    {
                        $data_manager = $namespace . '\\' .
                             Utilities :: get_package_name_from_namespace($namespace, true) . 'DataManager';
                    }
                    $storage_unit = $data_manager :: get_instance()->get_alias($condition_storage_unit);
                }
            }
            return DoctrineDatabase :: escape_column_name(
                $this->get_condition()->get_name(),
                $storage_unit,
                $this->get_connection()) . ' LIKE ' .
                 DoctrineDatabase :: quote(
                    $this->search_string($this->get_condition()->get_pattern()),
                    null,
                    true,
                    false,
                    $this->get_connection());
        }
    }

    /**
     * Translates a string with wildcard characters "?" (single character) and "*" (any character sequence) to a SQL
     * pattern for use in a LIKE condition. Should be suitable for any SQL flavor.
     *
     * @param $string string The string that contains wildcard characters.
     * @return string The escaped string.
     */
    public function search_string($string)
    {
        /*
         * ====================================================================== A brief explanation of these regexps:
         * - The first one escapes SQL wildcard characters, thus prefixing %, ', \ and _ with a backslash. - The second
         * one replaces asterisks that are not prefixed with a backslash (which escapes them) with the SQL equivalent,
         * namely a percent sign. - The third one is similar to the second: it replaces question marks that are not
         * escaped with the SQL equivalent _. ======================================================================
         */
        return preg_replace(
            array('/([%\'\\\\_])/e', '/(?<!\\\\)\*/', '/(?<!\\\\)\?/'),
            array("'\\\\\\\\' . '\\1'", '%', '_'),
            $string);
    }
}
