<?php
namespace common\libraries;

/**
 * This class describes a storage unit you want to join with
 * 
 * @author Sven Vanpoucke - Hogeschool Gent
 * @author Gillard Magali and Hans De Bisschop - Erasmushogeschool Brussel
 */
class Join implements Hashable
{
    /**
     * **************************************************************************************************************
     * Join Types *
     * **************************************************************************************************************
     */
    const TYPE_NORMAL = 1;
    const TYPE_LEFT = 2;
    const TYPE_RIGHT = 3;

    /**
     * **************************************************************************************************************
     * Properties *
     * **************************************************************************************************************
     */
    
    /**
     * The storage unit name with which you want to join
     * 
     * @var string
     */
    private $storage_unit_name;

    /**
     * The data manager of the joined storage unit
     * 
     * @var \common\libraries\DataManager
     */
    private $data_manager;

    /**
     * The join type
     * 
     * @var int
     */
    private $type;

    /**
     * The join condition
     * 
     * @var \common\libraries\Condition
     */
    private $condition;

    /**
     * **************************************************************************************************************
     * Constructor *
     * **************************************************************************************************************
     */
    
    /**
     * Constructs this join table
     * 
     * @param string $storage_unit_name
     * @param mixed $data_manager
     * @param \common\libraries\Condition $condition
     * @param int $type
     */
    public function __construct($storage_unit_name, $data_manager = null, $condition = null, $type = self :: TYPE_NORMAL)
    {
        $this->set_storage_unit_name($storage_unit_name);
        $this->set_data_manager($data_manager);
        $this->set_condition($condition);
        $this->set_type($type);
    }

    /**
     * **************************************************************************************************************
     * Getters & Setters *
     * **************************************************************************************************************
     */
    
    /**
     * Returns the storage unit name
     * 
     * @return string
     */
    public function get_storage_unit_name()
    {
        return $this->storage_unit_name;
    }

    /**
     * Sets the storage unit name
     * 
     * @param string $storage_unit_name
     */
    public function set_storage_unit_name($storage_unit_name)
    {
        $this->storage_unit_name = $storage_unit_name;
    }

    /**
     * Returns the data manager
     * 
     * @return \common\libraries\DataManager
     */
    public function get_data_manager()
    {
        return $this->data_manager;
    }

    /**
     * Sets the data manager
     * 
     * @param \common\libraries\DataManager $data_manager
     */
    public function set_data_manager($data_manager)
    {
        $this->data_manager = $data_manager;
    }

    /**
     * Returns the condition
     * 
     * @return \common\libraries\Condition
     */
    public function get_condition()
    {
        return $this->condition;
    }

    /**
     * Sets the condition
     * 
     * @param \common\libraries\Condition $condition
     */
    public function set_condition(\common\libraries\Condition $condition)
    {
        $this->condition = $condition;
    }

    /**
     * Returns the type
     * 
     * @return int
     */
    public function get_type()
    {
        return $this->type;
    }

    /**
     * Sets the type
     * 
     * @param int $type
     */
    public function set_type($type)
    {
        $this->type = $type;
    }

    public function hash($hash_parts = array())
    {
        $hash_parts[] = $this->storage_unit_name;
        $data_manager = $this->data_manager;
        $hash_parts[] = $data_manager :: class_name();
        $hash_parts[] = $this->condition->hash();
        $hash_parts[] = $this->type;
        
        return md5(serialize($hash_parts));
    }
}
