<?php
namespace common\libraries;

use Exception;

class QueryCache
{
    /**
     * The instance of the QueryCache
     *
     * @var \common\libraries\QueryCache
     */
    private static $instance;
    
    /**
     * The cache
     *
     * @var multitype:multitype:mixed
     */
    private $cache;

    /**
     * Get an instance of the QueryCache
     *
     * @return \common\libraries\QueryCache
     */
    public static function get_instance()
    {
        if (! isset(self :: $instance))
        {
            self :: $instance = new self();
        }
        return self :: $instance;
    }

    /**
     * Get a DataClass object from the cache
     *
     * @param $class string
     * @param $parameters \common\libraries\DataClassResultParameters
     * @return boolean
     */
    public static function get($hash)
    {
        $instance = self :: get_instance();
        
        if (self :: exists($hash))
        {
            return $instance->cache[$hash];
        }
        else
        {
            return false;
        }
    }

    /**
     * Returns whether a DataClass object exists in the cache
     *
     * @param $class string
     * @param $parameters \common\libraries\DataClassParameters
     * @return boolean
     */
    public static function exists($hash)
    {
        $instance = self :: get_instance();
        
        if (isset($instance->cache[$hash]))
        {
            return true;
        }
        else
        {
            return false;
        }
    }

    /**
     * Clear the cache for a specific DataClass type
     *
     * @param $class string
     * @return boolean
     */
    public static function truncate()
    {
        $instance = self :: get_instance();
        
        if (isset($instance->cache))
        {
            unset($instance->cache);
        }
        
        return true;
    }

    /**
     * Set the cache value for a specific DataClass object type, hash
     *
     * @param $class string
     * @param $hash string
     * @param $value mixed
     */
    public static function set_cache($hash, $value)
    {
        $instance = self :: get_instance();
        $instance->cache[$hash] = $value;
    }

    public static function reset()
    {
        $instance = self :: get_instance();
        $instance->cache = array();
    }

    public static function add($result, $hash)
    {
        if (! is_string($hash) && strlen($hash) != 32)
        {
            throw new Exception('Illegal hash passed to the QueryCache');
        }
        
        if (! self :: get($hash))
        {
            self :: set_cache($hash, $result);
        }
        
        return true;
    }

    public static function hash()
    {
        $backtrace = debug_backtrace();
        $called_class = $backtrace[1];
        
        $parts = array();
        $parts[] = $called_class['class'];
        $parts[] = $called_class['type'];
        $parts[] = $called_class['function'];
        
        foreach ($called_class['args'] as $argument)
        {
            $parts[] = $argument;
        }

        return md5(serialize($parts));
    }
}
