<?php
namespace common\libraries;

/**
 * $Id: translation.class.php 128 2009-11-09 13:13:20Z vanpouckesven $
 *
 * @package common.translation
 */
class Translation
{
    const PACKAGE_DELIMITER = '.';
    const PACKAGE_COMMON = 'common';

    /**
     * Instance of this class for the singleton pattern.
     */
    private static $instance;

    private static $called_class;

    private static $recently_added;

    /**
     * Language strings defined in the language-files. Stored as an associative array.
     */
    private $strings;

    /**
     * The language we're currently translating too
     */
    private $language;

    /**
     * The application we're currently translating
     */
    private $application;

    /**
     * To determine wether we should show the variable in a tooltip window or not (used for translation purposes)
     */
    private $show_variable_in_translation;

    /**
     * Determines whether or not to use the caching
     *
     * @var boolean
     */
    private $use_caching = true;

    /**
     * A list with reserved words that can not be used as a variable in the translation files
     *
     * @var string[]
     */
    private $reserved_words = array('true', 'false', 'on', 'off', 'null', 'yes', 'no', 'none');

    /**
     * Constructor.
     */
    private function __construct($language = null)
    {
        if (is_null($language))
        {
//            $this->language = LocalSetting :: get('platform_language');
//            if (! CommonDataManager :: is_language_active($this->language))
//            {
//                $this->language = PlatformSetting :: get('platform_language');
//            }
            global $language_interface;
            $this->language = $language_interface;

            if (Request :: get('install_running') != 1 && file_exists(
                dirname(__FILE__) . '/../configuration/configuration.php'))
            {
                $this->show_variable_in_translation = PlatformSetting :: get('show_variable_in_translation');
                $this->use_caching = ! PlatformSetting :: get('write_new_variables_to_translation_file');
            }
        }
        else
        {
            $this->language = $language;
        }

        $this->strings = array();
        
        if ($this->use_caching)
        {
            $cache_file = Path :: get_cache_path() . 'translation/' . $this->language;

            if (file_exists($cache_file))
            {
                $cached_translations = file_get_contents($cache_file);
                if ($cached_translations)
                {
                    $instance->strings[$this->language] = unserialize($cached_translations);
                }
            }
        }
    }

    /**
     *
     * @return Translation
     */
    public static function get_instance()
    {
        if (! isset(self :: $instance))
        {
            self :: $instance = new self();
        }
        return self :: $instance;
    }

    /**
     * TODO: This comment does not fit here. Returns the instance of this class.
     *
     * @param $variable String
     * @param $parameters Array (always use capital letters) Example: Translation :: get('UserCount', array('COUNT' =>
     *            $usercount)); $lang['user']['UserCount'] = There are {COUNT} users on the system;
     * @return string
     */
    public function get($variable, $parameters = array(), $context = null, $isocode = null)
    {
        $instance = self :: get_instance();

        $backtrace = debug_backtrace();
        self :: $called_class = $backtrace[1]['class'];
        // self :: $called_class = get_called_class();

        $translation = $instance->translate($variable, $context, $isocode);

        if (empty($parameters))
        {
            return $translation;
        }
        else
        {
            return preg_replace(
                '#\{([A-Z0-9\-_]+)\}#e',
                'isset($parameters[\'\\1\']) ? $parameters[\'\\1\'] : \'\'',
                $translation);
        }
    }

    public function get_language()
    {
        $instance = self :: get_instance();
        return $instance->language;
    }

    public function set_language($language)
    {
        $instance = self :: get_instance();
        $instance->language = $language;
    }

    public function get_application()
    {
        $instance = self :: get_instance();
        return $instance->application;
    }

    public function set_application($application)
    {
        $instance = self :: get_instance();
        $instance->application = $application;
    }

    public function translate($variable, $context = null, $isocode = null)
    {
        $instance = self :: get_instance();
        if (is_null($isocode))
        {
            $language = $instance->language;
        }
        else
        {
            $language = $isocode;
        }
        $strings = & $instance->strings;

        $value = null;

        if (! $context)
        {
            if (count(explode('\\', self :: $called_class)) > 1)
            {
                $context = Utilities :: get_namespace_from_classname(self :: $called_class);
            }
        }

        if (! isset($strings[$language]) && $this->use_caching)
        {
            self :: load_cache($language);
        }

        if (! isset($strings[$language][$context]))
        {
            $instance->add_context_internationalization($language, $context);
        }

        if (isset($strings[$language][$context][$variable]))
        {
            $value = $strings[$language][$context][$variable];
        }

        if (! $value || $value == '' || $value == ' ')
        {
            if (is_null($value) && PlatformSetting :: get('write_new_variables_to_translation_file') &&
                 ! array_key_exists($variable, $strings[$language][$context]) && count($strings[$language][$context]) > 0)
            {
                $this->add_variable_to_context_internationalization($language, $context, $variable);
            }

            if (PlatformSetting :: get('hide_dcda_markup'))
            {
                return $variable;
            }
            else
            {
                return '[CDA context={' . $context . '}]' . $variable . '[/CDA]';
            }
        }

        if ($this->show_variable_in_translation)
        {
            return '<span title="' . $context . ' - ' . $variable . '">' . $value . '</span>';
        }

        return $value;
    }

    public function add_context_internationalization($language, $context)
    {
        $called_class = explode('\\', $context);
        $path = Path :: get(SYS_PATH) . implode('/', $called_class) . '/resources/i18n/' . $language . '.i18n';

        if (! is_readable($path))
        {
            return;
        }

        $strings = parse_ini_file($path);

        $instance = self :: get_instance();
        $instance->strings[$language][$context] = $strings;

        if ($this->use_caching)
        {
            self :: cache($language);
        }
    }

    public function cache($language)
    {
        $instance = self :: get_instance();
        $strings = $instance->strings[$language];
        Filesystem :: write_to_file(Path :: get_cache_path() . 'translation/' . $language, serialize($strings));
    }

    public function load_cache($language)
    {
        $instance = self :: get_instance();
        $cache_file = Path :: get_cache_path() . 'translation/' . $language;

        if (file_exists($cache_file))
        {
            $cached_translations = file_get_contents($cache_file);
            if ($cached_translations)
            {
                $translations = unserialize($cached_translations);
                if ($translations != false)
                {
                    $instance->strings[$language] = $translations;
                }
            }
        }
    }

    public function add_variable_to_context_internationalization($language, $context, $variable)
    {
        if (in_array(strtolower($variable), $this->reserved_words))
        {
            continue;
        }

        if (! in_array($variable, self :: $recently_added[$language][$context]))
        {
            $path = Path :: get(SYS_PATH) . Path :: namespace_to_path($context) . '/resources/i18n/' . $language .
                 '.i18n';
            if (is_writable(dirname($path)))
            {
                if (! $handle = fopen($path, 'a'))
                {
                    return;
                }

                $string = "\n" . $variable . ' = ""';

                // Write $somecontent to our opened file
                if (fwrite($handle, $string) === FALSE)
                {
                    return;
                }

                fclose($handle);

                self :: $recently_added[$language][$context][] = $variable;
            }
        }
    }
}
