<?php

namespace common\libraries;

/**
 * A rest message renderer to a plain format
 */

class PlainRestMessageRenderer extends RestMessageRenderer
{

    /**
     * Renders one object
     * @param DataClass $object 
     */
    public function render_object(DataClass $object)
    {
        $this->render_header();
        echo $this->render_object_body($object);
    }

    /**
     * Renders an array of objects
     * @param array $objects
     */
    public function render_multiple_objects(array $objects)
    {
        $this->render_header();
        
        $plain = array();
        
        foreach ($objects as $object)
        {
            $plain[] = 'Object';
            $plain[] = $this->render_object_body($object);
            $plain[] = '';
        }
        
        echo implode("\n", $plain);
    }

    /**
     * Renders the content of one object
     * @param DataClass $object
     * @return String
     */
    private function render_object_body(DataClass $object)
    {
        $plain = array();
        
        foreach ($object->get_properties() as $type => $properties)
        {
            $plain[] = '[' . $type . ']';
            
            foreach ($properties() as $property => $value)
            {
                $plain[] = $property . ' = ' . $value;
            }
            
            $plain[] = '';
        }
        
        return implode("\n", $plain);
    }

    /**
     * Renders the header
     */
    private function render_header()
    {
        header('Content-Type: text/plain');
    }
}
