<?php
namespace Doctrine\DBAL\Driver\PDOMsSql;

use Doctrine\DBAL\Connection;

class Driver implements \Doctrine\DBAL\Driver
{

    /**
     * Attempts to establish a connection with the underlying driver.
     * 
     * @param array $params
     * @param string $username
     * @param string $password
     * @param array $driverOptions
     * @return Doctrine\DBAL\Driver\Connection
     */
    public function connect(array $params, $username = null, $password = null, array $driverOptions = array())
    {
        $conn = new PDOConnection($this->_constructPdoDsn($params), $username, $password, $driverOptions);
        return $conn;
    }

    /**
     * Constructs the MsSql PDO DSN.
     * 
     * @return string The DSN.
     */
    private function _constructPdoDsn(array $params)
    {
        if (extension_loaded('pdo_dblib'))
        {
            $dsn = 'dblib:';
            if (isset($params['host']) && $params['host'] != '')
            {
                $dsn .= 'host=' . $params['host'] . '; ';
            }
            
            if (isset($params['dbname']))
            {
                $dsn .= 'dbname=' . $params['dbname'] . ';';
            }
        }
        elseif (extension_loaded('pdo_odbc'))
        {
            $dsn = 'odbc:' . $params['host'];
        }
        elseif (extension_loaded('pdo_sybase'))
        {
            $dsn = 'sybase:';
            if (isset($params['host']) && $params['host'] != '')
            {
                $dsn .= 'host=' . $params['host'] . '; ';
            }
            
            if (isset($params['dbname']))
            {
                $dsn .= 'dbname=' . $params['dbname'] . ';';
            }
        }
        
        return $dsn;
    }

    public function getDatabasePlatform()
    {
        return new \Doctrine\DBAL\Platforms\MsSqlPlatform();
    }

    public function getSchemaManager(\Doctrine\DBAL\Connection $conn)
    {
        return new \Doctrine\DBAL\Schema\MsSqlSchemaManager($conn);
    }

    public function getName()
    {
        return 'pdo_mssql';
    }

    public function getDatabase(\Doctrine\DBAL\Connection $conn)
    {
        $params = $conn->getParams();
        return $params['dbname'];
    }
}
