<?php
namespace common\libraries\upgrade\v4_0_0;

use common\libraries\CommonDataManager;
use common\libraries\package\Package;
use common\libraries\Path;
use common\libraries\RecordRetrievesParameters;
use common\libraries\Translation;

/**
 *
 * @author Hans De Bisschop
 * @author Magali Gillard
 */
class Upgrader extends \common\libraries\package\Upgrader
{

    /**
     * Constructor
     */
    public function __construct()
    {
        parent :: __construct(CommonDataManager :: get_instance());
    }

    /*
     * (non-PHPdoc) @see \common\libraries\package\Upgrader::run()
     */
    public function run()
    {
        $data_manager = $this->get_data_manager();

        $upgrader = $this;

        $success = $data_manager->transactional(
            function ($c) use($data_manager, $upgrader)
            {
                // Create the new languages storage unit
                $path = Path :: namespace_to_full_path('common\libraries') . 'php/package/install/language.xml';
                $storage_unit_info = \common\libraries\package\Upgrader :: parse_xml_file($path);

                if (! CommonDataManager :: create_storage_unit(
                    $storage_unit_info['name'],
                    $storage_unit_info['properties'],
                    $storage_unit_info['indexes']))
                {
                    $upgrader->add_message(
                        \common\libraries\package\Action :: TYPE_ERROR,
                        Translation :: get_instance()->get('StorageUnitCreationFailed', null, 'core\install') . ': <em>' .
                             $storage_unit_info['name'] . '</em>');
                    return false;
                }
                else
                {
                    $upgrader->add_message(
                        \common\libraries\package\Action :: TYPE_NORMAL,
                        Translation :: get_instance()->get('StorageUnitCreation', null, 'core\install') . ': <em>' .
                             $storage_unit_info['name'] . '</em>');
                }

                // Fill the new table with the data from the old one

                // AdminDataManager is still MDB2 based and doesn't support records retrieval so we "abuse" the
                // CommonDataManager for this specific query
                $prefix = CommonDataManager :: get_instance()->get_prefix();
                CommonDataManager :: get_instance()->set_prefix('admin_');
                $language_records = CommonDataManager :: get_instance()->records(
                    \common\libraries\Language :: class_name(),
                    new RecordRetrievesParameters());

                CommonDataManager :: get_instance()->set_prefix($prefix);

                while ($language_record = $language_records->next_result())
                {
                    if (! CommonDataManager :: create_record(
                        \common\libraries\Language :: class_name(),
                        $language_record))
                    {
                        return false;
                    }
                }

                // Rename the languages storage unit
                if (! \core\admin\AdminDataManager :: get_instance()->drop_storage_unit(
                    \common\libraries\Language :: get_table_name()))
                {
                    return false;
                }

                // Change some property types for the settings storage unit
                $attributes = array('type' => 'text');
                $success = $data_manager->alter_storage_unit(
                    \common\libraries\DataManager :: ALTER_STORAGE_UNIT_CHANGE,
                    \common\libraries\Setting :: get_table_name(),
                    \common\libraries\Setting :: PROPERTY_VALUE,
                    $attributes);

                if (! $success)
                {
                    return false;
                }

                $attributes = array('type' => 'integer', 'unsigned' => 1, 'length' => 1, 'notnull' => 1, 'default' => 0);
                $success = $data_manager->alter_storage_unit(
                    \common\libraries\DataManager :: ALTER_STORAGE_UNIT_CHANGE,
                    \common\libraries\Setting :: get_table_name(),
                    \common\libraries\Setting :: PROPERTY_USER_SETTING,
                    $attributes);

                if (! $success)
                {
                    return false;
                }

                // Change a property type for the registration storage unit
                $attributes = array('type' => 'integer', 'length' => 1, 'unsigned' => 1, 'notnull' => 1);
                $success = $data_manager->alter_storage_unit(
                    \common\libraries\DataManager :: ALTER_STORAGE_UNIT_CHANGE,
                    \common\libraries\Registration :: get_table_name(),
                    \common\libraries\Registration :: PROPERTY_STATUS,
                    $attributes);

                if (! $success)
                {
                    return false;
                }

                $package = Package :: get('common\libraries');
                $registration = CommonDataManager :: get_registration('common\libraries');
                $registration->set_version($package->get_version());
                if (! $registration->update())
                {
                    return false;
                }

                return true;
            });

        return $success;
    }
}
