<?php
namespace core\admin\announcement;

use common\libraries\Display;
use common\libraries\Utilities;
use repository\content_object\system_announcement\SystemAnnouncement;
use common\libraries\Breadcrumb;
use common\libraries\Translation;

class CreatorComponent extends Manager implements \repository\viewer\ViewerInterface
{

    /**
     * Runs this component and displays its output.
     */
    public function run()
    {
        if (! $this->get_user()->get_platformadmin())
        {
            $this->display_header();
            echo Display :: error_message(Translation :: get('NotAllowed'), true);
            $this->display_footer();
            exit();
        }
        
        if (! \repository\viewer\Manager :: is_ready_to_be_published())
        {
            $repository_viewer = \repository\viewer\Manager :: construct($this);
            $repository_viewer->run();
        }
        else
        {
            $objects = \repository\viewer\Manager :: get_selected_objects();
            $publisher = new Publisher($this, $objects);
            
            if ($publisher->ready_to_publish())
            {
                $success = $publisher->publish();
                
                $message = Translation :: get(
                    ($success ? 'ObjectPublished' : 'ObjectNotPublished'), 
                    array('OBJECT' => Translation :: get('Object')), 
                    Utilities :: COMMON_LIBRARIES);
                
                $parameters = array(self :: PARAM_ACTION => self :: ACTION_BROWSE);
                
                $this->redirect($message, ! $success, $parameters);
            }
            else
            {
                $this->display_header();
                echo $publisher->toHtml();
                $this->display_footer();
            }
        }
    }

    public function get_allowed_content_object_types()
    {
        return array(SystemAnnouncement :: class_name());
    }

    public function get_additional_parameters()
    {
        return array(\repository\viewer\Manager :: PARAM_ID, \repository\viewer\Manager :: PARAM_ACTION);
    }
}
