<?php
namespace core\admin\announcement;

use repository\ContentObjectRendition;
use repository\ContentObjectRenditionImplementation;
use repository\ContentObjectForm;
use common\libraries\Utilities;
use common\libraries\Breadcrumb;
use common\libraries\Display;
use common\libraries\Application;
use common\libraries\Translation;
use common\libraries\Request;

class EditorComponent extends Manager
{

    /**
     * Runs this component and displays its output.
     */
    public function run()
    {
        if (! $this->get_user()->get_platformadmin())
        {
            $this->display_header();
            echo Display :: error_message(Translation :: get('NotAllowed'), true);
            $this->display_footer();
            exit();
        }
        
        $id = Request :: get(self :: PARAM_SYSTEM_ANNOUNCEMENT_ID);
        $user = $this->get_user();
        
        if ($id)
        {
            $system_announcement_publication = DataManager :: retrieve_by_id(Publication :: class_name(), $id);
            
            $content_object = $system_announcement_publication->get_content_object();
            
            $form = ContentObjectForm :: factory(
                ContentObjectForm :: TYPE_EDIT, 
                $content_object, 
                'edit', 
                'post', 
                $this->get_url(
                    array(
                        self :: PARAM_ACTION => self :: ACTION_EDIT, 
                        self :: PARAM_SYSTEM_ANNOUNCEMENT_ID => $system_announcement_publication->get_id())));
            if ($form->validate() || Request :: get('validated'))
            {
                $form->update_content_object();
                if ($form->is_version())
                {
                    $system_announcement_publication->set_content_object_id($content_object->get_latest_version_id());
                    $system_announcement_publication->update();
                }
                
                $publications = array($system_announcement_publication);
                
                $publication_form = new PublicationForm(
                    PublicationForm :: TYPE_UPDATE, 
                    $publications, 
                    $this->get_url(array('validated' => 1)));
                
                if ($publication_form->validate())
                {
                    $success = $publication_form->handle_form_submit();
                    $this->redirect(
                        Translation :: get(
                            $success ? 'ObjectUpdated' : 'ObjectNotUpdated', 
                            array('OBJECT' => Translation :: get('SystemAnnouncementPublication')), 
                            Utilities :: COMMON_LIBRARIES), 
                        ($success ? false : true), 
                        array(self :: PARAM_ACTION => self :: ACTION_BROWSE));
                }
                else
                {
                    $this->display_header();
                    echo $publication_form->toHtml();
                    $this->display_footer();
                }
            }
            else
            {
                $this->display_header();
                $form->display();
                $this->display_footer();
            }
        }
        else
        {
            $this->display_error_page(
                htmlentities(
                    Translation :: get(
                        'NoObjectSelected', 
                        array('OBJECT' => Translation :: get('SystemAnnouncement')), 
                        Utilities :: COMMON_LIBRARIES)));
        }
    }

    public function get_additional_parameters()
    {
        return array(self :: PARAM_SYSTEM_ANNOUNCEMENT_ID);
    }
}
