<?php
namespace core\admin\announcement;

use common\libraries\Display;
use common\libraries\Breadcrumb;
use common\libraries\Utilities;
use common\libraries\Application;
use common\libraries\Request;
use common\libraries\Translation;
use common\libraries\AdministrationComponent;

class HiderComponent extends Manager implements AdministrationComponent
{

    /**
     * Runs this component and displays its output.
     */
    public function run()
    {
        if (! $this->get_user()->get_platformadmin())
        {
            $this->display_header();
            echo Display :: error_message(Translation :: get('NotAllowed'), true);
            $this->display_footer();
            exit();
        }
        
        $ids = Request :: get(self :: PARAM_SYSTEM_ANNOUNCEMENT_ID);
        $failures = 0;
        
        if (! empty($ids))
        {
            if (! is_array($ids))
            {
                $ids = array($ids);
            }
            
            foreach ($ids as $id)
            {
                $publication = DataManager :: retrieve_by_id(Publication :: class_name(), $id);
                $publication->toggle_visibility();
                
                if (! $publication->update())
                {
                    $failures ++;
                }
            }
            
            if ($failures)
            {
                if (count($ids) == 1)
                {
                    $message = 'ContentObjectNotToggled';
                    $parameter = array('OBJECT' => 'PublicationVisibility');
                }
                else
                {
                    $message = 'ContentObjectsNotToggled';
                    $parameter = array('OBJECTS' => 'PublicationsVisibility');
                }
            }
            else
            {
                if (count($ids) == 1)
                {
                    $message = 'ContentObjectToggled';
                    $parameter = array('OBJECT' => 'PublicationsVisibility');
                }
                else
                {
                    $message = 'ContentObjectsToggled';
                    $parameter = array('OBJECTS' => 'PublicationsVisibility');
                }
            }
            
            $this->redirect(
                Translation :: get($message, $parameter, Utilities :: COMMON_LIBRARIES), 
                ($failures ? true : false), 
                array(self :: PARAM_ACTION => self :: ACTION_BROWSE));
        }
        else
        {
            $this->display_error_page(
                htmlentities(
                    Translation :: get(
                        'NoObjectSelected', 
                        array('OBJECT' => 'Publication'), 
                        Utilities :: COMMON_LIBRARIES)));
        }
    }

    public function get_additional_parameters()
    {
        return array(self :: PARAM_SYSTEM_ANNOUNCEMENT_ID);
    }
}
