<?php
namespace core\admin\announcement;

use common\libraries\Utilities;
use common\libraries\Translation;
use repository\content_object\system_announcement\SystemAnnouncement;
use common\libraries\SubManager;

class Manager extends SubManager
{
    const PARAM_ACTION = 'announcement_action';
    const PARAM_SYSTEM_ANNOUNCEMENT_ID = 'announcement';
    const ACTION_CREATE = 'creator';
    const ACTION_BROWSE = 'browser';
    const ACTION_EDIT = 'editor';
    const ACTION_DELETE = 'deleter';
    const ACTION_VIEW = 'viewer';
    const ACTION_HIDE = 'hider';
    const DEFAULT_ACTION = self :: ACTION_BROWSE;

    public static function get_action_parameter()
    {
        return self :: PARAM_ACTION;
    }

    public function get_default_action()
    {
        return self :: DEFAULT_ACTION;
    }

    public static function launch($application)
    {
        parent :: launch(null, $application);
    }

    /*
     * Inherited
     */
    public static function get_content_object_publication_attributes($object_id, $type = null, $type = null, $offset = null,
        $count = null, $order_property = null)
    {
        return DataManager :: get_content_object_publication_attributes(
            $object_id,
            $type,
            $type,
            $offset,
            $count,
            $order_property);
    }

    /*
     * Inherited
     */
    public static function get_content_object_publication_attribute($publication_id)
    {
        return DataManager :: get_content_object_publication_attribute($publication_id);
    }

    public static function any_content_object_is_published($object_ids)
    {
        return DataManager :: any_content_object_is_published($object_ids);
    }

    public static function count_publication_attributes($user, $type = null, $condition = null)
    {
        return DataManager :: count_publication_attributes($type, $condition);
    }

    public static function delete_content_object_publications($object_id)
    {
        return DataManager :: delete_content_object_publications($object_id);
    }

    public static function delete_content_object_publication($publication_id)
    {
        return DataManager :: delete_content_object_publication($publication_id);
    }

    /**
     * Used to publish system announcements
     */
    public static function get_content_object_publication_locations($content_object)
    {
        $user = \user\DataManager :: retrieve(
            \user\User :: class_name(),
            (int) \common\libraries\Session :: get_user_id());

        if ($user->is_platform_admin())
        {
            $allowed_types = array(SystemAnnouncement :: class_name());

            $type = $content_object->get_type();
            if (in_array($type, $allowed_types))
            {
                $locations = array(Translation :: get('SystemAnnouncements'));
                return $locations;
            }
        }

        return array();
    }

    /**
     * Used to publish system announcements
     */
    public static function publish_content_object($content_object, $location)
    {
        $publication = new Publication();
        $publication->set_content_object_id($content_object->get_id());
        $publication->set_publisher($content_object->get_owner_id());
        $publication->create();

        return Translation :: get(
            'ObjectCreated',
            array('OBJECT' => Translation :: get('Publication')),
            Utilities :: COMMON_LIBRARIES);
    }

    public static function add_publication_attributes_elements($form)
    {
    }
}
