<?php
namespace core\admin\language\upgrade\v4_0_0;

use common\libraries\CommonDataManager;
use common\libraries\package\Package;
use common\libraries\EqualityCondition;
use common\libraries\PropertyConditionVariable;
use common\libraries\StaticConditionVariable;
use common\libraries\DataClassRetrievesParameters;

/**
 *
 * @author Hans De Bisschop
 * @author Magali Gillard
 */
class Upgrader extends \common\libraries\package\Upgrader
{

    /**
     * Constructor
     */
    public function __construct()
    {
        parent :: __construct(\core\admin\AdminDataManager :: get_instance());
    }

    /*
     * (non-PHPdoc) @see \common\libraries\package\Upgrader::run()
     */
    public function run()
    {
        $data_manager = $this->get_data_manager();

        $success = $data_manager->transactional(
            function ($c) use($data_manager)
            {
                $package = Package :: get('core\admin\language');
                $registration = CommonDataManager :: get_registration('admin\language');
                $registration->set_version($package->get_version());
                $registration->set_context($package->get_context());
                if (! $registration->update())
                {
                    return false;
                }

                return true;
            });

        if (! $success)
        {
            return false;
        }

        $condition = new EqualityCondition(
            new PropertyConditionVariable(
                \common\libraries\Setting :: class_name(),
                \common\libraries\Setting :: PROPERTY_CONTEXT),
            new StaticConditionVariable('admin\language'));
        $settings = CommonDataManager :: retrieves(
            \common\libraries\Setting :: class_name(),
            new DataClassRetrievesParameters($condition));

        while ($setting = $settings->next_result())
        {
            $setting->set_context('core\admin\language');
            if (! $setting->update())
            {
                return false;
            }
        }

        return true;
    }
}
