<?php
namespace core\admin;

use common\libraries\Condition;

/**
 *
 * @package admin.lib This is an interface for a data manager for the Home application. Data managers must implement
 *          this class.
 * @author Hans De Bisschop
 */
interface AdminDataManagerInterface
{

    public function initialize();

    public function create_storage_unit($name, $properties, $indexes);

    public function retrieve_feedback_publications($pid, $cid, $application);

    public function retrieve_feedback_publication($id);

    public function retrieve_validation($id);

    public function select_next_display_order($parent_category_id);

    public function delete_category($category);

    public function update_category($category);

    public function create_category($category);

    public function count_categories($conditions = null);

    public function retrieve_categories($condition = null, $offset = null, $count = null, $order_property = null);

    public function delete_validation($validation);

    public function update_validation($validation);

    public function create_validation($validation);

    public function delete_feedback_publication($feeback_publication);

    public function update_feedback_publication($feedback_publication);

    public function create_feedback_publication($feedback_publication);

    public function retrieve_validations($condition = null, $order_by = array (), $offset = 0, $max_objects = -1);

    public function count_validations($condition = null);

    // Dynamic Forms
    public function delete_dynamic_form($dynamic_form);

    public function update_dynamic_form($dynamic_form);

    public function create_dynamic_form($dynamic_form);

    public function count_dynamic_forms($conditions = null);

    public function retrieve_dynamic_forms($condition = null, $offset = null, $count = null, $order_property = null);

    public function delete_dynamic_form_element($dynamic_form_element);

    public function update_dynamic_form_element($dynamic_form_element);

    public function create_dynamic_form_element($dynamic_form_element);

    public function count_dynamic_form_elements($conditions = null);

    public function retrieve_dynamic_form_elements($condition = null, $offset = null, $count = null, $order_property = null);

    public function select_next_dynamic_form_element_order($dynamic_form_id);

    public function delete_dynamic_form_element_option($dynamic_form_element_option);

    public function update_dynamic_form_element_option($dynamic_form_element_option);

    public function create_dynamic_form_element_option($dynamic_form_element_option);

    public function count_dynamic_form_element_options($conditions = null);

    public function retrieve_dynamic_form_element_options($condition = null, $offset = null, $count = null, $order_property = null);

    public function select_next_dynamic_form_element_option_order($dynamic_form_element_id);

    public function delete_all_options_from_form_element($dynamic_form_element_id);

    public function delete_dynamic_form_element_value($dynamic_form_element_value);

    public function update_dynamic_form_element_value($dynamic_form_element_value);

    public function create_dynamic_form_element_value($dynamic_form_element_value);

    public function count_dynamic_form_element_values($conditions = null);

    public function retrieve_dynamic_form_element_values($condition = null, $offset = null, $count = null, $order_property = null);

    public function delete_dynamic_form_element_values_from_form($dynamic_form_id);

    /**
     *
     * @param $id int
     * @return Invitation
     */
    public function retrieve_invitation($id);

    public function retrieve_invitations($condition = null, $order_by = array (), $offset = 0, $max_objects = -1);

    public function create_invitation($invitation);

    public function delete_invitation($invitation);

    /**
     *
     * @param $code string
     * @return Invitation
     */
    public function retrieve_invitation_by_code($code);

    public function delete_feedback_from_publication($application, $publication_id);
}
