<?php

namespace core\admin;

use common\libraries\Utilities;
use common\libraries\Translation;
use common\libraries\FormValidator;
use common\libraries\DatetimeUtilities;
use common\libraries\ClientCache;
use common\libraries\SystemCache;

/**
 * A form to manage cache settings. I.e. to reset the cache date.
 */
class ResourceCacheManagerForm extends FormValidator
{

    /**
     *
     * @param $form_name string The name to use in the form tag.
     * @param $method string The method to use ('post' or 'get').
     * @param $action string The URL to which the form should be submitted.
     */
    public function __construct($form_name, $method = 'post', $action = null)
    {
        parent :: __construct($form_name, $method, $action);
        
        $this->build_form();
        $this->setDefaults();
    }

    public function reset_cache()
    {
        $values = $this->exportValues();
        if (isset($values['reset_client']) && $values['reset_client'])
        {
            SystemCache :: reset_client();
        }
        if (isset($values['reset_system']) && $values['reset_system'])
        {
            SystemCache :: reset_system();
        }
        if (isset($values['reset_user']) && $values['reset_user'])
        {
            SystemCache :: reset_user();
        }
        if (isset($values['reset_application']) && $values['reset_application'])
        {
            SystemCache :: reset_application();
        }
        return true;
    }

    private function build_form()
    {
        $this->addElement('category', Translation :: get('Cache'));
        $this->addElement('html', Translation :: get('ResetCacheInstruction'));
        $this->addElement('static', null, Translation :: get('ClientCacheDate'), $this->get_cache_date());
        $this->addElement('checkbox', 'reset_client', Translation :: get('ResetClientCache'));
        $this->addElement('checkbox', 'reset_system', Translation :: get('ResetSystemCache'));
        $this->addElement('checkbox', 'reset_user', Translation :: get('ResetUserCache'));
        $this->addElement('checkbox', 'reset_application', Translation :: get('ResetApplicationCache'));
        
        $buttons = array();
        $buttons[] = $this->createElement('style_submit_button', 'submit', Translation :: get('Reset', array(), Utilities :: COMMON_LIBRARIES), array(
                'class' => 'positive'));
        // $buttons[] = $this->createElement('style_cancel_button', 'cancel',
        // Translation :: get('Cancel', array(), Utilities :: COMMON_LIBRARIES),
        // array(
        // 'class' => 'normal empty'));
        $this->addGroup($buttons, 'buttons', null, '&nbsp;', false);
        $this->addElement('category');
    }

    private function get_cache_date()
    {
        $result = ClientCache :: get_start_time();
        $result = DatetimeUtilities :: format_locale_date('%Y.%m.%d %H:%M', $result);
        return $result;
    }

}
