<?php
namespace core\admin;
use common\libraries\Utilities;
use common\libraries\CoreApplication;
use common\libraries\Translation;
use common\libraries\PatternMatchCondition;

use reporting\ReportingData;
use reporting\ReportingFormatter;

use tracking\Tracker;

use user\integration\tracking\Visit;
use user\UserManager;
use reporting\ReportingChartFormatter;

class AdminMostUsedCoreApplicationsReportingBlock extends AdminReportingBlock
{

    public function count_data()
    {
        $reporting_data = new ReportingData();
        $reporting_data->set_rows(array(Translation :: get('NumberVisit')));

        $applications = CoreApplication :: get_list();
        foreach ($applications as $application)
        {
            $condition = new PatternMatchCondition(Visit :: PROPERTY_LOCATION, '*application=' . $application . '*');
            $count = Tracker :: count_data(Visit :: CLASS_NAME, UserManager :: APPLICATION_NAME, $condition);

            $reporting_data->add_category($application);
            $reporting_data->add_data_category_row($application, Translation :: get('Number', array(), Utilities :: COMMON_LIBRARIES), $count);
        }
        return $reporting_data;
    }

    public function retrieve_data()
    {
        return $this->count_data();
    }

    public function get_application()
    {
        return AdminManager :: APPLICATION_NAME;
    }

    public function get_available_displaymodes()
    {
        $modes = array();
        $modes[ReportingChartFormatter :: DISPLAY_PIE] = Translation :: get('Chart:Pie', null, Utilities :: COMMON_LIBRARIES);
        $modes[ReportingChartFormatter :: DISPLAY_BAR] = Translation :: get('Chart:Bar', null, Utilities :: COMMON_LIBRARIES);
        $modes[ReportingFormatter :: DISPLAY_TABLE] = Translation :: get('Table', array(), Utilities :: COMMON_LIBRARIES);
        $modes[ReportingFormatter :: DISPLAY_TEXT] = Translation :: get('Text', null, Utilities :: COMMON_LIBRARIES);
        return $modes;
    }
}
