<?php
namespace core\admin;

use common\libraries\CommonDataManager;
use common\libraries\Registration;
use common\libraries\PlatformSetting;
use common\libraries\Translation;
use common\libraries\Theme;

/**
 * Simple connector class to facilitate rendering settings forms by preprocessing data from the datamanagers to a simple
 * array format.
 *
 * @author Hans De Bisschop
 * @package admin.settings $Id: settings_admin_connector.class.php 168 2009-11-12 11:53:23Z vanpouckesven $
 */
class SettingsConnector
{

    public function get_languages()
    {
        return CommonDataManager :: get_languages();
    }

    public function get_themes()
    {
        $options = Theme :: get_themes();

        return $options;
    }

    public function get_time_zones()
    {
        $content = file_get_contents(dirname(__FILE__) . '/timezones.txt');
        $content = explode("\n", $content);

        $timezones = array();

        foreach ($content as $timezone)
        {
            $timezone = trim($timezone);
            $timezones[$timezone] = $timezone;
        }

        return $timezones;
    }

    public function get_active_applications()
    {
        $registrations = CommonDataManager :: get_registrations_by_type(Registration :: TYPE_APPLICATION);

        $options = array();
        $options['home'] = Translation :: get('Homepage', array(), 'home');

        foreach ($registrations as $registration)
        {
            if ($registration->is_active())
            {
                $options[$registration->get_name()] = Translation :: get('TypeName', null, $registration->get_context());
            }
        }

        asort($options);

        return $options;
    }

    public function get_working_hours()
    {
        $start = 0;
        $end = 24;
        $working_hours = array();

        for ($i = $start; $i <= $end; $i ++)
        {
            $working_hours[$i] = $i;
        }

        return $working_hours;
    }

    public function is_allowed_to_change_platform_language()
    {
        return PlatformSetting :: get('allow_user_change_platform_language', 'user') == 1;
    }

    public function is_allowed_to_change_platform_timezone()
    {
        return PlatformSetting :: get('allow_user_change_platform_timezone', 'user') == 1;
    }

    public function is_allowed_to_change_theme()
    {
        return PlatformSetting :: get('allow_user_theme_selection', 'user') == 1;
    }
}
