<?php
namespace core\help;

use common\libraries\CommonDataManager;
use common\libraries\PlatformSetting;
use common\libraries\CoreApplication;
use common\libraries\Translation;
use common\libraries\Session;
use common\libraries\DynamicAction;
use common\libraries\Theme;
use common\libraries\Redirect;
use common\libraries\Application;
use common\libraries\LocalSetting;
use common\libraries\ToolbarItem;
use user\UserDataManager;
use user\User;

/**
 * $Id: help_manager.class.php 226 2009-11-13 14:44:03Z chellee $
 *
 * @package help.lib.help_manager
 */

/**
 * A user manager provides some functionalities to the admin to manage his users. For each functionality a component is
 * available.
 */
class Manager extends CoreApplication
{
    const APPLICATION_NAME = 'help';
    const PARAM_HELP_ITEM = 'help_item';
    const ACTION_UPDATE_HELP_ITEM = 'updater';
    const ACTION_RIGHT_EDITS = 'rights_editor';
    const ACTION_BROWSE_HELP_ITEMS = 'browser';
    const DEFAULT_ACTION = self :: ACTION_BROWSE_HELP_ITEMS;

    public function __construct($user = null)
    {
        parent :: __construct($user);
    }

    public static function get_application_platform_admin_links($application = self :: APPLICATION_NAME)
    {
        $links = array();
        $links[] = new DynamicAction(
            Translation :: get('List'),
            Translation :: get('ListDescription'),
            Theme :: get_image_path() . 'admin/list.png',
            Redirect :: get_link(
                self :: APPLICATION_NAME,
                array(Application :: PARAM_ACTION => self :: ACTION_BROWSE_HELP_ITEMS),
                array(),
                false,
                Redirect :: TYPE_CORE));

        $info = parent :: get_application_platform_admin_links(self :: APPLICATION_NAME);
        $info['links'] = $links;

        return $info;
    }

    public function count_help_items($condition)
    {
        return HelpDataManager :: get_instance()->count_help_items($condition);
    }

    public function retrieve_help_items($condition = null, $offset = null, $count = null, $order_property = null)
    {
        return HelpDataManager :: get_instance()->retrieve_help_items($condition, $offset, $count, $order_property);
    }

    public function retrieve_help_item($name, $language)
    {
        return HelpDataManager :: get_instance()->retrieve_help_item($name, $language);
    }

    public static function get_help_url($name)
    {
        $help_item = self :: get_help_item_by_name($name);
        if ($help_item)
            return '<a class="help" href="' . $help_item->get_url() . '" target="about:blank">' . Translation :: get(
                'Help') . '</a>';
    }

    public static function get_tool_bar_help_item($help_item)
    {
        $hide_empty_pages = PlatformSetting :: get('hide_empty_pages', self :: context());
        $help_item = self :: get_help_item_by_name($help_item[0], $help_item[1]);

        if ($help_item instanceof HelpItem && ($help_item->has_url() || $hide_empty_pages == '0'))
        {
            return new ToolbarItem(
                Translation :: get('Help'),
                Theme :: get_common_image_path() . 'action_help.png',
                $help_item ? $help_item->get_url() : '',
                ToolbarItem :: DISPLAY_ICON_AND_LABEL,
                false,
                'help',
                'about:blank');
        }
        else
        {
            return false;
        }
    }

    private static function get_help_item_by_name($context, $identifier)
    {
        $user_id = Session :: get_user_id();
        $user = UserDataManager :: retrieve_by_id(User :: CLASS_NAME, $user_id);

        $language = LocalSetting :: get('platform_language');

        $help_item = HelpDataManager :: get_instance()->retrieve_help_item_by_context($context, $identifier, $language);

        $autocomplete_page = PlatformSetting :: get('autocomplete_missing_pages', self :: context());
        $autocomplete_languages = PlatformSetting :: get('autocomplete_all_languages', self :: context());

        if ($help_item instanceof HelpItem)
        {
            return $help_item;
        }
        elseif ($autocomplete_page)
        {
            if ($autocomplete_languages)
            {
                $installed_languages = CommonDataManager :: get_languages();

                foreach ($installed_languages as $iso_code => $installed_language)
                {
                    $language_item = HelpDataManager :: get_instance()->retrieve_help_item_by_context(
                        $context,
                        $identifier,
                        $installed_language);

                    if (! $language_item instanceof HelpItem)
                    {
                        $language_item = new HelpItem();
                        $language_item->set_context($context);
                        $language_item->set_identifier($identifier);
                        $language_item->set_language($iso_code);
                        $language_item->create();
                    }

                    if ($installed_language == $language)
                    {
                        $help_item = $language_item;
                    }
                }
            }
            else
            {
                $help_item = new HelpItem();
                $help_item->set_context($context);
                $help_item->set_identifier($identifier);
                $help_item->set_language($language);
                $help_item->create();
            }

            return $help_item;
        }
        else
        {
            return false;
        }
    }

    /**
     * Helper function for the Application class, pending access to class constants via variables in PHP 5.3 e.g. $name
     * = $class :: DEFAULT_ACTION DO NOT USE IN THIS APPLICATION'S CONTEXT Instead use: - self :: DEFAULT_ACTION in the
     * context of this class - YourApplicationManager :: DEFAULT_ACTION in all other application classes
     */
    public function get_default_action()
    {
        return self :: DEFAULT_ACTION;
    }
}
