<?php
namespace core\home\integration\core\home;

use repository\ContentObject;
use common\libraries\Translation;
use common\libraries\EqualityCondition;
use common\libraries\Session;
use common\libraries\AndCondition;
use common\libraries\OrCondition;

/**
 * Simple connector class to facilitate rendering settings forms by preprocessing data from the datamanagers to a simple
 * array format.
 *
 * @author Hans De Bisschop
 */
class Connector
{

    /**
     * Returns a list of objects for the specified types.
     *
     * @param array $types
     * @return array
     */
    public static function get_objects($types)
    {
        $result = array();

        $conditions = array();
        $conditions[] = new EqualityCondition(ContentObject :: PROPERTY_OWNER_ID, Session :: get_user_id());

        $types_condition = array();
        foreach ($types as $type)
        {
            $types_condition[] = new EqualityCondition(ContentObject :: PROPERTY_TYPE, $type);
        }
        $conditions[] = new OrCondition($types_condition);
        $condition = new AndCondition($conditions);

        $objects = \repository\DataManager :: retrieve_active_content_objects(ContentObject :: class_name(), $condition);

        if ($objects->size() == 0)
        {
            $result[0] = Translation :: get('CreateObjectFirst');
        }
        else
        {
            while ($object = $objects->next_result())
            {
                $result[$object->get_id()] = $object->get_title();
            }
        }

        return $result;
    }

    public function get_external_objects()
    {
        return self :: get_objects(External :: get_supported_types());
    }

    /**
     * Returns a list of objects that can be linked to a static block.
     *
     * @return array
     */
    public function get_static_objects()
    {
        $result = array();

        $conditions = array();
        $conditions[] = new EqualityCondition(ContentObject :: PROPERTY_OWNER_ID, Session :: get_user_id());

        $types = StaticContent :: get_supported_types();
        $types_condition = array();
        foreach ($types as $type)
        {
            $types_condition[] = new EqualityCondition(ContentObject :: PROPERTY_TYPE, $type);
        }
        $conditions[] = new OrCondition($types_condition);
        $condition = new AndCondition($conditions);

        $objects = \repository\DataManager :: retrieve_active_content_objects(ContentObject :: class_name(), $condition);

        if ($objects->size() == 0)
        {
            $result[0] = Translation :: get('CreateObjectFirst');
        }
        else
        {
            while ($object = $objects->next_result())
            {
                $result[$object->get_id()] = $object->get_title();
            }
        }

        return $result;
    }
}
