<?php
namespace core\home;

use user\User;
use common\libraries\AndCondition;
use common\libraries\EqualityCondition;
use common\libraries\DataClassCache;
use common\libraries\Session;
use common\libraries\JsonAjaxResult;
use common\libraries\AjaxManager;

/**
 *
 * @package home.ajax
 * @author Hans De Bisschop
 */
class HomeAjaxBlockConfig extends AjaxManager
{
    const PARAM_BLOCK = 'block';
    const PARAM_DATA = 'data';
    const PROPERTY_BLOCK = 'block';

    /*
     * (non-PHPdoc) @see common\libraries.AjaxManager::required_parameters()
     */
    public function required_parameters()
    {
        return array(self :: PARAM_BLOCK, self :: PARAM_DATA);
    }

    /*
     * (non-PHPdoc) @see common\libraries.AjaxManager::run()
     */
    public function run()
    {
        $user_id = DataManager :: determine_user_id();

        if ($user_id === false)
        {
            JsonAjaxResult :: not_allowed();
        }

        $block = intval($this->get_parameter(self :: PARAM_BLOCK));
        $data = $this->get_parameter(self :: PARAM_DATA);

        $block = DataManager :: retrieve(HomeBlock :: class_name(), $block);

        if ($block->get_user() == $user_id)
        {
            $homeblock_config = $block->parse_settings();
            $values = $this->get_parameter(self :: PARAM_DATA);

            $problems = 0;

            foreach ($homeblock_config['settings'] as $category_name => $settings)
            {
                foreach ($settings as $name => $setting)
                {
                    if ($setting['locked'] != 'true')
                    {
                        $conditions = array();
                        $conditions[] = new EqualityCondition(HomeBlockConfig :: PROPERTY_BLOCK_ID, $block->get_id());
                        $conditions[] = new EqualityCondition(HomeBlockConfig :: PROPERTY_VARIABLE, $name);
                        $condition = new AndCondition($conditions);

                        $block_config = DataManager :: retrieve(HomeBlockConfig :: class_name(), $condition);
                        $block_config->set_value($values[$name]);
                        if (! $block_config->update())
                        {
                            $problems ++;
                        }
                    }
                }
            }

            if ($problems > 0)
            {
                JsonAjaxResult :: general_error();
            }
            else
            {
                DataClassCache :: truncates(array(HomeBlock :: class_name(), HomeBlockConfig :: class_name()));
                $user = \user\DataManager :: retrieve(User :: class_name(), (int) Session :: get_user_id());
                $renderer = HomeRenderer :: factory(HomeRenderer :: TYPE_DEFAULT, $user);
                $html = Block :: factory($renderer, $block)->as_html();

                $result = new JsonAjaxResult(200);
                $result->set_property(self :: PROPERTY_BLOCK, $html);
                $result->display();
            }
        }
        else
        {
            JsonAjaxResult :: not_allowed();
        }
    }
}
