<?php
namespace core\home;

use common\libraries\Translation;
use common\libraries\AjaxManager;
use common\libraries\JsonAjaxResult;

/**
 *
 * @author Hans De Bisschop @dependency repository.content_object.assessment_multiple_choice_question;
 */
class HomeAjaxBlockSort extends AjaxManager
{
    const PARAM_COLUMN = 'column';
    const PARAM_ORDER = 'order';

    /*
     * (non-PHPdoc) @see common\libraries.AjaxManager::required_parameters()
     */
    public function required_parameters()
    {
        return array(self :: PARAM_COLUMN, self :: PARAM_ORDER);
    }

    public function get_blocks()
    {
        $block_data = explode('&', $this->get_parameter(self :: PARAM_ORDER));
        $blocks = array();

        foreach ($block_data as $block)
        {
            $block_split = explode('=', $block);
            $blocks[] = $block_split[1];
        }

        return $blocks;
    }

    /*
     * (non-PHPdoc) @see common\libraries.AjaxManager::run()
     */
    public function run()
    {
        $user_id = DataManager :: determine_user_id();

        if ($user_id === false)
        {
            JsonAjaxResult :: not_allowed();
        }

        $column_data = explode('_', $this->get_parameter(self :: PARAM_COLUMN));
        $blocks = $this->get_blocks();

        $column = DataManager :: retrieve(HomeColumn :: class_name(), intval($column_data[2]));

        if ($column->get_user() == $user_id)
        {
            $errors = 0;

            $i = 1;
            foreach ($blocks as $block_id)
            {
                $block = DataManager :: retrieve(HomeBlock :: class_name(), intval($block_id));
                if ($block)
                {
                    $block->set_column($column->get_id());
                    $block->set_sort($i);
                    if (! $block->update())
                    {
                        $errors ++;
                    }
                    $i ++;
                }
            }

            if ($errors > 0)
            {
                JsonAjaxResult :: error(409, Translation :: get('OneOrMoreBlocksNotUpdated'));
            }
            else
            {
                JsonAjaxResult :: success();
            }
        }
        else
        {
            JsonAjaxResult :: not_allowed();
        }
    }
}
