<?php
namespace core\home;

use common\libraries\DataClassRetrievesParameters;
use common\libraries\Application;
use common\libraries\ObjectTableOrder;
use common\libraries\AndCondition;
use common\libraries\EqualityCondition;
use common\libraries\Theme;
use common\libraries\Session;
use common\libraries\Translation;
use common\libraries\AjaxManager;
use common\libraries\JsonAjaxResult;

/**
 *
 * @author Hans De Bisschop
 */
class HomeAjaxColumnAdd extends AjaxManager
{
    const PARAM_ROW = 'row';
    const PROPERTY_HTML = 'html';
    const PROPERTY_WIDTH = 'width';

    /*
     * (non-PHPdoc) @see common\libraries.AjaxManager::required_parameters()
     */
    public function required_parameters()
    {
        return array(self :: PARAM_ROW);
    }

    /*
     * (non-PHPdoc) @see common\libraries.AjaxManager::run()
     */
    public function run()
    {
        $user_id = DataManager :: determine_user_id();

        if ($user_id === false)
        {
            JsonAjaxResult :: not_allowed();
        }

        $post_row = $this->get_parameter(self :: PARAM_ROW);

        if (isset($post_row))
        {
            $row_data = explode('_', $this->get_parameter(self :: PARAM_ROW));
            $row_id = $row_data[2];

            // Retrieve the columns of the current row to alter their width
            $conditions = array();
            $conditions[] = new EqualityCondition(HomeColumn :: PROPERTY_ROW, $row_id);
            $conditions[] = new EqualityCondition(HomeColumn :: PROPERTY_USER, $user_id);
            // $conditions[] = new InequalityCondition(HomeColumn :: PROPERTY_WIDTH, InequalityCondition ::
            // GREATER_THAN_OR_EQUAL, '25');
            $condition = new AndCondition($conditions);
            $parameters = new DataClassRetrievesParameters(
                $condition,
                null,
                null,
                new ObjectTableOrder(HomeColumn :: PROPERTY_WIDTH, SORT_DESC));
            $columns = DataManager :: retrieves(HomeColumn :: class_name(), $parameters);

            $width_conditions = array();
            $width_conditions[] = new EqualityCondition(HomeColumn :: PROPERTY_ROW, $row_id);
            $width_conditions[] = new EqualityCondition(HomeColumn :: PROPERTY_USER, $user_id);
            $width_condition = new AndCondition($width_conditions);
            $columns_width = DataManager :: retrieves(HomeColumn :: class_name(), $width_condition);

            $width_total = $columns_width->size() - 1;
            while ($col = $columns_width->next_result())
            {
                $width_total += $col->get_width();
            }

            // Create the new column + a dummy block for it
            $new_column = new HomeColumn();
            $new_column->set_row($row_id);
            $new_column->set_title(Translation :: get('NewColumn'));
            $new_column->set_width('19');
            $new_column->set_user($user_id);
            if (! $new_column->create())
            {
                JsonAjaxResult :: general_error(Translation :: get('ColumnNotAdded'));
            }

            // // $block = new HomeBlock();
            // // $block->set_column($new_column->get_id());
            // // $block->set_title(Translation :: get('DummyBlock'));
            // // $registration = DataManager ::
            // retrieve_home_block_registration_by_context_and_block('repository\content_object\link', 'linker');
            // // $block->set_registration_id($registration->get_id());
            // // $block->set_visibility('1');
            // // $block->set_user($user_id);
            // // if (! $block->create())
            // // {
            // // JsonAjaxResult :: general_error(Translation :: get('ColumnBlockNotAdded'));
            // // }
            //
            // $user = UserDataManager :: get_instance()->retrieve_user(Session :: get_user_id());
            //
            // $home_renderer = HomeRenderer :: factory(HomeRenderer :: TYPE_DEFAULT, $user);
            // $block_component = Block :: factory($home_renderer, $block);

            // Render the actual html to be displayed
            $html[] = '<div class="portal_column" id="portal_column_' . $new_column->get_id() . '" style="width: ' .
                 $new_column->get_width() . '%;">';
            // $html[] = $block_component->as_html();

            $html[] = '<div class="empty_portal_column" style="display:block;">';
            $html[] = htmlspecialchars(Translation :: get('EmptyColumnText'));
            $img = Theme :: get_image_path(Application :: determine_namespace('home')) . 'action_remove_column.png';
            $html[] = '<div class="deleteColumn"><a href="#"><img src="' . $img . '" alt="' .
                 Translation :: get('RemoveColumn') . '"/></a></div>';
            $html[] = '</div>';
            $html[] = '</div>';

            // Update the older columns width and add them to the JSON object
            $border = 1;
            $free_width = max(100 - $width_total, 0);
            $width_to_remove = max(20 - $free_width, 0);

            while ($column = $columns->next_result())
            {
                if ($width_to_remove > 0)
                {
                    $delta = max($column->get_width() - 19, 0);
                    $delta = min($width_to_remove, $delta);
                    if ($delta > 0)
                    {
                        $column->set_width($column->get_width() - $delta);
                        $column->update();
                        $width_to_remove = max($width_to_remove - $delta - $border, 0);
                    }
                }
            }

            $width_conditions = array();
            $width_conditions[] = new EqualityCondition(HomeColumn :: PROPERTY_ROW, $row_id);
            $width_conditions[] = new EqualityCondition(HomeColumn :: PROPERTY_USER, $user_id);
            $width_condition = new AndCondition($width_conditions);

            $columns_width = DataManager :: retrieves(HomeColumn :: class_name(), $width_condition);
            $widths = array();
            while ($col = $columns_width->next_result())
            {
                $widths['portal_column_' . $col->get_id()] = $col->get_width();
            }

            $result = new JsonAjaxResult(200);
            $result->set_property(self :: PROPERTY_HTML, implode("\n", $html));
            $result->set_property(self :: PROPERTY_WIDTH, $widths);
            $result->display();
        }
        else
        {
            JsonAjaxResult :: bad_request();
        }
    }
}
