<?php
namespace core\home;

use common\libraries\CoreApplication;
use common\libraries\Translation;
use common\libraries\DynamicAction;
use common\libraries\Theme;
use common\libraries\Redirect;
use common\libraries\Application;
use common\libraries\package\PackageList;
use common\libraries\CommonDataManager;

/**
 * $Id: home_manager.class.php 227 2009-11-13 14:45:05Z kariboe $
 *
 * @package home.lib.home_manager
 */

/**
 * A user manager provides some functionalities to the admin to manage his users. For each functionality a component is
 * available.
 */
class HomeManager extends CoreApplication
{
    const APPLICATION_NAME = 'home';
    const PARAM_HOME_ID = 'id';
    const PARAM_HOME_TYPE = 'type';
    const PARAM_DIRECTION = 'direction';
    const PARAM_TAB_ID = 'tab';
    const PARAM_OBJECT_ID = 'object_id';
    const PARAM_PARENT_ID = 'parent_id';
    const ACTION_VIEW_HOME = 'home';
    const ACTION_MANAGE_HOME = 'manager';
    const ACTION_EDIT_HOME = 'editor';
    const ACTION_DELETE_HOME = 'deleter';
    const ACTION_MOVE_HOME = 'mover';
    const ACTION_CREATE_HOME = 'creator';
    const ACTION_CONFIGURE_HOME = 'configurer';
    const ACTION_EDIT_HOME_ADMIN = 'admin_editor';
    const ACTION_EDIT_HOME_PERSONAL = 'home_editor';
    const ACTION_CONFIGURE_HOME_ADMIN = 'admin_configurer';
    const ACTION_CONFIGURE_HOME_PERSONAL = 'home_configurer';
    const ACTION_TRUNCATE = 'truncater';
    const ACTION_PERSONAL = 'personal';
    const ACTION_VIEW_ATTACHMENT = 'attachment_viewer';
    const DEFAULT_ACTION = self :: ACTION_VIEW_HOME;
    const TYPE_BLOCK = 'block';
    const TYPE_COLUMN = 'column';
    const TYPE_ROW = 'row';
    const TYPE_TAB = 'tab';

    private $parameters;

    private $user;

    private $breadcrumbs;

    public function __construct($user = null)
    {
        parent :: __construct($user);
    }

    public static function get_application_platform_admin_links($application = self :: APPLICATION_NAME)
    {
        $links = array();
        $links[] = new DynamicAction(
            Translation :: get('ManageDefault'),
            Translation :: get('ManageDefaultDescription'),
            Theme :: get_image_path() . 'admin/build.png',
            Redirect :: get_link(
                self :: APPLICATION_NAME,
                array(Application :: PARAM_ACTION => self :: ACTION_MANAGE_HOME),
                array(),
                false,
                Redirect :: TYPE_CORE));
        $info = parent :: get_application_platform_admin_links(self :: APPLICATION_NAME);
        $info['links'] = $links;

        return $info;
    }

    public function get_home_tab_creation_url()
    {
        return $this->get_url(
            array(self :: PARAM_ACTION => self :: ACTION_CREATE_HOME, self :: PARAM_HOME_TYPE => self :: TYPE_TAB));
    }

    public function get_home_row_creation_url()
    {
        return $this->get_url(
            array(self :: PARAM_ACTION => self :: ACTION_CREATE_HOME, self :: PARAM_HOME_TYPE => self :: TYPE_ROW));
    }

    public function get_home_block_creation_url()
    {
        return $this->get_url(
            array(self :: PARAM_ACTION => self :: ACTION_CREATE_HOME, self :: PARAM_HOME_TYPE => self :: TYPE_BLOCK));
    }

    public function get_home_column_creation_url()
    {
        return $this->get_url(
            array(self :: PARAM_ACTION => self :: ACTION_CREATE_HOME, self :: PARAM_HOME_TYPE => self :: TYPE_COLUMN));
    }

    public function get_home_tab_editing_url($home_tab)
    {
        return $this->get_url(
            array(
                self :: PARAM_ACTION => self :: ACTION_EDIT_HOME_ADMIN,
                self :: PARAM_HOME_TYPE => self :: TYPE_TAB,
                self :: PARAM_HOME_ID => $home_tab->get_id()));
    }

    public function get_home_row_editing_url($home_row)
    {
        return $this->get_url(
            array(
                self :: PARAM_ACTION => self :: ACTION_EDIT_HOME_ADMIN,
                self :: PARAM_HOME_TYPE => self :: TYPE_ROW,
                self :: PARAM_HOME_ID => $home_row->get_id()));
    }

    public function get_home_block_editing_url($home_block)
    {
        return $this->get_url(
            array(
                self :: PARAM_ACTION => self :: ACTION_EDIT_HOME_ADMIN,
                self :: PARAM_HOME_TYPE => self :: TYPE_BLOCK,
                self :: PARAM_HOME_ID => $home_block->get_id()));
    }

    public function get_home_block_configuring_url($home_block)
    {
        return $this->get_url(
            array(
                self :: PARAM_ACTION => self :: ACTION_CONFIGURE_HOME_ADMIN,
                self :: PARAM_HOME_ID => $home_block->get_id()));
    }

    public function get_home_column_editing_url($home_column)
    {
        return $this->get_url(
            array(
                self :: PARAM_ACTION => self :: ACTION_EDIT_HOME_ADMIN,
                self :: PARAM_HOME_TYPE => self :: TYPE_COLUMN,
                self :: PARAM_HOME_ID => $home_column->get_id()));
    }

    public function get_home_tab_viewing_url($home_tab)
    {
        return $this->get_url(array(self :: PARAM_TAB_ID => $home_tab->get_id()));
    }

    public function get_home_tab_deleting_url($home_tab)
    {
        return $this->get_url(
            array(
                self :: PARAM_ACTION => self :: ACTION_DELETE_HOME,
                self :: PARAM_HOME_TYPE => self :: TYPE_TAB,
                self :: PARAM_HOME_ID => $home_tab->get_id()));
    }

    public function get_home_row_deleting_url($home_row)
    {
        return $this->get_url(
            array(
                self :: PARAM_ACTION => self :: ACTION_DELETE_HOME,
                self :: PARAM_HOME_TYPE => self :: TYPE_ROW,
                self :: PARAM_HOME_ID => $home_row->get_id()));
    }

    public function get_home_block_deleting_url($home_block)
    {
        return $this->get_url(
            array(
                self :: PARAM_ACTION => self :: ACTION_DELETE_HOME,
                self :: PARAM_HOME_TYPE => self :: TYPE_BLOCK,
                self :: PARAM_HOME_ID => $home_block->get_id()));
    }

    public function get_home_column_deleting_url($home_column)
    {
        return $this->get_url(
            array(
                self :: PARAM_ACTION => self :: ACTION_DELETE_HOME,
                self :: PARAM_HOME_TYPE => self :: TYPE_COLUMN,
                self :: PARAM_HOME_ID => $home_column->get_id()));
    }

    public function get_home_column_moving_url($home_column, $direction)
    {
        return $this->get_url(
            array(
                self :: PARAM_ACTION => self :: ACTION_MOVE_HOME,
                self :: PARAM_HOME_TYPE => self :: TYPE_COLUMN,
                self :: PARAM_HOME_ID => $home_column->get_id(),
                self :: PARAM_DIRECTION => $direction));
    }

    public function get_home_block_moving_url($home_block, $direction)
    {
        return $this->get_url(
            array(
                self :: PARAM_ACTION => self :: ACTION_MOVE_HOME,
                self :: PARAM_HOME_TYPE => self :: TYPE_BLOCK,
                self :: PARAM_HOME_ID => $home_block->get_id(),
                self :: PARAM_DIRECTION => $direction));
    }

    public function get_home_tab_moving_url($home_tab, $direction)
    {
        return $this->get_url(
            array(
                self :: PARAM_ACTION => self :: ACTION_MOVE_HOME,
                self :: PARAM_HOME_TYPE => self :: TYPE_TAB,
                self :: PARAM_HOME_ID => $home_tab->get_id(),
                self :: PARAM_DIRECTION => $direction));
    }

    public function get_home_row_moving_url($home_row, $direction)
    {
        return $this->get_url(
            array(
                self :: PARAM_ACTION => self :: ACTION_MOVE_HOME,
                self :: PARAM_HOME_TYPE => self :: TYPE_ROW,
                self :: PARAM_HOME_ID => $home_row->get_id(),
                self :: PARAM_DIRECTION => $direction));
    }

    /**
     * Helper function for the Application class, pending access to class constants via variables in PHP 5.3 e.g. $name
     * = $class :: DEFAULT_ACTION DO NOT USE IN THIS APPLICATION'S CONTEXT Instead use: - self :: DEFAULT_ACTION in the
     * context of this class - YourApplicationManager :: DEFAULT_ACTION in all other application classes
     */
    public function get_default_action()
    {
        return self :: DEFAULT_ACTION;
    }

    public static function get_installable_application_packages($include_installed = false)
    {
        $package_list = new PackageList(
            self :: context(),
            Translation :: get('TypeName', null, __NAMESPACE__),
            Theme :: get_image_path() . 'logo/16.png');

        $integration_list = new \common\libraries\package\PackageList(
            self :: context() . '\integration',
            Translation :: get('Integrations', null, __NAMESPACE__));

        $core_list = new \common\libraries\package\PackageList(
            self :: context() . '\integration\core',
            Translation :: get('Core', null, __NAMESPACE__));

        if (! CommonDataManager :: get_registration(self :: context() . '\integration\core\home') || $include_installed)
        {
            $core_list->add_package(self :: context() . '\integration\core\home');
        }

        $integration_list->add_child($core_list);
        $package_list->add_child($integration_list);

        return $package_list;
    }
}
