<?php
namespace core\home;

use common\libraries\StaticConditionVariable;
use common\libraries\PropertyConditionVariable;
use common\libraries\EqualityCondition;
use common\libraries\Translation;

/**
 * Extension of the generic installer for home blocks
 *
 * @author Hans De Bisschop
 */
abstract class HomeRemover extends \common\libraries\package\Remover
{

    /**
     * Perform additional installation steps
     *
     * @return boolean
     */
    public function extra()
    {
        if (! $this->deregister_home())
        {
            return $this->failed(Translation :: get('HomeFailed', null, HomeManager :: APPLICATION_NAME));
        }

        return true;
    }

    /**
     *
     * @return boolean
     */
    public function deregister_home()
    {
        $condition = new EqualityCondition(
                new PropertyConditionVariable(HomeBlockRegistration :: class_name(),
                        HomeBlockRegistration :: PROPERTY_CONTEXT), new StaticConditionVariable(static :: context()));
        $registrations = DataManager :: retrieves(HomeBlockRegistration :: class_name(), $condition);

        while ($registration = $registrations->next_result())
        {
            if ($registration->delete())
            {
                $this->add_message(self :: TYPE_NORMAL,
                        Translation :: get('DeregisteredBlock') . ': <em>' . $registration->get_block() . '</em>');
            }
            else
            {
                $this->add_message(self :: TYPE_ERROR,
                        Translation :: get('BlockDeregistrationFailed') . ': <em>' . $registration->get_block() . '</em>');
                return false;
            }
        }

        $this->add_message(self :: TYPE_NORMAL, Translation :: get('BlocksRemoved'));

        return true;
    }
}
