<?php
namespace core\install;

class CommandLineInstaller implements InstallerObserver
{

    const DEFAULT_INSTALL_VALUES_FILE = 'command_line_configuration.dist.php';

    private $config_file;

    private $installer;

    public function __construct($config_file)
    {
        $this->config_file = $config_file;
    }

    public function run()
    {
        $installer_config = new InstallerConfig();
        $default_config_file = __DIR__ . '/../' . self :: DEFAULT_INSTALL_VALUES_FILE;
        $installer_config->load_config_file($default_config_file);
        $installer_config->load_config_file($this->config_file);

        $installer_factory = new Factory();
        $this->installer = $installer_factory->build_installer($installer_config);

        $this->installer->add_observer($this);
        $this->installer->perform_install();
    }

    private function check_result(InstallerStepResult $result)
    {
        if ($result->get_success())
        {
            return "Ok";
        }

        $reason = implode(", ", $result->get_messages());
        return "Ko ({$reason})";
    }

    public function before_install()
    {
        echo "install started ...\n\n";
    }

    public function before_preprod()
    {
        echo "\tPRE-PRODUCTION\n";
    }

    public function preprod_db_created(InstallerStepResult $result)
    {
        echo "\t\t DB created ... " . $this->check_result($result) . "\n";
    }

    public function preprod_config_file_written(InstallerStepResult $result)
    {
        echo "\t\t Config File Written ... " . $this->check_result($result) . "\n";
    }

    public function after_preprod()
    {
        echo "\n";
    }

    public function before_filesystem_prepared()
    {
        echo "\tFILE SYSTEM PREPARATION\n";
    }

    public function after_filesystem_prepared(InstallerStepResult $result)
    {
        echo "\t\t File system prepared ... " . $this->check_result($result) . "\n";
    }

    public function after_install()
    {
        echo "\n\nInstallation completed !\n";
    }

    public function before_packages_install()
    {
        echo "\PACKAGES INSTALLATION\n";
    }

    public function after_packages_install()
    {
        echo "\n";
    }

    public function before_package_install($context)
    {
        echo "\t\t Installing package {$context} ... ";
    }

    public function after_package_install(InstallerStepResult $result)
    {
        echo $this->check_result($result) . "\n";
    }
}
