<?php
namespace core\install;

use common\libraries\ResourceManager;
use common\libraries\Path;
use common\libraries\Translation;
use HTML_QuickForm_Action;
use Exception;

/**
 * $Id: install_wizard_process.class.php 225 2009-11-13 14:43:20Z vanpouckesven $
 *
 * @package install.lib.installmanager.component.inc.wizard
 */

/**
 * This class implements the action to take after the user has completed a course maintenance wizard
 */
class InstallWizardProcess extends HTML_QuickForm_Action implements InstallerObserver
{

    /**
     * The repository tool in which the wizard runs.
     */
    private $parent;

    private $current_step;

    private $counter;

    private $optional_application_counter;

    private $optional;

    /**
     * Constructor
     *
     * @param $parent Tool The repository tool in which the wizard runs.
     */
    public function __construct($parent)
    {
        $this->parent = $parent;
        $this->current_step = "";
        $this->counter = 0;
        $this->optional_application_counter = 0;
        $this->optional = false;
    }

    public function perform($page, $actionName)
    {
        $this->installer = $this->build_installer($page);
        $this->installer->add_observer($this);

        $this->display_header($page);
        try
        {
            $this->installer->perform_install();
            // $page->controller->container(true);
        }
        catch (InstallFailedException $exception)
        {
            // TODO change this for a real error box
            $this->process_result(
                Translation :: get('PlatformInstallFailed') . ' - ' .
                $exception->get_package(), false, $exception->getMessage(),
                'resources/images/aqua/place_failed.png'
            );
        }
        $this->parent->display_footer();
    }

    private function build_installer($page)
    {
        if (array_key_exists('config_file', $_FILES))
        {
            $values = array();

            // TODO check why this gives install errors for missing path ??
            if ($_FILES['config_file']['tmp_name'])
            {
                require_once ($_FILES['config_file']['tmp_name']);
            }
        }
        else
        {
            $values = $page->controller->exportValues();
        }
        $url_append = str_replace('/install/index.php', '', $_SERVER['REQUEST_URI']);
        $values['url_append'] = $url_append;

        $factory = new Factory();
        $installer = $factory->build_installer_from_array($values);
        unset($values);
        return $installer;
    }

    public function display_header($page)
    {
        $values = $this->values;

        $this->parent->display_header();
        $all_pages = $page->controller->_pages;

        $total_number_of_pages = count($all_pages) + 1;

        $html[] = '<div id="progressbox">';
        $html[] = '<ul id="progresstrail">';

        $page_number = 1;

        foreach ($all_pages as $page)
        {
            $html[] = '<li class="active"><a href="#">' . $page_number . '.&nbsp;&nbsp;' . $page->get_breadcrumb() . '</a></li>';
            $page_number ++;
        }

        $html[] = '<li class="active"><a href="#">' . $total_number_of_pages . '.&nbsp;&nbsp;' . Translation :: get(
                'Installation') . '</a></li>';
        $html[] = '</ul>';
        $html[] = '<div class="clear"></div>';
        $html[] = '</div>';

        $html[] = '<div id="theForm" style="margin: 10px;">';
        $html[] = '<div id="select" class="row"><div class="formc formc_no_margin">';
        $html[] = '<b>' . Translation :: get('Step') . ' ' . $total_number_of_pages . ' ' . Translation :: get('of') . ' ' . $total_number_of_pages . ' &ndash; ' . Translation :: get(
                'Installation') . '</b><br />';

        $html[] = Translation :: get('InstallationDescription');
        $html[] = '</div>';
        $html[] = '</div></div>';
        $html[] = '<div class="clear"></div>';
        // $html[] = '<a href="#" id="showall">' . Translation :: get('ShowAll')
        // . '</a> - <a href="#" id="hideall">' . Translation :: get('HideAll')
        // . '</a><br />';
        $html[] = ResourceManager :: get_instance()->get_resource_html('resources/javascript/install_process.js');

        echo implode("\n", $html);
    }

    public function process_result($title, $result, $message, $image)
    {
        echo $this->display_install_block_header($title, $result, $image);
        echo $message;
        echo $this->display_install_block_footer();
    }

    public function display_install_block_header($title, $result, $image)
    {
        $counter = $this->counter;

        $result_class = ($result ? 'installation-step-successful' : 'installation-step-failed');

        $html = array();
        $html[] = '<div class="installation-step installation-step-collapsed ' . $result_class . '" style="background-image: url(' . $image . ');">';
        $html[] = '<div class="title">' . $title . '</div>';
        $html[] = '<div class="description">';

        return implode("\n", $html);
    }

    public function display_install_block_footer()
    {
        $html = array();
        $html[] = '</div>';
        $html[] = '</div>';
        return implode("\n", $html);
    }

    public function before_filesystem_prepared()
    {
    }

    public function after_filesystem_prepared(InstallerStepResult $result)
    {
        $image = 'resources/images/aqua/place_folder.png';
        $this->process_result(Translation :: get('Folders'), $result->get_success(),
                implode('<br />' . "\n", $result->get_messages()), $image);
        flush();
    }

    public function after_preprod()
    {
        echo '<div class="clear"></div>';
    }

    public function before_install()
    {
    }

    public function after_install()
    {
        $message = '<a href="../../index.php">' . Translation :: get('GoToYourNewlyCreatedPortal') . '</a>';
        $image = 'resources/images/aqua/place_finished.png';
        $this->process_result(Translation :: get('InstallationFinished'), true, $message, $image);
        flush();
    }

    public function before_preprod()
    {
        echo '<h3>' . Translation :: get('PreProduction') . '</h3>';
    }

    public function before_packages_install()
    {
        echo '<h3>' . Translation :: get('Packages') . '</h3>';
    }

    public function before_package_install($context)
    {
    }

    public function after_package_install(InstallerStepResult $result)
    {
        $image = Path :: get(WEB_PATH) . '../../' . Path :: namespace_to_path($result->get_context()) . '/resources/images/aqua/logo/22.png';
        $title = Translation :: get('TypeName', null, $result->get_context());

        $this->process_result($title, $result->get_success(), implode('<br />' . "\n", $result->get_messages()), $image);
        flush();
    }

    public function after_packages_install()
    {
        echo '<div class="clear"></div>';
    }

    public function preprod_config_file_written(InstallerStepResult $result)
    {
        $image = 'resources/images/aqua/place_config.png';
        $this->process_result(Translation :: get('Configuration'), $result->get_success(),
                implode('<br />' . "\n", $result->get_messages()), $image);
        flush();
    }

    public function preprod_db_created(InstallerStepResult $result)
    {
        $image = 'resources/images/aqua/place_database.png';
        $this->process_result(Translation :: get('Database'), $result->get_success(),
                implode('<br />' . "\n", $result->get_messages()), $image);
        flush();
    }
}
