<?php
namespace core\lynx\manager;

use common\libraries\Theme;
use common\libraries\Utilities;
use core\lynx\Action;
use common\libraries\CommonDataManager;
use common\libraries\Translation;

set_time_limit(0);

/**
 * Package installation
 *
 * @author Hans De Bisschop - Erasmus Hogeschool Brussel
 * @author Magali Gillard - Erasmus Hogeschool Brussel
 * @author Sven Vanpoucke - Hogeschool Gent - Cleanup, code refactoring and bugfixes, comments
 * @package admin.lib.package_installer
 */
class PackageRemover extends Action
{

    /**
     * Runs the package remover
     *
     * @return boolean
     */
    public function run()
    {
        if ($this->initialize() && $this->process())
        {
            $title = Translation :: get('Finished', null, Utilities :: get_namespace_parent(__NAMESPACE__));
            $image = Theme :: get_image_path(Utilities :: get_namespace_parent(__NAMESPACE__)) .
                 'package_action/finished.png';
            return $this->action_successful($title, $image, Translation :: get('PackageCompletelyRemoved'));
        }
        else
        {
            $title = Translation :: get('Failed', null, Utilities :: get_namespace_parent(__NAMESPACE__));
            $image = Theme :: get_image_path(Utilities :: get_namespace_parent(__NAMESPACE__)) .
                 'package_action/failed.png';
            return $this->action_failed($title, $image, Translation :: get('PackageRemoveFailed'));
        }
    }

    /**
     * Initializes the package installer
     *
     * @return boolean
     */
    public function initialize()
    {
        $title = Translation :: get('Initialization', null, Utilities :: get_namespace_parent(__NAMESPACE__));
        $image = Theme :: get_image_path(Utilities :: get_namespace_parent(__NAMESPACE__)) .
             'package_action/initialization.png';

        if (! $this->get_package() instanceof \common\libraries\package\Package)
        {
            return $this->action_failed($title, $image, Translation :: get('PackageAttributesNotFound'));
        }
        else
        {
            $this->add_message(Translation :: get('PackageAttributesFound'));
        }

        // Check registration
        if (! $this->is_package_registered())
        {
            return $this->action_failed($title, $image, Translation :: get('PackageIsNotInstalled'));
        }
        else
        {
            $this->add_message(Translation :: get('PackageNotYetRemoved'));
        }

        return $this->action_successful($title, $image, Translation :: get('PackageRemoveInitialized'));
    }

    /**
     * Checks if the package is registered
     *
     * @return boolean
     */
    public function is_package_registered()
    {
        return CommonDataManager :: is_registered($this->get_package()->get_context());
    }

    /**
     * Installs the package
     *
     * @return boolean
     */
    public function process()
    {
        $this->process_additional_packages($this->get_package()->get_context());

        return true;
    }

    public function process_additional_packages($context)
    {
        $remover = \common\libraries\package\Remover :: factory($context);

        $additional_packages = $remover->get_additional_packages();

        foreach ($additional_packages as $additional_package)
        {
            $this->process_additional_packages($additional_package);
        }

        $title = Translation :: get(
            'Removal',
            array('PACKAGE' => Translation :: get('TypeName', null, $this->get_package()->get_context())),
            Utilities :: get_namespace_parent(__NAMESPACE__));
        $image = Theme :: get_image_path(Utilities :: get_namespace_parent(__NAMESPACE__)) . 'package_action/removal.png';

        if (! $remover->run())
        {
            $this->add_message($remover->retrieve_message());
            return $this->action_failed($title, $image, Translation :: get('InitializationFailed'));
        }
        else
        {
            $this->add_message($remover->retrieve_message());
            $this->action_successful($title, $image);
        }
    }
}
