<?php
namespace core\lynx\manager;

use common\libraries\TreeMenuRenderer;
use common\libraries\BreadcrumbTrail;
use common\libraries\OptionsMenuRenderer;
use common\libraries\Utilities;
use HTML_Menu;
use HTML_Menu_ArrayRenderer;

class PackageTypeMenu extends HTML_Menu
{

    private $format;

    private $array_renderer;

    public function __construct($current_type, $format)
    {
        $this->format = $format;

        parent :: __construct(array($this->get_items(\common\libraries\package\PackageList :: get())));

        $this->array_renderer = new HTML_Menu_ArrayRenderer();
        $this->forceCurrentUrl($this->get_url($current_type));
    }

    private function get_items(\common\libraries\package\PackageList $package_list)
    {
        $item = array();
        $item['class'] = 'category';

        if (! is_null($package_list->get_type_icon()))
        {
            $item['style'] = 'background-image: url(' . $package_list->get_type_icon() . ')';
        }

        $item['title'] = $package_list->get_type_name();
        $item['url'] = $this->get_url($package_list->get_type());
        $item[OptionsMenuRenderer :: KEY_ID] = $package_list->get_type();

        $sub_items = array();

        foreach ($package_list->get_children() as $child)
        {
            $sub_items[] = $this->get_items($child);
        }

        // var_dump($sub_items);

        if (count($sub_items) > 0)
        {
            usort($sub_items, array('\core\lynx\manager\PackageTypeMenu', 'compare_items'));

            $item['sub'] = $sub_items;
        }

        return $item;
    }

    public static function compare_items($item_one, $item_two)
    {
        $item_one = strtolower($item_one['title']);
        $item_two = strtolower($item_two['title']);

        if ($item_one == $item_two)
        {
            return 0;
        }

        return ($item_one > $item_two) ? + 1 : - 1;
    }

    private function get_url($type)
    {
        return (str_replace('__type__', $type, $this->format));
    }

    public function get_breadcrumbs()
    {
        $trail = BreadcrumbTrail :: get_instance();
        // $this->render($this->array_renderer, 'urhere');
        // $breadcrumbs = $this->array_renderer->toArray();
        // foreach ($breadcrumbs as $crumb)
        // {
        // $str = Translation :: get('MyRepository');
        // if (substr($crumb['title'], 0, strlen($str)) == $str)
        // continue;
        // $trail->add(new Breadcrumb($crumb['url'], substr($crumb['title'], 0, strpos($crumb['title'], '('))));
        // }
        return $trail;
    }

    public function render_as_tree()
    {
        $renderer = new TreeMenuRenderer(Utilities :: get_classname_from_namespace(__CLASS__, true));
        $this->render($renderer, 'sitemap');
        return $renderer->toHTML();
    }
}
