<?php
namespace core\lynx;

use common\libraries\Display;
use core\lynx\manager\PackageUpgrader;
use common\libraries\CoreApplication;
use common\libraries\Utilities;
use common\libraries\CommonDataManager;
use common\libraries\Theme;
use common\libraries\Translation;
use common\libraries\ResourceManager;

class UpgraderComponent extends Manager
{

    private $packages = array();

    public function run()
    {
        set_time_limit(0);
        ini_set("memory_limit", - 1);

        $this->initialize();

        Display :: header();

        while (($package = $this->get_next_package()) != null)
        {
            $package_upgrader = new PackageUpgrader($package, false);
            $success = $package_upgrader->run();

            $upgrader = $package_upgrader->get_upgrader();
            if($upgrader)
            {
                $this->add_packages($upgrader->get_additional_packages());
            }

            $image = Theme :: get_image_path($package) . 'logo/48.png';
            $title = Translation :: get('TypeName', null, $package);
            $result = $package_upgrader->get_result(true);

            echo $this->render_upgrade_step($image, $title, $result);

            flush();

            if (! $success)
            {
                break;
            }
        }

        if ($success)
        {
            echo $this->upgrade_successfull();
        }

        echo ResourceManager :: get_instance()->get_resource_html('core/lynx/resources/javascript/lynx_process.js');

        Display :: footer();
    }

    public function render_upgrade_step($image, $title, $result)
    {
        $html = array();
        $html[] = '<div class="package_upgrade upgrade-step-collapsed" style="background-image: url(' . $image . ');">';
        $html[] = '<div class="package"><div class="title">' . $title . '</div></div>';
        $html[] = '<div class="description">';
        $html[] = $result;
        $html[] = '</div>';
        $html[] = '</div>';
        return implode("\n", $html);
    }

    /**
     *
     * @return multitype:string
     */
    public function get_packages()
    {
        return $this->packages;
    }

    /**
     *
     * @param multitype:string $packages
     */
    public function set_packages($packages)
    {
        $this->packages = $packages;
    }

    /**
     *
     * @param string $context
     */
    public function add_package($context)
    {
        array_push($this->packages, $context);
    }

    /**
     *
     * @param multitype:string $packages
     */
    public function add_packages($packages)
    {
        foreach ($packages as $package)
        {
            $this->add_package($package);
        }
    }

    /**
     *
     * @return string
     */
    public function get_next_package()
    {
        return array_shift($this->packages);
    }

    public function initialize()
    {
        $this->add_package('common\libraries');
        $this->add_packages(CoreApplication :: get_packages_from_filesystem());
        $this->add_packages(Utilities :: get_common_extensions_packages_from_filesystem());

        $external_repository_registrations = CommonDataManager :: get_registrations_by_type(
            'common\extensions\external_repository_manager\implementation');
        foreach ($external_repository_registrations as $external_repository_registration)
        {
            $this->add_package($external_repository_registration->get_context());
        }

        $external_repository_registrations = CommonDataManager :: get_registrations_by_type(
            'common\extensions\video_conferencing_manager\implementation');
        foreach ($external_repository_registrations as $external_repository_registration)
        {
            $this->add_package($external_repository_registration->get_context());
        }
    }

    public function upgrade_successfull()
    {
        $image = Theme :: get_image_path(__NAMESPACE__) . 'package_action/finished.png';
        $title = Translation :: get('PlatformUpgraded');
        $result = Translation :: get(
            'CoreUpgraded',
            array('URL' => $this->get_url(array(self :: PARAM_ACTION => self :: ACTION_CONTENT_OBJECT_UPGRADE))));

        return $this->render_upgrade_step($image, $title, $result);
    }
}
