<?php
namespace core\lynx\remote;

use common\libraries\Theme;
use common\libraries\Translation;
use common\libraries\ToolbarItem;
use common\libraries\DynamicVisualTab;
use common\libraries\DataClassRetrievesParameters;
use common\libraries\DynamicVisualTabsRenderer;
use common\libraries\AndCondition;
use common\libraries\PatternMatchCondition;
use common\libraries\DataClassCountParameters;
use common\libraries\PropertyConditionVariable;
use common\libraries\StaticConditionVariable;
use common\libraries\EqualityCondition;
use common\libraries\ObjectTableOrder;
use common\libraries\DataClassRetrieveParameters;
use common\libraries\NewObjectTableSupport;
use common\libraries\Request;
use common\libraries\ActionBarRenderer;

class BrowserComponent extends Manager implements NewObjectTableSupport
{

    private $action_bar;

    private $current_source;
    const STATUS_INSTALLED = 1;
    const STATUS_AVAILABLE = 2;

    /**
     * Runs this component and displays its output.
     */
    public function run()
    {
        $parameters = new DataClassCountParameters(
                new EqualityCondition(
                        new PropertyConditionVariable(\core\lynx\source\Source :: class_name(),
                                \core\lynx\source\Source :: PROPERTY_STATUS),
                        new StaticConditionVariable(\core\lynx\source\Source :: STATUS_ACTIVE)));
        $count = \core\lynx\source\DataManager :: count(\core\lynx\source\Source :: class_name(), $parameters);

        if ($count == 0)
        {
            $this->display_header();
            echo 'NO ACTIVE SOURCE REPOSITORIES';
            $this->display_footer();
        }

        $current_source_id = Request :: get(\core\lynx\source\Manager :: PARAM_SOURCE_ID);

        if (! $current_source_id)
        {
            $parameters = new DataClassRetrieveParameters(
                    new EqualityCondition(
                            new PropertyConditionVariable(\core\lynx\source\Source :: class_name(),
                                    \core\lynx\source\Source :: PROPERTY_STATUS),
                            new StaticConditionVariable(\core\lynx\source\Source :: STATUS_ACTIVE)),
                    array(new ObjectTableOrder(\core\lynx\source\Source :: PROPERTY_NAME)));
            $this->current_source = \core\lynx\source\DataManager :: retrieve(\core\lynx\source\Source :: class_name(),
                    $parameters);
        }
        else
        {
            $this->current_source = \core\lynx\source\DataManager :: retrieve_by_id(
                    \core\lynx\source\Source :: class_name(), (int) $current_source_id);

            if (! $this->current_source instanceof \core\lynx\source\Source || $this->current_source->get_status() == \core\lynx\source\Source :: STATUS_INACTIVE)
            {
                $parameters = new DataClassRetrieveParameters(
                        new EqualityCondition(
                                new PropertyConditionVariable(\core\lynx\source\Source :: class_name(),
                                        \core\lynx\source\Source :: PROPERTY_STATUS),
                                new StaticConditionVariable(\core\lynx\source\Source :: STATUS_ACTIVE)),
                        array(new ObjectTableOrder(\core\lynx\source\Source :: PROPERTY_NAME)));
                $this->current_source = \core\lynx\source\DataManager :: retrieve(
                        \core\lynx\source\Source :: class_name(), $parameters);
            }
        }

        $this->set_parameter(\core\lynx\source\Manager :: PARAM_SOURCE_ID, $this->current_source->get_id());

        $this->action_bar = $this->get_action_bar();
        $table = new PackageTable($this);

        $tabs = new DynamicVisualTabsRenderer('remote_source', $table->as_html());

        $parameters = new DataClassRetrievesParameters(
                new EqualityCondition(
                        new PropertyConditionVariable(\core\lynx\source\Source :: class_name(),
                                \core\lynx\source\Source :: PROPERTY_STATUS),
                        new StaticConditionVariable(\core\lynx\source\Source :: STATUS_ACTIVE)), null, null,
                array(new ObjectTableOrder(\core\lynx\source\Source :: PROPERTY_NAME)));

        $sources = \core\lynx\source\DataManager :: retrieves(\core\lynx\source\Source :: class_name(), $parameters);

        while ($source = $sources->next_result())
        {
            $url = $this->get_url(array(\core\lynx\source\Manager :: PARAM_SOURCE_ID => $source->get_id()));
            $tabs->add_tab(
                    new DynamicVisualTab($source->get_id(), $source->get_name(), null, $url,
                            $source->get_id() == $this->get_source()->get_id()));
        }

        $this->display_header();
        echo $this->action_bar->as_html();
        echo $tabs->render();
        $this->display_footer();
    }

    public function get_object_table_condition($object_table_class_name)
    {
        $query = $this->action_bar->get_query();
        $conditions = array();
        $conditions[] = new EqualityCondition(Package :: PROPERTY_SOURCE_ID, $this->get_source()->get_id());

        if (isset($query) && $query != '')
        {
            $conditions[] = new PatternMatchCondition(Package :: PROPERTY_CONTEXT, '*' . $query . '*');
        }

        return new AndCondition($conditions);
    }

    public function get_action_bar()
    {
        $action_bar = new ActionBarRenderer(ActionBarRenderer :: TYPE_HORIZONTAL);
        $action_bar->set_search_url(
                $this->get_url(array(\core\lynx\source\Manager :: PARAM_SOURCE_ID => $this->get_source()->get_id())));

        $action_bar->add_common_action(
                new ToolbarItem(Translation :: get('Synchronize'), Theme :: get_image_path() . 'action/synchronize.png',
                        $this->get_url(array(Manager :: PARAM_ACTION => Manager :: ACTION_SYNCHRONIZE)),
                        ToolbarItem :: DISPLAY_ICON_AND_LABEL));

        return $action_bar;
    }

    public function get_source()
    {
        return $this->current_source;
    }
}
