<?php
namespace core\lynx\remote;

use common\libraries\Filesystem;
use common\libraries\Path;
use common\libraries\FileProperties;
use common\libraries\Filecompression;
use common\libraries\Translation;
use common\libraries\Request;

class DownloadComponent extends Manager
{

    /**
     * Runs this component and displays its output.
     */
    public function run()
    {
        try
        {
            $package = DataManager :: retrieve_by_id(Package :: class_name(),
                    (int) Request :: get(self :: PARAM_PACKAGE_ID));
        }
        catch (\Exception $exception)
        {
            $this->redirect($exception->getMessage(), true, array(self :: PARAM_ACTION => self :: ACTION_BROWSE));
        }

        if ($package->is_downloadable())
        {
            $source_filename = $package->get_source_filename();
            $file_properties = FileProperties :: from_url($source_filename);

            if ($file_properties->get_path())
            {
                $temp_path = Path :: get_temp_path(self :: context()) . 'download/' . $package->get_id();

                if (! file_exists($temp_path) && Filesystem :: create_dir(dirname($temp_path)))
                {
                    if (copy($source_filename, $temp_path))
                    {
                        $file_compression = Filecompression :: factory();
                        $path = $file_compression->extract_file($temp_path, false);

                        if ($path)
                        {
                            Filesystem :: remove($temp_path);
                            if (Filesystem :: recurse_move($path,
                                    Path :: namespace_to_full_path($package->get_context())))
                            {
                                $this->redirect(Translation :: get('PackageDownloaded'), false,
                                        array(self :: PARAM_ACTION => self :: ACTION_BROWSE));
                            }
                            else
                            {
                                $this->redirect(Translation :: get('PackageNotMoved'), true,
                                        array(self :: PARAM_ACTION => self :: ACTION_BROWSE));
                            }
                        }
                        else
                        {
                            $this->redirect(Translation :: get('ZipExtractFailed'), true,
                                    array(self :: PARAM_ACTION => self :: ACTION_BROWSE));
                        }
                    }
                    else
                    {
                        $this->redirect(
                                Translation :: get('PackageNotDownloadable',
                                        array('TYPE' => Translation :: get('SourceFileNotCopied'))), true,
                                array(self :: PARAM_ACTION => self :: ACTION_BROWSE));
                    }
                }
            }
            else
            {
                $this->redirect(Translation :: get('PackageNotDownloadable', array('TYPE' => $source_filename)), true,
                        array(self :: PARAM_ACTION => self :: ACTION_BROWSE));
            }
        }
        else
        {
            $this->redirect(Translation :: get('PackageNotDownloadable', array('TYPE' => $package->get_name())), true,
                    array(self :: PARAM_ACTION => self :: ACTION_BROWSE));
        }
    }
}
