<?php
namespace core\lynx\remote;

use common\libraries\Translation;
use common\libraries\DataClassRetrievesParameters;
use common\libraries\AndCondition;
use common\libraries\StaticConditionVariable;
use common\libraries\PropertyConditionVariable;
use common\libraries\EqualityCondition;
use common\libraries\DataClassRetrieveParameters;

class SynchronizerComponent extends Manager
{

    public function run()
    {
        $parameters = new DataClassRetrievesParameters(
                new EqualityCondition(
                        new PropertyConditionVariable(\core\lynx\source\Source :: class_name(),
                                \core\lynx\source\Source :: PROPERTY_STATUS),
                        new StaticConditionVariable(\core\lynx\source\Source :: STATUS_ACTIVE)));
        $sources = \core\lynx\source\DataManager :: retrieves(\core\lynx\source\Source :: class_name(), $parameters);

        $failures = 0;

        while ($source = $sources->next_result())
        {

            // Retrieve the source package list
            $packages = Package :: collection($source);

            foreach ($packages as $package)
            {
                $conditions = array();
                $conditions[] = new EqualityCondition(
                        new PropertyConditionVariable(Package :: class_name(), Package :: PROPERTY_SOURCE_ID),
                        new StaticConditionVariable($source->get_id()));
                $conditions[] = new EqualityCondition(
                        new PropertyConditionVariable(Package :: class_name(), Package :: PROPERTY_CONTEXT),
                        new StaticConditionVariable($package->get_context()));
                $conditions[] = new EqualityCondition(
                        new PropertyConditionVariable(Package :: class_name(), Package :: PROPERTY_VERSION),
                        new StaticConditionVariable($package->get_version()));
                $condition = new AndCondition($conditions);

                $parameters = new DataClassRetrieveParameters($condition);
                $existing_object = DataManager :: retrieve(Package :: class_name(), $parameters);

                if ($existing_object instanceof Package)
                {
                    $package->set_id($existing_object->get_id());
                    try
                    {
                        $package->update();
                    }
                    catch (\Exception $e)
                    {
                        $failures ++;
                    }
                }
                else
                {
                    if (! $package->create())
                    {
                        $failures ++;
                    }
                }
            }
        }

        $this->redirect(Translation :: get($failures > 0 ? 'SomeSynchronizationsFailed' : 'SynchronizationFinished'),
                $failures > 0,
                array(\core\lynx\Manager :: PARAM_ACTION => \core\lynx\Manager :: ACTION_REMOTE,
                        self :: PARAM_ACTION => null));
    }
}
