<?php
namespace core\menu;

use common\libraries\DataClass;
use common\libraries\DataClassNoResultException;
use common\libraries\DataClassRetrieveParameters;
use common\libraries\EqualityCondition;
use common\libraries\PropertyConditionVariable;
use common\libraries\StaticConditionVariable;
use common\libraries\RecordRetrieveParameters;
use common\libraries\ObjectNotExistException;
use common\libraries\DataClassProperties;
use common\libraries\DataClassJoin;
use common\libraries\Joins;

/**
 * $Id: menu_data_manager.class.php 157 2009-11-10 13:44:02Z vanpouckesven $
 *
 * @package menu.lib
 */
/**
 * This is a skeleton for a data manager for the Users table. Data managers must extend this class and implement its
 * abstract methods.
 *
 * @author Hans De Bisschop
 * @author Dieter De Neef
 */
class DataManager extends \common\libraries\DataManager
{

    /**
     * Gets the type of DataManager to be instantiated
     *
     * @return string
     */
    public static function get_type()
    {
        return 'doctrine';
    }

    public static function retrieve_item($id, $type = null)
    {
        if (! isset($id) || strlen($id) == 0 || $id == DataClass :: NO_UID)
        {
            throw new DataClassNoResultException(
                Item :: class_name(),
                DataClassRetrieveParameters :: generate((int) $id));
        }

        if (is_null($type))
        {
            $type = self :: determine_item_type($id);
        }

        $condition = new EqualityCondition(
            new PropertyConditionVariable(Item :: class_name(), Item :: PROPERTY_ID),
            new StaticConditionVariable($id));

        $parameters = new DataClassRetrieveParameters($condition);
        return self :: fetch_item($parameters, $type);
    }

    /**
     * Get the type of the item matching the given id.
     *
     * @param int $id The id of the item.
     * @return string The type string.
     */
    public static function determine_item_type($id)
    {
        $condition = new EqualityCondition(Item :: PROPERTY_ID, $id);
        $parameters = new RecordRetrieveParameters(
            new DataClassProperties(
                array(new PropertyConditionVariable(Item :: class_name(), Item :: PROPERTY_TYPE))),
            $condition);
        $type = self :: record(Item :: class_name(), $parameters);
        if (isset($type[Item :: PROPERTY_TYPE]))
        {
            return $type[Item :: PROPERTY_TYPE];
        }
        else
        {
            throw new ObjectNotExistException($id);
        }
    }

    private static function fetch_item($parameters, $type)
    {
        if ($type :: is_extended())
        {
            $join = new DataClassJoin(
                Item :: class_name(),
                new EqualityCondition(
                    new PropertyConditionVariable(Item :: class_name(), Item :: PROPERTY_ID),
                    new PropertyConditionVariable($type, $type :: PROPERTY_ID)));
            if ($parameters->get_joins() instanceof Joins)
            {
                $joins = $parameters->get_joins();
                $joins->add($join);
                $parameters->set_joins($joins);
            }
            else
            {
                $joins = new Joins(array($join));
                $parameters->set_joins($joins);
            }
        }

        return self :: retrieve($type, $parameters);
    }
}