<?php

namespace core\menu;

use common\libraries\Utilities;

class LinkItem extends Item
{
    const PROPERTY_URL = 'url';
    const PROPERTY_TARGET = 'target';

    const TARGET_BLANK = 0;
    const TARGET_SELF = 1;
    const TARGET_PARENT = 2;
    const TARGET_TOP = 3;

    const CLASS_NAME = __CLASS__;

    public function __construct($default_properties = array(), $additional_properties = null)
    {
        parent :: __construct($default_properties, $additional_properties);
        $this->set_type(__CLASS__);
    }

    public static function get_type_name()
    {
        return Utilities :: get_classname_from_namespace(self :: CLASS_NAME);
    }

    public function get_url()
    {
        return $this->get_additional_property(self :: PROPERTY_URL);
    }

    public function set_url($url)
    {
        return $this->set_additional_property(self :: PROPERTY_URL, $url);
    }

    public function get_target()
    {
        return $this->get_additional_property(self :: PROPERTY_TARGET);
    }

    public function set_target($target)
    {
        return $this->set_additional_property(self :: PROPERTY_TARGET, $target);
    }

    public static function get_additional_property_names()
    {
        return array(self :: PROPERTY_URL, self :: PROPERTY_TARGET);
    }

    /**
     * @return string
     */
    public function get_target_string()
    {
        return self :: target_string($this->get_target());
    }

    /**
     * @return string
     */
    public static function target_string($target)
    {
        switch ($target)
        {
            case self :: TARGET_BLANK :
                return '_blank';
                break;
            case self :: TARGET_SELF :
                return '_self';
                break;
            case self :: TARGET_PARENT :
                return '_parent';
                break;
            case self :: TARGET_TOP :
                return '_top';
                break;
        }
    }

    public function get_target_types($types_only = false)
    {
        $types = array();

        $types[self :: TARGET_BLANK] = self :: target_string(self :: TARGET_BLANK);
        $types[self :: TARGET_SELF] = self :: target_string(self :: TARGET_SELF);
        $types[self :: TARGET_PARENT] = self :: target_string(self :: TARGET_PARENT);
        $types[self :: TARGET_TOP] = self :: target_string(self :: TARGET_TOP);

        return ($types_only ? array_keys($types) : $types);
    }

}
