<?php
namespace core\menu;

use common\libraries\Utilities;
use common\libraries\DataClass;
use common\libraries\EqualityCondition;

class ItemTitle extends DataClass
{
    const CLASS_NAME = __CLASS__;
    const PROPERTY_ITEM_ID = 'item_id';
    const PROPERTY_TITLE = 'title';
    const PROPERTY_SORT = 'sort';
    const PROPERTY_ISOCODE = 'isocode';

    private $titles;

    /**
     * Get the default properties of all items.
     * 
     * @return array The property names.
     */
    public static function get_default_property_names()
    {
        return parent :: get_default_property_names(
                array(self :: PROPERTY_ITEM_ID, self :: PROPERTY_TITLE, self :: PROPERTY_SORT, self :: PROPERTY_ISOCODE));
    }

    /**
     * inherited
     */
    public function get_data_manager()
    {
        return MenuDataManager :: get_instance();
    }

    public function get_item_id()
    {
        return $this->get_default_property(self :: PROPERTY_ITEM_ID);
    }

    public function set_item_id($item_id)
    {
        $this->set_default_property(self :: PROPERTY_ITEM_ID, $item_id);
    }

    public function get_sort()
    {
        return $this->get_default_property(self :: PROPERTY_SORT);
    }

    public function set_sort($sort)
    {
        $this->set_default_property(self :: PROPERTY_SORT, $sort);
    }

    public function get_title()
    {
        return $this->get_default_property(self :: PROPERTY_TITLE);
    }

    public function set_title($title)
    {
        $this->set_default_property(self :: PROPERTY_TITLE, $title);
    }

    public function get_isocode()
    {
        return $this->get_default_property(self :: PROPERTY_ISOCODE);
    }

    public function set_isocode($isocode)
    {
        $this->set_default_property(self :: PROPERTY_ISOCODE, $isocode);
    }

    public function create()
    {
        $menu_data_manager = $this->get_data_manager();
        $condition = new EqualityCondition(self :: PROPERTY_ITEM_ID, $this->get_item_id());
        $sort = $menu_data_manager->retrieve_next_sort_value(self :: get_table_name(), self :: PROPERTY_SORT, 
                $condition);
        
        $this->set_sort($sort);
        
        $success = parent :: create($this);
        if (! $success)
        {
            return false;
        }
        
        return true;
    }

    public static function get_table_name()
    {
        return Utilities :: get_classname_from_namespace(self :: CLASS_NAME, true);
    }
}
