<?php
namespace core\menu;

use common\libraries\DelegateComponent;

use common\libraries\Translation;
use common\libraries\Request;
use common\libraries\EqualityCondition;
use common\libraries\AdministrationComponent;
use common\libraries\ActionBarRenderer;
use common\libraries\ToolbarItem;
use common\libraries\Theme;
use common\libraries\BreadcrumbTrail;

class MenuManagerBrowserComponent extends MenuManager implements AdministrationComponent, DelegateComponent
{
    private $action_bar;

    public function run()
    {
        $this->check_allowed();
        $this->show_navigation_item_list();
    }

    public function show_navigation_item_list()
    {
        $this->action_bar = $this->get_action_bar();
        $this->parent = Request :: get(self :: PARAM_PARENT);

        $parameters = $this->get_parameters(true);

        $table = new ItemBrowserTable($this, $parameters, $this->get_condition());
        $this->display_header();

        echo $this->action_bar->as_html();

        echo '<div style="float: left; width: 12%; overflow:auto;">';
        echo $this->get_menu()->render_as_tree();
        echo '</div>';
        echo '<div style="float: right; width: 85%;">';
        echo $table->as_html();
        echo '</div>';
        $this->display_footer();
    }

    public function get_action_bar()
    {
        $action_bar = new ActionBarRenderer(ActionBarRenderer :: TYPE_HORIZONTAL);

        $action_bar->add_common_action(new ToolbarItem(Translation :: get('AddApplicationItem'), Theme :: get_image_path() . 'types/' . Item :: TYPE_APPLICATION . '.png', $this->get_url(array(
                self :: PARAM_ACTION => self :: ACTION_CREATE, self :: PARAM_TYPE => ApplicationItem::class_name())), ToolbarItem :: DISPLAY_ICON_AND_LABEL));
        $action_bar->add_common_action(new ToolbarItem(Translation :: get('AddCategoryItem'), Theme :: get_image_path() . 'types/' . Item :: TYPE_CATEGORY . '.png', $this->get_url(array(
                self :: PARAM_ACTION => self :: ACTION_CREATE, self :: PARAM_TYPE => CategoryItem::class_name())), ToolbarItem :: DISPLAY_ICON_AND_LABEL));
        $action_bar->add_common_action(new ToolbarItem(Translation :: get('AddLinkItem'), Theme :: get_image_path() . 'types/' . Item :: TYPE_LINK . '.png', $this->get_url(array(
                self :: PARAM_ACTION => self :: ACTION_CREATE, self :: PARAM_TYPE => LinkItem::class_name())), ToolbarItem :: DISPLAY_ICON_AND_LABEL));

        return $action_bar;
    }

    public function get_condition()
    {
        $condition = null;
        $parent = (isset($this->parent) ? $this->parent : 0);
        $condition = new EqualityCondition(Item :: PROPERTY_PARENT, $parent);
        return $condition;
    }

    public function add_additional_breadcrumbs(BreadcrumbTrail $breadcrumbtrail)
    {
        $breadcrumbtrail->add_help('menu_browser');
    }

    public function get_additional_parameters()
    {
        return array(MenuManager :: PARAM_ITEM);
    }
}
