<?php
namespace core\menu;

use common\libraries\Translation;
use common\libraries\Utilities;
use common\libraries\Request;
use common\libraries\AdministrationComponent;
use common\libraries\BreadcrumbTrail;
use common\libraries\Breadcrumb;
use common\libraries\EqualityCondition;
use common\libraries\ObjectTableOrder;
use common\libraries\DataClassRetrievesParameters;

class MenuManagerDeleterComponent extends MenuManager implements AdministrationComponent
{

    public function run()
    {
        $this->check_allowed();
        $items = Request :: get(MenuManager :: PARAM_ITEM);
        $parent_ids = array();
        $failures = 0;

        if (! empty($items))
        {
            if (! is_array($items))
            {
                $items = array($items);
            }

            foreach ($items as $id)
            {
                $item = DataManager :: retrieve_item(intval($id));
                $parent_ids[$item->get_parent()] = $item->get_parent();

                if (! $item->delete())
                {
                    $failures ++;
                }
            }

            // Reassign sorts to the remaining menu items
            foreach ($parent_ids as $parent_id)
            {
                $condition = new EqualityCondition(Item :: PROPERTY_PARENT, $parent_id);
                $order_by = array();
                $order_by[] = new ObjectTableOrder(Item :: PROPERTY_SORT);
                $parameters = new DataClassRetrievesParameters($condition, null, null, $order_by);

                $remaining_items = DataManager :: retrieves(Item :: class_name(), $parameters);
                $count = 1;
                while ($remaining_item = $remaining_items->next_result())
                {
                    // var_dump($count);
                    $remaining_item->set_sort($count);
                    if (! $remaining_item->update())
                    {
                        break;
                    }
                    $count ++;
                }
            }

            $message = $this->get_result($failures, count($items), 'SelectedItemNotDeleted', 'SelectedItemsNotDeleted',
                    'SelectedItemDeleted', 'SelectedItemsDeleted');

            $this->redirect($message, ($failures ? true : false),
                    array(MenuManager :: PARAM_ACTION => MenuManager :: ACTION_BROWSE,
                            MenuManager :: PARAM_PARENT => $item->get_parent()));
        }
        else
        {
            $this->display_error_page(Translation :: get('NoObjectsSelected', null, Utilities :: COMMON_LIBRARIES));
        }
    }

    public function add_additional_breadcrumbs(BreadcrumbTrail $breadcrumbtrail)
    {
        $breadcrumbtrail->add(
                new Breadcrumb($this->get_menu_home_url(), Translation :: get('MenuManagerBrowserComponent')));
        $breadcrumbtrail->add_help('menu_deleter');
    }

    public function get_additional_parameters()
    {
        return array(MenuManager :: PARAM_ITEM, MenuManager :: PARAM_DIRECTION);
    }
}
