<?php
namespace core\menu;
use common\libraries\Translation;
use common\libraries\Utilities;
use common\libraries\Theme;
use common\libraries\Toolbar;
use common\libraries\ToolbarItem;

class ItemBrowserTableCellRenderer extends DefaultItemTableCellRenderer
{
    /**
     * The repository browser component
     */
    private $browser;

    /**
     * Constructor
     *
     * @param $browser MenuManagerManagerBrowserComponent
     */
    public function __construct($browser)
    {
        parent :: __construct();
        $this->browser = $browser;
    }

    // Inherited
    public function render_cell($column, $menu)
    {
        if ($column === ItemBrowserTableColumnModel :: get_modification_column())
        {
            return $this->get_modification_links($menu);
        }
        return parent :: render_cell($column, $menu);
    }

    /**
     * Gets the action links to display
     *
     * @param $menu Object The menu object for which the action links should be returned
     * @return string A HTML representation of the action links
     */
    private function get_modification_links($menu)
    {
        $order = $menu->get_sort();
        $max = DataManager :: count(Item :: class_name(), $this->browser->get_condition());

        if ($max == 1)
        {
            $index = 'single';
        }
        else
        {
            if ($order == 1)
            {
                $index = 'first';
            }
            else
            {
                if ($order == $max)
                {
                    $index = 'last';
                }
                else
                {
                    $index = 'middle';
                }
            }
        }

        $toolbar = new Toolbar();

        $toolbar->add_item(new ToolbarItem(Translation :: get('Edit', null, Utilities :: COMMON_LIBRARIES), Theme :: get_common_image_path() . 'action_edit.png', $this->browser->get_item_editing_url($menu), ToolbarItem :: DISPLAY_ICON));

        if ($index == 'first' || $index == 'single')
        {
            $toolbar->add_item(new ToolbarItem(Translation :: get('MoveUpNA', null, Utilities :: COMMON_LIBRARIES), Theme :: get_common_image_path() . 'action_up_na.png', null, ToolbarItem :: DISPLAY_ICON));
        }
        else
        {
            $toolbar->add_item(new ToolbarItem(Translation :: get('MoveUp', null, Utilities :: COMMON_LIBRARIES), Theme :: get_common_image_path() . 'action_up.png', $this->browser->get_item_moving_url($menu, 'up'), ToolbarItem :: DISPLAY_ICON));
        }

        if ($index == 'last' || $index == 'single')
        {
            $toolbar->add_item(new ToolbarItem(Translation :: get('MoveDownNA', null, Utilities :: COMMON_LIBRARIES), Theme :: get_common_image_path() . 'action_down_na.png', null, ToolbarItem :: DISPLAY_ICON));
        }
        else
        {
            $toolbar->add_item(new ToolbarItem(Translation :: get('MoveDown', null, Utilities :: COMMON_LIBRARIES), Theme :: get_common_image_path() . 'action_down.png', $this->browser->get_item_moving_url($menu, 'down'), ToolbarItem :: DISPLAY_ICON));
        }

        $toolbar->add_item(new ToolbarItem(Translation :: get('Delete', null, Utilities :: COMMON_LIBRARIES), Theme :: get_common_image_path() . 'action_delete.png', $this->browser->get_item_deleting_url($menu), ToolbarItem :: DISPLAY_ICON, true));

        return $toolbar->as_html();
    }
}
