<?php
namespace core\menu;

use common\libraries\AndCondition;
use common\libraries\EqualityCondition;
use common\libraries\Translation;
use common\libraries\Utilities;
use common\libraries\Request;
use common\libraries\AdministrationComponent;
use common\libraries\BreadcrumbTrail;
use common\libraries\Breadcrumb;

class MenuManagerMoverComponent extends MenuManager implements AdministrationComponent
{

    public function run()
    {
        $this->check_allowed();
        $direction = Request :: get(MenuManager :: PARAM_DIRECTION);
        $item = intval(Request :: get(MenuManager :: PARAM_ITEM));

        if (isset($direction) && isset($item))
        {
            $move_item = DataManager :: retrieve_item((int) $item);

            $max = DataManager :: count(
                Item :: class_name(),
                new EqualityCondition(Item :: PROPERTY_PARENT, $move_item->get_parent()));

            $display_order = $move_item->get_sort();
            $new_place = ($display_order + ($direction == MenuManager :: PARAM_DIRECTION_UP ? - 1 : 1));

            if ($new_place > 0 && $new_place <= $max)
            {
                $move_item->set_sort($new_place);

                $conditions[] = new EqualityCondition(Item :: PROPERTY_SORT, $new_place);
                $conditions[] = new EqualityCondition(Item :: PROPERTY_PARENT, $move_item->get_parent());
                $condition = new AndCondition($conditions);

                $items = DataManager :: retrieves(Item :: class_name(), $condition);
                $new_item = $items->next_result();
                $new_item->set_sort($display_order);

                if (! $move_item->update() || ! $new_item->update())
                {
                    $success = false;
                }
                else
                {
                    $success = true;
                }
            }

            $message = $success ? Translation :: get(
                'ObjectMoved',
                array('OBJECT' => Translation :: get('MenuManagerItem')),
                Utilities :: COMMON_LIBRARIES) : Translation :: get(
                'ObjectNotMoved',
                array('OBJECT' => Translation :: get('MenuManagerItem')),
                Utilities :: COMMON_LIBRARIES);
            $this->redirect(
                $message,
                ($success ? false : true),
                array(
                    MenuManager :: PARAM_ACTION => MenuManager :: ACTION_BROWSE,
                    MenuManager :: PARAM_PARENT => $move_item->get_parent()));
        }
        else
        {
            $this->display_error_page(Translation :: get('NoObjectsSelected', null, Utilities :: COMMON_LIBRARIES));
        }
    }

    public function add_additional_breadcrumbs(BreadcrumbTrail $breadcrumbtrail)
    {
        $breadcrumbtrail->add(
            new Breadcrumb($this->get_menu_home_url(), Translation :: get('MenuManagerBrowserComponent')));
        $breadcrumbtrail->add_help('menu_mover');
    }

    public function get_additional_parameters()
    {
        return array(MenuManager :: PARAM_ITEM, MenuManager :: PARAM_DIRECTION);
    }
}
