<?php
namespace core\menu;

use common\libraries\Display;
use common\libraries\Application;
use common\libraries\CoreApplication;
use common\libraries\Translation;
use common\libraries\Utilities;
use common\libraries\Request;
use common\libraries\DynamicAction;
use common\libraries\Theme;
use common\libraries\Redirect;

/**
 * $Id: menu_manager.class.php 223 2009-11-13 14:39:28Z vanpouckesven $
 *
 * @package menu.lib.menu_manager.component
 */

/**
 * A user manager provides some functionalities to the admin to manage his users. For each functionality a component is
 * available.
 */
class MenuManager extends CoreApplication
{
    const APPLICATION_NAME = 'menu';
    const PARAM_DIRECTION = 'direction';
    const PARAM_ITEM = 'item';
    const PARAM_TYPE = 'type';
    const PARAM_PARENT = 'parent';
    const ACTION_CREATE = 'creator';
    const ACTION_BROWSE = 'browser';
    const ACTION_EDIT = 'editor';
    const ACTION_DELETE = 'deleter';
    const ACTION_MOVE = 'mover';
    const PARAM_DIRECTION_UP = 'up';
    const PARAM_DIRECTION_DOWN = 'down';
    const DEFAULT_ACTION = self :: ACTION_BROWSE;

    public static function get_application_platform_admin_links($application = self :: APPLICATION_NAME)
    {
        $links = array();
        $links[] = new DynamicAction(Translation :: get('Manage'), Translation :: get('ManageDescription'),
                Theme :: get_image_path() . 'admin/sort.png',
                Redirect :: get_link(self :: APPLICATION_NAME,
                        array(Application :: PARAM_ACTION => self :: ACTION_BROWSE), array(), false, Redirect :: TYPE_CORE));

        $info = parent :: get_application_platform_admin_links(self :: APPLICATION_NAME);
        $info['links'] = $links;

        return $info;
    }

    public function get_item_creation_url()
    {
        return $this->get_url(array(self :: PARAM_ACTION => self :: ACTION_ADD));
    }

    public function get_item_editing_url($navigation_item)
    {
        return $this->get_url(
                array(self :: PARAM_ACTION => self :: ACTION_EDIT, self :: PARAM_ITEM => $navigation_item->get_id()));
    }

    public function get_item_deleting_url($navigation_item)
    {
        return $this->get_url(
                array(self :: PARAM_ACTION => self :: ACTION_DELETE, self :: PARAM_ITEM => $navigation_item->get_id()));
    }

    public function get_item_moving_url($navigation_item, $direction)
    {
        return $this->get_url(
                array(self :: PARAM_ACTION => self :: ACTION_MOVE, self :: PARAM_ITEM => $navigation_item->get_id(),
                        self :: PARAM_DIRECTION => $direction));
    }

    /**
     * Helper function for the Application class, pending access to class constants via variables in PHP 5.3 e.g. $name
     * = $class :: DEFAULT_ACTION DO NOT USE IN THIS APPLICATION'S CONTEXT Instead use: - self :: DEFAULT_ACTION in the
     * context of this class - YourApplicationManager :: DEFAULT_ACTION in all other application classes
     */
    public function get_default_action()
    {
        return self :: DEFAULT_ACTION;
    }

    public function get_menu()
    {
        if (! isset($this->menu))
        {
            $temp_replacement = '__ITEM__';
            $url_format = $this->get_url(
                    array(Application :: PARAM_ACTION => MenuManager :: ACTION_BROWSE,
                            MenuManager :: PARAM_PARENT => $temp_replacement));
            $url_format = str_replace($temp_replacement, '%s', $url_format);
            $this->menu = new ItemMenu(Request :: get(self :: PARAM_PARENT), $url_format);
        }
        return $this->menu;
    }

    public function get_menu_home_url()
    {
        return $this->get_url(array(Application :: PARAM_ACTION => MenuManager :: ACTION_BROWSE));
    }

    public function check_allowed()
    {
        if (! $this->get_user()->is_platform_admin())
        {
            $this->display_header();
            Display :: error_message(Translation :: get('NotAllowed', null, Utilities :: COMMON_LIBRARIES));
            $this->display_footer();
            exit();
        }
    }
}
