<?php
namespace core\metadata;

use common\libraries\CommonDataManager;
use common\libraries\FormValidator;
use common\libraries\Translation;
use common\libraries\ResourceManager;
use common\libraries\Path;
use common\libraries\Utilities;
use common\libraries\Request;


class MetadataForm extends FormValidator
{
    private $elements = array();
    private $prefixes = array();
    private $parent_type;
    private $object_id;

    const PARENT_USER = 'user';
    const PARENT_CONTENT_OBJECT = 'content_object';
    const PARENT_ID = 'parent_id';

    public function __construct($name, $method, $action)
    {
        $this->retrieve_elements();
        $this->retrieve_prefixes();

        parent :: __construct($name, $method, $action);

        // default parent_type = content object
        $this->parent_type = ContentObjectMetadataEditorForm :: TYPE;
    }

    // sets the type of metadata to edit
    public function set_parent_type($parent_type)
    {
        $this->parent_type = $parent_type;
    }

    // sets...
    public function get_parent_type()
    {
        return $this->parent_type;
    }

    public function get_object_id()
    {
        return Request :: get($this->parent_type);
    }

    public function build_basic_form()
    {
        $this->build_empty_element_value();

        $buttons[] = $this->createElement('style_submit_button', 'submit', Translation :: get('Create', null, Utilities :: COMMON_LIBRARIES), array(
                'class' => 'positive'));
        $buttons[] = $this->createElement('style_reset_button', 'reset', Translation :: get('Reset', null, Utilities :: COMMON_LIBRARIES), array(
                'class' => 'normal empty'));

        $this->addGroup($buttons, 'buttons', null, '&nbsp;', false);
    }

    public function retrieve_elements()
    {
        $mdm = MetadataDataManager :: get_instance();

        $elements = $mdm->retrieve_metadata_elements();

        while ($element = $elements->next_result())
        {
            $this->elements[$element->get_id()] = $element->get_ns_prefix() . ':' . $element->get_name();
        }
    }

    public function retrieve_prefixes()
    {
        $mdm = MetadataDataManager :: get_instance();

        if ($prefixes = $mdm->retrieve_prefixes())
        {
            $this->prefixes = $prefixes;
        }
        else
        {
            $this->prefixes[] = Translation :: get('NoNamespaceDefined');
        }

        // $this->prefixes[0] = 'Select a namespace';

    }

    public function get_elements()
    {
        return $this->elements;
    }

    public function get_prefixes()
    {
        return $this->prefixes;
    }

    /**
     * creates a new and empty property value input field
     */
    public function build_empty_element_value()
    {

        $this->addElement('select', MetadataElement :: PROPERTY_NS_PREFIX, Translation :: get('MetadataElement'), $this->get_prefixes(), array(
                'class' => 'ns_prefix'));
        $this->addElement('html', '<div class="object_id" style="display:none">' . $this->get_object_id() . '</div>');
        $this->addElement('html', '<div class="object_type" style="display:none">' . $this->parent_type . '</div>');
        $this->addElement('html', ResourceManager :: get_instance()->get_resource_html(Path :: get(WEB_CORE_PATH) . 'metadata/resources/javascript/format_metadata_value.js'));

    }

    /**
     * triggers creation if element with name MetadataElementValue :: PROPERTY_VALUE is not empty
     *
     * @return bool success
     */
    public function edit_metadata()
    {
        // create new property value
        if (Request :: post(MetadataElementValue :: PROPERTY_VALUE))
        {
            return $this->create_metadata_element_value();
        }
        return true;
    }

    /*
     * creates content object property value or user property value depending on $this->parent_type (user or
     * content_object) @return MetadataElementValue or false
     */
    public function create_metadata_element_value($id = null)
    {
        $values = $this->exportValues();

        $class = __NAMESPACE__ . '\\' . Utilities :: underscores_to_camelcase($this->parent_type) . 'MetadataElementValue';

        $metadata_element_value = new $class();

        $function = 'set_' . $this->get_parent_type() . '_id';

        if (! is_null($id))
        {
            $metadata_element_value->$function($id);
        }
        else
        {
            $metadata_element_value->$function($values[self :: PARENT_ID]);
        }

        $metadata_element_value->set_element_id(Request :: post(MetadataElementValue :: PROPERTY_ELEMENT_ID));
        $metadata_element_value->set_value(Request :: post(MetadataElementValue :: PROPERTY_VALUE));

        if ($metadata_element_value->create())
        {
            return $metadata_element_value;
        }
        return false;
    }

    public function setDefaults($defaults)
    {
        if (! isset($defaults[MetadataElement :: PROPERTY_NS_PREFIX]))
        {
            $default_ns = CommonDataManager :: retrieve_setting_from_variable_name(MetadataManager :: SETTING_DEFAULT_NAMESPACE, MetadataManager :: context());
            $defaults[MetadataElement :: PROPERTY_NS_PREFIX] = $default_ns->get_value();

        }
        parent :: setDefaults($defaults);
    }
}
