<?php
namespace core\metadata;


use common\libraries\FormValidator;
use common\libraries\Translation;
use common\libraries\Filesystem;
use common\libraries\Path;
use common\libraries\Utilities;
use common\libraries\ResourceManager;

/**
 * A form to import a ContentObject.
 */
class MetadataImportForm extends FormValidator
{
    const IMPORT_FILE_NAME = 'metadata_file';

    private $category;
    private $user;
    private $import_type;
    private $show_category;
    private $messages;
    private $warnings;
    private $errors;
    private $log;

    /**
     * Constructor.
     *
     * @param $form_name string The name to use in the form tag.
     * @param $method string The method to use ('post' or 'get').
     * @param $action string The URL to which the form should be submitted.
     */
    public function __construct($form_name, $method = 'post', $action = null, $category, $user, $import_type = null, $show_category = true)
    {
        parent :: __construct($form_name, $method, $action);
        $this->build_basic_form();
        $this->setDefaults();
    }

    /**
     * Builds a form to import a learning object.
     */
    private function build_basic_form()
    {

        $this->addElement('file', self :: IMPORT_FILE_NAME,
                Translation :: get('FileName', null, Utilities :: COMMON_LIBRARIES));
        $buttons[] = $this->createElement('style_submit_button', 'submit',
                Translation :: get('Import', null, Utilities :: COMMON_LIBRARIES),
                array('class' => 'positive import', 'id' => 'import_button'));
        $this->addGroup($buttons, 'buttons', null, '&nbsp;', false);

        $this->addElement('html',
                ResourceManager :: get_instance()->get_resource_html(
                        Path :: get(WEB_PATH) . 'repository/resources/javascript/import.js'));
    }

    public function get_types()
    {
        $folder = dirname(__FILE__) . '/import/';
        $folders = Filesystem :: get_directory_content($folder, Filesystem :: LIST_DIRECTORIES, false);
        foreach ($folders as $f)
        {
            if (strpos($f, '.svn') !== false || strpos($f, 'csv') !== false)
                continue;

            $types[$f] = Translation :: get('Type' . Utilities :: underscores_to_camelcase($f), null,
                    Utilities :: COMMON_LIBRARIES);
        }

        return $types;
    }

    public function get_messages()
    {
        return empty($this->messages) ? array() : $this->messages;
    }

    public function get_warnings()
    {
        return empty($this->warnings) ? array() : $this->warnings;
    }

    public function get_errors()
    {
        return empty($this->errors) ? array() : $this->errors;
    }

    /**
     * Sets default values.
     *
     * @param $defaults array Default values for this form's parameters.
     */
    public function setDefaults($defaults = array())
    {
        parent :: setDefaults($defaults);
    }

    public function set_values($defaults)
    {
        parent :: setDefaults($defaults);
    }

    public function import_metadata()
    {
        $importer = new MetadataCpoImport();
        $result = $importer->import_metadata();
        $this->log .= '<p style="color: blue;">' . implode("</br>", $importer->get_log()) . '</p>';
        // $this->messages = $importer->get_messages();
        // $this->warnings = $importer->get_warnings();
        // $this->errors = $importer->get_errors();
        return $result;
    }

    public function get_log()
    {
        return $this->log;
    }

    /**
     * Displays the form
     */
    public function display()
    {
        parent :: display();
    }

    public function get_path($path_type)
    {
        return Path :: get($path_type);
    }

}
