<?php
namespace core\metadata;
use common\libraries\DataClass;
use common\libraries\Utilities;
/**
 * metadata
 */

/**
 * This class describes a MetadataAttributeValue data object
 * @author Sven Vanpoucke
 * @author Jens Vanderheyden
 */
class MetadataAttributeValue extends DataClass
{
    const CLASS_NAME = __CLASS__;
    
    /**
     * MetadataAttributeValue properties
     */
    const PROPERTY_PARENT_ID = 'parent_id';
    const PROPERTY_ATTRIBUTE_ID = 'attribute_id';
    const PROPERTY_RELATION = 'relation';
    const PROPERTY_VALUE = 'value';
    const PROPERTY_VALUE_TYPE = 'value_type';
    const PROPERTY_CONTENT_OBJECT_ID = 'content_object_id';
    
    const VALUE_TYPE_NONE = '0';
    const VALUE_TYPE_ID = '1';
    const VALUE_TYPE_VALUE = '2';
    
    const RELATION_CONTENT_OBJECT_PROPERTY = '1';
    const RELATION_PROPERTY_VALUE = '2';

    /**
     * Get the default properties
     * @return array The property names.
     */
    public static function get_default_property_names()
    {
        return array(self :: PROPERTY_ID, self :: PROPERTY_PARENT_ID, self :: PROPERTY_ATTRIBUTE_ID, 
                self :: PROPERTY_RELATION, self :: PROPERTY_VALUE, self :: PROPERTY_VALUE_TYPE);
    }

    public function get_data_manager()
    {
        return MetadataDataManager :: get_instance();
    }

    /**
     * Returns the element_value_id of this MetadataAttributeValue.
     * @return the element_value_id.
     */
    public function get_parent_id()
    {
        return $this->get_default_property(self :: PROPERTY_PARENT_ID);
    }

    /**
     * Sets the element_value_id of this MetadataAttributeValue.
     * @param element_value_id
     */
    public function set_parent_id($parent_id)
    {
        $this->set_default_property(self :: PROPERTY_PARENT_ID, $parent_id);
    }

    /**
     * Returns the attribute_id of this MetadataAttributeValue.
     * @return the attribute_id.
     */
    public function get_attribute_id()
    {
        return $this->get_default_property(self :: PROPERTY_ATTRIBUTE_ID);
    }

    /**
     * Sets the attribute_id of this MetadataAttributeValue.
     * @param attribute_id
     */
    public function set_attribute_id($attribute_id)
    {
        $this->set_default_property(self :: PROPERTY_ATTRIBUTE_ID, $attribute_id);
    }

    /**
     * Returns the value of this MetadataAttribute.
     * @return the value.
     */
    public function get_value()
    {
        return $this->get_default_property(self :: PROPERTY_VALUE);
    }

    /**
     * Sets the value of this MetadataAttribute.
     * @param value
     */
    public function set_value($value)
    {
        $this->set_default_property(self :: PROPERTY_VALUE, $value);
    }

    /**
     * Returns the value_type of this MetadataAttribute.
     * @return the value_type.
     */
    public function get_value_type()
    {
        return $this->get_default_property(self :: PROPERTY_VALUE_TYPE);
    }

    /**
     * Sets the value_type of this MetadataAttribute.
     * @param value_type
     */
    public function set_value_type($value_type)
    {
        $this->set_default_property(self :: PROPERTY_VALUE_TYPE, $value_type);
    }

    public function get_relation()
    {
        return $this->get_default_property(self :: PROPERTY_RELATION);
    }

    public function set_relation($relation)
    {
        $this->set_default_property(self :: PROPERTY_RELATION, $relation);
    }

    public function get_content_object_id()
    {
        return $this->get_default_property(self :: PROPERTY_CONTENT_OBJECT_ID);
    }

    public function set_content_object_id($content_object_id)
    {
        $this->set_default_property(self :: PROPERTY_CONTENT_OBJECT_ID, $content_object_id);
    }

    public static function get_table_name()
    {
        return Utilities :: camelcase_to_underscores(Utilities :: get_classname_from_namespace(self :: CLASS_NAME));
    }
}
