<?php
namespace core\metadata;

use common\libraries\Translation;
use common\libraries\Request;
use common\libraries\Utilities;
use common\libraries\BreadcrumbTrail;
use common\libraries\Breadcrumb;
use common\libraries\Redirect;
use common\libraries\DynamicTabsRenderer;
use core\admin\AdminManager;

/**
 * Component to edit an existing metadata_element object
 * @author Sven Vanpoucke
 * @author Jens Vanderheyden
 */
class MetadataManagerMetadataElementUpdaterComponent extends MetadataManager
{

    /**
     * Runs this component and displays its output.
     */
    public function run()
    {
        $metadata_element = $this->retrieve_metadata_element(Request :: get(MetadataManager :: PARAM_METADATA_ELEMENT));
        $form = new MetadataElementForm(MetadataElementForm :: TYPE_EDIT, $metadata_element, $this->get_url(array(
                MetadataManager :: PARAM_METADATA_ELEMENT => $metadata_element->get_id())), $this->get_user());

        if ($form->validate())
        {
            $success = $form->update_metadata_element();
            $this->redirect(Translation :: get($success ? 'ObjectUpdated' : 'ObjectNotUpdated', array(
                    'OBJECT' => Translation :: get('MetadataElement')), Utilities :: COMMON_LIBRARIES), ! $success, array(
                    MetadataManager :: PARAM_ACTION => MetadataManager :: ACTION_BROWSE_METADATA_ELEMENTS));
        }
        else
        {
            $trail = BreadcrumbTrail :: get_instance();

            $trail->add(new BreadCrumb(Redirect :: get_link(AdminManager :: APPLICATION_NAME, array(
                    AdminManager :: PARAM_ACTION => AdminManager :: ACTION_ADMIN_BROWSER), array(), false, Redirect :: TYPE_CORE), Translation :: get('Admin')));
            $trail->add(new BreadCrumb(Redirect :: get_link(AdminManager :: APPLICATION_NAME, array(
                    AdminManager :: PARAM_ACTION => AdminManager :: ACTION_ADMIN_BROWSER,
                    DynamicTabsRenderer :: PARAM_SELECTED_TAB => MetadataManager :: APPLICATION_NAME), array(), false, Redirect :: TYPE_CORE), Translation :: get('Metadata')));
            $trail->add(new BreadCrumb($this->get_url(), Translation :: get('UpdateObject', array(
                    'OBJECT' => Translation :: get('MetadataElement')), Utilities :: COMMON_LIBRARIES)));

            $this->display_header();
            $form->display();
            $this->display_footer();
        }
    }
}
