<?php
namespace core\metadata;

use common\libraries\EqualityCondition;
use common\libraries\OrCondition;
use common\libraries\AndCondition;
use common\libraries\DynamicAction;
use common\libraries\ArrayResultSet;
use common\libraries\Translation;
use common\libraries\Application;
use common\libraries\Theme;
use common\libraries\Redirect;
use common\libraries\CoreApplication;

/**
 * A metadata manager
 *
 * @author Jens Vanderheyden
 */
class MetadataManager extends CoreApplication
{
    const APPLICATION_NAME = 'metadata';
    const SETTING_DEFAULT_ELEMENT = 'default_element';
    const SETTING_DEFAULT_NAMESPACE = 'default_namespace';
    const SETTING_DEFAULT_VALUE = 'default_value';
    const PARAM_CONTENT_OBJECT = 'content_object';
    const PARAM_USER = 'user';
    const PARAM_GROUP = 'group';
    const PARAM_METADATA_ATTRIBUTE_ASSOCIATION = 'metadata_attribute_association';
    const PARAM_METADATA_DEFAULT_VALUE = 'metadata_default_value';
    const PARAM_METADATA_ELEMENT_NESTING = 'metadata_element_nesting';
    const PARAM_METADATA_NAMESPACE = 'metadata_namespace';
    const PARAM_CONTENT_OBJECT_PROPERTY_METADATA = 'content_object_property_metadata';
    const PARAM_METADATA_ELEMENT = 'metadata_element';
    const PARAM_METADATA_PROPERTY_VALUE = 'metadata_element_value';
    const PARAM_METADATA_ATTRIBUTE = 'metadata_attribute';
    const PARAM_METADATA_ATTRIBUTE_VALUE = 'metadata_attribute_value';
    const PARAM_MOVE = 'move';
    const ACTION_BROWSE_METADATA_TYPES = 'metadata_types_browser';
    const ACTION_EDIT_ASSOCIATIONS = 'metadata_associations_editor';
    const ACTION_QUICK_EDIT_METADATA = 'metadata_quick_edit';
    const ACTION_DELETE_METADATA_NAMESPACE = 'metadata_namespace_deleter';
    const ACTION_EDIT_METADATA_NAMESPACE = 'metadata_namespace_updater';
    const ACTION_CREATE_METADATA_NAMESPACE = 'metadata_namespace_creator';
    const ACTION_BROWSE_METADATA_NAMESPACES = 'metadata_namespaces_browser';
    const ACTION_DELETE_CONTENT_OBJECT_PROPERTY_METADATA = 'content_object_property_metadata_deleter';
    const ACTION_EDIT_CONTENT_OBJECT_PROPERTY_METADATA = 'content_object_property_metadata_updater';
    const ACTION_CREATE_CONTENT_OBJECT_PROPERTY_METADATA = 'content_object_property_metadata_creator';
    const ACTION_BROWSE_CONTENT_OBJECT_PROPERTY_METADATAS = 'content_object_property_metadatas_browser';
    const ACTION_DELETE_METADATA_ELEMENT = 'metadata_element_deleter';
    const ACTION_MOVE_METADATA_ELEMENT = 'metadata_element_mover';
    const ACTION_EDIT_METADATA_ELEMENT = 'metadata_element_updater';
    const ACTION_CREATE_METADATA_ELEMENT = 'metadata_element_creator';
    const ACTION_BROWSE_METADATA_ELEMENTS = 'metadata_elements_browser';
    const ACTION_BROWSE_CONTENT_OBJECT_METADATA_PROPERTY_VALUES = 'content_object_metadata_element_values_browser';
    const ACTION_DELETE_CONTENT_OBJECT_METADATA_PROPERTY_VALUE = 'content_object_metadata_element_value_deleter';
    const ACTION_EDIT_CONTENT_OBJECT_METADATA = 'content_object_metadata_editor';
    const ACTION_BROWSE_USER_METADATA_PROPERTY_VALUES = 'user_metadata_element_values_browser';
    const ACTION_DELETE_USER_METADATA_PROPERTY_VALUE = 'user_metadata_element_value_deleter';
    const ACTION_EDIT_USER_METADATA = 'user_metadata_editor';
    const ACTION_BROWSE_GROUP_METADATA_PROPERTY_VALUES = 'group_metadata_element_values_browser';
    const ACTION_DELETE_GROUP_METADATA_PROPERTY_VALUE = 'group_metadata_element_value_deleter';
    const ACTION_EDIT_GROUP_METADATA = 'group_metadata_editor';
    const ACTION_DELETE_METADATA_ATTRIBUTE = 'metadata_attribute_deleter';
    const ACTION_EDIT_METADATA_ATTRIBUTE = 'metadata_attribute_updater';
    const ACTION_CREATE_METADATA_ATTRIBUTE = 'metadata_attribute_creator';
    const ACTION_BROWSE_METADATA_ATTRIBUTES = 'metadata_attributes_browser';
    const ACTION_DELETE_CONTENT_OBJECT_METADATA_ATTRIBUTE_VALUE = 'content_object_metadata_attribute_value_deleter';
    const ACTION_BROWSE_METADATA_DEFAULT_VALUES = 'metadata_default_values_browser';
    const ACTION_DELETE_METADATA_DEFAULT_VALUE = 'metadata_default_value_deleter';
    const ACTION_EDIT_METADATA_DEFAULT_VALUE = 'metadata_default_value_updater';
    const ACTION_CREATE_METADATA_DEFAULT_VALUE = 'metadata_default_value_creator';
    const ACTION_IMPORT_METADATA_DEFAULT_VALUE = 'metadata_default_value_importer';
    const ACTION_EXPORT_METADATA = 'metadata_exporter';
    const ACTION_IMPORT_METADATA = 'metadata_importer';
    const DEFAULT_ACTION = self :: ACTION_BROWSE_METADATA_TYPES;

    private $display_order_total = array();

    /**
     * Constructor
     *
     * @param User $user The current user
     */
    public function __construct($user = null)
    {
        parent :: __construct($user);
        // $this->parse_input_from_table();
    }

    /**
     * Helper function for the Application class, pending access to class constants via variables in PHP 5.3 e.g. $name
     * = $class :: DEFAULT_ACTION DO NOT USE IN THIS APPLICATION'S CONTEXT Instead use: - self :: DEFAULT_ACTION in the
     * context of this class - YourApplicationManager :: DEFAULT_ACTION in all other application classes
     */
    public function get_default_action()
    {
        return self :: DEFAULT_ACTION;
    }

    public function retrieve_metadata_attribute_associations($condition = null, $offset = null, $count = null,
        $order_property = null)
    {
        return MetadataDataManager :: get_instance()->retrieve_metadata_attribute_associations(
            $condition,
            $offset,
            $count,
            $order_property);
    }

    public function retrieve_metadata_element_nestings($condition = null, $offset = null, $count = null, $order_property = null)
    {
        return MetadataDataManager :: get_instance()->retrieve_metadata_element_nestings(
            $condition,
            $offset,
            $count,
            $order_property);
    }

    public function delete_metadata_element_nestings($table_name, $metadata_element)
    {
        return MetadataDataManager :: get_instance()->delete_metadata_element_nestings($table_name, $metadata_element);
    }

    public function delete_metadata_attribute_associations($table_name, $metadata_element)
    {
        return MetadataDataManager :: get_instance()->delete_metadata_attribute_associations(
            $table_name,
            $metadata_element);
    }

    public function retrieve_metadata_attribute_association($id)
    {
        return MetadataDataManager :: get_instance()->retrieve_metadata_attribute_association($id);
    }

    public function count_metadata_namespaces($condition)
    {
        return MetadataDataManager :: get_instance()->count_metadata_namespaces($condition);
    }

    public function retrieve_metadata_namespaces($condition = null, $offset = null, $count = null, $order_property = null)
    {
        return MetadataDataManager :: get_instance()->retrieve_metadata_namespaces(
            $condition,
            $offset,
            $count,
            $order_property);
    }

    public function retrieve_metadata_namespace($id)
    {
        return MetadataDataManager :: get_instance()->retrieve_metadata_namespace($id);
    }

    public function count_content_object_property_metadatas($condition)
    {
        return MetadataDataManager :: get_instance()->count_content_object_property_metadatas($condition);
    }

    public function retrieve_content_object_property_metadatas($condition = null, $offset = null, $count = null,
        $order_property = null)
    {
        return MetadataDataManager :: get_instance()->retrieve_content_object_property_metadatas(
            $condition,
            $offset,
            $count,
            $order_property);
    }

    public function retrieve_content_object_property_metadata($id)
    {
        return MetadataDataManager :: get_instance()->retrieve_content_object_property_metadata($id);
    }

    public function count_metadata_elements($condition)
    {
        return MetadataDataManager :: get_instance()->count_metadata_elements($condition);
    }

    public function retrieve_metadata_elements($condition = null, $offset = null, $count = null, $order_property = null)
    {
        return MetadataDataManager :: get_instance()->retrieve_metadata_elements(
            $condition,
            $offset,
            $count,
            $order_property);
    }

    public static function retrieve_metadata_element($id)
    {
        return MetadataDataManager :: get_instance()->retrieve_metadata_element($id);
    }

    public function count_content_object_metadata_element_values($condition)
    {
        return MetadataDataManager :: get_instance()->count_content_object_metadata_element_values($condition);
    }

    public static function retrieve_content_object_metadata_element_values($condition = null, $offset = null, $count = null,
        $order_property = null)
    {
        return MetadataDataManager :: get_instance()->retrieve_content_object_metadata_element_values(
            $condition,
            $offset,
            $count,
            $order_property);
    }

    public static function retrieve_content_object_metadata_element_value($id)
    {
        return MetadataDataManager :: get_instance()->retrieve_content_object_metadata_element_value($id);
    }

    public static function count_user_metadata_element_values($condition)
    {
        return MetadataDataManager :: get_instance()->count_user_metadata_element_values($condition);
    }

    public static function retrieve_user_metadata_element_values($condition = null, $offset = null, $count = null,
        $order_property = null)
    {
        return MetadataDataManager :: get_instance()->retrieve_user_metadata_element_values(
            $condition,
            $offset,
            $count,
            $order_property);
    }

    public static function retrieve_user_metadata_element_value($id)
    {
        return MetadataDataManager :: get_instance()->retrieve_user_metadata_element_value($id);
    }

    public function count_group_metadata_element_values($condition)
    {
        return MetadataDataManager :: get_instance()->count_group_metadata_element_values($condition);
    }

    public static function retrieve_group_metadata_element_values($condition = null, $offset = null, $count = null,
        $order_property = null)
    {
        return MetadataDataManager :: get_instance()->retrieve_group_metadata_element_values(
            $condition,
            $offset,
            $count,
            $order_property);
    }

    public function retrieve_group_metadata_element_value($id)
    {
        return MetadataDataManager :: get_instance()->retrieve_group_metadata_element_value($id);
    }

    // function count_metadata_element_values($condition)
    // {
    // return MetadataDataManager :: get_instance()->count_metadata_element_values($condition);
    // }
    //
    // static function retrieve_metadata_element_values($condition = null, $offset = null, $count = null,
    // $order_property = null)
    // {
    // return MetadataDataManager :: get_instance()->retrieve_metadata_element_values($condition, $offset, $count,
    // $order_property);
    // }
    //
    // function retrieve_metadata_element_value($id)
    // {
    // return MetadataDataManager :: get_instance()->retrieve_metadata_element_value($id);
    // }
    public static function get_content_object_publication_attributes($object_id, $type = null, $condition = null, $count = null, $offset = null, 
        $order_property = null)
    {
        return MetadataDataManager :: get_instance()->get_content_object_publication_attributes(
            $object_id,
            $type,
            $condition,
            $count,
            $offset,
            $order_property);
    }

    public static function get_content_object_publication_attribute($publication_id)
    {
        return MetadataDataManager :: get_instance()->get_content_object_publication_attribute($publication_id);
    }

    public static function count_publication_attributes($user = null, $object_id = null, $condition = null)
    {
        return MetadataDataManager :: get_instance()->count_publication_attributes($user, $object_id, $condition);
    }

    /**
     * Returns an array with all the metadata for a content-object array key =
     * metadata_property-namespace:metadata_property-name array value = metadata_property-value
     *
     * @param <int> $co_id id of the content-object
     */
    public static function retrieve_metadata_for_content_object($co_id)
    {
        $condition = new EqualityCondition(MetadataElementValue :: PROPERTY_CONTENT_OBJECT_ID, $co_id);
        $metadata_element_values = MetadataManager :: retrieve_content_object_metadata_element_values($condition);
        $metadata = array();
        while ($mpv = $metadata_element_values->next_result())
        {
            $mpti = $mpv->get_element_id();
            $type = self :: retrieve_metadata_element($mpti);
            $metadata[$type->get_ns_prefix() . ':' . $type->get_name()] = $mpv->get_value();
        }

        return $metadata;
    }

    public function count_metadata_attributes($condition)
    {
        return MetadataDataManager :: get_instance()->count_metadata_attributes($condition);
    }

    public function retrieve_metadata_attributes($condition = null, $offset = null, $count = null, $order_property = null)
    {
        return MetadataDataManager :: get_instance()->retrieve_metadata_attributes(
            $condition,
            $offset,
            $count,
            $order_property);
    }

    public function retrieve_metadata_attribute($id)
    {
        return MetadataDataManager :: get_instance()->retrieve_metadata_attribute($id);
    }

    public function count_content_object_metadata_attribute_values($condition)
    {
        return MetadataDataManager :: get_instance()->count_metadata_attribute_values($condition);
    }

    public function retrieve_metadata_attribute_values($condition = null, $offset = null, $count = null, $order_property = null)
    {
        return MetadataDataManager :: get_instance()->retrieve_metadata_attribute_values(
            $condition,
            $offset,
            $count,
            $order_property);
    }

    public function retrieve_metadata_attribute_value($id)
    {
        return MetadataDataManager :: get_instance()->retrieve_metadata_attribute_value($id);
    }

    // function count_metadata_attribute_values($condition)
    // {
    // return MetadataDataManager :: get_instance()->count_metadata_attribute_values($condition);
    // }
    //
    // function retrieve_metadata_attribute_values($condition = null, $offset = null, $count = null, $order_property =
    // null)
    // {
    // return MetadataDataManager :: get_instance()->retrieve_metadata_attribute_values($condition, $offset, $count,
    // $order_property);
    // }
    //
    // function retrieve_metadata_attribute_value($id)
    // {
    // return MetadataDataManager :: get_instance()->retrieve_metadata_attribute_value($id);
    // }
    public function retrieve_metadata_default_values($condition = null, $offset = null, $count = null, $order_property = null)
    {
        return MetadataDataManager :: get_instance()->retrieve_metadata_default_values(
            $condition = null,
            $offset = null,
            $count = null,
            $order_property = null);
    }

    public function retrieve_metadata_default_value($id)
    {
        return MetadataDataManager :: get_instance()->retrieve_metadata_default_value($id);
    }

    public function count_metadata_default_values($condition = null)
    {
        return MetadataDataManager :: get_instance()->count_metadata_default_values(
            $condition = null,
            $offset = null,
            $count = null,
            $order_property = null);
    }

    /*
     * @param array conditions - equality conditons with MetadataAttributeAssociation :: PROPERTY_PARENT_ID @return
     * array allowed_metadata_attributes
     */
    public function retrieve_allowed_metadata_attributes(array $conditions_allowed)
    {
        if (count($conditions_allowed))
        {
            $condition1 = (count($conditions_allowed) === 1) ? $conditions_allowed[0] : new OrCondition(
                $conditions_allowed);
            $type_condition = new EqualityCondition(
                MetadataAttributeAssociation :: PROPERTY_CHILD_TYPE,
                MetadataManager :: PARAM_METADATA_ATTRIBUTE);
            $condition = new AndCondition($condition1, $type_condition);

            $allowed_metadata_attributes = $this->retrieve_metadata_attribute_associations($condition);
            return $this->format_allowed_metadata_attributes($allowed_metadata_attributes);
        }
        return array();
    }

    /*
     * @param ArrayResultSet $allowed_metadata_attributes @return allowed_metadata_attribute_arr [parent_id][child_id]
     */
    public function format_allowed_metadata_attributes(ArrayResultSet $allowed_metadata_attributes)
    {
        $allowed_metadata_attribute_arr = array();

        while ($allowed_metadata_attribute = $allowed_metadata_attributes->next_result())
        {
            $allowed_metadata_attribute_arr[$allowed_metadata_attribute->get_parent_id()][$allowed_metadata_attribute->get_child_id()] = $allowed_metadata_attribute->get_child_id();
        }
        return $allowed_metadata_attribute_arr;
    }

    // Url Creation
    public function get_edit_associations_url($metadata_element)
    {
        return $this->get_url(
            array(
                self :: PARAM_ACTION => self :: ACTION_EDIT_ASSOCIATIONS,
                self :: PARAM_METADATA_ELEMENT => $metadata_element->get_id()));
    }

    public function get_update_metadata_attribute_association_url($metadata_attribute_association)
    {
        return $this->get_url(
            array(
                self :: PARAM_ACTION => self :: ACTION_EDIT_METADATA_ATTRIBUTE_ASSOCIATION,
                self :: PARAM_METADATA_ATTRIBUTE_ASSOCIATION => $metadata_attribute_association->get_id()));
    }

    public function get_delete_metadata_attribute_association_url($metadata_attribute_association)
    {
        return $this->get_url(
            array(
                self :: PARAM_ACTION => self :: ACTION_DELETE_METADATA_ATTRIBUTE_ASSOCIATION,
                self :: PARAM_METADATA_ATTRIBUTE_ASSOCIATION => $metadata_attribute_association->get_id()));
    }

    public function get_browse_metadata_attribute_associations_url()
    {
        return $this->get_url(array(self :: PARAM_ACTION => self :: ACTION_BROWSE_METADATA_ATTRIBUTE_ASSOCIATIONS));
    }

    public function get_create_metadata_namespace_url()
    {
        return $this->get_url(array(self :: PARAM_ACTION => self :: ACTION_CREATE_METADATA_NAMESPACE));
    }

    public function get_update_metadata_namespace_url($metadata_namespace)
    {
        return $this->get_url(
            array(
                self :: PARAM_ACTION => self :: ACTION_EDIT_METADATA_NAMESPACE,
                self :: PARAM_METADATA_NAMESPACE => $metadata_namespace->get_id()));
    }

    public function get_delete_metadata_namespace_url($metadata_namespace)
    {
        return $this->get_url(
            array(
                self :: PARAM_ACTION => self :: ACTION_DELETE_METADATA_NAMESPACE,
                self :: PARAM_METADATA_NAMESPACE => $metadata_namespace->get_id()));
    }

    public function get_browse_metadata_namespaces_url()
    {
        return $this->get_url(array(self :: PARAM_ACTION => self :: ACTION_BROWSE_METADATA_NAMESPACES));
    }

    public function get_create_content_object_property_metadata_url()
    {
        return $this->get_url(array(self :: PARAM_ACTION => self :: ACTION_CREATE_CONTENT_OBJECT_PROPERTY_METADATA));
    }

    public function get_update_content_object_property_metadata_url($content_object_property_metadata)
    {
        return $this->get_url(
            array(
                self :: PARAM_ACTION => self :: ACTION_EDIT_CONTENT_OBJECT_PROPERTY_METADATA,
                self :: PARAM_CONTENT_OBJECT_PROPERTY_METADATA => $content_object_property_metadata->get_id()));
    }

    public function get_delete_content_object_property_metadata_url($content_object_property_metadata)
    {
        return $this->get_url(
            array(
                self :: PARAM_ACTION => self :: ACTION_DELETE_CONTENT_OBJECT_PROPERTY_METADATA,
                self :: PARAM_CONTENT_OBJECT_PROPERTY_METADATA => $content_object_property_metadata->get_id()));
    }

    public function get_browse_content_object_property_metadatas_url()
    {
        return $this->get_url(array(self :: PARAM_ACTION => self :: ACTION_BROWSE_CONTENT_OBJECT_PROPERTY_METADATAS));
    }

    public function get_edit_metadata_element_nesting_url($metadata_element)
    {
        return $this->get_url(
            array(
                self :: PARAM_ACTION => self :: ACTION_EDIT_METADATA_ELEMENT_NESTING,
                self :: PARAM_METADATA_ELEMENT => $metadata_element->get_id()));
    }

    public function get_update_metadata_element_nesting_url($metadata_element_nesting)
    {
        return $this->get_url(
            array(
                self :: PARAM_ACTION => self :: ACTION_EDIT_METADATA_ELEMENT_NESTING,
                self :: PARAM_METADATA_ELEMENT_NESTING => $metadata_element_nesting->get_id()));
    }

    public function get_delete_metadata_element_nesting_url($metadata_element_nesting)
    {
        return $this->get_url(
            array(
                self :: PARAM_ACTION => self :: ACTION_DELETE_METADATA_ELEMENT_NESTING,
                self :: PARAM_METADATA_ELEMENT_NESTING => $metadata_element_nesting->get_id()));
    }

    public function get_browse_metadata_element_nestings_url()
    {
        return $this->get_url(array(self :: PARAM_ACTION => self :: ACTION_BROWSE_METADATA_ELEMENT_NESTINGS));
    }

    public function get_create_metadata_element_url()
    {
        return $this->get_url(array(self :: PARAM_ACTION => self :: ACTION_CREATE_METADATA_ELEMENT));
    }

    public function get_update_metadata_element_url($metadata_element)
    {
        return $this->get_url(
            array(
                self :: PARAM_ACTION => self :: ACTION_EDIT_METADATA_ELEMENT,
                self :: PARAM_METADATA_ELEMENT => $metadata_element->get_id()));
    }

    public function get_delete_metadata_element_url($metadata_element)
    {
        return $this->get_url(
            array(
                self :: PARAM_ACTION => self :: ACTION_DELETE_METADATA_ELEMENT,
                self :: PARAM_METADATA_ELEMENT => $metadata_element->get_id()));
    }

    public function get_browse_metadata_elements_url()
    {
        return $this->get_url(array(self :: PARAM_ACTION => self :: ACTION_BROWSE_METADATA_ELEMENTS));
    }

    public function get_create_metadata_default_value_url()
    {
        return $this->get_url(array(self :: PARAM_ACTION => self :: ACTION_CREATE_METADATA_DEFAULT_VALUE));
    }

    public function get_update_metadata_default_value_url($metadata_default_value)
    {
        return $this->get_url(
            array(
                self :: PARAM_ACTION => self :: ACTION_EDIT_METADATA_DEFAULT_VALUE,
                self :: PARAM_METADATA_DEFAULT_VALUE => $metadata_default_value->get_id()));
    }

    public function get_delete_metadata_default_value_url(MetadataDefaultValue $metadata_default_value)
    {
        return $this->get_url(
            array(
                self :: PARAM_ACTION => self :: ACTION_DELETE_METADATA_DEFAULT_VALUE,
                self :: PARAM_METADATA_DEFAULT_VALUE => $metadata_default_value->get_id()));
    }

    public function get_browse_metadata_default_values_url(MetadataElement $metadata_element)
    {
        return $this->get_url(
            array(
                self :: PARAM_ACTION => self :: ACTION_BROWSE_METADATA_DEFAULT_VALUES,
                MetadataManager :: PARAM_METADATA_ELEMENT => $metadata_element->get_id()));
    }

    public function get_edit_content_object_metadata_element_values_url($content_object)
    {
        return $this->get_url(
            array(
                self :: PARAM_ACTION => self :: ACTION_EDIT_CONTENT_OBJECT_METADATA,
                self :: PARAM_CONTENT_OBJECT => $content_object->get_id()));
    }

    public function get_delete_content_object_metadata_element_value_url($content_object_metadata_element_value)
    {
        return $this->get_url(
            array(
                self :: PARAM_ACTION => self :: ACTION_DELETE_METADATA_PROPERTY_VALUE,
                self :: PARAM_METADATA_PROPERTY_VALUE => $content_object_metadata_element_value->get_id()));
    }

    public function get_browse_content_object_metadata_element_values_url()
    {
        return $this->get_url(
            array(self :: PARAM_ACTION => self :: ACTION_BROWSE_CONTENT_OBJECT_METADATA_PROPERTY_VALUES));
    }

    public function get_edit_user_metadata_element_values_url($user)
    {
        return $this->get_url(
            array(self :: PARAM_ACTION => self :: ACTION_EDIT_USER_METADATA, self :: PARAM_USER => $user->get_id()));
    }

    public function get_delete_user_metadata_element_value_url($user_metadata_element_value)
    {
        return $this->get_url(
            array(
                self :: PARAM_ACTION => self :: ACTION_DELETE_USER_METADATA_PROPERTY_VALUE,
                self :: PARAM_USER_METADATA_PROPERTY_VALUE => $user_metadata_element_value->get_id()));
    }

    public function get_browse_user_metadata_element_values_url()
    {
        return $this->get_url(array(self :: PARAM_ACTION => self :: ACTION_BROWSE_USER_METADATA_PROPERTY_VALUES));
    }

    public function get_edit_group_metadata_element_values_url($group)
    {
        return $this->get_url(
            array(self :: PARAM_ACTION => self :: ACTION_EDIT_GROUP_METADATA, self :: PARAM_GROUP => $group->get_id()));
    }

    public function get_delete_group_metadata_element_value_url($group_metadata_element_value)
    {
        return $this->get_url(
            array(
                self :: PARAM_ACTION => self :: ACTION_DELETE_GROUP_METADATA_PROPERTY_VALUE,
                self :: PARAM_GROUP_METADATA_PROPERTY_VALUE => $group_metadata_element_value->get_id()));
    }

    public function get_browse_group_metadata_element_values_url()
    {
        return $this->get_url(array(self :: PARAM_ACTION => self :: ACTION_BROWSE_GROUP_METADATA_PROPERTY_VALUES));
    }

    // function get_edit_metadata_element_values_url($content_object)
    // {
    // return $this->get_url(array(self :: PARAM_ACTION => self :: ACTION_EDIT_METADATA,
    // self :: PARAM_CONTENT_OBJECT => $content_object->get_id()));
    // }
    //
    // function get_delete_metadata_element_value_url($metadata_element_value)
    // {
    // return $this->get_url(array(self :: PARAM_ACTION => self :: ACTION_DELETE_METADATA_PROPERTY_VALUE,
    // self :: PARAM_METADATA_PROPERTY_VALUE => $metadata_element_value->get_id()));
    // }
    //
    // function get_browse_metadata_element_values_url()
    // {
    // return $this->get_url(array(self :: PARAM_ACTION => self :: ACTION_BROWSE_METADATA_PROPERTY_VALUES));
    // }
    public function get_create_metadata_attribute_url()
    {
        return $this->get_url(array(self :: PARAM_ACTION => self :: ACTION_CREATE_METADATA_ATTRIBUTE));
    }

    public function get_update_metadata_attribute_url($metadata_attribute)
    {
        return $this->get_url(
            array(
                self :: PARAM_ACTION => self :: ACTION_EDIT_METADATA_ATTRIBUTE,
                self :: PARAM_METADATA_ATTRIBUTE => $metadata_attribute->get_id()));
    }

    public function get_delete_metadata_attribute_url($metadata_attribute)
    {
        return $this->get_url(
            array(
                self :: PARAM_ACTION => self :: ACTION_DELETE_METADATA_ATTRIBUTE,
                self :: PARAM_METADATA_ATTRIBUTE => $metadata_attribute->get_id()));
    }

    public function get_browse_metadata_attributes_url()
    {
        return $this->get_url(array(self :: PARAM_ACTION => self :: ACTION_BROWSE_METADATA_ATTRIBUTES));
    }

    public function get_create_metadata_attribute_value_url()
    {
        return $this->get_url(array(self :: PARAM_ACTION => self :: ACTION_CREATE_METADATA_ATTRIBUTE_VALUE));
    }

    public function get_update_metadata_attribute_value_url($metadata_attribute_value)
    {
        return $this->get_url(
            array(
                self :: PARAM_ACTION => self :: ACTION_EDIT_METADATA_ATTRIBUTE_VALUE,
                self :: PARAM_METADATA_ATTRIBUTE_VALUE => $metadata_attribute_value->get_id()));
    }

    public function get_delete_metadata_attribute_value_url($metadata_attribute_value)
    {
        return $this->get_url(
            array(
                self :: PARAM_ACTION => self :: ACTION_DELETE_METADATA_ATTRIBUTE_VALUE,
                self :: PARAM_METADATA_ATTRIBUTE_VALUE => $metadata_attribute_value->get_id()));
    }

    public function get_move_metadata_element_url($metadata_element, $move)
    {
        return $this->get_url(
            array(
                self :: PARAM_ACTION => self :: ACTION_MOVE_METADATA_ELEMENT,
                self :: PARAM_MOVE => $move,
                self :: PARAM_METADATA_ELEMENT => $metadata_element->get_id()));
    }

    public function get_display_order_total($namespace)
    {
        if (! isset($this->display_order_total[$namespace]))
        {
            $condition = new EqualityCondition(MetadataElement :: PROPERTY_NAMESPACE, $namespace);
            $this->display_order_total[$namespace] = MetadataDataManager :: get_instance()->count_metadata_elements(
                $condition);
        }
        return $this->display_order_total[$namespace];
    }

    public function get_browse_url()
    {
        return $this->get_url(array(self :: PARAM_ACTION => self :: ACTION_BROWSE));
    }

    public static function get_application_platform_admin_links()
    {
        $links = array();
        $links[] = new DynamicAction(
            Translation :: get('MetadataNamespacesBrowser'),
            Translation :: get('MetadataNamespacesDescription'),
            Theme :: get_image_path() . 'admin/build.png',
            Redirect :: get_link(
                self :: APPLICATION_NAME,
                array(Application :: PARAM_ACTION => self :: ACTION_BROWSE_METADATA_NAMESPACES),
                array(),
                false,
                Redirect :: TYPE_CORE));
        $links[] = new DynamicAction(
            Translation :: get('MetadataElementsBrowser'),
            Translation :: get('MetadataElementsDescription'),
            Theme :: get_image_path() . 'admin/build.png',
            Redirect :: get_link(
                self :: APPLICATION_NAME,
                array(Application :: PARAM_ACTION => self :: ACTION_BROWSE_METADATA_ELEMENTS),
                array(),
                false,
                Redirect :: TYPE_CORE));
        $links[] = new DynamicAction(
            Translation :: get('MetadataAttributesBrowser'),
            Translation :: get('MetadataAttributesDescription'),
            Theme :: get_image_path() . 'admin/build.png',
            Redirect :: get_link(
                self :: APPLICATION_NAME,
                array(Application :: PARAM_ACTION => self :: ACTION_BROWSE_METADATA_ATTRIBUTES),
                array(),
                false,
                Redirect :: TYPE_CORE));
        $links[] = new DynamicAction(
            Translation :: get('ContentObjectPropertyMetadatasBrowser'),
            Translation :: get('ContentObjectPropertyMetadatasBrowserDescription'),
            Theme :: get_image_path() . 'admin/build.png',
            Redirect :: get_link(
                self :: APPLICATION_NAME,
                array(Application :: PARAM_ACTION => self :: ACTION_BROWSE_CONTENT_OBJECT_PROPERTY_METADATAS),
                array(),
                false,
                Redirect :: TYPE_CORE));
        $links[] = new DynamicAction(
            Translation :: get('ExportMetadaTitle'),
            Translation :: get('ExportMetadataDescription'),
            Theme :: get_image_path() . 'admin/build.png',
            Redirect :: get_link(
                self :: APPLICATION_NAME,
                array(Application :: PARAM_ACTION => self :: ACTION_EXPORT_METADATA),
                array(),
                false,
                Redirect :: TYPE_CORE));
        $links[] = new DynamicAction(
            Translation :: get('ImportMetadaTitle'),
            Translation :: get('ImportMetadataDescription'),
            Theme :: get_image_path() . 'admin/build.png',
            Redirect :: get_link(
                self :: APPLICATION_NAME,
                array(Application :: PARAM_ACTION => self :: ACTION_IMPORT_METADATA),
                array(),
                false,
                Redirect :: TYPE_CORE));

        $info = parent :: get_application_platform_admin_links(self :: APPLICATION_NAME);
        $info['links'] = $links;
        return $info;
    }
}
