<?php
namespace core\metadata;

use common\libraries\DataClass;
use common\libraries\Utilities;
use common\libraries\EqualityCondition;
use common\libraries\Translation;
use common\libraries\Setting;
/**
 * metadata
 */

/**
 * This class describes a MetadataNamespace data object
 * @author Sven Vanpoucke
 * @author Jens Vanderheyden
 */
class MetadataNamespace extends DataClass
{
    const CLASS_NAME = __CLASS__;

    /**
     * MetadataNamespace properties
     */
    const PROPERTY_NS_PREFIX = 'ns_prefix';
    const PROPERTY_NAME = 'name';
    const PROPERTY_URL = 'url';

    /**
     * Get the default properties
     * @return array The property names.
     */
    public static function get_default_property_names()
    {
        return array(parent :: PROPERTY_ID, self :: PROPERTY_NS_PREFIX, self :: PROPERTY_NAME, self :: PROPERTY_URL);
    }

    public function get_data_manager()
    {
        return MetadataDataManager :: get_instance();
    }

    /**
     * Returns the ns_prefix of this MetadataNamespace.
     * @return the ns_prefix.
     */
    public function get_ns_prefix()
    {
        return $this->get_default_property(self :: PROPERTY_NS_PREFIX);
    }

    /**
     * Sets the ns_prefix of this MetadataNamespace.
     * @param ns_prefix
     */
    public function set_ns_prefix($ns_prefix)
    {
        $this->set_default_property(self :: PROPERTY_NS_PREFIX, $ns_prefix);
    }

    /**
     * Returns the name of this MetadataNamespace.
     * @return the name.
     */
    public function get_name()
    {
        return $this->get_default_property(self :: PROPERTY_NAME);
    }

    /**
     * Sets the name of this MetadataNamespace.
     * @param name
     */
    public function set_name($name)
    {
        $this->set_default_property(self :: PROPERTY_NAME, $name);
    }

    /**
     * Returns the url of this MetadataNamespace.
     * @return the url.
     */
    public function get_url()
    {
        return $this->get_default_property(self :: PROPERTY_URL);
    }

    /**
     * Sets the url of this MetadataNamespace.
     * @param url
     */
    public function set_url($url)
    {
        $this->set_default_property(self :: PROPERTY_URL, $url);
    }

    public static function get_table_name()
    {
        return Utilities :: camelcase_to_underscores(Utilities :: get_classname_from_namespace(self :: CLASS_NAME));
    }

    /*
     * object creation if ns_prefix is unique
     * return MetadaNamespace or false
     */
    public function create()
    {
        $success = parent :: create();

        //set as default if it is the first namespace being added
        if ($this->get_data_manager()->count_metadata_namespaces() === 1)
        {
            $setting = new Setting();
            $setting->set_application(MetadataManager :: APPLICATION_NAME);
            $setting->set_variable(MetadataManager :: SETTING_DEFAULT_NAMESPACE);
            $setting->set_value(self :: get_id());

            $setting->create();
        }
        return $success;
    }

    public function delete()
    {
        //only deletes if 0 children
        $mdm = $this->get_data_manager();

        $condition = new EqualityCondition(MetadataElement :: PROPERTY_NS_PREFIX, $this->get_ns_prefix());
        $count = $mdm->count_metadata_elements($condition);

        $condition = new EqualityCondition(MetadataAttribute :: PROPERTY_NS_PREFIX, $this->get_ns_prefix());
        $count += $mdm->count_metadata_attributes($condition);

        if ($count === 0)
        {
            return parent :: delete();
        }
        else
        {
            $this->add_error(Translation :: get('ChildrenExist'));
            return false;
        }
    }
}
