<?php

namespace core\migration;

use common\libraries\CommonDataManager;

use common\libraries\FormValidator;
use core\admin\AdminDataManager;
use common\libraries\Utilities;
use common\libraries\Translation;
use common\libraries\Filesystem;
use common\libraries\Request;
use common\libraries\Display;

/**
 * $Id: group_form.class.php 224 2009-11-13 14:40:30Z kariboe $
 *
 * @package groups.lib.forms
 */
class MigrationForm extends FormValidator
{
    const SETTING_PLATFORM = 'platform';
    const SETTING_PLATFORM_PATH = 'platform_path';
    const SETTING_MOVE_FILES = 'move_files';
    const SETTING_IN_MIGRATION = 'in_migration';

    const PARAM_MIGRATE = 'migrate';
    const PARAM_SELECT_BLOCKS = 'select_blocks';

    public function __construct($action)
    {
        parent :: __construct('migration_settings', 'post', $action);

        $this->build_form();
        $this->setDefaults();
    }

    public function build_form()
    {
        $values = $this->exportValues();

        $this->addElement('category', Translation :: get('General'));

        $this->addElement('select', self :: SETTING_PLATFORM, Translation :: get('Platform'), $this->get_platforms());
        $this->addRule(self :: SETTING_PLATFORM, Translation :: get('ThisFieldIsRequired', null, Utilities :: COMMON_LIBRARIES), 'required');

        $this->addElement('text', self :: SETTING_PLATFORM_PATH, Translation :: get('PlatformPath'), array(
                "size" => "50"));
        $this->addRule(self :: SETTING_PLATFORM_PATH, Translation :: get('ThisFieldIsRequired', null, Utilities :: COMMON_LIBRARIES), 'required');

        $this->addElement('checkbox', self :: SETTING_MOVE_FILES, null, Translation :: get('MoveFiles'));

        $this->addElement('category');

        $select_blocks = (Request :: post(self :: PARAM_SELECT_BLOCKS) || Request :: post(self :: PARAM_SUBMIT));

        if ($select_blocks)
        {
            $this->freeze(self :: SETTING_PLATFORM);

            $this->addElement('category', Translation :: get('MigrationBlocks'));

            $blocks = $this->get_blocks(Request :: post(self :: SETTING_PLATFORM));
            foreach ($blocks as $block)
            {
                $this->addElement('checkbox', self :: PARAM_MIGRATE . '[' . $block . ']', null, Translation :: get('Migrate' . Utilities :: underscores_to_camelcase($block)));
            }

            $this->addElement('category');

            $buttons[] = $this->createElement('style_submit_button', self :: PARAM_SELECT_BLOCKS, Translation :: get('SelectBlocks'), array(
                    'class' => 'positive update',
                    'style' => 'display: none;'));
            $buttons[] = $this->createElement('style_submit_button', self :: PARAM_SUBMIT, Translation :: get('Migrate'), array(
                    'class' => 'positive update'));
        }
        else
        {
            $buttons[] = $this->createElement('style_submit_button', self :: PARAM_SELECT_BLOCKS, Translation :: get('SelectBlocks'), array(
                    'class' => 'positive update'));
            $buttons[] = $this->createElement('style_submit_button', self :: PARAM_SUBMIT, Translation :: get('Migrate'), array(
                    'class' => 'positive update',
                    'style' => 'display: none;'));
        }

        $buttons[] = $this->createElement('style_reset_button', self :: PARAM_RESET, Translation :: get('Reset', null, Utilities :: COMMON_LIBRARIES), array(
                'class' => 'normal empty'));

        $this->addGroup($buttons, 'buttons', null, '&nbsp;', false);
    }

    public function validate()
    {
        $values = $this->exportValues();
        if (isset($values['submit']))
        {
            return parent :: validate();
        }

        return false;
    }

    public function create_migration_settings()
    {
        $values = $this->exportValues();
        $adm = AdminDataManager :: get_instance();

        $settings = array(self :: SETTING_PLATFORM, self :: SETTING_PLATFORM_PATH, self :: SETTING_MOVE_FILES);
        foreach ($settings as $setting)
        {
            $value = $values[$setting];

            if (is_null($value))
            {
                $value = 0;
            }

            $setting_obj = CommonDataManager :: retrieve_setting_from_variable_name($setting, MigrationManager :: context());
            if (! $setting_obj)
            {
                Display :: error_page(Translation :: get('SettingNotFound', array('SETTING' => '"' . $setting . '"'), Utilities :: COMMON_LIBRARIES));
            }
            $setting_obj->set_value($value);
            $setting_obj->update();
        }

        $setting = CommonDataManager :: retrieve_setting_from_variable_name(self :: SETTING_IN_MIGRATION, MigrationManager :: context());
        $setting->set_value(1);
        $setting->update();

        foreach ($values[self :: PARAM_MIGRATE] as $block => $value)
        {
            $migration_block_registration = new MigrationBlockRegistration();
            $migration_block_registration->set_name($block);
            $migration_block_registration->create();
        }
    }

    public function get_platforms()
    {
        $dir = dirname(__FILE__) . '/../platform/';
        $contents = Filesystem :: get_directory_content($dir, Filesystem :: LIST_DIRECTORIES, false);

        $folders = array();

        foreach ($contents as $content)
        {
            $folders[$content] = $content;
        }

        return $folders;
    }

    public function get_blocks($platform)
    {
        $properties = MigrationProperties :: factory($platform);
        return $properties->get_migration_blocks();
    }

}
