<?php
namespace core\migration;

use common\libraries\CommonDataManager;
use application\weblcms\course\Course;
use common\libraries\Translation;
use common\libraries\Utilities;
use application\weblcms\WeblcmsDataManager;
use common\libraries\EqualityCondition;
use application\weblcms\CourseModule;
use common\libraries\PlatformSetting;
use application\weblcms\CourseSettingsController;
use application\weblcms\CourseSettingsConnector;
use application\weblcms\CourseManagementRights;
use application\weblcms\course\CourseDataManager;

/**
 * $Id: dokeos185_course.class.php 221 2009-11-13 14:36:41Z vanpouckesven $
 *
 * @package migration.platform.dokeos185
 */
require_once dirname(__FILE__) . '/../dokeos185_migration_data_class.class.php';
require_once dirname(__FILE__) . '/dokeos185_user.class.php';

/**
 * This class represents an old Dokeos 1.8.5 course
 *
 * @author David Van Wayenbergh
 * @author Sven Vanpoucke
 */
class Dokeos185Course extends Dokeos185MigrationDataClass
{
    const CLASS_NAME = __CLASS__;
    const TABLE_NAME = 'course';
    const DATABASE_NAME = 'main_database';

    /**
     * course properties
     */
    const PROPERTY_CODE = 'code';
    const PROPERTY_DIRECTORY = 'directory';
    const PROPERTY_DB_NAME = 'db_name';
    const PROPERTY_COURSE_LANGUAGE = 'course_language';
    const PROPERTY_TITLE = 'title';
    const PROPERTY_DESCRIPTION = 'description';
    const PROPERTY_CATEGORY_CODE = 'category_code';
    const PROPERTY_VISIBILITY = 'visibility';
    const PROPERTY_TUTOR_NAME = 'tutor_name';
    const PROPERTY_VISUAL_CODE = 'visual_code';
    const PROPERTY_DEPARTMENT_NAME = 'department_name';
    const PROPERTY_DEPARTMENT_URL = 'department_url';
    const PROPERTY_LAST_VISIT = 'last_visit';
    const PROPERTY_LAST_EDIT = 'last_edit';
    const PROPERTY_CREATION_DATE = 'creation_date';
    const PROPERTY_EXPIRATION_DATE = 'expiration_date';
    const PROPERTY_TARGET_COURSE_CODE = 'target_course_code';
    const PROPERTY_SUBSCRIBE = 'subscribe';
    const PROPERTY_UNSUBSCRIBE = 'unsubscribe';
    const PROPERTY_REGISTRATION_CODE = 'registration_code';

    /**
     * Get the default properties of all courses.
     *
     * @return array The property names.
     */
    public static function get_default_property_names()
    {
        return array(
            self :: PROPERTY_CODE,
            self :: PROPERTY_DIRECTORY,
            self :: PROPERTY_DB_NAME,
            self :: PROPERTY_COURSE_LANGUAGE,
            self :: PROPERTY_TITLE,
            self :: PROPERTY_DESCRIPTION,
            self :: PROPERTY_CATEGORY_CODE,
            self :: PROPERTY_VISIBILITY,
            self :: PROPERTY_TUTOR_NAME,
            self :: PROPERTY_VISUAL_CODE,
            self :: PROPERTY_DEPARTMENT_URL,
            self :: PROPERTY_LAST_VISIT,
            self :: PROPERTY_LAST_EDIT,
            self :: PROPERTY_CREATION_DATE,
            self :: PROPERTY_EXPIRATION_DATE,
            self :: PROPERTY_TARGET_COURSE_CODE,
            self :: PROPERTY_SUBSCRIBE,
            self :: PROPERTY_UNSUBSCRIBE,
            self :: PROPERTY_REGISTRATION_CODE,
            self :: PROPERTY_DEPARTMENT_NAME);
    }

    /**
     * COURSE GETTERS AND SETTERS
     */

    /**
     * Returns the code of this course.
     *
     * @return String The code.
     */
    public function get_code()
    {
        return $this->get_default_property(self :: PROPERTY_CODE);
    }

    /**
     * Returns the directory of this course.
     *
     * @return String The directory.
     */
    public function get_directory()
    {
        return $this->get_default_property(self :: PROPERTY_DIRECTORY);
    }

    /**
     * Returns the db_name of this course.
     *
     * @return String The db_name.
     */
    public function get_db_name()
    {
        return $this->get_default_property(self :: PROPERTY_DB_NAME);
    }

    /**
     * Returns the course_language of this course.
     *
     * @return String The course_language.
     */
    public function get_course_language()
    {
        return $this->get_default_property(self :: PROPERTY_COURSE_LANGUAGE);
    }

    /**
     * Returns the title of this course.
     *
     * @return String The title.
     */
    public function get_title()
    {
        return $this->get_default_property(self :: PROPERTY_TITLE);
    }

    /**
     * Returns the description of this course.
     *
     * @return String The discription.
     */
    public function get_description()
    {
        return $this->get_default_property(self :: PROPERTY_DESCRIPTION);
    }

    /**
     * Returns the category_code of this course.
     *
     * @return String The category_code.
     */
    public function get_category_code()
    {
        return $this->get_default_property(self :: PROPERTY_CATEGORY_CODE);
    }

    /**
     * Returns the visibility of this course.
     *
     * @return int The visibility.
     */
    public function get_visibility()
    {
        return $this->get_default_property(self :: PROPERTY_VISIBILITY);
    }

    /**
     * Returns the tutor_name of this course.
     *
     * @return String The tutor_name.
     */
    public function get_tutor_name()
    {
        return $this->get_default_property(self :: PROPERTY_TUTOR_NAME);
    }

    /**
     * Returns the visual_code of this course.
     *
     * @return String The visual_code.
     */
    public function get_visual_code()
    {
        return $this->get_default_property(self :: PROPERTY_VISUAL_CODE);
    }

    /**
     * Returns the department_name of this course.
     *
     * @return String The department_name.
     */
    public function get_department_name()
    {
        return $this->get_default_property(self :: PROPERTY_DEPARTMENT_NAME);
    }

    /**
     * Returns the department_url of this course.
     *
     * @return String The department_url.
     */
    public function get_department_url()
    {
        return $this->get_default_property(self :: PROPERTY_DEPARTMENT_URL);
    }

    /**
     * Returns the last_visit of this course.
     *
     * @return String The last_visit.
     */
    public function get_last_visit()
    {
        return $this->get_default_property(self :: PROPERTY_LAST_VISIT);
    }

    /**
     * Returns the last_edit of this course.
     *
     * @return String The last_edit.
     */
    public function get_last_edit()
    {
        return $this->get_default_property(self :: PROPERTY_LAST_EDIT);
    }

    /**
     * Returns the creation_date of this course.
     *
     * @return String The creation_date.
     */
    public function get_creation_date()
    {
        return $this->get_default_property(self :: PROPERTY_CREATION_DATE);
    }

    /**
     * Returns the expiration_date of this course.
     *
     * @return String The expiration_date.
     */
    public function get_expiration_date()
    {
        return $this->get_default_property(self :: PROPERTY_EXPIRATION_DATE);
    }

    /**
     * Returns the target_course_code of this course.
     *
     * @return String The target_course_code.
     */
    public function get_target_course_code()
    {
        return $this->get_default_property(self :: PROPERTY_TARGET_COURSE_CODE);
    }

    /**
     * Returns the subscribe of this course.
     *
     * @return int The subscribe.
     */
    public function get_subscribe()
    {
        return $this->get_default_property(self :: PROPERTY_SUBSCRIBE);
    }

    /**
     * Returns the unsubscribe of this course.
     *
     * @return int The unsubscribe.
     */
    public function get_unsubscribe()
    {
        return $this->get_default_property(self :: PROPERTY_UNSUBSCRIBE);
    }

    /**
     * Returns the registration_code of this course.
     *
     * @return String The registration_code.
     */
    public function get_registration_code()
    {
        return $this->get_default_property(self :: PROPERTY_REGISTRATION_CODE);
    }

    /**
     * Sets the code of this course.
     *
     * @param String $code The code.
     */
    public function set_code($code)
    {
        $this->set_default_property(self :: PROPERTY_CODE, $code);
    }

    /**
     * Check if the course is valid
     *
     * @return true if the course is valid
     */
    public function is_valid()
    {
        $folder = $this->get_data_manager()->get_sys_path() . '/courses/' . $this->get_directory();
        $database_exists = $this->get_data_manager()->database_exists(strtolower($this->get_db_name()));

        if (! $this->get_code() || ! $database_exists || ! file_exists($folder) ||
             $this->get_failed_element($this->get_category_code(), 'main_database.course_category'))
        {
            $this->create_failed_element($this->get_code());
            $this->set_message(
                Translation :: get('GeneralInvalidMessage', array('TYPE' => 'course', 'ID' => $this->get_code())));
            return false;
        }

        return true;
    }

    // check if visual code is avalaible
    public function check_visual_code($visual_code, $index = 0)
    {
        if (WeblcmsDataManager :: is_course_code_available($visual_code))
        {
            return $visual_code;
        }
        else
        {
            $index ++;
            return $this->check_visual_code($visual_code . $index, $index);
        }
    }

    /**
     * Convert to new course
     *
     * @return the new course
     */
    public function convert_data()
    {
        // control if the weblcms application exists
        $is_registered = CommonDataManager :: is_registered('application\weblcms');

        if ($is_registered)
        {
            // Course - General
            $chamilo_course = new Course();

            // title & visual_code
            $chamilo_course->set_title($this->get_title());
            $new_visual_code = $this->check_visual_code($this->get_visual_code());
            $chamilo_course->set_visual_code($new_visual_code);

            // titular id
            $tutor_name = trim($this->get_tutor_name());
            $pos = strpos($tutor_name, ','); // more than 1 person in course tutor name
            if ($pos > 0)
            {
                $tutor_name = trim(substr($tutor_name, 0, $pos));
            }

            $titular = $this->get_data_manager()->retrieve_user_by_fullname($tutor_name);

            if ($titular)
            {
                $titular_id = $this->get_id_reference($titular->get_user_id(), 'main_database.user');
            }
            else
            {
                $titular_id = 0;
            }

            $chamilo_course->set_titular_id($titular_id);

            // category
            $category_id = $this->get_id_reference($this->get_category_code(), 'main_database.course_category');
            if ($category_id)
            {
                $chamilo_course->set_category_id($category_id);
            }

            // departement_name & url
            // $chamilo_course->set_external_name($this->get_department_name()); //no longer supported in C3
            // $chamilo_course->set_external_url($this->get_department_url());

            // Course - Settings
            $iso_lang = self :: $language_to_iso_mapping[$this->get_course_language()];
            if (CommonDataManager :: is_language_active($iso_lang))
            {
                $chamilo_course->set_language($iso_lang);
            }
            else
            {
                $chamilo_course->set_language(PlatformSetting :: get('platform_language'));
            }

            // Courses - Creation/Modification Dates
            $chamilo_course->set_default_property(Course :: PROPERTY_LAST_VISIT, $this->get_last_visit());
            $chamilo_course->set_default_property(Course :: PROPERTY_LAST_EDIT, $this->get_last_edit());
            $chamilo_course->set_default_property(Course :: PROPERTY_CREATION_DATE, $this->get_creation_date());
            $chamilo_course->set_default_property(Course :: PROPERTY_EXPIRATION_DATE, $this->get_expiration_date());

            // create course in database
            if ($chamilo_course->create())
            {

                // settings of the course (duplicate in course settings table)
                $settings = array();
                $settings[CourseSettingsController :: SETTING_PARAM_COURSE_SETTINGS] = array();
                $settings[CourseSettingsController :: SETTING_PARAM_COURSE_SETTINGS][CourseSettingsConnector :: CATEGORY] = $chamilo_course->get_category_id();
                $settings[CourseSettingsController :: SETTING_PARAM_COURSE_SETTINGS][CourseSettingsConnector :: LANGUAGE] = $chamilo_course->get_language();
                $settings[CourseSettingsController :: SETTING_PARAM_COURSE_SETTINGS][CourseSettingsConnector :: TITULAR] = $chamilo_course->get_titular_id();

                // visibility = 3: Open - access allowed for the whole world
                // visibility = 2: Open - access allowed for users registered on the platform
                // visibility = 1: Private access (site accessible only to people on the user list)
                // visibility = 0: Completely closed; the course is only accessible to the course admin.

                // chamilo: only 1 and 0 (open and closed)
                if ($this->get_visibility() >= 1)
                { // visibility=2 is also possible
                    $settings[CourseSettingsController :: SETTING_PARAM_COURSE_SETTINGS][CourseSettingsConnector :: VISIBILITY] = 1;
                }
                else
                {
                    $settings[CourseSettingsController :: SETTING_PARAM_COURSE_SETTINGS][CourseSettingsConnector :: VISIBILITY] = 0;
                }

                // Course - Lay-out
                $settings[CourseSettingsController :: SETTING_PARAM_COURSE_SETTINGS][CourseSettingsConnector :: MENU_LAYOUT] = CourseSettingsConnector :: MENU_LAYOUT_RIGHT_WITH_TEXT;

                // Course - Tools
                // $chamilo_course->set_max_number_of_members(0); //no longer supported in C3 (coursetype and
                // coursegroup, but not in course)
                // $settings[SETTING_PARAM_TOOL_SETTINGS]['course_group']['maximum_number_of_members_per_group'] = 0;
                // //default is 0 anyway

                $chamilo_course->create_course_settings_from_values($settings, true);

                // Course - Rights
                $rights = array();
                $rights[CourseManagementRights :: PARAM_RIGHT_OPTION][CourseManagementRights :: DIRECT_SUBSCRIBE_RIGHT] = $this->get_subscribe() +
                     1; // convert
                                                                                                                                                        // 0=don't
                                                                                                                                                        // allow,
                                                                                                                                                        // 1=allow
                                                                                                                                                        // in
                                                                                                                                                        // Dokeos
                                                                                                                                                        // to
                                                                                                                                                        // 1=nobody,2=everyone
                                                                                                                                                        // in
                                                                                                                                                        // Chamilo
                $rights[CourseManagementRights :: PARAM_RIGHT_OPTION][CourseManagementRights :: DIRECT_UNSUBSCRIBE_RIGHT] = $this->get_unsubscribe() +
                     1;
                CourseManagementRights :: get_instance()->create_rights_from_values($chamilo_course, $rights);

                // set all standard tools invisible
                // $condition = new EqualityCondition(CourseModule::PROPERTY_COURSE_CODE, $chamilo_course->get_id());
                // WeblcmsDataManager :: get_instance()->update_course_module_visibility($condition,0);
                CourseDataManager :: set_tool_visibility_by_tool_name($chamilo_course->get_id(), null, 0);
            }

            // Add id references to temp table
            $this->create_id_reference($this->get_code(), $chamilo_course->get_id());
            $this->set_message(
                Translation :: get(
                    'GeneralConvertedMessage',
                    array('TYPE' => 'course', 'OLD_ID' => $this->get_code(), 'NEW_ID' => $chamilo_course->get_id())));
        }
    }

    public static function get_table_name()
    {
        return Utilities :: camelcase_to_underscores(substr(Utilities :: get_classname_from_namespace(__CLASS__), 9));
    }

    public static function get_class_name()
    {
        return self :: CLASS_NAME;
    }

    public function get_database_name()
    {
        return self :: DATABASE_NAME;
    }
}
