<?php
namespace core\migration;


use common\libraries\Translation;
use common\libraries\Utilities;
use common\libraries\Setting;
/**
 * $Id: dokeos185_course_setting.class.php 221 2009-11-13 14:36:41Z vanpouckesven $
 * @package migration.lib.platform.dokeos185
 */
require_once dirname(__FILE__) . '/../dokeos185_course_data_migration_data_class.class.php';

/**
 * This class presents a dokeos185 course_setting
 *
 * @author Sven Vanpoucke
 */
class Dokeos185CourseSetting extends Dokeos185CourseDataMigrationDataClass
{
    const CLASS_NAME = __CLASS__;
    const TABLE_NAME = 'course_setting';

    /**
     * Dokeos185CourseSetting properties
     */
    const PROPERTY_ID = 'id';
    const PROPERTY_VARIABLE = 'variable';
    const PROPERTY_SUBKEY = 'subkey';
    const PROPERTY_TYPE = 'type';
    const PROPERTY_CATEGORY = 'category';
    const PROPERTY_VALUE = 'value';
    const PROPERTY_TITLE = 'title';
    const PROPERTY_COMMENT = 'comment';
    const PROPERTY_SUBKEYTEXT = 'subkeytext';

    /**
     * Default properties stored in an associative array.
     */
    private $defaultProperties;

    /**
     * Creates a new Dokeos185CourseSetting object
     * @param array $defaultProperties The default properties
     */
    public function __construct($defaultProperties = array())
    {
        $this->defaultProperties = $defaultProperties;
    }

    /**
     * Gets a default property by name.
     * @param string $name The name of the property.
     */
    public function get_default_property($name)
    {
        return $this->defaultProperties[$name];
    }

    /**
     * Gets the default properties
     * @return array An associative array containing the properties.
     */
    public function get_default_properties()
    {
        return $this->defaultProperties;
    }

    /**
     * Get the default properties
     * @return array The property names.
     */
    public static function get_default_property_names()
    {
        return array(self :: PROPERTY_ID, self :: PROPERTY_VARIABLE, self :: PROPERTY_SUBKEY, self :: PROPERTY_TYPE,
                self :: PROPERTY_CATEGORY, self :: PROPERTY_VALUE, self :: PROPERTY_TITLE, self :: PROPERTY_COMMENT,
                self :: PROPERTY_SUBKEYTEXT);
    }

    /**
     * Sets a default property by name.
     * @param string $name The name of the property.
     * @param mixed $value The new value for the property.
     */
    public function set_default_property($name, $value)
    {
        $this->defaultProperties[$name] = $value;
    }

    /**
     * Sets the default properties of this class
     */
    public function set_default_properties($defaultProperties)
    {
        $this->defaultProperties = $defaultProperties;
    }

    /**
     * Returns the id of this Dokeos185CourseSetting.
     * @return the id.
     */
    public function get_id()
    {
        return $this->get_default_property(self :: PROPERTY_ID);
    }

    /**
     * Sets the id of this Dokeos185CourseSetting.
     * @param id
     */
    public function set_id($id)
    {
        $this->set_default_property(self :: PROPERTY_ID, $id);
    }

    /**
     * Returns the variable of this Dokeos185CourseSetting.
     * @return the variable.
     */
    public function get_variable()
    {
        return $this->get_default_property(self :: PROPERTY_VARIABLE);
    }

    /**
     * Sets the variable of this Dokeos185CourseSetting.
     * @param variable
     */
    public function set_variable($variable)
    {
        $this->set_default_property(self :: PROPERTY_VARIABLE, $variable);
    }

    /**
     * Returns the subkey of this Dokeos185CourseSetting.
     * @return the subkey.
     */
    public function get_subkey()
    {
        return $this->get_default_property(self :: PROPERTY_SUBKEY);
    }

    /**
     * Sets the subkey of this Dokeos185CourseSetting.
     * @param subkey
     */
    public function set_subkey($subkey)
    {
        $this->set_default_property(self :: PROPERTY_SUBKEY, $subkey);
    }

    /**
     * Returns the type of this Dokeos185CourseSetting.
     * @return the type.
     */
    public function get_type()
    {
        return $this->get_default_property(self :: PROPERTY_TYPE);
    }

    /**
     * Sets the type of this Dokeos185CourseSetting.
     * @param type
     */
    public function set_type($type)
    {
        $this->set_default_property(self :: PROPERTY_TYPE, $type);
    }

    /**
     * Returns the category of this Dokeos185CourseSetting.
     * @return the category.
     */
    public function get_category()
    {
        return $this->get_default_property(self :: PROPERTY_CATEGORY);
    }

    /**
     * Sets the category of this Dokeos185CourseSetting.
     * @param category
     */
    public function set_category($category)
    {
        $this->set_default_property(self :: PROPERTY_CATEGORY, $category);
    }

    /**
     * Returns the value of this Dokeos185CourseSetting.
     * @return the value.
     */
    public function get_value()
    {
        return $this->get_default_property(self :: PROPERTY_VALUE);
    }

    /**
     * Sets the value of this Dokeos185CourseSetting.
     * @param value
     */
    public function set_value($value)
    {
        $this->set_default_property(self :: PROPERTY_VALUE, $value);
    }

    /**
     * Returns the title of this Dokeos185CourseSetting.
     * @return the title.
     */
    public function get_title()
    {
        return $this->get_default_property(self :: PROPERTY_TITLE);
    }

    /**
     * Sets the title of this Dokeos185CourseSetting.
     * @param title
     */
    public function set_title($title)
    {
        $this->set_default_property(self :: PROPERTY_TITLE, $title);
    }

    /**
     * Returns the comment of this Dokeos185CourseSetting.
     * @return the comment.
     */
    public function get_comment()
    {
        return $this->get_default_property(self :: PROPERTY_COMMENT);
    }

    /**
     * Sets the comment of this Dokeos185CourseSetting.
     * @param comment
     */
    public function set_comment($comment)
    {
        $this->set_default_property(self :: PROPERTY_COMMENT, $comment);
    }

    /**
     * Returns the subkeytext of this Dokeos185CourseSetting.
     * @return the subkeytext.
     */
    public function get_subkeytext()
    {
        return $this->get_default_property(self :: PROPERTY_SUBKEYTEXT);
    }

    /**
     * Sets the subkeytext of this Dokeos185CourseSetting.
     * @param subkeytext
     */
    public function set_subkeytext($subkeytext)
    {
        $this->set_default_property(self :: PROPERTY_SUBKEYTEXT, $subkeytext);
    }

    /**
     * Check if the course setting is valid
     * @param array $array the parameters for the validation
     * @return true if the course setting is valid
     */
    public function is_valid()
    {

        if (! $this->get_id() || ! ($this->get_variable() || $this->get_category()))
        {
            $this->create_failed_element($this->get_id());
            $this->set_message(Translation :: get('GeneralInvalidMessage', array('TYPE' => 'course_setting',
                    'ID' => $this->get_id())));

            return false;
        }
        return true;
    }

    /**
     * Convert to new course setting
     * @param array $array the parameters for the conversion
     * @return the new course setting
     * @todo implementation
     */
    public function convert_data()
    {
        $chamilo_setting = new Setting();
        $chamilo_setting->set_variable($this->get_variable());
        $chamilo_setting->set_value($this->get_value());
        $chamilo_setting->set_application($this->get_category());

     //
    //$chamilo_setting->create();
    }

    public static function get_table_name()
    {
        return Utilities :: camelcase_to_underscores(substr(Utilities :: get_classname_from_namespace(__CLASS__), 9));
    }

    public static function get_class_name()
    {
        return self :: CLASS_NAME;
    }

}
