<?php
namespace core\migration;

use repository\content_object\document\Document;
use repository\ContentObject;
use repository\content_object\file\File;
use common\libraries\Text;
use common\libraries\Translation;
use repository\RepositoryDataManager;
use common\libraries\Utilities;
use common\libraries\Path;

/**
 * $Id: dokeos185_forum_forum.class.php 221 2009-11-13 14:36:41Z vanpouckesven $
 *
 * @package migration.lib.platform.dokeos185
 */
require_once dirname(__FILE__) . '/../dokeos185_course_data_migration_data_class.class.php';

/**
 * This class presents a Dokeos185 forum_forum
 *
 * @author Sven Vanpoucke
 */
class Dokeos185ForumAttachment extends Dokeos185CourseDataMigrationDataClass
{
    const CLASS_NAME = __CLASS__;
    const TABLE_NAME = 'forum_attachment';

    /**
     * Dokeos185ForumForum properties
     */
    const PROPERTY_ID = 'id';
    const PROPERTY_PATH = 'path';
    const PROPERTY_COMMENT = 'comment';
    const PROPERTY_SIZE = 'size';
    const PROPERTY_POST_ID = 'post_id';
    const PROPERTY_FILENAME = 'filename';

    /**
     * Get the default properties
     *
     * @return array The property names.
     */
    public static function get_default_property_names()
    {
        return array(self :: PROPERTY_ID, self :: PROPERTY_PATH, self :: PROPERTY_COMMENT, self :: PROPERTY_SIZE,
                self :: PROPERTY_POST_ID, self :: PROPERTY_FILENAME);
    }

    /**
     * Returns the id of this Dokeos185ForumForum.
     *
     * @return the id.
     */
    public function get_id()
    {
        return $this->get_default_property(self :: PROPERTY_ID);
    }

    /**
     * Returns the path of this Dokeos185ForumForum.
     *
     * @return the path.
     */
    public function get_path()
    {
        return $this->get_default_property(self :: PROPERTY_PATH);
    }

    /**
     * Returns the comment of this Dokeos185ForumForum.
     *
     * @return the comment.
     */
    public function get_comment()
    {
        return $this->get_default_property(self :: PROPERTY_COMMENT);
    }

    /**
     * Returns the size of this Dokeos185ForumForum.
     *
     * @return the size.
     */
    public function get_size()
    {
        return $this->get_default_property(self :: PROPERTY_SIZE);
    }

    /**
     * Returns the post_id of this Dokeos185ForumForum.
     *
     * @return the post_id.
     */
    public function get_post_id()
    {
        return $this->get_default_property(self :: PROPERTY_POST_ID);
    }

    /**
     * Returns the filename of this Dokeos185ForumForum.
     *
     * @return the filename.
     */
    public function get_filename()
    {
        return $this->get_default_property(self :: PROPERTY_FILENAME);
    }

    /**
     * Check if the forum attachment is valid
     *
     * @return true if the forum is valid
     */
    public function is_valid()
    {
        $this->set_item_property(
                $this->get_data_manager()->get_item_property($this->get_course(), 'forum_attachment', $this->get_id()));

        $course = $this->get_course();
        $path = $this->get_data_manager()->get_sys_path() . '/courses/' . $course->get_directory() . '/upload/forum/' .
                 $this->get_path();

        if (! $this->get_post_id() || ! $this->get_filename() || ! $this->get_path() || ! file_exists($path) ||
                 ! $this->item_property)
        {
            $this->create_failed_element($this->get_id());
            $this->set_message(
                    Translation :: get('GeneralInvalidMessage',
                            array('TYPE' => 'forum_attachment', 'ID' => $this->get_id())));
            return false;
        }
        return true;
    }

    /**
     * Convert to new forum
     *
     * @param array $array the parameters for the conversion
     * @return the new forum
     */
    public function convert_data()
    {
        $course = $this->get_course();
        $path = $this->get_data_manager()->get_sys_path() . '/courses/' . $course->get_directory() . '/upload/forum/';

        $post_id = $this->get_id_reference($this->get_post_id(), $this->get_database_name() . '.forum_post');
        $post = \repository\DataManager :: retrieve_content_object($post_id, ForumPost :: get_type_name());

        $hash = md5($this->get_filename());
        $new_path = Path :: get(SYS_REPO_PATH) . $post->get_owner_id() . '/' . Text :: char_at($hash, 0) . '/';
        $file_exists = file_exists($new_path . $hash);

        // $converted_path = iconv('UTF-8', 'ISO-8859-1', $path);
        // $converted_filename = iconv('UTF-8', 'ISO-8859-1', $this->get_path());
        $converted_path = $path;
        $converted_filename = $this->get_path();

        $migrated_hash = $this->migrate_file($path, $new_path, $converted_filename, $hash);

        if ($file_exists && $hash == $migrated_hash)
        {
            $document = RepositoryDataManager :: retrieve_file_from_hash($post->get_owner_id(), $migrated_hash);
        }
        else
        {
            if (ContentObject :: is_available('repository\content_object\document\Document'))
            {
                $document = new Document();
            }
            else
            {
                $document = new File();
            }
            $document->set_filename($this->get_filename());
            $document->set_path($post->get_owner_id() . '/' . Text :: char_at($migrated_hash, 0) . '/' . $migrated_hash);
            $document->set_filesize($this->get_size());
            $document->set_hash($migrated_hash);
            $document->set_title($this->get_filename());
            $document->set_description($this->get_filename());
            $document->set_owner_id($post->get_owner_id());
            $document->set_creation_date(strtotime($this->get_item_property()->get_insert_date()));
            $document->set_modification_date(strtotime($this->get_item_property()->get_lastedit_date()));

            $repository_category_course_id = $this->get_repository_migration_course_category_id($new_user_id);
            $chamilo_category_id = RepositoryDataManager :: get_repository_category_by_name_or_create_new(
                    $post->get_owner_id(), Translation :: get('Documents'), $repository_category_course_id);
            $document->set_parent_id($chamilo_category_id);

            if ($this->get_item_property()->get_visibility() == 2)
            {
                $document->set_state(1);
            }

            $document->create();
        }

        if ($document)
        {
            $post->attach_content_object($document->get_id());
        }

        // Add id references to temp table
        $this->create_id_reference($this->get_id(), $document->get_id());
        $this->set_message(
                Translation :: get('GeneralConvertedMessage',
                        array('TYPE' => 'forum_attachment', 'OLD_ID' => $this->get_id(),
                                'NEW_ID' => $document->get_id())));
    }

    public static function get_table_name()
    {
        return Utilities :: camelcase_to_underscores(substr(Utilities :: get_classname_from_namespace(__CLASS__), 9));
    }

    public static function get_class_name()
    {
        return self :: CLASS_NAME;
    }
}
