<?php
namespace core\migration;
use Exception;
/**
 * $Id: dokeos185_group_rel_tutor.class.php 221 2009-11-13 14:36:41Z vanpouckesven $
 * @package migration.platform.dokeos185
 */

/**
 * This class represents an old Dokeos 1.8.5 Group Tutor Relation
 *
 * @author David Van Wayenbergh
 */

class Dokeos185GroupRelTutor extends Dokeos185MigrationDataClass
{
    private static $mgdm;
    
    /**
     * group tutor relation properties
     */
    const PROPERTY_ID = 'id';
    const PROPERTY_USER_ID = 'user_id';
    const PROPERTY_GROUP_ID = 'group_id';
    
    /**
     * Default properties of the group tutor relation object, stored in an associative
     * array.
     */
    private $defaultProperties;

    /**
     * Creates a new group tutor relation object.
     * @param array $defaultProperties The default properties of the group tutor relation
     * object. Associative array.
     */
    public function __construct($defaultProperties = array ())
    {
        $this->defaultProperties = $defaultProperties;
    }

    /**
     * Gets a default property of this group tutor relation object by name.
     * @param string $name The name of the property.
     */
    public function get_default_property($name)
    {
        return $this->defaultProperties[$name];
    }

    /**
     * Gets the default properties of this group tutor relation.
     * @return array An associative array containing the properties.
     */
    public function get_default_properties()
    {
        return $this->defaultProperties;
    }

    /**
     * Get the default properties of all link categories.
     * @return array The property names.
     */
    public static function get_default_property_names()
    {
        return array(self :: PROPERTY_ID, self :: PROPERTY_USER_ID, self :: PROPERTY_GROUP_ID);
    }

    /**
     * Sets a default property of this group tutor relation by name.
     * @param string $name The name of the property.
     * @param mixed $value The new value for the property.
     */
    public function set_default_property($name, $value)
    {
        $this->defaultProperties[$name] = $value;
    }

    /**
     * Sets the default properties of this link.
     * @param array $defaultProperties An associative array containing the properties.
     */
    public function set_default_properties($defaultProperties)
    {
        return $this->defaultProperties = $defaultProperties;
    }

    /**
     * Returns the id of this group tutor relation.
     * @return int The id.
     */
    public function get_id()
    {
        return $this->get_default_property(self :: PROPERTY_ID);
    }

    /**
     * Returns the user_id of this group tutor relation.
     * @return String The user_id.
     */
    public function get_user_id()
    {
        return $this->get_default_property(self :: PROPERTY_USER_ID);
    }

    /**
     * Returns the group_id of this group tutor relation.
     * @return String The group_id.
     */
    public function get_group_id()
    {
        return $this->get_default_property(self :: PROPERTY_GROUP_ID);
    }

    /**
     * Check if the group tutor relation is valid
     * @param Course $course the course
     * @return true if the group tutor relation is valid
     */
    public function is_valid_group_rel_tutor($array)
    {
        $course = $array['course'];
    }

    /**
     * Convert to new group tutor relation
     * @param Course $course the course
     * @return the new group tutor relation
     */
    public function convert_data()
    {
        $course = $array['course'];
    }

    /**
     * Retrieve all group tutor relations from the database
     * @param array $parameters parameters for the retrieval
     * @return array of group tutor relations
     */
    public static function retrieve_data($parameters)
    {
        self :: $mgdm = $parameters['mgdm'];
        
        $coursedb = $parameters['course']->get_db_name();
        $tablename = 'group_rel_tutor';
        $classname = 'Dokeos185GroupRelTutor';
        
        return self :: $mgdm->get_all($coursedb, $tablename, $classname, $tool_name, $parameters['offset'], $parameters['limit']);
    }

    public static function get_database_table($parameters)
    {
        $array = array();
        $array['database'] = $parameters['course']->get_db_name();
        $array['table'] = 'group_rel_tutor';
        return $array;
    }

    public function is_valid()
    {
        throw new Exception("Unimplemented method " . __FILE__ . "#" . __METHOD__ . "(" . __LINE__ . ")");
    }

    public static function get_table_name()
    {
        throw new Exception("Unimplemented method " . __FILE__ . "#" . __METHOD__ . "(" . __LINE__ . ")");
    }

    public static function get_class_name()
    {
        throw new Exception("Unimplemented method " . __FILE__ . "#" . __METHOD__ . "(" . __LINE__ . ")");
    }

    public function get_database_name()
    {
        throw new Exception("Unimplemented method " . __FILE__ . "#" . __METHOD__ . "(" . __LINE__ . ")");
    }

}
