<?php
namespace core\migration;
/**
 * $Id: dokeos185_php_session.class.php 221 2009-11-13 14:36:41Z vanpouckesven $
 * @package migration.lib.platform.dokeos185
 */
use Exception;
/**
 * This class presents a Dokeos185 php_session
 *
 * @author Sven Vanpoucke
 */
class Dokeos185PhpSession extends Dokeos185MigrationDataClass
{
    private static $mgdm;
    
    /**
     * Dokeos185PhpSession properties
     */
    const PROPERTY_SESSION_ID = 'session_id';
    const PROPERTY_SESSION_NAME = 'session_name';
    const PROPERTY_SESSION_TIME = 'session_time';
    const PROPERTY_SESSION_START = 'session_start';
    const PROPERTY_SESSION_VALUE = 'session_value';
    
    /**
     * Default properties stored in an associative array.
     */
    private $defaultProperties;

    /**
     * Creates a new Dokeos185PhpSession object
     * @param array $defaultProperties The default properties
     */
    public function __construct($defaultProperties = array ())
    {
        $this->defaultProperties = $defaultProperties;
    }

    /**
     * Gets a default property by name.
     * @param string $name The name of the property.
     */
    public function get_default_property($name)
    {
        return $this->defaultProperties[$name];
    }

    /**
     * Gets the default properties
     * @return array An associative array containing the properties.
     */
    public function get_default_properties()
    {
        return $this->defaultProperties;
    }

    /**
     * Get the default properties
     * @return array The property names.
     */
    public static function get_default_property_names()
    {
        return array(self :: PROPERTY_SESSION_ID, self :: PROPERTY_SESSION_NAME, self :: PROPERTY_SESSION_TIME, 
                self :: PROPERTY_SESSION_START, self :: PROPERTY_SESSION_VALUE);
    }

    /**
     * Sets a default property by name.
     * @param string $name The name of the property.
     * @param mixed $value The new value for the property.
     */
    public function set_default_property($name, $value)
    {
        $this->defaultProperties[$name] = $value;
    }

    /**
     * Sets the default properties of this class
     */
    public function set_default_properties($defaultProperties)
    {
        $this->defaultProperties = $defaultProperties;
    }

    /**
     * Returns the session_id of this Dokeos185PhpSession.
     * @return the session_id.
     */
    public function get_session_id()
    {
        return $this->get_default_property(self :: PROPERTY_SESSION_ID);
    }

    /**
     * Returns the session_name of this Dokeos185PhpSession.
     * @return the session_name.
     */
    public function get_session_name()
    {
        return $this->get_default_property(self :: PROPERTY_SESSION_NAME);
    }

    /**
     * Returns the session_time of this Dokeos185PhpSession.
     * @return the session_time.
     */
    public function get_session_time()
    {
        return $this->get_default_property(self :: PROPERTY_SESSION_TIME);
    }

    /**
     * Returns the session_start of this Dokeos185PhpSession.
     * @return the session_start.
     */
    public function get_session_start()
    {
        return $this->get_default_property(self :: PROPERTY_SESSION_START);
    }

    /**
     * Returns the session_value of this Dokeos185PhpSession.
     * @return the session_value.
     */
    public function get_session_value()
    {
        return $this->get_default_property(self :: PROPERTY_SESSION_VALUE);
    }

    /**
     * Check if the php session is valid
     * @param array $array the parameters for the validation
     * @return true if the php session is valid
     */
    public function is_valid()
    {
        throw new Exception("Unimplemented method " . __FILE__ . "#" . __METHOD__ . "(" . __LINE__ . ")");
    }

    public static function get_table_name()
    {
        throw new Exception("Unimplemented method " . __FILE__ . "#" . __METHOD__ . "(" . __LINE__ . ")");
    }

    public static function get_class_name()
    {
        throw new Exception("Unimplemented method " . __FILE__ . "#" . __METHOD__ . "(" . __LINE__ . ")");
    }

    public function get_database_name()
    {
        throw new Exception("Unimplemented method " . __FILE__ . "#" . __METHOD__ . "(" . __LINE__ . ")");
    }

    /**
     * Convert to new php session
     * @param array $array the parameters for the conversion
     * @return the new php session
     */
    public function convert_data()
    {
    
    }

    /**
     * Retrieve all php sessions from the database
     * @param array $parameters parameters for the retrieval
     * @return array of php sessions
     */
    public static function retrieve_data($parameters)
    {
        $old_mgdm = $parameters['mgdm'];
        
        $db = 'main_database';
        $tablename = 'php_session';
        $classname = 'Dokeos185PhpSession';
        
        return $old_mgdm->get_all($db, $tablename, $classname, $tool_name, $parameters['offset'], $parameters['limit']);
    }

    public static function get_database_table($parameters)
    {
        $array = array();
        $array['database'] = 'main_database';
        $array['table'] = 'php_session';
        return $array;
    }
}
