<?php
namespace core\migration;
/**
 * $Id: dokeos185_session_rel_course.class.php 221 2009-11-13 14:36:41Z vanpouckesven $
 * @package migration.lib.platform.dokeos185
 */
use Exception;
/**
 * This class presents a Dokeos185 session_rel_course
 *
 * @author Sven Vanpoucke
 */
class Dokeos185SessionRelCourse extends Dokeos185MigrationDataClass
{
    private static $mgdm;
    
    /**
     * Dokeos185SessionRelCourse properties
     */
    const PROPERTY_ID_SESSION = 'id_session';
    const PROPERTY_COURSE_CODE = 'course_code';
    const PROPERTY_ID_COACH = 'id_coach';
    const PROPERTY_NBR_USERS = 'nbr_users';
    
    /**
     * Default properties stored in an associative array.
     */
    private $defaultProperties;

    /**
     * Creates a new Dokeos185SessionRelCourse object
     * @param array $defaultProperties The default properties
     */
    public function __construct($defaultProperties = array ())
    {
        $this->defaultProperties = $defaultProperties;
    }

    /**
     * Gets a default property by name.
     * @param string $name The name of the property.
     */
    public function get_default_property($name)
    {
        return $this->defaultProperties[$name];
    }

    /**
     * Gets the default properties
     * @return array An associative array containing the properties.
     */
    public function get_default_properties()
    {
        return $this->defaultProperties;
    }

    /**
     * Get the default properties
     * @return array The property names.
     */
    public static function get_default_property_names()
    {
        return array(self :: PROPERTY_ID_SESSION, self :: PROPERTY_COURSE_CODE, self :: PROPERTY_ID_COACH, 
                self :: PROPERTY_NBR_USERS);
    }

    /**
     * Sets a default property by name.
     * @param string $name The name of the property.
     * @param mixed $value The new value for the property.
     */
    public function set_default_property($name, $value)
    {
        $this->defaultProperties[$name] = $value;
    }

    /**
     * Sets the default properties of this class
     */
    public function set_default_properties($defaultProperties)
    {
        $this->defaultProperties = $defaultProperties;
    }

    /**
     * Returns the id_session of this Dokeos185SessionRelCourse.
     * @return the id_session.
     */
    public function get_id_session()
    {
        return $this->get_default_property(self :: PROPERTY_ID_SESSION);
    }

    /**
     * Returns the course_code of this Dokeos185SessionRelCourse.
     * @return the course_code.
     */
    public function get_course_code()
    {
        return $this->get_default_property(self :: PROPERTY_COURSE_CODE);
    }

    /**
     * Returns the id_coach of this Dokeos185SessionRelCourse.
     * @return the id_coach.
     */
    public function get_id_coach()
    {
        return $this->get_default_property(self :: PROPERTY_ID_COACH);
    }

    /**
     * Returns the nbr_users of this Dokeos185SessionRelCourse.
     * @return the nbr_users.
     */
    public function get_nbr_users()
    {
        return $this->get_default_property(self :: PROPERTY_NBR_USERS);
    }

    /**
     * Checks if a sessionrelcourse is valid
     * @param Array $array
     * @return Boolean
     */
    public function is_valid()
    {
        throw new Exception("Unimplemented method " . __FILE__ . "#" . __METHOD__ . "(" . __LINE__ . ")");
    }

    public static function get_table_name()
    {
        throw new Exception("Unimplemented method " . __FILE__ . "#" . __METHOD__ . "(" . __LINE__ . ")");
    }

    public static function get_class_name()
    {
        throw new Exception("Unimplemented method " . __FILE__ . "#" . __METHOD__ . "(" . __LINE__ . ")");
    }

    public function get_database_name()
    {
        throw new Exception("Unimplemented method " . __FILE__ . "#" . __METHOD__ . "(" . __LINE__ . ")");
    }

    /**
     * migrate sessionrelcourse, sets category
     * @param Array $array
     * @return
     */
    public function convert_data()
    {
    
    }

    /**
     * Gets all the sessionrelcourse of a course
     * @param Array $array
     * @return Array of dokeos185sessionrelcourse
     */
    public static function retrieve_data($parameters)
    {
        self :: $mgdm = $parameters['mgdm'];
        
        $db = 'main_database';
        $tablename = 'session_rel_course';
        $classname = 'Dokeos185SessionRelCourse';
        
        return self :: $mgdm->get_all($db, $tablename, $classname, $tool_name, $parameters['offset'], $parameters['limit']);
    }

    public static function get_database_table($parameters)
    {
        $array = array();
        $array['database'] = 'main_database';
        $array['table'] = 'session_rel_course';
        return $array;
    }
}
