<?php
namespace core\migration;
/**
 * $Id: dokeos185_shared_survey.class.php 221 2009-11-13 14:36:41Z vanpouckesven $
 * @package migration.lib.platform.dokeos185
 */
use Exception;

/**
 * This class presents a Dokeos185 shared_survey
 *
 * @author Sven Vanpoucke
 */
class Dokeos185SharedSurvey extends Dokeos185MigrationDataClass
{
    private static $mgdm;
    
    /**
     * Dokeos185SharedSurvey properties
     */
    const PROPERTY_SURVEY_ID = 'survey_id';
    const PROPERTY_CODE = 'code';
    const PROPERTY_TITLE = 'title';
    const PROPERTY_SUBTITLE = 'subtitle';
    const PROPERTY_AUTHOR = 'author';
    const PROPERTY_LANG = 'lang';
    const PROPERTY_TEMPLATE = 'template';
    const PROPERTY_INTRO = 'intro';
    const PROPERTY_SURVEYTHANKS = 'surveythanks';
    const PROPERTY_CREATION_DATE = 'creation_date';
    const PROPERTY_COURSE_CODE = 'course_code';
    
    /**
     * Default properties stored in an associative array.
     */
    private $defaultProperties;

    /**
     * Creates a new Dokeos185SharedSurvey object
     * @param array $defaultProperties The default properties
     */
    public function __construct($defaultProperties = array ())
    {
        $this->defaultProperties = $defaultProperties;
    }

    /**
     * Gets a default property by name.
     * @param string $name The name of the property.
     */
    public function get_default_property($name)
    {
        return $this->defaultProperties[$name];
    }

    /**
     * Gets the default properties
     * @return array An associative array containing the properties.
     */
    public function get_default_properties()
    {
        return $this->defaultProperties;
    }

    /**
     * Get the default properties
     * @return array The property names.
     */
    public static function get_default_property_names()
    {
        return array(self :: PROPERTY_SURVEY_ID, self :: PROPERTY_CODE, self :: PROPERTY_TITLE, 
                self :: PROPERTY_SUBTITLE, self :: PROPERTY_AUTHOR, self :: PROPERTY_LANG, self :: PROPERTY_TEMPLATE, 
                self :: PROPERTY_INTRO, self :: PROPERTY_SURVEYTHANKS, self :: PROPERTY_CREATION_DATE, 
                self :: PROPERTY_COURSE_CODE);
    }

    /**
     * Sets a default property by name.
     * @param string $name The name of the property.
     * @param mixed $value The new value for the property.
     */
    public function set_default_property($name, $value)
    {
        $this->defaultProperties[$name] = $value;
    }

    /**
     * Sets the default properties of this class
     */
    public function set_default_properties($defaultProperties)
    {
        $this->defaultProperties = $defaultProperties;
    }

    /**
     * Returns the survey_id of this Dokeos185SharedSurvey.
     * @return the survey_id.
     */
    public function get_survey_id()
    {
        return $this->get_default_property(self :: PROPERTY_SURVEY_ID);
    }

    /**
     * Returns the code of this Dokeos185SharedSurvey.
     * @return the code.
     */
    public function get_code()
    {
        return $this->get_default_property(self :: PROPERTY_CODE);
    }

    /**
     * Returns the title of this Dokeos185SharedSurvey.
     * @return the title.
     */
    public function get_title()
    {
        return $this->get_default_property(self :: PROPERTY_TITLE);
    }

    /**
     * Returns the subtitle of this Dokeos185SharedSurvey.
     * @return the subtitle.
     */
    public function get_subtitle()
    {
        return $this->get_default_property(self :: PROPERTY_SUBTITLE);
    }

    /**
     * Returns the author of this Dokeos185SharedSurvey.
     * @return the author.
     */
    public function get_author()
    {
        return $this->get_default_property(self :: PROPERTY_AUTHOR);
    }

    /**
     * Returns the lang of this Dokeos185SharedSurvey.
     * @return the lang.
     */
    public function get_lang()
    {
        return $this->get_default_property(self :: PROPERTY_LANG);
    }

    /**
     * Returns the template of this Dokeos185SharedSurvey.
     * @return the template.
     */
    public function get_template()
    {
        return $this->get_default_property(self :: PROPERTY_TEMPLATE);
    }

    /**
     * Returns the intro of this Dokeos185SharedSurvey.
     * @return the intro.
     */
    public function get_intro()
    {
        return $this->get_default_property(self :: PROPERTY_INTRO);
    }

    /**
     * Returns the surveythanks of this Dokeos185SharedSurvey.
     * @return the surveythanks.
     */
    public function get_surveythanks()
    {
        return $this->get_default_property(self :: PROPERTY_SURVEYTHANKS);
    }

    /**
     * Returns the creation_date of this Dokeos185SharedSurvey.
     * @return the creation_date.
     */
    public function get_creation_date()
    {
        return $this->get_default_property(self :: PROPERTY_CREATION_DATE);
    }

    /**
     * Returns the course_code of this Dokeos185SharedSurvey.
     * @return the course_code.
     */
    public function get_course_code()
    {
        return $this->get_default_property(self :: PROPERTY_COURSE_CODE);
    }

    /**
     * Checks if a shared survey is valid
     * @param Array $array
     * @return Boolean
     */
    public function is_valid()
    {
        throw new Exception("Unimplemented method " . __FILE__ . "#" . __METHOD__ . "(" . __LINE__ . ")");
    }

    public static function get_table_name()
    {
        throw new Exception("Unimplemented method " . __FILE__ . "#" . __METHOD__ . "(" . __LINE__ . ")");
    }

    public static function get_class_name()
    {
        throw new Exception("Unimplemented method " . __FILE__ . "#" . __METHOD__ . "(" . __LINE__ . ")");
    }

    public function get_database_name()
    {
        throw new Exception("Unimplemented method " . __FILE__ . "#" . __METHOD__ . "(" . __LINE__ . ")");
    }

    /**
     * migrate shared survey, sets category
     * @param Array $array
     * @return
     */
    public function convert_data()
    {
    
    }

    /**
     * Gets all the shared surveys of a course
     * @param Array $array
     * @return
     */
    public static function retrieve_data($parameters)
    {
        self :: $mgdm = $parameters['mgdm'];
        
        $db = 'main_database';
        $tablename = 'shared_survey';
        $classname = 'Dokeos185SharedSurvey';
        
        return self :: $mgdm->get_all($db, $tablename, $classname, $tool_name, $parameters['offset'], $parameters['limit']);
    }

    public static function get_database_table($parameters)
    {
        $array = array();
        $array['database'] = 'main_database';
        $array['table'] = 'shared_survey';
        return $array;
    }
}
