<?php
namespace core\migration;
/**
 * $Id: dokeos185_survey_question_option.class.php 221 2009-11-13 14:36:41Z vanpouckesven $
 * @package migration.lib.platform.dokeos185
 */

/**
 * This class presents a Dokeos185 survey_question_option
 *
 * @author Sven Vanpoucke
 */
use common\libraries\Utilities;
use repository\content_object\survey_select_question\SurveySelectQuestionOption;
use repository\content_object\survey_select_question\SurveySelectQuestion;
use repository\content_object\survey_multiple_choice_question\SurveyMultipleChoiceQuestionOption;
use repository\content_object\survey_multiple_choice_question\SurveyMultipleChoiceQuestion;
use common\libraries\Translation;

class Dokeos185SurveyQuestionOption extends Dokeos185CourseDataMigrationDataClass
{
    const CLASS_NAME = __CLASS__;
    const TABLE_NAME = 'survey_question_option';
    
    /**
     * Dokeos185SurveyQuestionOption properties
     */
    const PROPERTY_QUESTION_OPTION_ID = 'question_option_id';
    const PROPERTY_QUESTION_ID = 'question_id';
    const PROPERTY_SURVEY_ID = 'survey_id';
    const PROPERTY_OPTION_TEXT = 'option_text';
    const PROPERTY_SORT = 'sort';
    
    /**
     * Default properties stored in an associative array.
     */
    private $defaultProperties;

    /**
     * Creates a new Dokeos185SurveyQuestionOption object
     * @param array $defaultProperties The default properties
     */
    public function __construct($defaultProperties = array ())
    {
        $this->defaultProperties = $defaultProperties;
    }

    /**
     * Gets a default property by name.
     * @param string $name The name of the property.
     */
    public function get_default_property($name)
    {
        return $this->defaultProperties[$name];
    }

    /**
     * Gets the default properties
     * @return array An associative array containing the properties.
     */
    public function get_default_properties()
    {
        return $this->defaultProperties;
    }

    /**
     * Get the default properties
     * @return array The property names.
     */
    public static function get_default_property_names()
    {
        return array(self :: PROPERTY_QUESTION_OPTION_ID, self :: PROPERTY_QUESTION_ID, self :: PROPERTY_SURVEY_ID, 
                self :: PROPERTY_OPTION_TEXT, self :: PROPERTY_SORT);
    }

    /**
     * Sets a default property by name.
     * @param string $name The name of the property.
     * @param mixed $value The new value for the property.
     */
    public function set_default_property($name, $value)
    {
        $this->defaultProperties[$name] = $value;
    }

    /**
     * Sets the default properties of this class
     */
    public function set_default_properties($defaultProperties)
    {
        $this->defaultProperties = $defaultProperties;
    }

    /**
     * Returns the question_option_id of this Dokeos185SurveyQuestionOption.
     * @return the question_option_id.
     */
    public function get_question_option_id()
    {
        return $this->get_default_property(self :: PROPERTY_QUESTION_OPTION_ID);
    }

    /**
     * Returns the question_id of this Dokeos185SurveyQuestionOption.
     * @return the question_id.
     */
    public function get_question_id()
    {
        return $this->get_default_property(self :: PROPERTY_QUESTION_ID);
    }

    /**
     * Returns the survey_id of this Dokeos185SurveyQuestionOption.
     * @return the survey_id.
     */
    public function get_survey_id()
    {
        return $this->get_default_property(self :: PROPERTY_SURVEY_ID);
    }

    /**
     * Returns the option_text of this Dokeos185SurveyQuestionOption.
     * @return the option_text.
     */
    public function get_option_text()
    {
        return $this->get_default_property(self :: PROPERTY_OPTION_TEXT);
    }

    /**
     * Returns the sort of this Dokeos185SurveyQuestionOption.
     * @return the sort.
     */
    public function get_sort()
    {
        return $this->get_default_property(self :: PROPERTY_SORT);
    }

    /**
     * Checks if surveyquestionoption is valid
     * @param Array $array
     * @return Boolean
     */
    public function is_valid()
    {
        
        if (! $this->get_option_text() || ! $this->get_id_reference($this->get_question_id(), $this->get_database_name() . '.survey_question'))
        {
            $this->create_failed_element($this->get_id());
            $this->set_message(Translation :: get('GeneralInvalidMessage', array('TYPE' => 'survey_question_option', 
                    'ID' => $this->get_question_id())));
            return false;
        }
        return true;
    }

    /**
     * Convert to LearningStyleSurveyAnswer, sets category
     * @param Array $array
     * @return LearningStyleSurveyAnswer
     */
    public function convert_data()
    {
        //retrieve the refered survey question
        $survey_question_id = $this->get_id_reference($this->get_question_id(), $this->get_database_name() . '.survey_question');
        $survey_question = \repository\DataManager :: retrieve_content_object($survey_question_id);
        
        //only multiple choice and select have non standard options. These need to be added
        switch (get_class($survey_question))
        {
            case SurveyMultipleChoiceQuestion :: CLASS_NAME :
                $option = new SurveyMultipleChoiceQuestionOption($this->get_option_text());
                $survey_question->add_option($option);
                $survey_question->update();
                break;
            case SurveySelectQuestion :: CLASS_NAME :
                $option = new SurveySelectQuestionOption($this->get_option_text());
                $survey_question->add_option($option);
                $survey_question->update();
                break;
        }
        
        return $survey_question;
    }

    public static function get_class_name()
    {
        return self :: CLASS_NAME;
    }

    public static function get_table_name()
    {
        return Utilities :: camelcase_to_underscores(substr(Utilities :: get_classname_from_namespace(__CLASS__), 9));
    }
}
