<?php
namespace core\migration;

use common\libraries\Translation;
use common\libraries\Utilities;

use Exception;

/**
 * Generic parser framework to parse several included objects from a given dokeos 185 field
 * @author svenvanpoucke
 *
 */
class Dokeos185TextFieldParser
{
        const TYPE_FLASH_HTM = 'flash_htm';
        const TYPE_EMBEDDED_MEDIA = 'embedded_media';
        const TYPE_IMAGE = 'image';

	/**
	 * List with included objects so we don't need to parse the text field again for included objects
	 * @var int[]
	 */
	private $included_objects;

	public function __construct()
	{
		$this->included_objects = array();
	}

	public function factory($type = self :: TYPE_IMAGE)
	{
		$location = dirname(__FILE__) . '/text_field_parser/dokeos185_' . $type . '_text_field_parser.class.php';
		if(!file_exists($location))
		{
                    throw new Exception(Translation :: get('CanNotLoadDokeos185TextFieldParser', array('TYPE' => $type)));
		}

		require_once($location);

		$class = __NAMESPACE__ . '\\' . 'Dokeos185' . Utilities :: underscores_to_camelcase($type) . 'TextFieldParser';
		return new $class();
	}

	public function set_included_objects($included_objects)
	{
		$this->included_objects = $included_objects;
	}

	public function get_included_objects()
	{
		return $this->included_objects;
	}

	public function add_included_object(ContentObject $object)
	{
		$this->included_objects[] = $object;
	}

}
