<?php

namespace core\migration;

use application\weblcms\WeblcmsManager;
use common\libraries\EqualityCondition;
use rights\RightsLocation;
use rights\DataManager as RightsDataManager;
use common\libraries\Translation;
use common\libraries\MessageLogger;
use repository\RepositoryManager;

/**
 * Post processing of the locations in chamilo because we create the locations without left and right values for
 * performance reasons
 */
class LocationsPostProcess extends MigrationPostProcess
{
    const NAME = 'locations';
    
    private $index;
    
    private $context;
    
    private $counter_succes = 0;
    private $counter_failed = 0;

    // DMTODO: the retrieve_rights_locations can propbably be swapped out for generic retrieves
    // provided that the RightsLocations classes of the WebLCMS en Repository app are used.
    public function run()
    {
        $condition = new EqualityCondition(RightsLocation::PROPERTY_PARENT_ID, 0);
        
        //post process the weblcms locations
        $this->context = WeblcmsManager :: APPLICATION_NAME;
        $roots = RightsDataManager :: retrieve_rights_locations($this->context, $condition);
        $this->update_roots($roots->as_array());                
        
        //post process the repo locations
        $this->context = RepositoryManager :: APPLICATION_NAME;
        $roots = RightsDataManager :: retrieve_rights_locations($this->context, $condition);
        $this->update_roots($roots->as_array()); 
        
        $this->log_message_and_file(Translation :: get('LocationsFixed', array('COUNTER' => $this->counter_succes)),
		MessageLogger :: TYPE_NORMAL);
        $this->log_message_and_file(Translation :: get('LocationsFailed', array('COUNTER' => $this->counter_failed)),
		MessageLogger :: TYPE_NORMAL);
    }
    
    /**
     * Sets the left and right values of the supplied roots
     * @param array $roots 
     */
    public function update_roots(array $roots)
    {
        
        foreach($roots as $root)
        {
            $this->index = 1;
            if(!$this->update_children_recursive($root))
            {
                $this->get_file_logger()->log_message(Translation :: get('LocationTreeUpdateFailed', array('ID' => $root->get_id())),
		MessageLogger :: TYPE_ERROR);
            }
            else
            {
                $this->get_file_logger()->log_message(Translation :: get('LocationTreeUpdated', array('ID' => $root->get_id())),
		MessageLogger :: TYPE_NORMAL);
            }
        }
        
    }
    
    /**
     * Sets the right and left values of the children
     * @param RightsLocation $parent 
     */
    public function update_children_recursive(RightsLocation $parent)
    {
        $succes = true;
        
        $parent->set_left_value($this->index);
        $this->index ++ ;
        
        // DMTODO: this is useful functionality whenever creating nested set nodes in batch
        // It could be extrapolated to a static method on nestedSet rather than be implemented
        // for this specific case alone.
        $condition = new EqualityCondition(RightsLocation :: PROPERTY_PARENT_ID, $parent->get_id());
        $children =  RightsDataManager :: retrieve_rights_locations($this->context, $condition);
        
        while($child = $children->next_result())
        {
            $succes = $this->update_children_recursive($child);
        }
        
        $parent->set_context($this->context);
        $parent->set_right_value($this->index);
        $this->index ++;
        
        if(!$parent->update())
        {
            $this->get_file_logger()->log_message(Translation :: get('LocationUpdateFailed', array('ID' => $parent->get_id())),
		MessageLogger :: TYPE_ERROR);
            $this->counter_failed ++;
            $succes = false;
        }
        else
        {
            $this->get_file_logger()->log_message(Translation :: get('LocationUpdated', array('ID' => $parent->get_id())),
		MessageLogger :: TYPE_NORMAL);
            $this->counter_succes ++;
            $succes = $succes && true;
        }
        
        return $succes;

    }

    public function get_name()
    {
	return self :: NAME;
    }
}
