<?php
namespace group\integration\tracking;

use common\libraries\EqualityCondition;

class Change extends \tracking\ChangesTracker
{
    const PROPERTY_TARGET_USER_ID = 'target_user_id';

    /**
     * Get the default properties of all aggregate trackers.
     *
     * @return array The property names.
     */
    public static function get_default_property_names()
    {
        return parent :: get_default_property_names(array(self :: PROPERTY_TARGET_USER_ID));
    }

    public function validate_parameters(array $parameters = array())
    {
        parent :: validate_parameters($parameters);

        if ($parameters[self :: PROPERTY_TARGET_USER_ID])
        {
            $this->set_target_user_id($parameters[self :: PROPERTY_TARGET_USER_ID]);
        }
        else
        {
            $this->set_target_user_id(0);
        }
    }

    public function empty_tracker($event)
    {
        $condition = new EqualityCondition('action', $event->get_name());
        return $this->remove($condition);
    }

    public function export($start_date, $end_date, $event)
    {
        $conditions = array();
        $conditions[] = new EqualityCondition('action', $event->get_name());
        return parent :: export($start_date, $end_date, $conditions);
    }

    public function is_summary_tracker()
    {
        return false;
    }

    /**
     *
     * @return the $user_id
     */
    public function get_target_user_id()
    {
        return $this->get_default_property(self :: PROPERTY_TARGET_USER_ID);
    }

    /**
     *
     * @param $user_id the $user_id to set
     */
    public function set_target_user_id($target_user_id)
    {
        $this->set_default_property(self :: PROPERTY_TARGET_USER_ID, $target_user_id);
    }

    public static function get_table_name()
    {
        return 'group_' . parent :: get_table_name();
    }
}
