<?php

namespace group;


use common\libraries\EqualityCondition;

use common\libraries\Mdb2Database;
use common\libraries\AndCondition;
use common\libraries\DataClassCountParameters;
use common\libraries\DataClassRetrievesParameters;
use common\libraries\PropertyConditionVariable;
use common\libraries\StaticConditionVariable;
use user\User;

/**
 * $Id: database_group_data_manager.class.php 232 2009-11-16 10:11:48Z vanpouckesven $
 * @package group.lib.datamanager
 */

/**
  ==============================================================================
 * This is a data manager that uses a database for storage. It was written
 * for MySQL, but should be compatible with most SQL flavors.
 *
 * @author Tim De Pauw
 * @author Bart Mollet
 * @author Sven Vanpoucke
 * @author Hans De Bisschop
  ==============================================================================
 */
class Mdb2DataManager extends Mdb2Database implements GroupDataManagerInterface
{
    public function initialize()
    {
        parent :: initialize();
        $this->set_prefix('group_');
    }

    /**
     * Counts a resultset of entire users
     */
    // DEAD/OK
    // There are no known callers in the public repository
    // Has been mapped to the new Join library and could be moved to a static variant in the DM if need be
    // $condition can also refer to parts of the user table, which makes it different from an ordinary retrieve from GroupRelUser
    // Should perhaps pass in one or more group_ids, so that the condition can be pushed inside the join.
    public function count_group_rel_user_users($condition = null)
    {
        if(! is_null($condition)) // Joining with the User table only makes sense when conditions are imposed
        {                         // that may use data from the User table.
            $join = new DataClassJoin(User :: get_table_name(), 
                    new EqualityCondition(
                            new PropertyConditionVariable(GroupRelUser :: class_name(), GroupRelUser :: PROPERTY_USER_ID), 
                            new PropertyConditionVariable(User :: class_name(),         User :: PROPERTY_ID)));
            $joins = new Joins(array($join));
        }

        $parameters = new DataClassCountParameters($condition, $joins);
        DataManager :: count(GroupRelUser :: class_name(), $parameters);
    }

    /**
     * Returns a resultset of entire users
     */
    // DEAD/OK
    // There are no known callers in the public repository
    // Uses the Join lib and could thus be moved to the DM
    // Order of arguments has been changed to conform to the one of DataClassRetrievesParameters
    public function retrieve_group_rel_user_users($condition = null, $count = null, $offset = null, $order_by = null)
    {
        $join = new DataClassJoin(GroupRelUser :: get_table_name(), 
                new EqualityCondition(
                        new PropertyConditionVariable(GroupRelUser :: class_name(), GroupRelUser :: PROPERTY_USER_ID), 
                        new PropertyConditionVariable(User :: class_name(),         User :: PROPERTY_ID)));
        $joins = new Joins(array($join));

        $parameters = new DataClassRetrievesParameters($condition, $count, $offset, $order_by, $joins);
        return \user\DataManager :: retrieves(User :: class_name(), $parameters);
    }

    // DEAD/OK
    // There are no known callers in the public repository
    // Uses the Join lib and could thus be moved to the DM
    public function retrieve_group_rel_user_by_official_code_and_code($official_code, $group_code)
    {
        $conditions = array();
        $conditions[] = new EqualityCondition(
                new PropertyConditionVariable(User :: class_name(), User :: PROPERTY_OFFICIAL_CODE), 
                new StaticConditionVariable($official_code));
        $conditions[] = new EqualityCondition(
                new PropertyConditionVariable(Group :: class_name(), Group :: PROPERTY_CODE), 
                new StaticConditionVariable($group_code));
        $condition = new AndCondition($conditions);
        
        $joins   = array();
        $joins[] = new DataClassJoin(Group :: class_name(), 
                   new EqualityCondition(
                        new PropertyConditionVariable(GroupRelUser :: class_name(), GroupRelUser :: PROPERTY_GROUP_ID), 
                        new PropertyConditionVariable(Group :: class_name(),        Group :: PROPERTY_ID)));
        $joins[] = new DataClassJoin(User :: class_name(), 
                   new EqualityCondition(
                        new PropertyConditionVariable(GroupRelUser :: class_name(), GroupRelUser :: PROPERTY_USER_ID), 
                        new PropertyConditionVariable(User :: class_name(),         User :: PROPERTY_ID)));
        $joins = new Joins(array($joins));

        $parameters = new DataClassRetrievesParameters($condition, null, null, array(), $joins);
        return self :: retrieves(GroupRelUser :: class_name(), $parameters);
    }

    // DEAD/OK
    // Could be made static and moved to the DataManager, no known callers though
    // What is the purpose of passing along a group_id?
    public function is_groupname_available($groupname, $group_id = null)
    {
        $condition = new EqualityCondition(Group :: PROPERTY_NAME, $groupname);
        
        if ($group_id)
        {
            $condition = new AndCondition(array(
                $condition, 
                new EqualityCondition(Group :: PROPERTY_ID, $group_id)));
        }
        
        return ! (static :: count(Group :: class_name(), new DataClassCountParameters($condition)) == 1);
    }

    // DEAD/OK:
    // There are no known callers of this method
    // Uses the Join lib and could thus be moved to the DM
    // Should perhaps pass in one or more group_ids, so that the condition can be pushed inside the join and so callers need not know how this query is constructed.
    public function retrieve_usable_groups($condition = null, $offset = null, $count = null, $order_property = null)
    {
        $joins   = array();
        $joins[] = new DataClassJoin(GroupUseGroup :: class_name(), 
                   new EqualityCondition(
                        new PropertyConditionVariable(GroupUseGroup :: class_name(), GroupUseGroup :: PROPERTY_USE_GROUP_ID), 
                        new PropertyConditionVariable(Group :: class_name(),         Group :: PROPERTY_ID)));
        $joins = new Joins(array($joins));

        $parameters = new DataClassRetrievesParameters($condition, $count, $offset, $order_by, $joins);
        return self :: retrieves(Group :: class_name(), $parameters); //DMTODO DISTINCT
    }

    // DEAD/OK
    // There are no known callers of this method
    // Uses the Join lib and could thus be moved to the DM
    // Shouldn't this have an additional condition variable?
    /**
     * Retrieves the groups a user is linked to
     * Use the condition to limit or exclude certain groups
     * @param <type> $user_id
     * @return Mdb2ResultSet
     */
    public function retrieve_direct_subscribed_groups($user_id)
    {
        
        $condition = new EqualityCondition(
                new PropertyConditionVariable(GroupRelUser :: class_name(), GroupRelUser :: PROPERTY_USER_ID),
                new StaticConditionVariable($user_id));
        
        $joins   = array();
        $joins[] = new DataClassJoin(GroupRelUser :: class_name(), 
                   new EqualityCondition(
                        new PropertyConditionVariable(GroupRelUser :: class_name(), GroupRelUser :: PROPERTY_GROUP_ID), 
                        new PropertyConditionVariable(Group :: class_name(),        Group :: PROPERTY_ID)));
        $joins = new Joins(array($joins));

        $parameters = new DataClassRetrievesParameters($condition, null, null, null, $joins);
        return self :: retrieves(Group :: class_name(), $parameters);
    }
    
    private $direct_subscribed_groups_cache;
    private $only_retrieve_ids;

    /**
     * Retrieves the groups a user is linked to
     * Use the condition to limit or exclude certain groups
     * @param <type> $user_id
     * @param <type> $condition
     * @param <type> $only_retrieve_ids
     * @return <array>
     */
    // TODO: Needs to be PORTED for Hogeschool Gent!
    // To be ported to a static variant, should use the join library
    // If needed, adapt the method above to also allow returneing ids only
    public function retrieve_direct_subscribed_groups_array($user_id, $only_retrieve_ids = false)
    {
        $cache_hash = md5($user_id . ':' . (int) $only_retrieve_ids);
        
        if (is_null($this->direct_subscribed_groups_cache[$cache_hash]))
        {
            $direct_subscribed_groups = $this->retrieve_direct_subscribed_groups($user_id);
            if (! $direct_subscribed_groups->is_empty())
            {
                if ($only_retrieve_ids)
                {
                    while ($direct_subscribed_group = $direct_subscribed_groups->next_result())
                    {
                        $this->direct_subscribed_groups_cache[$cache_hash][] = $direct_subscribed_group->get_id();
                    }
                }
                else
                {
                    while ($direct_subscribed_group = $direct_subscribed_groups->next_result())
                    {
                        $this->direct_subscribed_groups_cache[$cache_hash][] = $direct_subscribed_group;
                    }
                }
            }
        }
        return $this->direct_subscribed_groups_cache[$cache_hash];
    }
    
    // DEAD/OK, can be made static and moved to the DM
    // There appear to be no callers of this method
    public function unsubscribe_user_from_all_groups($user)
    {
        $condition = new EqualityCondition(
                new PropertyConditionVariable(GroupRelUser :: class_name(), GroupRelUser :: PROPERTY_USER_ID),
                new StaticConditionVariable($user->get_id()));
        return DataManager :: deletes(GroupRelUser :: get_table_name(), $condition);
    }
}
