<?php
namespace group;

use common\libraries\Display;
use common\libraries\Application;
use common\libraries\Translation;
use common\libraries\Utilities;
use common\libraries\EqualityCondition;
use common\libraries\Request;
use common\libraries\AndCondition;
use common\libraries\NotCondition;
use common\libraries\OrCondition;
use common\libraries\PatternMatchCondition;
use common\libraries\ToolbarItem;
use common\libraries\Theme;
use common\libraries\ActionBarSearchForm;
use common\libraries\AdministrationComponent;
use common\libraries\Breadcrumb;
use common\libraries\BreadcrumbTrail;
use common\libraries\ActionBarRenderer;
use common\libraries\NewObjectTableSupport;
use user\User;

/**
 * $Id: subscribe_user_browser.class.php 224 2009-11-13 14:40:30Z kariboe $
 *
 * @package group.lib.group_manager.component
 */
class GroupManagerSubscribeUserBrowserComponent extends GroupManager implements AdministrationComponent,
        NewObjectTableSupport
{

    private $group;

    private $ab;

    /**
     * Runs this component and displays its output.
     */
    public function run()
    {
        $group_id = Request :: get(GroupManager :: PARAM_GROUP_ID);

        if (isset($group_id))
        {
            $this->group = $this->retrieve_group($group_id);
        }

        if (! $this->get_user()->is_platform_admin())
        {
            $this->display_header();
            Display :: error_message(Translation :: get('NotAllowed', null, Utilities :: COMMON_LIBRARIES));
            $this->display_footer();
            exit();
        }
        $this->ab = $this->get_action_bar();
        $output = $this->get_user_subscribe_html();

        $this->display_header();
        echo $this->ab->as_html() . '<br />';
        echo $output;
        $this->display_footer();
    }

    public function get_user_subscribe_html()
    {
        $this->set_parameter(ActionBarSearchForm :: PARAM_SIMPLE_SEARCH_QUERY, $this->ab->get_query());

        $table = new SubscribeUserBrowserTable($this);

        $html = array();
        $html[] = $table->as_html();

        return implode($html, "\n");
    }

    public function get_object_table_condition($object_table_class_name)
    {
        $condition = new EqualityCondition(GroupRelUser :: PROPERTY_GROUP_ID,
                Request :: get(GroupRelUser :: PROPERTY_GROUP_ID));

        $users = $this->retrieve_group_rel_users($condition);

        $conditions = array();
        while ($user = $users->next_result())
        {
            $conditions[] = new NotCondition(new EqualityCondition(User :: PROPERTY_ID, $user->get_user_id()));
        }

        $query = $this->ab->get_query();

        if (isset($query) && $query != '')
        {
            $or_conditions[] = new PatternMatchCondition(User :: PROPERTY_FIRSTNAME, '*' . $query . '*');
            $or_conditions[] = new PatternMatchCondition(User :: PROPERTY_LASTNAME, '*' . $query . '*');
            $or_conditions[] = new PatternMatchCondition(User :: PROPERTY_USERNAME, '*' . $query . '*');
            $conditions[] = new OrCondition($or_conditions);
        }

        if (count($conditions) == 0)
        {
            return null;
        }

        $condition = new AndCondition($conditions);

        return $condition;
    }

    public function get_group()
    {
        return $this->group;
    }

    public function get_action_bar()
    {
        $group = $this->group;

        $action_bar = new ActionBarRenderer(ActionBarRenderer :: TYPE_HORIZONTAL);

        $action_bar->set_search_url($this->get_url(array(GroupManager :: PARAM_GROUP_ID => $group->get_id())));

        $action_bar->add_common_action(
                new ToolbarItem(Translation :: get('ShowAll', null, Utilities :: COMMON_LIBRARIES),
                        Theme :: get_common_image_path() . 'action_browser.png',
                        $this->get_url(array(GroupManager :: PARAM_GROUP_ID => $group->get_id())),
                        ToolbarItem :: DISPLAY_ICON_AND_LABEL));
        // $action_bar->add_common_action(new ToolbarItem(Translation :: get('ShowGroup'), Theme ::
        // get_common_image_path().'action_browser.png', $this->get_url(array(Application :: PARAM_ACTION =>
        // GroupManager :: ACTION_BROWSE_GROUPS, GroupManager :: PARAM_GROUP_ID => $group->get_id()), ToolbarItem ::
        // DISPLAY_ICON_AND_LABEL));

        return $action_bar;
    }

    public function add_additional_breadcrumbs(BreadcrumbTrail $breadcrumbtrail)
    {
        $breadcrumbtrail->add(
                new Breadcrumb(
                        $this->get_url(
                                array(Application :: PARAM_ACTION => GroupManager :: ACTION_BROWSE_GROUPS)),
                        Translation :: get('GroupManagerBrowserComponent')));
        $breadcrumbtrail->add(
                new Breadcrumb(
                        $this->get_url(
                                array(Application :: PARAM_ACTION => GroupManager :: ACTION_VIEW_GROUP,
                                        GroupManager :: PARAM_GROUP_ID => Request :: get(GroupManager :: PARAM_GROUP_ID))),
                        Translation :: get('GroupManagerViewerComponent')));
        $breadcrumbtrail->add_help('group general');
    }

    public function get_additional_parameters()
    {
        return array(GroupManager :: PARAM_GROUP_ID);
    }
}
