<?php
namespace group;

use user\UserSearchForm;
use common\libraries\Export;
use common\libraries\Import;
use common\libraries\Application;
use common\libraries\CoreApplication;
use common\libraries\Translation;
use common\libraries\Utilities;
use common\libraries\Request;
use common\libraries\DynamicAction;
use common\libraries\Theme;
use common\libraries\redirect;
use common\libraries\Display;
use common\libraries\AndCondition;
use common\libraries\EqualityCondition;
use common\libraries\DataClassRetrieveParameters;
use common\libraries\DataClassRetrievesParameters;
use rights\RightsManager;
use rights\GroupRightManager;
use core\metadata\MetadataManager;
use common\libraries\CommonDataManager;
use common\libraries\package\PackageList;

/**
 * $Id: group_manager.class.php 224 2009-11-13 14:40:30Z kariboe $
 *
 * @package group.lib.group_manager
 */

/**
 * A user manager provides some functionalities to the admin to manage his users. For each functionality a component is
 * available.
 */
class GroupManager extends CoreApplication
{
    const APPLICATION_NAME = 'group';
    const PARAM_GROUP_ID = 'group_id';
    const PARAM_GROUP_REL_USER_ID = 'group_rel_user_id';
    const PARAM_USER_ID = 'user_id';
    const PARAM_FIRSTLETTER = 'firstletter';
    const PARAM_COMPONENT_ACTION = 'action';
    const ACTION_CREATE_GROUP = 'creator';
    const ACTION_BROWSE_GROUPS = 'browser';
    const ACTION_EDIT_GROUP = 'editor';
    const ACTION_DELETE_GROUP = 'deleter';
    const ACTION_MOVE_GROUP = 'mover';
    const ACTION_TRUNCATE_GROUP = 'truncater';
    const ACTION_VIEW_GROUP = 'viewer';
    const ACTION_EXPORT = 'exporter';
    const ACTION_IMPORT = 'importer';
    const ACTION_IMPORT_GROUP_USERS = 'group_user_importer';
    const ACTION_SUBSCRIBE_USER_TO_GROUP = 'subscriber';
    const ACTION_SUBSCRIBE_USER_BROWSER = 'subscribe_user_browser';
    const ACTION_UNSUBSCRIBE_USER_FROM_GROUP = 'unsubscriber';
    const ACTION_MANAGE_RIGHTS_TEMPLATES = 'group_rights_template_manager';
    const ACTION_MANAGE_USAGE = 'usage_manager';
    const DEFAULT_ACTION = self :: ACTION_BROWSE_GROUPS;

    private $parameters;

    private $search_parameters;

    private $user_search_parameters;

    private $search_form;

    private $user_search_form;

    private $user_id;

    private $user;

    private $category_menu;

    private $quota_url;

    private $publication_url;

    private $create_url;

    private $recycle_bin_url;
    protected $breadcrumbs;

    public function __construct($user = null)
    {
        parent :: __construct($user);
        $this->create_url = $this->get_url(array(self :: PARAM_ACTION => self :: ACTION_CREATE_GROUP));
    }

    private function display_search_form()
    {
        echo $this->get_search_form()->display();
    }

    public function display_user_search_form()
    {
        echo $this->get_user_search_form()->display();
    }

    public function count_groups($condition = null)
    {
        return DataManager :: count(Group :: class_name(), $condition);
    }

    public function count_group_rel_users($condition = null)
    {
        return DataManager :: count(GroupRelUser :: class_name(), $condition);
    }

    public function get_search_condition()
    {
        return $this->get_search_form()->get_condition();
    }

    public function get_user_search_condition()
    {
        return $this->get_user_search_form()->get_condition();
    }

    private function get_search_form()
    {
        if (! isset($this->search_form))
        {
            $this->search_form = new GroupSearchForm($this, $this->get_url());
        }
        return $this->search_form;
    }

    private function get_user_search_form()
    {
        if (! isset($this->user_search_form))
        {
            $this->user_search_form = new UserSearchForm(
                $this,
                $this->get_url(array(self :: PARAM_GROUP_ID => Request :: get(self :: PARAM_GROUP_ID))));
        }
        return $this->user_search_form;
    }

    public function get_search_validate()
    {
        return $this->get_search_form()->validate();
    }

    public function get_user_search_validate()
    {
        return $this->get_user_search_form()->validate();
    }

    /**
     * Gets the parameter list
     *
     * @param boolean $include_search Include the search parameters in the returned list?
     * @return array The list of parameters.
     */
    public function get_parameters($include_search = false, $include_user_search = false)
    {
        $parms = parent :: get_parameters();

        if ($include_search && isset($this->search_parameters))
        {
            $parms = array_merge($this->search_parameters, $parms);
        }

        if ($include_user_search && isset($this->user_search_parameters))
        {
            $parms = array_merge($this->user_search_parameters, $parms);
        }

        return $parms;
    }

    public function retrieve_groups($condition = null, $offset = null, $count = null, $order_property = null)
    {
        return DataManager :: retrieves(
            Group :: class_name(),
            new DataClassRetrievesParameters($condition, $count, $offset, $order_property));
    }

    public static function retrieve_group_rel_users($condition = null, $offset = null, $count = null, $order_property = null)
    {
        return DataManager :: retrieves(
            GroupRelUser :: class_name(),
            new DataClassRetrievesParameters($condition, $count, $offset, $order_property));
    }

    public static function retrieve_group_rel_user($user_id, $group_id)
    {
        $conditions = array();
        $conditions[] = new EqualityCondition(GroupRelUser :: PROPERTY_USER_ID, $user_id);
        $conditions[] = new EqualityCondition(GroupRelUser :: PROPERTY_GROUP_ID, $group_id);
        $condition = new AndCondition($conditions);

        return DataManager :: retrieve(GroupRelUser :: class_name(), new DataClassRetrieveParameters($condition));
    }

    /**
     * Sets the active URL in the navigation menu.
     *
     * @param string $url The active URL.
     */
    public function force_menu_url($url)
    {
        // $this->get_category_menu()->forceCurrentUrl($url);
    }

    public function retrieve_group($id)
    {
        return DataManager :: retrieve_by_id(Group :: class_name(), $id);
    }

    public static function get_application_platform_admin_links($application = self :: APPLICATION_NAME)
    {
        $links = array();
        $links[] = new DynamicAction(
            Translation :: get('List', null, Utilities :: COMMON_LIBRARIES),
            Translation :: get('ListDescription'),
            Theme :: get_image_path() . 'admin/list.png',
            Redirect :: get_link(
                self :: APPLICATION_NAME,
                array(Application :: PARAM_ACTION => GroupManager :: ACTION_BROWSE_GROUPS),
                array(),
                false,
                Redirect :: TYPE_CORE));
        $links[] = new DynamicAction(
            Translation :: get('Create', null, Utilities :: COMMON_LIBRARIES),
            Translation :: get('CreateDescription'),
            Theme :: get_image_path() . 'admin/add.png',
            Redirect :: get_link(
                self :: APPLICATION_NAME,
                array(
                    Application :: PARAM_ACTION => GroupManager :: ACTION_CREATE_GROUP,
                    GroupManager :: PARAM_GROUP_ID => 0),
                array(),
                false,
                Redirect :: TYPE_CORE));
        $links[] = new DynamicAction(
            Translation :: get('Export', null, Utilities :: COMMON_LIBRARIES),
            Translation :: get('ExportDescription'),
            Theme :: get_image_path() . 'admin/export.png',
            Redirect :: get_link(
                self :: APPLICATION_NAME,
                array(Application :: PARAM_ACTION => GroupManager :: ACTION_EXPORT),
                array(),
                false,
                Redirect :: TYPE_CORE));
        $links[] = new DynamicAction(
            Translation :: get('Import', null, Utilities :: COMMON_LIBRARIES),
            Translation :: get('ImportDescription'),
            Theme :: get_image_path() . 'admin/import.png',
            Redirect :: get_link(
                self :: APPLICATION_NAME,
                array(Application :: PARAM_ACTION => GroupManager :: ACTION_IMPORT),
                array(),
                false,
                Redirect :: TYPE_CORE));
        $links[] = new DynamicAction(
            Translation :: get('ImportGroupUsers'),
            Translation :: get('ImportGroupUsersDescription'),
            Theme :: get_image_path() . 'admin/import.png',
            Redirect :: get_link(
                self :: APPLICATION_NAME,
                array(Application :: PARAM_ACTION => GroupManager :: ACTION_IMPORT_GROUP_USERS),
                array(),
                false,
                Redirect :: TYPE_CORE));
        $links[] = new DynamicAction(
            Translation :: get('ManageGroupUsage'),
            Translation :: get('ManageGroupUsageDescription'),
            Theme :: get_image_path() . 'admin/manage.png',
            Redirect :: get_link(
                self :: APPLICATION_NAME,
                array(Application :: PARAM_ACTION => GroupManager :: ACTION_MANAGE_USAGE),
                array(),
                false,
                Redirect :: TYPE_CORE));

        $info = parent :: get_application_platform_admin_links(self :: APPLICATION_NAME);
        $info['links'] = $links;
        $info['search'] = Redirect :: get_link(
            self :: APPLICATION_NAME,
            array(Application :: PARAM_ACTION => GroupManager :: ACTION_BROWSE_GROUPS),
            array(),
            false,
            Redirect :: TYPE_CORE);

        return $info;
    }

    public function get_group_editing_url($group)
    {
        return $this->get_url(
            array(self :: PARAM_ACTION => self :: ACTION_EDIT_GROUP, self :: PARAM_GROUP_ID => $group->get_id()));
    }

    public function get_create_group_url($parent_id)
    {
        return $this->get_url(
            array(self :: PARAM_ACTION => self :: ACTION_CREATE_GROUP, self :: PARAM_GROUP_ID => $parent_id));
    }

    public function get_group_emptying_url($group)
    {
        return $this->get_url(
            array(self :: PARAM_ACTION => self :: ACTION_TRUNCATE_GROUP, self :: PARAM_GROUP_ID => $group->get_id()));
    }

    public function get_group_viewing_url($group)
    {
        return $this->get_url(
            array(self :: PARAM_ACTION => self :: ACTION_VIEW_GROUP, self :: PARAM_GROUP_ID => $group->get_id()));
    }

    public function get_group_metadata_url($group)
    {
        return $this->get_url(
            array(
                self :: PARAM_APPLICATION => MetadataManager :: APPLICATION_NAME,
                MetadataManager :: PARAM_ACTION => MetadataManager :: ACTION_EDIT_GROUP_METADATA,
                MetadataManager :: PARAM_GROUP => $group->get_id()));
    }

    public function get_group_rel_user_unsubscribing_url($groupreluser)
    {
        return $this->get_url(
            array(
                self :: PARAM_ACTION => self :: ACTION_UNSUBSCRIBE_USER_FROM_GROUP,
                self :: PARAM_GROUP_REL_USER_ID => $groupreluser->get_group_id() . '|' . $groupreluser->get_user_id()));
    }

    public function get_group_rel_user_subscribing_url($group, $user)
    {
        return $this->get_url(
            array(
                self :: PARAM_ACTION => self :: ACTION_SUBSCRIBE_USER_TO_GROUP,
                self :: PARAM_GROUP_ID => $group->get_id(),
                self :: PARAM_USER_ID => $user->get_id()));
    }

    public function get_group_suscribe_user_browser_url($group)
    {
        return $this->get_url(
            array(
                self :: PARAM_ACTION => self :: ACTION_SUBSCRIBE_USER_BROWSER,
                self :: PARAM_GROUP_ID => $group->get_id()));
    }

    public function get_group_delete_url($group)
    {
        return $this->get_url(
            array(self :: PARAM_ACTION => self :: ACTION_DELETE_GROUP, self :: PARAM_GROUP_ID => $group->get_id()));
    }

    public function get_import_url()
    {
        return $this->get_url(array(self :: PARAM_ACTION => self :: ACTION_IMPORT));
    }

    public function get_export_url()
    {
        return $this->get_url(array(self :: PARAM_ACTION => self :: ACTION_EXPORT));
    }

    public function get_move_group_url($group)
    {
        return $this->get_url(
            array(self :: PARAM_ACTION => self :: ACTION_MOVE_GROUP, self :: PARAM_GROUP_ID => $group->get_id()));
    }

    public function get_manage_group_rights_url($group)
    {
        return $this->get_url(
            array(
                Application :: PARAM_APPLICATION => RightsManager :: APPLICATION_NAME,
                Application :: PARAM_ACTION => RightsManager :: ACTION_MANAGE_GROUP_RIGHTS,
                GroupRightManager :: PARAM_GROUP_RIGHT_ACTION => GroupRightManager :: ACTION_BROWSE_GROUP_RIGHTS,
                GroupRightManager :: PARAM_GROUP => $group->get_id()));
    }

    /**
     * Helper function for the Application class, pending access to class constants via variables in PHP 5.3 e.g. $name
     * = $class :: DEFAULT_ACTION DO NOT USE IN THIS APPLICATION'S CONTEXT Instead use: - self :: DEFAULT_ACTION in the
     * context of this class - YourApplicationManager :: DEFAULT_ACTION in all other application classes
     */
    public function get_default_action()
    {
        return self :: DEFAULT_ACTION;
    }

    public static function get_installable_application_packages($include_installed = false)
    {
        $package_list = new PackageList(
            self :: context(),
            Translation :: get('TypeName', null, __NAMESPACE__),
            Theme :: get_image_path() . 'logo/16.png');

        $integration_list = new \common\libraries\package\PackageList(
            self :: context() . '\integration',
            Translation :: get('Integrations', null, __NAMESPACE__));

        if (! CommonDataManager :: get_registration(self :: context() . '\integration\tracking') || $include_installed)
        {
            $integration_list->add_package(self :: context() . '\integration\tracking');
        }

        if (! CommonDataManager :: get_registration(self :: context() . '\integration\webservice') || $include_installed)
        {
            $integration_list->add_package(self :: context() . '\integration\webservice');
        }

        $package_list->add_child($integration_list);

        return $package_list;
    }
}
