<?php
namespace reporting;

use common\libraries\EqualityCondition;
use common\libraries\Mdb2Database;

/**
 * $Id: database_reporting_data_manager.class.php 232 2009-11-16 10:11:48Z vanpouckesven $
 *
 * @package reporting.lib.data_manager
 * @author Michael Kyndt
 */

class Mdb2ReportingDataManager extends Mdb2Database implements ReportingDataManagerInterface
{

    public function initialize()
    {
        parent :: initialize();
        $this->set_prefix('reporting_');
    }

    /**
     * Creates a reporting block in the database
     *
     * @param $reporting_block ReportingBlock
     */
    public function create_reporting_block($reporting_block)
    {
        return $this->create($reporting_block);
    }

    /**
     * Updates an reporting block (needed for change of activity)
     *
     * @param $reporting_block ReportingBlock
     */
    public function update_reporting_block($reporting_block)
    {
        $condition = new EqualityCondition(ReportingBlock :: PROPERTY_ID, $reporting_block->get_id());
        return $this->update($reporting_block, $condition);
    }

    /**
     * Retrieves the reporting block with the given name
     *
     * @param $name String
     * @return ReportingBlock $reporting_block
     */
    public function retrieve_reporting_block_by_name($blockname)
    {
        $condition = new EqualityCondition(ReportingBlockRegistration :: PROPERTY_BLOCK, $blockname);
        return $this->retrieve_object(ReportingBlockRegistration :: get_table_name(), $condition, null,
                ReportingBlockRegistration :: CLASS_NAME);
    }

    /**
     * Retrieves all reporting blocks
     *
     * @return array of reporting blocks
     */
    public function retrieve_reporting_blocks($condition = null, $offset = null, $max_objects = null, $order_by = null)
    {
        return $this->retrieve_objects(ReportingBlock :: get_table_name(), $condition, $offset, $max_objects, $order_by,
                ReportingBlock :: CLASS_NAME);
    }

    /**
     * Count reporting blocks for a given condition
     *
     * @param $condition Condition
     * @return Int reporting block count
     */
    public function count_reporting_blocks($condition = null)
    {
        return $this->count_objects(ReportingBlock :: get_table_name(), $condition);
    }

    /**
     * Retrieves a reporting block by given id
     *
     * @param $reporting_block_id int
     * @return ReportingBlock $reporting_block
     */
    public function retrieve_reporting_block($reporting_block_id)
    {
        $condition = new EqualityCondition(ReportingBlock :: PROPERTY_ID, $reporting_block_id);
        return $this->retrieve_object(ReportingBlock :: get_table_name(), $condition, null,
                ReportingBlock :: CLASS_NAME);
    }

    public function create_reporting_template_registration($reporting_template_registration)
    {
        return $this->create($reporting_template_registration);
    } // create_reporting_template_registration

    public function create_reporting_block_registration($reporting_block_registration)
    {
        return $this->create($reporting_block_registration);
    } // create_reporting_template_registration

    public function update_reporting_template_registration($reporting_template_registration)
    {
        $condition = new EqualityCondition(ReportingTemplateRegistration :: PROPERTY_ID,
                $reporting_template_registration->get_id());
        return $this->update($reporting_template_registration, $condition);
    } // update_reporting_template_registration

    // function retrieve_reporting_template_registration_by_name($reporting_template_registration_name)
      // {
      // $condition = new EqualityCondition(ReportingTemplateRegistration :: PROPERTY_NAME,
      // $reporting_template_registration_name);
      // return $this->retrieve_object(ReportingTemplateRegistration :: get_table_name(), $condition);
      // }//retrieve_reporting_template_registration_by_name

    public function retrieve_reporting_template_registrations($condition = null, $offset = null, $max_objects = null,
            $order_property = null)
    {
        return $this->retrieve_objects(ReportingTemplateRegistration :: get_table_name(), $condition, $offset,
                $max_objects, $order_property, ReportingTemplateRegistration :: CLASS_NAME);
    } // retrieve_reporting_template_registrations

    public function retrieve_reporting_block_registrations($condition = null, $offset = null, $max_objects = null, $order_property = null)
    {
        return $this->retrieve_objects(ReportingBlockRegistration :: get_table_name(), $condition, $offset,
                $max_objects, $order_property, ReportingBlockRegistration :: CLASS_NAME);
    }

    public function count_reporting_template_registrations($condition = null)
    {
        return $this->count_objects(ReportingTemplateRegistration :: get_table_name(), $condition);
    } // count_reporting_template_registrations

    public function retrieve_reporting_template_registration($reporting_template_registration_id)
    {
        $condition = new EqualityCondition(ReportingTemplateRegistration :: PROPERTY_ID,
                $reporting_template_registration_id);
        return $this->retrieve_object(ReportingTemplateRegistration :: get_table_name(), $condition, null,
                ReportingTemplateRegistration :: CLASS_NAME);
    } // retrieve_reporting_template_registration

    public function retrieve_reporting_template_registration_by_condition($condition)
    {
        return $this->retrieve_object(ReportingTemplateRegistration :: get_table_name(), $condition, array(),
                ReportingTemplateRegistration :: CLASS_NAME);
    }

    public function retrieve_reporting_block_registration($reporting_block_registration_id)
    {
        $condition = new EqualityCondition(ReportingBlockRegistration :: PROPERTY_ID, $reporting_block_registration_id);
        return $this->retrieve_object(ReportingBlockRegistration :: get_table_name(), $condition, array(),
                ReportingBlockRegistration :: CLASS_NAME);
    }

    public function delete_reporting_block_registrations($condition = null)
    {
        return $this->delete_objects(ReportingBlockRegistration :: get_table_name(), $condition);
    }

    public function delete_reporting_template_registrations($condition = null)
    {
        return $this->delete_objects(ReportingTemplateRegistration :: get_table_name(), $condition);
    }

    public function delete_reporting_blocks($condition = null)
    {
        return $this->delete_objects(ReportingBlock :: get_table_name(), $condition);
    }

    public function delete_orphaned_block_template_relations()
    {
        $query = 'DELETE FROM ' . $this->escape_table_name('reporting_template_registration_rel_reporting_block') . ' WHERE ';
        $query .= parent :: escape_column_name('reporting_template_registration_id') . ' NOT IN (SELECT ' . parent :: escape_column_name(
                ReportingTemplateRegistration :: PROPERTY_ID) . ' FROM ' . $this->escape_table_name(
                ReportingTemplateRegistration :: get_table_name()) . ') OR ';
        $query .= parent :: escape_column_name('reporting_block_id') . ' NOT IN (SELECT ' . parent :: escape_column_name(
                ReportingBlockRegistration :: PROPERTY_ID) . ' FROM ' . $this->escape_table_name(
                ReportingBlockRegistration :: get_table_name()) . ')';
        return $this->query($query);
    }

    public function retrieve_reporting_template_object($classname)
    {
        $condition = new EqualityCondition(ReportingTemplateRegistration :: PROPERTY_TEMPLATE, $classname);

        return $this->retrieve_object(ReportingTemplateRegistration :: get_table_name(), $condition, null,
                ReportingTemplateRegistration :: CLASS_NAME);
    }
}
