<?php
namespace reporting\upgrade\v4_0_0;

use common\libraries\CommonDataManager;
use common\libraries\package\Package;
use core\lynx\manager\PackageInstaller;
use common\libraries\Registration;
use common\libraries\Utilities;

/**
 *
 * @author Hans De Bisschop
 * @author Magali Gillard
 */
class Upgrader extends \common\libraries\package\Upgrader
{

    /**
     * Constructor
     */
    public function __construct()
    {
        parent :: __construct(CommonDataManager :: get_instance());
    }

    /*
     * (non-PHPdoc) @see \common\libraries\package\Upgrader::run()
     */
    public function run()
    {
        $data_manager = $this->get_data_manager();

        $success = $data_manager->transactional(
            function ($c) use($data_manager)
            {
                $package = Package :: get('reporting');
                $registration = CommonDataManager :: get_registration('reporting');
                $registration->set_version($package->get_version());
                if (! $registration->update())
                {
                    return false;
                }

                if (! \reporting\ReportingDataManager :: get_instance()->drop_storage_unit(
                    'reporting_block_registration'))
                {
                    return false;
                }

                if (! \reporting\ReportingDataManager :: get_instance()->drop_storage_unit(
                    'reporting_template_registration'))
                {
                    return false;
                }

                if (! \reporting\ReportingDataManager :: get_instance()->drop_storage_unit(
                    'reporting_template_registration_rel_reporting_block'))
                {
                    return false;
                }

                return true;
            });

        if (! $success)
        {
            return false;
        }

        $packages = array();
        $packages[] = 'reporting\viewer';

        foreach ($packages as $package)
        {
            $installer = new PackageInstaller($package);
            if (! $installer->run())
            {
                return false;
            }
        }

        $registration = CommonDataManager :: get_registration('common\extensions\reporting_viewer');
        if (! $registration->delete())
        {
            return false;
        }

        // Webservices
        $package_info = Package :: get('reporting\integration\webservice');

        $application_registration = new Registration();

        $parent_namespace = Utilities :: get_namespace_parent('reporting\integration\webservice');
        $application_registration->set_type((! $parent_namespace ? 'core' : $parent_namespace));
        $application_registration->set_name($package_info->get_code());
        $application_registration->set_category($package_info->get_category());
        $application_registration->set_version($package_info->get_version());
        $application_registration->set_status(Registration :: STATUS_ACTIVE);
        $application_registration->set_context('reporting\integration\webservice');

        if (! $application_registration->create())
        {
            return false;
        }

        return true;
    }
}
