<?php
namespace reporting\viewer;

use common\libraries\Request;
use common\libraries\SubManager;

/**
 *
 * @author Hans De Bisschop & Magali Gillard
 * @package reporting.viewer
 */
class Manager extends SubManager
{
    const PARAM_ACTION = 'reporting_viewer_action';
    const PARAM_SHOW_ALL = 'show_all';
    const PARAM_VIEWS = 'views';
    const PARAM_FORMAT = 'format';
    const PARAM_VIEW = 'view';
    const PARAM_BLOCK_ID = 'block_id';
    const ACTION_VIEW = 'viewer';
    const ACTION_SAVE = 'saver';
    const DEFAULT_ACTION = self :: ACTION_VIEW;

    /**
     *
     * @var \reporting\ReportingTemplate
     */
    private $template;

    /**
     *
     * @return string
     */
    public static function get_action_parameter()
    {
        return self :: PARAM_ACTION;
    }

    /**
     *
     * @return string
     */
    public function get_default_action()
    {
        return self :: DEFAULT_ACTION;
    }

    /**
     *
     * @param \reporting\ReportingTemplate $template
     */
    public function set_template(\reporting\ReportingTemplate $template)
    {
        $this->template = $template;
    }

    /**
     *
     * @return \reporting\ReportingTemplate
     */
    public function get_template()
    {
        return $this->template;
    }

    /**
     *
     * @param string $template_name
     */
    public function set_template_by_name($template_name)
    {
        $this->template = new $template_name($this->get_parent());
    }

    /**
     *
     * @return boolean
     */
    public function show_all()
    {
        $show_all = Request :: get(self :: PARAM_SHOW_ALL);
        return $show_all == 1 ? true : false;
    }

    /**
     *
     * @return int
     */
    public function get_current_block()
    {
        return Request :: get(self :: PARAM_BLOCK_ID);
    }

    /**
     *
     * @return multitype:string
     */
    public function get_current_view()
    {
        return Request :: get(self :: PARAM_VIEWS);
    }

    /**
     *
     * @param \common\libraries\Application $application
     * @param string $template
     */
    public static function launch($application, $template)
    {
        static :: construct($application, $template)->run();
    }

    /**
     *
     * @param \common\libraries\Application $application
     * @param string $template
     * @return \common\libraries\SubManager
     */
    public static function construct($application, $template)
    {
        $component = parent :: construct($application);
        $component->set_template_by_name($template);
        return $component;
    }
}
