<?php
namespace reporting\viewer;

use common\libraries\Utilities;

/**
 *
 * @author Hans De Bisschop & Magali Gillard
 * @package reporting.viewer
 */
abstract class BlockRendition
{
    const FORMAT_XML = 'xml';
    const FORMAT_HTML = 'html';
    const FORMAT_JSON = 'json';
    const FORMAT_CSV = 'csv';

    /**
     *
     * @var \reporting\BlockRenditionImplementation
     */
    private $rendition_implementation;

    /**
     *
     * @param \reporting\BlockRenditionImplementation $rendition_implementation
     */
    public function __construct(\reporting\BlockRenditionImplementation $rendition_implementation)
    {
        $this->rendition_implementation = $rendition_implementation;
    }

    /**
     *
     * @return \reporting\BlockRenditionImplementation
     */
    public function get_rendition_implementation()
    {
        return $this->rendition_implementation;
    }

    /**
     *
     * @param \reporting\BlockRenditionImplementation $rendition_implementation
     */
    public function set_rendition_implementation(\reporting\BlockRenditionImplementation $rendition_implementation)
    {
        $this->rendition_implementation = $rendition_implementation;
    }

    public function get_context()
    {
        return $this->rendition_implementation->get_context();
    }

    public function set_context($context)
    {
        $this->rendition_implementation->set_context($context);
    }

    /**
     *
     * @return \reporting\ReportingBlock
     */
    public function get_block()
    {
        return $this->rendition_implementation->get_block();
    }

    /**
     *
     * @param \reporting\ReportingBlock $block
     */
    public function set_block($block)
    {
        $this->rendition_implementation->set_block($block);
    }

    /**
     *
     * @param \reporting\BlockRenditionImplementation $rendition_implementation
     * @return string
     */
    public static function launch($rendition_implementation)
    {
        return self :: factory($rendition_implementation)->render();
    }

    /**
     *
     * @param \reporting\BlockRenditionImplementation $rendition_implementation
     * @return \reporting\BlockRendition
     */
    public static function factory($rendition_implementation)
    {
        $class = __NAMESPACE__ . '\\' . Utilities :: underscores_to_camelcase($rendition_implementation->get_format()) . Utilities :: underscores_to_camelcase(
                $rendition_implementation->get_view()) . 'BlockRendition';
        return new $class($rendition_implementation);
    }

    public static function get_format()
    {
        return static :: FORMAT;
    }

    public static function get_view()
    {
        return static :: VIEW;
    }
}
