<?php
namespace reporting\viewer;

use common\libraries\Translation;
use common\libraries\Utilities;
use common\libraries\Path;
use common\libraries\Filesystem;

/**
 *
 * @author Hans De Bisschop & Magali Gillard
 * @package reporting.viewer
 */
class CsvDefaultTemplateRendition extends CsvTemplateRendition
{

    public function render()
    {
        if ($this->get_context()->show_all())
        {
            $views = $this->get_context()->get_current_view();
            $specific_views = array();

            foreach ($views as $key => $view)
            {
                if ($view == static :: get_format())
                {
                    $specific_views[] = $key;
                }
            }

            // Render the specific views as called from another view which can show multiple blocks
            if (count($specific_views) > 0)
            {
                if (count($specific_views) == 1)
                {
                    $current_block = $this->get_template()->get_block($specific_views[0]);
                    $file_name = Translation :: get(Utilities :: get_classname_from_object($current_block), null,
                            Utilities :: get_namespace_from_object($current_block)) . date('_Y-m-d_H-i-s') . '.csv';

                    $data = BlockRenditionImplementation :: launch($this, $current_block, $this->get_format(),
                            CsvBlockRendition :: VIEW_DEFAULT);
                }
                else
                {
                    $data = array();

                    foreach ($specific_views as $specific_view)
                    {
                        $block = $this->get_template()->get_block($specific_view);
                        $rendered_block = BlockRenditionImplementation :: launch($this, $block, $this->get_format(),
                                CsvBlockRendition :: VIEW_DEFAULT);
                        $data[] = array(
                                Translation :: get(Utilities :: get_classname_from_object($block), null,
                                        Utilities :: get_namespace_from_object($block)));
                        $data[] = array();
                        $data = array_merge($data, $rendered_block);
                        $data[] = array();
                    }

                    $file_name = Translation :: get(Utilities :: get_classname_from_object($this->get_template()), null,
                            Utilities :: get_namespace_from_object($this->get_template())) . date('_Y-m-d_H-i-s') . '.csv';
                }
            }
            // No specific view was set and we are rendering everything, so render everything
            else
            {
                $data = array();

                foreach ($this->get_template()->get_blocks() as $key => $block)
                {
                    $rendered_block = BlockRenditionImplementation :: launch($this, $block, $this->get_format(),
                            CsvBlockRendition :: VIEW_DEFAULT);
                    $data[] = array(
                            Translation :: get(Utilities :: get_classname_from_object($block), null,
                                    Utilities :: get_namespace_from_object($block)));
                    $data[] = array();
                    $data = array_merge($data, $rendered_block);
                    $data[] = array();
                }

                $file_name = Translation :: get(Utilities :: get_classname_from_object($this->get_template()), null,
                        Utilities :: get_namespace_from_object($this->get_template())) . date('_Y-m-d_H-i-s') . '.csv';
            }
        }
        else
        {
            $current_block_id = $this->get_context()->get_current_block();
            $current_block = $this->get_template()->get_block($current_block_id);
            $file_name = Translation :: get(Utilities :: get_classname_from_object($current_block), null,
                    Utilities :: get_namespace_from_object($current_block)) . date('_Y-m-d_H-i-s') . '.csv';

            $data = BlockRenditionImplementation :: launch($this, $current_block, $this->get_format(),
                    CsvBlockRendition :: VIEW_DEFAULT);
        }

        $file = Path :: get(SYS_ARCHIVE_PATH) . Filesystem :: create_unique_name(Path :: get(SYS_ARCHIVE_PATH),
                $file_name);

        $handle = fopen($file, 'a+');

        foreach ($data as $row)
        {
            fputcsv($handle, $row);
        }

        fclose($handle);

        return $file;
    }
}
