<?php
namespace reporting\viewer;

use common\libraries\Utilities;

/**
 *
 * @author Hans De Bisschop & Magali Gillard
 * @package reporting.viewer
 */
abstract class TemplateRenditionImplementation extends AbstractTemplateRenditionImplementation
{

    /**
     *
     * @param \common\libraries\Application $context
     * @param ReportingTemplate $template
     * @param string $format
     * @param string $view
     */
    public static function launch($context, ReportingTemplate $template, $format = TemplateRendition::FORMAT_HTML, $view = TemplateRendition::VIEW_DEFAULT)
    {
        return self :: factory($context, $template, $format, $view)->render();
    }

    /**
     *
     * @param \common\libraries\Application $context
     * @param ReportingTemplate $template
     * @param string $format
     * @param string $view
     * @return \reporting\DummyContentObjectRenditionImplementation \reporting\TemplateRenditionImplementation
     */
    public static function factory($context, ReportingTemplate $template, $format = TemplateRendition::FORMAT_HTML, $view = TemplateRendition::VIEW_DEFAULT)
    {
        $namespace = Utilities :: get_namespace_from_object($template);
        $class = $namespace . '\\' . Utilities :: underscores_to_camelcase($format) . Utilities :: underscores_to_camelcase(
                $view) . 'TemplateRenditionImplementation';

        if (! class_exists($class, true))
        {
            return new DummyTemplateRenditionImplementation($context, $template, $format, $view);
        }
        else
        {
            return new $class($context, $template);
        }
    }

    /**
     *
     * @return string
     */
    public function get_view()
    {
        $class_name_parts = explode('_', Utilities :: get_classname_from_object($this, true));
        return $class_name_parts[1];
    }

    /**
     *
     * @return string
     */
    public function get_format()
    {
        $class_name_parts = explode('_', Utilities :: get_classname_from_object($this, true));
        return $class_name_parts[0];
    }
}
