<?php
namespace repository\content_object\assessment;

use common\libraries\Request;
use common\libraries\Translation;
use common\libraries\StaticConditionVariable;
use common\libraries\PropertyConditionVariable;
use common\libraries\DataClassRetrievesParameters;
use common\libraries\EqualityCondition;
use repository\ComplexContentObjectItem;

class AssessmentBuilderFeedbackOptionerComponent extends AssessmentBuilder
{
    public function run()
    {
        $feedback_option_type = Request :: get(self :: PARAM_FEEDBACK_OPTION_TYPE);
        $complex_content_object_items = \repository\DataManager :: retrieve_complex_content_object_items(
            ComplexContentObjectItem :: class_name(),
            new DataClassRetrievesParameters(
                new EqualityCondition(
                    new PropertyConditionVariable(
                        ComplexContentObjectItem :: class_name(),
                        ComplexContentObjectItem :: PROPERTY_PARENT),
                    new StaticConditionVariable($this->get_parent_content_object_id()))));

        $supported_types = array(
            \repository\content_object\assessment_multiple_choice_question\ComplexAssessmentMultipleChoiceQuestion :: class_name(),
            \repository\content_object\assessment_select_question\ComplexAssessmentSelectQuestion :: class_name());

        $failures = 0;
        $questions = 0;

        while ($complex_content_object_item = $complex_content_object_items->next_result())
        {
            if (in_array($complex_content_object_item->class_name(), $supported_types))
            {
                $questions ++;

                $complex_content_object_item->set_feedback_options($feedback_option_type);
                if (! $complex_content_object_item->update())
                {
                    $failures ++;
                }
            }
        }

        if ($failures > 0)
        {
            if ($failures == $questions && $questions == 1)
            {
                $message = 'FeedbackOptionNotSuccessfullySet';
            }
            elseif ($failures == $questions && $questions > 1)
            {
                $message = 'FeedbackOptionsNotSuccessfullySet';
            }
            else
            {
                $message = 'SomeFeedbackOptionsNotSuccessfullySet';
            }
        }
        else
        {
            if ($questions == 0)
            {
                $message = 'NoFeedbackOptions';
            }
            elseif ($questions == 1)
            {
                $message = 'FeedbackOptionSuccessfullySet';
            }
            else
            {
                $message = 'FeedbackOptionsSuccessfullySet';
            }
        }

        $this->redirect(
            Translation :: get($message),
            false,
            array(
                self :: PARAM_BUILDER_ACTION => self :: ACTION_BROWSE,
                self :: PARAM_COMPLEX_CONTENT_OBJECT_ITEM_ID => $this->get_complex_content_object_item_id()));
    }
}
