<?php
namespace repository\content_object\assessment_match_text_question;

use common\libraries\Translation;
use repository\ComplexContentObjectItemForm;

/**
 *
 * @package repository.lib.content_object.match_text_question
 */

/**
 * This class represents a form to create or update complex assessments
 */
class ComplexAssessmentMatchTextQuestionForm extends ComplexContentObjectItemForm
{
    const PROPERTY_RECALCULATE_WEIGHT = 'recalculate_weight';

    public function get_elements()
    {
        $elements[] = $this->createElement(
            'checkbox',
            self :: PROPERTY_RECALCULATE_WEIGHT,
            Translation :: get('RecalculateWeight'));

        $elements[] = $this->createElement(
            'text',
            ComplexAssessmentMatchTextQuestion :: PROPERTY_WEIGHT,
            Translation :: get('Weight'),
            array("size" => "50", 'disabled' => 'disabled'));

        $elements[] = $this->createElement(
            'checkbox',
            ComplexAssessmentMatchTextQuestion :: PROPERTY_FEEDBACK_ANSWER,
            Translation :: get('FeedbackAnswer'));

        return $elements;
    }

    public function get_default_values()
    {
        $cloi = $this->get_complex_content_object_item();

        if (isset($cloi))
        {
            $defaults[ComplexAssessmentMatchTextQuestion :: PROPERTY_WEIGHT] = $cloi->get_weight() ? $cloi->get_weight() : 0;
            $defaults[self :: PROPERTY_RECALCULATE_WEIGHT] = 1;
            $defaults[ComplexAssessmentMatchTextQuestion :: PROPERTY_FEEDBACK_ANSWER] = $cloi->get_feedback_answer() ? $cloi->get_feedback_answer() : 0;
        }

        return $defaults;
    }

    // Inherited
    public function create_complex_content_object_item()
    {
        $cloi = $this->get_complex_content_object_item();
        $values = $this->exportValues();
        $cloi->set_weight($values[ComplexAssessmentMatchTextQuestion :: PROPERTY_WEIGHT]);
        $cloi->set_feedback_answer($values[ComplexAssessmentMatchTextQuestion :: PROPERTY_FEEDBACK_ANSWER]);

        return parent :: create_complex_content_object_item();
    }

    public function create_cloi_from_values($values)
    {
        $cloi = $this->get_complex_content_object_item();
        $cloi->set_weight($values[ComplexAssessmentMatchTextQuestion :: PROPERTY_WEIGHT]);
        $cloi->set_feedback_answer($values[ComplexAssessmentMatchTextQuestion :: PROPERTY_FEEDBACK_ANSWER]);

        return parent :: create_complex_content_object_item();
    }

    public function update_cloi_from_values($values)
    {
        $cloi = $this->get_complex_content_object_item();

        if ($values[self :: PROPERTY_RECALCULATE_WEIGHT] == 1)
        {
            $cloi->set_weight($cloi->get_ref_object()->get_default_weight());
        }
        else
        {
            $cloi->set_weight($values[ComplexAssessmentMatchTextQuestion :: PROPERTY_WEIGHT]);
        }

        $cloi->set_feedback_answer($values[ComplexAssessmentMatchTextQuestion :: PROPERTY_FEEDBACK_ANSWER]);

        return parent :: update_complex_content_object_item();
    }

    // Inherited
    public function update_complex_content_object_item()
    {
        $cloi = $this->get_complex_content_object_item();
        $values = $this->exportValues();

        if ($values[self :: PROPERTY_RECALCULATE_WEIGHT] == 1)
        {
            $cloi->set_weight($cloi->get_ref_object()->get_default_weight());
        }
        else
        {
            $cloi->set_weight($values[ComplexAssessmentMatchTextQuestion :: PROPERTY_WEIGHT]);
        }

        $cloi->set_feedback_answer($values[ComplexAssessmentMatchTextQuestion :: PROPERTY_FEEDBACK_ANSWER]);

        return parent :: update_complex_content_object_item();
    }
}
