<?php
namespace repository\content_object\assignment;

use common\libraries\PropertyConditionVariable;
use common\libraries\StaticConditionVariable;
use common\libraries\ResourceManager;
use common\libraries\Path;
use repository\ContentObjectAdvancedFilter;
use common\libraries\AndCondition;
use common\libraries\Session;
use common\libraries\Translation;
use common\libraries\InequalityCondition;

class AdvancedFilter extends ContentObjectAdvancedFilter
{

    public function get_conditions()
    {
        $session_filter = Session :: retrieve('advanced_filter');
        $conditions = array();
        if ($this->get_form()->validate() || isset($session_filter))
        {
            if ($this->get_form()->validate())
            {
                $values = $this->get_form()->exportValues();
            }
            else
            {
                $values = unserialize(Session :: retrieve('advanced_filter'));
            }
            
            $filter = serialize($values);
            if ($this->get_form()->validate())
            {
                Session :: register('advanced_filter', $filter);
            }
            
            if ($values['start_time'] && $values['end_time'])
            {
                $date_conditions = array();
                
                $date_conditions[] = new InequalityCondition(
                        new PropertyConditionVariable(Assignment :: class_name(), Assignment :: PROPERTY_START_TIME), 
                        InequalityCondition :: GREATER_THAN_OR_EQUAL, 
                        new StaticConditionVariable(strtotime($values['start_time'])));
                
                $conditions[] = new InequalityCondition(
                        new PropertyConditionVariable(Assignment :: class_name(), Assignment :: PROPERTY_END_TIME), 
                        InequalityCondition :: GREATER_THAN_OR_EQUAL, 
                        new StaticConditionVariable(strtotime($values['end_time'])));
                
                $conditions[] = new AndCondition($date_conditions);
            }
            else
            {
                if ($values['start_time'])
                {
                    $conditions[] = new InequalityCondition(
                            new PropertyConditionVariable(Assignment :: class_name(), Assignment :: PROPERTY_START_TIME), 
                            InequalityCondition :: GREATER_THAN_OR_EQUAL, 
                            new StaticConditionVariable(strtotime($values['start_time'])));
                }
                elseif ($values['end_time'])
                {
                    $conditions[] = new InequalityCondition(
                            new PropertyConditionVariable(Assignment :: class_name(), Assignment :: PROPERTY_END_TIME), 
                            InequalityCondition :: GREATER_THAN_OR_EQUAL, 
                            new StaticConditionVariable(strtotime($values['end_time'])));
                }
            }
        }
        
        return $conditions;
    }

    public function get_parameters()
    {
        $session_filter = Session :: retrieve('advanced_filter');
        $html = array();
        if ($this->get_form()->validate() || isset($session_filter))
        {
            if ($this->get_form()->validate())
            {
                $values = $this->get_form()->exportValues();
            }
            else
            {
                $values = unserialize(Session :: retrieve('advanced_filter'));
            }
            
            if ($values['start_time'] && $values['end_time'])
            {
                $html[] = '<div class="parameter" id="parameter_assignment_time">' . Translation :: get('Between', 
                        array('START' => $values['start_time'], 'END' => $values['end_time'])) . '</div>';
            }
            else
            {
                if ($values['start_time'])
                {
                    $html[] = '<div class="parameter" id="parameter_assignment_start_time">' . Translation :: get(
                            'After', array('START' => $values['start_time'])) . '</div>';
                }
                elseif ($values['end_time'])
                {
                    $html[] = '<div class="parameter" id="parameter_assignment_end_time">' . Translation :: get(
                            'Before', array('END' => $values['end_time'])) . '</div>';
                }
            }
        }
        
        return implode("\n", $html);
    }

    public function add_elements()
    {
        $form = $this->get_form();
        
        $form->addElement('category', Translation :: get('Time'));
        $start_time = array();
        $start_time[] = $form->createElement('static', '', '', 
                '<span style="display:inline-block;width:100px">' . Translation :: get('StartTime') . '</span>');
        $start_time[] = $form->createElement('text', Assignment :: PROPERTY_START_TIME, Translation :: get('StartTime'), 
                'id="start_time" style="width:100px;"');
        $form->addGroup($start_time, 'start_time', null, null, false);
        $form->get_renderer()->setGroupElementTemplate('{element}', 'start_time');
        
        $end_time = array();
        $end_time[] = $form->createElement('static', '', '', 
                '<span style="display:inline-block; width:100px">' . Translation :: get('EndTime') . '</span>');
        $end_time[] = $form->createElement('text', Assignment :: PROPERTY_END_TIME, Translation :: get('EndTime'), 
                'id="end_time" style="width:100px;"');
        $form->addGroup($end_time, 'end_time', null, null, false);
        $form->get_renderer()->setGroupElementTemplate('{element}', 'end_time');
        $form->addElement('category');
        
        $form->addElement('html', 
                ResourceManager :: get_instance()->get_resource_html(
                        Path :: namespace_to_full_path(__NAMESPACE__, true) . 'resources/javascript/search.js'));
    }
}
