<?php
namespace repository\content_object\calendar_event;

use common\libraries\EqualityCondition;
use common\libraries\ResourceManager;
use common\libraries\Path;
use repository\RepositoryDataManager;
use repository\ContentObjectAdvancedFilter;
use common\libraries\AndCondition;
use common\libraries\Session;
use common\libraries\Translation;
use common\libraries\InequalityCondition;

class AdvancedFilter extends ContentObjectAdvancedFilter
{

    public function get_conditions()
    {
        $session_filter = Session :: retrieve('advanced_filter');
        $conditions = array();
        if ($this->get_form()->validate() || isset($session_filter))
        {
            if ($this->get_form()->validate())
            {
                $values = $this->get_form()->exportValues();
            }
            else
            {
                $values = unserialize(Session :: retrieve('advanced_filter'));
            }
            
            $filter = serialize($values);
            if ($this->get_form()->validate())
            {
                Session :: register('advanced_filter', $filter);
            }
            
            if ($values['start_date'] && $values['end_date'])
            {
                $date_conditions = array();
                $date_conditions[] = new InequalityCondition(CalendarEvent :: PROPERTY_START_DATE, InequalityCondition :: GREATER_THAN_OR_EQUAL, strtotime($values['start_date']), RepositoryDataManager ::get_alias_type_table(), true);
                $date_conditions[] = new InequalityCondition(CalendarEvent :: PROPERTY_END_DATE, InequalityCondition :: LESS_THAN_OR_EQUAL, strtotime($values['end_date']), RepositoryDataManager ::get_alias_type_table(), true);
                $conditions[] = new AndCondition($date_conditions);
            }
            else
            {
                if ($values['start_date'])
                {
                    $conditions[] = new InequalityCondition(CalendarEvent :: PROPERTY_START_DATE, InequalityCondition :: GREATER_THAN_OR_EQUAL, strtotime($values['start_date']), RepositoryDataManager ::get_alias_type_table(), true);
                }
                elseif ($values['end_date'])
                {
                    $conditions[] = new InequalityCondition(CalendarEvent :: PROPERTY_END_DATE, InequalityCondition :: LESS_THAN_OR_EQUAL, strtotime($values['end_date']), RepositoryDataManager ::get_alias_type_table(), true);
                
                }
            }
            
            if (isset($values[CalendarEvent :: PROPERTY_FREQUENCY]) && $values[CalendarEvent :: PROPERTY_FREQUENCY] != - 1)
            {
                $conditions[] = new EqualityCondition(CalendarEvent :: PROPERTY_FREQUENCY, $values[CalendarEvent :: PROPERTY_FREQUENCY], RepositoryDataManager ::get_alias_type_table(), true);
            }
        }
        
        return $conditions;
    }

    public function get_parameters()
    {
        $session_filter = Session :: retrieve('advanced_filter');
        $html = array();
        if ($this->get_form()->validate() || isset($session_filter))
        {
            if ($this->get_form()->validate())
            {
                $values = $this->get_form()->exportValues();
            }
            else
            {
                $values = unserialize(Session :: retrieve('advanced_filter'));
            }
            
            if ($values['start_date'] && $values['end_date'])
            {
                $html[] = '<div class="parameter" id="parameter_calendar_event_date">' . Translation :: get('Between', array(
                        'START' => $values['start_date'], 'END' => $values['end_date'])) . '</div>';
            }
            else
            {
                if ($values['start_date'])
                {
                    $html[] = '<div class="parameter" id="parameter_calendar_event_start_date">' . Translation :: get('After', array(
                            'START' => $values['start_date'])) . '</div>';
                }
                elseif ($values['end_date'])
                {
                    $html[] = '<div class="parameter" id="parameter_calendar_event_end_date">' . Translation :: get('Before', array(
                            'END' => $values['end_date'])) . '</div>';
                }
            }
            
            if (isset($values[CalendarEvent :: PROPERTY_FREQUENCY]) && $values[CalendarEvent :: PROPERTY_FREQUENCY] != - 1)
            {
                if ($values[CalendarEvent :: PROPERTY_FREQUENCY] == CalendarEvent :: REPEAT_TYPE_NONE)
                {
                    $translation = Translation :: get('NoRepeat');
                }
                else
                {
                    $translation = Translation :: get('RepeatSearchParameter', array(
                            'REPEAT' => CalendarEvent :: repeat_as_string($values[CalendarEvent :: PROPERTY_FREQUENCY])));
                }
                $html[] = '<div class="parameter" id="parameter_calendar_event_' . CalendarEvent :: PROPERTY_FREQUENCY . '">' . $translation . '</div>';
            }
        
        }
        
        return implode("\n", $html);
    }

    public function add_elements()
    {
        $form = $this->get_form();
        
        $form->addElement('category', Translation :: get('Date'));
        $start_date = array();
        $start_date[] = $form->createElement('static', '', '', '<span style="display:inline-block;width:100px">' . Translation :: get('StartDate') . '</span>');
        $start_date[] = $form->createElement('text', CalendarEvent :: PROPERTY_START_DATE, Translation :: get('StartDate'), 'id="start_date" style="width:100px;"');
        $form->addGroup($start_date, 'start_date', null, null, false);
        $form->get_renderer()->setGroupElementTemplate('{element}', 'start_date');
        
        $end_date = array();
        $end_date[] = $form->createElement('static', '', '', '<span style="display:inline-block; width:100px">' . Translation :: get('EndDate') . '</span>');
        $end_date[] = $form->createElement('text', CalendarEvent :: PROPERTY_END_DATE, Translation :: get('EndDate'), 'id="end_date" style="width:100px;"');
        $form->addGroup($end_date, 'end_date', null, null, false);
        $form->get_renderer()->setGroupElementTemplate('{element}', 'end_date');
        $form->addElement('category');
        
        $form->addElement('category', Translation :: get('RepeatType'));
        $options = CalendarEvent :: get_frequency_options();
        $options[0] = Translation :: get('NoRepeat');
        $options[- 1] = Translation :: get('SelectRepeatType');
        ksort($options);
        $form->addElement('select', CalendarEvent :: PROPERTY_FREQUENCY, Translation :: get('RepeatType'), $options);
        $form->addElement('category');
        
        $form->addElement('html', ResourceManager :: get_instance()->get_resource_html(Path :: namespace_to_full_path(__NAMESPACE__, true) . 'resources/javascript/search.js'));
    }
}
