<?php
namespace repository\content_object\calendar_event;

use common\libraries\Path;
use vevent;

require_once Path :: get_plugin_path() . 'icalcreator/iCalcreator.class.php';
class IcalDefaultExportImplementation extends IcalExportImplementation
{

    public function render()
    {
        $content_object = $this->get_content_object();
        $vevent = new vevent();
        $vevent->setProperty('summary', mb_convert_encoding($content_object->get_title(), 'UTF-8'));

        $vevent->setProperty('dtstart', $this->get_date_in_ical_format($content_object->get_start_date()));
        $vevent->setProperty('dtend', $this->get_date_in_ical_format($content_object->get_end_date()));

        $vevent->setProperty(
            'description',
            str_replace(array("\n", "\r", "\r\n"), '', $content_object->get_description()));

        $vevent->setProperty('organizer', $content_object->get_owner()->get_email());
        $vevent->setProperty('attendee', $content_object->get_owner()->get_email());

        if ($content_object->has_frequency())
        {
            $vevent->setProperty('rrule', $this->get_rrule());
        }

        $ical = $this->get_context()->get_ical();
        $ical->setComponent($vevent);
    }

    public function get_rrule()
    {
        $rrule = array();

        $content_object = $this->get_content_object();
        $frequency = $content_object->get_frequency();
        switch ($frequency)
        {
            case CalendarEvent :: FREQ_DAILY :
                $rrule['FREQ'] = 'DAILY';
                break;
            case CalendarEvent :: FREQ_WEEKLY :
                $rrule['FREQ'] = 'WEEKLY';

                break;
            case CalendarEvent :: FREQ_MONTHLY :
                $rrule['FREQ'] = 'MONTHLY';
                break;
            case CalendarEvent :: FREQ_YEARLY :
                $rrule['FREQ'] = 'YEARLY';
                break;
            case CalendarEvent :: FREQ_BIWEEK :
                $rrule['FREQ'] = 'WEEKLY';
                $rrule['INTERVAL'] = '2';
                break;
            case CalendarEvent :: FREQ_WEEK_DAYS :
                $rrule['FREQ'] = 'DAILY';
                $rrule['BYDAY'] = array(
                    array('DAY' => 'MO'),
                    array('DAY' => 'TU'),
                    array('DAY' => 'WE'),
                    array('DAY' => 'TH'),
                    array('DAY' => 'FR'));
                break;
        }

        if (! $content_object->frequency_is_indefinately())
        {
            $rrule['UNTIL'] = $this->get_date_in_ical_format($content_object->get_until());
        }

        if ($content_object->get_frequency_count() > 0)
        {
            $rrule['COUNT'] = $content_object->get_frequency_count();
        }

        if ($content_object->get_frequency_interval() > 0)
        {
            $rrule['INTERVAL'] = $content_object->get_frequency_interval();
        }

        if ($content_object->get_byday())
        {
            $rrule['BYDAY'] = CalendarEvent :: get_byday_parts($content_object->get_byday());
        }

        if ($content_object->get_bymonthday())
        {
            $rrule['BYMONTHDAY'] = $content_object->get_bymonthday();
        }

        if ($content_object->get_bymonth())
        {
            $rrule['BYMONTH'] = $content_object->get_bymonth();
        }

        return $rrule;
    }

    public function get_date_in_ical_format($date)
    {
        $y = date('Y', $date);
        $m = date('m', $date);
        $d = date('d', $date);
        $h = date('H', $date);
        $M = date('i', $date);
        $s = date('s', $date);

        return $y . $m . $d . 'T' . $h . $M . $s;
    }
}
