<?php
namespace repository\content_object\dailymotion;

use common\libraries\Text;
use common\libraries\Utilities;
use common\libraries\Versionable;
use repository\ContentObject;
use common\libraries\EqualityCondition;
use repository\ExternalInstance;
use common\libraries\AndCondition;

/**
 * $Id: dailymotion.class.php 2010-06-08 package repository.lib.content_object.dailymotion
 *
 * @author Shoira Mukhsinova
 */
class Dailymotion extends ContentObject implements Versionable
{
    const CLASS_NAME = __CLASS__;
    const PROPERTY_URL = 'url';
    const PROPERTY_HEIGHT = 'height';
    const PROPERTY_WIDTH = 'width';

    public function get_url()
    {
        return $this->get_additional_property(self :: PROPERTY_URL);
    }

    public function set_url($url)
    {
        return $this->set_additional_property(self :: PROPERTY_URL, $url);
    }

    public function get_height()
    {
        return $this->get_additional_property(self :: PROPERTY_HEIGHT);
    }

    public function set_height($height)
    {
        return $this->set_additional_property(self :: PROPERTY_HEIGHT, $height);
    }

    public function get_width()
    {
        return $this->get_additional_property(self :: PROPERTY_WIDTH);
    }

    public function set_width($width)
    {
        return $this->set_additional_property(self :: PROPERTY_WIDTH, $width);
    }

    public static function get_additional_property_names()
    {
        return array(self :: PROPERTY_URL, self :: PROPERTY_HEIGHT, self :: PROPERTY_WIDTH);
    }

    public function get_video_id()
    {
        $video_url = $this->get_url();
        $video_url_components = parse_url($video_url);
        $video_query_components = Text :: parse_query_string($video_url_components['query']);

        return str_replace("video", "swf", $video_url_components["path"]);
    }

    public function get_video_url()
    {
        return 'http://www.dailymotion.com/' . $this->get_video_id();
    }

    public static function get_type_name()
    {
        return Utilities :: camelcase_to_underscores(Utilities :: get_classname_from_namespace(self :: CLASS_NAME));
    }

    public static function is_type_available()
    {
        $conditions = array();
        $conditions[] = new EqualityCondition(
            ExternalInstance :: PROPERTY_TYPE,
            \common\extensions\external_repository_manager\Manager :: get_namespace('dailymotion'));
        $conditions[] = new EqualityCondition(ExternalInstance :: PROPERTY_ENABLED, 1);
        $condition = new AndCondition($conditions);

        $external_repositories = \repository\DataManager :: retrieves(ExternalInstance :: class_name(), $condition);
        return $external_repositories->size() == 1;
    }
}
