<?php
namespace repository\content_object\dailymotion\upgrade\v4_0_0;

use common\libraries\CommonDataManager;
use common\libraries\package\Package;
use common\libraries\DataClassProperties;
use common\libraries\DataClassProperty;
use common\libraries\PropertyConditionVariable;
use common\libraries\StaticConditionVariable;
use common\libraries\EqualityCondition;
use common\libraries\SubselectCondition;

/**
 *
 * @author Hans De Bisschop
 * @author Magali Gillard
 */
class Upgrader extends \common\libraries\package\Upgrader
{

    /**
     * Constructor
     */
    public function __construct()
    {
        parent :: __construct(\repository\content_object\dailymotion\DataManager :: get_instance());
    }

    /*
     * (non-PHPdoc) @see \common\libraries\package\Upgrader::run()
     */
    public function run()
    {
        $data_manager = $this->get_data_manager();

        $success = $data_manager->transactional(
            function ($c) use($data_manager)
            {
                $package = Package :: get('repository\content_object\dailymotion');
                $registration = CommonDataManager :: get_registration('repository\content_object\dailymotion');
                $registration->set_version($package->get_version());
                if (! $registration->update())
                {
                    return false;
                }

                $attributes = array('type' => 'text', 'length' => 255, 'notnull' => 1, 'fixed' => 1);
                $success = $data_manager->alter_storage_unit(
                    \common\libraries\DataManager :: ALTER_STORAGE_UNIT_CHANGE,
                    \repository\content_object\dailymotion\Dailymotion :: get_table_name(),
                    \repository\content_object\dailymotion\Dailymotion :: PROPERTY_URL,
                    $attributes);

                if (! $success)
                {
                    return false;
                }

                $properties = new DataClassProperties();
                $properties->add(
                    new DataClassProperty(
                        new PropertyConditionVariable(
                            \repository\ContentObject :: class_name(),
                            \repository\ContentObject :: PROPERTY_TYPE),
                        new StaticConditionVariable('repository\content_object\dailymotion\Dailymotion')));

                $condition = new EqualityCondition(
                    new PropertyConditionVariable(
                        \repository\ContentObject :: class_name(),
                        \repository\ContentObject :: PROPERTY_TYPE),
                    new StaticConditionVariable('dailymotion'));

                if (! \repository\DataManager :: updates(
                    \repository\ContentObject :: class_name(),
                    $properties,
                    $condition))
                {
                    return false;
                }

                $properties = new DataClassProperties();
                $properties->add(
                    new DataClassProperty(
                        new PropertyConditionVariable(
                            \repository\ComplexContentObjectItem :: class_name(),
                            \repository\ComplexContentObjectItem :: PROPERTY_TYPE),
                        new StaticConditionVariable('repository\content_object\dailymotion\ComplexDailymotion')));

                $subselect_condition = new EqualityCondition(
                    new PropertyConditionVariable(
                        \repository\ContentObject :: class_name(),
                        \repository\ContentObject :: PROPERTY_TYPE),
                    new StaticConditionVariable('repository\content_object\dailymotion\Dailymotion'));

                $condition = new SubselectCondition(
                    new PropertyConditionVariable(
                        \repository\ComplexContentObjectItem :: class_name(),
                        \repository\ComplexContentObjectItem :: PROPERTY_REF),
                    new PropertyConditionVariable(
                        \repository\ContentObject :: class_name(),
                        \repository\ContentObject :: PROPERTY_ID),
                    null,
                    $subselect_condition);

                if (! \repository\DataManager :: updates(
                    \repository\ComplexContentObjectItem :: class_name(),
                    $properties,
                    $condition))
                {
                    return false;
                }

                return true;
            });

        return $success;
    }
}
