<?php
namespace repository\content_object\document\upgrade\v4_0_0;

use common\libraries\CommonDataManager;
use common\libraries\OrCondition;
use common\libraries\package\Package;
use common\libraries\DataClassProperties;
use common\libraries\DataClassProperty;
use common\libraries\PropertyConditionVariable;
use common\libraries\StaticConditionVariable;
use common\libraries\EqualityCondition;
use common\libraries\Translation;
use common\libraries\Utilities;
use core\lynx\manager\PackageInstaller;
use common\libraries\PatternMatchCondition;
use common\libraries\DataClassRetrievesParameters;
use core\lynx\manager\PackageRemover;
use common\libraries\DataClassCache;
use common\libraries\Filesystem;
use common\libraries\SubselectCondition;

/**
 *
 * @author Hans De Bisschop
 * @author Magali Gillard
 */
class Upgrader extends \common\libraries\package\Upgrader
{

    /**
     * Constructor
     */
    public function __construct()
    {
        parent :: __construct(\repository\content_object\document\DataManager :: get_instance());
    }

    /*
     * (non-PHPdoc) @see \common\libraries\package\Upgrader::run()
     */
    public function run()
    {
        $data_manager = $this->get_data_manager();

        $success = $data_manager->transactional(
            function ($c) use($data_manager)
            {
                $package = Package :: get('repository\content_object\document');
                $registration = CommonDataManager :: get_registration('repository\content_object\document');
                $registration->set_version($package->get_version());
                if (! $registration->update())
                {
                    return false;
                }

                $properties = new DataClassProperties();
                $properties->add(
                    new DataClassProperty(
                        new PropertyConditionVariable(
                            \repository\ContentObject :: class_name(),
                            \repository\ContentObject :: PROPERTY_TYPE),
                        new StaticConditionVariable('repository\content_object\document\Document')));

                $condition = new EqualityCondition(
                    new PropertyConditionVariable(
                        \repository\ContentObject :: class_name(),
                        \repository\ContentObject :: PROPERTY_TYPE),
                    new StaticConditionVariable('document'));

                if (! \repository\DataManager :: updates(
                    \repository\ContentObject :: class_name(),
                    $properties,
                    $condition))
                {
                    return false;
                }

                return true;
            });

        if ($success)
        {
            $this->add_message(
                \common\libraries\package\Upgrader :: TYPE_NORMAL,
                Translation :: get('UpgradeRegistrationSuccessful', null, Utilities :: COMMON_LIBRARIES));
        }
        else
        {
            $this->add_message(
                \common\libraries\package\Upgrader :: TYPE_ERROR,
                Translation :: get('UpgradeRegistrationFailed', null, Utilities :: COMMON_LIBRARIES));
        }

        $packages = array();
        $packages[] = 'repository\content_object\file';
        $packages[] = 'repository\content_object\page';
        $packages[] = 'repository\content_object\webpage';

        foreach ($packages as $package)
        {
            $installer = new PackageInstaller($package);
            $installer->run();
        }

        $data_manager = \repository\DataManager :: get_instance();
        $upgrader = $this;

        $success = $success && $data_manager->transactional(
            function ($c) use($data_manager, $upgrader)
            {
                try
                {
                    $success = $data_manager->copy_storage_unit('document', 'file', false);
                    if ($success)
                    {
                        $upgrader->add_message(
                            \common\libraries\package\Upgrader :: TYPE_NORMAL,
                            Translation :: get('UpgradeStorageUnitDocumentToFileSuccessful', null,
                                Utilities :: COMMON_LIBRARIES));
                    }
                    else
                    {
                        $upgrader->add_message(
                            \common\libraries\package\Upgrader :: TYPE_ERROR,
                            Translation :: get('UpgradeStorageUnitDocumentToFileFailed', null,
                                Utilities :: COMMON_LIBRARIES));
                    }


                    $properties = new DataClassProperties();
                    $properties->add(
                        new DataClassProperty(
                            new PropertyConditionVariable(
                                \repository\ContentObject :: class_name(),
                                \repository\ContentObject :: PROPERTY_TYPE),
                            new StaticConditionVariable('repository\content_object\file\File')));

                    $condition = new EqualityCondition(
                        new PropertyConditionVariable(
                            \repository\ContentObject :: class_name(),
                            \repository\ContentObject :: PROPERTY_TYPE),
                        new StaticConditionVariable('repository\content_object\document\Document'));

                    $success = $success && (\repository\DataManager :: updates(
                        \repository\ContentObject :: class_name(),
                        $properties,
                        $condition));

                    if($success)
                    {
                        $upgrader->add_message(
                            \common\libraries\package\Upgrader :: TYPE_NORMAL,
                            Translation :: get('UpgradeTypeDocumentToFileSuccessful', null,
                                Utilities :: COMMON_LIBRARIES));
                    }
                    else
                    {
                        $upgrader->add_message(
                            \common\libraries\package\Upgrader :: TYPE_ERROR,
                            Translation :: get('UpgradeTypeDocumentToFileFailed', null,
                                Utilities :: COMMON_LIBRARIES));
                    }

                    $conditions = array();

                    $conditions[] = new PatternMatchCondition(
                        new PropertyConditionVariable(
                            \repository\content_object\file\File :: class_name(),
                            \repository\content_object\file\File :: PROPERTY_FILENAME),
                        '*.htm'
                    );

                    $conditions[] = new PatternMatchCondition(
                        new PropertyConditionVariable(
                            \repository\content_object\file\File :: class_name(),
                            \repository\content_object\file\File :: PROPERTY_FILENAME),
                        '*.html'
                    );

                    $condition = new OrCondition($conditions);

                    $content_objects = \repository\DataManager :: retrieve_content_objects(
                        \repository\content_object\file\File :: class_name(),
                        new DataClassRetrievesParameters($condition));

                    while ($content_object = $content_objects->next_result())
                    {
                        if(!file_exists($content_object->get_full_path()))
                        {
                            $upgrader->add_message(
                                \common\libraries\package\Upgrader::TYPE_WARNING,
                                Translation :: get('FileDoesNotExist', array('FILE' => $content_object->get_full_path()))
                            );

                            continue;
                        }

                        $content_object->set_description(
                            $upgrader->process_document(file_get_contents($content_object->get_full_path())));
                        $content_object->set_type('repository\content_object\page\Page');

                        if (! $content_object->update(false))
                        {
                            $upgrader->add_message(
                                \common\libraries\package\Upgrader :: TYPE_ERROR,
                                Translation :: get('UpgradeProcessFileFailed', null,
                                    Utilities :: COMMON_LIBRARIES));
                            return false;
                        }

                        if (! Filesystem :: remove($content_object->get_full_path()))
                        {
                            $upgrader->add_message(
                                \common\libraries\package\Upgrader :: TYPE_ERROR,
                                Translation :: get('UpgradeProcessFileRemoveFromDiskFailed', null,
                                    Utilities :: COMMON_LIBRARIES));
                            return false;
                        }

                        $condition = new EqualityCondition(
                            new PropertyConditionVariable(
                                \repository\content_object\file\File :: class_name(),
                                \repository\content_object\file\File :: PROPERTY_ID),
                            new StaticConditionVariable($content_object->get_id()));

                        if (! \repository\DataManager :: deletes(
                            \repository\content_object\file\File :: class_name(),
                            $condition))
                        {
                            $upgrader->add_message(
                                \common\libraries\package\Upgrader :: TYPE_ERROR,
                                Translation :: get('UpgradeProcessFileDeleteFileContentObjectFailed', null,
                                    Utilities :: COMMON_LIBRARIES));
                            return false;
                        }
                    }

                    $success = $success && (\repository\DataManager :: deletes
                        (\repository\content_object\document\Document ::
                        class_name(), null));

                    if($success)
                    {
                        $upgrader->add_message(
                            \common\libraries\package\Upgrader :: TYPE_NORMAL,
                            Translation :: get('UpgradeDeleteDocumentSuccessful', null,
                                Utilities :: COMMON_LIBRARIES));
                    }
                    else
                    {
                        $upgrader->add_message(
                            \common\libraries\package\Upgrader :: TYPE_ERROR,
                            Translation :: get('UpgradeDeleteDocumentFailed', null,
                                Utilities :: COMMON_LIBRARIES));
                    }

                    $properties = new DataClassProperties();
                    $properties->add(
                        new DataClassProperty(
                            new PropertyConditionVariable(
                                \repository\ComplexContentObjectItem :: class_name(),
                                \repository\ComplexContentObjectItem :: PROPERTY_TYPE),
                            new StaticConditionVariable('repository\content_object\file\ComplexFile')));

                    $subselect_condition = new EqualityCondition(
                        new PropertyConditionVariable(
                            \repository\ContentObject :: class_name(),
                            \repository\ContentObject :: PROPERTY_TYPE),
                        new StaticConditionVariable('repository\content_object\file\File'));

                    $condition = new SubselectCondition(
                        new PropertyConditionVariable(
                            \repository\ComplexContentObjectItem :: class_name(),
                            \repository\ComplexContentObjectItem :: PROPERTY_REF),
                        new PropertyConditionVariable(
                            \repository\ContentObject :: class_name(),
                            \repository\ContentObject :: PROPERTY_ID),
                        null,
                        $subselect_condition);

                    $success = $success && (\repository\DataManager :: updates(
                        \repository\ComplexContentObjectItem :: class_name(),
                        $properties,
                        $condition));

                    if($success)
                    {
                        $upgrader->add_message(
                            \common\libraries\package\Upgrader :: TYPE_NORMAL,
                            Translation :: get('UpgradeComplexFileSuccessful', null,
                                Utilities :: COMMON_LIBRARIES));
                    }
                    else
                    {
                        $upgrader->add_message(
                            \common\libraries\package\Upgrader :: TYPE_ERROR,
                            Translation :: get('UpgradeComplexContentObjectItemTypeFailed', null,
                                Utilities :: COMMON_LIBRARIES));
                    }

                    $properties = new DataClassProperties();
                    $properties->add(
                        new DataClassProperty(
                            new PropertyConditionVariable(
                                \repository\ComplexContentObjectItem :: class_name(),
                                \repository\ComplexContentObjectItem :: PROPERTY_TYPE),
                            new StaticConditionVariable('repository\content_object\webpage\ComplexWebpage')));

                    $subselect_condition = new EqualityCondition(
                        new PropertyConditionVariable(
                            \repository\ContentObject :: class_name(),
                            \repository\ContentObject :: PROPERTY_TYPE),
                        new StaticConditionVariable('repository\content_object\webpage\Webpage'));

                    $condition = new SubselectCondition(
                        new PropertyConditionVariable(
                            \repository\ComplexContentObjectItem :: class_name(),
                            \repository\ComplexContentObjectItem :: PROPERTY_REF),
                        new PropertyConditionVariable(
                            \repository\ContentObject :: class_name(),
                            \repository\ContentObject :: PROPERTY_ID),
                        null,
                        $subselect_condition);

                    $success = $success && ( \repository\DataManager :: updates(
                        \repository\ComplexContentObjectItem :: class_name(),
                        $properties,
                        $condition));

                    if($success)
                    {
                        $upgrader->add_message(
                            \common\libraries\package\Upgrader :: TYPE_NORMAL,
                            Translation :: get('UpgradeComplexWebPagesSuccessful', null,
                                Utilities :: COMMON_LIBRARIES));
                    }
                    else
                    {
                        $upgrader->add_message(
                            \common\libraries\package\Upgrader :: TYPE_ERROR,
                            Translation :: get('UpgradeComplexContentObjectItemTypeFailed', null,
                                Utilities :: COMMON_LIBRARIES));
                    }

                    return $success;
                }
                catch(\Exception $e)
                {
                    echo $e->getMessage() . "\n";
                    echo $e->getTraceAsString();

                    return false;
                }
            });

        if ($success)
        {
            $this->add_message(
                \common\libraries\package\Upgrader :: TYPE_NORMAL,
                Translation :: get('UpgradeDatabaseSuccessful', null, Utilities :: COMMON_LIBRARIES));
        }
        else
        {
            $this->add_message(
                \common\libraries\package\Upgrader :: TYPE_ERROR,
                Translation :: get('UpgradeDatabaseFailed', null, Utilities :: COMMON_LIBRARIES));
        }

        DataClassCache :: reset();

        $package_remover = new PackageRemover('repository\content_object\document');
        $success = $success && $package_remover->run();

        return $success;
    }

    public function process_document($file)
    {
        $dom_document = new \DOMDocument('1.0', 'UTF-8');
        $dom_document->loadHTML('<?xml encoding="UTF-8">' . $file);
        $dom_xpath = new \DOMXPath($dom_document);

        // Convert images to resources
        $images = $dom_xpath->query('//img');

        foreach ($images as $image_node)
        {
            $url = $image_node->getAttribute('src');
            $url_parts = parse_url($url);
            parse_str($url_parts['query'], $query_parts);

            if (isset($query_parts['application']) && $query_parts['application'] == 'repository' &&
                 isset($query_parts['go']) && $query_parts['go'] == 'document_downloader' &&
                 isset($query_parts['object']) && is_numeric($query_parts['object']))
            {
                $resource = $dom_document->createElement('resource');

                $parent_node = $image_node->parentNode;
                $parent_node->insertBefore($resource, $image_node);

                $resource->setAttribute('source', $query_parts['object']);
                $resource->setAttribute('type', 'file');
                $resource->setAttribute('width', $image_node->getAttribute('width'));
                $resource->setAttribute('height', $image_node->getAttribute('height'));

                $parent_node->removeChild($image_node);
            }
        }

        // Convert files to resources
        $links = $dom_xpath->query('//a');

        foreach ($links as $link_node)
        {
            $url = $link_node->getAttribute('href');
            $url_parts = parse_url($url);
            parse_str($url_parts['query'], $query_parts);

            if (isset($query_parts['application']) && $query_parts['application'] == 'repository' &&
                 isset($query_parts['go']) && $query_parts['go'] == 'document_downloader' &&
                 isset($query_parts['object']) && is_numeric($query_parts['object']))
            {
                $resource = $dom_document->createElement('resource');

                $parent_node = $link_node->parentNode;
                $parent_node->insertBefore($resource, $link_node);

                $resource->setAttribute('source', $query_parts['object']);
                $resource->setAttribute('type', 'file');

                $parent_node->removeChild($link_node);
            }
        }

        // Convert pdf / htm(l) to resources
        $iframes = $dom_xpath->query('//iframe');

        foreach ($iframes as $iframe_node)
        {
            $url = $iframe_node->getAttribute('src');
            $url_parts = parse_url($url);
            parse_str($url_parts['query'], $query_parts);

            if (isset($query_parts['application']) && $query_parts['application'] == 'repository' &&
                isset($query_parts['go']) && $query_parts['go'] == 'document_downloader' &&
                isset($query_parts['object']) && is_numeric($query_parts['object']))
            {
                $resource = $dom_document->createElement('resource');

                $parent_node = $iframe_node->parentNode;
                $parent_node->insertBefore($resource, $iframe_node);

                $resource->setAttribute('source', $query_parts['object']);
                $resource->setAttribute('type', 'file');

                $parent_node->removeChild($iframe_node);
            }
        }

        // Convert video / audio / flv to resources
        $objects = $dom_xpath->query('//object');

        for($i = $objects->length - 1; $i >= 0; $i--)
        {
            $object_node = $objects->item($i);

            $params = $dom_xpath->query('//param', $object_node);

            foreach ($params as $param_node)
            {
                if ($param_node->getAttribute('name') == 'url')
                {
                    $url = $param_node->getAttribute('value');
                    $url_parts = parse_url($url);
                    parse_str($url_parts['query'], $query_parts);

                    if (isset($query_parts['application']) && $query_parts['application'] == 'repository' &&
                         isset($query_parts['go']) && $query_parts['go'] == 'document_downloader' &&
                         isset($query_parts['object']) && is_numeric($query_parts['object']))
                    {
                        $resource = $dom_document->createElement('resource');

                        $parent_node = $object_node->parentNode;
                        if(!$parent_node)
                        {
                            continue;
                        }

                        $parent_node->insertBefore($resource, $object_node);

                        $resource->setAttribute('source', $query_parts['object']);
                        $resource->setAttribute('type', 'file');

                        $parent_node->removeChild($object_node);
                    }
                }
            }
        }

        // Convert mp3 to resources
        $objects = $dom_xpath->query('//object');

        for($i = $objects->length - 1; $i >= 0; $i--)
        {
            $object_node = $objects->item($i);

            $params = $dom_xpath->query('//param', $object_node);

            foreach ($params as $param_node)
            {
                if ($param_node->getAttribute('name') == 'FlashVars')
                {
                    $value = $param_node->getAttribute('value');
                    parse_str($value, $mp3);
                    $url_parts = parse_url($mp3['mp3']);
                    parse_str($url_parts['query'], $query_parts);

                    if (isset($query_parts['application']) && $query_parts['application'] == 'repository' &&
                         isset($query_parts['go']) && $query_parts['go'] == 'document_downloader' &&
                         isset($query_parts['object']) && is_numeric($query_parts['object']))
                    {
                        $resource = $dom_document->createElement('resource');

                        $parent_node = $object_node->parentNode;

                        if(!$parent_node)
                        {
                            continue;
                        }

                        $parent_node->insertBefore($resource, $object_node);

                        $resource->setAttribute('source', $query_parts['object']);
                        $resource->setAttribute('type', 'file');

                        $parent_node->removeChild($object_node);
                    }
                }
            }
        }

        // Convert swf to resources
        $embeds = $dom_xpath->query('//object/embed');

        for($i = $embeds->length - 1; $i >= 0; $i--)
        {
            $embed_node = $embeds->item($i);

            $url = $embed_node->getAttribute('src');
            $url_parts = parse_url($url);
            parse_str($url_parts['query'], $query_parts);

            if (isset($query_parts['application']) && $query_parts['application'] == 'repository' &&
                 isset($query_parts['go']) && $query_parts['go'] == 'document_downloader' &&
                 isset($query_parts['object']) && is_numeric($query_parts['object']))
            {
                $resource = $dom_document->createElement('resource');

                $object_parent_node = $embed_node->parentNode;
                $parent_node = $object_parent_node->parentNode;

                if(!$parent_node)
                {
                    continue;
                }

                $parent_node->insertBefore($resource, $object_parent_node);

                $resource->setAttribute('source', $query_parts['object']);
                $resource->setAttribute('type', 'file');

                $width = $embed_node->getAttribute('width');

                if($width)
                {
                    $resource->setAttribute('width', $width);
                }

                $height = $embed_node->getAttribute('height');

                if($height)
                {
                    $resource->setAttribute('height', $height);
                }

                $parent_node->removeChild($object_parent_node);
            }
        }

        return $dom_document->saveHTML();
    }
}