<?php
namespace repository\content_object\forum;

use common\libraries\SubselectCondition;
use common\libraries\Request;
use common\libraries\Translation;
use common\libraries\Utilities;
use common\libraries\EqualityCondition;
use common\libraries\AndCondition;
use repository\ComplexBuilder;
use repository\ComplexContentObjectItem;
use repository\ContentObject;
use repository\RepositoryManager;

/**
 * $Id: browser.class.php 200 2009-11-13 12:30:04Z kariboe $
 * 
 * @package repository.lib.complex_builder.glossary.component
 */
class ForumBuilderMoverComponent extends ForumBuilder
{

    public function run()
    {
        $id = Request :: get(ComplexBuilder :: PARAM_SELECTED_COMPLEX_CONTENT_OBJECT_ITEM_ID);
        $parent_complex_content_object_item = Request :: get(ComplexBuilder :: PARAM_COMPLEX_CONTENT_OBJECT_ITEM_ID);
        $direction = Request :: get(ComplexBuilder :: PARAM_DIRECTION);
        $success = true;
        
        if (isset($id))
        {
            $complex_content_object_item = \repository\DataManager :: retrieve_complex_content_object_item($id);
            $parent = $complex_content_object_item->get_parent();
            
            $conditions = array();
            $conditions[] = new EqualityCondition(ComplexContentObjectItem :: PROPERTY_PARENT, $parent);
            $subcondition = new EqualityCondition(ContentObject :: PROPERTY_TYPE, Forum :: class_name());
            $conditions[] = new SubselectCondition(ComplexContentObjectItem :: PROPERTY_REF, 
                    ContentObject :: PROPERTY_ID, 'content_object', $subcondition);
            $condition = new AndCondition($conditions);
            $max = \repository\DataManager :: count_complex_content_object_items(ComplexContentObjectItem :: class_name(), 
                    $condition);
            
            $display_order = $complex_content_object_item->get_display_order();
            $new_place = ($display_order + ($direction == RepositoryManager :: PARAM_DIRECTION_UP ? - 1 : 1));
            if ($new_place > 0 && $new_place <= $max)
            {
                $complex_content_object_item->set_display_order($new_place);
                $conditions[] = new EqualityCondition(ComplexContentObjectItem :: PROPERTY_DISPLAY_ORDER, $new_place, 
                        ComplexContentObjectItem :: get_table_name());
                $conditions[] = new EqualityCondition(ComplexContentObjectItem :: PROPERTY_PARENT, $parent, 
                        ComplexContentObjectItem :: get_table_name());
                $subcondition = new EqualityCondition(ContentObject :: PROPERTY_TYPE, Forum :: class_name());
                $conditions[] = new SubselectCondition(ComplexContentObjectItem :: PROPERTY_REF, 
                        ContentObject :: PROPERTY_ID, 'content_object', $subcondition);
                $condition = new AndCondition($conditions);
                $items = \repository\DataManager :: retrieve_complex_content_object_items(
                        ComplexContentObjectItem :: class_name(), $condition);
                $new_complex_content_object_item = $items->next_result();
                $new_complex_content_object_item->set_display_order($display_order);
                
                if (! $complex_content_object_item->update() || ! $new_complex_content_object_item->update())
                {
                    $success = false;
                }
            }
            
            if ($parent == $root)
            {
                $parent = null;
            }
            
            $message = Translation :: get(($success ? 'ObjectMoved' : 'ObjectNotMoved'),
                    array('OBJECT' => Translation :: get('ComplexContentObjectItems')), Utilities :: COMMON_LIBRARIES);
            
            $this->redirect($message, ($success ? false : true), 
                    array(ComplexBuilder :: PARAM_BUILDER_ACTION => ComplexBuilder :: ACTION_BROWSE, 
                            ComplexBuilder :: PARAM_COMPLEX_CONTENT_OBJECT_ITEM_ID => $parent_complex_content_object_item));
        }
        else
        {
            $this->display_error_page(
                    htmlentities(Translation :: get('NoObjectSelected', null, Utilities :: COMMON_LIBRARIES)));
        }
    }
}
