<?php
namespace repository\content_object\forum;

use common\libraries\Breadcrumb;
use common\libraries\BreadcrumbTrail;
use common\libraries\DelegateComponent;
use common\libraries\Translation;
use common\libraries\Utilities;
use common\libraries\EqualityCondition;
use common\libraries\Display;
use repository\ComplexContentObjectItem;
use repository\ComplexDisplay;
use repository\ContentObjectForm;

/**
 * $Id: forum_subforum_editor.class.php 200 2009-11-13 12:30:04Z kariboe $
 * 
 * @package repository.lib.complex_display.forum.component
 */
class ForumComplexDisplayForumSubforumEditorComponent extends ForumComplexDisplay implements DelegateComponent
{

    public function run()
    {
        if ($this->get_parent()->is_allowed(EDIT_RIGHT))
        {
            $selected_complex_content_object_item = $this->get_selected_complex_content_object_item();
            
            $url = $this->get_url(
                    array(ComplexDisplay :: PARAM_DISPLAY_ACTION => ForumComplexDisplay :: ACTION_EDIT_SUBFORUM, 
                            ComplexDisplay :: PARAM_COMPLEX_CONTENT_OBJECT_ITEM_ID => $this->get_complex_content_object_item_id(), 
                            ComplexDisplay :: PARAM_SELECTED_COMPLEX_CONTENT_OBJECT_ITEM_ID => $selected_complex_content_object_item->get_id()));
            
            $forum_object = \repository\DataManager :: retrieve_content_object(
                    $selected_complex_content_object_item->get_ref());
            
            BreadcrumbTrail :: get_instance()->add(
                    new Breadcrumb(
                            $this->get_url(
                                    array(
                                            ComplexDisplay :: PARAM_DISPLAY_ACTION => ForumComplexDisplay :: ACTION_VIEW_FORUM, 
                                            ComplexDisplay :: PARAM_COMPLEX_CONTENT_OBJECT_ITEM_ID => null)), 
                            $this->get_root_content_object()->get_title()));
            
            if ($this->get_complex_content_object_item())
            {
                
                $forums_with_key_cloi = array();
                $forums_with_key_cloi = $this->retrieve_children_from_root_to_cloi(
                        $this->get_root_content_object()->get_id(), $this->get_complex_content_object_item()->get_id());
                
                if ($forums_with_key_cloi)
                {
                    
                    foreach ($forums_with_key_cloi as $key => $value)
                    {
                        
                        BreadcrumbTrail :: get_instance()->add(
                                new Breadcrumb(
                                        $this->get_url(
                                                array(ComplexDisplay :: PARAM_COMPLEX_CONTENT_OBJECT_ITEM_ID => $key)), 
                                        $value->get_title()));
                    }
                }
                else
                {
                    throw new \Exception('The forum you requested has not been found');
                }
            }
            BreadcrumbTrail :: get_instance()->add(
                    new Breadcrumb($this->get_url(), 
                            Translation :: get('SubforumEditor', array('SUBFORUM' => $forum_object->get_title()))));
            $form = ContentObjectForm :: factory(ContentObjectForm :: TYPE_EDIT, $forum_object, 'edit', 'post', $url);
            
            if ($form->validate())
            {
                $success = $form->update_content_object();
                if ($form->is_version())
                {
                    $old_id = $selected_complex_content_object_item->get_ref();
                    $new_id = $forum_object->get_latest_version()->get_id();
                    $selected_complex_content_object_item->set_ref($new_id);
                    $selected_complex_content_object_item->update();
                    
                    $children = \repository\DataManager :: retrieve_complex_content_object_items(
                            ComplexContentObjectItem :: class_name(), 
                            new EqualityCondition(ComplexContentObjectItem :: PROPERTY_PARENT, $old_id, 
                                    ComplexContentObjectItem :: get_table_name()));
                    while ($child = $children->next_result())
                    {
                        $child->set_parent($new_id);
                        $child->update();
                    }
                }
                
                $this->my_redirect($success);
            }
            else
            {
                $this->display_header();
                $form->display();
                $this->display_footer();
            }
        }
        else
        {
            Display :: not_allowed();
        }
    }

    private function my_redirect($success)
    {
        $message = htmlentities(
                Translation :: get(($success ? 'ObjectUpdated' : 'ObjectNotUpdated'), 
                        array('OBJECT' => Translation :: get('Subforum')), Utilities :: COMMON_LIBRARIES));
        
        $params = array();
        $params[ComplexDisplay :: PARAM_DISPLAY_ACTION] = ForumComplexDisplay :: ACTION_VIEW_FORUM;
        $params[ComplexDisplay :: PARAM_COMPLEX_CONTENT_OBJECT_ITEM_ID] = $this->get_complex_content_object_item_id();
        
        $this->redirect($message, ($success ? false : true), $params);
    }
}
