<?php
namespace repository\content_object\forum;

use common\libraries\Breadcrumb;
use common\libraries\BreadcrumbTrail;
use common\libraries\DelegateComponent;
use common\libraries\Translation;
use common\libraries\Utilities;


use repository\ComplexDisplay;
use repository\content_object\forum_topic\ForumTopic;
use repository\ComplexContentObjectItem;

/**
 * $Id: forum_topic_creator.class.php 205 2009-11-13 12:57:33Z vanpouckesven $
 * 
 * @package repository.lib.complex_display.forum.component
 */
class ForumComplexDisplayForumTopicCreatorComponent extends ForumComplexDisplay implements 
        \repository\viewer\ViewerInterface, 
        DelegateComponent
{

    public function run()
    {
        if (! \repository\viewer\Manager :: is_ready_to_be_published())
        {
            BreadcrumbTrail :: get_instance()->add(
                    new Breadcrumb(
                            $this->get_url(
                                    array(
                                            ComplexDisplay :: PARAM_DISPLAY_ACTION => ForumComplexDisplay :: ACTION_VIEW_FORUM, 
                                            ComplexDisplay :: PARAM_COMPLEX_CONTENT_OBJECT_ITEM_ID => null)), 
                            $this->get_root_content_object()->get_title()));
            
            if ($this->get_complex_content_object_item())
            {
                
                $forums_with_key_cloi = $this->retrieve_children_from_root_to_cloi(
                        $this->get_root_content_object()->get_id(), $this->get_complex_content_object_item()->get_id());
                
                if ($forums_with_key_cloi)
                {
                    
                    foreach ($forums_with_key_cloi as $key => $value)
                    {
                        
                        BreadcrumbTrail :: get_instance()->add(
                                new Breadcrumb(
                                        $this->get_url(
                                                array(ComplexDisplay :: PARAM_COMPLEX_CONTENT_OBJECT_ITEM_ID => $key)), 
                                        $value->get_title()));
                    }
                }
                else
                {
                    throw new \Exception('The forum you requested has not been found');
                }
            }
            
            $repository_viewer = \repository\viewer\Manager :: construct($this);
            $repository_viewer->set_parameter(ComplexDisplay :: PARAM_DISPLAY_ACTION, 
                    ForumComplexDisplay :: ACTION_CREATE_TOPIC);
            $repository_viewer->set_parameter(ComplexDisplay :: PARAM_COMPLEX_CONTENT_OBJECT_ITEM_ID, 
                    $this->get_complex_content_object_item_id());
            $repository_viewer->run();
        }
        else
        {
            $object_id = \repository\viewer\Manager :: get_selected_objects();
            
            if (! is_array($object_id))
            {
                $object_id = array($object_id);
            }
            
            $failures = 0;
            foreach ($object_id as $key => $value)
            {
                $cloi = ComplexContentObjectItem :: factory(ForumTopic :: class_name());
                
                if ($this->get_complex_content_object_item())
                {
                    $cloi->set_parent($this->get_complex_content_object_item()->get_ref());
                }
                else
                {
                    $cloi->set_parent($this->get_root_content_object_id());
                }
                
                $cloi->set_ref($value);
                $cloi->set_user_id($this->get_user_id());

                $cloi->set_display_order(
                    \repository\DataManager :: select_next_display_order(
                        $cloi->get_parent(), ForumTopic :: class_name()
                    )
                );

                if (! $cloi->create())
                {
                    $failures ++;
                }
            }
            
            $this->my_redirect($failures == 0);
        }
    }

    private function my_redirect($success)
    {
        $message = htmlentities(
                Translation :: get(($success ? 'ObjectCreated' : 'ObjectNotCreated'), 
                        array('OBJECT' => Translation :: get('ForumTopic')), Utilities :: COMMON_LIBRARIES));
        
        $params = array();
        $params[ComplexDisplay :: PARAM_DISPLAY_ACTION] = ForumComplexDisplay :: ACTION_VIEW_FORUM;
        $params[ComplexDisplay :: PARAM_COMPLEX_CONTENT_OBJECT_ITEM_ID] = $this->get_complex_content_object_item_id();
        
        $this->redirect($message, ($success ? false : true), $params);
    }

    public function get_allowed_content_object_types()
    {
        return array(ForumTopic :: class_name());
    }
}
