<?php
namespace repository\content_object\forum\upgrade\v4_0_0;

use common\libraries\CommonDataManager;
use common\libraries\DataClassProperties;
use common\libraries\DataClassProperty;
use common\libraries\EqualityCondition;
use common\libraries\package\Package;
use common\libraries\PropertyConditionVariable;
use common\libraries\StaticConditionVariable;
use common\libraries\SubselectCondition;
use common\libraries\Translation;
use common\libraries\Utilities;
use repository\ComplexContentObjectItem;
use repository\content_object\assignment\Assignment;
use repository\content_object\forum\Forum;
use repository\ContentObject;

/**
 *
 * @author Pieterjan Broekaert
 */
class Upgrader extends \common\libraries\package\Upgrader
{

    protected static $CONTENT_OBJECT_CLASS = Forum :: CLASS_NAME; // Change this value!

    protected function get_namespace()
    {
        return call_user_func(self :: $CONTENT_OBJECT_CLASS .'::context');
    }

    protected function get_class_name()
    {
        return call_user_func(self :: $CONTENT_OBJECT_CLASS .'::class_name');
    }

    protected function get_type_name()
    {
        return call_user_func(self :: $CONTENT_OBJECT_CLASS .'::get_type_name');
    }

    /**
     * Constructor
     */
    public function __construct()
    {
        parent :: __construct(\repository\content_object\forum\DataManager :: get_instance());
    }

    /*
     * (non-PHPdoc) @see \common\libraries\package\Upgrader::run()
     */
    public function run()
    {
        $data_manager = $this->get_data_manager();
        $upgrader = $this;

        $success = $data_manager->transactional(
            function ($c) use($data_manager, $upgrader)
            {
                $success = $upgrader->upgrade_db($data_manager);

                if($success)
                {
                    $upgrader->add_message(\common\libraries\package\Upgrader :: TYPE_NORMAL, Translation :: get('UpgradeDatabaseSuccessful', null, Utilities :: COMMON_LIBRARIES));
                }
                else
                {
                    $upgrader->add_message(\common\libraries\package\Upgrader :: TYPE_ERROR, Translation :: get('UpgradeDatabaseFailed', null, Utilities :: COMMON_LIBRARIES));
                }

                if(!$success)
                {
                    return false;
                }

                $success = $upgrader->update_db_content($data_manager);

                if($success)
                {
                    $upgrader->add_message(\common\libraries\package\Upgrader :: TYPE_NORMAL, Translation :: get('UpdateDatabaseContentSuccessful', null, Utilities :: COMMON_LIBRARIES));
                }
                else
                {
                    $upgrader->add_message(\common\libraries\package\Upgrader :: TYPE_ERROR, Translation :: get('UpdateDatabaseContentFailed', null, Utilities :: COMMON_LIBRARIES));
                }

                if(!$success)
                {
                    return false;
                }

                $success = $upgrader->upgrade_registration($data_manager);

                if($success)
                {
                    $upgrader->add_message(\common\libraries\package\Upgrader :: TYPE_NORMAL, Translation :: get('UpgradeRegistrationSuccessful', null, Utilities :: COMMON_LIBRARIES));
                }
                else
                {
                    $upgrader->add_message(\common\libraries\package\Upgrader :: TYPE_ERROR, Translation :: get('UpgradeRegistrationFailed', null, Utilities :: COMMON_LIBRARIES));
                }

                return $success;

            });

        if ($success)
        {
            $this->add_message(self :: TYPE_NORMAL, Translation :: get('UpgradeDatabaseSuccessful', null, Utilities :: COMMON_LIBRARIES));
        }
        else
        {
            $this->add_message(self :: TYPE_ERROR, Translation :: get('UpgradeDatabaseFailed', null, Utilities :: COMMON_LIBRARIES));
        }

        return $success;
    }

    /**
     * Upgrade the table structures
     * @param $data_manager
     * @return bool
     */
    public function upgrade_db($data_manager)
    {
        return true;
    }

    /**
     * Performs the database content upgrade.
     *
     * @param type $data_manager
     * @return boolean
     */
    public function update_db_content($data_manager)
    {
        $properties = new DataClassProperties();
        $properties->add(
            new DataClassProperty(
                new PropertyConditionVariable(
                    ContentObject :: class_name(),
                    ContentObject :: PROPERTY_TYPE),
                new StaticConditionVariable($this->get_class_name()))
        );

        $condition = new EqualityCondition(
            new PropertyConditionVariable(
                ContentObject :: class_name(),
                ContentObject :: PROPERTY_TYPE),
            new StaticConditionVariable($this->get_type_name())
        );

        if (! \repository\DataManager :: updates(
            ContentObject :: class_name(),
            $properties,
            $condition))
        {
            return false;
        }

        $properties = new DataClassProperties();
        $properties->add(
            new DataClassProperty(
                new PropertyConditionVariable(
                    ComplexContentObjectItem :: class_name(),
                    ComplexContentObjectItem :: PROPERTY_TYPE),
                new StaticConditionVariable(
                    'repository\content_object\forum\ComplexForum')));

        $subselect_condition = new EqualityCondition(
            new PropertyConditionVariable(
                \repository\ContentObject :: class_name(),
                \repository\ContentObject :: PROPERTY_TYPE),
            new StaticConditionVariable(
                'repository\content_object\forum\Forum'));

        $condition = new SubselectCondition(
            new PropertyConditionVariable(
                ComplexContentObjectItem :: class_name(),
                ComplexContentObjectItem :: PROPERTY_REF),
            new PropertyConditionVariable(
                \repository\ContentObject :: class_name(),
                \repository\ContentObject :: PROPERTY_ID),
            null,
            $subselect_condition);

        if (! \repository\DataManager :: updates(
            ComplexContentObjectItem :: class_name(),
            $properties,
            $condition))
        {
            return false;
        }

        return true;
    }

    /**
     * Upgrades the package registration.
     *
     * @return boolean
     */
    public function upgrade_registration()
    {
        $package = Package :: get($this->get_namespace());
        $registration = CommonDataManager :: get_registration($this->get_namespace());
        $registration->set_version($package->get_version());
        if (! $registration->update())
        {
            return false;
        }
        return true;
    }
}
