<?php
namespace repository\content_object\glossary;



use common\libraries\Breadcrumb;
use common\libraries\BreadcrumbTrail;
use common\libraries\DelegateComponent;
use common\libraries\Translation;
use common\libraries\Utilities;
use repository\content_object\glossary_item\GlossaryItem;
use repository\ComplexContentObjectItem;

/**
 * Description of glossary_item_creator
 *
 * @author Anthony Hurst (Hogeschool Gent)
 */
class GlossaryComplexDisplayGlossaryItemCreatorComponent extends GlossaryComplexDisplay implements
        \repository\viewer\ViewerInterface,
        DelegateComponent
{

    public function run()
    {
        if (! \repository\viewer\Manager :: is_ready_to_be_published())
        {
            BreadcrumbTrail :: get_instance()->add(
                    new Breadcrumb(
                            $this->get_url(
                                    array(self :: PARAM_DISPLAY_ACTION => self :: ACTION_VIEW_COMPLEX_CONTENT_OBJECT,
                                            self :: PARAM_COMPLEX_CONTENT_OBJECT_ITEM_ID => null)),
                            $this->get_root_content_object()->get_title()));

            $repository_viewer = \repository\viewer\Manager :: construct($this);
            $repository_viewer->set_parameter(self :: PARAM_DISPLAY_ACTION, self :: ACTION_CREATE_GLOSSARY_ITEM);
            $repository_viewer->set_parameter(self :: PARAM_COMPLEX_CONTENT_OBJECT_ITEM_ID,
                    $this->get_complex_content_object_item_id());
            $repository_viewer->run();
        }
        else
        {
            $object_ids = \repository\viewer\Manager :: get_selected_objects();
            if (! is_array($object_ids))
            {
                $object_ids = array($object_ids);
            }

            $failures = 0;
            foreach ($object_ids as $object_id)
            {
                $ccoi = ComplexContentObjectItem :: factory(GlossaryItem :: class_name());
                $ccoi->set_parent($this->get_root_content_object_id());
                $ccoi->set_ref($object_id);
                $ccoi->set_user_id($this->get_user_id());
                $ccoi->set_display_order(\repository\DataManager :: select_next_display_order($ccoi->get_parent()));
                if (! $ccoi->create())
                {
                    $failures ++;
                }
            }

            $this->my_redirect($failures == 0);
        }
    }

    private function my_redirect($success)
    {
        $message = htmlentities(
                Translation :: get(($success ? 'ObjectCreated' : 'ObjectNotCreated'),
                        array('OBJECT' => Translation :: get('GlossaryItem')), Utilities :: COMMON_LIBRARIES));

        $parameters = array();
        $parameters[self :: PARAM_DISPLAY_ACTION] = self :: ACTION_VIEW_COMPLEX_CONTENT_OBJECT;
        $parameters[self :: PARAM_COMPLEX_CONTENT_OBJECT_ITEM_ID] = $this->get_complex_content_object_item_id();

        $this->redirect($message, ! $success, $parameters);
    }

    public function get_allowed_content_object_types()
    {
        return array(GlossaryItem :: class_name());
    }
}
