<?php
namespace repository\content_object\glossary;

use repository\DefaultContentObjectTableCellRenderer;
use common\libraries\Translation;
use common\libraries\ToolbarItem;
use common\libraries\Toolbar;
use common\libraries\Theme;
use repository\content_object\glossary_item\GlossaryItem;
use common\libraries\Utilities;

/**
 * $Id: glossary_viewer_table_cell_renderer.class.php 200 2009-11-13 12:30:04Z kariboe $
 *
 * @package repository.lib.complex_display.glossary.component.glossary_viewer
 */
/**
 * This class is a cell renderer for a publication candidate table
 */
class GlossaryViewerTableCellRenderer extends DefaultContentObjectTableCellRenderer
{

    private $table_actions;

    private $renderer;

    private $dm;

    private $glossary_item;

    /**
     * Constructor.
     *
     * @param string $publish_url_format URL for publishing the selected learning object.
     * @param string $edit_and_publish_url_format URL for editing and publishing the selected learning object.
     */
    public function __construct($renderer)
    {
        $this->table_actions = array();
        $this->renderer = $renderer;
    }

    /*
     * Inherited
     */
    public function render_cell($column, $glossary_item)
    {
        $component = $this->renderer->get_component();

        if ($column === GlossaryViewerTableColumnModel :: get_action_column())
        {
            return $this->get_actions($glossary_item);
        }

        if (! $this->glossary_item || $this->glossary_item->get_id() != $glossary_item->get_ref())
            $this->glossary_item = \repository\DataManager :: retrieve_content_object($glossary_item->get_ref(),
                    GlossaryItem :: class_name());

        switch ($column->get_name())
        {
            case GlossaryItem :: PROPERTY_TITLE :
                return $this->glossary_item->get_title();
            case GlossaryItem :: PROPERTY_DESCRIPTION :
                return $this->glossary_item->get_description();
        }
    }

    public function get_actions($glossary_item)
    {
        $component = $this->renderer->get_component();

        $toolbar = new Toolbar();

        if ($component->is_allowed_to_edit_content_object())
        {
            $toolbar->add_item(
                new ToolbarItem(
                    Translation :: get('Edit', null, Utilities :: COMMON_LIBRARIES),
                    Theme :: get_common_image_path() . 'action_edit.png',
                    $component->get_complex_content_object_item_update_url($glossary_item),
                    ToolbarItem :: DISPLAY_ICON
                )
            );
        }

        if ($component->is_allowed_to_delete_child())
        {
            $toolbar->add_item(
                new ToolbarItem(Translation :: get('Delete', null, Utilities :: COMMON_LIBRARIES),
                    Theme :: get_common_image_path() . 'action_delete.png',
                    $component->get_complex_content_object_item_delete_url($glossary_item),
                    ToolbarItem :: DISPLAY_ICON, true
                )
            );
        }

        return $toolbar->as_html();
    }
}
