<?php
namespace repository\content_object\learning_path;

class Autoloader
{

    /**
     * The array mapping class names to paths
     *
     * @var multitype:string
     */
     private static $map = array(
         'Autoloader' => '/autoloader.class.php',
         'LearningPathDifference' => '/lib/difference.class.php',
         'LearningPathDifferenceDisplay' => '/lib/difference_display.class.php',
         'LearningPathDisplay' => '/lib/display.class.php',
         'LearningPathBuilder' => '/lib/builder/learning_path_builder.class.php',
         'LearningPathBuilderAttachmentViewerComponent' => '/lib/builder/component/attachment_viewer.class.php',
         'LearningPathBuilderBrowserComponent' => '/lib/builder/component/browser.class.php',
         'LearningPathBuilderCreatorComponent' => '/lib/builder/component/creator.class.php',
         'LearningPathBuilderDeleterComponent' => '/lib/builder/component/deleter.class.php',
         'LearningPathBuilderFeedbackConfigurerComponent' => '/lib/builder/component/feedback_configurer.class.php',
         'LearningPathBuilderMasteryScoreSetterComponent' => '/lib/builder/component/mastery_score_setter.class.php',
         'LearningPathBuilderMoverComponent' => '/lib/builder/component/mover.class.php',
         'LearningPathBuilderParentChangerComponent' => '/lib/builder/component/parent_changer.class.php',
         'LearningPathBuilderPrerequisitesBuilderComponent' => '/lib/builder/component/prerequisites_builder.class.php',
         'LearningPathBuilderPreviewComponent' => '/lib/builder/component/preview.class.php',
         'LearningPathBuilderUpdaterComponent' => '/lib/builder/component/updater.class.php',
         'LearningPathBuilderViewerComponent' => '/lib/builder/component/viewer.class.php',
         'LearningPathBrowserTableCellRenderer' => '/lib/builder/component/browser/learning_path_browser_table_cell_renderer.class.php',
         'FeedbackConfigurerForm' => '/lib/builder/component/feedback_configurer/feedback_configurer_form.class.php',
         'PrerequisitesBuilderForm' => '/lib/builder/component/prerequisites_builder/prerequisites_builder_form.class.php',
         'LearningPathComplexDisplay' => '/lib/complex_display/learning_path_complex_display.class.php',
         'LearningPathComplexDisplayPreview' => '/lib/complex_display/learning_path_complex_display_preview.class.php',
         'LearningPathComplexDisplaySupport' => '/lib/complex_display/learning_path_complex_display_support.class.php',
         'LearningPathContentObjectDisplay' => '/lib/complex_display/learning_path_content_object_display.class.php',
         'LearningPathDisplayEmbedder' => '/lib/complex_display/learning_path_display_embedder.class.php',
         'LearningPathTree' => '/lib/complex_display/learning_path_tree.class.php',
         'PrerequisitesTranslator' => '/lib/complex_display/prerequisites_translator.class.php',
         'RuleConditionTranslator' => '/lib/complex_display/rule_condition_translator.class.php',
         'LearningPathComplexDisplayAttachmentViewerComponent' => '/lib/complex_display/component/attachment_viewer.class.php',
         'LearningPathComplexDisplayEmbedderComponent' => '/lib/complex_display/component/embedder.class.php',
         'LearningPathComplexDisplayViewerComponent' => '/lib/complex_display/component/viewer.class.php',
         'LearningPathAssessmentContentObjectDisplay' => '/lib/complex_display/content_object_display/assessment.class.php',
         'LearningPathBlogContentObjectDisplay' => '/lib/complex_display/content_object_display/blog.class.php',
         'LearningPathDocumentContentObjectDisplay' => '/lib/complex_display/content_object_display/document.class.php',
         'LearningPathForumContentObjectDisplay' => '/lib/complex_display/content_object_display/forum.class.php',
         'LearningPathGlossaryContentObjectDisplay' => '/lib/complex_display/content_object_display/glossary.class.php',
         'LearningPathHotpotatoesContentObjectDisplay' => '/lib/complex_display/content_object_display/hotpotatoes.class.php',
         'LearningPathLinkContentObjectDisplay' => '/lib/complex_display/content_object_display/link.class.php',
         'LearningPathScormItemContentObjectDisplay' => '/lib/complex_display/content_object_display/scorm_item.class.php',
         'LearningPathWikiContentObjectDisplay' => '/lib/complex_display/content_object_display/wiki.class.php',
         'LearningPathDisplayEmbedderComplexContentObjectComponent' => '/lib/complex_display/embedder/complex_content_object.class.php',
         'LearningPathDisplayEmbedderContentObjectComponent' => '/lib/complex_display/embedder/content_object.class.php',
         'LearningPathDisplayEmbedderDocumentComponent' => '/lib/complex_display/embedder/document.class.php',
         'DummyLpAttemptTracker' => '/lib/complex_display/preview/dummy_lp_attempt_tracker.class.php',
         'DummyLpiAttemptTracker' => '/lib/complex_display/preview/dummy_lpi_attempt_tracker.class.php',
         'ComplexLearningPath' => '/lib/data_class/complex_content_object_item.class.php',
         'LearningPath' => '/lib/data_class/content_object.class.php',
         'DataManager' => '/lib/data_manager/data_manager.class.php',
         'DataManagerInterface' => '/lib/data_manager/data_manager_interface.class.php',
         'DoctrineDataManager' => '/lib/data_manager/implementation/doctrine.class.php',
         'Mdb2DataManager' => '/lib/data_manager/implementation/mdb2.class.php',
         'ComplexLearningPathForm' => '/lib/form/complex_content_object_item.class.php',
         'LearningPathForm' => '/lib/form/content_object.class.php',
         'ExportImplementation' => '/lib/implementation/export.class.php',
         'ImportImplementation' => '/lib/implementation/import.class.php',
         'CpoExportImplementation' => '/lib/implementation/export/cpo.class.php',
         'CpoDefaultExportImplementation' => '/lib/implementation/export/cpo/default.class.php',
         'Activator' => '/package/activate/activator.class.php',
         'Deactivator' => '/package/deactivate/deactivator.class.php',
         'Installer' => '/package/install/installer.class.php',
         'Remover' => '/package/remove/remover.class.php'
    );

    /**
     * Try to load the class
     *
     * @param $classname string
     * @return boolean
     */
    public static function load($classname)
    {
        if (isset(self :: $map[$classname]))
        {
            require_once __DIR__ . self :: $map[$classname];
            return true;
        }

        return false;
    }

    /**
     * Synchronize the autoloader
     *
     * @param $update boolean
     * @return multitype:string
     */
    public static function synch($update)
    {
        return \common\libraries\AutoloaderUtilities :: synch(__DIR__, __DIR__, $update);
    }

}
?>