<?php
namespace repository\content_object\learning_path;

use repository\ContentObjectRenditionImplementation;
use repository\ContentObjectRendition;
use common\libraries\Request;
use common\libraries\Translation;
use common\libraries\Breadcrumb;
use common\libraries\BreadcrumbTrail;
use repository\ComplexBuilder;
use common\libraries\DelegateComponent;

/**
 *
 * @author Hans De Bisschop
 * @package repository.content_object.adaptive_assessment
 */
class LearningPathBuilderFeedbackConfigurerComponent extends LearningPathBuilder implements DelegateComponent
{

    public function run()
    {
        $complex_content_object_item_id = Request :: get(
            LearningPathBuilder :: PARAM_SELECTED_COMPLEX_CONTENT_OBJECT_ITEM_ID);
        $parent_complex_content_object_item = Request :: get(
            LearningPathBuilder :: PARAM_COMPLEX_CONTENT_OBJECT_ITEM_ID);

        $menu_trail = $this->get_complex_content_object_breadcrumbs();
        $trail = BreadcrumbTrail :: get_instance();

        $parameters = array(
            LearningPathBuilder :: PARAM_COMPLEX_CONTENT_OBJECT_ITEM_ID => $parent_complex_content_object_item,
            LearningPathBuilder :: PARAM_SELECTED_COMPLEX_CONTENT_OBJECT_ITEM_ID => $complex_content_object_item_id);

        $trail->add(new Breadcrumb($this->get_url($parameters), Translation :: get('ConfigureAssessment')));

        if (! $complex_content_object_item_id)
        {
            $this->display_header();
            $this->display_error_message(Translation :: get('NoObjectSelected'));
            $this->display_footer();
            exit();
        }

        $selected_complex_content_object_item = \repository\DataManager :: retrieve_complex_content_object_item(
            $complex_content_object_item_id);
        $learning_path_item = \repository\DataManager :: retrieve_content_object(
            $selected_complex_content_object_item->get_ref());
        $form = new FeedbackConfigurerForm($this->get_user(), $learning_path_item, $this->get_url($parameters));

        if ($form->validate())
        {
            $succes = $form->configure_feedback();
            $message = $succes ? 'FeedbackConfigured' : 'FeedbackNotConfigured';
            $this->redirect(
                Translation :: get($message),
                ! $succes,
                array_merge(
                    $parameters,
                    array(ComplexBuilder :: PARAM_BUILDER_ACTION => ComplexBuilder :: ACTION_BROWSE)));
        }
        else
        {
            $this->display_header();
            echo ContentObjectRenditionImplementation :: launch(
                $selected_complex_content_object_item->get_ref_object()->get_reference_object(),
                ContentObjectRendition :: FORMAT_HTML,
                ContentObjectRendition :: VIEW_FULL,
                $this);
            $form->display();
            $this->display_footer();
        }
    }
}
