<?php
namespace repository\content_object\learning_path;

use common\libraries\Utilities;
use common\libraries\FormValidator;
use common\libraries\Translation;
use repository\content_object\assessment\FeedbackDisplayConfiguration;
use repository\content_object\learning_path_item\LearningPathItem;
use common\libraries\ResourceManager;
use common\libraries\Path;

/**
 *
 * @author Hans De Bisschop
 * @package repository.content_object.adaptive_assessment
 */
class FeedbackConfigurerForm extends FormValidator
{

    /**
     *
     * @var User
     */
    private $user;

    /**
     *
     * @var ComplexAdaptiveAssessmentItem
     */
    private $learning_path_item;

    /**
     * Creates a new AccountForm
     */
    public function __construct($user, $learning_path_item, $action)
    {
        parent :: __construct('feedback_configuration', 'post', $action);

        $this->user = $user;
        $this->learning_path_item = $learning_path_item;

        $this->setDefaults();
        $this->build_form();
    }

    public function build_form()
    {
        $this->addElement('checkbox', LearningPathItem :: PROPERTY_HINTS_ALLOWED, Translation :: get('HintsAllowed'));
        $this->addElement(
            'checkbox',
            LearningPathItem :: PROPERTY_FEEDBACK_SUMMARY,
            Translation :: get('FeedbackSummary'));
        $this->addElement(
            'checkbox',
            LearningPathItem :: PROPERTY_FEEDBACK_PER_PAGE,
            Translation :: get('FeedbackPerPage'));

        $feedback_type_options = array();
        $feedback_type_options[FeedbackDisplayConfiguration :: TYPE_TEXT] = Translation :: get('TextFeedback');
        $feedback_type_options[FeedbackDisplayConfiguration :: TYPE_NUMERIC] = Translation :: get('NumericFeedback');
        $feedback_type_options[FeedbackDisplayConfiguration :: TYPE_BOTH] = Translation :: get('TextAndNumericFeedback');

        $this->addElement(
            'select',
            LearningPathItem :: PROPERTY_FEEDBACK_TYPE,
            Translation :: get('FeedbackType'),
            $feedback_type_options);

        $this->addElement(
            'html',
            ResourceManager :: get_instance()->get_resource_html(
                Path :: namespace_to_full_path(__NAMESPACE__, true) . 'resources/javascript/feedback_form.js'));

        $buttons = array();
        $buttons[] = $this->createElement(
            'style_submit_button',
            'submit',
            Translation :: get('Save', null, Utilities :: COMMON_LIBRARIES),
            array('class' => 'positive update'));
        $buttons[] = $this->createElement(
            'style_reset_button',
            'reset',
            Translation :: get('Reset', null, Utilities :: COMMON_LIBRARIES),
            array('class' => 'normal empty'));
        $this->addGroup($buttons, 'buttons', null, '&nbsp;', false);
    }

    public function setDefaults($defaults = array())
    {
        $defaults[LearningPathItem :: PROPERTY_HINTS_ALLOWED] = $this->learning_path_item->get_hints_allowed();
        $defaults[LearningPathItem :: PROPERTY_FEEDBACK_SUMMARY] = $this->learning_path_item->get_feedback_summary();
        $defaults[LearningPathItem :: PROPERTY_FEEDBACK_PER_PAGE] = $this->learning_path_item->get_feedback_per_page();
        $defaults[LearningPathItem :: PROPERTY_FEEDBACK_TYPE] = $this->learning_path_item->get_feedback_type();

        parent :: setDefaults($defaults);
    }

    public function configure_feedback()
    {
        $values = $this->exportValues();

        if (isset($values[LearningPathItem :: PROPERTY_HINTS_ALLOWED]))
        {
            $this->learning_path_item->set_hints_allowed($values[LearningPathItem :: PROPERTY_HINTS_ALLOWED]);
        }
        else
        {
            $this->learning_path_item->set_hints_allowed(0);
        }

        if (isset($values[LearningPathItem :: PROPERTY_FEEDBACK_SUMMARY]))
        {
            $this->learning_path_item->set_feedback_summary($values[LearningPathItem :: PROPERTY_FEEDBACK_SUMMARY]);
        }
        else
        {
            $this->learning_path_item->set_feedback_summary(0);
        }

        if (isset($values[LearningPathItem :: PROPERTY_FEEDBACK_PER_PAGE]))
        {
            $this->learning_path_item->set_feedback_per_page($values[LearningPathItem :: PROPERTY_FEEDBACK_PER_PAGE]);
        }
        else
        {
            $this->learning_path_item->set_feedback_per_page(0);
        }

        $this->learning_path_item->set_feedback_type($values[LearningPathItem :: PROPERTY_FEEDBACK_TYPE]);

        return $this->learning_path_item->update();
    }
}
