<?php
namespace repository\content_object\learning_path_item;

use repository\content_object\assessment\FeedbackDisplayConfiguration;
use repository\content_object\learning_path\LearningPath;
use common\libraries\HelperContentObjectSupport;
use common\libraries\Utilities;
use common\libraries\Versionable;
use repository\ContentObject;

/**
 * $Id: learning_path_item.class.php 200 2009-11-13 12:30:04Z kariboe $
 *
 * @package repository.lib.content_object.learning_path_item
 */
class LearningPathItem extends ContentObject implements Versionable, HelperContentObjectSupport
{
    const PROPERTY_REFERENCE = 'reference_id';
    const PROPERTY_MAX_ATTEMPTS = 'max_attempts';
    const PROPERTY_MASTERY_SCORE = 'mastery_score';
    const PROPERTY_HINTS_ALLOWED = 'hints_allowed';
    const PROPERTY_FEEDBACK_SUMMARY = 'feedback_summary';
    const PROPERTY_FEEDBACK_PER_PAGE = 'feedback_per_page';
    const PROPERTY_FEEDBACK_TYPE = 'feedback_type';
    const CLASS_NAME = __CLASS__;

    /**
     *
     * @var LearningPath
     */
    private $reference_object;

    public static function get_type_name()
    {
        return Utilities :: camelcase_to_underscores(Utilities :: get_classname_from_namespace(self :: CLASS_NAME));
    }

    public static function get_additional_property_names()
    {
        return array(
            self :: PROPERTY_REFERENCE,
            self :: PROPERTY_MAX_ATTEMPTS,
            self :: PROPERTY_MASTERY_SCORE,
            self :: PROPERTY_HINTS_ALLOWED,
            self :: PROPERTY_FEEDBACK_SUMMARY,
            self :: PROPERTY_FEEDBACK_PER_PAGE,
            self :: PROPERTY_FEEDBACK_TYPE);
    }

    public function get_reference()
    {
        return $this->get_additional_property(self :: PROPERTY_REFERENCE);
    }

    public function set_reference($reference)
    {
        $this->set_additional_property(self :: PROPERTY_REFERENCE, $reference);
    }

    public function get_reference_object()
    {
        if (! $this->reference_object instanceof LearningPath)
        {
            $this->reference_object = \repository\DataManager :: retrieve_content_object($this->get_reference());
        }
        return $this->reference_object;
    }

    public function get_max_attempts()
    {
        return $this->get_additional_property(self :: PROPERTY_MAX_ATTEMPTS);
    }

    public function set_max_attempts($max_attempts)
    {
        $this->set_additional_property(self :: PROPERTY_MAX_ATTEMPTS, $max_attempts);
    }

    public function get_mastery_score()
    {
        return $this->get_additional_property(self :: PROPERTY_MASTERY_SCORE);
    }

    public function set_mastery_score($mastery_score)
    {
        $this->set_additional_property(self :: PROPERTY_MASTERY_SCORE, $mastery_score);
    }

    /**
     *
     * @return boolean
     */
    public function get_hints_allowed()
    {
        return $this->get_additional_property(self :: PROPERTY_HINTS_ALLOWED);
    }

    public function set_hints_allowed($hints_allowed)
    {
        $this->set_additional_property(self :: PROPERTY_HINTS_ALLOWED, $hints_allowed);
    }

    /**
     *
     * @return boolean
     */
    public function get_feedback_summary()
    {
        return $this->get_additional_property(self :: PROPERTY_FEEDBACK_SUMMARY);
    }

    public function set_feedback_summary($feedback_summary)
    {
        $this->set_additional_property(self :: PROPERTY_FEEDBACK_SUMMARY, $feedback_summary);
    }

    /**
     *
     * @return boolean
     */
    public function get_feedback_per_page()
    {
        return $this->get_additional_property(self :: PROPERTY_FEEDBACK_PER_PAGE);
    }

    public function set_feedback_per_page($feedback_per_page)
    {
        $this->set_additional_property(self :: PROPERTY_FEEDBACK_PER_PAGE, $feedback_per_page);
    }

    /**
     *
     * @return int
     */
    public function get_feedback_type()
    {
        return $this->get_additional_property(self :: PROPERTY_FEEDBACK_TYPE);
    }

    public function set_feedback_type($feedback_type)
    {
        $this->set_additional_property(self :: PROPERTY_FEEDBACK_TYPE, $feedback_type);
    }

    /**
     *
     * @return \repository\content_object\assessment\FeedbackDisplayConfiguration
     */
    public function get_feedback_configuration()
    {
        return new FeedbackDisplayConfiguration(
            $this->get_hints_allowed(),
            $this->get_feedback_summary(),
            $this->get_feedback_per_page(),
            $this->get_feedback_type());
    }
}
