<?php
namespace repository\content_object\learning_path_item\upgrade\v4_0_0;

use common\libraries\CommonDataManager;
use common\libraries\package\Package;
use common\libraries\DataClassProperties;
use common\libraries\DataClassProperty;
use common\libraries\PropertyConditionVariable;
use common\libraries\StaticConditionVariable;
use common\libraries\EqualityCondition;
use common\libraries\SubselectCondition;

/**
 *
 * @author Hans De Bisschop
 * @author Magali Gillard
 */
class Upgrader extends \common\libraries\package\Upgrader
{

    /**
     * Constructor
     */
    public function __construct()
    {
        parent :: __construct(\repository\content_object\learning_path_item\DataManager :: get_instance());
    }

    /*
     * (non-PHPdoc) @see \common\libraries\package\Upgrader::run()
     */
    public function run()
    {
        $data_manager = $this->get_data_manager();

        $success = $data_manager->transactional(
            function ($c) use($data_manager)
            {
                $package = Package :: get('repository\content_object\learning_path_item');
                $registration = CommonDataManager :: get_registration('repository\content_object\learning_path_item');
                $registration->set_version($package->get_version());
                if (! $registration->update())
                {
                    return false;
                }

                $attributes = array('type' => 'integer', 'length' => 3, 'unsigned' => 1);
                $success = $data_manager->alter_storage_unit(
                    \common\libraries\DataManager :: ALTER_STORAGE_UNIT_CHANGE,
                    \repository\content_object\learning_path_item\LearningPathItem :: get_table_name(),
                    \repository\content_object\learning_path_item\LearningPathItem :: PROPERTY_MASTERY_SCORE,
                    $attributes);

                if (! $success)
                {
                    return false;
                }

                $attributes = array('type' => 'boolean', 'notnull' => 1, 'default' => 1);
                $success = $data_manager->alter_storage_unit(
                    \common\libraries\DataManager :: ALTER_STORAGE_UNIT_ADD,
                    \repository\content_object\learning_path_item\LearningPathItem :: get_table_name(),
                    \repository\content_object\learning_path_item\LearningPathItem :: PROPERTY_HINTS_ALLOWED,
                    $attributes);

                if (! $success)
                {
                    return false;
                }

                $attributes = array('type' => 'boolean', 'notnull' => 1, 'default' => 1);
                $success = $data_manager->alter_storage_unit(
                    \common\libraries\DataManager :: ALTER_STORAGE_UNIT_ADD,
                    \repository\content_object\learning_path_item\LearningPathItem :: get_table_name(),
                    \repository\content_object\learning_path_item\LearningPathItem :: PROPERTY_FEEDBACK_SUMMARY,
                    $attributes);

                if (! $success)
                {
                    return false;
                }

                $attributes = array('type' => 'boolean', 'notnull' => 0, 'default' => 1);
                $success = $data_manager->alter_storage_unit(
                    \common\libraries\DataManager :: ALTER_STORAGE_UNIT_ADD,
                    \repository\content_object\learning_path_item\LearningPathItem :: get_table_name(),
                    \repository\content_object\learning_path_item\LearningPathItem :: PROPERTY_FEEDBACK_PER_PAGE,
                    $attributes);

                if (! $success)
                {
                    return false;
                }

                $attributes = array('type' => 'integer', 'notnull' => 1, 'default' => 0, 'unsigned' => 1, 'length' => 1);
                $success = $data_manager->alter_storage_unit(
                    \common\libraries\DataManager :: ALTER_STORAGE_UNIT_ADD,
                    \repository\content_object\learning_path_item\LearningPathItem :: get_table_name(),
                    \repository\content_object\learning_path_item\LearningPathItem :: PROPERTY_FEEDBACK_TYPE,
                    $attributes);

                if (! $success)
                {
                    return false;
                }

                $attributes = array('type' => 'text');
                $success = $data_manager->alter_storage_unit(
                    \common\libraries\DataManager :: ALTER_STORAGE_UNIT_CHANGE,
                    \repository\content_object\learning_path_item\ComplexLearningPathItem :: get_table_name(),
                    \repository\content_object\learning_path_item\ComplexLearningPathItem :: PROPERTY_PREREQUISITES,
                    $attributes);

                if (! $success)
                {
                    return false;
                }

                $properties = new DataClassProperties();
                $properties->add(
                    new DataClassProperty(
                        new PropertyConditionVariable(
                            \repository\ContentObject :: class_name(),
                            \repository\ContentObject :: PROPERTY_TYPE),
                        new StaticConditionVariable('repository\content_object\learning_path_item\LearningPathItem')));

                $condition = new EqualityCondition(
                    new PropertyConditionVariable(
                        \repository\ContentObject :: class_name(),
                        \repository\ContentObject :: PROPERTY_TYPE),
                    new StaticConditionVariable('learning_path_item'));

                if (! \repository\DataManager :: updates(
                    \repository\ContentObject :: class_name(),
                    $properties,
                    $condition))
                {
                    return false;
                }

                $properties = new DataClassProperties();
                $properties->add(
                    new DataClassProperty(
                        new PropertyConditionVariable(
                            \repository\ComplexContentObjectItem :: class_name(),
                            \repository\ComplexContentObjectItem :: PROPERTY_TYPE),
                        new StaticConditionVariable(
                            'repository\content_object\learning_path_item\ComplexLearningPathItem')));

                $subselect_condition = new EqualityCondition(
                    new PropertyConditionVariable(
                        \repository\ContentObject :: class_name(),
                        \repository\ContentObject :: PROPERTY_TYPE),
                    new StaticConditionVariable('repository\content_object\learning_path_item\LearningPathItem'));

                $condition = new SubselectCondition(
                    new PropertyConditionVariable(
                        \repository\ComplexContentObjectItem :: class_name(),
                        \repository\ComplexContentObjectItem :: PROPERTY_REF),
                    new PropertyConditionVariable(
                        \repository\ContentObject :: class_name(),
                        \repository\ContentObject :: PROPERTY_ID),
                    null,
                    $subselect_condition);

                if (! \repository\DataManager :: updates(
                    \repository\ComplexContentObjectItem :: class_name(),
                    $properties,
                    $condition))
                {
                    return false;
                }

                return true;
            });

        return $success;
    }
}
