<?php

namespace repository\content_object\link;

use common\libraries\Includeable;

use common\libraries\Utilities;
use common\libraries\Versionable;
use repository\ContentObject;

/**
 * $Id: link.class.php 200 2009-11-13 12:30:04Z kariboe $
 * @package repository.lib.content_object.link
 */
class Link extends ContentObject implements Versionable, Includeable
{
    
    const PROPERTY_URL = 'url';
    const PROPERTY_SHOW_IN_IFRAME = 'show_in_iframe';
    
    const CLASS_NAME = __CLASS__;

    public static function get_type_name()
    {
        return Utilities :: camelcase_to_underscores(Utilities :: get_classname_from_namespace(self :: CLASS_NAME));
    }

    /**
     * If the content object "accept" the properties passed as argument it returns a new instance of itself based on properties passed as argument. Otherwise returns false.
     *
     * @param array $properties
     * @return ContentObject | array | false
     */
    public static function accept($properties)
    {
        $url = isset($properties[self :: PROPERTY_URL]) ? $properties[self :: PROPERTY_URL] : '';
        $status = isset($properties[self :: PROPERTY_SHOW_IN_IFRAME]) ? $properties[self :: PROPERTY_SHOW_IN_IFRAME] : false;
        if ($url)
        {
            $result = new self();
            $result->set_url($url);
            $result->set_show_in_iframe($status);
            $result = array(1000000 => $result); //returns an heavy weight because a link is the most generic form of item resource
            return $result;
        }
        else
        {
            return false;
        }
    }

    public function get_url()
    {
        return $this->get_additional_property(self :: PROPERTY_URL);
    }

    public function set_url($url)
    {
        $url = self :: complete_url($url);
        return $this->set_additional_property(self :: PROPERTY_URL, $url);
    }

    public function get_show_in_iframe()
    {
        return $this->get_additional_property(self :: PROPERTY_SHOW_IN_IFRAME);
    }

    public function set_show_in_iframe($status)
    {
        return $this->set_additional_property(self :: PROPERTY_SHOW_IN_IFRAME, $status);
    }

    public static function get_additional_property_names()
    {
        return array(self :: PROPERTY_URL, self :: PROPERTY_SHOW_IN_IFRAME);
    }

    /**
     * Validates the url, URL beginning with / are internal URL's and considered complete, URLS that contain :// are considered complete as well.
     * In any other case the URL is appended with 'http://' at the beginning.
     * @param String $url 
     * @return String completed url
     */
    public static function complete_url($url)
    {
        if (substr($url, 0, 1) == '/' || strstr($url, '://'))
        {
            return $url;
        }
        else
        {
            return 'http://' . $url;
        }
    }
    
    public static function get_searchable_property_names()
    {
    	return array(self :: PROPERTY_URL);
    }
}
