<?php
namespace repository\content_object\link\upgrade\v4_0_0;

use common\libraries\CommonDataManager;
use common\libraries\DataClassProperties;
use common\libraries\DataClassProperty;
use common\libraries\EqualityCondition;
use common\libraries\package\Package;
use common\libraries\PropertyConditionVariable;
use common\libraries\Registration;
use common\libraries\StaticConditionVariable;
use common\libraries\SubselectCondition;
use common\libraries\Utilities;
use repository\ComplexContentObjectItem;
use repository\content_object\link\Link;
use repository\ContentObject;
use common\libraries\DataClassRetrievesParameters;

/**
 * Database schema & content upgrader for Chamilo 3.x to 4.0.0.
 */
class Upgrader extends \common\libraries\package\Upgrader
{

    /**
     * Constructor
     */
    public function __construct()
    {
        parent :: __construct(\repository\content_object\link\DataManager :: get_instance());
    }

    /*
     * (non-PHPdoc) @see \common\libraries\package\Upgrader::run()
     */
    public function run()
    {
        $data_manager = $this->get_data_manager();
        $upgrader = $this;

        $success = $data_manager->transactional(
            function ($c) use($data_manager, $upgrader)
            {

                $attributes = array('name' => Link :: PROPERTY_URL, 'type' => 'text', 'notnull' => true);
                $success = $data_manager->alter_storage_unit(
                    \common\libraries\DataManager :: ALTER_STORAGE_UNIT_CHANGE,
                    Link :: get_table_name(),
                    Link :: PROPERTY_URL,
                    $attributes);

                if (! $success)
                {
                    return false;
                }

                $properties = new DataClassProperties();
                $properties->add(
                    new DataClassProperty(
                        new PropertyConditionVariable(ContentObject :: class_name(), ContentObject :: PROPERTY_TYPE),
                        new StaticConditionVariable(Link :: class_name())));

                $condition = new EqualityCondition(
                    new PropertyConditionVariable(ContentObject :: class_name(), ContentObject :: PROPERTY_TYPE),
                    new StaticConditionVariable(Link :: get_type_name()));

                if (! \repository\DataManager :: updates(ContentObject :: class_name(), $properties, $condition))
                {
                    return false;
                }

                $properties = new DataClassProperties();
                $properties->add(
                    new DataClassProperty(
                        new PropertyConditionVariable(
                            ComplexContentObjectItem :: class_name(),
                            ComplexContentObjectItem :: PROPERTY_TYPE),
                        new StaticConditionVariable('repository\content_object\link\ComplexLink')));

                $subselect_condition = new EqualityCondition(
                    new PropertyConditionVariable(ContentObject :: class_name(), ContentObject :: PROPERTY_TYPE),
                    new StaticConditionVariable('repository\content_object\link\Link'));

                $condition = new SubselectCondition(
                    new PropertyConditionVariable(
                        ComplexContentObjectItem :: class_name(),
                        ComplexContentObjectItem :: PROPERTY_REF),
                    new PropertyConditionVariable(ContentObject :: class_name(), ContentObject :: PROPERTY_ID),
                    null,
                    $subselect_condition);

                if (! \repository\DataManager :: updates(
                    ComplexContentObjectItem :: class_name(),
                    $properties,
                    $condition))
                {
                    return false;
                }

                $package = Package :: get('repository\content_object\link');
                $registration = CommonDataManager :: get_registration('repository\content_object\link');
                $registration->set_version($package->get_version());
                if (! $registration->update())
                {
                    return false;
                }

                // New subpackage registrations
                $subpackages = array('repository\content_object\link\integration\core\home');

                foreach ($subpackages as $subpackage)
                {
                    $package_info = Package :: get($subpackage);
                    $application_registration = new Registration();

                    $parent_namespace = Utilities :: get_namespace_parent($subpackage);
                    $application_registration->set_type((! $parent_namespace ? 'core' : $parent_namespace));

                    $application_registration->set_name($package_info->get_code());
                    $application_registration->set_category($package_info->get_category());
                    $application_registration->set_version($package_info->get_version());
                    $application_registration->set_status(Registration :: STATUS_ACTIVE);
                    $application_registration->set_context($subpackage);

                    if (! $application_registration->create())
                    {
                        return false;
                    }
                }

                // Home block registrations
                $condition = new EqualityCondition(
                    new PropertyConditionVariable(
                        \core\home\HomeBlockRegistration :: class_name(),
                        \core\home\HomeBlockRegistration :: PROPERTY_CONTEXT),
                    new StaticConditionVariable('repository\content_object\link'));
                $home_block_registrations = \core\home\DataManager :: retrieves(
                    \core\home\HomeBlockRegistration :: class_name(),
                    new DataClassRetrievesParameters($condition));

                while ($home_block_registration = $home_block_registrations->next_result())
                {
                    $home_block_registration->set_context('repository\content_object\link\integration\core\home');
                    if (! $home_block_registration->update())
                    {
                        return false;
                    }
                }

                return true;
            });

        return $success;
    }
}
