<?php
namespace repository\content_object\rss_feed\integration\core\home;

use common\libraries\Translation;
use common\libraries\Theme;
use common\libraries\StringUtilities;

class Feeder extends \repository\integration\core\home\Block
{

    /**
     * Returns the list of type names that this block can map to.
     *
     * @return array
     */
    public static function get_supported_types()
    {
        $result = array();
        $result[] = \repository\content_object\rss_feed\RssFeed :: get_type_name();
        return $result;
    }

    public function __construct($parent, $block_info, $configuration)
    {
        parent :: __construct($parent, $block_info, $configuration);
        $this->default_title = Translation :: get('Feeder');
    }

    public function is_visible()
    {
        return true; // i.e.display on homepage when anonymous
    }

    /**
     * Returns the html to display when the block is configured.
     *
     * @return string
     */
    public function display_content()
    {
        $content_object = $this->get_object();

        $html = array();
        $feed = \repository\content_object\rss_feed\RenditionImplementation :: parse_file($content_object->get_url());
        if ($feed)
        {
            $target = $this->get_link_target();
            $target = $target ? 'target="' . $target . '"' : 'target="_blank"';
            $icon = Theme :: get_image_path('\repository\content_object\rss_feed') . 'logo/' . Theme :: ICON_MINI . '.png';
            $html[] = '<div class="tool_menu">';
            $html[] = '<ul>';

            $count_valid = 0;

            foreach ($feed['items'] as $item)
            {
                if (! $item['link'] || ! $item['title'])
                {
                    continue;
                }

                $count_valid ++;

                $html[] = '<li class="tool_list_menu" style="background-image: url(' . $icon . ')"><a href="' . htmlentities(
                        $item['link']) . '" ' . $target . '>' . $item['title'] . '</a></li>';
            }

            $html[] = '</ul>';
            $html[] = '<div class="clear"></div>';
            $html[] = '</div>';
        }

        if (! $feed || $count_valid == 0)
        {
            $html[] = '<span style="font-weight: bold;">' . Translation :: get('NoFeedsFound') . '</span>';
        }

        return '<div style="height: 4px;"></div>' . implode(StringUtilities :: NEW_LINE, $html);
    }
}
