<?php

namespace repository\content_object\task\upgrade\v4_0_0;

use common\libraries\CommonDataManager;
use common\libraries\package\Package;
use repository\content_object\task\Task;
use common\libraries\DataClassProperties;
use common\libraries\DataClassProperty;
use common\libraries\EqualityCondition;
use common\libraries\PropertyConditionVariable;
use common\libraries\StaticConditionVariable;
use repository\ContentObject;
use common\libraries\SubselectCondition;

/**
 * Database schema & content upgrader for Chamilo 3.x to 4.0.0.
 */
class Upgrader extends \common\libraries\package\Upgrader
{

    /**
     * Constructor
     */
    public function __construct()
    {
        parent :: __construct(\repository\content_object\task\DataManager :: get_instance());
    }

    /*
     * (non-PHPdoc) @see \common\libraries\package\Upgrader::run()
     */
    public function run()
    {
        $data_manager = $this->get_data_manager();
        $upgrader = $this;

        $success = $data_manager->transactional(
            function ($c) use($data_manager, $upgrader)
            {

                $attributes = array(
                    'name' => Task :: PROPERTY_REPEAT_TYPE,
                    'type' => 'integer',
                    'length' => '3', // Don't know why, but this is the length required to alter the column to the correct datatype => integer(10) unsigned
                    'unsigned' => '1',
                    'notnull' => '1'
                );
                $success = $data_manager->alter_storage_unit(\common\libraries\DataManager :: ALTER_STORAGE_UNIT_CHANGE,
                    Task :: get_table_name(),
                    Task :: PROPERTY_REPEAT_TYPE,
                    $attributes
                );

                if (! $success)
                {
                    return false;
                }

                $properties = new DataClassProperties();
                $properties->add(
                    new DataClassProperty(
                        new PropertyConditionVariable(
                            ContentObject :: class_name(),
                            ContentObject :: PROPERTY_TYPE),
                        new StaticConditionVariable(Task :: class_name()))
                );

                $condition = new EqualityCondition(
                    new PropertyConditionVariable(
                        ContentObject :: class_name(),
                        ContentObject :: PROPERTY_TYPE),
                    new StaticConditionVariable(Task :: get_type_name())
                );

                if (! \repository\DataManager :: updates(
                    ContentObject :: class_name(),
                    $properties,
                    $condition))
                {
                    return false;
                }

                $properties = new DataClassProperties();
                $properties->add(
                    new DataClassProperty(
                        new PropertyConditionVariable(
                            \repository\ComplexContentObjectItem :: class_name(),
                            \repository\ComplexContentObjectItem :: PROPERTY_TYPE),
                        new StaticConditionVariable('repository\content_object\task\ComplexTask')));

                $subselect_condition = new EqualityCondition(
                    new PropertyConditionVariable(
                        \repository\ContentObject :: class_name(),
                        \repository\ContentObject :: PROPERTY_TYPE),
                    new StaticConditionVariable('repository\content_object\task\Task'));

                $condition = new SubselectCondition(
                    new PropertyConditionVariable(
                        \repository\ComplexContentObjectItem :: class_name(),
                        \repository\ComplexContentObjectItem :: PROPERTY_REF),
                    new PropertyConditionVariable(
                        \repository\ContentObject :: class_name(),
                        \repository\ContentObject :: PROPERTY_ID),
                    null,
                    $subselect_condition);

                if (! \repository\DataManager :: updates(
                    \repository\ComplexContentObjectItem :: class_name(),
                    $properties,
                    $condition))
                {
                    return false;
                }
                
                $package = Package :: get('repository\content_object\task');
                $registration = CommonDataManager :: get_registration('repository\content_object\task');
                $registration->set_version($package->get_version());
                if (! $registration->update())
                {
                    return false;
                }

                return true;
            });

        return $success;
    }
}
