<?php

namespace repository\content_object\template\upgrade\v4_0_0;

use common\libraries\CommonDataManager;
use common\libraries\package\Package;
use repository\content_object\template\Template;
use common\libraries\DataClassProperties;
use common\libraries\DataClassProperty;
use common\libraries\EqualityCondition;
use common\libraries\PropertyConditionVariable;
use common\libraries\StaticConditionVariable;
use repository\ContentObject;

/**
 * Database schema & content upgrader for Chamilo 3.x to 4.0.0.
 */
class Upgrader extends \common\libraries\package\Upgrader
{

    /**
     * Constructor
     */
    public function __construct()
    {
        parent :: __construct(\repository\content_object\template\DataManager :: get_instance());
    }

    /*
     * (non-PHPdoc) @see \common\libraries\package\Upgrader::run()
     */
    public function run()
    {
        $data_manager = $this->get_data_manager();
        $upgrader = $this;

        $success = $data_manager->transactional(
            function ($c) use($data_manager, $upgrader)
            {

                $attributes = array(
                    'name' => Template :: PROPERTY_DESIGN,
                    'type' => 'text'
                );
                $success = $data_manager->alter_storage_unit(\common\libraries\DataManager :: ALTER_STORAGE_UNIT_CHANGE,
                    Template :: get_table_name(),
                    Template :: PROPERTY_DESIGN,
                    $attributes
                );

                if (! $success)
                {
                    return false;
                }

                $properties = new DataClassProperties();
                $properties->add(
                    new DataClassProperty(
                        new PropertyConditionVariable(
                            ContentObject :: class_name(),
                            ContentObject :: PROPERTY_TYPE),
                        new StaticConditionVariable(Template :: class_name()))
                );

                $condition = new EqualityCondition(
                    new PropertyConditionVariable(
                        ContentObject :: class_name(),
                        ContentObject :: PROPERTY_TYPE),
                    new StaticConditionVariable(Template :: get_type_name())
                );

                if (! \repository\DataManager :: updates(
                    ContentObject :: class_name(),
                    $properties,
                    $condition))
                {
                    return false;
                }

                $package = Package :: get('repository\content_object\template');
                $registration = CommonDataManager :: get_registration('repository\content_object\template');
                $registration->set_version($package->get_version());
                if (! $registration->update())
                {
                    return false;
                }

                return true;
            });

        return $success;
    }
}
