<?php
namespace repository\content_object\webpage;

use common\libraries\AndCondition;
use common\libraries\EqualityCondition;

class DataManager extends \common\libraries\DataManager
{

    /**
     * Gets the type of DataManager to be instantiated
     * 
     * @return string
     */
    public static function get_type()
    {
        return 'doctrine';
    }

    public static function get_webpage_id_by_hash($hash)
    {
        $condition = new EqualityCondition(Webpage :: PROPERTY_HASH, $hash);
        $webpage = \repository\DataManager :: retrieve_active_content_objects(Webpage :: class_name(), $condition)->next_result();
        
        if ($webpage)
        {
            return $webpage->get_id();
        }
        
        return false;
    }

    public static function retrieve_webpage_from_hash($user_id, $hash)
    {
        $conditions = array();
        $conditions[] = new EqualityCondition(Webpage :: PROPERTY_HASH, $hash, 'webpage');
        $conditions[] = new EqualityCondition(Webpage :: PROPERTY_OWNER_ID, $user_id);
        $condition = new AndCondition($conditions);
        
        return \repository\DataManager :: retrieve_content_object_by_condition(Webpage :: class_name(), $condition);
    }

    public static function get_webpage_by_filename($filename)
    {
        $condition = new EqualityCondition(Webpage :: PROPERTY_FILENAME, $filename, 'webpage');
        return \repository\DataManager :: retrieve_content_object_by_condition(Webpage :: class_name(), $condition);
    }

    public static function is_only_webpage_occurence($path)
    {
        $condition = new EqualityCondition(Webpage :: PROPERTY_PATH, $path);
        $count = \repository\DataManager :: count_content_objects(Webpage :: class_name(), $condition);
        
        return ($count == 1 ? true : false);
    }
}
