<?php
namespace repository\content_object\wiki;

use repository\content_object\wiki_page\ComplexWikiPage;
use common\libraries\DelegateComponent;
use common\libraries\Request;
use common\libraries\Translation;
use repository\ComplexDisplay;
use repository\ComplexContentObjectItem;
use repository\content_object\wiki_page\WikiPage;

/**
 * $Id: wiki_page_creator.class.php 205 2009-11-13 12:57:33Z vanpouckesven $
 *
 * @package repository.lib.complex_display.wiki.component
 */
/*
 * This is the compenent that allows the user to create a wiki_page. Author: Stefan Billiet Author: Nick De Feyter
 */
class WikiComplexDisplayWikiPageCreatorComponent extends WikiComplexDisplay implements
        \repository\viewer\ViewerInterface,
        DelegateComponent
{

    public function run()
    {
        if (! \repository\viewer\Manager :: is_ready_to_be_published())
        {
            $repo_viewer = \repository\viewer\Manager :: construct($this);
            $repo_viewer->set_parameter(ComplexDisplay :: PARAM_DISPLAY_ACTION,
                    WikiComplexDisplay :: ACTION_CREATE_PAGE);
            $repo_viewer->run();
        }
        else
        {
            $objects = \repository\viewer\Manager :: get_selected_objects();

            if (! is_array($objects))
            {
                $objects = array($objects);
            }

            foreach ($objects as $object)
            {
                $complex_content_object_item = ComplexContentObjectItem :: factory(ComplexWikiPage :: class_name());
                $complex_content_object_item->set_ref($object);
                $complex_content_object_item->set_parent($this->get_root_content_object()->get_id());
                $complex_content_object_item->set_user_id($this->get_user_id());
                $complex_content_object_item->set_display_order(
                        \repository\DataManager :: select_next_display_order($this->get_root_content_object()->get_id()));
                $complex_content_object_item->set_is_homepage(0);
                $complex_content_object_item->create();
            }

            $this->redirect(Translation :: get('WikiItemCreated'), '',
                    array(WikiComplexDisplay :: PARAM_DISPLAY_ACTION => WikiComplexDisplay :: ACTION_VIEW_WIKI_PAGE,
                            ComplexDisplay :: PARAM_SELECTED_COMPLEX_CONTENT_OBJECT_ITEM_ID => $complex_content_object_item->get_id()));
        }
    }

    public function display_header()
    {
        parent :: display_header();

        $repository_viewer_action = Request :: get(\repository\viewer\Manager :: PARAM_ACTION);

        switch ($repository_viewer_action)
        {
            case \repository\viewer\Manager :: ACTION_BROWSER :
                $title = 'BrowseAvailableWikiPages';
                break;
            case \repository\viewer\Manager :: ACTION_CREATOR :
                $title = 'CreateWikiPage';
                break;
            case \repository\viewer\Manager :: ACTION_VIEWER :
                $title = 'PreviewWikiPage';
                break;
            default :
                $title = 'CreateWikiPage';
                break;
        }

        $html = array();
        $html[] = '<div class="wiki-pane-content-title">' . Translation :: get($title) . '</div>';
        $html[] = '<div class="wiki-pane-content-subtitle">' . Translation :: get('In') . ' ' . $this->get_root_content_object()->get_title() . '</div>';
        echo implode("\n", $html);
    }

    public function get_allowed_content_object_types()
    {
        return array(WikiPage :: class_name());
    }
}
