<?php
namespace repository;

use common\libraries\Utilities;
use common\libraries\Filesystem;
use common\libraries\Text;
use common\libraries\Path;
use repository\content_object\document\Document;
use repository\content_object\file\File;
use common\libraries\AjaxManager;
use common\libraries\JsonAjaxResult;
use common\libraries\NoAuthenticationSupport;

class RepositoryAjaxUploadImage extends AjaxManager implements NoAuthenticationSupport
{
    /*
     * (non-PHPdoc) @see common\libraries.AjaxManager::required_parameters()
     */
    public function required_parameters()
    {
        return array();
    }

    /*
     * (non-PHPdoc) @see common\libraries.AjaxManager::run()
     */
    public function run()
    {
        if (! empty($_FILES))
        {

            $upload_path = Path :: get(SYS_REPO_PATH);
            $owner = $this->get_parameter(self :: PARAM_USER_ID);

            $filename = $_FILES['Filedata']['name'];
            $hash = md5($_FILES['Filedata']['name']);

            $path = $owner . '/' . Text :: char_at($hash, 0);
            $full_path = $upload_path . $path;

            Filesystem :: create_dir($full_path);
            $hash = Filesystem :: create_unique_name($full_path, $hash);

            $path = $path . '/' . $hash;
            $full_path = $full_path . '/' . $hash;

            move_uploaded_file($_FILES['Filedata']['tmp_name'], $full_path) or
                 die('Failed to create "' . $full_path . '"');

            if (ContentObject :: is_available('repository\content_object\document\Document'))
            {
                $document = new Document();
            }
            else
            {
                $document = new File();
            }

            $document->set_owner_id($owner);
            $document->set_parent_id(0);
            $document->set_path($path);
            $document->set_filename($filename);
            $document->set_filesize(Filesystem :: get_disk_space($full_path));
            $document->set_hash($hash);

            $title_parts = explode('.', $filename);
            $extension = array_pop($title_parts);
            $title = Utilities :: underscores_to_camelcase_with_spaces(implode('_', $title_parts));
            $document->set_title($title);
            $document->set_description($title);
            $document->create();

            $dimensions = getimagesize($full_path);

            $path = Path :: get(WEB_PATH) . RepositoryManager :: get_document_downloader_url($document->get_id());

            $properties = array();
            $properties[ContentObject :: PROPERTY_ID] = $document->get_id();
            $properties[ContentObject :: PROPERTY_TITLE] = $document->get_title();
            $properties['fullPath'] = $full_path;
            $properties['webPath'] = $path;
            $properties[File :: PROPERTY_FILENAME] = $document->get_filename();
            $properties[File :: PROPERTY_PATH] = $document->get_path();
            $properties[File :: PROPERTY_FILESIZE] = $document->get_filesize();
            $properties['width'] = $dimensions[0];
            $properties['height'] = $dimensions[1];
            $properties['type'] = $document->get_extension();
            $properties['owner'] = $owner;

            $result = new JsonAjaxResult(200);
            $result->set_properties($properties);
            $result->display();
        }
    }
}
