<?php
namespace repository;

use common\libraries\SubManager;
use common\libraries\Display;
use common\libraries\Translation;
use Exception;

abstract class ComplexDisplayPreview extends SubManager implements ComplexDisplaySupport
{

    /**
     *
     * @param string $type
     * @param Application $application
     */
    public static function launch($type, $application)
    {
        $class = $type . 'ComplexDisplayPreview';

        if (! class_exists($class))
        {
            throw new Exception(Translation :: get('ComplexDisplayPreviewTypeDoesNotExist', array('type' => $type)));
        }

        $preview = new $class($application);
        $preview->run();
    }

    /**
     * Abstract method implementened by every extension. Displays the actual complex display.
     */
    abstract public function run();

    /**
     * Method always has to be implemented for a class implementing the ComplexDisplay
     *
     * @return ContentObject
     */
    public function get_root_content_object()
    {
        return $this->get_parent()->get_root_content_object();
    }

    /**
     * Inform the user that the requested functionality is not available in preview mode
     *
     * @param string $message
     */
    public function not_available($message)
    {
        $this->display_header();
        Display :: normal_message($message);
        $this->display_footer();
        exit();
    }

    public function is_allowed($right)
    {
        return true;
    }

    public function is_allowed_to_add_child()
    {
        return true;
    }

    public function is_allowed_to_delete_child()
    {
        return true;
    }

    public function is_allowed_to_delete_feedback()
    {
        return true;
    }

    public function is_allowed_to_edit_content_object()
    {
        return true;
    }

    public function is_allowed_to_edit_feedback()
    {
        return true;
    }

    public function is_allowed_to_view_content_object()
    {
        return true;
    }
}
