<?php
namespace repository;

use common\libraries\Utilities;
use common\libraries\Request;
use common\libraries\Translation;
use common\libraries\Display;



/**
 *
 * @author Sven Vanpoucke
 */
class ComplexDisplayComponentCreatorComponent extends ComplexDisplayComponent implements \repository\viewer\ViewerInterface
{

    public function run()
    {
        if ($this->get_parent()->get_parent()->is_allowed_to_add_child())
        {
            $complex_content_object_item_id = $this->get_complex_content_object_item_id();
            
            if (! $this->get_root_content_object())
            {
                $this->display_header();
                $this->display_error_message(Translation :: get('NoParentSelected'));
                $this->display_footer();
                exit();
            }
            
            $type = Request :: get('type');
            
            if (! \repository\viewer\Manager :: is_ready_to_be_published())
            {
                $repository_viewer = \repository\viewer\Manager :: construct($this);
                $repository_viewer->set_maximum_select(\repository\viewer\Manager :: SELECT_SINGLE);
                $repository_viewer->set_parameter(ComplexDisplay :: PARAM_DISPLAY_ACTION, 
                        ComplexDisplay :: ACTION_CREATE_COMPLEX_CONTENT_OBJECT_ITEM);
                $repository_viewer->set_parameter('cid', $complex_content_object_item_id);
                $repository_viewer->set_parameter('type', $type);
                $repository_viewer->run();
            }
            else
            {
                $cloi = ComplexContentObjectItem :: factory($type);
                
                $cloi->set_ref(\repository\viewer\Manager :: get_selected_objects());
                $cloi->set_user_id($this->get_user_id());
                
                if ($complex_content_object_item_id)
                {
                    $complex_content_object_item = DataManager :: retrieve_complex_content_object_item(
                            $complex_content_object_item_id);
                    $cloi->set_parent($complex_content_object_item->get_ref());
                }
                else
                {
                    $cloi->set_parent($this->get_root_content_object()->get_id());
                }
                
                $cloi->set_display_order(DataManager :: select_next_display_order($cloi->get_parent()));
                
                $succes = $cloi->create();
                $this->my_redirect($complex_content_object_item_id, $succes);
            }
        }
        else
        {
            Display :: not_allowed();
            return;
        }
    }

    private function my_redirect($complex_content_object_item_id, $succes)
    {
        $message = htmlentities(
                Translation :: get(($succes ? 'ObjectCreated' : 'ObjectNotCreated'), 
                        array('OBJECT' => Translation :: get('ContentObject')), Utilities :: COMMON_LIBRARIES));
        
        $params = array();
        $params['cid'] = $complex_content_object_item_id;
        $params[ComplexDisplay :: PARAM_DISPLAY_ACTION] = ComplexDisplay :: ACTION_VIEW_COMPLEX_CONTENT_OBJECT;
        
        $this->redirect($message, (! $succes), $params);
    }

    public function get_allowed_content_object_types()
    {
        return array(Request :: get('type'));
    }
}
