<?php
namespace repository;

use common\libraries\Utilities;
use common\libraries\Filesystem;
use common\libraries\Filecompression;
use common\libraries\Path;
use common\libraries\ImscpManifestWriter;

class QtiContentObjectExportController extends ContentObjectExportController
{

    private $manifest;

    private $directory;

    private $toc;

    public function __construct(ExportParameters $parameters)
    {
        parent :: __construct($parameters);

        $this->manifest = new ImscpManifestWriter();
        $this->manifest = $this->manifest->add_manifest();
        $this->toc = $this->manifest->add_organizations()->add_organization();

        $this->prepare_file_system();
    }

    public function prepare_file_system()
    {
        $directory = Path :: get(SYS_TEMP_PATH) . $this->get_parameters()->get_user() . '/export_qti/';
        if (! is_dir($directory))
        {
            mkdir($directory, 0777, true);
        }

        $this->directory = $directory;
    }

    /**
     *
     * @return ImscpManifestWriter
     */
    public function get_manifest()
    {
        return $this->manifest;
    }

    public function get_toc()
    {
        return $this->toc;
    }

    public function get_directory()
    {
        return $this->directory;
    }

    public function run()
    {
        foreach ($this->get_parameters()->get_content_object_ids() as $content_object_id)
        {
            $content_object = \repository\DataManager :: retrieve_content_object($content_object_id);
            $this->process($content_object);
        }

        $this->write_manifest();

        return $this->zip();
    }

    public function zip()
    {
        $zip = Filecompression :: factory();
        $zippath = $zip->create_archive($this->get_directory());
        Filesystem :: remove($this->get_directory());
        return $zippath;
    }

    public function process($content_object)
    {
        $export_types = ContentObjectExportImplementation :: get_types_for_object(
                Utilities :: get_namespace_from_object($content_object));
        if (in_array(ContentObjectExport :: FORMAT_QTI, $export_types))
        {
            $exporter = QtiSerializerExport :: factory($this, $content_object);
            $exporter->export_content_object();
        }
    }

    public function write_manifest()
    {
        $xml = $this->get_manifest()->saveXML();
        $file_name = ImscpManifestWriter :: MANIFEST_NAME;

        $file_path = $this->get_directory() . $file_name;
        Filesystem :: write_to_file($file_path, $xml);
        return $file_path;
    }

    public function get_filename()
    {
        return 'export_qti.zip';
    }
}
