<?php
namespace repository;

use common\libraries\Translation;
use common\libraries\FileProperties;
use common\libraries\EqualityCondition;
use common\libraries\Path;
use common\libraries\Utilities;
use user\User;

/**
 * Extension of the generic installer for content objects
 *
 * @author Hans De Bisschop
 */
abstract class ContentObjectInstaller extends \common\libraries\package\Installer
{

    /**
     * Constructor
     *
     * @param $values multitype:mixed
     */
    public function __construct($values)
    {
        parent :: __construct($values, $this -> get_data_manager());
    }

    public function get_data_manager()
    {
        return DataManager :: get_instance();
    }
    
    /**
     * Perform additional installation steps
     *
     * @return boolean
     */
    public function extra()
    {
        if (! $this->import_content_object())
        {
            return false;
        }

        return true;
    }

    /**
     * Import a sample content object (if available)
     *
     * @return boolean
     */
    public function import_content_object()
    {
        $context = Utilities :: get_namespace_from_object($this);
        $file = Path :: namespace_to_full_path($context) . '/php/package/install/example.cpo';

        if (file_exists($file))
        {
            $condition = new EqualityCondition(User :: PROPERTY_PLATFORMADMIN, 1);
            $user = \user\DataManager :: retrieves(User :: class_name(), $condition)->next_result();

            \common\libraries\Session :: register('_uid', $user->get_id());

            $parameters = ImportParameters :: factory(ContentObjectImport :: FORMAT_CPO, $user->get_id(), 0,
                    FileProperties :: from_path($file));
            $import = ContentObjectImportController :: factory($parameters);
            $import->run();

            \common\libraries\Session :: unregister('_uid');

            if ($import->has_messages(ContentObjectImportController :: TYPE_ERROR))
            {
                $message = Translation :: get('ContentObjectImportFailed');
                $this->failed($message);
                return false;
            }
            else
            {
                $this->add_message(self :: TYPE_NORMAL, Translation :: get('ImportSuccessfull'));
            }
        }
        return true;
    }
}
