<?php
namespace repository;

use common\libraries\Mdb2Database;
use common\libraries\EqualityCondition;
use common\libraries\AndCondition;

/**
 * This is a data manager that uses a database for storage. It was written for MySQL, but should be compatible with most
 * SQL flavors.
 *
 * @author Tim De Pauw
 * @author Bart Mollet
 * @author Hans De Bisschop
 * @author Dieter De Neef
 * @package repository.lib.data_manager
 */
abstract class Mdb2RepositoryDataManager extends Mdb2Database implements DataManagerInterface
{
    const ALIAS_CONTENT_OBJECT_VERSION_TABLE = 'lov';
    const ALIAS_CONTENT_OBJECT_ATTACHMENT_TABLE = 'loa';
    const ALIAS_TYPE_TABLE = 'tt';
    const ALIAS_ADMIN_REGISTRATION_TABLE = 'reg';

    // Inherited.
    public function initialize()
    {
        parent :: initialize();
        // PEAR :: setErrorHandling(PEAR_ERROR_CALLBACK, array(get_class(),
        // 'handle_error'));
        $this->set_prefix('repository_');
    }

    /**
     * Handles PEAR errors. If an error is encountered, the program dies with a descriptive error message.
     *
     * @param $error DB_Error The error object.
     */
    public static function handle_error($error)
    {
        print_r($error);
        // For debugging only. May create a security hazard.
        die(__FILE__ . ':' . __LINE__ . ': ' . $error->getMessage() . ' (' . $error->getDebugInfo() . ')');
    }

    // TODO :: mediamosa manager
    public function retrieve_external_repository_user_quotum($user_id, $external_repository_id)
    {
        $condition2 = new EqualityCondition(ExternalRepositoryUserQuotum :: PROPERTY_USER_ID, $user_id);
        $condition1 = new EqualityCondition(ExternalRepositoryUserQuotum :: PROPERTY_EXTERNAL_REPOSITORY_ID,
                $external_repository_id);

        $condition = new AndCondition($condition1, $condition2);
        return $this->retrieve_object(ExternalRepositoryUserQuotum :: get_table_name(), $condition, array(),
                ExternalRepositoryUserQuotum :: CLASS_NAME);
    }

    // TODO : mediamosa datamanger
    public function create_external_repository_user_quotum(ExternalRepositoryUserQuotum $external_repository_user_quotum)
    {
        return $this->create($external_repository_user_quotum);
    }
    // TODO : mediamosa datamanger
    public function update_external_repository_user_quotum(ExternalRepositoryUserQuotum $external_repository_user_quotum)
    {
        $condition = new EqualityCondition(ExternalRepositoryUserQuotum :: PROPERTY_ID,
                $external_repository_user_quotum->get_id());
        return $this->update($external_repository_user_quotum, $condition);
    }
    // TODO : mediamosa datamanger
    public function delete_external_repository_user_quotum(ExternalRepositoryUserQuotum $external_repository_user_quotum)
    {
        $condition = new EqualityCondition(ExternalRepositoryUserQuotum :: PROPERTY_ID,
                $external_repository_user_quotum->get_id());
        return $this->delete(ExternalRepositoryUserQuotum :: get_table_name(), $condition);
    }
}
