<?php
namespace repository;

use common\libraries\Utilities;
use common\libraries\EqualityCondition;
use common\libraries\AndCondition;
use common\libraries\StringUtilities;
use Exception;

/**
 *
 * @author Hans De Bisschop
 */
class ExternalSync extends RepositoryDataClass
{
    const CLASS_NAME = __CLASS__;
    const PROPERTY_CONTENT_OBJECT_ID = 'content_object_id';
    const PROPERTY_CONTENT_OBJECT_TIMESTAMP = 'content_object_timestamp';
    const PROPERTY_EXTERNAL_ID = 'external_id';
    const PROPERTY_EXTERNAL_OBJECT_ID = 'external_object_id';
    const PROPERTY_EXTERNAL_OBJECT_TIMESTAMP = 'external_object_timestamp';
    const PROPERTY_EXTERNAL_USER_ID = 'external_user_id';
    const PROPERTY_STATE = 'state';
    const SYNC_STATUS_ERROR = 0;
    const SYNC_STATUS_EXTERNAL = 1;
    const SYNC_STATUS_INTERNAL = 2;
    const SYNC_STATUS_IDENTICAL = 3;
    const SYNC_STATUS_CONFLICT = 4;
    const STATE_ACTIVE = 1;
    const STATE_INACTIVE = 0;

    /**
     *
     * @var ContentObject
     */
    private $content_object;

    /**
     *
     * @var ExternalObject
     */
    private $external_object;

    /**
     *
     * @var int
     */
    private $synchronization_status;

    /**
     *
     * @var ExternalRepository
     */
    private $external;

    /**
     *
     * @param $content_object_id int
     */
    public function set_content_object_id($content_object_id)
    {
        if (isset($content_object_id) && is_numeric($content_object_id))
        {
            $this->set_default_property(self :: PROPERTY_CONTENT_OBJECT_ID, $content_object_id);
        }
    }

    /**
     *
     * @return int
     */
    public function get_content_object_id()
    {
        return $this->get_default_property(self :: PROPERTY_CONTENT_OBJECT_ID);
    }

    /**
     *
     * @param $external_object_id string
     */
    public function set_external_object_id($external_object_id)
    {
        if (StringUtilities :: has_value($external_object_id))
        {
            $this->set_default_property(self :: PROPERTY_EXTERNAL_OBJECT_ID, $external_object_id);
        }
    }

    /**
     *
     * @return string
     */
    public function get_external_object_id()
    {
        return $this->get_default_property(self :: PROPERTY_EXTERNAL_OBJECT_ID);
    }

    /**
     *
     * @param $datetime int
     */
    public function set_content_object_timestamp($datetime)
    {
        if (isset($datetime) && is_numeric($datetime))
        {
            $this->set_default_property(self :: PROPERTY_CONTENT_OBJECT_TIMESTAMP, $datetime);
        }
    }

    /**
     *
     * @return int
     */
    public function get_content_object_timestamp()
    {
        return $this->get_default_property(self :: PROPERTY_CONTENT_OBJECT_TIMESTAMP);
    }

    /**
     *
     * @param $datetime int
     */
    public function set_external_object_timestamp($datetime)
    {
        if (isset($datetime) && is_numeric($datetime))
        {
            $this->set_default_property(self :: PROPERTY_EXTERNAL_OBJECT_TIMESTAMP, $datetime);
        }
    }

    /**
     *
     * @return int
     */
    public function get_external_object_timestamp()
    {
        return $this->get_default_property(self :: PROPERTY_EXTERNAL_OBJECT_TIMESTAMP);
    }

    /**
     *
     * @param $external_id int
     */
    public function set_external_id($external_id)
    {
        if (isset($external_id) && is_numeric($external_id))
        {
            $this->set_default_property(self :: PROPERTY_EXTERNAL_ID, $external_id);
        }
    }

    /**
     *
     * @return int
     */
    public function get_external_id()
    {
        return $this->get_default_property(self :: PROPERTY_EXTERNAL_ID);
    }

    /**
     *
     * @param $external_user_id string
     */
    public function set_external_user_id($external_user_id)
    {
        if (StringUtilities :: has_value($external_user_id))
        {
            $this->set_default_property(self :: PROPERTY_EXTERNAL_USER_ID, $external_user_id);
        }
    }

    /**
     *
     * @return string
     */
    public function get_external_user_id()
    {
        return $this->get_default_property(self :: PROPERTY_EXTERNAL_USER_ID);
    }

    /**
     *
     * @param $state int
     */
    public function set_state($state)
    {
        $this->set_default_property(self :: PROPERTY_STATE, $state);
    }

    /**
     *
     * @return int
     */
    public function get_state()
    {
        return $this->get_default_property(self :: PROPERTY_STATE);
    }

    /**
     *
     * @param $property_names array
     */
    public static function get_default_property_names($extended_property_names = array())
    {
        $extended_property_names[] = self :: PROPERTY_CONTENT_OBJECT_ID;
        $extended_property_names[] = self :: PROPERTY_CONTENT_OBJECT_TIMESTAMP;
        $extended_property_names[] = self :: PROPERTY_EXTERNAL_ID;
        $extended_property_names[] = self :: PROPERTY_EXTERNAL_OBJECT_ID;
        $extended_property_names[] = self :: PROPERTY_EXTERNAL_OBJECT_TIMESTAMP;
        $extended_property_names[] = self :: PROPERTY_EXTERNAL_USER_ID;
        $extended_property_names[] = self :: PROPERTY_STATE;
        
        return parent :: get_default_property_names($extended_property_names);
    }

    /**
     *
     * @return string
     */
    public static function get_table_name()
    {
        return Utilities :: get_classname_from_namespace(self :: CLASS_NAME, true);
    }

    /**
     *
     * @return boolean
     */
    public function create()
    {
        $now = time();
        $this->set_creation_date($now);
        $this->set_modification_date($now);
        return parent :: create();
    }

    /**
     *
     * @return boolean
     */
    public function update()
    {
        if (! $this->is_identified())
        {
            throw new Exception('ExternalSync object could not be saved as its identity is not set');
        }
        
        $this->set_modification_date(time());
        return parent :: update();
    }

    /**
     *
     * @return ContentObject
     */
    public function get_content_object()
    {
        if (! isset($this->content_object))
        {
            $this->content_object = \repository\DataManager :: retrieve_content_object($this->get_content_object_id());
        }
        return $this->content_object;
    }

    /**
     *
     * @return ExternalObject
     */
    public function get_external_object()
    {
        if (! isset($this->external_object))
        {
            $external_instance = $this->get_external();
            $type = $external_instance->get_type();
            $class = Utilities :: get_namespace_parent(Utilities :: get_namespace_parent($type)) . '\DataConnector';
            $this->external_object = $class :: get_instance($external_instance)->retrieve_external_object($this);
        }
        return $this->external_object;
    }

    /**
     *
     * @return ExternalRepository
     */
    public function get_external()
    {
        if (! isset($this->external))
        {
            $this->external = \repository\DataManager :: retrieve_by_id(ExternalInstance :: class_name(), 
                    $this->get_external_id());
        }
        return $this->external;
    }

    public function get_synchronization_status($content_object_date = null, $external_object_date = null)
    {
        if (! isset($this->synchronization_status))
        {
            if (is_null($content_object_date))
            {
                $content_object_date = $this->get_content_object()->get_modification_date();
            }
            if (is_null($external_object_date))
            {
                $external_object_date = $this->get_external_object()->get_created();
            }
            
            if ($content_object_date > $this->get_content_object_timestamp())
            {
                if ($external_object_date > $this->get_external_object_timestamp())
                {
                    $this->synchronization_status = self :: SYNC_STATUS_CONFLICT;
                }
                elseif ($external_object_date == $this->get_external_object_timestamp())
                {
                    $this->synchronization_status = self :: SYNC_STATUS_EXTERNAL;
                }
                else
                {
                    $this->synchronization_status = self :: SYNC_STATUS_ERROR;
                }
            }
            elseif ($content_object_date == $this->get_content_object_timestamp())
            {
                if ($external_object_date > $this->get_external_object_timestamp())
                {
                    $this->synchronization_status = self :: SYNC_STATUS_INTERNAL;
                }
                elseif ($external_object_date == $this->get_external_object_timestamp())
                {
                    $this->synchronization_status = self :: SYNC_STATUS_IDENTICAL;
                }
                else
                {
                    $this->synchronization_status = self :: SYNC_STATUS_ERROR;
                }
            }
            else
            {
                $this->synchronization_status = self :: SYNC_STATUS_ERROR;
            }
        }
        
        return $this->synchronization_status;
    }

    /**
     * *********************************************************************** Fat model methods
     * ***********************************************************************
     */
    
    /**
     *
     * @param $content_object_id int
     * @return ExternalSync
     */
    public static function get_by_content_object_id($content_object_id)
    {
        $conditions = new EqualityCondition(self :: PROPERTY_CONTENT_OBJECT_ID, $content_object_id);
        return \repository\DataManager :: retrieve_external_sync($conditions);
    }

    /**
     *
     * @param $content_object_id int
     * @param $external_id int
     * @return ExternalSync
     */
    public static function get_by_content_object_id_and_external_id($content_object_id, $external_id)
    {
        $conditions = array();
        $conditions[] = new EqualityCondition(self :: PROPERTY_CONTENT_OBJECT_ID, $content_object_id);
        $conditions[] = new EqualityCondition(self :: PROPERTY_EXTERNAL_ID, $external_id);
        $condition = new AndCondition($conditions);
        return \repository\DataManager :: retrieve_external_sync($condition);
    }

    /**
     *
     * @param $external_object_id int
     * @param $external_id int
     * @return ExternalSync
     */
    public static function get_by_external_object_id_and_external_id($external_object_id, $external_id)
    {
        $conditions = array();
        $conditions[] = new EqualityCondition(self :: PROPERTY_EXTERNAL_OBJECT_ID, $external_object_id);
        $conditions[] = new EqualityCondition(self :: PROPERTY_EXTERNAL_ID, $external_id);
        $condition = new AndCondition($conditions);
        
        return \repository\DataManager :: retrieve_external_sync($conditions);
    }

    /**
     *
     * @param $content_object ContentObject
     * @param $external_object ExternalObject
     * @param $external_id int
     * @return boolean
     */
    public static function quicksave(ContentObject $content_object, $external_object, $external_id)
    {
        $sync = new ExternalSync();
        $sync->set_content_object_id($content_object->get_id());
        $sync->set_content_object_timestamp($content_object->get_modification_date());
        $sync->set_external_id((int) $external_id);
        $sync->set_external_object_id((string) $external_object->get_id());
        $sync->set_external_object_timestamp($external_object->get_modified());
        $sync->set_external_user_id($external_object->get_owner_id());
        $sync->set_state(self :: STATE_ACTIVE);
        return $sync->create();
    }
}
