<?php
namespace repository;

use common\libraries\DataClassRetrievesParameters;
use common\libraries\Utilities;
use common\libraries\EqualityCondition;
use common\libraries\Session;
use common\libraries\AndCondition;
use common\libraries\DataClass;

/**
 * $Id: external_repository_user_setting.class.php 211 2009-11-13 13:28:39Z vanpouckesven $
 * 
 * @author Sven Vanpoucke
 * @package user.lib
 */
class ExternalUserSetting extends DataClass
{
    const CLASS_NAME = __CLASS__;
    const PROPERTY_USER_ID = 'user_id';
    const PROPERTY_SETTING_ID = 'setting_id';
    const PROPERTY_VALUE = 'value';

    /**
     * A static array containing all user settings of external repository instances
     * 
     * @var array
     */
    private static $settings;

    /**
     * Get the default properties of all users quota objects.
     * 
     * @return array The property names.
     */
    public static function get_default_property_names()
    {
        return parent :: get_default_property_names(
                array(self :: PROPERTY_USER_ID, self :: PROPERTY_SETTING_ID, self :: PROPERTY_VALUE));
    }

    public function get_user_id()
    {
        return $this->get_default_property(self :: PROPERTY_USER_ID);
    }

    public function set_user_id($user_id)
    {
        $this->set_default_property(self :: PROPERTY_USER_ID, $user_id);
    }

    public function get_setting_id()
    {
        return $this->get_default_property(self :: PROPERTY_SETTING_ID);
    }

    public function set_setting_id($setting_id)
    {
        $this->set_default_property(self :: PROPERTY_SETTING_ID, $setting_id);
    }

    public function get_value()
    {
        return $this->get_default_property(self :: PROPERTY_VALUE);
    }

    public function set_value($value)
    {
        $this->set_default_property(self :: PROPERTY_VALUE, $value);
    }

    public static function get_table_name()
    {
        return Utilities :: get_classname_from_namespace(self :: CLASS_NAME, true);
    }

    public static function get_class_name()
    {
        return self :: CLASS_NAME;
    }

    /**
     *
     * @param string $variable
     * @param int $external_repository_id
     * @return mixed
     */
    public static function get($variable, $external_id, $user_id = null)
    {
        if (is_null($user_id) || ! is_numeric($user_id))
        {
            $user_id = Session :: get_user_id();
        }
        
        if (! isset(self :: $settings[$external_id][$user_id]))
        {
            self :: load($external_id, $user_id);
        }
        
        return (isset(self :: $settings[$external_id][$user_id][$variable]) ? self :: $settings[$external_id][$user_id][$variable] : null);
    }

    public static function load($external_id, $user_id)
    {
        $condition = new EqualityCondition(ExternalSetting :: PROPERTY_EXTERNAL_ID, $external_id);
        $settings = \repository\DataManager :: retrieves(ExternalSetting :: class_name(), $condition);
        
        $setting_ids = array();
        while ($setting = $settings->next_result())
        {
            $conditions = array();
            $conditions[] = new EqualityCondition(self :: PROPERTY_USER_ID, $user_id);
            $conditions[] = new EqualityCondition(self :: PROPERTY_SETTING_ID, $setting->get_id());
            $condition = new AndCondition($conditions);
            
            $parameters = new DataClassRetrievesParameters($condition, 1);
            $user_settings = \repository\DataManager :: retrieves(ExternalUserSetting :: class_name(), $parameters);
            
            if ($user_settings->size() == 1)
            {
                $user_setting = $user_settings->next_result();
                self :: $settings[$external_id][$user_id][$setting->get_variable()] = $user_setting->get_value();
            }
        }
    }

    public static function prepare_for_setting($external_id, $variable)
    {
        $user_setting = new ExternalUserSetting();
        $conditions[] = new EqualityCondition(ExternalSetting :: PROPERTY_EXTERNAL_ID, $external_id);
        $conditions[] = new EqualityCondition(ExternalSetting :: PROPERTY_VARIABLE, $variable);
        $conditions[] = new EqualityCondition(ExternalSetting :: PROPERTY_USER_SETTING, 1);
        $condition = new AndCondition($conditions);
        
        $parameters = new DataClassRetrievesParameters($condition, 1);
        $settings = \repository\DataManager :: retrieves(ExternalSetting :: class_name(), $parameters);
        
        if ($settings->size() > 0)
        {
            $user_setting->set_setting_id($settings->next_result()->get_id());
            return $user_setting;
        }
        else
        {
            return false;
        }
    }
}
