<?php

namespace repository;

use common\libraries\Translation;
use common\libraries\FormValidator;
use common\libraries\Utilities;
use common\libraries\Session;
use common\libraries\OptionsMenuRenderer;
use common\libraries\PlatformSetting;
use common\libraries\Request;

/**
 * Add a "resource" i.e. url to Chamilo.
 */
class AddResourceForm extends FormValidator
{
    
    const PROPERTY_URL = 'url';
    
    private $owner_id = 0;

    public function __construct($form_name, $method = 'post', $action = null, $defaults = array())
    {
        parent :: __construct($form_name, $method, $action);
        $this->owner_id = Session :: get_user_id();
        $this->build_form();
        $this->add_footer();
        $this->set_defaults($defaults);
    }

    protected function allows_category_selection()
    {
        return true;
    }

    /**
     * Gets the categories defined in the user's repository.
     * @return array The categories.
     */
    public function get_categories()
    {
        $categorymenu = new ContentObjectCategoryMenu($this->get_owner_id());
        $renderer = new OptionsMenuRenderer();
        $categorymenu->render($renderer, 'sitemap');
        return $renderer->toArray();
    }

    /**
     * Returns the ID of the owner of the learning object being created.
     *
     * @return int The ID.
     *
     */
    protected function get_owner_id()
    {
        return $this->owner_id;
    }

    /**
     * Sets the ID of the owner of the learning object being created.
     *
     * @param int The owner id.
     */
    protected function set_owner_id($owner_id)
    {
        $this->owner_id = $owner_id;
    }

    protected function build_form($htmleditor_options = array())
    {
        
        $this->addElement('category', Translation :: get('Properties'));
        $this->add_textfield(self :: PROPERTY_URL, Translation :: get('URL'), true, array('size' => '92'));
        $this->addElement('category');
        
        $this->addElement('category', Translation :: get('GeneralProperties'));
        
        $this->addElement('html', '<div id="message"></div>');
        $this->add_textfield(ContentObject :: PROPERTY_TITLE, Translation :: get('Title', array(), Utilities :: get_namespace_from_object($this)), true, array(
                'size' => '92', 'id' => 'title'));
        if ($this->allows_category_selection())
        {
            $select = $this->add_select(ContentObject :: PROPERTY_PARENT_ID, Translation :: get('CategoryTypeName'), $this->get_categories());
        
     //$select->setSelected($this->content_object->get_parent_id());
        //todo
        }
        $value = PlatformSetting :: get('description_required', 'repository');
        $required = ($value == 1) ? true : false;
        $this->add_html_editor(ContentObject :: PROPERTY_DESCRIPTION, Translation :: get('Description', array(), Utilities :: get_namespace_from_object($this)), $required, $htmleditor_options);
        
        $this->addElement('category');
        
        $this->addElement('category', Translation :: get('Bookmarklet'));
        
        $bookmarklet = new AddResourceBookmarklet();
        $this->addElement('html', $bookmarklet->display());
        
        $this->addElement('category');
    }

    public function set_defaults($defaults = array())
    {
        if (empty($defaults))
        {
            if (Request :: post('init'))
            {
                if ($value = Request :: post(self :: PROPERTY_URL))
                {
                    $defaults[self :: PROPERTY_URL] = urldecode($value);
                }
                if ($value = Request :: post(ContentObject :: PROPERTY_TITLE))
                {
                    $defaults[ContentObject :: PROPERTY_TITLE] = urldecode($value);
                }
                if ($value = Request :: post(ContentObject :: PROPERTY_DESCRIPTION))
                {
                    $defaults[ContentObject :: PROPERTY_DESCRIPTION] = urldecode($value);
                }
            }
            else
            {
                if ($url = Request :: get(self :: PROPERTY_URL))
                {
                    $defaults[self :: PROPERTY_URL] = $url;
                }
                if ($title = Request :: get(ContentObject :: PROPERTY_TITLE))
                {
                    $defaults[ContentObject :: PROPERTY_TITLE] = $title;
                }
                if ($description = Request :: get(ContentObject :: PROPERTY_DESCRIPTION))
                {
                    $defaults[ContentObject :: PROPERTY_DESCRIPTION] = $description;
                }
            }
        }
        $this->setDefaults($defaults);
    }

    /**
     * Adds a footer to the form, including a submit button.
     */
    protected function add_footer()
    {
        //$this->addElement('html', ResourceManager :: get_instance()->get_resource_html(Path :: get(WEB_PATH) . 'common/libraries/resources/javascript/content_object_form.js'));
        $buttons = array();
        $buttons[] = $this->createElement('style_submit_button', 'submit', Translation :: get('Create', null, Utilities :: COMMON_LIBRARIES), array(
                'class' => 'positive'));
        //$buttons[] = $this->createElement('style_reset_button', 'reset', Translation :: get('Reset', null, Utilities :: COMMON_LIBRARIES), array('class' => 'normal empty'));
        $this->addGroup($buttons, 'buttons', null, '&nbsp;', false);
    }

}
