<?php
namespace repository;

use common\libraries\AndCondition;
use common\libraries\EqualityCondition;
use common\libraries\Utilities;
use common\libraries\Request;
use common\libraries\Application;
use common\libraries\Translation;

/**
 * Description of content_object_share_browser
 * 
 * @author Pieterjan Broekaert
 */
class RepositoryManagerContentObjectShareRightsDeleterComponent extends RepositoryManager
{

    /**
     * Runs this component and displays its output.
     */
    public function run()
    {
        $content_object_id = Request :: get(RepositoryManager :: PARAM_CONTENT_OBJECT_ID);
        $failures = 0;
        $is_in_shared_objects_browser = false; // to redirect to the right page
        
        $user_ids = Request :: get(RepositoryManager :: PARAM_TARGET_USER);
        if (! is_null($user_ids) && ! is_array($user_ids))
        {
            $user_ids = array($user_ids);
        }
        
        $group_ids = Request :: get(RepositoryManager :: PARAM_TARGET_GROUP);
        if (! is_null($group_ids) && ! is_array($group_ids))
        {
            $group_ids = array($group_ids);
        }
        
        if ($user_ids) // delete the shares with selected users
        {
            foreach ($user_ids as $user_id)
            {
                $conditions = array();
                $conditions[] = new EqualityCondition(ContentObjectUserShare :: PROPERTY_CONTENT_OBJECT_ID, 
                        $content_object_id);
                $conditions[] = new EqualityCondition(ContentObjectUserShare :: PROPERTY_USER_ID, $user_id);
                $condition = new AndCondition($conditions);
                
                $content_object_user_shares = DataManager :: retrieves(ContentObjectUserShare :: class_name(), 
                        $condition);
                while ($content_object_user_share = $content_object_user_shares->next_result())
                {
                    if (! $content_object_user_share->delete())
                    {
                        $failures ++;
                    }
                }
            }
        }
        else 
            if ($group_ids) // delete the shares with selected groups
            {
                foreach ($group_ids as $group_id)
                {
                    $conditions = array();
                    $conditions[] = new EqualityCondition(ContentObjectGroupShare :: PROPERTY_CONTENT_OBJECT_ID, 
                            $content_object_id);
                    $conditions[] = new EqualityCondition(ContentObjectGroupShare :: PROPERTY_GROUP_ID, $group_id);
                    $condition = new AndCondition($conditions);
                    
                    $content_object_group_shares = DataManager :: retrieves(ContentObjectGroupShare :: class_name(), 
                            $condition);
                    while ($content_object_group_share = $content_object_group_shares->next_result())
                    {
                        if (! $content_object_group_share->delete())
                        {
                            $failures ++;
                        }
                    }
                }
            }
            else // if no group or user selected, delete the share for everyone
                 // (delete was called upon in shared objects browser)
            {
                $is_in_shared_objects_browser = true;
                if (! is_array($content_object_id))
                {
                    $content_object_id = array($content_object_id);
                }
                foreach ($content_object_id as $id)
                {
                    $condition = new EqualityCondition(ContentObjectUserShare :: PROPERTY_CONTENT_OBJECT_ID, 
                            $content_object_id);
                    $content_object_user_shares = DataManager :: retrieves(ContentObjectUserShare :: class_name(), 
                            $condition);
                    while ($content_object_user_share = $content_object_user_shares->next_result())
                    {
                        if (! $content_object_user_share->delete())
                        {
                            $failures ++;
                        }
                    }
                    
                    $condition = new EqualityCondition(ContentObjectGroupShare :: PROPERTY_CONTENT_OBJECT_ID, 
                            $content_object_id);
                    $content_object_group_shares = DataManager :: retrieves(ContentObjectGroupShare :: class_name(), 
                            $condition);
                    while ($content_object_group_share = $content_object_group_shares->next_result())
                    {
                        if (! $content_object_group_share->delete())
                        {
                            $failures ++;
                        }
                    }
                }
            }
        
        if ($failures > 0)
        {
            $message = Translation :: get('ObjectsNotDeleted', 
                    array('OBJECTS' => Translation :: get('SharedContentObjects')), Utilities :: COMMON_LIBRARIES);
        }
        
        else
        {
            $message = Translation :: get('ObjectsDeleted', 
                    array('OBJECTS' => Translation :: get('SharedContentObjects')), Utilities :: COMMON_LIBRARIES);
        }
        
        $parameters = $this->get_parameters();
        $parameters[Application :: PARAM_ACTION] = ($is_in_shared_objects_browser ? RepositoryManager :: ACTION_BROWSE_SHARED_CONTENT_OBJECTS : RepositoryManager :: ACTION_EDIT_CONTENT_OBJECT_SHARE_RIGHTS);
        
        $this->redirect($message, ($failures ? true : false), $parameters);
    }

    public function get_additional_parameters()
    {
        $parameters[] = ContentObjectUserShare :: PROPERTY_USER_ID;
        $parameters[] = ContentObjectGroupShare :: PROPERTY_GROUP_ID;
        $parameters[] = RepositoryManager :: PARAM_CONTENT_OBJECT_ID;
        $parameters[] = RepositoryManager :: PARAM_CATEGORY_ID;
        $parameters[] = ContentObjectShare :: PARAM_TYPE;
        $parameters[] = RepositoryManager :: PARAM_SHARED_VIEW;
        
        return $parameters;
    }
}
