<?php
namespace repository;

use common\libraries\Translation;
use common\libraries\ToolbarItem;
use common\libraries\Toolbar;
use common\libraries\Theme;
use common\libraries\Utilities;

/**
 * $Id: external_link_browser_table_cell_renderer.class.php 204 2009-11-13
 * 12:51:30Z kariboe $
 *
 * @package repository.lib.repository_manager.component.link_browser
 */

/**
 * Cell rendere for the content object browser table
 */
class ExternalLinkBrowserTableCellRenderer extends DefaultExternalLinkTableCellRenderer
{
    /**
     * The repository browser component
     */
    private $browser;

    /**
     * Constructor
     *
     * @param $browser RepositoryManagerBrowserComponent           
     */
    public function __construct($browser)
    {
        parent :: __construct($browser);
        $this->browser = $browser;
    }
    
    // Inherited
    public function render_cell($column, $object)
    {
        if ($column === ExternalLinkBrowserTableColumnModel :: get_modification_column())
        {
            return $this->get_modification_links($object);
        }
        
        return parent :: render_cell($column, $object);
    }

    /**
     * Gets the action links to display
     *
     * @param $content_object ContentObject
     *            The content object for which the
     *            action links should be returned
     * @return string A HTML representation of the action links
     */
    private function get_modification_links($object)
    {
        
        $toolbar = new Toolbar();
        if ($object->get_state() == ExternalSync :: STATE_ACTIVE)
        {
            $toolbar->add_item(new ToolbarItem(Translation :: get('View', null, Utilities :: COMMON_LIBRARIES), Theme :: get_common_image_path() . 'action_details.png', $this->browser->get_external_instance_viewing_url($object), ToolbarItem :: DISPLAY_ICON));
        }
        else
        {
            $toolbar->add_item(new ToolbarItem(Translation :: get('ExternalInstanceViewNotAllowed'), Theme :: get_common_image_path() . 'action_details_na.png', null, ToolbarItem :: DISPLAY_ICON));
                }
        return $toolbar->as_html();
    
    }
}
