<?php
namespace repository;

use common\libraries\Utilities;
use common\libraries\Toolbar;

class RepositoryBrowserGalleryTableCellRenderer extends DefaultContentObjectGalleryTableCellRenderer
{
    /**
     * The repository browser component
     */
    private $browser;

    /**
     * Constructor
     * 
     * @param $browser RepositoryManagerBrowserComponent           
     */
    public function __construct($browser)
    {
        parent :: __construct();
        $this->browser = $browser;
    }
    
    // Inherited
    public function render_cell($content_object)
    {
        $html = array();
        
        $html[] = parent :: render_cell($content_object);
        $html[] = '<div class="thumbnail_action">';
        $html[] = $this->get_modification_links($content_object);
        $html[] = '</div>';
        
        return implode("\n", $html);
    }

    public function get_cell_content(ContentObject $content_object)
    {
        $display = ContentObjectRenditionImplementation :: factory($content_object, ContentObjectRendition :: FORMAT_HTML, ContentObjectRendition :: VIEW_THUMBNAIL, $this->browser);
        
        $html[] = '<h4>' . Utilities :: truncate_string($content_object->get_title(), 25, false) . '</h4>';
        $html[] = '<a href="' . htmlentities($this->browser->get_content_object_viewing_url($content_object)) . '">' . $display->render() . '</a>';
        
        return implode("\n", $html);
    }

    private function get_modification_links($content_object)
    {
        $toolbar = new Toolbar();
        $toolbar->add_items($this->browser->get_content_object_actions($content_object));
        return $toolbar->as_html();
    }
}
