<?php
namespace repository;

use common\libraries\Utilities;
use common\libraries\Request;

class HtmlEditorFileComponent extends RepositoryManager
{
    const PARAM_PLUGIN = 'plugin';

    public function __construct($user)
    {
        parent :: __construct($user);
        $this->set_parameter('CKEditor', Request :: get('CKEditor'));
        $this->set_parameter('CKEditorFuncNum', Request :: get('CKEditorFuncNum'));
        $this->set_parameter('langCode', Request :: get('langCode'));
    }

    public function run()
    {
        $plugin = $this->get_plugin();
        $this->set_parameter(self :: PARAM_PLUGIN, $plugin);

        if (! \repository\viewer\Manager :: is_ready_to_be_published())
        {
            $class = 'repository\HtmlEditor' . Utilities :: underscores_to_camelcase($plugin) . 'RepositoryViewer';
            $repository_viewer = $class :: construct($this);
            $repository_viewer->set_maximum_select(\repository\viewer\Manager :: SELECT_SINGLE);
            // $this->content_object_types = call_user_func(array(get_class($repository_viewer)));
            $repository_viewer->run();
        }
        else
        {
            $processor = HtmlEditorProcessor :: factory(
                $plugin,
                $this,
                \repository\viewer\Manager :: get_selected_objects());

            $this->small_header();
            $processor->run();
            $this->small_footer();
        }
    }

    public function get_plugin()
    {
        return Request :: get(self :: PARAM_PLUGIN);
    }

    public function get_allowed_content_object_types()
    {
        $class = 'repository\HtmlEditor' . Utilities :: underscores_to_camelcase($this->get_plugin()) .
        'RepositoryViewer';
        return call_user_func(array($class, 'get_allowed_content_object_types'));
    }

    public function display_header()
    {
        $this->small_header();
    }

    public function display_footer()
    {
        $this->small_footer();

    }
}
