<?php
namespace repository;

use common\libraries\ArrayResultSet;
use common\libraries\ObjectTableDataProvider;

class LinkBrowserTableDataProvider extends ObjectTableDataProvider
{

    /**
     * The type of link
     *
     * @var Integer
     */
    private $type;

    public function __construct($browser, $condition, $type)
    {
        $this->type = $type;
        parent :: __construct($browser, $condition);
    }

    public function get_objects($offset, $count, $order_property = null)
    {
        $order_property = $this->get_order_property($order_property);
        if ($this->type == LinkBrowserTable :: TYPE_PUBLICATIONS)
        {
            return $this->get_browser()->get_object()->get_publications($count, $offset,
                    $order_property);
        }

        if ($this->type == LinkBrowserTable :: TYPE_PARENTS)
        {
            return $this->get_browser()->get_object()->get_parents($order_property, $offset, $count);
        }

        if ($this->type == LinkBrowserTable :: TYPE_CHILDREN)
        {
            return $this->get_browser()->get_object()->get_children($order_property, $offset, $count);
        }

        if ($this->type == LinkBrowserTable :: TYPE_ATTACHED_TO)
        {
            return new ArrayResultSet(
                    $this->get_browser()->get_object()->get_attachers($order_property, $offset, $count));
        }

        if ($this->type == LinkBrowserTable :: TYPE_ATTACHES)
        {
            return new ArrayResultSet(
                    $this->get_browser()->get_object()->get_attachments(ContentObject :: ATTACHMENT_NORMAL,
                            $order_property, $offset, $count));
        }

        if ($this->type == LinkBrowserTable :: TYPE_INCLUDED_IN)
        {
            return new ArrayResultSet(
                    $this->get_browser()->get_object()->get_includers($order_property, $offset, $count));
        }

        if ($this->type == LinkBrowserTable :: TYPE_INCLUDES)
        {
            return new ArrayResultSet($this->get_browser()->get_object()->get_includes($order_property, $offset, $count));
        }
    }

    /**
     * Gets the number of content objects in the table
     *
     * @return int
     */
    public function get_object_count()
    {
        if ($this->type == LinkBrowserTable :: TYPE_PUBLICATIONS)
        {
            return $this->get_browser()->get_object()->count_publications($this->get_browser()->get_user());
        }

        if ($this->type == LinkBrowserTable :: TYPE_PARENTS)
        {
            return $this->get_browser()->get_object()->count_parents();
        }

        if ($this->type == LinkBrowserTable :: TYPE_CHILDREN)
        {
            return $this->get_browser()->get_object()->count_children();
        }

        if ($this->type == LinkBrowserTable :: TYPE_ATTACHED_TO)
        {
            return $this->get_browser()->get_object()->count_attachers();
        }

        if ($this->type == LinkBrowserTable :: TYPE_ATTACHES)
        {
            return $this->get_browser()->get_object()->count_attachments();
        }

        if ($this->type == LinkBrowserTable :: TYPE_INCLUDED_IN)
        {
            return $this->get_browser()->get_object()->count_includers();
        }

        if ($this->type == LinkBrowserTable :: TYPE_INCLUDES)
        {
            return $this->get_browser()->get_object()->count_includes();
        }

        return 0;
    }
}
