<?php
namespace repository;

use common\libraries\DataClassRetrievesParameters;
use common\libraries\Request;
use common\libraries\InCondition;
use common\libraries\Translation;
use common\libraries\Utilities;
use common\libraries\BreadcrumbTrail;

/**
 * Repository manager component which can be used to share content objects
 */
class RepositoryManagerShareContentObjectsComponent extends RepositoryManager
{

    /**
     * Runs this component and displays its output.
     */
    public function run()
    {
        $object_ids = Request :: get(self :: PARAM_CONTENT_OBJECT_ID);

        if (! empty($object_ids))
        {
            if (! is_array($object_ids))
            {
                $object_ids = array($object_ids);
            }

            $condition = new InCondition(ContentObject :: PROPERTY_ID, $object_ids, ContentObject :: get_table_name());
            $parameters = new DataClassRetrievesParameters($condition);
            $content_objects = DataManager :: retrieve_active_content_objects(ContentObject :: class_name(),
                    $parameters);

            $sharer = ShareManager :: launch($this, $content_objects->as_array());
        }
        else
        {
            $this->display_warning_page(
                    htmlentities(
                            Translation :: get('NoObjectSelected',
                                    array('OBJECT' => Translation :: get('ContentObject')),
                                    Utilities :: COMMON_LIBRARIES)));
        }
    }

    public function add_additional_breadcrumbs(BreadcrumbTrail $breadcrumbtrail)
    {
        $breadcrumbtrail->add_help('repository_share_content_objects');
    }

    public function get_additional_parameters()
    {
        return array(self :: PARAM_CONTENT_OBJECT_ID);
    }
}
