<?php
namespace repository;

use common\libraries\Request;
use common\libraries\Translation;
use common\libraries\Utilities;
use common\libraries\EqualityCondition;
use common\libraries\AndCondition;
use common\libraries\Breadcrumb;
use rights\RightsLocationEntityRight;
use rights\DataManager as RightsDataManager;
use rights\UserEntity;
use common\libraries\BreadcrumbTrail;

/**
 * Default repository manager component which allows the user to delete a user entity share
 *
 * @author Pieterjan Broekaert Hogeschool Gent
 */
class RepositoryManagerSharedContentObjectsDeleterComponent extends RepositoryManager
{

    /**
     * Runs this component and displays its output.
     *
     * @todo remove code duplication with repository component mover component
     */
    public function run()
    {
        $ids = Request :: get(RepositoryManager :: PARAM_CONTENT_OBJECT_ID);
        if (! empty($ids))
        {
            if (! is_array($ids))
            {
                $ids = array($ids);
            }

            foreach ($ids as $id)
            {
                $content_object = \repository\DataManager :: retrieve_content_object($id);
                // retrieve location_id
                $location_id = RepositoryRights :: get_instance()->get_location_id_by_identifier_from_user_subtree(
                        RepositoryRights :: TYPE_USER_CONTENT_OBJECT, $id, $content_object->get_owner_id());

                // retrieve the rights entity right
                $conditions = array();
                $conditions[] = new EqualityCondition(RightsLocationEntityRight :: PROPERTY_ENTITY_TYPE,
                        UserEntity :: ENTITY_TYPE);
                $conditions[] = new EqualityCondition(RightsLocationEntityRight :: PROPERTY_LOCATION_ID, $location_id);
                $conditions[] = new EqualityCondition(RightsLocationEntityRight :: PROPERTY_ENTITY_ID,
                        $this->get_user_id());
                $condition = new AndCondition($conditions);

                // delete the share
                // DMTODO: this could probably be replaced by a generic retrieves by passing the appropriate
                // RightsLocation subclass (and using the corresponding DM).
                $rights_entity_right = RightsDataManager :: retrieve_rights_location_rights(
                        RepositoryManager :: APPLICATION_NAME, $condition)->next_result();
                if ($rights_entity_right)
                {
                    $rights_entity_right->set_context(RepositoryManager :: APPLICATION_NAME);

                    if (! $rights_entity_right->delete())
                    {
                        $failures ++;
                    }
                }
                else
                {
                    $failures ++;
                }

                // remove the share category link
                $share_category_relation_object = DataManager :: retrieve_shared_content_object_rel_category_for_user_and_content_object(
                        $this->get_user_id(), $id);
                if ($share_category_relation_object)
                {
                    if (! $share_category_relation_object->delete())
                    {
                        $failures ++;
                    }
                }
            }

            if ($failures)
            {
                if (count($ids) == 1)
                {
                    $message = Translation :: get('ObjectNotDeleted',
                            array('OBJECT' => Translation :: get('ContentObject')), Utilities :: COMMON_LIBRARIES);
                }
                else
                {
                    $message = Translation :: get('ObjectsNotDeleted',
                            array('OBJECTS' => Translation :: get('ContentObjects')), Utilities :: COMMON_LIBRARIES);
                }
            }
            else
            {
                if (count($ids) == 1)
                {
                    $message = Translation :: get('ObjectDeleted',
                            array('OBJECT' => Translation :: get('ContentObject')), Utilities :: COMMON_LIBRARIES);
                }
                else
                {
                    $message = Translation :: get('ObjectsDeleted',
                            array('OBJECTS' => Translation :: get('ContentObjects')), Utilities :: COMMON_LIBRARIES);
                }
            }

            $parameters = array();
            $parameters[RepositoryManager :: PARAM_ACTION] = RepositoryManager :: ACTION_BROWSE_SHARED_CONTENT_OBJECTS;
            $this->redirect($message, ($failures ? true : false), $parameters);
        }
        else
        {
            $this->display_error_page(
                    htmlentities(
                            Translation :: get('NoObjectSelected',
                                    array('OBJECT' => Translation :: get('ContentObject')),
                                    Utilities :: COMMON_LIBRARIES)));
        }
    }

    public function add_additional_breadcrumbs(BreadcrumbTrail $breadcrumbtrail)
    {
        $breadcrumbtrail->add(
                new Breadcrumb(
                        $this->get_url(
                                array(
                                        RepositoryManager :: PARAM_ACTION => RepositoryManager :: ACTION_BROWSE_SHARED_CONTENT_OBJECTS)),
                        Translation :: get('RepositoryManagerSharedContentObjectBrowserComponent')));
        $breadcrumbtrail->add_help('share_mover');
    }

    public function get_additional_parameters()
    {
        return array(self :: PARAM_SHARED_VIEW, RepositoryManager :: PARAM_CONTENT_OBJECT_ID);
    }
}
