<?php
namespace repository;

use common\libraries\Application;
use common\libraries\AdministrationComponent;
use common\libraries\Breadcrumb;
use common\libraries\BreadcrumbTrail;
use common\libraries\FileProperties;
use common\libraries\Filesystem;
use common\libraries\InCondition;
use common\libraries\Path;
use common\libraries\Request;
use common\libraries\StringUtilities;
use common\libraries\Theme;
use common\libraries\Translation;
use common\libraries\Utilities;

/**
 * $Id: template_importer.class.php 204 2009-11-13 12:51:30Z kariboe $
 *
 * @package repository.lib.repository_manager.component
 */
/**
 * Repository manager component which provides functionality to delete an object from the users repository.
 */
class RepositoryManagerTemplateImporterComponent extends RepositoryManager implements AdministrationComponent
{

    /**
     * Runs this component and displays its output.
     */
    public function run()
    {
        $extra_params = array();

        $dummy_user_id = 0;

        $type = Request :: get(self :: PARAM_IMPORT_TYPE);

        if ($type)
        {
            // $import_form = new ContentObjectImportForm('import', 'post', $this->get_url($extra_params), 0, $user,
            // null, false);
            $import_form = ContentObjectImportForm :: factory($type, $this, 'post', $this->get_url($extra_params),
                    FALSE);

            if ($import_form->validate())
            {
                $values = $import_form->exportValues();
                $parent_id = $values[ContentObject :: PROPERTY_PARENT_ID];
                $new_category_name = $values[ContentObjectImportForm :: NEW_CATEGORY];

                if (! StringUtilities :: is_null_or_empty($new_category_name, true))
                {
                    $new_category = new RepositoryCategory();
                    $new_category->set_name($new_category_name);
                    $new_category->set_parent($parent_id);
                    $new_category->set_user_id($this->get_user_id());
                    $new_category->set_type(RepositoryCategory :: TYPE_NORMAL);
                    if (! $new_category->create())
                    {
                        throw new \Exception(Translation :: get('CategoryCreationFailed'));
                    }
                    else
                    {
                        $category_id = $new_category->get_id();
                    }
                }
                else
                {
                    $category_id = $parent_id;
                }

                if (isset($_FILES[ContentObjectImportForm :: IMPORT_FILE_NAME]))
                {
                    $file = FileProperties :: from_upload($_FILES[ContentObjectImportForm :: IMPORT_FILE_NAME]);
                }
                else
                {
                    $file = null;
                }

                $parameters = ImportParameters :: factory(
                        $import_form->exportValue(ContentObjectImportForm :: PROPERTY_TYPE), $dummy_user_id,
                        $category_id, $file, $values);
                $controller = ContentObjectImportController :: factory($parameters);
                $co_ids = $controller->run();

                $condition = new InCondition(ContentObject :: PROPERTY_ID, $co_ids);
                $content_objects = DataManager :: retrieve_active_content_objects(ContentObject :: class_name(),
                        $condition);
                $success = true;
                while ($content_object = $content_objects->next_result())
                {
                    $content_object->set_owner_id($dummy_user_id);
                    $content_object->update() ? $success : $success = false;
                }
                // $content_object = $import_form->import_content_object();

                if (! $co_ids || count($co_ids) < 1 || ! $success)
                {
                    $message = Translation :: get('ObjectNotImported',
                            array('OBJECT' => Translation :: get('ContentObject')), Utilities :: COMMON_LIBRARIES);
                }
                else
                {
                    $message = Translation :: get('ObjectImported',
                            array('OBJECT' => Translation :: get('ContentObject')), Utilities :: COMMON_LIBRARIES);
                }

                $this->redirect($message, ! $co_ids || count($co_ids) < 1,
                        array(Application :: PARAM_ACTION => RepositoryManager :: ACTION_BROWSE_TEMPLATES));
            }
            else
            {
                $this->display_header();
                $import_form->display();
                $this->display_footer();
            }
        }
        else
        {
            BreadcrumbTrail :: get_instance()->add(
                    new Breadcrumb($this->get_url(), Translation :: get('ChooseImportFormat')));

            $this->display_header();

            $html = array();
            foreach ($this->get_types() as $type => $name)
            {
                $html[] = '<a href="' . $this->get_url(array(self :: PARAM_IMPORT_TYPE => $type)) . '">';
                $html[] = '<div class="create_block" style="background-image: url(' . Theme :: get_image_path() . 'import/' . $type . '.png);">';
                $html[] = $name;
                $html[] = '</div>';
                $html[] = '</a>';
            }
            echo implode("\n", $html);

            $this->display_footer();
        }
    }

    public function add_additional_breadcrumbs(BreadcrumbTrail $breadcrumbtrail)
    {
        $breadcrumbtrail->add_help('template_importer');
    }

    public function get_additional_parameters()
    {
        $parameters = array();
        if (Request :: get(self :: PARAM_IMPORT_TYPE))
        {
            $parameters[] = self :: PARAM_IMPORT_TYPE;
        }
        return $parameters;
    }

    public function get_types()
    {
        $types_folder = Path :: get_repository_path() . 'lib/content_object_import/';
        $folders = Filesystem :: get_directory_content($types_folder, Filesystem :: LIST_DIRECTORIES, false);
        $types = array();
        foreach ($folders as $folder)
        {
            $types[$folder] = Translation :: get('ImportType' . Utilities :: underscores_to_camelcase($folder));
        }
        return $types;
    }
}
