<?php
namespace repository;

use common\libraries\Request;
use common\libraries\ActionBarRenderer;
use common\libraries\Theme;
use common\libraries\Translation;
use common\libraries\Utilities;
use common\libraries\DynamicVisualTabsRenderer;
use common\libraries\DynamicVisualTab;
use common\libraries\ToolbarItem;

use user\UserManager;
use group\GroupManager;
use common\libraries\BreadcrumbTrail;

/**
 * Browser for content object share rights
 *
 * @author Pieterjan Broekaert
 * @author Sven Vanpoucke
 */
class ShareManagerBrowserComponent extends ShareManager
{
    const PARAM_TYPE = 'type';

    const TAB_DETAILS = 0;
    const TAB_SUBGROUPS = 1;

    const TYPE_USER = 'user';
    const TYPE_GROUP = 'group';

    private $type;

    /**
     * Runs this component and displays its output.
     */
    public function run()
    {
        $this->type = Request :: get(self :: PARAM_TYPE);
        if (empty($this->type))
        {
            $this->type = self :: TYPE_USER;
        }

        $this->action_bar = $this->get_action_bar();

        //display the component
        $this->display_header();
        echo $this->action_bar->as_html();
        echo $this->display_content_objects();
        $this->display_body();
        $this->display_footer();
    }

    /**
     * Displays the body
     */
    private function display_body()
    {
        switch ($this->type)
        {
            case self :: TYPE_USER :
                $table = $this->get_users_browser_html();
                break;
            case self :: TYPE_GROUP :
                $table = $this->get_groups_browser_html();
                break;
            default :
                $table = '';
                break;
        }

        $renderer_name = Utilities :: get_classname_from_namespace(__CLASS__, true);
        $tabs = new DynamicVisualTabsRenderer($renderer_name, $table);

        $label = htmlentities(Translation :: get('Users', null, UserManager :: APPLICATION_NAME));
        $link = $this->get_url(array(self :: PARAM_TYPE => self :: TYPE_USER));
        $tabs->add_tab(new DynamicVisualTab('users', $label, Theme :: get_image_path(UserManager :: APPLICATION_NAME) . 'logo/' . Theme :: ICON_SMALL . '.png', $link, ($this->type == self :: TYPE_USER)));

        $label = htmlentities(Translation :: get('Groups', null, GroupManager :: APPLICATION_NAME));
        $link = $this->get_url(array(self :: PARAM_TYPE => self :: TYPE_GROUP));
        $tabs->add_tab(new DynamicVisualTab('users', $label, Theme :: get_image_path(GroupManager :: APPLICATION_NAME) . 'logo/' . Theme :: ICON_SMALL . '.png', $link, ($this->type == self :: TYPE_GROUP)));

        echo $tabs->render();
    }

    /**
     * Displays the users you shared with and the rights
     */
    private function get_users_browser_html()
    {
        $browser_table = new ContentObjectUserShareRightsBrowserTable($this, $this->get_parameters(), $condition);
        return $browser_table->as_html();
    }

    /**
     * Displays the groups you shared with and the rights
     */
    private function get_groups_browser_html()
    {
        $browser_table = new ContentObjectGroupShareRightsBrowserTable($this, $this->get_parameters(), $condition);

        return $browser_table->as_html();
    }

    /**
     * create an action bar
     */
    public function get_action_bar()
    {
        $action_bar = new ActionBarRenderer(ActionBarRenderer :: TYPE_HORIZONTAL);

        $action_bar->add_common_action(new ToolbarItem(Translation :: get('ShareWithOtherUsersGroups'), Theme :: get_common_image_path() . 'action_rights.png', $this->get_url(array(
                self :: PARAM_SHARE_ACTION => self :: ACTION_ADD_ENTITIES))), ToolbarItem :: DISPLAY_ICON_AND_LABEL);

        return $action_bar;
    }

    /**
     * add additional breadcrumbs before the auto generated share_rights_browser breadcrumb
     * @param BreadcrumbTrail $breadcrumbtrail
     */
    public function add_additional_breadcrumbs(BreadcrumbTrail $breadcrumbtrail)
    {
        $breadcrumbtrail->add_help("repository_content_object_share_rights_browser");
    }

    public function get_additional_parameters()
    {
        return array(self :: PARAM_TYPE, RepositoryManager :: PARAM_CONTENT_OBJECT_ID);
    }

}
