<?php
namespace repository;

use common\libraries\Translation;
use common\libraries\Utilities;
use common\libraries\EqualityCondition;
use common\libraries\DataClass;
use common\libraries\NotCondition;
use common\libraries\AndCondition;

/**
 * $Id: user_view.class.php 204 2009-11-13 12:51:30Z kariboe $
 * 
 * @package repository.lib
 */
/**
 *
 * @author Sven Vanpoucke
 */
class UserView extends DataClass
{
    const CLASS_NAME = __CLASS__;
    const PROPERTY_USER_ID = 'user_id';
    const PROPERTY_NAME = 'name';
    const PROPERTY_DESCRIPTION = 'description';

    /**
     * Get the default properties of all user_views.
     * 
     * @return array The property names.
     */
    public static function get_default_property_names()
    {
        return parent :: get_default_property_names(
                array(self :: PROPERTY_NAME, self :: PROPERTY_DESCRIPTION, self :: PROPERTY_USER_ID));
    }

    /**
     * Returns the name of this user_view.
     * 
     * @return String The name
     */
    public function get_name()
    {
        return $this->get_default_property(self :: PROPERTY_NAME);
    }

    /**
     * Returns the description of this user_view.
     * 
     * @return String The description
     */
    public function get_description()
    {
        return $this->get_default_property(self :: PROPERTY_DESCRIPTION);
    }

    /**
     * Sets the name of this user_view.
     * 
     * @param String $name the name.
     */
    public function set_name($name)
    {
        $this->set_default_property(self :: PROPERTY_NAME, $name);
    }

    /**
     * Sets the description of this user_view.
     * 
     * @param String $description the description.
     */
    public function set_description($description)
    {
        $this->set_default_property(self :: PROPERTY_DESCRIPTION, $description);
    }

    public function get_user_id()
    {
        return $this->get_default_property(self :: PROPERTY_USER_ID);
    }

    public function set_user_id($user_id)
    {
        $this->set_default_property(self :: PROPERTY_USER_ID, $user_id);
    }

    public function delete()
    {
        $condition = new EqualityCondition(UserViewRelContentObject :: PROPERTY_VIEW_ID, $this->get_id());
        $relations = DataManager :: retrieves(UserViewRelContentObject :: class_name(), $condition);
        
        while ($relation = $relations->next_result())
        {
            if (! $relation->delete())
            {
                return false;
            }
        }
        return parent :: delete();
    }

    public function create($values)
    {
        $conditions = array();
        $conditions[] = new EqualityCondition(self :: PROPERTY_NAME, $this->get_name());
        $conditions[] = new EqualityCondition(self :: PROPERTY_USER_ID, $this->get_user_id());
        $condition = new AndCondition($conditions);
        $views = DataManager :: count(UserView :: class_name(), $condition);
        if ($views > 0)
        {
            $this->add_error(Translation :: get('UserViewNameNotUnique'));
            return false;
        }
        
        $success = parent :: create();
        
        $registrations = DataManager :: get_registered_types();
        foreach ($registrations as $registration)
        {
            $uvrlo = new UserViewRelContentObject();
            $uvrlo->set_view_id($this->get_id());
            $uvrlo->set_content_object_type($registration);
            
            if (in_array($registration, $values))
            {
                $uvrlo->create();
            }
        }
        
        return $success;
    }

    public function update($values)
    {
        $conditions[] = new EqualityCondition(self :: PROPERTY_NAME, $this->get_name());
        $conditions[] = new NotCondition(new EqualityCondition(self :: PROPERTY_ID, $this->get_id()));
        $conditions[] = new EqualityCondition(self :: PROPERTY_USER_ID, $this->get_user_id());
        $condition = new AndCondition($conditions);
        
        $views = DataManager :: count(UserView :: class_name(), $condition);
        if ($views > 0)
        {
            $this->add_error(Translation :: get('UserViewNameNotUnique'));
            return false;
        }
        
        return parent :: update();
    }

    public static function get_table_name()
    {
        return Utilities :: get_classname_from_namespace(self :: CLASS_NAME, true);
    }
}
