<?php
namespace repository;

use common\libraries\Utilities;
use common\libraries\Translation;
use common\libraries\ObjectTableFormAction;
use common\libraries\ObjectTable;

/**
 * $Id: content_object_table.class.php 191 2009-11-13 11:50:28Z chellee $
 *
 * @package application.common.repo_viewer.component.content_object_table
 */
/**
 * This class represents a table with learning objects which are candidates for publication.
 */
class HandbookItemContentObjectTable extends ObjectTable
{
    const DEFAULT_NAME = 'handbook_item_content_object_table';

    /**
     * Constructor.
     *
     * @param int $owner The id of the current user.
     * @param array $types The types of objects that can be published in current location.
     * @param string $query The search query, or null if none.
     * @param string $publish_url_format URL for publishing the selected learning object.
     * @param string $edit_and_publish_url_format URL for editing and publishing the selected learning object.
     * @see PublicationCandidateTableCellRenderer::PublicationCandidateTableCellRenderer()
     */
    public function __construct($parent, $owner, $types, $query, $table_actions)
    {
        $data_provider = new HandbookItemContentObjectTableDataProvider($owner, $types, $query, $parent);
        $column_model = new HandbookItemContentObjectTableColumnModel();
        $cell_renderer = new HandbookItemContentObjectTableCellRenderer($parent);
        parent :: __construct(
            $data_provider,
            HandbookItemContentObjectTable :: DEFAULT_NAME,
            $column_model,
            $cell_renderer);

        if ($parent->get_maximum_select() != \repository\viewer\Manager :: SELECT_SINGLE)
        {
            $actions = array();
            $actions[] = new ObjectTableFormAction(
                \repository\viewer\Manager :: PARAM_PUBLISH_SELECTED,
                Translation :: get('PublishSelected', null, Utilities :: COMMON_LIBRARIES),
                false);
        }

        $this->set_form_actions($actions);
    }
}
