<?php
namespace repository\quota;

use common\libraries\DataClassRetrievesParameters;
use common\libraries\StaticConditionVariable;
use common\libraries\PropertyConditionVariable;
use common\libraries\DataClassCountParameters;
use common\libraries\Utilities;
use common\libraries\DatetimeUtilities;
use repository\ContentObject;
use common\libraries\PropertiesTable;
use common\libraries\InCondition;
use common\libraries\AndCondition;
use common\libraries\EqualityCondition;
use common\libraries\NewObjectTableSupport;
use common\libraries\Theme;
use common\libraries\DynamicContentTab;
use common\libraries\DynamicTabsRenderer;
use common\libraries\PlatformSetting;
use common\libraries\ToolbarItem;
use common\libraries\ActionBarRenderer;
use common\libraries\Filesystem;
use common\libraries\Translation;

class BrowserComponent extends Manager implements NewObjectTableSupport
{

    private $table_type;

    private $calculator;

    public function run()
    {
        $reset_cache = (bool) \common\libraries\Request :: get(self :: PARAM_RESET_CACHE);
        $this->calculator = new Calculator($this->get_user(), $reset_cache);

        $this->display_header();

        echo $this->get_action_bar()->as_html();
        $user_quota = array();
        $user_quota[] = '<h3>' . htmlentities(Translation :: get('UsedDiskSpace')) . '</h3>';
        $user_quota[] = Calculator :: get_bar(
            $this->calculator->get_user_disk_quota_percentage(),
            Filesystem :: format_file_size($this->calculator->get_used_user_disk_quota()) . ' / ' . Filesystem :: format_file_size(
                $this->calculator->get_maximum_user_disk_quota()));
        $user_quota[] = '<div style="clear: both;">&nbsp;</div>';

        $user_quota[] = $this->get_statistics();
        $user_quota[] = '<div style="clear: both;">&nbsp;</div>';

        $condition = new EqualityCondition(Request :: PROPERTY_USER_ID, $this->get_user_id());
        $user_requests = DataManager :: count(Request :: class_name(), $condition);

        if ($user_requests > 0 || \repository\quota\rights\Rights :: get_instance()->quota_is_allowed())
        {
            $tabs = new DynamicTabsRenderer('quota');
            $tabs->add_tab(
                new DynamicContentTab(
                    'personal',
                    Translation :: get('Personal'),
                    Theme :: get_image_path() . 'tab/personal.png',
                    implode("\n", $user_quota)));

            if ($user_requests > 0)
            {
                $this->table_type = RequestTable :: TYPE_PERSONAL;
                $table = new RequestTable($this);
                $tabs->add_tab(
                    new DynamicContentTab(
                        'personal_request',
                        Translation :: get('YourRequests'),
                        Theme :: get_image_path() . 'tab/personal_request.png',
                        $table->as_html()));
            }

            if (\repository\quota\rights\Rights :: get_instance()->quota_is_allowed())
            {
                $platform_quota = array();
                $platform_quota[] = '<h3>' . htmlentities(Translation :: get('AggregatedUserDiskQuotas')) . '</h3>';
                $platform_quota[] = Calculator :: get_bar(
                    $this->calculator->get_aggregated_user_disk_quota_percentage(),
                    Filesystem :: format_file_size($this->calculator->get_used_aggregated_user_disk_quota()) . ' / ' . Filesystem :: format_file_size(
                        $this->calculator->get_maximum_aggregated_user_disk_quota()));
                $platform_quota[] = '<div style="clear: both;">&nbsp;</div>';

                $platform_quota[] = '<h3>' . htmlentities(Translation :: get('ReservedDiskSpace')) . '</h3>';
                $platform_quota[] = Calculator :: get_bar(
                    $this->calculator->get_reserved_disk_space_percentage(),
                    Filesystem :: format_file_size($this->calculator->get_used_reserved_disk_space()) . ' / ' . Filesystem :: format_file_size(
                        $this->calculator->get_maximum_reserved_disk_space()));
                $platform_quota[] = '<div style="clear: both;">&nbsp;</div>';

                $platform_quota[] = '<h3>' . htmlentities(Translation :: get('AllocatedDiskSpace')) . '</h3>';
                $platform_quota[] = Calculator :: get_bar(
                    $this->calculator->get_allocated_disk_space_percentage(),
                    Filesystem :: format_file_size($this->calculator->get_used_allocated_disk_space()) . ' / ' . Filesystem :: format_file_size(
                        $this->calculator->get_maximum_allocated_disk_space()));
                $platform_quota[] = '<div style="clear: both;">&nbsp;</div>';

                $tabs->add_tab(
                    new DynamicContentTab(
                        'platform',
                        Translation :: get('Platform'),
                        Theme :: get_image_path() . 'tab/platform.png',
                        implode("\n", $platform_quota)));

                $target_users = \repository\quota\rights\Rights :: get_instance()->get_target_users($this->get_user());

                if (count($target_users) > 0)
                {
                    $target_condition = new InCondition(Request :: PROPERTY_USER_ID, $target_users);
                }
                else
                {
                    $target_condition = new EqualityCondition(Request :: PROPERTY_USER_ID, - 1);
                }

                $conditions = array();
                $conditions[] = new EqualityCondition(Request :: PROPERTY_DECISION, Request :: DECISION_PENDING);
                if (! $this->get_user()->is_platform_admin())
                {
                    $conditions[] = $target_condition;
                }
                $condition = new AndCondition($conditions);

                if (DataManager :: count(Request :: class_name(), $condition) > 0)
                {
                    $this->table_type = RequestTable :: TYPE_PENDING;
                    $table = new RequestTable($this);
                    $tabs->add_tab(
                        new DynamicContentTab(
                            RequestTable :: TYPE_PENDING,
                            Translation :: get('PendingRequests'),
                            Theme :: get_image_path() . 'decision/22/' . Request :: DECISION_PENDING . '.png',
                            $table->as_html()));
                }

                $conditions = array();
                $conditions[] = new EqualityCondition(Request :: PROPERTY_DECISION, Request :: DECISION_GRANTED);
                if (! $this->get_user()->is_platform_admin())
                {
                    $conditions[] = $target_condition;
                }
                $condition = new AndCondition($conditions);
                if (DataManager :: count(Request :: class_name(), $condition) > 0)
                {
                    $this->table_type = RequestTable :: TYPE_GRANTED;
                    $table = new RequestTable($this);
                    $tabs->add_tab(
                        new DynamicContentTab(
                            RequestTable :: TYPE_GRANTED,
                            Translation :: get('GrantedRequests'),
                            Theme :: get_image_path() . 'decision/22/' . Request :: DECISION_GRANTED . '.png',
                            $table->as_html()));
                }

                $conditions = array();
                $conditions[] = new EqualityCondition(Request :: PROPERTY_DECISION, Request :: DECISION_DENIED);
                if (! $this->get_user()->is_platform_admin())
                {
                    $conditions[] = $target_condition;
                }
                $condition = new AndCondition($conditions);

                if (DataManager :: count(Request :: class_name(), $condition) > 0)
                {
                    $this->table_type = RequestTable :: TYPE_DENIED;
                    $table = new RequestTable($this);
                    $tabs->add_tab(
                        new DynamicContentTab(
                            RequestTable :: TYPE_DENIED,
                            Translation :: get('DeniedRequests'),
                            Theme :: get_image_path() . 'decision/22/' . Request :: DECISION_DENIED . '.png',
                            $table->as_html()));
                }
            }
        }

        if ($user_requests > 0 || \repository\quota\rights\Rights :: get_instance()->quota_is_allowed())
        {
            echo $tabs->render();
        }
        else
        {
            echo implode("\n", $user_quota);
        }

        $this->display_footer();
    }

    public function get_statistics()
    {
        $html = array();
        $html[] = '<h3>' . htmlentities(Translation :: get('RepositoryStatistics')) . '</h3>';
        /**
         * Disabled right now since database quota are not enforced anywhere, this is just a visual reference $html[] =
         * Calculator :: get_bar($this->calculator->get_user_database_percentage(),
         * $this->calculator->get_used_database_quota() . ' / ' . $this->calculator->get_maximum_database_quota());
         */

        $properties = array();
        $properties[Translation :: get('NumberOfContentObjects')] = $this->calculator->get_used_database_quota();

        $type_counts = array();
        $condition = new EqualityCondition(
            new PropertyConditionVariable(ContentObject :: class_name(), ContentObject :: PROPERTY_OWNER_ID),
            new StaticConditionVariable($this->get_user_id()));
        $most_used = null;

        foreach (\repository\DataManager :: get_registered_types() as $type)
        {
            $type_counts[$type] = \repository\DataManager :: count_active_content_objects(
                $type,
                new DataClassCountParameters($condition));
            if ($type_counts[$type] > $type_counts[$most_used])
            {
                $most_used = $type;
            }
        }

        arsort($type_counts);

        $properties[Translation :: get('MostUsedContentObjectType')] = Translation :: get(
            'TypeName',
            null,
            Utilities :: get_namespace_from_classname($most_used)) . ' (' . $type_counts[$most_used] . ')';

        $reference_count = $type_counts[$most_used] / 2;

        unset($type_counts[$most_used]);

        $frequent = array();

        foreach ($type_counts as $type => $count)
        {
            if ($count >= $reference_count)
            {
                $frequent[] = Translation :: get('TypeName', null, Utilities :: get_namespace_from_classname($type)) .
                     ' (' . $count . ')';
            }
        }

        $properties[Translation :: get('OtherFrequentlyUsedContentObjectTypes')] = implode('<br />', $frequent);

        $properties[Translation :: get('AvailableDiskSpace')] = Filesystem :: format_file_size(
            $this->calculator->get_available_user_disk_quota());

        $condition = new EqualityCondition(ContentObject :: PROPERTY_OWNER_ID, $this->get_user_id());
        $oldest_object = \repository\DataManager :: retrieve_active_content_objects(
            ContentObject :: class_name(),
            new DataClassRetrievesParameters($condition))->next_result();

        if ($oldest_object instanceof ContentObject)
        {
            $properties[Translation :: get('OldestContentObject')] = '<a href="' .
                 $this->get_parent()->get_content_object_viewing_url($oldest_object) . '">' . $oldest_object->get_title() .
                 '</a> - ' . DatetimeUtilities :: format_locale_date(null, $oldest_object->get_creation_date());
        }

        $table = new PropertiesTable($properties);
        $html[] = '<div class="quota_statistics">';
        $html[] = $table->toHTML();
        $html[] = '</div>';
        return implode("\n", $html);
    }

    /**
     *
     * @see @see common\libraries.NewObjectTableSupport::get_object_table_condition()
     */
    public function get_object_table_condition($object_table_class_name)
    {
        $conditions = array();

        switch ($this->table_type)
        {
            case RequestTable :: TYPE_PENDING :
                $conditions[] = new EqualityCondition(Request :: PROPERTY_DECISION, Request :: DECISION_PENDING);
                break;
            case RequestTable :: TYPE_PERSONAL :
                $conditions[] = new EqualityCondition(Request :: PROPERTY_USER_ID, $this->get_user_id());
                break;
            case RequestTable :: TYPE_GRANTED :
                $conditions[] = new EqualityCondition(Request :: PROPERTY_DECISION, Request :: DECISION_GRANTED);
                break;
            case RequestTable :: TYPE_DENIED :
                $conditions[] = new EqualityCondition(Request :: PROPERTY_DECISION, Request :: DECISION_DENIED);
                break;
        }

        if (! $this->get_user()->is_platform_admin() &&
             \repository\quota\rights\Rights :: get_instance()->quota_is_allowed() &&
             $this->table_type != RequestTable :: TYPE_PERSONAL)
        {
            $target_users = \repository\quota\rights\Rights :: get_instance()->get_target_users($this->get_user());

            if (count($target_users) > 0)
            {
                $conditions[] = new InCondition(Request :: PROPERTY_USER_ID, $target_users);
            }
            else
            {
                $conditions[] = new EqualityCondition(Request :: PROPERTY_USER_ID, - 1);
            }
        }

        return new AndCondition($conditions);
    }

    public function get_action_bar()
    {
        $action_bar = new ActionBarRenderer(ActionBarRenderer :: TYPE_HORIZONTAL);

        $allow_upgrade = PlatformSetting :: get('allow_upgrade', __NAMESPACE__);
        $maximum_user_disk_space = PlatformSetting :: get('maximum_user', __NAMESPACE__);

        if ($this->calculator->upgrade_allowed())
        {
            $action_bar->add_common_action(
                new ToolbarItem(
                    Translation :: get('UpgradeQuota'),
                    Theme :: get_image_path() . 'action/upgrade.png',
                    $this->get_url(array(self :: PARAM_ACTION => self :: ACTION_UPGRADE))));
        }

        if ($this->calculator->request_allowed())
        {
            $action_bar->add_common_action(
                new ToolbarItem(
                    Translation :: get('RequestUpgrade'),
                    Theme :: get_image_path() . 'action/request.png',
                    $this->get_url(array(self :: PARAM_ACTION => self :: ACTION_CREATE))));
        }

        if ($this->get_user()->is_platform_admin())
        {
            $action_bar->add_tool_action(
                new ToolbarItem(
                    Translation :: get('ConfigureManagementRights'),
                    Theme :: get_image_path() . 'action/rights.png',
                    $this->get_url(array(self :: PARAM_ACTION => self :: ACTION_RIGHTS))));

            $action_bar->add_tool_action(
                new ToolbarItem(
                    Translation :: get('ResetTotal'),
                    Theme :: get_image_path() . 'action/reset.png',
                    $this->get_url(array(self :: PARAM_RESET_CACHE => 1))));
        }

        return $action_bar;
    }

    public function get_table_type()
    {
        return $this->table_type;
    }
}
