<?php
namespace repository\quota;

use common\libraries\Theme;
use common\libraries\Utilities;
use common\libraries\ToolbarItem;
use common\libraries\Translation;
use common\libraries\Filesystem;
use common\libraries\Toolbar;
use common\libraries\DatetimeUtilities;
use common\libraries\NewObjectTableCellRendererActionsColumnSupport;
use common\libraries\NewObjectTableCellRenderer;

class RequestTableCellRenderer extends NewObjectTableCellRenderer implements
    NewObjectTableCellRendererActionsColumnSupport
{

    public function render_cell($column, $object)
    {
        $calculator = new Calculator($object->get_user());

        switch ($column->get_name())
        {
            case Translation :: get('User') :
                return $object->get_user()->get_fullname();
            case Request :: PROPERTY_QUOTA :
                return Filesystem :: format_file_size($object->get_quota());
            case Request :: PROPERTY_CREATION_DATE :
                return DatetimeUtilities :: format_locale_date(null, $object->get_creation_date());
            case Request :: PROPERTY_DECISION_DATE :
                return DatetimeUtilities :: format_locale_date(null, $object->get_decision_date());
            case Request :: PROPERTY_DECISION :
                return $object->get_decision_icon();
            case Translation :: get('UsedDiskSpace') :
                return Filesystem :: format_file_size($calculator->get_used_user_disk_quota());
            case Translation :: get('MaximumUsedDiskSpace') :
                return Filesystem :: format_file_size($calculator->get_maximum_user_disk_quota());
        }
        return parent :: render_cell($column, $object);
    }

    public function get_object_actions($object)
    {
        $toolbar = new Toolbar();

        if (\repository\quota\rights\Rights :: get_instance()->quota_is_allowed())
        {
            if (! $object->was_granted() && (\repository\quota\rights\Rights :: get_instance()->is_target_user(
                $this->get_component()->get_user(),
                $object->get_user_id()) || $this->get_component()->get_user()->is_platform_admin()))
            {
                $toolbar->add_item(
                    new ToolbarItem(
                        Translation :: get('Grant'),
                        Theme :: get_image_path() . 'action/grant.png',
                        $this->get_component()->get_url(
                            array(
                                Manager :: PARAM_ACTION => Manager :: ACTION_GRANT,
                                Manager :: PARAM_REQUEST_ID => $object->get_id())),
                        ToolbarItem :: DISPLAY_ICON));
            }

            if ($object->is_pending() && (\repository\quota\rights\Rights :: get_instance()->is_target_user(
                $this->get_component()->get_user(),
                $object->get_user_id()) || $this->get_component()->get_user()->is_platform_admin()))
            {
                $toolbar->add_item(
                    new ToolbarItem(
                        Translation :: get('Deny'),
                        Theme :: get_image_path() . 'action/deny.png',
                        $this->get_component()->get_url(
                            array(
                                Manager :: PARAM_ACTION => Manager :: ACTION_DENY,
                                Manager :: PARAM_REQUEST_ID => $object->get_id())),
                        ToolbarItem :: DISPLAY_ICON));
            }
        }

        if ($this->get_component()->get_user()->is_platform_admin() ||
             ($this->get_component()->get_user_id() == $object->get_user_id() && $object->is_pending()))
        {
            $toolbar->add_item(
                new ToolbarItem(
                    Translation :: get('Delete', null, Utilities :: COMMON_LIBRARIES),
                    Theme :: get_common_image_path() . 'action_delete.png',
                    $this->get_component()->get_url(
                        array(
                            Manager :: PARAM_ACTION => Manager :: ACTION_DELETE,
                            Manager :: PARAM_REQUEST_ID => $object->get_id())),
                    ToolbarItem :: DISPLAY_ICON));
        }

        return $toolbar->as_html();
    }
}
