<?php
namespace repository\quota\rights;

use common\libraries\InCondition;
use group\Group;
use group\DataManager as GroupDataManager;
use rights\RightsLocationEntityRight;
use rights\NewPlatformGroupEntity;
use rights\NewUserEntity;
use rights\RightsUtil;
use common\libraries\Translation;
use common\libraries\EqualityCondition;

class Rights extends RightsUtil
{
    const VIEW_RIGHT = '1';

    private static $instance;

    private static $target_users;

    /**
     *
     * @return \repository\quota\rights\Rights
     */
    public static function get_instance()
    {
        if (! isset(self :: $instance))
        {
            self :: $instance = new self();
        }
        return self :: $instance;
    }

    public static function get_available_rights()
    {
        return array(Translation :: get('ViewRight') => self :: VIEW_RIGHT);
    }

    public function quota_is_allowed()
    {
        $entities = array();
        $entities[NewUserEntity :: ENTITY_TYPE] = new NewUserEntity();
        $entities[NewPlatformGroupEntity :: ENTITY_TYPE] = new NewPlatformGroupEntity();

        return parent :: is_allowed(
            self :: VIEW_RIGHT,
            __NAMESPACE__,
            null,
            $entities,
            0,
            self :: TYPE_ROOT,
            0,
            self :: TREE_TYPE_ROOT);
    }

    public function get_quota_view_rights_location_entity_right($entity_id, $entity_type)
    {
        return parent :: get_rights_location_entity_right(
            __NAMESPACE__,
            self :: VIEW_RIGHT,
            $entity_id,
            $entity_type,
            self :: get_quota_root_id());
    }

    public function invert_quota_location_entity_right($right_id, $entity_id, $entity_type)
    {
        return parent :: invert_location_entity_right(
            __NAMESPACE__,
            $right_id,
            $entity_id,
            $entity_type,
            self :: get_quota_root_id());
    }

    public function get_quota_targets_entities()
    {
        return parent :: get_target_entities(self :: VIEW_RIGHT, __NAMESPACE__);
    }

    public function get_quota_root()
    {
        return parent :: get_root(__NAMESPACE__);
    }

    public function get_quota_root_id()
    {
        return parent :: get_root_id(__NAMESPACE__);
    }

    public function create_quota_root()
    {
        return parent :: create_location(__NAMESPACE__);
    }

    public function get_quota_location_entity_right($entity_id, $entity_type)
    {
        return \rights\DataManager :: retrieve_rights_location_entity_right(
            __NAMESPACE__,
            self :: VIEW_RIGHT,
            $entity_id,
            $entity_type,
            $this->get_quota_root_id());
    }

    public function get_target_users(\user\User $user)
    {
        if (! isset(self :: $target_users[$user->get_id()]))
        {
            $allowed_groups = array();

            $location_entity_right = $this->get_quota_location_entity_right(
                $user->get_id(),
                NewUserEntity :: ENTITY_TYPE);
            if ($location_entity_right instanceof RightsLocationEntityRight)
            {
                $condition = new EqualityCondition(
                    RightsLocationEntityRightGroup :: PROPERTY_LOCATION_ENTITY_RIGHT_ID,
                    $location_entity_right->get_id());
                $right_groups = DataManager :: retrieves(RightsLocationEntityRightGroup :: class_name(), $condition);

                while ($right_group = $right_groups->next_result())
                {
                    if (! in_array($right_group->get_group_id(), $allowed_groups))
                    {
                        $allowed_groups[] = $right_group->get_group_id();
                    }
                }
            }

            $user_group_ids = $user->get_groups(true);

            foreach ($user_group_ids as $user_group_id)
            {
                $location_entity_right = $this->get_quota_location_entity_right(
                    $user_group_id,
                    NewPlatformGroupEntity :: ENTITY_TYPE);
                if ($location_entity_right instanceof RightsLocationEntityRight)
                {
                    $condition = new EqualityCondition(
                        RightsLocationEntityRightGroup :: PROPERTY_LOCATION_ENTITY_RIGHT_ID,
                        $location_entity_right->get_id());
                    $right_groups = DataManager :: retrieves(RightsLocationEntityRightGroup :: class_name(), $condition);

                    while ($right_group = $right_groups->next_result())
                    {
                        if (! in_array($right_group->get_group_id(), $allowed_groups))
                        {
                            $allowed_groups[] = $right_group->get_group_id();
                        }
                    }
                }
            }

            self :: $target_users[$user->get_id()] = array();

            if (count($allowed_groups) > 0)
            {
                $condition = new InCondition(Group :: PROPERTY_ID, $allowed_groups);
                $groups = GroupDataManager :: retrieves(Group :: class_name(), $condition);

                while ($group = $groups->next_result())
                {
                    $user_ids = $group->get_users(true, true);

                    foreach ($user_ids as $user_id)
                    {
                        if (! in_array($user_id, self :: $target_users[$user->get_id()]))
                        {
                            self :: $target_users[$user->get_id()][] = $user_id;
                        }
                    }
                }
            }
        }

        return self :: $target_users[$user->get_id()];
    }

    public function is_target_user(\user\User $user, $target_user_id)
    {
        return in_array($target_user_id, $this->get_target_users($user));
    }
}
