<?php
namespace repository\upgrade\v4_0_0;

use common\libraries\CommonDataManager;
use common\libraries\package\Package;
use core\lynx\manager\PackageInstaller;
use common\libraries\Registration;
use common\libraries\Utilities;
use common\libraries\EqualityCondition;
use common\libraries\PropertyConditionVariable;
use common\libraries\StaticConditionVariable;
use common\libraries\DataClassRetrievesParameters;
use common\libraries\AndCondition;

/**
 *
 * @author Hans De Bisschop
 * @author Magali Gillard
 */
class Upgrader extends \common\libraries\package\Upgrader
{

    /**
     * Constructor
     */
    public function __construct()
    {
        parent :: __construct(\repository\DataManager :: get_instance());
    }

    /*
     * (non-PHPdoc) @see \common\libraries\package\Upgrader::run()
     */
    public function run()
    {
        $data_manager = $this->get_data_manager();

        $success = $data_manager->transactional(
            function ($c) use($data_manager)
            {
                $package = Package :: get('repository');
                $registration = CommonDataManager :: get_registration('repository');
                $registration->set_version($package->get_version());
                if (! $registration->update())
                {
                    return false;
                }

                $attributes = array('type' => 'integer', 'length' => 1, 'unsigned' => 1, 'notnull' => 1, 'default' => 2);
                $success = $data_manager->alter_storage_unit(
                    \common\libraries\DataManager :: ALTER_STORAGE_UNIT_CHANGE,
                    \repository\ContentObject :: get_table_name(),
                    \repository\ContentObject :: PROPERTY_STATE,
                    $attributes);

                if (! $success)
                {
                    return false;
                }

                $attributes = array('type' => 'integer', 'length' => 3, 'unsigned' => 1, 'notnull' => 1, 'default' => 1);
                $success = $data_manager->alter_storage_unit(
                    \common\libraries\DataManager :: ALTER_STORAGE_UNIT_CHANGE,
                    \repository\ContentObject :: get_table_name(),
                    \repository\ContentObject :: PROPERTY_CURRENT,
                    $attributes);

                if (! $success)
                {
                    return false;
                }

                $attributes = array('type' => 'text', 'length' => 255, 'notnull' => 1);
                $success = $data_manager->alter_storage_unit(
                    \common\libraries\DataManager :: ALTER_STORAGE_UNIT_CHANGE,
                    \repository\ContentObject :: get_table_name(),
                    \repository\ContentObject :: PROPERTY_TYPE,
                    $attributes);

                if (! $success)
                {
                    return false;
                }

                $attributes = array('type' => 'text', 'notnull' => 1);
                $success = $data_manager->alter_storage_unit(
                    \common\libraries\DataManager :: ALTER_STORAGE_UNIT_CHANGE,
                    \repository\ContentObject :: get_table_name(),
                    \repository\ContentObject :: PROPERTY_TITLE,
                    $attributes);

                if (! $success)
                {
                    return false;
                }

                $attributes = array('type' => 'text');
                $success = $data_manager->alter_storage_unit(
                    \common\libraries\DataManager :: ALTER_STORAGE_UNIT_CHANGE,
                    \repository\ContentObject :: get_table_name(),
                    \repository\ContentObject :: PROPERTY_DESCRIPTION,
                    $attributes);

                if (! $success)
                {
                    return false;
                }

                $attributes = array('type' => 'text', 'notnull' => 1);
                $success = $data_manager->alter_storage_unit(
                    \common\libraries\DataManager :: ALTER_STORAGE_UNIT_CHANGE,
                    'content_object_metadata',
                    'value',
                    $attributes);

                if (! $success)
                {
                    return false;
                }

                $attributes = array('type' => 'text');
                $success = $data_manager->alter_storage_unit(
                    \common\libraries\DataManager :: ALTER_STORAGE_UNIT_CHANGE,
                    \repository\ExternalInstance :: get_table_name(),
                    \repository\ExternalInstance :: PROPERTY_DESCRIPTION,
                    $attributes);

                if (! $success)
                {
                    return false;
                }

                $attributes = array('type' => 'text');
                $success = $data_manager->alter_storage_unit(
                    \common\libraries\DataManager :: ALTER_STORAGE_UNIT_CHANGE,
                    \repository\ExternalSetting :: get_table_name(),
                    \repository\ExternalSetting :: PROPERTY_VALUE,
                    $attributes);

                if (! $success)
                {
                    return false;
                }

                $attributes = array('type' => 'integer', 'unsigned' => 1, 'length' => 1);
                $success = $data_manager->alter_storage_unit(
                    \common\libraries\DataManager :: ALTER_STORAGE_UNIT_CHANGE,
                    \repository\ExternalSetting :: get_table_name(),
                    \repository\ExternalSetting :: PROPERTY_USER_SETTING,
                    $attributes);

                if (! $success)
                {
                    return false;
                }

                $attributes = array('type' => 'text');
                $success = $data_manager->alter_storage_unit(
                    \common\libraries\DataManager :: ALTER_STORAGE_UNIT_CHANGE,
                    \repository\ExternalUserSetting :: get_table_name(),
                    \repository\ExternalUserSetting :: PROPERTY_VALUE,
                    $attributes);

                if (! $success)
                {
                    return false;
                }

                $attributes = array('type' => 'text', 'length' => 200, 'notnull' => 1, 'default' => '');
                $success = $data_manager->alter_storage_unit(
                    \common\libraries\DataManager :: ALTER_STORAGE_UNIT_CHANGE,
                    \repository\RepositoryCategory :: get_table_name(),
                    \repository\RepositoryCategory :: PROPERTY_NAME,
                    $attributes);

                if (! $success)
                {
                    return false;
                }

                $attributes = array('type' => 'integer', 'unsigned' => 1, 'length' => 3, 'notnull' => 1);
                $success = $data_manager->alter_storage_unit(
                    \common\libraries\DataManager :: ALTER_STORAGE_UNIT_CHANGE,
                    \repository\RepositoryCategory :: get_table_name(),
                    \repository\RepositoryCategory :: PROPERTY_TYPE,
                    $attributes);

                if (! $success)
                {
                    return false;
                }

                $attributes = array('type' => 'integer', 'unsigned' => 1, 'notnull' => 1, 'default' => '');
                $success = $data_manager->alter_storage_unit(
                    \common\libraries\DataManager :: ALTER_STORAGE_UNIT_CHANGE,
                    \repository\RightsLocation :: get_table_name(),
                    \repository\RightsLocation :: PROPERTY_PARENT_ID,
                    $attributes);

                if (! $success)
                {
                    return false;
                }

                $attributes = array('type' => 'text', 'length' => 100, 'notnull' => 1, 'default' => '');
                $success = $data_manager->alter_storage_unit(
                    \common\libraries\DataManager :: ALTER_STORAGE_UNIT_CHANGE,
                    \repository\UserView :: get_table_name(),
                    \repository\UserView :: PROPERTY_NAME,
                    $attributes);

                if (! $success)
                {
                    return false;
                }

                $attributes = array('type' => 'text');
                $success = $data_manager->alter_storage_unit(
                    \common\libraries\DataManager :: ALTER_STORAGE_UNIT_CHANGE,
                    \repository\UserView :: get_table_name(),
                    \repository\UserView :: PROPERTY_DESCRIPTION,
                    $attributes);

                if (! $success)
                {
                    return false;
                }

                $attributes = array('type' => 'integer', 'unsigned' => 1, 'length' => 1, 'notnull' => 1, 'default' => 1);
                $success = $data_manager->alter_storage_unit(
                    \common\libraries\DataManager :: ALTER_STORAGE_UNIT_CHANGE,
                    \repository\RightsLocation :: get_table_name(),
                    \repository\RightsLocation :: PROPERTY_INHERIT,
                    $attributes);

                if (! $success)
                {
                    return false;
                }

                $attributes = array('type' => 'integer', 'unsigned' => 1, 'notnull' => 1);
                $success = $data_manager->alter_storage_unit(
                    \common\libraries\DataManager :: ALTER_STORAGE_UNIT_CHANGE,
                    \repository\ExternalSync :: get_table_name(),
                    \repository\ExternalSync :: PROPERTY_STATE,
                    $attributes);

                if (! $success)
                {
                    return false;
                }

                $success = $data_manager->alter_storage_unit_index(
                    \common\libraries\DataManager :: ALTER_STORAGE_UNIT_DROP_PRIMARY_KEY,
                    \repository\ContentObjectAttachment :: get_table_name());

                if (! $success)
                {
                    return false;
                }

                $attributes = array('type' => 'integer', 'unsigned' => 1, 'notnull' => 1, 'autoincrement' => 'true');
                $success = $data_manager->alter_storage_unit(
                    \common\libraries\DataManager :: ALTER_STORAGE_UNIT_ADD,
                    \repository\ContentObjectAttachment :: get_table_name(),
                    \repository\ContentObjectAttachment :: PROPERTY_ID,
                    $attributes);

                if (! $success)
                {
                    return false;
                }

                $success = $data_manager->alter_storage_unit_index(
                    \common\libraries\DataManager :: ALTER_STORAGE_UNIT_DROP_PRIMARY_KEY,
                    \repository\ContentObjectGroupShare :: get_table_name());

                if (! $success)
                {
                    return false;
                }

                $attributes = array('type' => 'integer', 'unsigned' => 1, 'notnull' => 1, 'autoincrement' => 'true');
                $success = $data_manager->alter_storage_unit(
                    \common\libraries\DataManager :: ALTER_STORAGE_UNIT_ADD,
                    \repository\ContentObjectGroupShare :: get_table_name(),
                    \repository\ContentObjectGroupShare :: PROPERTY_ID,
                    $attributes);

                if (! $success)
                {
                    return false;
                }

                $success = $data_manager->alter_storage_unit_index(
                    \common\libraries\DataManager :: ALTER_STORAGE_UNIT_ADD_UNIQUE,
                    \repository\ContentObjectGroupShare :: get_table_name(),
                    'rcogs_co_group',
                    array(
                        \repository\ContentObjectGroupShare :: PROPERTY_GROUP_ID,
                        \repository\ContentObjectGroupShare :: PROPERTY_CONTENT_OBJECT_ID));

                if (! $success)
                {
                    return false;
                }

                $attributes = array('type' => 'integer', 'unsigned' => 1, 'notnull' => 1, 'autoincrement' => 'true');
                $success = $data_manager->alter_storage_unit(
                    \common\libraries\DataManager :: ALTER_STORAGE_UNIT_ADD,
                    \repository\ContentObjectInclude :: get_table_name(),
                    \repository\ContentObjectInclude :: PROPERTY_ID,
                    $attributes);

                if (! $success)
                {
                    return false;
                }

                $success = $data_manager->alter_storage_unit_index(
                    \common\libraries\DataManager :: ALTER_STORAGE_UNIT_DROP_PRIMARY_KEY,
                    \repository\ContentObjectUserShare :: get_table_name());

                if (! $success)
                {
                    return false;
                }

                $attributes = array('type' => 'integer', 'unsigned' => 1, 'notnull' => 1, 'autoincrement' => 'true');
                $success = $data_manager->alter_storage_unit(
                    \common\libraries\DataManager :: ALTER_STORAGE_UNIT_ADD,
                    \repository\ContentObjectUserShare :: get_table_name(),
                    \repository\ContentObjectUserShare :: PROPERTY_ID,
                    $attributes);

                if (! $success)
                {
                    return false;
                }

                $success = $data_manager->alter_storage_unit_index(
                    \common\libraries\DataManager :: ALTER_STORAGE_UNIT_DROP_PRIMARY_KEY,
                    \repository\UserViewRelContentObject :: get_table_name());

                if (! $success)
                {
                    return false;
                }

                $attributes = array('type' => 'integer', 'unsigned' => 1, 'notnull' => 1, 'autoincrement' => 'true');
                $success = $data_manager->alter_storage_unit(
                    \common\libraries\DataManager :: ALTER_STORAGE_UNIT_ADD,
                    \repository\UserViewRelContentObject :: get_table_name(),
                    \repository\UserViewRelContentObject :: PROPERTY_ID,
                    $attributes);

                if (! $success)
                {
                    return false;
                }

                $attributes = array('type' => 'text', 'length' => 200, 'notnull' => 1);
                $success = $data_manager->alter_storage_unit(
                    \common\libraries\DataManager :: ALTER_STORAGE_UNIT_ADD,
                    \repository\ComplexContentObjectItem :: get_table_name(),
                    \repository\ComplexContentObjectItem :: PROPERTY_TYPE,
                    $attributes);

                if (! $success)
                {
                    return false;
                }

                $success = $data_manager->alter_storage_unit(
                    \common\libraries\DataManager :: ALTER_STORAGE_UNIT_DROP,
                    \repository\UserViewRelContentObject :: get_table_name(),
                    'visibility');

                if (! $success)
                {
                    return false;
                }

                if (! $data_manager->drop_storage_unit('content_object_number_seq'))
                {
                    return false;
                }

                if (! CommonDataManager :: get_registration('common\extensions\repo_viewer')->delete())
                {
                    return false;
                }

                if (! \tracking\TrackingDataManager :: get_instance()->drop_storage_unit('repository_changes'))
                {
                    return false;
                }

                $conditions = array();
                $conditions[] = new EqualityCondition(
                    new PropertyConditionVariable(
                        \tracking\TrackerRegistration :: class_name(),
                        \tracking\TrackerRegistration :: PROPERTY_TRACKER),
                    new StaticConditionVariable('repository_changes_tracker'));
                $conditions[] = new EqualityCondition(
                    new PropertyConditionVariable(
                        \tracking\TrackerRegistration :: class_name(),
                        \tracking\TrackerRegistration :: PROPERTY_CONTEXT),
                    new StaticConditionVariable('repository'));
                $condition = new AndCondition($conditions);

                if (! \tracking\TrackingDataManager :: get_instance()->new_delete(
                    \tracking\TrackerRegistration :: class_name(),
                    $condition))
                {
                    return false;
                }

                return true;
            });

        $install = array();
        $install[] = 'repository\viewer';

        foreach ($install as $context)
        {
            $installer = new PackageInstaller($context);
            $installer->run();
        }

        // Home
        $condition = new EqualityCondition(
            new PropertyConditionVariable(
                \core\home\HomeBlockRegistration :: class_name(),
                \core\home\HomeBlockRegistration :: PROPERTY_CONTEXT),
            new StaticConditionVariable('repository'));
        $home_block_registrations = \core\home\DataManager :: retrieves(
            \core\home\HomeBlockRegistration :: class_name(),
            new DataClassRetrievesParameters($condition));

        while ($home_block_registration = $home_block_registrations->next_result())
        {
            $home_block_registration->set_context('repository\integration\core\home');
            $home_block_registration->update();
        }

        $package_info = Package :: get('repository\integration\core\home');
        $application_registration = new Registration();

        $parent_namespace = Utilities :: get_namespace_parent('repository\integration\core\home');
        $application_registration->set_type((! $parent_namespace ? 'core' : $parent_namespace));
        $application_registration->set_name($package_info->get_code());
        $application_registration->set_category($package_info->get_category());
        $application_registration->set_version($package_info->get_version());
        $application_registration->set_status(Registration :: STATUS_ACTIVE);
        $application_registration->set_context('repository\integration\core\home');

        if (! $application_registration->create())
        {
            return false;
        }

        // Webservices
        $package_info = Package :: get('repository\integration\webservice');

        $application_registration = new Registration();

        $parent_namespace = Utilities :: get_namespace_parent('repository\integration\webservice');
        $application_registration->set_type((! $parent_namespace ? 'core' : $parent_namespace));
        $application_registration->set_name($package_info->get_code());
        $application_registration->set_category($package_info->get_category());
        $application_registration->set_version($package_info->get_version());
        $application_registration->set_status(Registration :: STATUS_ACTIVE);
        $application_registration->set_context('repository\integration\webservice');

        if (! $application_registration->create())
        {
            return false;
        }

        return $success;
    }

    /*
     * (non-PHPdoc) @see \common\libraries\package\Upgrader::get_additional_packages()
     */
    public function get_additional_packages()
    {
        $packages = array();
        $packages[] = 'repository\quota';
        return $packages;
    }
}
