<?php

namespace rights\upgrade\v4_0_0;

use common\libraries\CommonDataManager;
use common\libraries\package\Package;
use common\libraries\Translation;
use common\libraries\Utilities;

/**
 * Database schema & content upgrader for Chamilo 3.x to 4.0.0.
 *
 * Instead of putting everything in the run() method, this class makes calls to
 * upgrade_db() and upgrade_registration(). Since the only thing that really differs
 * across Upgraders is upgrade_db() implementation, the other methods could/should
 * be moved to the \common\libraries\package\Upgrader.
 */
class Upgrader extends \common\libraries\package\Upgrader
{

    /**
     * Constructor
     */
    public function __construct()
    {
        parent :: __construct(CommonDataManager :: get_instance());
    }

    /*
     * (non-PHPdoc) @see \common\libraries\package\Upgrader::run()
     */
    public function run()
    {
        $data_manager = $this->get_data_manager();
        $upgrader = $this;

        $success = $data_manager->transactional(
            function ($c) use($data_manager, $upgrader)
            {
                $success = $upgrader->upgrade_db($data_manager);

                if($success)
                {
                    $upgrader->add_message(\common\libraries\package\Upgrader :: TYPE_NORMAL, Translation :: get('UpgradeDatabaseSuccessful', null, Utilities :: COMMON_LIBRARIES));
                }
                else
                {
                    $upgrader->add_message(\common\libraries\package\Upgrader :: TYPE_ERROR, Translation :: get('UpgradeDatabaseFailed', null, Utilities :: COMMON_LIBRARIES));
                }

                if(!$success)
                {
                    return false;
                }

                $success = $upgrader->update_db_content($data_manager);

                if($success)
                {
                    $upgrader->add_message(\common\libraries\package\Upgrader :: TYPE_NORMAL, Translation :: get('UpdateDatabaseContentSuccessful', null, Utilities :: COMMON_LIBRARIES));
                }
                else
                {
                    $upgrader->add_message(\common\libraries\package\Upgrader :: TYPE_ERROR, Translation :: get('UpdateDatabaseContentFailed', null, Utilities :: COMMON_LIBRARIES));
                }

                if(!$success)
                {
                    return false;
                }

                $success = $upgrader->upgrade_registration($data_manager);

                if($success)
                {
                    $upgrader->add_message(\common\libraries\package\Upgrader :: TYPE_NORMAL, Translation :: get('UpgradeRegistrationSuccessful', null, Utilities :: COMMON_LIBRARIES));
                }
                else
                {
                    $upgrader->add_message(\common\libraries\package\Upgrader :: TYPE_ERROR, Translation :: get('UpgradeRegistrationFailed', null, Utilities :: COMMON_LIBRARIES));
                }

                return $success;
            }
        );

        return $success;
    }

    /**
     * Performs the database upgrade.
     *
     * @param type $data_manager
     * @return boolean
     */
    public function upgrade_db($data_manager)
    {
        // Remove old tables
        if(\tracking\TrackingDataManager :: get_instance()->storage_unit_exist('rights_templates_rights_changes'))
        {
            if (!\tracking\TrackingDataManager :: get_instance()->drop_storage_unit('rights_templates_rights_changes'))
            {
                return false;
            }
        }
        return true;
    }

    /**
     * Performs the database content upgrade.
     *
     * @param type $data_manager
     * @return boolean
     */
    public function update_db_content($data_manager)
    {
        return true;
    }

    /**
     * Upgrades the package registration.
     *
     * @param type $data_manager
     * @return boolean
     */
    public function upgrade_registration($data_manager)
    {
        $package = Package :: get('rights');
        $registration = CommonDataManager :: get_registration('rights');
        $registration->set_version($package->get_version());
        if (! $registration->update())
        {
            return false;
        }

        return true;
    }
}
