<?php

namespace script;

use common\libraries\Path;
use Console_Getopt;
use core\install\CommandLineInstaller;

require_once __DIR__ . '/bootstrap.php';
require_once __DIR__ . '/../common/libraries/php/shared/filesystem/path.class.php';
require_once __DIR__ . '/../common/libraries/php/shared/utilities.class.php';

ini_set('include_path', realpath(Path :: get_plugin_path() . 'pear') . PATH_SEPARATOR . realpath(Path :: get_plugin_path() . 'google/library') . PATH_SEPARATOR . get_include_path());


require_once 'Console/Getopt.php';

spl_autoload_register('common\libraries\Utilities::autoload');

$cg = new Console_Getopt();
$args = $cg->readPHPArgv();
ChamiloScript :: fire($args);

class ChamiloScript
{

    static function fire($args)
    {
        $script = new ChamiloScript($args);
        $script->run();
    }

    private $args;

    function __construct($args = array())
    {
        $this->args = $args;
    }

    public function run()
    {
        $command = '';
        if (isset($this->args[1]))
        {
            $command = $this->args[1];
        }

        switch ($command)
        {
            default :
                $this->usage();
                break;
            case 'install' :
                $this->install();
                break;
            case 'update-autoloader' :
                $this->update_autoloader();
                break;
            case 'fix-line-endings' :
                $this->fix_line_endings();
                break;
        }
    }

    private function update_autoloader()
    {
        array_shift($this->args); // remove script name
        array_shift($this->args); // remove command flag
        $dir = reset($this->args);
        $dir = realpath(__DIR__ . '/../' . $dir);
        echo "process $dir \n";

        require_once __DIR__ . '/../common/libraries/php/shared/util/various/autoloader_utilities.class.php';
        require_once __DIR__ . '/../common/libraries/php/shared/util/various/code_utilities.class.php';

        set_time_limit(0);
        \common\libraries\AutoloaderUtilities :: synch($dir, $dir, true);
        echo "done $dir \n";
    }

    private function usage()
    {
        echo "USAGE : " . $this->args[0] . " [FLAG] \n" . "FLAG LIST : \n";
        echo "\t usage \t\t\t\t\t\t: print this message\n";
        echo "\t install [-c FILE | --configuration=FILE] \t: install chamilo according to config\n";
        echo "\t update-autoloader PATH \t\t\t: update the autoloader of PATH \n";
        echo "\t fix-line-endings PATH \t\t\t\t: fixes the line endings of PATH \n";
    }

    /**
     * Fixes the line endings of a given path
     */
    private function fix_line_endings()
    {
        $path = $this->args[2];
        if(!$path)
        {
            throw new \InvalidArgumentException('path required');
        }

        $dir = realpath(__DIR__ . '/../' . $path . '/php/');

        echo "process $dir \n";

        $iterator = new \RegexIterator(
            new \RecursiveIteratorIterator(
                new \RecursiveDirectoryIterator($dir)
            ),
            '/^.+\.php$/i',
            \RegexIterator :: GET_MATCH
        );

        foreach($iterator as $matches)
        {
            $file = $matches[0];
            $contents = file_get_contents($file);

            $contents = str_replace("\r\n", "\n", $contents);
            $contents = str_replace("\r", "\n", $contents);

            file_put_contents($file, $contents);
        }
    }

    private function install()
    {
        array_shift($this->args); // remove script name
        array_shift($this->args); // remove 'install' flag
        $shortOpts = "c:";
        $longOpts = array("configuration=");
        $options = $this->condense_arguments($shortOpts, $longOpts);

        if (! isset($options['c']) && ! isset($options['--configuration']))
        {
            echo 'Please provide a configuration file for the installer';
            return;
        }

        $config_file = isset($options['c']) ? $options['c'] : $options['--configuration'];

        $installer = new CommandLineInstaller($config_file);
        try
        {
            $installer->run();
        }
        catch (\Exception $e)
        {
            echo "Installation Failed !\n";
            echo $e->getMessage();
        }
    }

    private function condense_arguments($shortOpts, $longOpts)
    {
        $cg = new Console_Getopt();
        $params = $cg->getopt2($this->args, $shortOpts, $longOpts);
        $parsed = array();
        foreach ($params[0] as $param)
        {
            $parsed[$param[0]] = $param[1];
        }
        return $parsed;
    }

}
