<?php
namespace tracking;

use common\libraries\Condition;
use common\libraries\Utilities;
use common\libraries\Application;
use common\libraries\DataClass;
use common\libraries\EqualityCondition;

abstract class Tracker extends DataClass
{

    public function __construct($defaultProperties = array (), $optionalProperties = array())
    {
        parent :: __construct($defaultProperties, $optionalProperties);
    }

    /**
     *
     * @var Event
     */
    private $event;

    /**
     *
     * @return Event
     */
    public function get_event()
    {
        return $this->event;
    }

    /**
     *
     * @param $event Event
     */
    public function set_event(Event $event)
    {
        $this->event = $event;
    }

    /**
     * inherited
     */
    public function get_data_manager()
    {
        return TrackingDataManager :: get_instance();
    }

    /**
     *
     * @deprecated Use run() instead
     */
    public function track(array $parameters = array())
    {
        return $this->run($parameters);
    }

    abstract public function validate_parameters(array $parameters = array());

    /**
     * Write the values of the properties from the tracker to the database
     *
     * @return boolean
     */
    public function run(array $parameters = array())
    {
        $this->validate_parameters($parameters);
        return $this->create();
    }

    public function create()
    {
        return $this->get_data_manager()->create_tracker_item($this);
    }

    /**
     * Update the values of the properties from the tracker to the database
     *
     * @return boolean
     */
    public function update()
    {
        return $this->get_data_manager()->update_tracker_item($this);
    }

    public function delete()
    {
        $condition = new EqualityCondition(self :: PROPERTY_ID, $this->get_id());
        return $this->remove($condition);
    }

    /**
     * Removes tracker items with a given condition
     *
     * @param $condition Condition
     */
    public function remove(Condition $condition = null)
    {
        return $this->get_data_manager()->remove_tracker_items($this->get_table_name(), $condition);
    }

    /**
     * Retrieves tracker items with a given condition
     *
     * @param Condition condition on which we want to retrieve the trackers
     * @return array Array of tracker items
     * @deprecated Use get_data()->as_array() now
     */
    public function retrieve_tracker_items($condition)
    {
        $trkdmg = TrackingDataManager :: get_instance();
        return $trkdmg->retrieve_tracker_items($this->get_table_name(), $condition, 0, - 1, null, get_class($this))->as_array();
    }

    /**
     * Retrieves tracker items with a given condition
     *
     * @param Condition condition on which we want to retrieve the trackers
     * @return ObjectResultSet ResultSet of tracker items
     * @deprecated Use get_data() now
     */
    public function retrieve_tracker_items_result_set($condition, $offset = null, $max_objects = null, $order_by = array())
    {
        return TrackingDataManager :: get_instance()->retrieve_tracker_items($this->get_table_name(), $condition,
                $offset, $max_objects, $order_by, get_class($this));
    }

    /**
     *
     * @param Condition condition on which we want to retrieve the trackers
     * @deprecated Use count_data() now
     */
    public function count_tracker_items($condition)
    {
        $trkdmg = TrackingDataManager :: get_instance();
        return $trkdmg->count_tracker_items($this->get_table_name(), $condition);
    }

    /**
     *
     * @param $type string
     * @param $application string
     * @return Tracker The tracker object
     */
    public static function factory($type, $context)
    {
        $class = $context . '\\' . Utilities :: underscores_to_camelcase($type);
        return new $class();
    }

    /**
     *
     * @param $type string
     * @param $application string
     * @param $condition Condition
     * @param $offset int
     * @param $max_objects int
     * @param $order_by ObjectTableOrder
     * @return ObjectResultSet The tracker data resultset
     */
    public static function get_data($class_name, $application, $condition, $offset = null, $max_objects = null, $order_by = array())
    {
        $table_name = call_user_func(array($class_name, 'get_table_name'));

        return self :: get_data_manager()->retrieve_tracker_items($table_name, $condition, $offset, $max_objects,
                $order_by, $class_name);
    }

    /**
     *
     * @param $type string
     * @param $application string
     * @param $condition Condition
     * @param $order_by ObjectTableOrder
     * @return Tracker The tracker
     */
    public static function get_singular_data($class_name, $application, $condition, $order_by = array())
    {
        $table_name = call_user_func(array($class_name, 'get_table_name'));

        return self :: get_data_manager()->retrieve_tracker_item_by_condition($table_name, $condition, $order_by,
                $class_name);
    }

    public static function count_data($class_name, $application, $condition)
    {
        $table_name = call_user_func(array($class_name, 'get_table_name'));

        return self :: get_data_manager()->count_tracker_items($table_name, $condition);
    }

    public static function count_group_data($class_name, $group, $count, $condition)
    {
        $table_name = call_user_func(array($class_name, 'get_table_name'));

        return self :: get_data_manager()->count_group_tracker_items($table_name, $group, $count, $condition);
    }

    /**
     *
     * @param $type string
     * @param $application string
     * @param $condition Condition
     * @return boolean
     */
    public static function remove_data($class_name, $application, $condition = null)
    {
        $table_name = call_user_func(array($class_name, 'get_table_name'));

        return self :: get_data_manager()->remove_tracker_items($table_name, $condition);
    }
}
