<?php
namespace user\integration\tracking;

use common\libraries\EqualityCondition;

class LoginLogout extends \tracking\SimpleTracker
{
    const CLASS_NAME = __CLASS__;
    const PROPERTY_USER_ID = 'user_id';
    const PROPERTY_DATE = 'date';
    const PROPERTY_IP = 'ip';
    const PROPERTY_TYPE = 'type';

    public function validate_parameters(array $parameters = array())
    {
        $user = $parameters['user'];
        $server = $parameters['server'];

        $this->set_user_id($user->get_id());
        $this->set_date(time());
        $this->set_ip($server['REMOTE_ADDR']);
        $this->set_type($this->get_event()->get_name());
    }

    public function empty_tracker($event)
    {
        $condition = new EqualityCondition('type', $event->get_name());
        return $this->remove($condition);
    }

    public function export($start_date, $end_date, $event)
    {
        $conditions = array();
        $conditions[] = new EqualityCondition('type', $event->get_name());
        return parent :: export($start_date, $end_date, $conditions);
    }

    /**
     * Get's the userid of the login tracker
     *
     * @return int $userid the userid
     */
    public function get_user_id()
    {
        return $this->get_default_property(self :: PROPERTY_USER_ID);
    }

    /**
     * Sets the userid of the login tracker
     *
     * @param int $userid the userid
     */
    public function set_user_id($userid)
    {
        $this->set_default_property(self :: PROPERTY_USER_ID, $userid);
    }

    /**
     * Get's the date of the login tracker
     *
     * @return int $date the date
     */
    public function get_date()
    {
        return $this->get_default_property(self :: PROPERTY_DATE);
    }

    /**
     * Sets the date of the login tracker
     *
     * @param int $date the date
     */
    public function set_date($date)
    {
        $this->set_default_property(self :: PROPERTY_DATE, $date);
    }

    /**
     * Get's the ip of the login tracker
     *
     * @return int $ip the ip
     */
    public function get_ip()
    {
        return $this->get_default_property(self :: PROPERTY_IP);
    }

    /**
     * Sets the ip of the login tracker
     *
     * @param int $ip the ip
     */
    public function set_ip($ip)
    {
        $this->set_default_property(self :: PROPERTY_IP, $ip);
    }

    /**
     * Get's the type of the login tracker
     *
     * @return int $type the type
     */
    public function get_type()
    {
        return $this->get_default_property(self :: PROPERTY_TYPE);
    }

    /**
     * Sets the type of the login tracker
     *
     * @param int $type the type
     */
    public function set_type($type)
    {
        $this->set_default_property(self :: PROPERTY_TYPE, $type);
    }

    public static function get_default_property_names()
    {
        return parent :: get_default_property_names(
                array(self :: PROPERTY_TYPE, self :: PROPERTY_USER_ID, self :: PROPERTY_DATE, self :: PROPERTY_IP));
    }

    public static function get_table_name()
    {
        return 'user_' . parent :: get_table_name();
    }
}
