<?php
namespace user;

use group\GroupRelUser;
use group\GroupDataManager;
use common\libraries\Mdb2Database;

/**
 * Mdb2 implementation of the user\DataManager
 *
 * @author Hans De Bisschop
 * @package common.libraries
 */
class Mdb2DataManager extends Mdb2Database implements UserDataManagerInterface
{

    /**
     * Retrieve users by one or more groups
     *
     * @param $class_name string
     * @param $parameters \common\libraries\DataClassRetrievesParameters
     * @return \common\libraries\Mdb2ResultSet
     */
    public function new_retrieve_users_by_groups($class_name, $parameters)
    {
        return $this->retrieve_users_by_groups($parameters->get_condition(), $parameters->get_offset(),
                $parameters->get_count(), $parameters->get_order_by());
    }

    /**
     * Retrieve users by one or more groups
     *
     * @param $condition \common\libraries\Condition
     * @param $offset int
     * @param $max_objects int
     * @param $order_by multitype:\common\libraries\ObjectTableOrder
     * @return \common\libraries\Mdb2ResultSet
     * @deprecated Use new_retrieve_users_by_groups()
     */
    public function retrieve_users_by_groups($condition = null, $offset = null, $max_objects = null, $order_by = null)
    {
        $group_data_manager = GroupDataManager :: get_instance();
        $group_rel_user_alias = $group_data_manager->get_alias(GroupRelUser :: get_table_name());
        $user_alias = $this->get_alias(User :: get_table_name());

        $query = 'SELECT DISTINCT ' . $user_alias . '.*';
        $query .= ' FROM ' . $this->escape_table_name(User :: get_table_name()) . ' as ' . $user_alias;
        $query .= ' JOIN ' . $group_data_manager->escape_table_name(GroupRelUser :: get_table_name()) . ' as ' . $group_rel_user_alias . ' on ' . parent :: escape_column_name(
                User :: PROPERTY_ID, $user_alias) . ' = ' . parent :: escape_column_name(GroupRelUser :: PROPERTY_USER_ID,
                $group_rel_user_alias);

        return $this->retrieve_object_set($query, User :: get_table_name(), $condition, $offset, $max_objects,
                $order_by, User :: CLASS_NAME);
    }

    /**
     * Count users by one or more groups
     *
     * @param $class_name string
     * @param $parameters \common\libraries\DataClassCountParameters
     * @return int
     */
    public function new_count_users_by_groups($class_name, $parameters)
    {
        return $this->count_users_by_groups($parameters->get_condition());
    }

    /**
     * Count users by one or more groups
     *
     * @param $condition unknown_type
     * @return int
     * @deprecated Use new_count_users_by_groups()
     */
    public function count_users_by_groups($condition = null)
    {
        $group_data_manager = GroupDataManager :: get_instance();
        $group_rel_user_alias = $group_data_manager->get_alias(GroupRelUser :: get_table_name());
        $user_alias = $this->get_alias(User :: get_table_name());

        $query = 'SELECT COUNT(DISTINCT ' . $this->escape_table_name(User :: PROPERTY_ID, $user_alias) . ')';
        $query .= ' FROM ' . $this->escape_table_name(User :: get_table_name()) . ' as ' . $user_alias;
        $query .= ' JOIN ' . $group_data_manager->escape_table_name(GroupRelUser :: get_table_name()) . ' as ' . $group_rel_user_alias . ' on ' . parent :: escape_column_name(
                User :: PROPERTY_ID, $user_alias) . ' = ' . parent :: escape_column_name(GroupRelUser :: PROPERTY_USER_ID,
                $group_rel_user_alias);

        return $this->count_result_set($query, User :: get_table_name(), $condition);
    }

    /**
     * Retrieve users in combination with one or more templates
     *
     * @param $parameters \common\libraries\DataClassRetrievesParameters
     * @return \common\libraries\Mdb2ResultSet
     */
    public function retrieve_users_from_template(DataClassRetrievesParameters $parameters)
    {
        $user_alias = $this->get_alias(User :: get_table_name());
        $template_alias = $this->get_alias(UserRightsTemplate :: get_table_name());

        $query = 'SELECT ' . parent :: escape_column_name('*', $user_alias);
        $query .= ' FROM ' . $this->escape_table_name(User :: get_table_name()) . ' AS ' . $user_alias;
        $query .= ' JOIN ' . $this->escape_table_name(UserRightsTemplate :: get_table_name()) . ' as ' . $template_alias;
        $query .= ' ON ' . parent :: escape_column_name(User :: PROPERTY_ID, $user_alias) . '=' . parent :: escape_column_name(
                UserRightsTemplate :: PROPERTY_USER_ID, $template_alias);

        return $this->retrieve_object_set($query, User :: get_table_name(), $parameters->get_condition(),
                $parameters->get_offset(), $parameters->get_count(), $parameters->get_order_by(), User :: CLASS_NAME);
    }

    public function get_total_user_disk_quota()
    {
        $query = 'SELECT SUM(' . parent :: escape_column_name(User :: PROPERTY_DISK_QUOTA) . ') AS disk_quota FROM ' . $this->escape_table_name(
                User :: get_table_name());

        $res = $this->query($query);
        $record = $res->fetchRow(MDB2_FETCHMODE_OBJECT);
        return $record->disk_quota;
    }
}
