<?php
namespace user;

use common\libraries\NewObjectTableSupport;
use common\libraries\Translation;
use common\libraries\Request;
use common\libraries\AdministrationComponent;
use common\libraries\ActionBarRenderer;
use common\libraries\ConditionProperty;
use common\libraries\ToolbarItem;
use common\libraries\Theme;
use common\libraries\ActionBarSearchForm;
use common\libraries\Display;
use common\libraries\Application;
use common\libraries\BreadcrumbTrail;
use common\libraries\Utilities;

/**
 * $Id: admin_user_browser.class.php 211 2009-11-13 13:28:39Z vanpouckesven $
 *
 * @package user.lib.user_manager.component
 */
class UserManagerAdminUserBrowserComponent extends UserManager implements AdministrationComponent, NewObjectTableSupport
{

    private $firstletter;

    private $menu_breadcrumbs;

    private $action_bar;

    /**
     * Runs this component and displays its output.
     */
    public function run()
    {
        if (! $this->get_user()->is_platform_admin())
        {
            $this->display_header();
            Display :: error_message(Translation :: get("NotAllowedHere", null, Utilities :: COMMON_LIBRARIES));
            $this->display_footer();
            exit();
        }

        $this->firstletter = Request :: get(UserManager :: PARAM_FIRSTLETTER);

        $this->display_header();

        echo $this->get_action_bar()->as_html() . '<br />';

        // echo $menu;
        echo $this->get_user_html();
        $this->display_footer();
    }

    public function get_user_html()
    {
        $table = new AdminUserBrowserTable($this);

        $html = array();
        $html[] = '<div style="float: right; width: 100%;">';
        $html[] = $table->as_html();
        $html[] = '</div>';

        return implode($html, "\n");
    }

    public function get_parameters()
    {
        $parameters = parent :: get_parameters();
        if (isset($this->action_bar))
        {
            $parameters[ActionBarSearchForm :: PARAM_SIMPLE_SEARCH_QUERY] = $this->get_action_bar()->get_query();
        }
        return $parameters;
    }

    /*
     * (non-PHPdoc) @see common\libraries.NewObjectTableSupport::get_object_table_condition()
     */
    public function get_object_table_condition($class_name)
    {
        // construct search properties
        $search_properties = array();
        $search_properties[] = new ConditionProperty(User :: PROPERTY_FIRSTNAME);
        $search_properties[] = new ConditionProperty(User :: PROPERTY_LASTNAME);
        $search_properties[] = new ConditionProperty(User :: PROPERTY_USERNAME);
        $search_properties[] = new ConditionProperty(User :: PROPERTY_OFFICIAL_CODE);
        $search_properties[] = new ConditionProperty(User :: PROPERTY_EMAIL);

        // get conditions
        $condition = $this->get_action_bar()->get_conditions($search_properties);

        return $condition;
    }

    public function get_action_bar()
    {
        if (! isset($this->action_bar))
        {
            $this->action_bar = new ActionBarRenderer(ActionBarRenderer :: TYPE_HORIZONTAL);
            $this->action_bar->set_search_url($this->get_url(parent :: get_parameters()));

            if ($this->get_user()->is_platform_admin())
            {
                $this->action_bar->add_common_action(
                    new ToolbarItem(
                        Translation :: get('Add', null, Utilities :: COMMON_LIBRARIES),
                        Theme :: get_common_image_path() . 'action_add.png',
                        $this->get_url(array(Application :: PARAM_ACTION => UserManager :: ACTION_CREATE_USER)),
                        ToolbarItem :: DISPLAY_ICON_AND_LABEL));
            }

            $this->action_bar->add_common_action(
                new ToolbarItem(
                    Translation :: get('Show', null, Utilities :: COMMON_LIBRARIES),
                    Theme :: get_common_image_path() . 'action_browser.png',
                    $this->get_url(),
                    ToolbarItem :: DISPLAY_ICON_AND_LABEL));
        }
        return $this->action_bar;
    }

    public function add_additional_breadcrumbs(BreadcrumbTrail $breadcrumbtrail)
    {
        $breadcrumbtrail->add_help('user_browser');
    }
}
