<?php
namespace user;

use repository\quota\Calculator;
use common\libraries\Filesystem;
use common\libraries\Translation;
use common\libraries\Request;
use common\libraries\Theme;
use common\libraries\Display;
use common\libraries\ToolbarItem;
use common\libraries\AdministrationComponent;
use common\libraries\Utilities;
use common\libraries\ActionBarRenderer;
use common\libraries\Application;

/**
 * $Id: quota_viewer.class.php 211 2009-11-13 13:28:39Z vanpouckesven $
 * 
 * @package user.lib.user_manager.component
 */
/**
 * User manager component which displays the quota to the user. This component displays two progress-bars. The first one
 * displays the used disk space and the second one the number of learning objects in the users user.
 * 
 * @author Bart Mollet
 * @author Tim De Pauw
 * @author Hans De Bisschop
 * @author Dieter De Neef
 */
class UserManagerQuotaViewerComponent extends UserManager implements AdministrationComponent
{

    private $selected_user;

    /**
     * Runs this component and displays its output.
     */
    public function run()
    {
        if (! $this->get_user()->is_platform_admin())
        {
            $this->display_header();
            Display :: error_message(Translation :: get("NotAllowed", null, Utilities :: COMMON_LIBRARIES));
            $this->display_footer();
            exit();
        }
        $selected_user_id = Request :: get(UserManager :: PARAM_USER_USER_ID);
        if (! $selected_user_id)
        {
            $this->selected_user = $this->get_user();
        }
        else
        {
            $this->selected_user = \user\DataManager :: retrieve(User :: class_name(), (int) $selected_user_id);
        }
        $this->calculator = new Calculator($this->selected_user);
        
        $this->display_header();
        $this->display_action_bar();
        
        echo '<h3>' . htmlentities(Translation :: get('UsedDiskSpace')) . '</h3>';
        echo Calculator :: get_bar($this->calculator->get_user_disk_quota_percentage(), 
                Filesystem :: format_file_size($this->calculator->get_used_user_disk_quota()) . ' / ' . Filesystem :: format_file_size(
                        $this->calculator->get_maximum_user_disk_quota()));
        echo '<div style="clear: both;">&nbsp;</div>';
        
        $this->display_footer();
    }

    private function display_action_bar()
    {
        $action_bar = new ActionBarRenderer(ActionBarRenderer :: TYPE_HORIZONTAL);
        
        $action_bar->add_common_action(
                new ToolbarItem(Translation :: get('EditUser'), Theme :: get_common_image_path() . 'action_edit.png', 
                        $this->get_url(
                                array(Application :: PARAM_ACTION => UserManager :: ACTION_UPDATE_USER, 
                                        UserManager :: PARAM_USER_USER_ID => $this->selected_user->get_id())), 
                        ToolbarItem :: DISPLAY_ICON_AND_LABEL));
        
        echo $action_bar->as_html();
    }
}
