<?php
namespace webservice;

use common\libraries\NoAuthenticationWebservice;
use common\libraries\Utilities;
use common\libraries\Filesystem;
use common\libraries\Translation;

abstract class WebserviceInstaller extends \common\libraries\package\Installer
{

    /**
     * Perform additional installation steps
     *
     * @return boolean
     */
    public function extra()
    {
        if (! $this->register_webservices())
        {
            return $this->failed(
                    Translation :: get('WebserviceFailed', null, WebserviceManager :: APPLICATION_NAME));
        }

        return true;
    }

    public function register_webservices()
    {
        $methods = array('get', 'get_list', 'create', 'update', 'delete');

        $path = static :: get_path() . 'php/lib/';

        if (is_dir($path))
        {
            $folders = Filesystem :: get_directory_content($path, Filesystem :: LIST_DIRECTORIES, false);
            if (count($folders) > 0)
            {
                $this->add_message(self :: TYPE_NORMAL,
                        '<span class="subtitle">' . Translation :: get('TypeName', null,
                                WebserviceManager :: APPLICATION_NAME) . '</span>');

                $application_webservice_category = new WebserviceCategory();
                $application_webservice_category->set_name(Translation :: get('TypeName', null, static :: context()));
                if (! $application_webservice_category->create())
                {
                    $this->add_message(self :: TYPE_ERROR,
                            Translation :: get('WebserviceCategoryCreationFailed', null,
                                    WebserviceManager :: APPLICATION_NAME) . ' : <em>' . $application_webservice_category->get_name() . '</em>');
                    return false;
                }
                else
                {
                    $this->add_message(self :: TYPE_NORMAL,
                            Translation :: get('WebserviceCategoryCreated', null, WebserviceManager :: APPLICATION_NAME) . ': <em>' . $application_webservice_category->get_name() . '</em>');
                }

                foreach ($folders as $folder)
                {
                    $camelcase_folder = Utilities :: underscores_to_camelcase($folder);

                    $object_webservice_category = new WebserviceCategory();
                    $object_webservice_category->set_name(
                            Translation :: get($camelcase_folder, null, static :: context()));
                    $object_webservice_category->set_parent($application_webservice_category->get_id());
                    if (! $object_webservice_category->create())
                    {
                        $this->add_message(self :: TYPE_ERROR,
                                Translation :: get('WebserviceCategoryCreationFailed', null,
                                        WebserviceManager :: APPLICATION_NAME) . ' : <em>' . $object_webservice_category->get_name() . '</em>');
                        return false;
                    }
                    else
                    {
                        $this->add_message(self :: TYPE_NORMAL,
                                Translation :: get('WebserviceCategoryCreated', null,
                                        WebserviceManager :: APPLICATION_NAME) . ': <em>' . $object_webservice_category->get_name() . '</em>');
                    }

                    foreach ($methods as $method)
                    {
                        $file = $path . $folder . '/' . $method . '.class.php';
                        if (file_exists($file))
                        {
                            $camelcase_method = Utilities :: underscores_to_camelcase($method);

                            $webservice = new WebserviceRegistration();

                            $webservice->set_name(
                                    Translation :: get($camelcase_method, null, WebserviceManager :: APPLICATION_NAME));
                            $webservice->set_description(
                                    Translation :: get($camelcase_folder . $camelcase_method . 'Description', null,
                                            static :: context()));

                            $webservice->set_active(1);
                            $webservice->set_category($object_webservice_category->get_id());
                            $webservice->set_code(static :: context() . '_' . $folder . '_' . $method);

                            $class = static :: context() . '\\' . Utilities :: underscores_to_camelcase(
                                    $folder . '_' . $method . '_webservice_handler');
                            $object = new $class();
                            if ($object instanceof NoAuthenticationWebservice)
                            {
                                $webservice->set_authentication(0);
                            }
                            else
                            {
                                $webservice->set_authentication(1);
                            }

                            if (! $webservice->create())
                            {
                                $this->add_message(self :: TYPE_ERROR,
                                        Translation :: get('WebserviceCreationFailed', null,
                                                WebserviceManager :: APPLICATION_NAME) . ' : <em>' . $webservice->get_name() . '</em>');
                                return false;
                            }
                            else
                            {
                                $this->add_message(self :: TYPE_NORMAL,
                                        Translation :: get('WebserviceCreated', null,
                                                WebserviceManager :: APPLICATION_NAME) . ': <em>' . $webservice->get_name() . '</em>');
                            }
                        }
                    }
                }

                $this->add_message(self :: TYPE_NORMAL,
                        Translation :: get('WebserviceSucces', null, WebserviceManager :: APPLICATION_NAME));
                $this->add_message(self :: TYPE_NORMAL, '');
            }
        }

        return true;
    }
}
