<?php
namespace webservice;

use common\libraries\Translation;
use common\libraries\Request;
use common\libraries\Display;
use common\libraries\EqualityCondition;
use common\libraries\AndCondition;
use common\libraries\OrCondition;
use common\libraries\PatternMatchCondition;
use common\libraries\AdministrationComponent;
use common\libraries\BreadcrumbTrail;
use common\libraries\ActionBarRenderer;
use common\libraries\ActionBarSearchForm;
use common\libraries\Utilities;

/**
 * $Id: webservice_browser.class.php 208 2009-11-13 13:14:39Z vanpouckesven $
 *
 * @package webservices.lib.webservice_manager.component
 */

/**
 * Weblcms component which allows the user to manage his or her user subscriptions
 */
class WebserviceManagerWebserviceBrowserComponent extends WebserviceManager implements AdministrationComponent
{

    private $action_bar;

    /**
     * Runs this component and displays its output.
     */
    public function run()
    {
        $category = WebserviceDataManager :: get_instance()->retrieve_webservice_category(
                $this->get_webservice_category());
        if (! $this->get_user()->is_platform_admin())
        {
            $this->display_header();
            Display :: error_message(Translation :: get('NotAllowed', null, Utilities :: COMMON_LIBRARIES));
            $this->display_footer();
            exit();
        }

        $this->action_bar = $this->get_action_bar();
        $output = $this->get_user_html();
        $menu = $this->get_menu_html();

        $this->display_header();
        echo '<br />' . $this->action_bar->as_html() . '<br />';
        echo $output;
        echo $menu;
        $this->display_footer();
    }

    public function get_user_html()
    {
        $parameters = $this->get_parameters();
        $parameters[WebserviceManager :: PARAM_WEBSERVICE_CATEGORY_ID] = $this->get_webservice_category();
        $parameters[ActionBarSearchForm :: PARAM_SIMPLE_SEARCH_QUERY] = $this->action_bar->get_query();

        $table = new WebserviceBrowserTable($this, $parameters, $this->get_condition());

        $html = array();
        $html[] = '<div style="float: right; width: 80%;">';
        $html[] = $table->as_html();
        $html[] = '</div>';

        return implode($html, "\n");
    }

    public function get_menu_html()
    {
        $webservice_category_menu = new WebserviceCategoryMenu($this->get_webservice_category());
        $html = array();
        $html[] = '<div style="float: left; padding-right: 20px; width: 18%; overflow: auto; height: 100%;">';
        $html[] = $webservice_category_menu->render_as_tree();
        $html[] = '</div>';

        return implode($html, "\n");
    }

    public function get_condition()
    {
        $condition = new EqualityCondition(WebserviceRegistration :: PROPERTY_CATEGORY, $this->get_webservice_category());

        $query = $this->action_bar->get_query();
        if (isset($query) && $query != '')
        {
            $or_conditions = array();
            $or_conditions[] = new PatternMatchCondition(WebserviceRegistration :: PROPERTY_NAME, '*' . $query . '*');
            $or_condition = new OrCondition($or_conditions);

            $and_conditions = array();
            $and_conditions[] = $condition;
            $and_conditions[] = $or_condition;
            $condition = new AndCondition($and_conditions);
        }

        return $condition;
    }

    public function get_webservice()
    {
        return (Request :: get(WebserviceManager :: PARAM_WEBSERVICE_ID) ? Request :: get(
                WebserviceManager :: PARAM_WEBSERVICE_ID) : 0);
    }

    public function get_webservice_category()
    {
        return (Request :: get(WebserviceManager :: PARAM_WEBSERVICE_CATEGORY_ID) ? Request :: get(
                WebserviceManager :: PARAM_WEBSERVICE_CATEGORY_ID) : 0);
    }

    public function get_action_bar()
    {
        $action_bar = new ActionBarRenderer(ActionBarRenderer :: TYPE_HORIZONTAL);
        $action_bar->set_search_url(
                $this->get_url(
                        array(WebserviceManager :: PARAM_WEBSERVICE_CATEGORY_ID => $this->get_webservice_category())));
        return $action_bar;
    }

    public function add_additional_breadcrumbs(BreadcrumbTrail $breadcrumbtrail)
    {
        $breadcrumbtrail->add_help('webservice_rights_editor');
    }
}
