<?php
namespace webservice;

use common\libraries\Path;

use common\libraries\CommonDataManager;
use common\libraries\Registration;
use common\libraries\AdministrationComponent;
use common\libraries\Request;
use common\libraries\Translation;
use common\libraries\OrCondition;
use common\libraries\EqualityCondition;
use common\libraries\Utilities;
use common\libraries\WebApplication;
use common\libraries\CoreApplication;
use common\libraries\Filesystem;
use common\libraries\AndCondition;
use common\libraries\Application;
use common\libraries\Theme;

/**
 * Webservice component that scans for and registers new webservices
 *
 * @author Alexander Van Paemel
 * @package webservices.php.lib.webservice_manager.component
 */
class WebserviceManagerWebserviceRegistrationComponent extends WebserviceManager implements AdministrationComponent
{
    const PARAM_REGISTER = 'register';

    /**
     * Runs this component and displays its output.
     */
    public function run()
    {
        $this->display_header();

        echo '<p>' . Translation :: get('RegisterDescription') . '</p>';

        $url = $this->get_url(array(self :: PARAM_REGISTER => 1));
        echo '<button type="button" class="positive" value="submit" onClick="document.location = \'' . $url . '\'">' . Translation :: get(
            'RegisterWebservices') . '</button>';

        if (Request :: get(self :: PARAM_REGISTER) == 1)
        {
            $this->register_webservices();
        }

        $this->display_footer();
    }

    private function register_webservices()
    {
        $conditions = array();
        $conditions[] = new EqualityCondition(Registration :: PROPERTY_TYPE, Registration :: TYPE_CORE);
        $conditions[] = new EqualityCondition(Registration :: PROPERTY_TYPE, Registration :: TYPE_APPLICATION);
        $condition = new OrCondition($conditions);

        $registrations = CommonDataManager :: retrieves(Registration :: class_name(), $condition);

        while ($registration = $registrations->next_result())
        {
            $description = $this->register_application_webservices($registration->get_name());

            if (empty($description))
            {
                $description = '<em>' . Translation :: get('NoNewWebservicesFound') . '</em>';
            }

            if (WebApplication :: exists($registration->get_name()))
            {
                $context = $registration->get_type() . '/' . $registration->get_name();
            }
            else
            {
                $context = $registration->get_name();
            }

            $image = Theme :: get_image_path($context) . 'logo/16.png';
            $html = '<p><img src="' . $image . '" title="' . $registration->get_name();
            $html .= '" style="padding-right:5px;" />';
            $html .= '<b>' . $registration->get_name() . '</b><br />' . $description . '</p>';
            echo $html;
        }
    }

    private function register_application_webservices($registration_name)
    {
        $methods = array('get', 'get_list', 'create', 'update', 'delete');
        $output = array();
        $namespace = Application :: determine_namespace($registration_name);
        $path = Path :: namespace_to_full_path($namespace) . 'php/webservices/';

        if (! file_exists($path))
        {
            return $nr_created;
        }

        $folders = Filesystem :: get_directory_content($path, Filesystem :: LIST_DIRECTORIES, false);
        if (count($folders) > 0)
        {
            $name = Translation :: get('TypeName', null, $namespace);
            $conditions = array();
            $conditions[] = new EqualityCondition(WebserviceCategory :: PROPERTY_NAME, $name);
            $conditions[] = new EqualityCondition(WebserviceCategory :: PROPERTY_PARENT, 0);
            $condition = new AndCondition($conditions);
            $app_webservice_cat = WebserviceDataManager :: get_instance()->retrieve_webservice_categories($condition)->next_result();
            if (! $app_webservice_cat)
            {
                $app_webservice_cat = new WebserviceCategory();
                $app_webservice_cat->set_name($name);
                if (! $app_webservice_cat->create())
                {
                    $output[] = '<span style="color: red;">' . Translation :: get('WebserviceCategoryCreationFailed') .
                         ': <em>' . $name . '</em></span>';
                    return implode("<br />", $output);
                }
                $output[] = Translation :: get('WebserviceCategoryCreated') . ': <span style="color: blue;"><em>' . $name .
                     '</em></span>';
            }
        }

        foreach ($folders as $folder)
        {
            $name = Translation :: get(Utilities :: underscores_to_camelcase($folder), null, $namespace);
            $conditions = array();
            $conditions[] = new EqualityCondition(WebserviceCategory :: PROPERTY_NAME, $name);
            $conditions[] = new EqualityCondition(WebserviceCategory :: PROPERTY_PARENT, $app_webservice_cat->get_id());
            $condition = new AndCondition($conditions);
            $object_webservice_cat = WebserviceDataManager :: get_instance()->retrieve_webservice_categories($condition)->next_result();
            if (! $object_webservice_cat)
            {
                $object_webservice_cat = new WebserviceCategory();
                $object_webservice_cat->set_name($name);
                $object_webservice_cat->set_parent($app_webservice_cat->get_id());
                if (! $object_webservice_cat->create())
                {
                    $output[] = '<span style="color: red;">' . Translation :: get('WebserviceCategoryCreationFailed') .
                         ': <em>' . $name . '</em></span>';
                    continue;
                }
                $output[] = Translation :: get('WebserviceSubcategoryCreated') . ': <span style="color: blue;"><em>' .
                     $name . '</em></span>';
            }

            foreach ($methods as $method)
            {
                $file = $path . $folder . '/' . $method . '.class.php';
                if (file_exists($file))
                {
                    $camelcase_method = Utilities :: underscores_to_camelcase($method);
                    $name = Translation :: get($camelcase_method, null, WebserviceManager :: APPLICATION_NAME);
                    $conditions = array();
                    $conditions[] = new EqualityCondition(WebserviceRegistration :: PROPERTY_NAME, $name);
                    $conditions[] = new EqualityCondition(
                        WebserviceRegistration :: PROPERTY_CATEGORY,
                        $object_webservice_cat->get_id());
                    $condition = new AndCondition($conditions);
                    if (WebserviceDataManager :: get_instance()->count_webservices($condition) == 0)
                    {
                        $webservice = new WebserviceRegistration();
                        $webservice->set_name($name);
                        $webservice->set_description(
                            Translation :: get($camelcase_folder . $camelcase_method . 'Description', null, $namespace));
                        $webservice->set_active(1);
                        $webservice->set_category($object_webservice_cat->get_id());
                        $webservice->set_code($namespace . '_' . $folder . '_' . $method);

                        $class = $namespace . '\\' . Utilities :: underscores_to_camelcase(
                            $folder . '_' . $method . '_webservice_handler');
                        $object = new $class();
                        if ($object instanceof NoAuthenticationWebservice)
                        {
                            $webservice->set_authentication(0);
                        }
                        else
                        {
                            $webservice->set_authentication(1);
                        }

                        if (! $webservice->create())
                        {
                            $output[] = '<span style="color: red;">' .
                                 Translation :: get('WebserviceRegistrationFailed') . ': <em>' . $name . ' ' .
                                 $object_webservice_cat->get_name() . '</em></span>';
                        }
                        $output[] = Translation :: get('WebserviceRegistred') . ': <span style="color: green;"><em>' .
                             $name . ' ' . $object_webservice_cat->get_name() . '</em></span>';
                    }
                }
            }
        }

        return implode("<br />", $output);
    }
}
