<?php
namespace webservice;

use common\libraries\Application;
use common\libraries\CoreApplication;
use common\libraries\Redirect;
use common\libraries\Translation;
use common\libraries\DynamicAction;
use common\libraries\Theme;

/**
 * $Id: webservice_manager.class.php 208 2009-11-13 13:14:39Z vanpouckesven $
 *
 * @package webservices.lib.webservice_manager
 */

/**
 * A webservice manager provides some functionalities to the admin to manage his webservices.
 */
class WebserviceManager extends CoreApplication
{
    const APPLICATION_NAME = 'webservice';
    const PARAM_REMOVE_SELECTED = 'delete';
    const PARAM_FIRSTLETTER = 'firstletter';
    const PARAM_COMPONENT_ACTION = 'action';
    const PARAM_SOURCE = 'source';
    const PARAM_LOCATION_ID = 'location';
    const PARAM_WEBSERVICE_ID = 'webservice';
    const PARAM_WEBSERVICE_CATEGORY_ID = 'webservice_category_id';
    const ACTION_BROWSE_WEBSERVICES = 'webservice_browser';
    const ACTION_REGISTER_WEBSERVICES = 'webservice_registration';
    const ACTION_SIMULATE_SIGNATURE = 'signature_simulator';
    const DEFAULT_ACTION = self :: ACTION_BROWSE_WEBSERVICES;

    private $parameters;

    private $search_parameters;

    private $user;

    private $breadcrumbs;

    private $instance;

    public function __construct($user = null)
    {
        parent :: __construct($user);
    }

    public function retrieve_webservices($condition = null, $offset = null, $count = null, $order_property = null)
    {
        return WebserviceDataManager :: get_instance()->retrieve_webservices(
            $condition,
            $offset,
            $count,
            $order_property);
    }

    public function count_webservices($condition = null)
    {
        return WebserviceDataManager :: get_instance()->count_webservices($condition);
    }

    public function retrieve_webservice_categories($condition = null, $offset = null, $count = null, $order_property = null)
    {
        return WebserviceDataManager :: get_instance()->retrieve_webservice_categories(
            $condition,
            $offset,
            $count,
            $order_property);
    }

    public function retrieve_webservice($id)
    {
        return WebserviceDataManager :: get_instance()->retrieve_webservice($id);
    }

    public function retrieve_webservice_by_name($name)
    {
        return WebserviceDataManager :: get_instance()->retrieve_webservice_by_name($name);
    }

    public static function get_application_platform_admin_links($application = self :: APPLICATION_NAME)
    {
        $links = array();
        $links[] = new DynamicAction(
            Translation :: get('List'),
            Translation :: get('ListDescription'),
            Theme :: get_image_path() . 'admin/list.png',
            Redirect :: get_link(
                self :: APPLICATION_NAME,
                array(Application :: PARAM_ACTION => self :: ACTION_BROWSE_WEBSERVICES),
                array(),
                false,
                Redirect :: TYPE_CORE));
        $links[] = new DynamicAction(
            Translation :: get('Register'),
            Translation :: get('RegisterDescription'),
            Theme :: get_image_path() . 'admin/sort.png',
            Redirect :: get_link(
                self :: APPLICATION_NAME,
                array(Application :: PARAM_ACTION => self :: ACTION_REGISTER_WEBSERVICES),
                array(),
                false,
                Redirect :: TYPE_CORE));
        $links[] = new DynamicAction(
            Translation :: get('WebserviceManagerSignatureSimulatorComponent'),
            Translation :: get('SignatureSimulatorDescription'),
            Theme :: get_image_path() . 'admin/build.png',
            Redirect :: get_link(
                self :: APPLICATION_NAME,
                array(Application :: PARAM_ACTION => self :: ACTION_SIMULATE_SIGNATURE),
                array(),
                false,
                Redirect :: TYPE_CORE));
        $info = parent :: get_application_platform_admin_links(self :: APPLICATION_NAME);
        $info['links'] = $links;

        return $info;
    }

    /**
     * Helper function for the Application class, pending access to class constants via variables in PHP 5.3 e.g. $name
     * = $class :: DEFAULT_ACTION DO NOT USE IN THIS APPLICATION'S CONTEXT Instead use: - self :: DEFAULT_ACTION in the
     * context of this class - YourApplicationManager :: DEFAULT_ACTION in all other application classes
     */
    public function get_default_action()
    {
        return self :: DEFAULT_ACTION;
    }
}
